	subroutine savemat( type, mrows, ncols, imagf, namlen, name,
     $			rpart, ipart, lunit, irec, wtflg )
c
c SAVEMAT - A Fortran subroutine that writes a double-precision matrix
c           to a data file in the .MAT-file format used by MATLAB.
c
c For more information consult LOAD in the reference section of the MATLAB
c users guide.  See also LOADMAT.FOR, TESTLS.FOR, and FORMAT.FOR.
c
c	Description of inputs:
c
c	type	- matrix type flag; considering the type flag as a
c		  decimal integer, the ones decimal place is used to
c		  indicate numeric or textual interpretation of the
c		  matrix data (0 for numeric or 1 for textual); 
c		  the 1000's decimal place is used to indicate the
c		  machine format for the matrix data (0 for Intel 80(2)86 
c		  based machines, 1 for Motorola 68000 based machines,
c		  and 2 for Vax d format). For example, A flag of 1000 indicates 
c		  numeric data in a 68000 machine format and a flag of
c		  1001 indicates textual interpretation of the 68000
c		  machine format data.
c	mrows	- number of rows in matrix
c	ncols	- number of columns in matrix
c	imagf	- imaginary flag; 0 for no imaginary part or 1 if the 
c		  data has an imaginary part
c	namlen	- number of characters in matrix name plus 1 (for zero 
c		  byte string terminator supplied by this routine)
c	name	- character array holding the matrix name. (Be sure 
c		  character array has room for the zero byte terminator)
c	rpart	- real part of matrix (mrows x ncols double precision 
c		  elements stored column wise)
c	ipart	- imaginary part of matrix (only used if imagf = 1)
c	lunit	- logical unit number of output file
c	irec    - direct access file record counter (set to 1 initially
c		  and then leave alone)
c
c
c	Description of outputs:
c
c	wtflg	- write flag; 0 = good write, -1 = error during file write
c
c
c	NOTES : 
c
c	     o  THE OUTPUT FILE MUST BE OPENED WITH THE FOLLOWING STATEMENT:
c		  open(unit=lunit, file='file_name', form='unformatted',
c			  access=direct, recl=1) 
c
c	     o	Be sure to close the file when there are no more
c		matrices to be saved. 
c
c	     o	You may call SAVEMAT multiple times to write multiple matrices
c		onto the output file before closing it.
c
c	Author: S.N. Bangert 5-31-85
c	Revised 5-27-86 SNB
c
c  20 byte header
	integer  type, mrows, ncols, imagf, namlen
c
c  character string for name (length of name plus one)
	character  name(*)*1
c
c  double precision data arrays for example
	double precision  rpart(*), ipart(*)
c
c  output file logical unit number
	integer lunit
c
c  write flag
	integer  wtflg
c
c  direct access file record counter
	integer irec
c
c
c
	mn =mrows*ncols
c
c  write header
	call writec(lunit, irec, 4, type)
	call writec(lunit, irec, 4, mrows)
	call writec(lunit, irec, 4, ncols)
	call writec(lunit, irec, 4, imagf)
	call writec(lunit, irec, 4, namlen)
	call writec(lunit, irec, namlen-1, name)
	call writec(lunit, irec, 1, 0)
c
	call writec(lunit, irec, 8*mn, rpart)
c
	if ( imagf .eq. 1 )  call writec(lunit, irec, 8*mn, ipart)
c
c  good write
	wtflg = 0
	return
c
c  error during write
999	wtflg = -1
	return
	end
	subroutine writec( lunit, irec, nc, string )
c
	integer lunit, irec, nc
c
	character string(*)*1
c
	do 10 i=1,nc
		write(lunit,rec=irec) string(i)
		irec = irec + 1
10	continue
c
	return
	end
