      subroutine checo (h,ldh,n,ipvt,rcond,z)
      integer ldh,n,ipvt(n)
      real rcond
      complex h(ldh,n),z(n)
c
c     *** purpose:
c
c     checo factors a complex upper hessenberg matrix by gaussian
c     elimination and estimates the condition of the matrix.  the
c     linpack analogue of checo is cgeco.
c
c     if rcond is not needed, chefa may be substantially faster.  to
c     solve the linear system  h*x = b  (i.e., to compute
c     (inverse(h)*b)), follow checo by chesl.
c
c     on entry:
c
c        h     complex(ldh,n)
c              the matrix to be factored.
c
c        ldh   integer
c              the leading or row dimension of the array h as declared
c              in the main calling program.
c
c        n     integer
c              the order of the matrix h.
c
c     on return:
c
c        h     an upper triangular matrix and the multipliers which were
c              used to obtain it.  the factorization can be written
c              h = l*u  where l is a product of permutation and unit
c              lower bidiagonal matrices and u is upper triangular.
c
c        ipvt  integer(n)
c              an integer vector of pivot indices.
c
c        rcond real
c              an estimate of the reciprocal of the condition of h.
c              for the system  h*x = b , relative perturbations
c              in h and b of size epsilon may cause relative perturba-
c              tions in x of size  epsilon/rcond.  if rcond is so small
c              that the logical expression    (1.0+rcond) .eq. 1.0
c              is true, then h may be singular to working precision.
c              in particular, rcond is zero if exact singularity is
c              detected or the estimate underflows.
c
c        z     complex(n)
c              a work vector whose contents are usually unimportant.
c              if h is close to a singular matrix, then z is an
c              approximate null vector in the sense that
c              norm(h*z) = rcond*norm(h)*norm(z)
c
c     this version dated may 1980.
c     alan j. laub, university of southern california.
c
c     subroutines and functions called:
c
c     scl1,chefa,cl1nrm
c
      real scl1
c
c     internal variables:
c
      integer i,info,j,k,kb,km1,kp1,l
      real hnorm,s,sm,ynorm,znorm
      complex ek,t,wk,wkm
c
c     fortran functions called:
c
      complex cmplx,conjg
c
c     internal function:
c
      complex z1,z2,csign
      csign(z1,z2) = scl1(z1)*(z2/scl1(z2))
c
      if (n.gt.1) go to 10
c
c     special case of scalar (n = 1) system
c
      hnorm = scl1(h(1,1))
      ynorm = hnorm
      go to 250
   10 continue
c
c     compute complex 1-norm of h
c
      hnorm = 0.0e0
      do 30 j = 1,n
         s = 0.0e0
         do 20 i = 1,n
              s = s+scl1(h(i,j))
   20    continue
         if (s .gt. hnorm) hnorm = s
   30 continue
c
c     factor
c
      call chefa (h,ldh,n,ipvt,info)
c
c     rcond = 1/(norm(h)*(estimate of norm(inverse(h))).
c     estimate = norm(z)/norm(y)  where  h*z =y
c     and  transpose(h)*y = e.  the components of e are
c     chosen to cause maximum local growth in the elements
c     of w where  transpose(u)*w = e.  the vectors are
c     frequently rescaled to avoid overflow.
c
c     solve  transpose(u)*w = e
c
      ek = (1.0e0,0.0e0)
      do 40 j = 1,n
         z(j) = (0.0e0,0.0e0)
   40 continue
      do 130 k = 1,n
         if (scl1(z(k)) .ne. 0.0e0) ek = csign(ek,-z(k))
         if (scl1(ek-z(k)) .le. scl1(h(k,k))) go to 60
         s = scl1(h(k,k))/scl1(ek-z(k))
         do 50 i = 1,n
              z(i) = cmplx(s,0.0e0)*z(i)
   50    continue
         ek = cmplx(s,0.0e0)*ek
   60    continue
         wk = ek-z(k)
         wkm = -ek-z(k)
         s = scl1(wk)
         sm = scl1(wkm)
         if (scl1(h(k,k)) .eq. 0.0e0) go to 70
         wk = wk/conjg(h(k,k))
         wkm = wkm/conjg(h(k,k))
         go to 80
   70    continue
         wk = (1.0e0,0.0e0)
         wkm = (1.0e0,0.0e0)
   80    continue
         kp1 = k+1
         if (kp1 .gt. n) go to 120
         do 90 j = kp1,n
              sm = sm+scl1(z(j)+wkm*conjg(h(k,j)))
              z(j) = z(j)+wk*conjg(h(k,j))
              s = s+scl1(z(j))
   90    continue
         if (s .ge. sm) go to 110
         t = wkm-wk
         wk = wkm
         do 100 j = kp1,n
              z(j) = z(j)+t*conjg(h(k,j))
  100    continue
  110    continue
  120    continue
         z(k) = wk
  130 continue
      call cl1nrm (n,z,znorm)
c
c     solve  transpose(l)*y = w
c
      do 160 kb = 1,n
         k = n+1-kb
         if (k .lt. n) z(k) = z(k)-conjg(h(k+1,k))*z(k+1)
         if (scl1(z(k)) .le. 1.0e0) go to 150
         s = 1.0e0/scl1(z(k))
         do 140 i = 1,n
              z(i) = cmplx(s,0.0e0)*z(i)
  140    continue
  150    continue
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
  160 continue
      call cl1nrm (n,z,znorm)
c
      ynorm = 1.0e0
c
c     solve  l*v = y
c
      do 190 k = 1,n
         l = ipvt(k)
         t = z(l)
         z(l) = z(k)
         z(k) = t
         if (k .lt. n) z(k+1) = z(k+1)-t*h(k+1,k)
         if (scl1(z(k)) .le. 1.0e0) go to 180
         s = 1.0e0/scl1(z(k))
         do 170 i = 1,n
              z(i) = cmplx(s,0.0e0)*z(i)
  170    continue
         ynorm = s*ynorm
  180    continue
  190 continue
      call cl1nrm (n,z,znorm)
      ynorm = ynorm/znorm
c
c     solve  u*z = v
c
      do 240 kb = 1,n
         k = n+1-kb
         if (scl1(z(k)) .le. scl1(h(k,k))) go to 210
         s = scl1(h(k,k))/scl1(z(k))
         do 200 i = 1,n
              z(i) = cmplx(s,0.0e0)*z(i)
  200    continue
         ynorm = s*ynorm
  210    continue
         if (scl1(h(k,k)) .ne. 0.0e0) z(k) = z(k)/h(k,k)
         if (scl1(h(k,k)) .eq. 0.0e0) z(k) = (1.0e0,0.0e0)
         t = -z(k)
         if (k .le. 1) go to 230
         km1 = k-1
         do 220 i = 1,km1
              z(i) = z(i)+t*h(i,k)
  220    continue
  230    continue
  240 continue
      call cl1nrm (n,z,znorm)
      ynorm = ynorm/znorm
  250 continue
      if (hnorm .ne. 0.0e0) rcond = ynorm/hnorm
      if (hnorm .eq. 0.0e0) rcond = 0.0e0
      return
      end
