      subroutine chesl (h,ldh,n,ipvt,b)
      integer ldh,n,ipvt(n)
      complex h(ldh,n),b(n)
c
c     *** purpose:
c
c     chesl solves the complex upper hessenberg system
c                   h*x = b
c     using the factors computed by checo or chefa.
c     the linpack analogue of chesl is cgesl.
c
c     on entry:
c
c        h    complex(ldh,n)
c             the output from checo or chefa.
c
c        ldh  integer
c             the leading or row dimension of the array h as
c             declared in the main calling program.
c
c        n    integer
c             the order of the matrix h.
c
c        ipvt integer(n)
c             the pivot vector from checo or chefa.
c
c        b    complex(n)
c             the right hand side vector.
c
c     on return:
c
c        b    the solution vector x (= (inverse(h)*b).
c
c     error condition:
c
c        a division by zero will occur if the input factor contains a
c        zero on the diagonal.  technically, this indicates singularity
c        but it is often caused by improper arguments or improper
c        setting of ldh.  it will not occur if the subroutines are
c        called correctly and if checo has set rcond .gt. 0.0
c        or chefa has set info .eq. 0.
c
c     to compute  inverse(h)*g  where g is a matrix with m columns
c          call checo (h,ldh,n,ipvt,rcond,z)
c          if (rcond is too small) go to ...
c          do 10 j = 1,m
c             call chesl (h,ldh,n,ipvt,g(1,j))
c       10 continue
c
c     this version dated may 1980.
c     alan j. laub, university of southern california.
c
c     subroutines and functions called:
c
c     none
c
c     internal variables:
c
      integer i,k,kb,l,nm1
      complex t
c
c     fortran functions called:
c
c     none
c
      if (n .eq. 1) go to 50
      nm1 = n-1
c
c     first solve  l*y = b
c
      do 20 k = 1,nm1
         l = ipvt(k)
         t = b(l)
         if (l .eq. k) go to 10
         b(l) = b(k)
         b(k) = t
   10    continue
         b(k+1) = b(k+1)-t*h(k+1,k)
   20 continue
c
c     now solve  u*x = y
c
      do 40 kb = 1,nm1
         k = n-kb
         b(k+1) = b(k+1)/h(k+1,k+1)
         t = -b(k+1)
         do 30 i = 1,k
              b(i) = b(i)+t*h(i,k+1)
   30    continue
   40 continue
   50 continue
      b(1) = b(1)/h(1,1)
      return
      end
