c------------------------------------------------------------------
c
c	dfreq	created:	unknown authors	dim dark past
c		modified(mod1):	Roy Smith	22nd Jan. 1987.
c
c------------------------------------------------------------------
c
c	mod1:	When a constant matrix is entered (nx = 0)
c		the flag info in set to zero as balancing
c		the A matrix is not required.  Leaving the
c		flag unchanged causes the first result to be
c		flagged as an error.
c
c------------------------------------------------------------------
c
c
      subroutine dfreq (info,freq,nx,nu,ny,na,nb,nc,nd,ng,nh,
     +                  a,b,c,d,g,h,ipvt,zr,zc,ind1,ind2)
      integer info,nx,nu,ny,na,nb,nc,nd,ng,nh,ipvt(nx)
      integer ind1(*),ind2(*)
      real*8 a(na,nx),b(nb,nu),c(nc,nx),d(nd,nu),zr(nx)
      complex*16 freq,g(ng,nu),h(nh,nx),zc(nx)
c
c     *** purpose:
c
c     dfreq takes real matrices a (nx x nx), b (nx x nu), c (ny x nx),
c     and d(ny x nu) and forms the complex frequency response matrix
c           g(freq) := c * (((freq * i) - a)-inverse) * b + d
c     where  i = (nx x nx) identity matrix and freq is a complex
c     scalar parameter taking values along the imaginary axis for
c     continuous-time systems and on the unit circle for discrete-
c     time systems.
c
c     on entry:
c
c        info     integer
c                 > 0 for the first call of dfreq whereupon it is set = 0
c                 for all subsequent calls; may be set = 0 initially if a 
c                 is already in upper hessenberg form.
c
c        freq     complex
c                 a complex scalar (the frequency parameter).
c
c        nx       integer
c                 the number of the states.
c
c        nu       integer
c                 the number of inputs. 
c
c        ny       integer
c                 the number of outputs.
c
c        na       integer
c                 the leading or row dimension of the real array a
c                 as declared in the main calling program.
c
c        nb       integer
c                 the leading or row dimension of the real array b
c                 as declared in the main calling program.
c
c        nc       integer
c                 the leading or row dimension of the real array c
c                 as declared in the main calling program.
c
c        nd       integer
c                 the leading or row dimension of the real array d
c                 as declared in the main calling program.
c
c        ng       integer
c                 the leading or row dimension of the complex array g
c                 as declared in the main calling program.
c
c        nh       integer
c                 the leading or row dimension of the complex array h
c                 as declared in the main calling program.
c
c        a        real(na,nx)
c                 a real nx x nx matrix (the system matrix).
c
c        b        real(nb,nu)
c                 a real nx x nu matrix (the input matrix).
c
c        c        real(nc,nx)
c                 a real ny x nx matrix (the output matrix).
c
c        d        real(nd,nu)
c                 a real ny x nu matrix (the feed-through matrix).
c
c     on return:
c
c        info     this is output as the info parameter of subroutine chefa 
c                 (chefa may be consulted for details); normal return is 0.
c
c        a,b,c,d  a is in upper hessenberg form while b and c have been
c                 suitably transformed; if  info .eq. 0 these
c                 arrays are not further modified.  d is never modified.
c
c        h        complex(nh,nx)
c                 scratch array used for internal computations.
c
c        g        complex(ng,nu)
c                 the frequency response matrix g(freq).
c
c        ipvt     integer(nx)
c                 scratch array used for internal computations.
c                 (May share memory with zr).
c
c        zr       real(nx)
c                 scratch array used for internal computations.
c                 (May share memory with ipvt or zc, but not both).
c
c        zc       complex(nx)
c                 scratch array used for internal computations.
c                 (May share memory with zr).
c
c        ind1     integer(*)
c                 an integer vector that contains the starting index
c                 of each diagonal block
c
c        ind2     integer(*)
c                 an integer vector that contains the size 
c                 of each diagonal block
c
c     subroutines and functions called:
c
c     balanc(eispack),zhefa,zhesl,dhetr
c
c     internal variables:
c
      integer i,igh,j,k,kk,kmin,kmax,kp,low
      real*8 t
      complex*16 temp
c
      if (nx .eq. 0) goto 250
      if (info .eq. 0) go to 150
      call balanc (na,nx,a,low,igh,zr)
c
c     adjust b and c matrices based on information in the vector
c     zr which describes the balancing of a and is defined in the
c     subroutine balanc
c
      do 40 k = 1,nx
         kk = nx - k + 1
         if (kk .ge. low .and. kk .le. igh) go to 40
         if (kk .lt. low) kk = low - kk
         kp = zr(kk)
         if (kp .eq. kk) go to 40
c
c        permute rows of b
c
         do 20 j = 1,nu
            t = b(kk,j)
            b(kk,j) = b(kp,j)
            b(kp,j) = t
   20    continue
c
c        permute columns of c
c
         do 30 i = 1,ny
            t = c(i,kk)
            c(i,kk) = c(i,kp)
            c(i,kp) = t
   30    continue
c
   40 continue
      if (igh .eq. low) go to 80
      do 70 k = low,igh
         t = zr(k)
c
c        scale columns of permuted c
c
         do 50 i = 1,ny
            c(i,k) = c(i,k)*t
   50    continue
c
c        scale rows of permuted b
c
         do 60 j = 1,nu
            b(k,j) = b(k,j)/t
   60    continue
c
   70 continue
   80 continue
   90 continue
c
c     reduce a to hessenberg form by orthogonal similarities and
c     accumulate the orthogonal transformations into b and c
c
      call dhetr (na,nb,nc,ny,nu,nx,low,igh,a,b,c,zr)
      call dstrct (na,nx,a,ind1,ind2)
      info = 0
  150 continue
c
c     update  h := (freq * i) - a   with appropriate value of freq
c     and factor the complex hessenberg matrix
c
      k = 0
  160 continue
         k = k + 1
         kmin = ind1(k)
         kmax = kmin + ind2(k) - 1
         do 180 j = kmin,kmax
            do 170 i = kmin,kmax
               h(i,j) = - a(i,j)
  170       continue
            h(j,j) = freq + h(j,j)
  180    continue
         call zhefa (h(kmin,kmin),nh,ind2(k),ipvt(kmin),info)
         if (info .ne. 0) return
      if (kmax .lt. nx) goto 160
c
c     compute  g = c*(h-inverse)*b + d
c
      do 240 j = 1,nu
         do 200 k = 1,nx
            zc(k) = b(k,j)
  200    continue
         k = 0
  210    continue
            k = k + 1
	    kmin = ind1(k)
            call zhesl (h(kmin,kmin),nh,ind2(k),ipvt(kmin),zc(kmin))
         if (kmin + ind2(k) .le. nx) goto 210
         do 230 i = 1,ny
            temp = d(i,j)
            do 220 k = 1,nx
               temp = temp + c(i,k)*zc(k)
  220       continue
            g(i,j) = temp
  230    continue
  240 continue
c
c        g now contains the desired frequency response matrix
c
      return
c
c------------------------------------------------------------------
c
c		we now handle the constant matrix case
c
  250 continue
      do 270 j = 1,nu
         do 260 i = 1,ny
            g(i,j) = d(i,j)
  260    continue
  270 continue
      info = 0
      return
      end
c
c------------------------------------------------------------------
c
      subroutine dstrct (na,n,a,ind1,ind2)
      integer na,n,ind1(*),ind2(*),i,j,jj,k
      real*8 a(na,*)
      k = 1
      ind1(1) = 1
   10 continue
      i = ind1(k)
      j = n
   20 continue
      if (j .gt. i .and. a(i,j) .eq. 0.0) then
         j = j - 1
         goto 20
      endif
   30 continue
      if (j .lt. n) then
         if (i .lt. j .or. a(i+1,i) .ne. 0.0) then
            i = i + 1
            jj = n
   40       continue
            if (jj .gt. j .and. a(i,jj) .eq. 0.0) then
               jj = jj - 1
               goto 40
            endif
            j = jj
            goto 30
         endif
         ind2(k) = j - ind1(k) + 1
         k = k + 1
         ind1(k) = j + 1
         goto 10
      endif
      ind2(k) = n - ind1(k) + 1
      return
      end
