      subroutine dgefam (a,lda,n,ipvt,info)
      integer lda,n,ipvt(n),info
      double precision a(lda,n)
c
c     dgefam factors a double precision real matrix by
c     gaussian elimination.
c
c     dgefam is usually called by dgecom, but it can be called
c     directly with a modest saving in time if rcond is not
c     needed.
c
c     on entry:
c
c        a       double precision(lda,n)
c                the matrix to be factored.
c
c        lda     integer
c                the leading dimension of the array a as declared
c                in the main calling program.
c
c        n       integer
c                the order of the matrix a.
c
c     on return:
c
c        a       an upper triangular matrix and the multipliers
c                which were used to obtain it.
c                the factorization can be written  a = l * u where
c                l is a product of permutation and unit lower
c                triangular matrices and u is upper triangular.
c
c        ipvt    integer(n)
c                an integer vector of pivot indices.
c                   ipvt(k) = the index of the k-th pivot row
c                the determinant of a can be obtained on output by
c                   det(a) = s*a(1,1)*a(2,2)* ... *a(n,n)
c                where s = (-1)**(number of times ipvt(k) .ne. k)
c                but this result should be used with caution as it
c                may easily underflow or overflow.
c
c        info    integer
c                = 0  normal value.
c                = k  if  u(k,k) .eq. 0.0d0.  this is not an error
c                     condition for this subroutine, but it does
c                     indicate that dgeslm will divide by zero if
c                     called.
c                     use rcond in dgecom for a reliable indication of
c                     singularity.
c
c     this version is adapted from the linpack subroutine dgefa by
c     replacement of calls to the blas with in-line code.  modification
c     done by alan j. laub, university of southern california,
c     april 1980.
c
c     fortran functions called:
c
      double precision dabs
c
c     internal variables:
c
      integer i,j,k,kp1,l,nm1
      double precision t
c
c     gaussian elimination with partial pivoting
c
      ipvt(n) = 1
      info = 0
      nm1 = n-1
      if (nm1 .lt. 1) go to 100
      do 90 k=1,nm1
         kp1 = k+1
c
c        find l = pivot index
c
         l = k
         do 10 i=kp1,n
            if (dabs(a(i,k)) .gt. dabs(a(l,k))) l = i
   10    continue
         ipvt(k) = l
c
c        zero pivot implies this column already triangularized
c
         if (a(l,k) .eq. 0.0d0) go to 70
c
c        interchange if necessary
c
         if (l .eq. k) go to 20
         t = a(l,k)
         a(l,k) = a(k,k)
         a(k,k) = t
   20    continue
c
c        compute multipliers
c
         t = 1.0d0/a(k,k)
         do 30 i=kp1,n
            a(i,k) = t*a(i,k)
   30    continue
c
c        row elimination with column indexing
c
         do 60 j=kp1,n
            t = a(l,j)
            if (l .eq. k) go to 40
            a(l,j) = a(k,j)
            a(k,j) = t
   40       continue
            do 50 i=kp1,n
               a(i,j) = a(i,j)-t*a(i,k)
   50       continue
   60    continue
         go to 80
   70    continue
         info = k
   80    continue
   90 continue
  100 continue
      ipvt(n) = n
      if (a(n,n) .eq. 0.0d0) info = n
      return
      end
