      subroutine dgeslm (a,lda,n,ipvt,b)
      integer lda,n,ipvt(n)
      double precision a(lda,n),b(n)
c
c     dgeslm solves the (real) linear system
c            a * x = b
c     using the factors computed by dgecom or dgefam.
c
c     on entry:
c
c        a       double precision(lda,n)
c                the output from dgecom or dgefam.
c
c        lda     integer
c                the leading dimension of the array a as declared
c                in the main calling program.
c
c        n       integer
c                the order of the matrix a.
c
c        ipvt    integer(n)
c                the pivot vector from dgecom or dgefam.
c
c        b       double precision(n)
c                the right hand side vector.
c
c     on return:
c
c        b       the solution vector x.
c
c     error condition:
c
c        a division by zero will occur if the input factor contains a
c        zero on the diagonal.  technically this indicates singularity
c        but it is often caused by improper arguments or improper
c        setting of lda.  it will not occur if the subroutines are
c        called correctly and if dgecom has set rcond .gt. 0.0
c        or dgefam has set info .eq. 0.
c
c     to compute  inverse(a) * b  where b is a matrix with m columns
c             call dgecom (a,lda,n,ipvt,rcond,z)
c             if (rcond is too small) go to ...
c             do 10 j=1,m
c                call dgeslm (a,lda,n,ipvt,b(1,j))
c          10 continue
c
c     this version is adapted from the linpack subroutine dgesl by
c     replacement of calls to the blas with in-line code.  modifica-
c     tion done by alan j. laub, university of southern california,
c     april 1980.
c
c     fortran functions called:
c
c     none
c
c     internal variables:
c
      integer i,k,kb,km1,kp1,l,nm1
      double precision t
c
      if (n .eq. 1) go to 60
      nm1 = n-1
c
c     first solve l*y = b
c
      do 30 k=1,nm1
         kp1 = k+1
         l = ipvt(k)
         t = b(l)
         if (l .eq. k) go to 10
         b(l) = b(k)
         b(k) = t
   10    continue
         do 20 i=kp1,n
            b(i) = b(i)-t*a(i,k)
   20    continue
   30 continue
c
c     now solve  u*x = y
c
      do 50 kb=1,nm1
         k = n+1-kb
         km1 = k-1
         b(k) = b(k)/a(k,k)
         t = -b(k)
         do 40 i=1,km1
            b(i) = b(i)+t*a(i,k)
   40    continue
   50 continue
   60 continue
      b(1) = b(1)/a(1,1)
      return
      end
