      subroutine dhetr (na,nb,nc,l,m,n,low,igh,a,b,c,ort)
      integer na,nb,nc,l,m,n,low,igh
      real*8 a(na,n),b(nb,m),c(nc,n),ort(n)
c
c     *** purpose:
c
c     given a real general matrix a, dhetr reduces a submatrix
c     of a in rows and columns low through igh to upper hessenberg
c     form by orthogonal similarity transformations.  these
c     orthogonal transformations are further accumulated into rows
c     low through igh of an n x m matrix b and columns low
c     through igh of an l x n matrix c by premultiplication and
c     postmultiplication, respectively.
c
c     the code is adapted from the eispack subroutine orthes
c     and the orthes documentation may be consulted for further
c     numerical details concerning householder transformations.
c
c     on entry:
c
c        na,nb,nc integers
c                 the leading or row dimensions of the arrays
c                 a, b, and c, respectively, as declared in the
c                 main calling program.
c
c        l        integer
c                 number of rows of c.
c
c        m        integer
c                 number of columns of b.
c
c        n        integer
c                 order of a and number of rows of b and columns of c.
c
c        low,igh  integers
c                 integers determined by the balancing subroutine
c                 balanc.  if balanc has not been used, set  low = 1
c                 and  igh = n.
c
c        a        real(na,n)
c                 an n x n real general matrix to be reduced to
c                 upper hessenberg form.
c
c        b        real(nb,m)
c                 an n x m real matrix.
c
c        c        real(nc,n)
c                 an l x n real matrix.
c
c     on return:
c
c        a        an upper hessenberg matrix similar to (via an
c                 orthogonal matrix consisting of a sequence of
c                 householder transformations) the original matrix a;
c                 further information concerning the orthogonal
c                 transformations used in the reduction is contained
c                 in the elements below the first subdiagonal; see
c                 orthes documentation for details.
c
c        b        the original b matrix premultiplied by the transpose
c                 of the orthogonal transformation used to reduce a.
c
c        c        the original c matrix postmultiplied by the orthogonal
c                 transformation used to reduce a.
c
c        ort      real(n)
c                 a work vector containing information about the
c                 orthogonal transformations; see orthes documentation
c                 for details.
c
c     this version dated july 1980.
c     alan j. laub, university of southern california.
c
c     subroutines and functions called:
c
c     none
c
c     internal variables:
c
      integer i,ii,j,jj,k,kp1,kpn,la
      real*8 f,g,h,scale
c
      la = igh-1
      kp1 = low+1
      if (la .lt. kp1) go to 170
      do 160 k = kp1,la
         h = 0.0d0
         ort(k) = 0.0d0
         scale = 0.0d0
c
c        scale column
c
         do 10 i = k,igh
              scale = scale+abs(a(i,k-1))
   10    continue
         if (scale .eq. 0.0d0) go to 150
         kpn = k+igh
         do 20 ii = k,igh
              i = kpn-ii
              ort(i) = a(i,k-1)/scale
              h = h+ort(i)*ort(i)
   20    continue
         g = -sign(sqrt(h),ort(k))
         h = h-ort(k)*g
         ort(k) = ort(k)-g
c
c        form  (i-(u*transpose(u))/h)*a
c
         do 50 j = k,n
              f = 0.0d0
              do 30 ii = k,igh
                   i = kpn-ii
                   f = f+ort(i)*a(i,j)
   30         continue
              f = f/h
              do 40 i = k,igh
                   a(i,j) = a(i,j)-f*ort(i)
   40         continue
   50    continue
c
c        form  (i-(u*transpose(u))/h)*b
c
         do 80 j = 1,m
              f = 0.0d0
              do 60 ii = k,igh
                   i = kpn-ii
                   f = f+ort(i)*b(i,j)
   60         continue
              f = f/h
              do 70 i = k,igh
                   b(i,j) = b(i,j)-f*ort(i)
   70         continue
   80    continue
c
c        form  (i-(u*transpose(u))/h)*a*(i-(u*transpose(u))/h)
c
         do 110 i = 1,igh
              f = 0.0d0
              do 90 jj = k,igh
                   j = kpn-jj
                   f = f+ort(j)*a(i,j)
   90         continue
              f = f/h
              do 100 j = k,igh
                   a(i,j) = a(i,j)-f*ort(j)
  100         continue
  110    continue
c
c        form  c*(i-(u*transpose(u))/h)
c
         do 140 i = 1,l
              f = 0.0d0
              do 120 jj = k,igh
                   j = kpn-jj
                   f = f+ort(j)*c(i,j)
  120         continue
              f = f/h
              do 130 j = k,igh
                   c(i,j) = c(i,j)-f*ort(j)
  130         continue
  140    continue
         ort(k) = scale*ort(k)
         a(k,k-1) = scale*g
  150    continue
  160 continue
  170 continue
      return
      end
