      subroutine eigvec (h,z,n,low,igh,eps,wr,wi,nh,nz)
c
      integer i,j,k,m,n,en,ii,jj,na,nh,nz,nn,igh,low,enm2
      real*8 h(nh,n),wr(n),wi(n),z(nz,n)
      real*8 p,q,r,s,t,w,x,y,ra,sa,vi,vr,zz,small,eps
      complex z3
c
c     this subroutine is a translation of the algol procedure hqr2,
c     num. math. 16, 181-204(1970) by peters and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 372-395(1971).
c
c     this subroutine finds eigenvectors of a quasi-triangular matrix.
c     eigenvectors of a real general matrix can also be found
c     if  elmhes  and  eltran  or  orthes  and  ortran  have
c     been used to reduce this general matrix to hessenberg form
c     and to accumulate the similarity transformations.
c
c     on input-
c
c        h contains the quasi-triangular matrix,
c
c        z contains the transformation matrix produced by  eltran
c          after the reduction by  elmhes, or by  ortran  after the
c          reduction by  orthes, if performed.  if the eigenvectors
c          of the hessenberg matrix are desired, z must contain the
c          identity matrix,
c
c        n is the order of the matrix,
c
c        low and igh are integers determined by the balancing
c          subroutine  balanc.  if  balanc  has not been used,
c          set low=1, igh=n,
c
c        eps is a machine dependent parameter specifying
c          the relative precision of floating point arithmetic,
c
c        wr and wi contain the real and imaginary parts,
c          respectively, of the eigenvalues,
c
c        nh must be set to the row dimension of the h
c          array as declared in the calling program
c          dimension statement,
c
c        nz must be set to the row dimension of the z
c          array as declared in the calling program
c          dimension statement.
c
c     on output-
c
c        h has been destroyed,
c
c        z contains the real and imaginary parts of the eigenvectors.
c          if the i-th eigenvalue is real, the i-th column of z
c          contains its eigenvector.  if the i-th eigenvalue is complex
c          with positive imaginary part, the i-th and (i+1)-th
c          columns of z contain the real and imaginary parts of its
c          eigenvector.  the eigenvectors are unnormalized.
c
c     ------------------------------------------------------------------
c
      small = 0.0
c     ********** compute normalized small number **********
      do 40 i = 1, n
         do 40 j = max(1,i-1), n
         small = small + abs(h(i,j))
   40 continue
      small = eps*small
c
      if (small .eq. 0.0) go to 1001
c     ********** for en=n step -1 until 1 do -- **********
      do 800 nn = 1, n
         en = n + 1 - nn
         p = wr(en)
         q = wi(en)
         na = en - 1
         if (q) 710, 600, 800
c     ********** real vector **********
  600    m = en
         h(en,en) = 1.0
         if (na .eq. 0) go to 800
c     ********** for i=en-1 step -1 until 1 do -- **********
         do 700 ii = 1, na
            i = en - ii
            w = h(i,i) - p
            r = h(i,en)
            if (m .gt. na) go to 620
c
            do 610 j = m, na
  610       r = r + h(i,j) * h(j,en)
c
  620       if (wi(i) .ge. 0.0) go to 630
            zz = w
            s = r
            go to 700
  630       m = i
            if (wi(i) .ne. 0.0) go to 640
            t = w
            if (w .eq. 0.0) t = small
            h(i,en) = -r / t
            go to 700
c     ********** solve real equations **********
  640       x = h(i,i+1)
            y = h(i+1,i)
            q = (wr(i) - p) * (wr(i) - p) + wi(i) * wi(i)
            t = (x * s - zz * r) / q
            h(i,en) = t
            if (abs(x) .le. abs(zz)) go to 650
            h(i+1,en) = (-r - w * t) / x
            go to 700
  650       h(i+1,en) = (-s - y * t) / zz
  700    continue
c     ********** end real vector **********
         go to 800
c     ********** complex vector **********
  710    m = na
c     ********** last vector component chosen imaginary so that
c                eigenvector matrix is triangular **********
         if (abs(h(en,na)) .le. abs(h(na,en))) go to 720
         h(na,na) = q / h(en,na)
         h(na,en) = -(h(en,en) - p) / h(en,na)
         go to 730
c 720    z3 = cmplx(0.0,-h(na,en)) / cmplx(h(na,na)-p,q)
  720    z3 = cmplx(-h(na,en)) / cmplx(h(na,na)-p,q)
         h(na,na) = real(z3)
         h(na,en) = aimag(z3)
  730    h(en,na) = 0.0
         h(en,en) = 1.0
         enm2 = na - 1
         if (enm2 .eq. 0) go to 800
c     ********** for i=en-2 step -1 until 1 do -- **********
         do 790 ii = 1, enm2
            i = na - ii
            w = h(i,i) - p
            ra = 0.0
            sa = h(i,en)
c
            do 760 j = m, na
               ra = ra + h(i,j) * h(j,na)
               sa = sa + h(i,j) * h(j,en)
  760       continue
c
            if (wi(i) .ge. 0.0) go to 770
            zz = w
            r = ra
            s = sa
            go to 790
  770       m = i
            if (wi(i) .ne. 0.0) go to 780
            z3 = cmplx(-ra,-sa) / cmplx(w,q)
            h(i,na) = real(z3)
            h(i,en) = aimag(z3)
            go to 790
c     ********** solve complex equations **********
  780       x = h(i,i+1)
            y = h(i+1,i)
            vr = (wr(i) - p) * (wr(i) - p) + wi(i) * wi(i) - q * q
            vi = (wr(i) - p) * 2.0 * q
            if (vr .eq. 0.0 .and. vi .eq. 0.0) vr = small
     x       * (abs(w) + abs(q) + abs(x) + abs(y) + abs(zz))
            z3 = cmplx(x*r-zz*ra+q*sa,x*s-zz*sa-q*ra) / cmplx(vr,vi)
            h(i,na) = real(z3)
            h(i,en) = aimag(z3)
            if (abs(x) .le. abs(zz) + abs(q)) go to 785
            h(i+1,na) = (-ra - w * h(i,na) + q * h(i,en)) / x
            h(i+1,en) = (-sa - w * h(i,en) - q * h(i,na)) / x
            go to 790
  785       z3 = cmplx(-r-y*h(i,na),-s-y*h(i,en)) / cmplx(zz,q)
            h(i+1,na) = real(z3)
            h(i+1,en) = aimag(z3)
  790    continue
c     ********** end complex vector **********
  800 continue
c     ********** end back substitution.
c                vectors of isolated roots **********
      do 840 i = 1, n
         if (i .ge. low .and. i .le. igh) go to 840
c
         do 820 j = i, n
  820    z(i,j) = h(i,j)
c
  840 continue
c     ********** multiply by transformation matrix to give
c                vectors of original full matrix.
c                for j=n step -1 until low do -- **********
      do 880 jj = low, n
         j = n + low - jj
         m = min0(j,igh)
c
         do 880 i = low, igh
            zz = 0.0
c
            do 860 k = low, m
  860       zz = zz + z(i,k) * h(k,j)
c
            z(i,j) = zz
  880 continue
c
 1001 continue
      return
c     ********** last card of eigvec **********
      end
