      subroutine exchng (a,v,n,l,b1,b2,eps,fail,na,nv)
c
c     *****parameters
      integer n,b1,b2,l,na,nv
      double precision a(na,n),eps,v(nv,n)
      logical fail
c
c     *****local variables:
      integer i,it,j,l1,m
      double precision p,q,r,s,w,x,y,z
c
c     *****functions:
      double precision dabs,dsqrt,dmax1
c
c     *****subroutines called:
c     qrstep
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     given the upper hessenberg matrix a with consecutive b1 x b1 and
c     b2 x b2 diagonal blocks (b1, b2.le.2) starting at a(l,l), this
c     subroutine produces a unitary similarity transformation that
c     exchanges the blocks along with their eigenvalues. the
c     transformation is accumulated in v.
c
c     *****parameter description:
c     on input:
c        na,nv            row dimensions of the arrays containing a
c                         and v, respectively, as declared in the
c                         calling program dimension statement;
c
c        a                n x n matrix whose blocks are to be
c                         interchanged;
c
c        n                order of the matrix a;
c
c        l                position of the blocks;
c
c        b1               an integer containing the size of the first
c                         block;
c
c        b2               an integer containing the size of the second
c                         block;
c
c        eps              a convergence criterion (cf. hqr3).
c
c     on output:
c
c        fail             a logical variable which is .false. on a
c                         normal return. if thirty iterations were
c                         performed without convergence, fail is set to
c                         .true. and the element a(l+b2,l+b2-1) cannot
c                         be assumed zero.
c
c     *****history:
c              adapted from stewart's hqr3 software.
c
c     ------------------------------------------------------------------
c
      fail=.false.
      if (b1.eq.2) go to 70
      if (b2.eq.2) go to 40
      l1=l+1
      q=a(l+1,l+1)-a(l,l)
      p=a(l,l+1)
      r=dmax1(dabs(p),dabs(q))
      if (r.eq.0.0d0) return
      p=p/r
      q=q/r
      r=dsqrt(p**2+q**2)
      p=p/r
      q=q/r
      do 10 j=l,n
         s=p*a(l,j)+q*a(l+1,j)
         a(l+1,j)=p*a(l+1,j)-q*a(l,j)
         a(l,j)=s
10    continue
      do 20 i=1,l1
         s=p*a(i,l)+q*a(i,l+1)
         a(i,l+1)=p*a(i,l+1)-q*a(i,l)
         a(i,l)=s
20    continue
      do 30 i=1,n
         s=p*v(i,l)+q*v(i,l+1)
         v(i,l+1)=p*v(i,l+1)-q*v(i,l)
         v(i,l)=s
30    continue
      a(l+1,l)=0.0d0
      return
40    continue
      x=a(l,l)
      p=1.0d0
      q=1.0d0
      r=1.0d0
      call qrstep (a,v,p,q,r,l,l+2,n,na,nv)
      it=0
50    it=it+1
      if (it.le.30) go to 60
      fail=.true.
      return
60    continue
      p=a(l,l)-x
      q=a(l+1,l)
      r=0.0d0
      call qrstep (a,v,p,q,r,l,l+2,n,na,nv)
      if (dabs(a(l+2,l+1)).gt.eps*(dabs(a(l+1,l+1))+dabs(a(l+2,l+2))))
     +    go to 50
      a(l+2,l+1)=0.0d0
      return
70    continue
      m=l+2
      if (b2.eq.2) m=m+1
      x=a(l+1,l+1)
      y=a(l,l)
      w=a(l+1,l)*a(l,l+1)
      p=1.0d0
      q=1.0d0
      r=1.0d0
      call qrstep (a,v,p,q,r,l,m,n,na,nv)
      it=0
80    it=it+1
      if (it.le.30) go to 90
      fail=.true.
      return
90    continue
      z=a(l,l)
      r=x-z
      s=y-z
      p=(r*s-w)/a(l+1,l)+a(l,l+1)
      q=a(l+1,l+1)-z-r-s
      r=a(l+2,l+1)
      s=dabs(p)+dabs(q)+dabs(r)
      p=p/s
      q=q/s
      r=r/s
      call qrstep (a,v,p,q,r,l,m,n,na,nv)
      if (dabs(a(m-1,m-2)).gt.eps*(dabs(a(m-1,m-1))+dabs(a(m-2,m-2))))
     +    go to 80
      a(m-1,m-2)=0.0d0
      return
c
c     last line of exchng
c
      end
