      subroutine gradbk (n,mz,z,m,low,igh,cperm)
c
c     *****parameters:
      integer igh,low,m,mz,n
      double precision cperm(n),z(mz,n)
c
c     *****local variables:
      integer i,ighm1,ii,j,k
      double precision temp
c
c     *****functions:
c     none.
c
c     *****subroutines called:
c     none.
c
c     ---------------------------------------------------------------
c
c     *****purpose:
c     this subroutine back transforms the eigenvectors of a
c     generalized eigenvalue problem a*x = (lambda)*b*x graded by
c     the subroutine gradeq to those of the eigenproblem before
c     gradeq was applied.
c     ref.:  ward, r. c., balancing the generalized eigenvalue
c     problem, siam j. sci. stat. comput., vol. 2, no. 2, june 1981,
c     141-152.
c
c     *****parameter description:
c     on input:
c       mz      integer
c               row dimension of the array z as specified in the
c               calling program dimension statement;
c
c       n       integer
c               order of the matrices a and b in the eiggenproblem;
c
c       m       integer
c               specifies the number of eigenvectors to be trans-
c               formed;
c
c       z       real(mz,n)
c               contains the eigenvectors to be transformed;
c
c       low     integer
c               specifies the beginning index of the submatrices of
c               a and b which were graded;
c
c       igh     integer
c               specifies the ending index of the submatrices of
c               a and b which were graded;
c
c       cperm   real(n)
c               contains in its low through igh locations the column
c               permutations applied in grading the a and b
c               submatrices.
c
c     on output:
c       z       real(mz,n)
c               contains the transformed eigenvectors.
c
c     *****algorithm notes:
c     none.
c
c     *****history:
c     written by r. c. ward.......
c
c     ---------------------------------------------------------------
c
      if (low .eq. igh) go to 530
      ighm1 = igh-1
      do 520 ii = low,ighm1
         i = ighm1 + low - ii
         k = cperm(i)
         if (k .eq. i) go to 520
         do 510 j = 1,m
            temp = z(i,j)
            z(i,j) = z(k,j)
            z(k,j) = temp
  510    continue
  520 continue
  530 continue
      return
c
c     last line of gradbk
c
      end
