      subroutine hqr4 (a,v,n,nlow,nup,eps,ifail,na,nv)
c
c     *****parameters:
      integer n,na,nlow,nup,nv,ifail
      double precision a(na,n),eps,v(nv,n)
c
c     *****local variables:
      integer i,it,l,nl,nu
      double precision p,q,r,s,t,w,x,y,z,teps
c
c     *****functions:
      double precision dabs
c
c     *****subroutines called:
c     qrstep
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     this subroutine reduces the upper hessenberg matrix a to quasi-
c     triangular form by unitary similarity transformations. the
c     eigenvalues of a are contained in the 1 x 1 and 2 x 2
c     diagonal blocks of the reduced matrix.  the transformations are
c     accumulated in the array v.
c
c     *****parameter description:
c     on input:
c        na,nv            row dimensions of the arrays containing a and
c                         v, respectively, as declared in the calling
c                         program dimension statement;
c
c        a                n x n array containing the upper hessenberg
c                         matrix to be reduced;
c
c        n                order of the matrices a and v;
c
c        nlow,nup         a(nlow,nlow-1) and a(nup,1+nup) are assumed
c                         to be zero, and only rows nlow through nup
c                         and columns nlow through nup are transformed,
c                         resulting in the calculation of eigenvalues
c                         nlow through nup;
c
c        eps              a convergence criterion used to determine when
c                         a subdiagonal element of a is negligible.
c                         specifically, a(i+1,i) is regarded as
c                         negligible if dabs(a(i+1),i)).le.eps*
c                         (dabs(a(i+1,i+1))). this means that the final
c                         matrix returned by the program will be exactly
c                         similar to a + e where e is of order
c                         eps*norm(a), for any reasonably balanced norm
c                         such as the row-sum norm;
c
c        on output:
c
c        a                n x n array containing the reduced, quasi-
c                         triangular matrix;
c
c        v                n x n array containing the reducing
c                         transformations to be multiplied;
c
c        ifail            an integer whose value is
c                         0        if all the eigenvalues were isolated
c                         nonzero  if the nlow through ifail eigenvalues
c                                  were not isolated
c
c     *****history:
c              adapted from stewart's hqr3 software.
c
c     ------------------------------------------------------------------
c
      ifail = 0
      t = 0.0d0
      nu = nup
20    continue
      if (nu.lt.nlow) go to 120
      it = 0
      teps = eps
30    continue
      l = nu
40    continue
      if (l.eq.nlow) go to 50
      if (dabs(a(l,l-1)).le.teps*(dabs(a(l-1,l-1))+dabs(a(l,l))))
     +    go to 50
      l = l-1
      go to 40
50    continue
      x = a(nu,nu)
      if (l.eq.nu) then
	 a(nu,nu) = a(nu,nu)+t
	 if (nu.ne.nlow) a(nu,nu-1) = 0.0d0
	 nu  =  nu - 1
	 go to 20
      endif
      y = a(nu-1,nu-1)
      w = a(nu,nu-1)*a(nu-1,nu)
      if (l.eq.nu-1) then
	 a(nu,nu) = a(nu,nu)+t
	 a(nu-1,nu-1) = a(nu-1,nu-1)+t
	 if (nu.ne.nlow+1) a(nu-1,nu-2) = 0.0d0
	 nu  =  nu - 2
	 go to 20
      endif
      if (it.eq.30) then
	 if (ifail .eq. 0) ifail = nu
	 teps = teps*2.0d0
	 goto 30
      endif
      if (it.eq.10 .or. it.eq.20) then
	 t = t+x
	 do 60 i = nlow,nu
	    a(i,i) = a(i,i)-x
60       continue
	 s = dabs(a(nu,nu-1))+dabs(a(nu-1,nu-2))
	 x = 0.75d0*s
	 y = x
	 w = -0.4375d0*s**2
      endif
      it = it+1
      nl = nu-2
80    continue
      z = a(nl,nl)
      r = x-z
      s = y-z
      p = (r*s-w)/a(nl+1,nl)+a(nl,nl+1)
      q = a(nl+1,nl+1)-z-r-s
      r = a(nl+2,nl+1)
      s = dabs(p)+dabs(q)+dabs(r)
      p = p/s
      q = q/s
      r = r/s
      if (nl.eq.l) go to 90
      if (dabs(a(nl,nl-1))*(dabs(q)+dabs(r)).le.eps*dabs(p)*
     +    (dabs(a(nl-1,nl-1))+dabs(z)+dabs(a(nl+1,nl+1)))) go to 90
      nl = nl-1
      go to 80
90    continue
      call qrstep (a,v,p,q,r,nl,nu,n,na,nv)
      go to 30
100   continue
      do 110 i = nlow,nu
	 a(i,i) = a(i,i)+t
110   continue
      ifail = nu
120   continue
      return
c
c     last line of hqr4
c
      end
