      subroutine mlineq (na,nb,n,m,a,b,cond,ipvt,work)
c
c     *****parameters:
      integer na,nb,n,m,ipvt(n)
      double precision a(na,n),b(nb,m),cond,work(n)
c
c     *****local variables:
      integer j
c
c     *****subroutines called:
c     dgecom,dgeslm
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     this subroutine solves the matrix linear equation
c                             a*x = b
c     where a is an n x n (invertible) matrix and b is an n x m
c     matrix.  subroutine dgecom is called once for the lu-factor-
c     ization of a and subroutine dgeslm is called m times for
c     forward elimination and back substitution to produce the
c     m columns of the solution matrix x = (a-inverse)*b.  an
c     estimate of the condition of a is returned.  should a be
c     singular to working accuracy, a message to that effect is
c     produced.
c
c     *****parameter description:
c     on input:
c        na,nb            row dimensions of the arrays containing a and
c                         b, respectively, as declared in the calling
c                         program dimension statement;
c
c        n                order of the matrix a and number of rows of
c                         the matrix b;
c
c        m                number of columns of the matrix b;
c
c        a                n x n coefficient matrix;
c
c        b                n x m right hand side matrix.
c
c     on output:
c
c        b                solution matrix  x = (a-inverse)*b;
c
c        cond             an estimate of the condition of a.
c                         = 1/rcond  where rcond is the inverse
c                         of the condition estimate (see the linpack
c                         user's guide for details);
c
c        ipvt             pivot vector of length n (see dgecom
c                         documentation);
c
c        work             a real scratch vector of length n.
c
c     *****applications and usage restrictions:
c     (1)the value of cond should always be checked by the calling
c        program.  should a be near-singular (or singular to working
c        accuracy) the data should be investigated for possible
c        errors.  if there are none and the problem is apparently
c        well-posed and/or meaningful, singular value analysis may
c        then be a more reliable solution technique (e.g., eispack
c        subroutines  svd  and  minfit).
c     (2)mlineq can be used to compute the inverse of a:  simply solve
c        a*x = i where  i  is the n x n identity matrix.
c     (3)if the solution to x*a = b  (x = b*(a-inverse)) is desired,
c        simply transpose the solution of
c         t      t
c        a *x = b .
c
c     *****algorithm notes:
c     the contents of a are modified by this subroutine.  should the
c     original coefficients of a be needed subsequently, the
c     contents of a should be saved prior to the call to mlineq.
c
c     *****history:
c     written by alan j. laub (dep't. of elec. engrg. - systems,
c     university of southern california, los angeles, ca 90089,
c     ph.: (213)-743-5535), may 1980.
c     most recent version:  may 6, 1980.
c
c     ------------------------------------------------------------------
c
      call dgecom (a,na,n,ipvt,cond,work)
      if (cond .eq. 0.0d0) return
      cond = 1.0d0/cond
      do 30 j=1,m
c
c        compute  (j-th column of x) = (a-inverse)*(j-th column of b)
c
         call dgeslm (a,na,n,ipvt,b(1,j))
   30 continue
      return
      end
