      subroutine qrstep (a,v,p,q,r,nl,nu,n,na,nv)
c
c     *****parameters:
      integer n,na,nl,nu,nv
      double precision a(na,n),p,q,r,v(nv,n)
c
c     *****local variables:
      logical last
      integer i,j,k,nl2,nl3,num1
      double precision s,x,y,z
c
c     *****functions:
      integer min0
      double precision dabs,dsqrt
c
c     *****subroutines called:
c     none
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     this subroutine performs one implicit qr step on the upper
c     hessenberg matrix a. the shift is determined by the numbers p,q,
c     and r, and the step is applied to rows and columns nl throuth nu.
c     the transformations are accumulated in the array v.
c
c     *****parameter description:
c     on input:
c        na,nv            row dimensions of the arrays containing a
c                         and v, respectively, as declared in the
c                         calling program dimension statement;
c
c        a                n x n upper hessenberg matrix on which the qr
c                         step is to be performed;
c
c        p,q,r            parameters which determine the shift;
c
c        nl               the lower limit of the step;
c
c        nu               the upper limit of the step;
c
c        n                order of the matrix a.
c
c     on output:
c
c        v                n x n real scratch array containing the
c                         accumulated transformations.
c
c     *****history:
c              adapted from stewart's hqr3 software.
c
c     ------------------------------------------------------------------
c
      nl2=nl+2
      do 10 i=nl2,nu
         a(i,i-2)=0.0d0
10    continue
      if (nl2.eq.nu) go to 30
      nl3=nl+3
      do 20 i=nl3,nu
         a(i,i-3)=0.0d0
20    continue
30    continue
      num1=nu-1
      do 130 k=nl,num1
         last=k.eq.num1
         if (k.eq.nl) go to 40
         p=a(k,k-1)
         q=a(k+1,k-1)
         r=0.0d0
         if (.not.last) r=a(k+2,k-1)
         x=dabs(p)+dabs(q)+dabs(r)
         if (x.eq.0.0d0) go to 130
         p=p/x
         q=q/x
         r=r/x
40       continue
         s=dsqrt(p**2+q**2+r**2)
         if (p.lt.0.0d0) s=-s
         if (k.eq.nl) go to 50
         a(k,k-1)=-s*x
         go to 60
50       continue
         if (nl.ne.1) a(k,k-1)=-a(k,k-1)
60       continue
         p=p+s
         x=p/s
         y=q/s
         z=r/s
         q=q/p
         r=r/p
         do 80 j=k,n
            p=a(k,j)+q*a(k+1,j)
            if (last) go to 70
            p=p+r*a(k+2,j)
            a(k+2,j)=a(k+2,j)-p*z
70          continue
            a(k+1,j)=a(k+1,j)-p*y
            a(k,j)=a(k,j)-p*x
80       continue
         j=min0(k+3,nu)
         do 100 i=1,j
            p=x*a(i,k)+y*a(i,k+1)
            if (last) go to 90
            p=p+z*a(i,k+2)
            a(i,k+2)=a(i,k+2)-p*r
90          continue
            a(i,k+1)=a(i,k+1)-p*q
            a(i,k)=a(i,k)-p
100      continue
         do 120 i=1,n
            p=x*v(i,k)+y*v(i,k+1)
            if (last) go to 110
            p=p+z*v(i,k+2)
            v(i,k+2)=v(i,k+2)-p*r
110         continue
            v(i,k+1)=v(i,k+1)-p*q
            v(i,k)=v(i,k)-p
120      continue
130   continue
      return
c
c     last line of qrstep
c
      end
