      subroutine scalbk (n,mz,z,m,low,igh,cscale)
c
c     *****parameters:
      integer igh,low,m,mz,n
      double precision cscale(n),z(mz,n)
c
c     *****local variables:
      integer i,ighp1,ii,ir,j,k,lowm1
      double precision fi,temp
c
c     *****functions:
c     none.
c
c     *****subroutines called:
c     none.
c
c     ---------------------------------------------------------------
c
c     *****purpose:
c     this subroutine back transforms the eigenvectors of a
c     generalized eigenvalue problem a*x = (lambda)*b*x balanced by
c     subroutines reduce and/or scaleg to those of the original
c     problem.
c     ref.:  ward, r. c., balancing the generalized eigenvalue
c     problem, siam j. sci. stat. comput., vol. 2, no. 2, june 1981,
c     141-152.
c
c     *****parameter description:
c     on input:
c       mz      integer
c               row dimension of the array z as specified in the
c               calling program dimension statement;
c
c       n       integer
c               order of the matrices a and b in the eiggenproblem;
c
c       m       integer
c               specifies the number of eigenvectors to be trans-
c               formed;
c
c       z       real(mz,n)
c               contains the eigenvectors to be transformed;
c
c       low     integer
c               specifies the beginning index of the submatrices of
c               a and b which were scaled;
c
c       igh     integer
c               specifies the ending index of the submatrices of
c               a and b which were scaled;
c
c       cscale  real(n)
c               contains the reducing column permutations and scaling
c               information as returned from reduce and scaleg.  if
c               reduce was not called, set low to 1 and igh to n.
c               if scaleg was not called, set the scale factors in
c               cscale locations low through igh to zero.
c
c     on output:
c       z       real(mz,n)
c               contains the transformed eigenvectors.
c
c     *****algorithm notes:
c     none.
c
c     *****history:
c     written by r. c. ward.......
c
c     ---------------------------------------------------------------
c
      if (low .eq. igh) go to 545
c
c     apply scaling transformation
c
      do 540 i = low,igh
         ir = cscale(i)
         fi = 2.0d0**ir
         do 530 j = 1,m
            z(i,j) = z(i,j)*fi
  530    continue
  540 continue
  545 continue
c
c     apply reducing column permutations
c
      if (low .eq. 1) go to 570
      lowm1 = low-1
      do 560 ii = 1,lowm1
         i = low-ii
         k = cscale(i)
         if (k .eq. i) go to 560
         do 550 j = 1,m
            temp = z(i,j)
            z(i,j) = z(k,j)
            z(k,j) = temp
  550    continue
  560 continue
  570 continue
      if (igh .eq. n) go to 600
      ighp1 = igh+1
      do 590 i = ighp1,n
         k = cscale(i)
         if (k .eq. i) go to 590
         do 580 j = 1,m
            temp = z(i,j)
            z(i,j) = z(k,j)
            z(k,j) = temp
  580    continue
  590 continue
  600 continue
      return
c
c     last line of scalbk
c
      end
