      subroutine sfrmg (first,na,nb,nc,l,m,n,a,b,c,freq,ainvb,g,h,ipvt,
     +                  zc,zr,evre,evim,ierr,info,rcond)
      integer na,nb,nc,l,m,n,ipvt(n),ierr,info
      real a(na,n),b(nb,m),c(nc,n),zr(n),evre(n),evim(n),rcond
      complex freq,ainvb(nb,m),g(nc,m),h(na,n),zc(n)
      logical first
c
c     *** purpose:
c
c     sfrmg takes real matrices a (n x n), b (n x m), and c (l x n)
c     and forms the complex frequency response matrix
c           g(freq) := c * (((freq * i) - a)-inverse) * b
c     where  i = (n x n) identity matrix and  freq  is a complex
c     scalar parameter taking values along the imaginary axis for
c     continuous-time systems and on the unit circle for discrete-
c     time systems.
c
c     on entry:
c
c        first    logical
c                 set = .true.  for the first call of sfrmg whereupon
c                 it is set to .false. for all subsequent calls; may be
c                 set = .false.  initially if a is already in upper
c                 hessenberg form and neither balancing nor eigenvalues
c                 of a are desired.
c
c        na       integer
c                 the leading or row dimension of the real array a
c                 (and the complex array h) as declared in the main
c                 calling program.
c
c        nb       integer
c                 the leading or row dimension of the real array b
c                 (and the complex array ainvb) as declared in the main
c                 calling program.
c
c        nc       integer
c                 the leading or row dimension of the real array c
c                 (and the complex array g) as declared in the main
c                 calling program.
c
c        l        integer
c                 the number of rows of c (the number of outputs).
c
c        m        integer
c                 the number of columns of b (the number of inputs).
c
c        n        integer
c                 the order of the matrix a (the number of states);
c                 also = number of columns of c = number of rows of b.
c
c        a        real(na,n)
c                 a real n x n matrix (the system matrix); not needed as
c                 input if first .eq. .false.
c
c        b        real(nb,m)
c                 a real n x m matrix (the input matrix); not needed as
c                 input if first .eq. .false.
c
c        c        real(nc,n)
c                 a real l x n matrix (the output matrix); not needed as
c                 input if first .eq. .false.
c
c        freq     complex
c                 a complex scalar (the frequency parameter).
c
c        ierr     integer
c               --first = .false. : -ierr is ignored.
c               --first = .true.  : -if ierr .ge. 0, a will be balanced
c                                    and the eigenvalues of a will be
c                                    computed (recommended option).
c                                   -if ierr .lt. 0, a will not be bal-
c                                    anced and the eigenvalues of a will
c                                    not be computed.
c
c        info     integer
c               --if info .lt. 0, the condition of a with respect to
c                 inversion will be estimated (recommended option).
c               --if info .ge. 0, the condition of a with respect to
c                 inversion will not be estimated.
c
c     on return:
c
c        g        complex(nc,m)
c                 the frequency response matrix g(freq).
c
c        a,b,c    a is in upper hessenberg form while b and c have been
c                 suitably transformed; if  first .eq. .false.  these
c                 arrays are not further modified.
c
c        evre,    real(n),real(n)
c         evim    the real and imaginary parts, respectively, of the
c                 matrix a if sfrmg was initially called with
c                 first = .true. .and. ierr .ge. 0.
c
c        ierr     if (first = .true. .and. ierr .ge. 0 on input) then
c                 ierr is output as the ierr parameter of the eispack
c                 subroutine hqr (hqr documentation may be consulted for
c                 details); normal return is 0.
c
c        info   --ignored if info .lt. 0 on input.
c               --if info .ge. 0 on input, this is output as the info
c                 parameter of subroutine chefa (chefa may be consulted
c                 for details); normal return is 0.
c
c        rcond    real
c               --ignored if info .ge. 0 on input.
c               --if info .lt. 0 on input, this is output as the rcond
c                 parameter of subroutine checo (checo may be consulted
c                 for details); normal return is when
c                      (1.0 + rcond) .gt. 1.0.
c
c        ainvb    complex(nb,m)
c                 scratch array used for internal computations.
c
c        h        complex(na,n)
c                 scratch array used for internal computations.
c
c        ipvt     integer(n)
c                 scratch array used for internal computations.
c
c        zc       complex(n)
c                 scratch array used for internal computations.
c
c        zr       real(n)
c                 scratch array used for internal computations.
c
c     this version dated january 1981.
c     alan j. laub, university of southern california.
c
c     subroutines and functions called:
c
c     balanc(eispack),checo,chefa,chesl,hqr(eispack),shetr
c
c     internal variables:
c
      integer i,igh,j,k,kk,kp,low
      real t
c
c     fortran functions called:
c
      real real
      complex cmplx
c
      if (.not. first) go to 150
      if (ierr .ge. 0) go to 10
      low = 1
      igh = n
      go to 90
   10 continue
      call balanc (na,n,a,low,igh,zr)
c
c     adjust b and c matrices based on information in the vector
c     zr which describes the balancing of a and is defined in the
c     subroutine balanc
c
      do 40 k = 1,n
         kk = n-k+1
         if (kk .ge. low .and. kk .le. igh) go to 40
         if (kk .lt. low) kk = low-kk
         kp = zr(kk)
         if (kp .eq. kk) go to 40
c
c        permute rows of b
c
         do 20 j = 1,m
            t = b(kk,j)
            b(kk,j) = b(kp,j)
            b(kp,j) = t
   20    continue
c
c        permute columns of c
c
         do 30 i = 1,l
            t = c(i,kk)
            c(i,kk) = c(i,kp)
            c(i,kp) = t
   30    continue
c
   40 continue
      if (igh .eq. low) go to 80
      do 70 k = low,igh
         t = zr(k)
c
c        scale columns of permuted c
c
         do 50 i = 1,l
            c(i,k) = c(i,k)*t
   50    continue
c
c        scale rows of permuted b
c
         do 60 j = 1,m
            b(k,j) = b(k,j)/t
   60    continue
c
   70 continue
   80 continue
   90 continue
c
c     reduce a to hessenberg form by orthogonal similarities and
c     accumulate the orthogonal transformations into b and c
c
      call shetr (na,nb,nc,l,m,n,low,igh,a,b,c,zr)
      if (ierr .lt. 0) go to 140
c
c     temporarily store hessenberg form of a in the array h
c
      do 110 j = 1,n
         do 100 i = 1,n
            h(i,j) = cmplx(a(i,j),0.0e0)
  100    continue
  110 continue
c
c     compute the eigenvalues of a if that option is requested
c
      call hqr (na,n,low,igh,a,evre,evim,ierr)
c
c     restore upper hessenberg form of a to the array a
c
      do 130 j = 1,n
         do 120 i = 1,n
            a(i,j) = real(h(i,j))
  120    continue
  130 continue
  140 continue
      first = .false.
      if (ierr .gt. 0) return
c
c     update  h := (freq * i) - a   with appropriate value of freq
c
  150 continue
      do 170 j = 1,n
         do 160 i = 1,n
            h(i,j) = -cmplx(a(i,j),0.0e0)
  160    continue
         h(j,j) = freq+h(j,j)
  170 continue
      if (info .lt. 0) go to 180
c
c     factor the complex hessenberg matrix with no condition
c     estimate
c
      call chefa (h,na,n,ipvt,info)
      if (info .eq. 0) go to 190
      return
  180 continue
c
c     factor the complex hessenberg matrix and estimate its
c     condition
c
      call checo (h,na,n,ipvt,rcond,zc)
      t = 1.0e0+rcond
      if (rcond .eq. 0.0e0) return
  190 continue
c
c     compute  (h-inverse)*b
c
      do 220 j = 1,m
         do 200 i = 1,n
            zc(i) = cmplx(b(i,j),0.0e0)
  200    continue
         call chesl (h,na,n,ipvt,zc)
         do 210 i = 1,n
            ainvb(i,j) = zc(i)
  210    continue
  220 continue
c
c     compute  c*(h-inverse)*b
c
      do 260 j = 1,m
         do 230 i = 1,l
            g(i,j) = (0.0e0,0.0e0)
  230    continue
         do 250 k = 1,n
            do 240 i = 1,l
               g(i,j) = g(i,j)+cmplx(c(i,k),0.0e0)*ainvb(k,j)
  240       continue
c
c        g now contains the desired frequency response matrix
c
  250    continue
  260 continue
      return
      end
