	subroutine shrslc (a,b,c,m,n,na,nb,nc)
c
c   subroutines used  -  sysslv
c
c   shrslc is a fortran subroutine to solve the real matrix
c   equation a*x + x*b  = c, where c is symmetric, a is
c   in lower real schur form, and b is in upper real schur form.
c   shrslc uses the auxiliary subroutine sysslv,
c   which it communicates with through the common block
c   slvblk.
c
c   the parameters in the argument list are
c
c	a	a doubly subscripted array containing the
c		matrix a in lower real schur form.
c	b	a doubly subscripted array containing the
c		matrix b in upper real schur form.
c	c	a doubly subscripted array containing the
c		matrix c.
c	m	the order of the matrix a.
c	n	the order of the matrix b.
c	na	the first dimension of the array a.
c	nb	the first dimension of the array b.
c	nc	the first dimension of the array c.
c
	real*8 a(na,*),b(nb,*),c(nc,*),t,p
	integer m,n,na,nb,nc,k,kk,dk,l,ll,dl,lm1,i,ib,j,ja,nsys
	common /slvblk/ t(5,5),p(5),nsys
	l = 1
   10	continue
	    lm1 = l - 1
	    if (l .eq. n) then
		    dl = 1
		elseif (b(l+1,l) .eq. 0.0) then
		    dl = 1
		else
		    dl = 2
	    endif
	    ll = l + dl - 1
	    do 22 j=l,ll
		do 21 i=1,m
		    do 20 ib=1,lm1
			c(i,j) = c(i,j) - c(i,ib)*b(ib,j)
   20		    continue
   21		continue
   22	    continue
	    k = 1
   30	    continue
		if (k .eq. m) then
			dk = 1
		    elseif (a(k,k+1) .eq. 0.0) then
			dk = 1
		    else
			dk = 2
		endif
		kk = k + dk - 1
		if (dl .eq. 1) then
			if (dk .eq. 1 ) then
				do 40 ja=1,k-1
				    c(k,l) = c(k,l) - a(k,ja)*c(ja,l)
   40				continue
				t(1,1) = a(k,k) + b(l,l)
				if (t(1,1) .eq. 0.0) then
					if (c(k,l) .ne. 0.0) return 1
				    else
					c(k,l) = c(k,l)/t(1,1)
				endif
			    else
				p(1) = c(k,l)
				p(2) = c(kk,l)
				do 50 ja=1,k-1
				    p(1) = p(1) - a(k,ja)*c(ja,l)
				    p(2) = p(2) - a(kk,ja)*c(ja,l)
   50				continue
				t(1,1) = a(k,k) + b(l,l)
				t(1,2) = a(k,kk)
				t(2,1) = a(kk,k)
				t(2,2) = a(kk,kk) + b(l,l)
				nsys = 2
				call sysslv
				c(k,l) = p(1)
				c(kk,l) = p(2)
			endif
		    else
			if (dk .eq. 1) then
				p(1) = c(k,l)
				p(2) = c(k,ll)
				do 60 ja=1,k-1
				    p(1) = p(1) - a(k,ja)*c(ja,l)
				    p(2) = p(2) - a(k,ja)*c(ja,ll)
   60				continue
				t(1,1) = a(k,k) + b(l,l)
				t(1,2) = b(ll,l)
				t(2,1) = b(l,ll)
				t(2,2) = a(k,k) + b(ll,ll)
				nsys = 2
				call sysslv
				c(k,l) = p(1)
				c(k,ll) = p(2)
			    else
				p(1) = c(k,l)
				p(2) = c(kk,l)
				p(3) = c(k,ll)
				p(4) = c(kk,ll)
				do 70 ja=1,k-1
				    p(1) = p(1) - a(k,ja)*c(ja,l)
				    p(2) = p(2) - a(kk,ja)*c(ja,l)
				    p(3) = p(3) - a(k,ja)*c(ja,ll)
				    p(4) = p(4) - a(kk,ja)*c(ja,ll)
   70				continue
				t(1,1) = a(k,k) + b(l,l)
				t(1,2) = a(k,kk)
				t(1,3) = b(ll,l)
				t(1,4) = 0.0
				t(2,1) = a(kk,k)
				t(2,2) = a(kk,kk) + b(l,l)
				t(2,3) = 0.0
				t(2,4) = b(ll,l)
				t(3,1) = b(l,ll)
				t(3,2) = 0.0
				t(3,3) = a(k,k) + b(ll,ll)
				t(3,4) = a(k,kk)
				t(4,1) = 0.0
				t(4,2) = b(l,ll)
				t(4,3) = a(kk,k)
				t(4,4) = a(kk,kk) + b(ll,ll)
				nsys = 4
				call sysslv
				c(k,l) = p(1)
				c(kk,l) = p(2)
				c(k,ll) = p(3)
				c(kk,ll) = p(4)
			endif
		endif
		k = k + dk
	    if (k .le. m) goto 30
	    l = l + dl
	if (l .le. n) goto 10
	return
	end
