      subroutine split (a,v,n,l,e1,e2,na,nv)
c
c     *****parameters:
      integer l,n,na,nv
      double precision a(na,n),v(nv,n),e1,e2
c
c     *****local variables:
      integer i,j,l1
      double precision p,q,r,t,u,w,x,y,z
c
c     *****functions:
      double precision dabs,dsqrt
c
c     ****subroutines called:
c     none
c
c     ------------------------------------------------------------------
c
c     *****purpose:
c     given the upper-hessenberg matrix a with a 2 x 2 block starting at
c     a(l,l), this program determines if the corresponding eigenvalues
c     are real or complex. if they are real, a rotation is determined
c     that reduces the block to upper-triangular form with the
c     eigenvalue of largest absolute value appearing first. the
c     rotation is accumulated in the array v.
c
c     *****parameter description:
c     on input:
c        na,nv            row dimensions of the arrays containing
c                         a and v, respectively, as declared in the
c                         calling program dimension statement;
c
c        a                the upper hessenberg matrix whose 2 x 2 block
c                         is to be split;
c
c        n                order of the matrix a;
c
c        l                position of the 2 x 2 block.
c
c     on output:
c
c        v                an n x n array containing the accumulated
c                         splitting transformation;
c
c        e1,e2            real scalars. if the eigenvalues are complex,
c                         e1 and e2 contain their common real part and
c                         positive imaginary part (respectively).
c                         if the eigenvalues are real, e1 contains the
c                         one largest in absolute value and e2 contains
c                         the other one.
c
c     *****history:
c              adapted from stewart's hqr3 software.
c
c     ------------------------------------------------------------------
c
      x=a(l+1,l+1)
      y=a(l,l)
      w=a(l,l+1)*a(l+1,l)
      p=(y-x)/2.0d0
      q=p**2+w
      if (q.ge.0.0d0) go to 10
      e1=p+x
      e2=dsqrt(-q)
      return
10    continue
      z=dsqrt(q)
      if (p.lt.0.0d0) go to 20
      z=p+z
      go to 30
20    continue
      z=p-z
30    continue
      if (z.eq.0.0d0) go to 40
      r=-w/z
      go to 50
40    continue
      r=0.0d0
50    continue
      if (dabs(x+z).ge.dabs(x+r)) z=r
      y=y-x-z
      x=-z
      t=a(l,l+1)
      u=a(l+1,l)
      if (dabs(y)+dabs(u).le.dabs(t)+dabs(x)) go to 60
      q=u
      p=y
      go to 70
60    continue
      q=x
      p=t
70    continue
      r=dsqrt(p**2+q**2)
      if (r.gt.0.0d0) go to 80
      e1=a(l,l)
      e2=a(l+1,l+1)
      a(l+1,l)=0.0d0
      return
80    continue
      p=p/r
      q=q/r
      do 90 j=l,n
         z=a(l,j)
         a(l,j)=p*z+q*a(l+1,j)
         a(l+1,j)=p*a(l+1,j)-q*z
90    continue
      l1=l+1
      do 100 i=1,l1
         z=a(i,l)
         a(i,l)=p*z+q*a(i,l+1)
         a(i,l+1)=p*a(i,l+1)-q*z
100   continue
      do 110 i=1,n
         z=v(i,l)
         v(i,l)=p*z+q*v(i,l+1)
         v(i,l+1)=p*v(i,l+1)-q*z
110   continue
      a(l+1,l)=0.0d0
      e1=a(l,l)
      e2=a(l+1,l+1)
      return
c
c     last line of split
c
      end
