      subroutine zhefa (h,ldh,n,ipvt,info)
      integer ldh,n,ipvt(n),info
      complex*16 h(ldh,n)
c
c     *** purpose:
c
c     zhefa factors a complex upper hessenberg matrix by gaussian
c     elimination.  the linpack analogue of zhefa is zgefa.
c
c     zhefa is usually called by zheco, but it can be called directly
c     with a substantial saving in time if rcond is not needed.
c
c     on entry:
c
c        h    complex(ldh,n)
c             the matrix to be factored.
c
c        ldh  integer
c             the leading or row dimension of the array h as declared
c             in the main calling program.
c
c        n    integer
c             the order of the matrix h.
c
c     on return:
c
c        h    an upper triangular matrix and the multipliers which
c             were used to obtain it.
c             the factorization can be written  h = l*u  where l is
c             a product of permutation and unit lower bidiagonal
c             matrices and u is upper triangular.
c
c        ipvt integer(n)
c             an integer vector of pivot indices.
c
c        info integer
c             = 0  normal value
c             = k  if  u(k,k) .eq. (0.0,0.0)  this is not an error
c                  condition for this subroutine, but it does indicate
c                  that zhesl will attempt to divide by zero if called.
c                  use rcond in checo for a reliable indication of
c                  singularity.
c
c     this version dated may 1980.
c     alan j. laub, university of southern california.
c
c     subroutines and functions called:
c
c     dscl1
c
      real*8 dscl1
c
c     internal variables:
c
      integer j,k,kp1,l,nm1
      complex*16 t
c
c     fortran functions called:
c
c     none
c
c     gaussian elimination with partial pivoting
c
      info = 0
      if (n .eq. 0) return
      if (n .eq. 1) go to 70
      nm1 = n-1
      do 60 k = 1,nm1
         kp1 = k+1
c
c        find l = pivot index
c
         l = k
         if (dscl1(h(k+1,k)) .gt. dscl1(h(k,k))) l = kp1
         ipvt(k) = l
c
c        zero pivot implies this column already triangularized
c
         if (dscl1(h(l,k)) .eq. 0.0d0) go to 40
c
c        interchange if necessary
c
         if (l .eq. k) go to 10
         t = h(l,k)
         h(l,k) = h(k,k)
         h(k,k) = t
   10    continue
c
c        compute multipliers (note: linpack computes negative of
c        what is computed here)
c
         t = (1.0d0,0.0d0)/h(k,k)
         h(kp1,k) = t*h(kp1,k)
c
c        row elimination with column indexing
c
         do 30 j = kp1,n
              t = h(l,j)
              if (l .eq. k) go to 20
              h(l,j) = h(k,j)
              h(k,j) = t
   20         continue
              h(kp1,j) = h(kp1,j)-t*h(kp1,k)
   30    continue
         go to 50
   40    continue
         info = k
   50    continue
   60 continue
   70 continue
      ipvt(n) = n
      if (dscl1(h(n,n)) .eq. 0.0d0) info = n
      return
      end
