      SUBROUTINE ZPPFA(AP,N,INFO)
      INTEGER N,INFO
      COMPLEX*16 AP(1)
C
C     ZPPFA FACTORS A COMPLEX*16 HERMITIAN POSITIVE DEFINITE MATRIX
C     STORED IN PACKED FORM.
C
C     ZPPFA IS USUALLY CALLED BY ZPPCO, BUT IT CAN BE CALLED
C     DIRECTLY WITH A SAVING IN TIME IF  RCOND  IS NOT NEEDED.
C     (TIME FOR ZPPCO) = (1 + 18/N)*(TIME FOR ZPPFA) .
C
C     ON ENTRY
C
C        AP      COMPLEX*16 (N*(N+1)/2)
C                THE PACKED FORM OF A HERMITIAN MATRIX  A .  THE
C                COLUMNS OF THE UPPER TRIANGLE ARE STORED SEQUENTIALLY
C                IN A ONE-DIMENSIONAL ARRAY OF LENGTH  N*(N+1)/2 .
C                SEE COMMENTS BELOW FOR DETAILS.
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C     ON RETURN
C
C        AP      AN UPPER TRIANGULAR MATRIX  R , STORED IN PACKED
C                FORM, SO THAT  A = CTRANS(R)*R .
C
C        INFO    INTEGER
C                = 0  FOR NORMAL RETURN.
C                = K  IF THE LEADING MINOR OF ORDER  K  IS NOT
C                     POSITIVE DEFINITE.
C
C
C     PACKED STORAGE
C
C          THE FOLLOWING PROGRAM SEGMENT WILL PACK THE UPPER
C          TRIANGLE OF A HERMITIAN MATRIX.
C
C                K = 0
C                DO 20 J = 1, N
C                   DO 10 I = 1, J
C                      K = K + 1
C                      AP(K) = A(I,J)
C             10    CONTINUE
C             20 CONTINUE
C
C     LINPACK.  THIS VERSION DATED 08/14/78 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
C
C     SUBROUTINES AND FUNCTIONS
C
C     BLAS ZDOTC
C     FORTRAN DCMPLX,DCONJG,DSQRT
C
C     INTERNAL VARIABLES
C
      COMPLEX*16 ZDOTC,T
      DOUBLE PRECISION S
      INTEGER J,JJ,JM1,K,KJ,KK
      DOUBLE PRECISION DREAL,DIMAG
      COMPLEX*16 ZDUMR,ZDUMI
      DREAL(ZDUMR) = ZDUMR
      DIMAG(ZDUMI) = (0.0D0,-1.0D0)*ZDUMI
C     BEGIN BLOCK WITH ...EXITS TO 40
C
C
         JJ = 0
         DO 30 J = 1, N
            INFO = J
            S = 0.0D0
            JM1 = J - 1
            KJ = JJ
            KK = 0
            IF (JM1 .LT. 1) GO TO 20
            DO 10 K = 1, JM1
               KJ = KJ + 1
               T = AP(KJ) - ZDOTC(K-1,AP(KK+1),1,AP(JJ+1),1)
               KK = KK + K
               T = T/AP(KK)
               AP(KJ) = T
               S = S + DREAL(T*DCONJG(T))
   10       CONTINUE
   20       CONTINUE
            JJ = JJ + J
            S = DREAL(AP(JJ)) - S
C     ......EXIT
            IF (S .LE. 0.0D0 .OR. DIMAG(AP(JJ)) .NE. 0.0D0) GO TO 40
            AP(JJ) = DCMPLX(DSQRT(S),0.0D0)
   30    CONTINUE
         INFO = 0
   40 CONTINUE
      RETURN
      END
