      SUBROUTINE STRDI(T,LDT,N,DET,JOB,INFO)                            00000010
      INTEGER LDT,N,JOB,INFO                                            00000020
      REAL T(LDT,1),DET(2)                                              00000030
C                                                                       00000040
C     STRDI COMPUTES THE DETERMINANT AND INVERSE OF A REAL              00000050
C     TRIANGULAR MATRIX.                                                00000060
C                                                                       00000070
C     ON ENTRY                                                          00000080
C                                                                       00000090
C        T       REAL(LDT,N)                                            00000100
C                T CONTAINS THE TRIANGULAR MATRIX. THE ZERO             00000110
C                ELEMENTS OF THE MATRIX ARE NOT REFERENCED, AND         00000120
C                THE CORRESPONDING ELEMENTS OF THE ARRAY CAN BE         00000130
C                USED TO STORE OTHER INFORMATION.                       00000140
C                                                                       00000150
C        LDT     INTEGER                                                00000160
C                LDT IS THE LEADING DIMENSION OF THE ARRAY T.           00000170
C                                                                       00000180
C        N       INTEGER                                                00000190
C                N IS THE ORDER OF THE SYSTEM.                          00000200
C                                                                       00000210
C        JOB     INTEGER                                                00000220
C                = 010       NO DET, INVERSE OF LOWER TRIANGULAR.       00000230
C                = 011       NO DET, INVERSE OF UPPER TRIANGULAR.       00000240
C                = 100       DET, NO INVERSE.                           00000250
C                = 110       DET, INVERSE OF LOWER TRIANGULAR.          00000260
C                = 111       DET, INVERSE OF UPPER TRIANGULAR.          00000270
C                                                                       00000280
C     ON RETURN                                                         00000290
C                                                                       00000300
C        T       INVERSE OF ORIGINAL MATRIX IF REQUESTED.               00000310
C                OTHERWISE UNCHANGED.                                   00000320
C                                                                       00000330
C        DET     REAL(2)                                                00000340
C                DETERMINANT OF ORIGINAL MATRIX IF REQUESTED.           00000350
C                OTHERWISE NOT REFERENCED.                              00000360
C                DETERMINANT = DET(1) * 10.0**DET(2)                    00000370
C                WITH  1.0 .LE. ABS(DET(1)) .LT. 10.0                   00000380
C                OR  DET(1) .EQ. 0.0 .                                  00000390
C                                                                       00000400
C        INFO    INTEGER                                                00000410
C                INFO CONTAINS ZERO IF THE SYSTEM IS NONSINGULAR        00000420
C                AND THE INVERSE IS REQUESTED.                          00000430
C                OTHERWISE INFO CONTAINS THE INDEX OF                   00000440
C                A ZERO DIAGONAL ELEMENT OF T.                          00000450
C                                                                       00000460
C                                                                       00000470
C     LINPACK. THIS VERSION DATED 08/14/78 .                            00000480
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.      00000490
C                                                                       00000500
C     SUBROUTINES AND FUNCTIONS                                         00000510
C                                                                       00000520
C     BLAS SAXPY,SSCAL                                                  00000530
C     FORTRAN ABS,MOD                                                   00000540
C                                                                       00000550
C     INTERNAL VARIABLES                                                00000560
C                                                                       00000570
      REAL TEMP                                                         00000580
      REAL TEN                                                          00000590
      INTEGER I,J,K,KB,KM1,KP1                                          00000600
C                                                                       00000610
C     BEGIN BLOCK PERMITTING ...EXITS TO 180                            00000620
C                                                                       00000630
C        COMPUTE DETERMINANT                                            00000640
C                                                                       00000650
         IF (JOB/100 .EQ. 0) GO TO 70                                   00000660
            DET(1) = 1.0E0                                              00000670
            DET(2) = 0.0E0                                              00000680
            TEN = 10.0E0                                                00000690
            DO 50 I = 1, N                                              00000700
               DET(1) = T(I,I)*DET(1)                                   00000710
C           ...EXIT                                                     00000720
               IF (DET(1) .EQ. 0.0E0) GO TO 60                          00000730
   10          IF (ABS(DET(1)) .GE. 1.0E0) GO TO 20                     00000740
                  DET(1) = TEN*DET(1)                                   00000750
                  DET(2) = DET(2) - 1.0E0                               00000760
               GO TO 10                                                 00000770
   20          CONTINUE                                                 00000780
   30          IF (ABS(DET(1)) .LT. TEN) GO TO 40                       00000790
                  DET(1) = DET(1)/TEN                                   00000800
                  DET(2) = DET(2) + 1.0E0                               00000810
               GO TO 30                                                 00000820
   40          CONTINUE                                                 00000830
   50       CONTINUE                                                    00000840
   60       CONTINUE                                                    00000850
   70    CONTINUE                                                       00000860
C                                                                       00000870
C        COMPUTE INVERSE OF UPPER TRIANGULAR                            00000880
C                                                                       00000890
         IF (MOD(JOB/10,10) .EQ. 0) GO TO 170                           00000900
            IF (MOD(JOB,10) .EQ. 0) GO TO 120                           00000910
C              BEGIN BLOCK PERMITTING ...EXITS TO 110                   00000920
                  DO 100 K = 1, N                                       00000930
                     INFO = K                                           00000940
C              ......EXIT                                               00000950
                     IF (T(K,K) .EQ. 0.0E0) GO TO 110                   00000960
                     T(K,K) = 1.0E0/T(K,K)                              00000970
                     TEMP = -T(K,K)                                     00000980
                     CALL SSCAL(K-1,TEMP,T(1,K),1)                      00000990
                     KP1 = K + 1                                        00001000
                     IF (N .LT. KP1) GO TO 90                           00001010
                     DO 80 J = KP1, N                                   00001020
                        TEMP = T(K,J)                                   00001030
                        T(K,J) = 0.0E0                                  00001040
                        CALL SAXPY(K,TEMP,T(1,K),1,T(1,J),1)            00001050
   80                CONTINUE                                           00001060
   90                CONTINUE                                           00001070
  100             CONTINUE                                              00001080
                  INFO = 0                                              00001090
  110          CONTINUE                                                 00001100
            GO TO 160                                                   00001110
  120       CONTINUE                                                    00001120
C                                                                       00001130
C              COMPUTE INVERSE OF LOWER TRIANGULAR                      00001140
C                                                                       00001150
               DO 150 KB = 1, N                                         00001160
                  K = N + 1 - KB                                        00001170
                  INFO = K                                              00001180
C     ............EXIT                                                  00001190
                  IF (T(K,K) .EQ. 0.0E0) GO TO 180                      00001200
                  T(K,K) = 1.0E0/T(K,K)                                 00001210
                  TEMP = -T(K,K)                                        00001220
                  IF (K .NE. N) CALL SSCAL(N-K,TEMP,T(K+1,K),1)         00001230
                  KM1 = K - 1                                           00001240
                  IF (KM1 .LT. 1) GO TO 140                             00001250
                  DO 130 J = 1, KM1                                     00001260
                     TEMP = T(K,J)                                      00001270
                     T(K,J) = 0.0E0                                     00001280
                     CALL SAXPY(N-K+1,TEMP,T(K,K),1,T(K,J),1)           00001290
  130             CONTINUE                                              00001300
  140             CONTINUE                                              00001310
  150          CONTINUE                                                 00001320
               INFO = 0                                                 00001330
  160       CONTINUE                                                    00001340
  170    CONTINUE                                                       00001350
  180 CONTINUE                                                          00001360
      RETURN                                                            00001370
      END                                                               00001380
