      SUBROUTINE WGECO(AR,AI,LDA,N,IPVT,RCOND,ZR,ZI)
      INTEGER LDA,N,IPVT(1) 
      DOUBLE PRECISION AR(LDA,1),AI(LDA,1),ZR(1),ZI(1)  
      DOUBLE PRECISION RCOND
C
C     WGECO FACTORS A DOUBLE-COMPLEX MATRIX BY GAUSSIAN ELIMINATION 
C     AND ESTIMATES THE CONDITION OF THE MATRIX.
C
C     IF  RCOND  IS NOT NEEDED, WGEFA IS SLIGHTLY FASTER.
C     TO SOLVE  A*X = B , FOLLOW WGECO BY WGESL.
C     TO COMPUTE  INVERSE(A)*C , FOLLOW WGECO BY WGESL. 
C     TO COMPUTE  DETERMINANT(A) , FOLLOW WGECO BY WGEDI.
C     TO COMPUTE  INVERSE(A) , FOLLOW WGECO BY WGEDI.
C
C     ON ENTRY  
C
C        A       DOUBLE-COMPLEX(LDA, N) 
C                THE MATRIX TO BE FACTORED. 
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  A .
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C     ON RETURN 
C
C        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS 
C                WHICH WERE USED TO OBTAIN IT.  
C                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
C                L  IS A PRODUCT OF PERMUTATION AND UNIT LOWER  
C                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N) 
C                AN INTEGER VECTOR OF PIVOT INDICES.
C
C        RCOND   DOUBLE PRECISION
C                AN ESTIMATE OF THE RECIPROCAL CONDITION OF  A .
C                FOR THE SYSTEM  A*X = B , RELATIVE PERTURBATIONS
C                IN  A  AND  B  OF SIZE  EPSILON  MAY CAUSE 
C                RELATIVE PERTURBATIONS IN  X  OF SIZE  EPSILON/RCOND . 
C                IF  RCOND  IS SO SMALL THAT THE LOGICAL EXPRESSION 
C                           1.0 + RCOND .EQ. 1.0
C                IS TRUE, THEN  A  MAY BE SINGULAR TO WORKING
C                PRECISION.  IN PARTICULAR,  RCOND  IS ZERO  IF 
C                EXACT SINGULARITY IS DETECTED OR THE ESTIMATE  
C                UNDERFLOWS.
C
C        Z       DOUBLE-COMPLEX(N)  
C                A WORK VECTOR WHOSE CONTENTS ARE USUALLY UNIMPORTANT.  
C                IF  A  IS CLOSE TO A SINGULAR MATRIX, THEN  Z  IS  
C                AN APPROXIMATE NULL VECTOR IN THE SENSE THAT
C                NORM(A*Z) = RCOND*NORM(A)*NORM(Z) .
C
C     LINPACK. THIS VERSION DATED 07/01/79 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.  
C
C     SUBROUTINES AND FUNCTIONS 
C
C     LINPACK WGEFA 
C     BLAS WAXPY,WDOTC,WASUM
C     FORTRAN DABS,DMAX1
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION WDOTCR,WDOTCI,EKR,EKI,TR,TI,WKR,WKI,WKMR,WKMI
      DOUBLE PRECISION ANORM,S,WASUM,SM,YNORM,FLOP  
      INTEGER INFO,J,K,KB,KP1,L 
C
      DOUBLE PRECISION ZDUMR,ZDUMI  
      DOUBLE PRECISION CABS1
      CABS1(ZDUMR,ZDUMI) = DABS(ZDUMR) + DABS(ZDUMI)
C
C     COMPUTE 1-NORM OF A
C
      ANORM = 0.0D0 
      DO 10 J = 1, N
         ANORM = DMAX1(ANORM,WASUM(N,AR(1,J),AI(1,J),1))
   10 CONTINUE  
C
C     FACTOR
C
      CALL WGEFA(AR,AI,LDA,N,IPVT,INFO) 
C
C     RCOND = 1/(NORM(A)*(ESTIMATE OF NORM(INVERSE(A)))) .  
C     ESTIMATE = NORM(Z)/NORM(Y) WHERE  A*Z = Y  AND  CTRANS(A)*Y = E . 
C     CTRANS(A)  IS THE CONJUGATE TRANSPOSE OF A .  
C     THE COMPONENTS OF  E  ARE CHOSEN TO CAUSE MAXIMUM LOCAL
C     GROWTH IN THE ELEMENTS OF W  WHERE  CTRANS(U)*W = E . 
C     THE VECTORS ARE FREQUENTLY RESCALED TO AVOID OVERFLOW.
C
C     SOLVE CTRANS(U)*W = E 
C
      EKR = 1.0D0
      EKI = 0.0D0
      DO 20 J = 1, N
         ZR(J) = 0.0D0  
         ZI(J) = 0.0D0  
   20 CONTINUE  
      DO 110 K = 1, N
         CALL WSIGN(EKR,EKI,-ZR(K),-ZI(K),EKR,EKI)  
         IF (CABS1(EKR-ZR(K),EKI-ZI(K)) 
     *       .LE. CABS1(AR(K,K),AI(K,K))) GO TO 40  
            S = CABS1(AR(K,K),AI(K,K))  
     *          /CABS1(EKR-ZR(K),EKI-ZI(K)) 
            CALL WRSCAL(N,S,ZR,ZI,1)
            EKR = S*EKR 
            EKI = S*EKI 
   40    CONTINUE
         WKR = EKR - ZR(K)  
         WKI = EKI - ZI(K)  
         WKMR = -EKR - ZR(K)
         WKMI = -EKI - ZI(K)
         S = CABS1(WKR,WKI) 
         SM = CABS1(WKMR,WKMI)  
         IF (CABS1(AR(K,K),AI(K,K)) .EQ. 0.0D0) GO TO 50
            CALL WDIV(WKR,WKI,AR(K,K),-AI(K,K),WKR,WKI) 
            CALL WDIV(WKMR,WKMI,AR(K,K),-AI(K,K),WKMR,WKMI) 
         GO TO 60
   50    CONTINUE
            WKR = 1.0D0 
            WKI = 0.0D0 
            WKMR = 1.0D0
            WKMI = 0.0D0
   60    CONTINUE
         KP1 = K + 1
         IF (KP1 .GT. N) GO TO 100  
            DO 70 J = KP1, N
               CALL WMUL(WKMR,WKMI,AR(K,J),-AI(K,J),TR,TI)  
               SM = FLOP(SM + CABS1(ZR(J)+TR,ZI(J)+TI)) 
               CALL WAXPY(1,WKR,WKI,AR(K,J),-AI(K,J),1, 
     $                    ZR(J),ZI(J),1)
               S = FLOP(S + CABS1(ZR(J),ZI(J))) 
   70       CONTINUE
            IF (S .GE. SM) GO TO 90 
               TR = WKMR - WKR  
               TI = WKMI - WKI  
               WKR = WKMR
               WKI = WKMI
               DO 80 J = KP1, N 
                  CALL WAXPY(1,TR,TI,AR(K,J),-AI(K,J),1,
     $                       ZR(J),ZI(J),1) 
   80          CONTINUE 
   90       CONTINUE
  100    CONTINUE
         ZR(K) = WKR
         ZI(K) = WKI
  110 CONTINUE  
      S = 1.0D0/WASUM(N,ZR,ZI,1)
      CALL WRSCAL(N,S,ZR,ZI,1)  
C
C     SOLVE CTRANS(L)*Y = W 
C
      DO 140 KB = 1, N  
         K = N + 1 - KB 
         IF (K .GE. N) GO TO 120
            ZR(K) = ZR(K)
     *            + WDOTCR(N-K,AR(K+1,K),AI(K+1,K),1,ZR(K+1),ZI(K+1),1) 
            ZI(K) = ZI(K)
     *            + WDOTCI(N-K,AR(K+1,K),AI(K+1,K),1,ZR(K+1),ZI(K+1),1) 
  120    CONTINUE
         IF (CABS1(ZR(K),ZI(K)) .LE. 1.0D0) GO TO 130
            S = 1.0D0/CABS1(ZR(K),ZI(K))
            CALL WRSCAL(N,S,ZR,ZI,1)
  130    CONTINUE
         L = IPVT(K)
         TR = ZR(L) 
         TI = ZI(L) 
         ZR(L) = ZR(K)  
         ZI(L) = ZI(K)  
         ZR(K) = TR 
         ZI(K) = TI 
  140 CONTINUE  
      S = 1.0D0/WASUM(N,ZR,ZI,1)
      CALL WRSCAL(N,S,ZR,ZI,1)  
C
      YNORM = 1.0D0 
C
C     SOLVE L*V = Y 
C
      DO 160 K = 1, N
         L = IPVT(K)
         TR = ZR(L) 
         TI = ZI(L) 
         ZR(L) = ZR(K)  
         ZI(L) = ZI(K)  
         ZR(K) = TR 
         ZI(K) = TI 
         IF (K .LT. N)  
     *      CALL WAXPY(N-K,TR,TI,AR(K+1,K),AI(K+1,K),1,ZR(K+1),ZI(K+1), 
     *                 1)
         IF (CABS1(ZR(K),ZI(K)) .LE. 1.0D0) GO TO 150
            S = 1.0D0/CABS1(ZR(K),ZI(K))
            CALL WRSCAL(N,S,ZR,ZI,1)
            YNORM = S*YNORM 
  150    CONTINUE
  160 CONTINUE  
      S = 1.0D0/WASUM(N,ZR,ZI,1)
      CALL WRSCAL(N,S,ZR,ZI,1)  
      YNORM = S*YNORM
C
C     SOLVE  U*Z = V
C
      DO 200 KB = 1, N  
         K = N + 1 - KB 
         IF (CABS1(ZR(K),ZI(K)) 
     *       .LE. CABS1(AR(K,K),AI(K,K))) GO TO 170 
            S = CABS1(AR(K,K),AI(K,K))  
     *          /CABS1(ZR(K),ZI(K)) 
            CALL WRSCAL(N,S,ZR,ZI,1)
            YNORM = S*YNORM 
  170    CONTINUE
         IF (CABS1(AR(K,K),AI(K,K)) .EQ. 0.0D0) GO TO 180
            CALL WDIV(ZR(K),ZI(K),AR(K,K),AI(K,K),ZR(K),ZI(K))  
  180    CONTINUE
         IF (CABS1(AR(K,K),AI(K,K)) .NE. 0.0D0) GO TO 190
            ZR(K) = 1.0D0
            ZI(K) = 0.0D0
  190    CONTINUE
         TR = -ZR(K)
         TI = -ZI(K)
         CALL WAXPY(K-1,TR,TI,AR(1,K),AI(1,K),1,ZR(1),ZI(1),1)  
  200 CONTINUE  
C     MAKE ZNORM = 1.0  
      S = 1.0D0/WASUM(N,ZR,ZI,1)
      CALL WRSCAL(N,S,ZR,ZI,1)  
      YNORM = S*YNORM
C
      IF (ANORM .NE. 0.0D0) RCOND = YNORM/ANORM 
      IF (ANORM .EQ. 0.0D0) RCOND = 0.0D0
      RETURN
      END
      SUBROUTINE WGEFA(AR,AI,LDA,N,IPVT,INFO)
      INTEGER LDA,N,IPVT(1),INFO
      DOUBLE PRECISION AR(LDA,1),AI(LDA,1)  
C
C     WGEFA FACTORS A DOUBLE-COMPLEX MATRIX BY GAUSSIAN ELIMINATION.
C
C     WGEFA IS USUALLY CALLED BY WGECO, BUT IT CAN BE CALLED
C     DIRECTLY WITH A SAVING IN TIME IF  RCOND  IS NOT NEEDED.  
C     (TIME FOR WGECO) = (1 + 9/N)*(TIME FOR WGEFA) .
C
C     ON ENTRY  
C
C        A       DOUBLE-COMPLEX(LDA, N) 
C                THE MATRIX TO BE FACTORED. 
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  A .
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C     ON RETURN 
C
C        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS 
C                WHICH WERE USED TO OBTAIN IT.  
C                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
C                L  IS A PRODUCT OF PERMUTATION AND UNIT LOWER  
C                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N) 
C                AN INTEGER VECTOR OF PIVOT INDICES.
C
C        INFO    INTEGER
C                = 0  NORMAL VALUE. 
C                = K  IF  U(K,K) .EQ. 0.0 .  THIS IS NOT AN ERROR
C                     CONDITION FOR THIS SUBROUTINE, BUT IT DOES
C                     INDICATE THAT WGESL OR WGEDI WILL DIVIDE BY ZERO  
C                     IF CALLED.  USE  RCOND  IN WGECO FOR A RELIABLE
C                     INDICATION OF SINGULARITY.
C
C     LINPACK. THIS VERSION DATED 07/01/79 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.  
C
C     SUBROUTINES AND FUNCTIONS 
C
C     BLAS WAXPY,WSCAL,IWAMAX
C     FORTRAN DABS  
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION TR,TI
      INTEGER IWAMAX,J,K,KP1,L,NM1  
C
      DOUBLE PRECISION ZDUMR,ZDUMI  
      DOUBLE PRECISION CABS1
      CABS1(ZDUMR,ZDUMI) = DABS(ZDUMR) + DABS(ZDUMI)
C
C     GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING
C
      INFO = 0  
      NM1 = N - 1
      IF (NM1 .LT. 1) GO TO 70  
      DO 60 K = 1, NM1  
         KP1 = K + 1
C
C        FIND L = PIVOT INDEX
C
         L = IWAMAX(N-K+1,AR(K,K),AI(K,K),1) + K - 1
         IPVT(K) = L
C
C        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED  
C
         IF (CABS1(AR(L,K),AI(L,K)) .EQ. 0.0D0) GO TO 40
C
C           INTERCHANGE IF NECESSARY
C
            IF (L .EQ. K) GO TO 10  
               TR = AR(L,K) 
               TI = AI(L,K) 
               AR(L,K) = AR(K,K)
               AI(L,K) = AI(K,K)
               AR(K,K) = TR 
               AI(K,K) = TI 
   10       CONTINUE
C
C           COMPUTE MULTIPLIERS 
C
            CALL WDIV(-1.0D0,0.0D0,AR(K,K),AI(K,K),TR,TI)
            CALL WSCAL(N-K,TR,TI,AR(K+1,K),AI(K+1,K),1) 
C
C           ROW ELIMINATION WITH COLUMN INDEXING
C
            DO 30 J = KP1, N
               TR = AR(L,J) 
               TI = AI(L,J) 
               IF (L .EQ. K) GO TO 20
                  AR(L,J) = AR(K,J) 
                  AI(L,J) = AI(K,J) 
                  AR(K,J) = TR  
                  AI(K,J) = TI  
   20          CONTINUE 
               CALL WAXPY(N-K,TR,TI,AR(K+1,K),AI(K+1,K),1,AR(K+1,J),
     *                    AI(K+1,J),1)  
   30       CONTINUE
         GO TO 50
   40    CONTINUE
            INFO = K
   50    CONTINUE
   60 CONTINUE  
   70 CONTINUE  
      IPVT(N) = N
      IF (CABS1(AR(N,N),AI(N,N)) .EQ. 0.0D0) INFO = N
      RETURN
      END
      SUBROUTINE WGESL(AR,AI,LDA,N,IPVT,BR,BI,JOB)  
      INTEGER LDA,N,IPVT(1),JOB 
      DOUBLE PRECISION AR(LDA,1),AI(LDA,1),BR(1),BI(1)  
C
C     WGESL SOLVES THE DOUBLE-COMPLEX SYSTEM
C     A * X = B  OR  CTRANS(A) * X = B  
C     USING THE FACTORS COMPUTED BY WGECO OR WGEFA. 
C
C     ON ENTRY  
C
C        A       DOUBLE-COMPLEX(LDA, N) 
C                THE OUTPUT FROM WGECO OR WGEFA.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  A .
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C        IPVT    INTEGER(N) 
C                THE PIVOT VECTOR FROM WGECO OR WGEFA.  
C
C        B       DOUBLE-COMPLEX(N)  
C                THE RIGHT HAND SIDE VECTOR.
C
C        JOB     INTEGER
C                = 0         TO SOLVE  A*X = B ,
C                = NONZERO   TO SOLVE  CTRANS(A)*X = B  WHERE
C                            CTRANS(A)  IS THE CONJUGATE TRANSPOSE. 
C
C     ON RETURN 
C
C        B       THE SOLUTION VECTOR  X .
C
C     ERROR CONDITION
C
C        A DIVISION BY ZERO WILL OCCUR IF THE INPUT FACTOR CONTAINS A
C        ZERO ON THE DIAGONAL.  TECHNICALLY THIS INDICATES SINGULARITY  
C        BUT IT IS OFTEN CAUSED BY IMPROPER ARGUMENTS OR IMPROPER
C        SETTING OF LDA .  IT WILL NOT OCCUR IF THE SUBROUTINES ARE 
C        CALLED CORRECTLY AND IF WGECO HAS SET RCOND .GT. 0.0
C        OR WGEFA HAS SET INFO .EQ. 0 . 
C
C     TO COMPUTE  INVERSE(A) * C  WHERE  C  IS A MATRIX 
C     WITH  P  COLUMNS  
C           CALL WGECO(A,LDA,N,IPVT,RCOND,Z)
C           IF (RCOND IS TOO SMALL) GO TO ...
C           DO 10 J = 1, P  
C              CALL WGESL(A,LDA,N,IPVT,C(1,J),0)
C        10 CONTINUE
C
C     LINPACK. THIS VERSION DATED 07/01/79 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.  
C
C     SUBROUTINES AND FUNCTIONS 
C
C     BLAS WAXPY,WDOTC  
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION WDOTCR,WDOTCI,TR,TI  
      INTEGER K,KB,L,NM1
C
      NM1 = N - 1
      IF (JOB .NE. 0) GO TO 50  
C
C        JOB = 0 , SOLVE  A * X = B 
C        FIRST SOLVE  L*Y = B
C
         IF (NM1 .LT. 1) GO TO 30
         DO 20 K = 1, NM1
            L = IPVT(K) 
            TR = BR(L)  
            TI = BI(L)  
            IF (L .EQ. K) GO TO 10  
               BR(L) = BR(K)
               BI(L) = BI(K)
               BR(K) = TR
               BI(K) = TI
   10       CONTINUE
            CALL WAXPY(N-K,TR,TI,AR(K+1,K),AI(K+1,K),1,BR(K+1),BI(K+1), 
     *                 1)
   20    CONTINUE
   30    CONTINUE
C
C        NOW SOLVE  U*X = Y 
C
         DO 40 KB = 1, N
            K = N + 1 - KB  
            CALL WDIV(BR(K),BI(K),AR(K,K),AI(K,K),BR(K),BI(K))  
            TR = -BR(K) 
            TI = -BI(K) 
            CALL WAXPY(K-1,TR,TI,AR(1,K),AI(1,K),1,BR(1),BI(1),1)
   40    CONTINUE
      GO TO 100 
   50 CONTINUE  
C
C        JOB = NONZERO, SOLVE  CTRANS(A) * X = B
C        FIRST SOLVE  CTRANS(U)*Y = B
C
         DO 60 K = 1, N 
            TR = BR(K) - WDOTCR(K-1,AR(1,K),AI(1,K),1,BR(1),BI(1),1)
            TI = BI(K) - WDOTCI(K-1,AR(1,K),AI(1,K),1,BR(1),BI(1),1)
            CALL WDIV(TR,TI,AR(K,K),-AI(K,K),BR(K),BI(K))
   60    CONTINUE
C
C        NOW SOLVE CTRANS(L)*X = Y  
C
         IF (NM1 .LT. 1) GO TO 90
         DO 80 KB = 1, NM1  
            K = N - KB  
            BR(K) = BR(K)
     *            + WDOTCR(N-K,AR(K+1,K),AI(K+1,K),1,BR(K+1),BI(K+1),1) 
            BI(K) = BI(K)
     *            + WDOTCI(N-K,AR(K+1,K),AI(K+1,K),1,BR(K+1),BI(K+1),1) 
            L = IPVT(K) 
            IF (L .EQ. K) GO TO 70  
               TR = BR(L)
               TI = BI(L)
               BR(L) = BR(K)
               BI(L) = BI(K)
               BR(K) = TR
               BI(K) = TI
   70       CONTINUE
   80    CONTINUE
   90    CONTINUE
  100 CONTINUE  
      RETURN
      END
      SUBROUTINE WGEDI(AR,AI,LDA,N,IPVT,DETR,DETI,WORKR,WORKI,JOB)  
      INTEGER LDA,N,IPVT(1),JOB 
      DOUBLE PRECISION AR(LDA,1),AI(LDA,1),DETR(2),DETI(2),WORKR(1),
     *                 WORKI(1) 
C
C     WGEDI COMPUTES THE DETERMINANT AND INVERSE OF A MATRIX
C     USING THE FACTORS COMPUTED BY WGECO OR WGEFA. 
C
C     ON ENTRY  
C
C        A       DOUBLE-COMPLEX(LDA, N) 
C                THE OUTPUT FROM WGECO OR WGEFA.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  A .
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C        IPVT    INTEGER(N) 
C                THE PIVOT VECTOR FROM WGECO OR WGEFA.  
C
C        WORK    DOUBLE-COMPLEX(N)  
C                WORK VECTOR.  CONTENTS DESTROYED.  
C
C        JOB     INTEGER
C                = 11   BOTH DETERMINANT AND INVERSE.
C                = 01   INVERSE ONLY.
C                = 10   DETERMINANT ONLY.
C
C     ON RETURN 
C
C        A       INVERSE OF ORIGINAL MATRIX IF REQUESTED.
C                OTHERWISE UNCHANGED.
C
C        DET     DOUBLE-COMPLEX(2)  
C                DETERMINANT OF ORIGINAL MATRIX IF REQUESTED.
C                OTHERWISE NOT REFERENCED.  
C                DETERMINANT = DET(1) * 10.0**DET(2)
C                WITH  1.0 .LE. CABS1(DET(1) .LT. 10.0  
C                OR  DET(1) .EQ. 0.0 .  
C
C     ERROR CONDITION
C
C        A DIVISION BY ZERO WILL OCCUR IF THE INPUT FACTOR CONTAINS 
C        A ZERO ON THE DIAGONAL AND THE INVERSE IS REQUESTED.
C        IT WILL NOT OCCUR IF THE SUBROUTINES ARE CALLED CORRECTLY  
C        AND IF WGECO HAS SET RCOND .GT. 0.0 OR WGEFA HAS SET
C        INFO .EQ. 0 .  
C
C     LINPACK. THIS VERSION DATED 07/01/79 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.  
C
C     SUBROUTINES AND FUNCTIONS 
C
C     BLAS WAXPY,WSCAL,WSWAP
C     FORTRAN DABS,MOD  
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION TR,TI
      DOUBLE PRECISION TEN  
      INTEGER I,J,K,KB,KP1,L,NM1
C
      DOUBLE PRECISION ZDUMR,ZDUMI  
      DOUBLE PRECISION CABS1
      CABS1(ZDUMR,ZDUMI) = DABS(ZDUMR) + DABS(ZDUMI)
C
C     COMPUTE DETERMINANT
C
      IF (JOB/10 .EQ. 0) GO TO 80
         DETR(1) = 1.0D0
         DETI(1) = 0.0D0
         DETR(2) = 0.0D0
         DETI(2) = 0.0D0
         TEN = 10.0D0
         DO 60 I = 1, N 
            IF (IPVT(I) .EQ. I) GO TO 10
               DETR(1) = -DETR(1)
               DETI(1) = -DETI(1)
   10       CONTINUE
            CALL WMUL(AR(I,I),AI(I,I),DETR(1),DETI(1),DETR(1),DETI(1))  
C           ...EXIT 
C        ...EXIT
            IF (CABS1(DETR(1),DETI(1)) .EQ. 0.0D0) GO TO 70 
   20       IF (CABS1(DETR(1),DETI(1)) .GE. 1.0D0) GO TO 30 
               DETR(1) = TEN*DETR(1)
               DETI(1) = TEN*DETI(1)
               DETR(2) = DETR(2) - 1.0D0
               DETI(2) = DETI(2) - 0.0D0
            GO TO 20
   30       CONTINUE
   40       IF (CABS1(DETR(1),DETI(1)) .LT. TEN) GO TO 50
               DETR(1) = DETR(1)/TEN
               DETI(1) = DETI(1)/TEN
               DETR(2) = DETR(2) + 1.0D0
               DETI(2) = DETI(2) + 0.0D0
            GO TO 40
   50       CONTINUE
   60    CONTINUE
   70    CONTINUE
   80 CONTINUE  
C
C     COMPUTE INVERSE(U)
C
      IF (MOD(JOB,10) .EQ. 0) GO TO 160 
         DO 110 K = 1, N
            CALL WDIV(1.0D0,0.0D0,AR(K,K),AI(K,K),AR(K,K),AI(K,K))  
            TR = -AR(K,K)
            TI = -AI(K,K)
            CALL WSCAL(K-1,TR,TI,AR(1,K),AI(1,K),1) 
            KP1 = K + 1 
            IF (N .LT. KP1) GO TO 100
            DO 90 J = KP1, N
               TR = AR(K,J) 
               TI = AI(K,J) 
               AR(K,J) = 0.0D0  
               AI(K,J) = 0.0D0  
               CALL WAXPY(K,TR,TI,AR(1,K),AI(1,K),1,AR(1,J),AI(1,J),1)  
   90       CONTINUE
  100       CONTINUE
  110    CONTINUE
C
C        FORM INVERSE(U)*INVERSE(L) 
C
         NM1 = N - 1
         IF (NM1 .LT. 1) GO TO 150  
         DO 140 KB = 1, NM1 
            K = N - KB  
            KP1 = K + 1 
            DO 120 I = KP1, N
               WORKR(I) = AR(I,K)
               WORKI(I) = AI(I,K)
               AR(I,K) = 0.0D0  
               AI(I,K) = 0.0D0  
  120       CONTINUE
            DO 130 J = KP1, N
               TR = WORKR(J)
               TI = WORKI(J)
               CALL WAXPY(N,TR,TI,AR(1,J),AI(1,J),1,AR(1,K),AI(1,K),1)  
  130       CONTINUE
            L = IPVT(K) 
            IF (L .NE. K)
     *         CALL WSWAP(N,AR(1,K),AI(1,K),1,AR(1,L),AI(1,L),1)
  140    CONTINUE
  150    CONTINUE
  160 CONTINUE  
      RETURN
      END
      SUBROUTINE WPOFA(AR,AI,LDA,N,INFO)
      DOUBLE PRECISION AR(LDA,1),AI(LDA,1)  
      DOUBLE PRECISION S,TR,TI,WDOTCR,WDOTCI
      DO 30 J = 1, N
         INFO = J
         S = 0.0D0  
         JM1 = J-1  
         IF (JM1 .LT. 1) GO TO 20
         DO 10 K = 1, JM1
            TR = AR(K,J)-WDOTCR(K-1,AR(1,K),AI(1,K),1,AR(1,J),AI(1,J),1)
            TI = AI(K,J)-WDOTCI(K-1,AR(1,K),AI(1,K),1,AR(1,J),AI(1,J),1)
            CALL WDIV(TR,TI,AR(K,K),AI(K,K),TR,TI)  
            AR(K,J) = TR
            AI(K,J) = TI
            S = S + TR*TR + TI*TI
   10    CONTINUE
   20    CONTINUE
         S = AR(J,J) - S
         IF (S.LE.0.0D0 .OR. AI(J,J).NE.0.0D0) GO TO 40 
         AR(J,J) = DSQRT(S) 
   30 CONTINUE  
      INFO = 0  
   40 RETURN
      END
      SUBROUTINE RREF(AR,AI,LDA,M,N,EPS)
      DOUBLE PRECISION AR(LDA,1),AI(LDA,1),EPS,TOL,TR,TI,WASUM  
      TOL = 0.0D0
      DO 10 J = 1, N
         TOL = DMAX1(TOL,WASUM(M,AR(1,J),AI(1,J),1))
   10 CONTINUE  
      TOL = EPS*DFLOAT(2*MAX0(M,N))*TOL 
      K = 1 
      L = 1 
   20 IF (K.GT.M .OR. L.GT.N) RETURN
      I = IWAMAX(M-K+1,AR(K,L),AI(K,L),1) + K-1 
      IF (DABS(AR(I,L))+DABS(AI(I,L)) .GT. TOL) GO TO 30
         CALL WSET(M-K+1,0.0D0,0.0D0,AR(K,L),AI(K,L),1) 
         L = L+1
         GO TO 20
   30 CALL WSWAP(N-L+1,AR(I,L),AI(I,L),LDA,AR(K,L),AI(K,L),LDA) 
      CALL WDIV(1.0D0,0.0D0,AR(K,L),AI(K,L),TR,TI)  
      CALL WSCAL(N-L+1,TR,TI,AR(K,L),AI(K,L),LDA)
      AR(K,L) = 1.0D0
      AI(K,L) = 0.0D0
      DO 40 I = 1, M
         TR = -AR(I,L)  
         TI = -AI(I,L)  
         IF (I .NE. K) CALL WAXPY(N-L+1,TR,TI,  
     $                 AR(K,L),AI(K,L),LDA,AR(I,L),AI(I,L),LDA) 
   40 CONTINUE  
      K = K+1
      L = L+1
      GO TO 20  
      END
      SUBROUTINE HILBER(A,LDA,N)
      DOUBLE PRECISION A(LDA,N) 
C     GENERATE INVERSE HILBERT MATRIX
      DOUBLE PRECISION P,R  
      P = DFLOAT(N) 
      DO 20 I = 1, N
        IF (I.NE.1) P = (DFLOAT(N-I+1)*P*DFLOAT(N+I-1))/DFLOAT(I-1)**2  
        R = P*P 
        A(I,I) = R/DFLOAT(2*I-1)
        IF (I.EQ.N) GO TO 20
        IP1 = I+1
        DO 10 J = IP1, N
          R = -(DFLOAT(N-J+1)*R*(N+J-1))/DFLOAT(J-1)**2 
          A(I,J) = R/DFLOAT(I+J-1)  
          A(J,I) = A(I,J)
   10   CONTINUE
   20 CONTINUE  
      RETURN
      END
      SUBROUTINE HTRIDI(NM,N,AR,AI,D,E,E2,TAU)  
C
      INTEGER I,J,K,L,N,II,NM,JP1
      DOUBLE PRECISION AR(NM,N),AI(NM,N),D(N),E(N),E2(N),TAU(2,N)
      DOUBLE PRECISION F,G,H,FI,GI,HH,SI,SCALE  
      DOUBLE PRECISION FLOP,PYTHAG  
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF 
C     THE ALGOL PROCEDURE TRED1, NUM. MATH. 11, 181-195(1968)
C     BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE REDUCES A COMPLEX HERMITIAN MATRIX
C     TO A REAL SYMMETRIC TRIDIAGONAL MATRIX USING  
C     UNITARY SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT. 
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM  
C          DIMENSION STATEMENT. 
C
C        N IS THE ORDER OF THE MATRIX.  
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX HERMITIAN INPUT MATRIX. 
C          ONLY THE LOWER TRIANGLE OF THE MATRIX NEED BE SUPPLIED.  
C
C     ON OUTPUT.
C
C        AR AND AI CONTAIN INFORMATION ABOUT THE UNITARY TRANS- 
C          FORMATIONS USED IN THE REDUCTION IN THEIR FULL LOWER 
C          TRIANGLES.  THEIR STRICT UPPER TRIANGLES AND THE 
C          DIAGONAL OF AR ARE UNALTERED.
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE THE TRIDIAGONAL MATRIX.
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL 
C          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO.  
C
C        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
C          E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED.
C
C        TAU CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C
C     MODIFIED TO GET RID OF ALL COMPLEX ARITHMETIC, C. MOLER, 6/27/79. 
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY 
C
C     ------------------------------------------------------------------
C
      TAU(1,N) = 1.0D0  
      TAU(2,N) = 0.0D0  
C
      DO 100 I = 1, N
  100 D(I) = AR(I,I)
C     .......... FOR I=N STEP -1 UNTIL 1 DO -- ..........
      DO 300 II = 1, N  
         I = N + 1 - II 
         L = I - 1  
         H = 0.0D0  
         SCALE = 0.0D0  
         IF (L .LT. 1) GO TO 130
C     .......... SCALE ROW (ALGOL TOL THEN NOT NEEDED) ..........
         DO 120 K = 1, L
  120    SCALE = FLOP(SCALE + DABS(AR(I,K)) + DABS(AI(I,K)))
C
         IF (SCALE .NE. 0.0D0) GO TO 140
         TAU(1,L) = 1.0D0
         TAU(2,L) = 0.0D0
  130    E(I) = 0.0D0
         E2(I) = 0.0D0  
         GO TO 290  
C
  140    DO 150 K = 1, L
            AR(I,K) = FLOP(AR(I,K)/SCALE)
            AI(I,K) = FLOP(AI(I,K)/SCALE)
            H = FLOP(H + AR(I,K)*AR(I,K) + AI(I,K)*AI(I,K)) 
  150    CONTINUE
C
         E2(I) = FLOP(SCALE*SCALE*H)
         G = FLOP(DSQRT(H)) 
         E(I) = FLOP(SCALE*G)
         F = PYTHAG(AR(I,L),AI(I,L))
C     .......... FORM NEXT DIAGONAL ELEMENT OF MATRIX T ..........  
         IF (F .EQ. 0.0D0) GO TO 160
         TAU(1,L) = FLOP((AI(I,L)*TAU(2,I) - AR(I,L)*TAU(1,I))/F)
         SI = FLOP((AR(I,L)*TAU(2,I) + AI(I,L)*TAU(1,I))/F) 
         H = FLOP(H + F*G)  
         G = FLOP(1.0D0 + G/F)  
         AR(I,L) = FLOP(G*AR(I,L))  
         AI(I,L) = FLOP(G*AI(I,L))  
         IF (L .EQ. 1) GO TO 270
         GO TO 170  
  160    TAU(1,L) = -TAU(1,I)
         SI = TAU(2,I)  
         AR(I,L) = G
  170    F = 0.0D0  
C
         DO 240 J = 1, L
            G = 0.0D0
            GI = 0.0D0  
C     .......... FORM ELEMENT OF A*U .......... 
            DO 180 K = 1, J 
               G = FLOP(G + AR(J,K)*AR(I,K) + AI(J,K)*AI(I,K))  
               GI = FLOP(GI - AR(J,K)*AI(I,K) + AI(J,K)*AR(I,K))
  180       CONTINUE
C
            JP1 = J + 1 
            IF (L .LT. JP1) GO TO 220
C
            DO 200 K = JP1, L
               G = FLOP(G + AR(K,J)*AR(I,K) - AI(K,J)*AI(I,K))  
               GI = FLOP(GI - AR(K,J)*AI(I,K) - AI(K,J)*AR(I,K))
  200       CONTINUE
C     .......... FORM ELEMENT OF P ..........
  220       E(J) = FLOP(G/H)
            TAU(2,J) = FLOP(GI/H)
            F = FLOP(F + E(J)*AR(I,J) - TAU(2,J)*AI(I,J))
  240    CONTINUE
C
         HH = FLOP(F/(H + H))
C     .......... FORM REDUCED A ..........  
         DO 260 J = 1, L
            F = AR(I,J) 
            G = FLOP(E(J) - HH*F)
            E(J) = G
            FI = -AI(I,J)
            GI = FLOP(TAU(2,J) - HH*FI) 
            TAU(2,J) = -GI  
C
            DO 260 K = 1, J 
               AR(J,K) = FLOP(AR(J,K) - F*E(K) - G*AR(I,K)  
     X                           + FI*TAU(2,K) + GI*AI(I,K))
               AI(J,K) = FLOP(AI(J,K) - F*TAU(2,K) - G*AI(I,K)  
     X                           - FI*E(K) - GI*AR(I,K))
  260    CONTINUE
C
  270    DO 280 K = 1, L
            AR(I,K) = FLOP(SCALE*AR(I,K))
            AI(I,K) = FLOP(SCALE*AI(I,K))
  280    CONTINUE
C
         TAU(2,L) = -SI 
  290    HH = D(I)  
         D(I) = AR(I,I) 
         AR(I,I) = HH
         AI(I,I) = FLOP(SCALE*DSQRT(H)) 
  300 CONTINUE  
C
      RETURN
      END
      SUBROUTINE HTRIBK(NM,N,AR,AI,TAU,M,ZR,ZI) 
C
      INTEGER I,J,K,L,M,N,NM
      DOUBLE PRECISION AR(NM,N),AI(NM,N),TAU(2,N),ZR(NM,M),ZI(NM,M) 
      DOUBLE PRECISION H,S,SI,FLOP  
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF 
C     THE ALGOL PROCEDURE TRBAK1, NUM. MATH. 11, 181-195(1968)  
C     BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A COMPLEX HERMITIAN 
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
C     REAL SYMMETRIC TRIDIAGONAL MATRIX DETERMINED BY  HTRIDI.  
C
C     ON INPUT. 
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM  
C          DIMENSION STATEMENT. 
C
C        N IS THE ORDER OF THE MATRIX.  
C
C        AR AND AI CONTAIN INFORMATION ABOUT THE UNITARY TRANS- 
C          FORMATIONS USED IN THE REDUCTION BY  HTRIDI  IN THEIR
C          FULL LOWER TRIANGLES EXCEPT FOR THE DIAGONAL OF AR.  
C
C        TAU CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C
C        M IS THE NUMBER OF EIGENVECTORS TO BE BACK TRANSFORMED.
C
C        ZR CONTAINS THE EIGENVECTORS TO BE BACK TRANSFORMED
C          IN ITS FIRST M COLUMNS.  
C
C     ON OUTPUT.
C
C        ZR AND ZI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE TRANSFORMED EIGENVECTORS
C          IN THEIR FIRST M COLUMNS.
C
C     NOTE THAT THE LAST COMPONENT OF EACH RETURNED VECTOR  
C     IS REAL AND THAT VECTOR EUCLIDEAN NORMS ARE PRESERVED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY 
C
C     ------------------------------------------------------------------
C
      IF (M .EQ. 0) GO TO 200
C     .......... TRANSFORM THE EIGENVECTORS OF THE REAL SYMMETRIC
C                TRIDIAGONAL MATRIX TO THOSE OF THE HERMITIAN
C                TRIDIAGONAL MATRIX. .......... 
      DO 50 K = 1, N
C
         DO 50 J = 1, M 
            ZI(K,J) = FLOP(-ZR(K,J)*TAU(2,K))
            ZR(K,J) = FLOP(ZR(K,J)*TAU(1,K))
   50 CONTINUE  
C
      IF (N .EQ. 1) GO TO 200
C     .......... RECOVER AND APPLY THE HOUSEHOLDER MATRICES ..........  
      DO 140 I = 2, N
         L = I - 1  
         H = AI(I,I)
         IF (H .EQ. 0.0D0) GO TO 140
C
         DO 130 J = 1, M
            S = 0.0D0
            SI = 0.0D0  
C
            DO 110 K = 1, L 
               S = FLOP(S + AR(I,K)*ZR(K,J) - AI(I,K)*ZI(K,J))  
               SI = FLOP(SI + AR(I,K)*ZI(K,J) + AI(I,K)*ZR(K,J))
  110       CONTINUE
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
            S = FLOP((S/H)/H)
            SI = FLOP((SI/H)/H) 
C
            DO 120 K = 1, L 
               ZR(K,J) = FLOP(ZR(K,J) - S*AR(I,K) - SI*AI(I,K)) 
               ZI(K,J) = FLOP(ZI(K,J) - SI*AR(I,K) + S*AI(I,K)) 
  120       CONTINUE
C
  130    CONTINUE
C
  140 CONTINUE  
C
  200 RETURN
      END
      SUBROUTINE IMTQL2(NM,N,D,E,Z,IERR,JOB)
C
      INTEGER I,J,K,L,M,N,II,NM,MML,IERR
      DOUBLE PRECISION D(N),E(N),Z(NM,N)
      DOUBLE PRECISION B,C,F,G,P,R,S
      DOUBLE PRECISION FLOP 
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE IMTQL2,
C     NUM. MATH. 12, 377-383(1968) BY MARTIN AND WILKINSON, 
C     AS MODIFIED IN NUM. MATH. 15, 450(1970) BY DUBRULLE.  
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 241-248(1971).
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
C     OF A SYMMETRIC TRIDIAGONAL MATRIX BY THE IMPLICIT QL METHOD.  
C     THE EIGENVECTORS OF A FULL SYMMETRIC MATRIX CAN ALSO  
C     BE FOUND IF  TRED2  HAS BEEN USED TO REDUCE THIS  
C     FULL MATRIX TO TRIDIAGONAL FORM.  
C
C     ON INPUT. 
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM  
C          DIMENSION STATEMENT. 
C
C        N IS THE ORDER OF THE MATRIX.  
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE INPUT MATRIX.  
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
C          IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY.
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
C          REDUCTION BY  TRED2, IF PERFORMED.  IF THE EIGENVECTORS  
C          OF THE TRIDIAGONAL MATRIX ARE DESIRED, Z MUST CONTAIN
C          THE IDENTITY MATRIX. 
C
C      ON OUTPUT.
C
C        D CONTAINS THE EIGENVALUES IN ASCENDING ORDER.  IF AN  
C          ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT BUT  
C          UNORDERED FOR INDICES 1,2,...,IERR-1.
C
C        E HAS BEEN DESTROYED.  
C
C        Z CONTAINS ORTHONORMAL EIGENVECTORS OF THE SYMMETRIC
C          TRIDIAGONAL (OR FULL) MATRIX.  IF AN ERROR EXIT IS MADE, 
C          Z CONTAINS THE EIGENVECTORS ASSOCIATED WITH THE STORED
C          EIGENVALUES. 
C
C        IERR IS SET TO 
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE J-TH EIGENVALUE HAS NOT BEEN
C                     DETERMINED AFTER 30 ITERATIONS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY 
C
C     ------------------------------------------------------------------
C
C
C*****  
C     MODIFIED BY C. MOLER TO ELIMINATE MACHEP 11/22/78 
C     MODIFIED TO ADD JOB PARAMETER 08/27/79
C*****  
      IERR = 0  
      IF (N .EQ. 1) GO TO 1001  
C
      DO 100 I = 2, N
  100 E(I-1) = E(I) 
C
      E(N) = 0.0D0  
C
      DO 240 L = 1, N
         J = 0  
C     .......... LOOK FOR SMALL SUB-DIAGONAL ELEMENT .......... 
  105    DO 110 M = L, N
            IF (M .EQ. N) GO TO 120 
C*****  
            P = FLOP(DABS(D(M)) + DABS(D(M+1))) 
            S = FLOP(P + DABS(E(M)))
            IF (P .EQ. S) GO TO 120 
C*****  
  110    CONTINUE
C
  120    P = D(L)
         IF (M .EQ. L) GO TO 240
         IF (J .EQ. 30) GO TO 1000  
         J = J + 1  
C     .......... FORM SHIFT ..........  
         G = FLOP((D(L+1) - P)/(2.0D0*E(L)))
         R = FLOP(DSQRT(G*G+1.0D0)) 
         G = FLOP(D(M) - P + E(L)/(G + DSIGN(R,G))) 
         S = 1.0D0  
         C = 1.0D0  
         P = 0.0D0  
         MML = M - L
C     .......... FOR I=M-1 STEP -1 UNTIL L DO -- .......... 
         DO 200 II = 1, MML 
            I = M - II  
            F = FLOP(S*E(I))
            B = FLOP(C*E(I))
            IF (DABS(F) .LT. DABS(G)) GO TO 150 
            C = FLOP(G/F)
            R = FLOP(DSQRT(C*C+1.0D0))  
            E(I+1) = FLOP(F*R)  
            S = FLOP(1.0D0/R)
            C = FLOP(C*S)
            GO TO 160
  150       S = FLOP(F/G)
            R = FLOP(DSQRT(S*S+1.0D0))  
            E(I+1) = FLOP(G*R)  
            C = FLOP(1.0D0/R)
            S = FLOP(S*C)
  160       G = FLOP(D(I+1) - P)
            R = FLOP((D(I) - G)*S + 2.0D0*C*B)  
            P = FLOP(S*R)
            D(I+1) = G + P  
            G = FLOP(C*R - B)
            IF (JOB .EQ. 0) GO TO 185
C     .......... FORM VECTOR .......... 
            DO 180 K = 1, N 
               F = Z(K,I+1) 
               Z(K,I+1) = FLOP(S*Z(K,I) + C*F)  
               Z(K,I) = FLOP(C*Z(K,I) - S*F)
  180       CONTINUE
  185       CONTINUE
C
  200    CONTINUE
C
         D(L) = FLOP(D(L) - P)  
         E(L) = G
         E(M) = 0.0D0
         GO TO 105  
  240 CONTINUE  
C     .......... ORDER EIGENVALUES AND EIGENVECTORS ..........  
      DO 300 II = 2, N  
         I = II - 1 
         K = I  
         P = D(I)
C
         DO 260 J = II, N
            IF (D(J) .GE. P) GO TO 260  
            K = J
            P = D(J)
  260    CONTINUE
C
         IF (K .EQ. I) GO TO 300
         D(K) = D(I)
         D(I) = P
C
         IF (JOB .EQ. 0) GO TO 285  
         DO 280 J = 1, N
            P = Z(J,I)  
            Z(J,I) = Z(J,K) 
            Z(J,K) = P  
  280    CONTINUE
  285    CONTINUE
C
  300 CONTINUE  
C
      GO TO 1001
C     .......... SET ERROR -- NO CONVERGENCE TO AN  
C                EIGENVALUE AFTER 30 ITERATIONS ..........  
 1000 IERR = L  
 1001 RETURN
      END
      SUBROUTINE CORTH(NM,N,LOW,IGH,AR,AI,ORTR,ORTI)
C
      INTEGER I,J,M,N,II,JJ,LA,MP,NM,IGH,KP1,LOW
      DOUBLE PRECISION AR(NM,N),AI(NM,N),ORTR(IGH),ORTI(IGH)
      DOUBLE PRECISION F,G,H,FI,FR,SCALE
      DOUBLE PRECISION FLOP,PYTHAG  
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF 
C     THE ALGOL PROCEDURE ORTHES, NUM. MATH. 12, 349-368(1968)  
C     BY MARTIN AND WILKINSON.  
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 339-358(1971).
C
C     GIVEN A COMPLEX GENERAL MATRIX, THIS SUBROUTINE
C     REDUCES A SUBMATRIX SITUATED IN ROWS AND COLUMNS  
C     LOW THROUGH IGH TO UPPER HESSENBERG FORM BY
C     UNITARY SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT. 
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM  
C          DIMENSION STATEMENT. 
C
C        N IS THE ORDER OF THE MATRIX.  
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  CBAL.  IF  CBAL  HAS NOT BEEN USED,  
C          SET LOW=1, IGH=N.
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX INPUT MATRIX.
C
C     ON OUTPUT.
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE HESSENBERG MATRIX.  INFORMATION 
C          ABOUT THE UNITARY TRANSFORMATIONS USED IN THE REDUCTION  
C          IS STORED IN THE REMAINING TRIANGLES UNDER THE
C          HESSENBERG MATRIX.
C
C        ORTR AND ORTI CONTAIN FURTHER INFORMATION ABOUT THE
C          TRANSFORMATIONS.  ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY 
C
C     ------------------------------------------------------------------
C
      LA = IGH - 1  
      KP1 = LOW + 1 
      IF (LA .LT. KP1) GO TO 200
C
      DO 180 M = KP1, LA
         H = 0.0D0  
         ORTR(M) = 0.0D0
         ORTI(M) = 0.0D0
         SCALE = 0.0D0  
C     .......... SCALE COLUMN (ALGOL TOL THEN NOT NEEDED) ..........
         DO 90 I = M, IGH
   90    SCALE = FLOP(SCALE + DABS(AR(I,M-1)) + DABS(AI(I,M-1)))
C
         IF (SCALE .EQ. 0.0D0) GO TO 180
         MP = M + IGH
C     .......... FOR I=IGH STEP -1 UNTIL M DO -- .......... 
         DO 100 II = M, IGH 
            I = MP - II 
            ORTR(I) = FLOP(AR(I,M-1)/SCALE) 
            ORTI(I) = FLOP(AI(I,M-1)/SCALE) 
            H = FLOP(H + ORTR(I)*ORTR(I) + ORTI(I)*ORTI(I)) 
  100    CONTINUE
C
         G = FLOP(DSQRT(H)) 
         F = PYTHAG(ORTR(M),ORTI(M))
         IF (F .EQ. 0.0D0) GO TO 103
         H = FLOP(H + F*G)  
         G = FLOP(G/F)  
         ORTR(M) = FLOP((1.0D0 + G)*ORTR(M))
         ORTI(M) = FLOP((1.0D0 + G)*ORTI(M))
         GO TO 105  
C
  103    ORTR(M) = G
         AR(M,M-1) = SCALE  
C     .......... FORM (I-(U*UT)/H)*A .......... 
  105    DO 130 J = M, N
            FR = 0.0D0  
            FI = 0.0D0  
C     .......... FOR I=IGH STEP -1 UNTIL M DO -- .......... 
            DO 110 II = M, IGH  
               I = MP - II  
               FR = FLOP(FR + ORTR(I)*AR(I,J) + ORTI(I)*AI(I,J))
               FI = FLOP(FI + ORTR(I)*AI(I,J) - ORTI(I)*AR(I,J))
  110       CONTINUE
C
            FR = FLOP(FR/H) 
            FI = FLOP(FI/H) 
C
            DO 120 I = M, IGH
               AR(I,J) = FLOP(AR(I,J) - FR*ORTR(I) + FI*ORTI(I))
               AI(I,J) = FLOP(AI(I,J) - FR*ORTI(I) - FI*ORTR(I))
  120       CONTINUE
C
  130    CONTINUE
C     .......... FORM (I-(U*UT)/H)*A*(I-(U*UT)/H) ..........
         DO 160 I = 1, IGH  
            FR = 0.0D0  
            FI = 0.0D0  
C     .......... FOR J=IGH STEP -1 UNTIL M DO -- .......... 
            DO 140 JJ = M, IGH  
               J = MP - JJ  
               FR = FLOP(FR + ORTR(J)*AR(I,J) - ORTI(J)*AI(I,J))
               FI = FLOP(FI + ORTR(J)*AI(I,J) + ORTI(J)*AR(I,J))
  140       CONTINUE
C
            FR = FLOP(FR/H) 
            FI = FLOP(FI/H) 
C
            DO 150 J = M, IGH
               AR(I,J) = FLOP(AR(I,J) - FR*ORTR(J) - FI*ORTI(J))
               AI(I,J) = FLOP(AI(I,J) + FR*ORTI(J) - FI*ORTR(J))
  150       CONTINUE
C
  160    CONTINUE
C
         ORTR(M) = FLOP(SCALE*ORTR(M))  
         ORTI(M) = FLOP(SCALE*ORTI(M))  
         AR(M,M-1) = FLOP(-G*AR(M,M-1)) 
         AI(M,M-1) = FLOP(-G*AI(M,M-1)) 
  180 CONTINUE  
C
  200 RETURN
      END
      SUBROUTINE COMQR3(NM,N,LOW,IGH,ORTR,ORTI,HR,HI,WR,WI,ZR,ZI,IERR
     *                 ,JOB)
C*****  
C     MODIFICATION OF EISPACK COMQR2 TO ADD JOB PARAMETER
C     JOB = 0  OUTPUT H = SCHUR TRIANGULAR FORM, Z NOT USED 
C         = 1  OUTPUT H = SCHUR FORM, Z = UNITARY SIMILARITY
C         = 2  SAME AS COMQR2
C         = 3  OUTPUT H = HESSENBERG FORM, Z = UNITARY SIMILARITY
C     ALSO ELIMINATE MACHEP 
C     C. MOLER, 11/22/78 AND 09/14/80
C     OVERFLOW CONTROL IN EIGENVECTOR BACKSUBSTITUTION, 3/16/82 
C*****  
C
      INTEGER I,J,K,L,M,N,EN,II,JJ,LL,NM,NN,IGH,IP1,
     X        ITN,ITS,LOW,LP1,ENM1,IEND,IERR
      DOUBLE PRECISION HR(NM,N),HI(NM,N),WR(N),WI(N),ZR(NM,N),ZI(NM,N), 
     X       ORTR(IGH),ORTI(IGH)
      DOUBLE PRECISION SI,SR,TI,TR,XI,XR,YI,YR,ZZI,ZZR,NORM 
      DOUBLE PRECISION FLOP,PYTHAG  
C
C     THIS SUBROUTINE IS A TRANSLATION OF A UNITARY ANALOGUE OF THE 
C     ALGOL PROCEDURE  COMLR2, NUM. MATH. 16, 181-204(1970) BY PETERS
C     AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971).
C     THE UNITARY ANALOGUE SUBSTITUTES THE QR ALGORITHM OF FRANCIS  
C     (COMP. JOUR. 4, 332-345(1962)) FOR THE LR ALGORITHM.  
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
C     OF A COMPLEX UPPER HESSENBERG MATRIX BY THE QR
C     METHOD.  THE EIGENVECTORS OF A COMPLEX GENERAL MATRIX 
C     CAN ALSO BE FOUND IF  CORTH  HAS BEEN USED TO REDUCE  
C     THIS GENERAL MATRIX TO HESSENBERG FORM.
C
C     ON INPUT. 
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM  
C          DIMENSION STATEMENT. 
C
C        N IS THE ORDER OF THE MATRIX.  
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  CBAL.  IF  CBAL  HAS NOT BEEN USED,  
C          SET LOW=1, IGH=N.
C
C        ORTR AND ORTI CONTAIN INFORMATION ABOUT THE UNITARY TRANS- 
C          FORMATIONS USED IN THE REDUCTION BY  CORTH, IF PERFORMED.
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED.  IF THE EIGENVECTORS 
C          OF THE HESSENBERG MATRIX ARE DESIRED, SET ORTR(J) AND
C          ORTI(J) TO 0.0D0 FOR THESE ELEMENTS. 
C
C        HR AND HI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX UPPER HESSENBERG MATRIX.
C          THEIR LOWER TRIANGLES BELOW THE SUBDIAGONAL CONTAIN FURTHER  
C          INFORMATION ABOUT THE TRANSFORMATIONS WHICH WERE USED IN THE 
C          REDUCTION BY  CORTH, IF PERFORMED.  IF THE EIGENVECTORS OF
C          THE HESSENBERG MATRIX ARE DESIRED, THESE ELEMENTS MAY BE 
C          ARBITRARY.
C
C     ON OUTPUT.
C
C        ORTR, ORTI, AND THE UPPER HESSENBERG PORTIONS OF HR AND HI 
C          HAVE BEEN DESTROYED. 
C
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVALUES.  IF AN ERROR
C          EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT  
C          FOR INDICES IERR+1,...,N.
C
C        ZR AND ZI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVECTORS.  THE EIGENVECTORS 
C          ARE UNNORMALIZED.  IF AN ERROR EXIT IS MADE, NONE OF 
C          THE EIGENVECTORS HAS BEEN FOUND. 
C
C        IERR IS SET TO 
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE J-TH EIGENVALUE HAS NOT BEEN
C                     DETERMINED AFTER A TOTAL OF 30*N ITERATIONS.  
C
C     MODIFIED TO GET RID OF ALL COMPLEX ARITHMETIC, C. MOLER, 6/27/79. 
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY 
C
C     ------------------------------------------------------------------
C
      IERR = 0  
C*****  
      IF (JOB .EQ. 0) GO TO 150 
C*****  
C     .......... INITIALIZE EIGENVECTOR MATRIX ..........
      DO 100 I = 1, N
C
         DO 100 J = 1, N
            ZR(I,J) = 0.0D0 
            ZI(I,J) = 0.0D0 
            IF (I .EQ. J) ZR(I,J) = 1.0D0
  100 CONTINUE  
C     .......... FORM THE MATRIX OF ACCUMULATED TRANSFORMATIONS 
C                FROM THE INFORMATION LEFT BY CORTH ..........  
      IEND = IGH - LOW - 1  
      IF (IEND) 180, 150, 105
C     .......... FOR I=IGH-1 STEP -1 UNTIL LOW+1 DO -- ..........
  105 DO 140 II = 1, IEND
         I = IGH - II
         IF (ORTR(I) .EQ. 0.0D0 .AND. ORTI(I) .EQ. 0.0D0) GO TO 140 
         IF (HR(I,I-1) .EQ. 0.0D0 .AND. HI(I,I-1) .EQ. 0.0D0) GO TO 140 
C     .......... NORM BELOW IS NEGATIVE OF H FORMED IN CORTH .......... 
         NORM = FLOP(HR(I,I-1)*ORTR(I) + HI(I,I-1)*ORTI(I)) 
         IP1 = I + 1
C
         DO 110 K = IP1, IGH
            ORTR(K) = HR(K,I-1) 
            ORTI(K) = HI(K,I-1) 
  110    CONTINUE
C
         DO 130 J = I, IGH  
            SR = 0.0D0  
            SI = 0.0D0  
C
            DO 115 K = I, IGH
               SR = FLOP(SR + ORTR(K)*ZR(K,J) + ORTI(K)*ZI(K,J))
               SI = FLOP(SI + ORTR(K)*ZI(K,J) - ORTI(K)*ZR(K,J))
  115       CONTINUE
C
            SR = FLOP(SR/NORM)  
            SI = FLOP(SI/NORM)  
C
            DO 120 K = I, IGH
               ZR(K,J) = FLOP(ZR(K,J) + SR*ORTR(K) - SI*ORTI(K))
               ZI(K,J) = FLOP(ZI(K,J) + SR*ORTI(K) + SI*ORTR(K))
  120       CONTINUE
C
  130    CONTINUE
C
  140 CONTINUE  
C*****  
      IF (JOB .EQ. 3) GO TO 1001
C*****  
C     .......... CREATE REAL SUBDIAGONAL ELEMENTS ..........
  150 L = LOW + 1
C
      DO 170 I = L, IGH 
         LL = MIN0(I+1,IGH) 
         IF (HI(I,I-1) .EQ. 0.0D0) GO TO 170
         NORM = PYTHAG(HR(I,I-1),HI(I,I-1)) 
         YR = FLOP(HR(I,I-1)/NORM)  
         YI = FLOP(HI(I,I-1)/NORM)  
         HR(I,I-1) = NORM
         HI(I,I-1) = 0.0D0  
C
         DO 155 J = I, N
            SI = FLOP(YR*HI(I,J) - YI*HR(I,J))  
            HR(I,J) = FLOP(YR*HR(I,J) + YI*HI(I,J)) 
            HI(I,J) = SI
  155    CONTINUE
C
         DO 160 J = 1, LL
            SI = FLOP(YR*HI(J,I) + YI*HR(J,I))  
            HR(J,I) = FLOP(YR*HR(J,I) - YI*HI(J,I)) 
            HI(J,I) = SI
  160    CONTINUE
C*****  
         IF (JOB .EQ. 0) GO TO 170  
C*****  
         DO 165 J = LOW, IGH
            SI = FLOP(YR*ZI(J,I) + YI*ZR(J,I))  
            ZR(J,I) = FLOP(YR*ZR(J,I) - YI*ZI(J,I)) 
            ZI(J,I) = SI
  165    CONTINUE
C
  170 CONTINUE  
C     .......... STORE ROOTS ISOLATED BY CBAL ..........
  180 DO 200 I = 1, N
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 200 
         WR(I) = HR(I,I)
         WI(I) = HI(I,I)
  200 CONTINUE  
C
      EN = IGH  
      TR = 0.0D0
      TI = 0.0D0
      ITN = 30*N
C     .......... SEARCH FOR NEXT EIGENVALUE ..........  
  220 IF (EN .LT. LOW) GO TO 680
      ITS = 0
      ENM1 = EN - 1 
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT 
C                FOR L=EN STEP -1 UNTIL LOW DO -- ..........
  240 DO 260 LL = LOW, EN
         L = EN + LOW - LL  
         IF (L .EQ. LOW) GO TO 300  
C*****  
         XR = FLOP(DABS(HR(L-1,L-1)) + DABS(HI(L-1,L-1))
     X             + DABS(HR(L,L)) +DABS(HI(L,L)))  
         YR = FLOP(XR + DABS(HR(L,L-1)))
         IF (XR .EQ. YR) GO TO 300  
C*****  
  260 CONTINUE  
C     .......... FORM SHIFT ..........  
  300 IF (L .EQ. EN) GO TO 660  
      IF (ITN .EQ. 0) GO TO 1000
      IF (ITS .EQ. 10 .OR. ITS .EQ. 20) GO TO 320
      SR = HR(EN,EN)
      SI = HI(EN,EN)
      XR = FLOP(HR(ENM1,EN)*HR(EN,ENM1))
      XI = FLOP(HI(ENM1,EN)*HR(EN,ENM1))
      IF (XR .EQ. 0.0D0 .AND. XI .EQ. 0.0D0) GO TO 340  
      YR = FLOP((HR(ENM1,ENM1) - SR)/2.0D0) 
      YI = FLOP((HI(ENM1,ENM1) - SI)/2.0D0) 
      CALL WSQRT(YR**2-YI**2+XR,2.0D0*YR*YI+XI,ZZR,ZZI) 
      IF (YR*ZZR + YI*ZZI .GE. 0.0D0) GO TO 310 
      ZZR = -ZZR
      ZZI = -ZZI
  310 CALL WDIV(XR,XI,YR+ZZR,YI+ZZI,ZZR,ZZI)
      SR = FLOP(SR - ZZR)
      SI = FLOP(SI - ZZI)
      GO TO 340 
C     .......... FORM EXCEPTIONAL SHIFT ..........  
  320 SR = FLOP(DABS(HR(EN,ENM1)) + DABS(HR(ENM1,EN-2)))
      SI = 0.0D0
C
  340 DO 360 I = LOW, EN
         HR(I,I) = FLOP(HR(I,I) - SR)
         HI(I,I) = FLOP(HI(I,I) - SI)
  360 CONTINUE  
C
      TR = FLOP(TR + SR)
      TI = FLOP(TI + SI)
      ITS = ITS + 1 
      ITN = ITN - 1 
C     .......... REDUCE TO TRIANGLE (ROWS) ..........
      LP1 = L + 1
C
      DO 500 I = LP1, EN
         SR = HR(I,I-1) 
         HR(I,I-1) = 0.0D0  
         NORM = FLOP(DABS(HR(I-1,I-1)) + DABS(HI(I-1,I-1)) + DABS(SR))  
         NORM = FLOP(NORM*DSQRT((HR(I-1,I-1)/NORM)**2 + 
     X                     (HI(I-1,I-1)/NORM)**2 + (SR/NORM)**2))
         XR = FLOP(HR(I-1,I-1)/NORM)
         WR(I-1) = XR
         XI = FLOP(HI(I-1,I-1)/NORM)
         WI(I-1) = XI
         HR(I-1,I-1) = NORM 
         HI(I-1,I-1) = 0.0D0
         HI(I,I-1) = FLOP(SR/NORM)  
C
         DO 490 J = I, N
            YR = HR(I-1,J)  
            YI = HI(I-1,J)  
            ZZR = HR(I,J)
            ZZI = HI(I,J)
            HR(I-1,J) = FLOP(XR*YR + XI*YI + HI(I,I-1)*ZZR) 
            HI(I-1,J) = FLOP(XR*YI - XI*YR + HI(I,I-1)*ZZI) 
            HR(I,J) = FLOP(XR*ZZR - XI*ZZI - HI(I,I-1)*YR)  
            HI(I,J) = FLOP(XR*ZZI + XI*ZZR - HI(I,I-1)*YI)  
  490    CONTINUE
C
  500 CONTINUE  
C
      SI = HI(EN,EN)
      IF (SI .EQ. 0.0D0) GO TO 540  
      NORM = PYTHAG(HR(EN,EN),SI)
      SR = FLOP(HR(EN,EN)/NORM) 
      SI = FLOP(SI/NORM)
      HR(EN,EN) = NORM  
      HI(EN,EN) = 0.0D0 
      IF (EN .EQ. N) GO TO 540  
      IP1 = EN + 1  
C
      DO 520 J = IP1, N 
         YR = HR(EN,J)  
         YI = HI(EN,J)  
         HR(EN,J) = FLOP(SR*YR + SI*YI) 
         HI(EN,J) = FLOP(SR*YI - SI*YR) 
  520 CONTINUE  
C     .......... INVERSE OPERATION (COLUMNS) .......... 
  540 DO 600 J = LP1, EN
         XR = WR(J-1)
         XI = WI(J-1)
C
         DO 580 I = 1, J
            YR = HR(I,J-1)  
            YI = 0.0D0  
            ZZR = HR(I,J)
            ZZI = HI(I,J)
            IF (I .EQ. J) GO TO 560 
            YI = HI(I,J-1)  
            HI(I,J-1) = FLOP(XR*YI + XI*YR + HI(J,J-1)*ZZI) 
  560       HR(I,J-1) = FLOP(XR*YR - XI*YI + HI(J,J-1)*ZZR) 
            HR(I,J) = FLOP(XR*ZZR + XI*ZZI - HI(J,J-1)*YR)  
            HI(I,J) = FLOP(XR*ZZI - XI*ZZR - HI(J,J-1)*YI)  
  580    CONTINUE
C*****  
         IF (JOB .EQ. 0) GO TO 600  
C*****  
         DO 590 I = LOW, IGH
            YR = ZR(I,J-1)  
            YI = ZI(I,J-1)  
            ZZR = ZR(I,J)
            ZZI = ZI(I,J)
            ZR(I,J-1) = FLOP(XR*YR - XI*YI + HI(J,J-1)*ZZR) 
            ZI(I,J-1) = FLOP(XR*YI + XI*YR + HI(J,J-1)*ZZI) 
            ZR(I,J) = FLOP(XR*ZZR + XI*ZZI - HI(J,J-1)*YR)  
            ZI(I,J) = FLOP(XR*ZZI - XI*ZZR - HI(J,J-1)*YI)  
  590    CONTINUE
C
  600 CONTINUE  
C
      IF (SI .EQ. 0.0D0) GO TO 240  
C
      DO 630 I = 1, EN  
         YR = HR(I,EN)  
         YI = HI(I,EN)  
         HR(I,EN) = FLOP(SR*YR - SI*YI) 
         HI(I,EN) = FLOP(SR*YI + SI*YR) 
  630 CONTINUE  
C*****  
      IF (JOB .EQ. 0) GO TO 240 
C*****  
      DO 640 I = LOW, IGH
         YR = ZR(I,EN)  
         YI = ZI(I,EN)  
         ZR(I,EN) = FLOP(SR*YR - SI*YI) 
         ZI(I,EN) = FLOP(SR*YI + SI*YR) 
  640 CONTINUE  
C
      GO TO 240 
C     .......... A ROOT FOUND ..........
  660 HR(EN,EN) = FLOP(HR(EN,EN) + TR)  
      WR(EN) = HR(EN,EN)
      HI(EN,EN) = FLOP(HI(EN,EN) + TI)  
      WI(EN) = HI(EN,EN)
      EN = ENM1 
      GO TO 220 
C     .......... ALL ROOTS FOUND.  BACKSUBSTITUTE TO FIND
C                VECTORS OF UPPER TRIANGULAR FORM ..........
C
C*****  THE FOLLOWING SECTION CHANGED FOR OVERFLOW CONTROL  
C       C. MOLER, 3/16/82
C
  680 IF (JOB .NE. 2) GO TO 1001
C
      NORM = 0.0D0  
      DO 720 I = 1, N
         DO 720 J = I, N
            TR = FLOP(DABS(HR(I,J))) + FLOP(DABS(HI(I,J)))  
            IF (TR .GT. NORM) NORM = TR 
  720 CONTINUE  
      IF (N .EQ. 1 .OR. NORM .EQ. 0.0D0) GO TO 1001 
C     .......... FOR EN=N STEP -1 UNTIL 2 DO -- ..........  
      DO 800 NN = 2, N  
         EN = N + 2 - NN
         XR = WR(EN)
         XI = WI(EN)
         HR(EN,EN) = 1.0D0  
         HI(EN,EN) = 0.0D0  
         ENM1 = EN - 1  
C     .......... FOR I=EN-1 STEP -1 UNTIL 1 DO -- ..........
         DO 780 II = 1, ENM1
            I = EN - II 
            ZZR = 0.0D0 
            ZZI = 0.0D0 
            IP1 = I + 1 
            DO 740 J = IP1, EN  
               ZZR = FLOP(ZZR + HR(I,J)*HR(J,EN) - HI(I,J)*HI(J,EN))
               ZZI = FLOP(ZZI + HR(I,J)*HI(J,EN) + HI(I,J)*HR(J,EN))
  740       CONTINUE
            YR = FLOP(XR - WR(I))
            YI = FLOP(XI - WI(I))
            IF (YR .NE. 0.0D0 .OR. YI .NE. 0.0D0) GO TO 765 
               YR = NORM
  760          YR = FLOP(YR/100.0D0)
               YI = FLOP(NORM + YR) 
               IF (YI .NE. NORM) GO TO 760  
               YI = 0.0D0
  765       CONTINUE
            CALL WDIV(ZZR,ZZI,YR,YI,HR(I,EN),HI(I,EN))  
            TR = FLOP(DABS(HR(I,EN))) + FLOP(DABS(HI(I,EN)))
            IF (TR .EQ. 0.0D0) GO TO 780
            IF (TR + 1.0D0/TR .GT. TR) GO TO 780
            DO 770 J = I, EN
               HR(J,EN) = FLOP(HR(J,EN)/TR) 
               HI(J,EN) = FLOP(HI(J,EN)/TR) 
  770       CONTINUE
  780    CONTINUE
C
  800 CONTINUE  
C*****  
C     .......... END BACKSUBSTITUTION ..........
      ENM1 = N - 1  
C     .......... VECTORS OF ISOLATED ROOTS ..........
      DO  840 I = 1, ENM1
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 840 
         IP1 = I + 1
C
         DO 820 J = IP1, N  
            ZR(I,J) = HR(I,J)
            ZI(I,J) = HI(I,J)
  820    CONTINUE
C
  840 CONTINUE  
C     .......... MULTIPLY BY TRANSFORMATION MATRIX TO GIVE  
C                VECTORS OF ORIGINAL FULL MATRIX.
C                FOR J=N STEP -1 UNTIL LOW+1 DO -- ..........
      DO 880 JJ = LOW, ENM1 
         J = N + LOW - JJ
         M = MIN0(J,IGH)
C
         DO 880 I = LOW, IGH
            ZZR = 0.0D0 
            ZZI = 0.0D0 
C
            DO 860 K = LOW, M
               ZZR = FLOP(ZZR + ZR(I,K)*HR(K,J) - ZI(I,K)*HI(K,J))  
               ZZI = FLOP(ZZI + ZR(I,K)*HI(K,J) + ZI(I,K)*HR(K,J))  
  860       CONTINUE
C
            ZR(I,J) = ZZR
            ZI(I,J) = ZZI
  880 CONTINUE  
C
      GO TO 1001
C     .......... SET ERROR -- NO CONVERGENCE TO AN  
C                EIGENVALUE AFTER 30 ITERATIONS ..........  
 1000 IERR = EN 
 1001 RETURN
      END
      SUBROUTINE WSVDC(XR,XI,LDX,N,P,SR,SI,ER,EI,UR,UI,LDU,VR,VI,LDV,
     *                 WORKR,WORKI,JOB,INFO)
      INTEGER LDX,N,P,LDU,LDV,JOB,INFO  
      DOUBLE PRECISION XR(LDX,1),XI(LDX,1),SR(1),SI(1),ER(1),EI(1), 
     *                 UR(LDU,1),UI(LDU,1),VR(LDV,1),VI(LDV,1), 
     *                 WORKR(1),WORKI(1)
C
C
C     WSVDC IS A SUBROUTINE TO REDUCE A DOUBLE-COMPLEX NXP MATRIX X BY  
C     UNITARY TRANSFORMATIONS U AND V TO DIAGONAL FORM.  THE
C     DIAGONAL ELEMENTS S(I) ARE THE SINGULAR VALUES OF X.  THE 
C     COLUMNS OF U ARE THE CORRESPONDING LEFT SINGULAR VECTORS, 
C     AND THE COLUMNS OF V THE RIGHT SINGULAR VECTORS.  
C
C     ON ENTRY  
C
C         X         DOUBLE-COMPLEX(LDX,P), WHERE LDX.GE.N.  
C                   X CONTAINS THE MATRIX WHOSE SINGULAR VALUE  
C                   DECOMPOSITION IS TO BE COMPUTED.  X IS  
C                   DESTROYED BY WSVDC. 
C
C         LDX       INTEGER.
C                   LDX IS THE LEADING DIMENSION OF THE ARRAY X.
C
C         N         INTEGER.
C                   N IS THE NUMBER OF COLUMNS OF THE MATRIX X. 
C
C         P         INTEGER.
C                   P IS THE NUMBER OF ROWS OF THE MATRIX X.
C
C         LDU       INTEGER.
C                   LDU IS THE LEADING DIMENSION OF THE ARRAY U 
C                   (SEE BELOW).
C
C         LDV       INTEGER.
C                   LDV IS THE LEADING DIMENSION OF THE ARRAY V 
C                   (SEE BELOW).
C
C         WORK      DOUBLE-COMPLEX(N).  
C                   WORK IS A SCRATCH ARRAY.
C
C         JOB       INTEGER.
C                   JOB CONTROLS THE COMPUTATION OF THE SINGULAR
C                   VECTORS.  IT HAS THE DECIMAL EXPANSION AB
C                   WITH THE FOLLOWING MEANING  
C
C                        A.EQ.0    DO NOT COMPUTE THE LEFT SINGULAR 
C                                  VECTORS. 
C                        A.EQ.1    RETURN THE N LEFT SINGULAR VECTORS
C                                  IN U.
C                        A.GE.2    RETURNS THE FIRST MIN(N,P)
C                                  LEFT SINGULAR VECTORS IN U.  
C                        B.EQ.0    DO NOT COMPUTE THE RIGHT SINGULAR
C                                  VECTORS. 
C                        B.EQ.1    RETURN THE RIGHT SINGULAR VECTORS
C                                  IN V.
C
C     ON RETURN 
C
C         S         DOUBLE-COMPLEX(MM), WHERE MM=MIN(N+1,P).
C                   THE FIRST MIN(N,P) ENTRIES OF S CONTAIN THE 
C                   SINGULAR VALUES OF X ARRANGED IN DESCENDING 
C                   ORDER OF MAGNITUDE. 
C
C         E         DOUBLE-COMPLEX(P).  
C                   E ORDINARILY CONTAINS ZEROS.  HOWEVER SEE THE
C                   DISCUSSION OF INFO FOR EXCEPTIONS.  
C
C         U         DOUBLE-COMPLEX(LDU,K), WHERE LDU.GE.N.  
C                   IF JOBA.EQ.1 THEN K.EQ.N,
C                   IF JOBA.EQ.2 THEN K.EQ.MIN(N,P).
C                   U CONTAINS THE MATRIX OF RIGHT SINGULAR VECTORS.
C                   U IS NOT REFERENCED IF JOBA.EQ.0.  IF N.LE.P
C                   OR IF JOBA.GT.2, THEN U MAY BE IDENTIFIED WITH X
C                   IN THE SUBROUTINE CALL. 
C
C         V         DOUBLE-COMPLEX(LDV,P), WHERE LDV.GE.P.  
C                   V CONTAINS THE MATRIX OF RIGHT SINGULAR VECTORS.
C                   V IS NOT REFERENCED IF JOBB.EQ.0.  IF P.LE.N,
C                   THEN V MAY BE IDENTIFIED WHTH X IN THE  
C                   SUBROUTINE CALL.
C
C         INFO      INTEGER.
C                   THE SINGULAR VALUES (AND THEIR CORRESPONDING
C                   SINGULAR VECTORS) S(INFO+1),S(INFO+2),...,S(M)  
C                   ARE CORRECT (HERE M=MIN(N,P)).  THUS IF 
C                   INFO.EQ.0, ALL THE SINGULAR VALUES AND THEIR
C                   VECTORS ARE CORRECT.  IN ANY EVENT, THE MATRIX  
C                   B = CTRANS(U)*X*V IS THE BIDIAGONAL MATRIX  
C                   WITH THE ELEMENTS OF S ON ITS DIAGONAL AND THE  
C                   ELEMENTS OF E ON ITS SUPER-DIAGONAL (CTRANS(U)  
C                   IS THE CONJUGATE-TRANSPOSE OF U).  THUS THE 
C                   SINGULAR VALUES OF X AND B ARE THE SAME.
C
C     LINPACK. THIS VERSION DATED 07/03/79 .
C     G.W. STEWART, UNIVERSITY OF MARYLAND, ARGONNE NATIONAL LAB.
C
C     WSVDC USES THE FOLLOWING FUNCTIONS AND SUBPROGRAMS.
C
C     BLAS WAXPY,PYTHAG,WDOTCR,WDOTCI,WSCAL,WSWAP,WNRM2,RROTG
C     FORTRAN DABS,DIMAG,DMAX1  
C     FORTRAN MAX0,MIN0,MOD,DSQRT
C
C     INTERNAL VARIABLES
C
      INTEGER I,ITER,J,JOBU,K,KASE,KK,L,LL,LLS,LM1,LP1,LS,LU,M,MAXIT,
     *        MM,MM1,MP1,NCT,NCTP1,NCU,NRT,NRTP1
      DOUBLE PRECISION PYTHAG,WDOTCR,WDOTCI,TR,TI,RR,RI 
      DOUBLE PRECISION B,C,CS,EL,EMM1,F,G,WNRM2,SCALE,SHIFT,SL,SM,SN,
     *                 SMM1,T1,TEST,ZTEST,SMALL,FLOP
      LOGICAL WANTU,WANTV
C
      DOUBLE PRECISION ZDUMR,ZDUMI  
      DOUBLE PRECISION CABS1
      CABS1(ZDUMR,ZDUMI) = DABS(ZDUMR) + DABS(ZDUMI)
C
C     SET THE MAXIMUM NUMBER OF ITERATIONS. 
C
      MAXIT = 75
C
C     SMALL NUMBER, ROUGHLY MACHINE EPSILON, USED TO AVOID UNDERFLOW
C
      SMALL = 1.D0/2.D0**48 
C
C     DETERMINE WHAT IS TO BE COMPUTED. 
C
      WANTU = .FALSE.
      WANTV = .FALSE.
      JOBU = MOD(JOB,100)/10
      NCU = N
      IF (JOBU .GT. 1) NCU = MIN0(N,P)  
      IF (JOBU .NE. 0) WANTU = .TRUE.
      IF (MOD(JOB,10) .NE. 0) WANTV = .TRUE.
C
C     REDUCE X TO BIDIAGONAL FORM, STORING THE DIAGONAL ELEMENTS
C     IN S AND THE SUPER-DIAGONAL ELEMENTS IN E.
C
      INFO = 0  
      NCT = MIN0(N-1,P) 
      NRT = MAX0(0,MIN0(P-2,N)) 
      LU = MAX0(NCT,NRT)
      IF (LU .LT. 1) GO TO 190  
      DO 180 L = 1, LU  
         LP1 = L + 1
         IF (L .GT. NCT) GO TO 30
C
C           COMPUTE THE TRANSFORMATION FOR THE L-TH COLUMN AND  
C           PLACE THE L-TH DIAGONAL IN S(L).
C
            SR(L) = WNRM2(N-L+1,XR(L,L),XI(L,L),1)  
            SI(L) = 0.0D0
            IF (CABS1(SR(L),SI(L)) .EQ. 0.0D0) GO TO 20 
               IF (CABS1(XR(L,L),XI(L,L)) .EQ. 0.0D0) GO TO 10  
                  CALL WSIGN(SR(L),SI(L),XR(L,L),XI(L,L),SR(L),SI(L))
   10          CONTINUE 
               CALL WDIV(1.0D0,0.0D0,SR(L),SI(L),TR,TI) 
               CALL WSCAL(N-L+1,TR,TI,XR(L,L),XI(L,L),1)
               XR(L,L) = FLOP(1.0D0 + XR(L,L))  
   20       CONTINUE
            SR(L) = -SR(L)  
            SI(L) = -SI(L)  
   30    CONTINUE
         IF (P .LT. LP1) GO TO 60
         DO 50 J = LP1, P
            IF (L .GT. NCT) GO TO 40
            IF (CABS1(SR(L),SI(L)) .EQ. 0.0D0) GO TO 40 
C
C              APPLY THE TRANSFORMATION.
C
               TR = -WDOTCR(N-L+1,XR(L,L),XI(L,L),1,XR(L,J),XI(L,J),1)  
               TI = -WDOTCI(N-L+1,XR(L,L),XI(L,L),1,XR(L,J),XI(L,J),1)  
               CALL WDIV(TR,TI,XR(L,L),XI(L,L),TR,TI)
               CALL WAXPY(N-L+1,TR,TI,XR(L,L),XI(L,L),1,XR(L,J),
     *                    XI(L,J),1)
   40       CONTINUE
C
C           PLACE THE L-TH ROW OF X INTO  E FOR THE 
C           SUBSEQUENT CALCULATION OF THE ROW TRANSFORMATION.
C
            ER(J) = XR(L,J) 
            EI(J) = -XI(L,J)
   50    CONTINUE
   60    CONTINUE
         IF (.NOT.WANTU .OR. L .GT. NCT) GO TO 80
C
C           PLACE THE TRANSFORMATION IN U FOR SUBSEQUENT BACK
C           MULTIPLICATION. 
C
            DO 70 I = L, N  
               UR(I,L) = XR(I,L)
               UI(I,L) = XI(I,L)
   70       CONTINUE
   80    CONTINUE
         IF (L .GT. NRT) GO TO 170  
C
C           COMPUTE THE L-TH ROW TRANSFORMATION AND PLACE THE
C           L-TH SUPER-DIAGONAL IN E(L).
C
            ER(L) = WNRM2(P-L,ER(LP1),EI(LP1),1)
            EI(L) = 0.0D0
            IF (CABS1(ER(L),EI(L)) .EQ. 0.0D0) GO TO 100
               IF (CABS1(ER(LP1),EI(LP1)) .EQ. 0.0D0) GO TO 90  
                  CALL WSIGN(ER(L),EI(L),ER(LP1),EI(LP1),ER(L),EI(L))
   90          CONTINUE 
               CALL WDIV(1.0D0,0.0D0,ER(L),EI(L),TR,TI) 
               CALL WSCAL(P-L,TR,TI,ER(LP1),EI(LP1),1)  
               ER(LP1) = FLOP(1.0D0 + ER(LP1))  
  100       CONTINUE
            ER(L) = -ER(L)  
            EI(L) = +EI(L)  
            IF (LP1 .GT. N .OR. CABS1(ER(L),EI(L)) .EQ. 0.0D0)  
     *         GO TO 140
C
C              APPLY THE TRANSFORMATION.
C
               DO 110 I = LP1, N
                  WORKR(I) = 0.0D0  
                  WORKI(I) = 0.0D0  
  110          CONTINUE 
               DO 120 J = LP1, P
                  CALL WAXPY(N-L,ER(J),EI(J),XR(LP1,J),XI(LP1,J),1, 
     *                       WORKR(LP1),WORKI(LP1),1)
  120          CONTINUE 
               DO 130 J = LP1, P
                  CALL WDIV(-ER(J),-EI(J),ER(LP1),EI(LP1),TR,TI)
                  CALL WAXPY(N-L,TR,-TI,WORKR(LP1),WORKI(LP1),1,
     *                       XR(LP1,J),XI(LP1,J),1) 
  130          CONTINUE 
  140       CONTINUE
            IF (.NOT.WANTV) GO TO 160
C
C              PLACE THE TRANSFORMATION IN V FOR SUBSEQUENT 
C              BACK MULTIPLICATION. 
C
               DO 150 I = LP1, P
                  VR(I,L) = ER(I)
                  VI(I,L) = EI(I)
  150          CONTINUE 
  160       CONTINUE
  170    CONTINUE
  180 CONTINUE  
  190 CONTINUE  
C
C     SET UP THE FINAL BIDIAGONAL MATRIX OR ORDER M.
C
      M = MIN0(P,N+1)
      NCTP1 = NCT + 1
      NRTP1 = NRT + 1
      IF (NCT .GE. P) GO TO 200 
         SR(NCTP1) = XR(NCTP1,NCTP1)
         SI(NCTP1) = XI(NCTP1,NCTP1)
  200 CONTINUE  
      IF (N .GE. M) GO TO 210
         SR(M) = 0.0D0  
         SI(M) = 0.0D0  
  210 CONTINUE  
      IF (NRTP1 .GE. M) GO TO 220
         ER(NRTP1) = XR(NRTP1,M)
         EI(NRTP1) = XI(NRTP1,M)
  220 CONTINUE  
      ER(M) = 0.0D0 
      EI(M) = 0.0D0 
C
C     IF REQUIRED, GENERATE U.  
C
      IF (.NOT.WANTU) GO TO 350 
         IF (NCU .LT. NCTP1) GO TO 250  
         DO 240 J = NCTP1, NCU  
            DO 230 I = 1, N 
               UR(I,J) = 0.0D0  
               UI(I,J) = 0.0D0  
  230       CONTINUE
            UR(J,J) = 1.0D0 
            UI(J,J) = 0.0D0 
  240    CONTINUE
  250    CONTINUE
         IF (NCT .LT. 1) GO TO 340  
         DO 330 LL = 1, NCT 
            L = NCT - LL + 1
            IF (CABS1(SR(L),SI(L)) .EQ. 0.0D0) GO TO 300
               LP1 = L + 1  
               IF (NCU .LT. LP1) GO TO 270  
               DO 260 J = LP1, NCU  
                  TR = -WDOTCR(N-L+1,UR(L,L),UI(L,L),1,UR(L,J), 
     *                         UI(L,J),1)
                  TI = -WDOTCI(N-L+1,UR(L,L),UI(L,L),1,UR(L,J), 
     *                         UI(L,J),1)
                  CALL WDIV(TR,TI,UR(L,L),UI(L,L),TR,TI)
                  CALL WAXPY(N-L+1,TR,TI,UR(L,L),UI(L,L),1,UR(L,J), 
     *                       UI(L,J),1) 
  260          CONTINUE 
  270          CONTINUE 
               CALL WRSCAL(N-L+1,-1.0D0,UR(L,L),UI(L,L),1)  
               UR(L,L) = FLOP(1.0D0 + UR(L,L))  
               LM1 = L - 1  
               IF (LM1 .LT. 1) GO TO 290
               DO 280 I = 1, LM1
                  UR(I,L) = 0.0D0
                  UI(I,L) = 0.0D0
  280          CONTINUE 
  290          CONTINUE 
            GO TO 320
  300       CONTINUE
               DO 310 I = 1, N  
                  UR(I,L) = 0.0D0
                  UI(I,L) = 0.0D0
  310          CONTINUE 
               UR(L,L) = 1.0D0  
               UI(L,L) = 0.0D0  
  320       CONTINUE
  330    CONTINUE
  340    CONTINUE
  350 CONTINUE  
C
C     IF IT IS REQUIRED, GENERATE V.
C
      IF (.NOT.WANTV) GO TO 400 
         DO 390 LL = 1, P
            L = P - LL + 1  
            LP1 = L + 1 
            IF (L .GT. NRT) GO TO 370
            IF (CABS1(ER(L),EI(L)) .EQ. 0.0D0) GO TO 370
               DO 360 J = LP1, P
                  TR = -WDOTCR(P-L,VR(LP1,L),VI(LP1,L),1,VR(LP1,J), 
     *                         VI(LP1,J),1) 
                  TI = -WDOTCI(P-L,VR(LP1,L),VI(LP1,L),1,VR(LP1,J), 
     *                         VI(LP1,J),1) 
                  CALL WDIV(TR,TI,VR(LP1,L),VI(LP1,L),TR,TI)
                  CALL WAXPY(P-L,TR,TI,VR(LP1,L),VI(LP1,L),1,VR(LP1,J), 
     *                       VI(LP1,J),1)
  360          CONTINUE 
  370       CONTINUE
            DO 380 I = 1, P 
               VR(I,L) = 0.0D0  
               VI(I,L) = 0.0D0  
  380       CONTINUE
            VR(L,L) = 1.0D0 
            VI(L,L) = 0.0D0 
  390    CONTINUE
  400 CONTINUE  
C
C     TRANSFORM S AND E SO THAT THEY ARE REAL.  
C
      DO 420 I = 1, M
            TR = PYTHAG(SR(I),SI(I))
            IF (TR .EQ. 0.0D0) GO TO 405
            RR = SR(I)/TR
            RI = SI(I)/TR
            SR(I) = TR  
            SI(I) = 0.0D0
            IF (I .LT. M) CALL WDIV(ER(I),EI(I),RR,RI,ER(I),EI(I))  
            IF (WANTU) CALL WSCAL(N,RR,RI,UR(1,I),UI(1,I),1)
  405    CONTINUE
C     ...EXIT
         IF (I .EQ. M) GO TO 430
            TR = PYTHAG(ER(I),EI(I))
            IF (TR .EQ. 0.0D0) GO TO 410
            CALL WDIV(TR,0.0D0,ER(I),EI(I),RR,RI)
            ER(I) = TR  
            EI(I) = 0.0D0
            CALL WMUL(SR(I+1),SI(I+1),RR,RI,SR(I+1),SI(I+1))
            IF (WANTV) CALL WSCAL(P,RR,RI,VR(1,I+1),VI(1,I+1),1)
  410    CONTINUE
  420 CONTINUE  
  430 CONTINUE  
C
C     MAIN ITERATION LOOP FOR THE SINGULAR VALUES.  
C
      MM = M
      ITER = 0  
  440 CONTINUE  
C
C        QUIT IF ALL THE SINGULAR VALUES HAVE BEEN FOUND.
C
C     ...EXIT
         IF (M .EQ. 0) GO TO 700
C
C        IF TOO MANY ITERATIONS HAVE BEEN PERFORMED, SET
C        FLAG AND RETURN.
C
         IF (ITER .LT. MAXIT) GO TO 450 
            INFO = M
C     ......EXIT
            GO TO 700
  450    CONTINUE
C
C        THIS SECTION OF THE PROGRAM INSPECTS FOR
C        NEGLIGIBLE ELEMENTS IN THE S AND E ARRAYS.  ON 
C        COMPLETION THE VARIABLE KASE IS SET AS FOLLOWS.
C
C           KASE = 1     IF SR(M) AND ER(L-1) ARE NEGLIGIBLE AND L.LT.M 
C           KASE = 2     IF SR(L) IS NEGLIGIBLE AND L.LT.M  
C           KASE = 3     IF ER(L-1) IS NEGLIGIBLE, L.LT.M, AND  
C                        SR(L), ..., SR(M) ARE NOT NEGLIGIBLE (QR STEP).
C           KASE = 4     IF ER(M-1) IS NEGLIGIBLE (CONVERGENCE).
C
         DO 470 LL = 1, M
            L = M - LL  
C        ...EXIT
            IF (L .EQ. 0) GO TO 480 
            TEST = FLOP(DABS(SR(L)) + DABS(SR(L+1)))
            ZTEST = FLOP(TEST + DABS(ER(L))/2.0D0)  
            IF (SMALL*ZTEST .NE. SMALL*TEST) GO TO 460  
               ER(L) = 0.0D0
C        ......EXIT 
               GO TO 480
  460       CONTINUE
  470    CONTINUE
  480    CONTINUE
         IF (L .NE. M - 1) GO TO 490
            KASE = 4
         GO TO 560  
  490    CONTINUE
            LP1 = L + 1 
            MP1 = M + 1 
            DO 510 LLS = LP1, MP1
               LS = M - LLS + LP1
C           ...EXIT 
               IF (LS .EQ. L) GO TO 520 
               TEST = 0.0D0 
               IF (LS .NE. M) TEST = FLOP(TEST + DABS(ER(LS)))  
               IF (LS .NE. L + 1) TEST = FLOP(TEST + DABS(ER(LS-1)))
               ZTEST = FLOP(TEST + DABS(SR(LS))/2.0D0)  
               IF (SMALL*ZTEST .NE. SMALL*TEST) GO TO 500
                  SR(LS) = 0.0D0
C           ......EXIT  
                  GO TO 520 
  500          CONTINUE 
  510       CONTINUE
  520       CONTINUE
            IF (LS .NE. L) GO TO 530
               KASE = 3 
            GO TO 550
  530       CONTINUE
            IF (LS .NE. M) GO TO 540
               KASE = 1 
            GO TO 550
  540       CONTINUE
               KASE = 2 
               L = LS
  550       CONTINUE
  560    CONTINUE
         L = L + 1  
C
C        PERFORM THE TASK INDICATED BY KASE.
C
         GO TO (570, 600, 620, 650), KASE
C
C        DEFLATE NEGLIGIBLE SR(M).  
C
  570    CONTINUE
            MM1 = M - 1 
            F = ER(M-1) 
            ER(M-1) = 0.0D0 
            DO 590 KK = L, MM1  
               K = MM1 - KK + L 
               T1 = SR(K)
               CALL RROTG(T1,F,CS,SN)
               SR(K) = T1
               IF (K .EQ. L) GO TO 580  
                  F = FLOP(-SN*ER(K-1)) 
                  ER(K-1) = FLOP(CS*ER(K-1))
  580          CONTINUE 
               IF (WANTV) CALL RROT(P,VR(1,K),1,VR(1,M),1,CS,SN)
               IF (WANTV) CALL RROT(P,VI(1,K),1,VI(1,M),1,CS,SN)
  590       CONTINUE
         GO TO 690  
C
C        SPLIT AT NEGLIGIBLE SR(L). 
C
  600    CONTINUE
            F = ER(L-1) 
            ER(L-1) = 0.0D0 
            DO 610 K = L, M 
               T1 = SR(K)
               CALL RROTG(T1,F,CS,SN)
               SR(K) = T1
               F = FLOP(-SN*ER(K))  
               ER(K) = FLOP(CS*ER(K))
               IF (WANTU) CALL RROT(N,UR(1,K),1,UR(1,L-1),1,CS,SN)  
               IF (WANTU) CALL RROT(N,UI(1,K),1,UI(1,L-1),1,CS,SN)  
  610       CONTINUE
         GO TO 690  
C
C        PERFORM ONE QR STEP.
C
  620    CONTINUE
C
C           CALCULATE THE SHIFT.
C
            SCALE = DMAX1(DABS(SR(M)),DABS(SR(M-1)),DABS(ER(M-1)),  
     *                    DABS(SR(L)),DABS(ER(L)))  
            SM = SR(M)/SCALE
            SMM1 = SR(M-1)/SCALE
            EMM1 = ER(M-1)/SCALE
            SL = SR(L)/SCALE
            EL = ER(L)/SCALE
            B = FLOP(((SMM1 + SM)*(SMM1 - SM) + EMM1**2)/2.0D0) 
            C = FLOP((SM*EMM1)**2)  
            SHIFT = 0.0D0
            IF (B .EQ. 0.0D0 .AND. C .EQ. 0.0D0) GO TO 630  
               SHIFT = FLOP(DSQRT(B**2+C))  
               IF (B .LT. 0.0D0) SHIFT = -SHIFT 
               SHIFT = FLOP(C/(B + SHIFT))  
  630       CONTINUE
            F = FLOP((SL + SM)*(SL - SM) - SHIFT)
            G = FLOP(SL*EL) 
C
C           CHASE ZEROS.
C
            MM1 = M - 1 
            DO 640 K = L, MM1
               CALL RROTG(F,G,CS,SN)
               IF (K .NE. L) ER(K-1) = F
               F = FLOP(CS*SR(K) + SN*ER(K))
               ER(K) = FLOP(CS*ER(K) - SN*SR(K))
               G = FLOP(SN*SR(K+1)) 
               SR(K+1) = FLOP(CS*SR(K+1))
               IF (WANTV) CALL RROT(P,VR(1,K),1,VR(1,K+1),1,CS,SN)  
               IF (WANTV) CALL RROT(P,VI(1,K),1,VI(1,K+1),1,CS,SN)  
               CALL RROTG(F,G,CS,SN)
               SR(K) = F
               F = FLOP(CS*ER(K) + SN*SR(K+1))  
               SR(K+1) = FLOP(-SN*ER(K) + CS*SR(K+1))
               G = FLOP(SN*ER(K+1)) 
               ER(K+1) = FLOP(CS*ER(K+1))
               IF (WANTU .AND. K .LT. N)
     *            CALL RROT(N,UR(1,K),1,UR(1,K+1),1,CS,SN)  
               IF (WANTU .AND. K .LT. N)
     *            CALL RROT(N,UI(1,K),1,UI(1,K+1),1,CS,SN)  
  640       CONTINUE
            ER(M-1) = F 
            ITER = ITER + 1 
         GO TO 690  
C
C        CONVERGENCE
C
  650    CONTINUE
C
C           MAKE THE SINGULAR VALUE  POSITIVE
C
            IF (SR(L) .GE. 0.0D0) GO TO 660 
               SR(L) = -SR(L)
             IF (WANTV) CALL WRSCAL(P,-1.0D0,VR(1,L),VI(1,L),1) 
  660       CONTINUE
C
C           ORDER THE SINGULAR VALUE.
C
  670       IF (L .EQ. MM) GO TO 680
C           ...EXIT 
               IF (SR(L) .GE. SR(L+1)) GO TO 680
               TR = SR(L)
               SR(L) = SR(L+1)  
               SR(L+1) = TR 
               IF (WANTV .AND. L .LT. P)
     *            CALL WSWAP(P,VR(1,L),VI(1,L),1,VR(1,L+1),VI(1,L+1),1) 
               IF (WANTU .AND. L .LT. N)
     *            CALL WSWAP(N,UR(1,L),UI(1,L),1,UR(1,L+1),UI(1,L+1),1) 
               L = L + 1
            GO TO 670
  680       CONTINUE
            ITER = 0
            M = M - 1
  690    CONTINUE
      GO TO 440 
  700 CONTINUE  
      RETURN
      END
      SUBROUTINE WQRDC(XR,XI,LDX,N,P,QRAUXR,QRAUXI,JPVT,WORKR,WORKI,
     *                 JOB) 
      INTEGER LDX,N,P,JOB
      INTEGER JPVT(1)
      DOUBLE PRECISION XR(LDX,1),XI(LDX,1),QRAUXR(1),QRAUXI(1), 
     *                 WORKR(1),WORKI(1)
C
C     WQRDC USES HOUSEHOLDER TRANSFORMATIONS TO COMPUTE THE QR  
C     FACTORIZATION OF AN N BY P MATRIX X.  COLUMN PIVOTING 
C     BASED ON THE 2-NORMS OF THE REDUCED COLUMNS MAY BE
C     PERFORMED AT THE USERS OPTION.
C
C     ON ENTRY  
C
C        X       DOUBLE-COMPLEX(LDX,P), WHERE LDX .GE. N.
C                X CONTAINS THE MATRIX WHOSE DECOMPOSITION IS TO BE 
C                COMPUTED.  
C
C        LDX     INTEGER.
C                LDX IS THE LEADING DIMENSION OF THE ARRAY X.
C
C        N       INTEGER.
C                N IS THE NUMBER OF ROWS OF THE MATRIX X.
C
C        P       INTEGER.
C                P IS THE NUMBER OF COLUMNS OF THE MATRIX X.
C
C        JPVT    INTEGER(P).
C                JPVT CONTAINS INTEGERS THAT CONTROL THE SELECTION  
C                OF THE PIVOT COLUMNS.  THE K-TH COLUMN X(K) OF X
C                IS PLACED IN ONE OF THREE CLASSES ACCORDING TO THE 
C                VALUE OF JPVT(K).  
C
C                   IF JPVT(K) .GT. 0, THEN X(K) IS AN INITIAL  
C                                      COLUMN.  
C
C                   IF JPVT(K) .EQ. 0, THEN X(K) IS A FREE COLUMN.  
C
C                   IF JPVT(K) .LT. 0, THEN X(K) IS A FINAL COLUMN. 
C
C                BEFORE THE DECOMPOSITION IS COMPUTED, INITIAL COLUMNS  
C                ARE MOVED TO THE BEGINNING OF THE ARRAY X AND FINAL
C                COLUMNS TO THE END.  BOTH INITIAL AND FINAL COLUMNS
C                ARE FROZEN IN PLACE DURING THE COMPUTATION AND ONLY
C                FREE COLUMNS ARE MOVED.  AT THE K-TH STAGE OF THE  
C                REDUCTION, IF X(K) IS OCCUPIED BY A FREE COLUMN
C                IT IS INTERCHANGED WITH THE FREE COLUMN OF LARGEST 
C                REDUCED NORM.  JPVT IS NOT REFERENCED IF
C                JOB .EQ. 0.
C
C        WORK    DOUBLE-COMPLEX(P). 
C                WORK IS A WORK ARRAY.  WORK IS NOT REFERENCED IF
C                JOB .EQ. 0.
C
C        JOB     INTEGER.
C                JOB IS AN INTEGER THAT INITIATES COLUMN PIVOTING.  
C                IF JOB .EQ. 0, NO PIVOTING IS DONE.
C                IF JOB .NE. 0, PIVOTING IS DONE.
C
C     ON RETURN 
C
C        X       X CONTAINS IN ITS UPPER TRIANGLE THE UPPER 
C                TRIANGULAR MATRIX R OF THE QR FACTORIZATION.
C                BELOW ITS DIAGONAL X CONTAINS INFORMATION FROM 
C                WHICH THE UNITARY PART OF THE DECOMPOSITION
C                CAN BE RECOVERED.  NOTE THAT IF PIVOTING HAS
C                BEEN REQUESTED, THE DECOMPOSITION IS NOT THAT  
C                OF THE ORIGINAL MATRIX X BUT THAT OF X 
C                WITH ITS COLUMNS PERMUTED AS DESCRIBED BY JPVT.
C
C        QRAUX   DOUBLE-COMPLEX(P). 
C                QRAUX CONTAINS FURTHER INFORMATION REQUIRED TO RECOVER 
C                THE UNITARY PART OF THE DECOMPOSITION. 
C
C        JPVT    JPVT(K) CONTAINS THE INDEX OF THE COLUMN OF THE
C                ORIGINAL MATRIX THAT HAS BEEN INTERCHANGED INTO
C                THE K-TH COLUMN, IF PIVOTING WAS REQUESTED.
C
C     LINPACK. THIS VERSION DATED 07/03/79 .
C     G.W. STEWART, UNIVERSITY OF MARYLAND, ARGONNE NATIONAL LAB.
C
C     WQRDC USES THE FOLLOWING FUNCTIONS AND SUBPROGRAMS.
C
C     BLAS WAXPY,PYTHAG,WDOTCR,WDOTCI,WSCAL,WSWAP,WNRM2 
C     FORTRAN DABS,DIMAG,DMAX1,MIN0 
C
C     INTERNAL VARIABLES
C
      INTEGER J,JP,L,LP1,LUP,MAXJ,PL,PU 
      DOUBLE PRECISION MAXNRM,WNRM2,TT  
      DOUBLE PRECISION PYTHAG,WDOTCR,WDOTCI,NRMXLR,NRMXLI,TR,TI,FLOP
      LOGICAL NEGJ,SWAPJ
C
      DOUBLE PRECISION ZDUMR,ZDUMI  
      DOUBLE PRECISION CABS1
      CABS1(ZDUMR,ZDUMI) = DABS(ZDUMR) + DABS(ZDUMI)
C
      PL = 1
      PU = 0
      IF (JOB .EQ. 0) GO TO 60  
C
C        PIVOTING HAS BEEN REQUESTED.  REARRANGE THE COLUMNS
C        ACCORDING TO JPVT. 
C
         DO 20 J = 1, P 
            SWAPJ = JPVT(J) .GT. 0  
            NEGJ = JPVT(J) .LT. 0
            JPVT(J) = J 
            IF (NEGJ) JPVT(J) = -J  
            IF (.NOT.SWAPJ) GO TO 10
               IF (J .NE. PL)
     *            CALL WSWAP(N,XR(1,PL),XI(1,PL),1,XR(1,J),XI(1,J),1)
               JPVT(J) = JPVT(PL)
               JPVT(PL) = J 
               PL = PL + 1  
   10       CONTINUE
   20    CONTINUE
         PU = P 
         DO 50 JJ = 1, P
            J = P - JJ + 1  
            IF (JPVT(J) .GE. 0) GO TO 40
               JPVT(J) = -JPVT(J)
               IF (J .EQ. PU) GO TO 30  
                  CALL WSWAP(N,XR(1,PU),XI(1,PU),1,XR(1,J),XI(1,J),1)
                  JP = JPVT(PU) 
                  JPVT(PU) = JPVT(J)
                  JPVT(J) = JP  
   30          CONTINUE 
               PU = PU - 1  
   40       CONTINUE
   50    CONTINUE
   60 CONTINUE  
C
C     COMPUTE THE NORMS OF THE FREE COLUMNS.
C
      IF (PU .LT. PL) GO TO 80  
      DO 70 J = PL, PU  
         QRAUXR(J) = WNRM2(N,XR(1,J),XI(1,J),1) 
         QRAUXI(J) = 0.0D0  
         WORKR(J) = QRAUXR(J)
         WORKI(J) = QRAUXI(J)
   70 CONTINUE  
   80 CONTINUE  
C
C     PERFORM THE HOUSEHOLDER REDUCTION OF X.
C
      LUP = MIN0(N,P)
      DO 210 L = 1, LUP 
         IF (L .LT. PL .OR. L .GE. PU) GO TO 120
C
C           LOCATE THE COLUMN OF LARGEST NORM AND BRING IT  
C           INTO THE PIVOT POSITION.
C
            MAXNRM = 0.0D0  
            MAXJ = L
            DO 100 J = L, PU
               IF (QRAUXR(J) .LE. MAXNRM) GO TO 90  
                  MAXNRM = QRAUXR(J)
                  MAXJ = J  
   90          CONTINUE 
  100       CONTINUE
            IF (MAXJ .EQ. L) GO TO 110  
               CALL WSWAP(N,XR(1,L),XI(1,L),1,XR(1,MAXJ),XI(1,MAXJ),1)  
               QRAUXR(MAXJ) = QRAUXR(L) 
               QRAUXI(MAXJ) = QRAUXI(L) 
               WORKR(MAXJ) = WORKR(L)
               WORKI(MAXJ) = WORKI(L)
               JP = JPVT(MAXJ)  
               JPVT(MAXJ) = JPVT(L) 
               JPVT(L) = JP 
  110       CONTINUE
  120    CONTINUE
         QRAUXR(L) = 0.0D0  
         QRAUXI(L) = 0.0D0  
         IF (L .EQ. N) GO TO 200
C
C           COMPUTE THE HOUSEHOLDER TRANSFORMATION FOR COLUMN L.
C
            NRMXLR = WNRM2(N-L+1,XR(L,L),XI(L,L),1) 
            NRMXLI = 0.0D0  
            IF (CABS1(NRMXLR,NRMXLI) .EQ. 0.0D0) GO TO 190  
               IF (CABS1(XR(L,L),XI(L,L)) .EQ. 0.0D0) GO TO 130 
                 CALL WSIGN(NRMXLR,NRMXLI,XR(L,L),XI(L,L),NRMXLR,NRMXLI)
  130          CONTINUE 
               CALL WDIV(1.0D0,0.0D0,NRMXLR,NRMXLI,TR,TI)
               CALL WSCAL(N-L+1,TR,TI,XR(L,L),XI(L,L),1)
               XR(L,L) = FLOP(1.0D0 + XR(L,L))  
C
C              APPLY THE TRANSFORMATION TO THE REMAINING COLUMNS,
C              UPDATING THE NORMS.  
C
               LP1 = L + 1  
               IF (P .LT. LP1) GO TO 180
               DO 170 J = LP1, P
                  TR = -WDOTCR(N-L+1,XR(L,L),XI(L,L),1,XR(L,J), 
     *                         XI(L,J),1)
                  TI = -WDOTCI(N-L+1,XR(L,L),XI(L,L),1,XR(L,J), 
     *                         XI(L,J),1)
                  CALL WDIV(TR,TI,XR(L,L),XI(L,L),TR,TI)
                  CALL WAXPY(N-L+1,TR,TI,XR(L,L),XI(L,L),1,XR(L,J), 
     *                       XI(L,J),1) 
                  IF (J .LT. PL .OR. J .GT. PU) GO TO 160
                  IF (CABS1(QRAUXR(J),QRAUXI(J)) .EQ. 0.0D0)
     *               GO TO 160  
                     TT = 1.0D0 - (PYTHAG(XR(L,J),XI(L,J))/QRAUXR(J))**2
                     TT = DMAX1(TT,0.0D0)
                     TR = FLOP(TT)  
                     TT = FLOP(1.0D0+0.05D0*TT*(QRAUXR(J)/WORKR(J))**2) 
                     IF (TT .EQ. 1.0D0) GO TO 140
                        QRAUXR(J) = QRAUXR(J)*DSQRT(TR) 
                        QRAUXI(J) = QRAUXI(J)*DSQRT(TR) 
                     GO TO 150  
  140                CONTINUE
                        QRAUXR(J) = WNRM2(N-L,XR(L+1,J),XI(L+1,J),1)
                        QRAUXI(J) = 0.0D0
                        WORKR(J) = QRAUXR(J)
                        WORKI(J) = QRAUXI(J)
  150                CONTINUE
  160             CONTINUE  
  170          CONTINUE 
  180          CONTINUE 
C
C              SAVE THE TRANSFORMATION. 
C
               QRAUXR(L) = XR(L,L)  
               QRAUXI(L) = XI(L,L)  
               XR(L,L) = -NRMXLR
               XI(L,L) = -NRMXLI
  190       CONTINUE
  200    CONTINUE
  210 CONTINUE  
      RETURN
      END
      SUBROUTINE WQRSL(XR,XI,LDX,N,K,QRAUXR,QRAUXI,YR,YI,QYR,QYI,QTYR,  
     *                 QTYI,BR,BI,RSDR,RSDI,XBR,XBI,JOB,INFO)
      INTEGER LDX,N,K,JOB,INFO  
      DOUBLE PRECISION XR(LDX,1),XI(LDX,1),QRAUXR(1),QRAUXI(1),YR(1),
     *                 YI(1),QYR(1),QYI(1),QTYR(1),QTYI(1),BR(1),BI(1), 
     *                 RSDR(1),RSDI(1),XBR(1),XBI(1)
C
C     WQRSL APPLIES THE OUTPUT OF WQRDC TO COMPUTE COORDINATE
C     TRANSFORMATIONS, PROJECTIONS, AND LEAST SQUARES SOLUTIONS.
C     FOR K .LE. MIN(N,P), LET XK BE THE MATRIX 
C
C            XK = (X(JPVT(1)),X(JPVT(2)), ... ,X(JPVT(K)))  
C
C     FORMED FROM COLUMNNS JPVT(1), ... ,JPVT(K) OF THE ORIGINAL
C     N X P MATRIX X THAT WAS INPUT TO WQRDC (IF NO PIVOTING WAS
C     DONE, XK CONSISTS OF THE FIRST K COLUMNS OF X IN THEIR
C     ORIGINAL ORDER).  WQRDC PRODUCES A FACTORED UNITARY MATRIX Q  
C     AND AN UPPER TRIANGULAR MATRIX R SUCH THAT
C
C              XK = Q * (R) 
C                       (0) 
C
C     THIS INFORMATION IS CONTAINED IN CODED FORM IN THE ARRAYS 
C     X AND QRAUX.  
C
C     ON ENTRY  
C
C        X      DOUBLE-COMPLEX(LDX,P).  
C               X CONTAINS THE OUTPUT OF WQRDC. 
C
C        LDX    INTEGER.
C               LDX IS THE LEADING DIMENSION OF THE ARRAY X.
C
C        N      INTEGER.
C               N IS THE NUMBER OF ROWS OF THE MATRIX XK.  IT MUST  
C               HAVE THE SAME VALUE AS N IN WQRDC.  
C
C        K      INTEGER.
C               K IS THE NUMBER OF COLUMNS OF THE MATRIX XK.  K 
C               MUST NNOT BE GREATER THAN MIN(N,P), WHERE P IS THE  
C               SAME AS IN THE CALLING SEQUENCE TO WQRDC.
C
C        QRAUX  DOUBLE-COMPLEX(P).  
C               QRAUX CONTAINS THE AUXILIARY OUTPUT FROM WQRDC. 
C
C        Y      DOUBLE-COMPLEX(N)
C               Y CONTAINS AN N-VECTOR THAT IS TO BE MANIPULATED
C               BY WQRSL.
C
C        JOB    INTEGER.
C               JOB SPECIFIES WHAT IS TO BE COMPUTED.  JOB HAS  
C               THE DECIMAL EXPANSION ABCDE, WITH THE FOLLOWING 
C               MEANING.
C
C                    IF A.NE.0, COMPUTE QY. 
C                    IF B,C,D, OR E .NE. 0, COMPUTE QTY.
C                    IF C.NE.0, COMPUTE B.  
C                    IF D.NE.0, COMPUTE RSD.
C                    IF E.NE.0, COMPUTE XB. 
C
C               NOTE THAT A REQUEST TO COMPUTE B, RSD, OR XB
C               AUTOMATICALLY TRIGGERS THE COMPUTATION OF QTY, FOR  
C               WHICH AN ARRAY MUST BE PROVIDED IN THE CALLING  
C               SEQUENCE.
C
C     ON RETURN 
C
C        QY     DOUBLE-COMPLEX(N).  
C               QY CONNTAINS Q*Y, IF ITS COMPUTATION HAS BEEN
C               REQUESTED.  
C
C        QTY    DOUBLE-COMPLEX(N).  
C               QTY CONTAINS CTRANS(Q)*Y, IF ITS COMPUTATION HAS
C               BEEN REQUESTED.  HERE CTRANS(Q) IS THE CONJUGATE
C               TRANSPOSE OF THE MATRIX Q.  
C
C        B      DOUBLE-COMPLEX(K)
C               B CONTAINS THE SOLUTION OF THE LEAST SQUARES PROBLEM
C
C                    MINIMIZE NORM2(Y - XK*B),  
C
C               IF ITS COMPUTATION HAS BEEN REQUESTED.  (NOTE THAT  
C               IF PIVOTING WAS REQUESTED IN WQRDC, THE J-TH
C               COMPONENT OF B WILL BE ASSOCIATED WITH COLUMN JPVT(J)
C               OF THE ORIGINAL MATRIX X THAT WAS INPUT INTO WQRDC.)
C
C        RSD    DOUBLE-COMPLEX(N).  
C               RSD CONTAINS THE LEAST SQUARES RESIDUAL Y - XK*B,
C               IF ITS COMPUTATION HAS BEEN REQUESTED.  RSD IS  
C               ALSO THE ORTHOGONAL PROJECTION OF Y ONTO THE
C               ORTHOGONAL COMPLEMENT OF THE COLUMN SPACE OF XK.
C
C        XB     DOUBLE-COMPLEX(N).  
C               XB CONTAINS THE LEAST SQUARES APPROXIMATION XK*B,
C               IF ITS COMPUTATION HAS BEEN REQUESTED.  XB IS ALSO  
C               THE ORTHOGONAL PROJECTION OF Y ONTO THE COLUMN SPACE
C               OF X.
C
C        INFO   INTEGER.
C               INFO IS ZERO UNLESS THE COMPUTATION OF B HAS
C               BEEN REQUESTED AND R IS EXACTLY SINGULAR.  IN
C               THIS CASE, INFO IS THE INDEX OF THE FIRST ZERO  
C               DIAGONAL ELEMENT OF R AND B IS LEFT UNALTERED.  
C
C     THE PARAMETERS QY, QTY, B, RSD, AND XB ARE NOT REFERENCED 
C     IF THEIR COMPUTATION IS NOT REQUESTED AND IN THIS CASE
C     CAN BE REPLACED BY DUMMY VARIABLES IN THE CALLING PROGRAM.
C     TO SAVE STORAGE, THE USER MAY IN SOME CASES USE THE SAME  
C     ARRAY FOR DIFFERENT PARAMETERS IN THE CALLING SEQUENCE.  A
C     FREQUENTLY OCCURING EXAMPLE IS WHEN ONE WISHES TO COMPUTE 
C     ANY OF B, RSD, OR XB AND DOES NOT NEED Y OR QTY.  IN THIS 
C     CASE ONE MAY IDENTIFY Y, QTY, AND ONE OF B, RSD, OR XB, WHILE 
C     PROVIDING SEPARATE ARRAYS FOR ANYTHING ELSE THAT IS TO BE 
C     COMPUTED.  THUS THE CALLING SEQUENCE  
C
C          CALL WQRSL(X,LDX,N,K,QRAUX,Y,DUM,Y,B,Y,DUM,110,INFO) 
C
C     WILL RESULT IN THE COMPUTATION OF B AND RSD, WITH RSD 
C     OVERWRITING Y.  MORE GENERALLY, EACH ITEM IN THE FOLLOWING
C     LIST CONTAINS GROUPS OF PERMISSIBLE IDENTIFICATIONS FOR
C     A SINGLE CALLINNG SEQUENCE.
C
C          1. (Y,QTY,B) (RSD) (XB) (QY) 
C
C          2. (Y,QTY,RSD) (B) (XB) (QY) 
C
C          3. (Y,QTY,XB) (B) (RSD) (QY) 
C
C          4. (Y,QY) (QTY,B) (RSD) (XB) 
C
C          5. (Y,QY) (QTY,RSD) (B) (XB) 
C
C          6. (Y,QY) (QTY,XB) (B) (RSD) 
C
C     IN ANY GROUP THE VALUE RETURNED IN THE ARRAY ALLOCATED TO 
C     THE GROUP CORRESPONDS TO THE LAST MEMBER OF THE GROUP.
C
C     LINPACK. THIS VERSION DATED 07/03/79 .
C     G.W. STEWART, UNIVERSITY OF MARYLAND, ARGONNE NATIONAL LAB.
C
C     WQRSL USES THE FOLLOWING FUNCTIONS AND SUBPROGRAMS.
C
C     BLAS WAXPY,WCOPY,WDOTCR,WDOTCI
C     FORTRAN DABS,DIMAG,MIN0,MOD
C
C     INTERNAL VARIABLES
C
      INTEGER I,J,JJ,JU,KP1 
      DOUBLE PRECISION WDOTCR,WDOTCI,TR,TI,TEMPR,TEMPI  
      LOGICAL CB,CQY,CQTY,CR,CXB
C
      DOUBLE PRECISION ZDUMR,ZDUMI  
      DOUBLE PRECISION CABS1
      CABS1(ZDUMR,ZDUMI) = DABS(ZDUMR) + DABS(ZDUMI)
C
C     SET INFO FLAG.
C
      INFO = 0  
C
C     DETERMINE WHAT IS TO BE COMPUTED. 
C
      CQY = JOB/10000 .NE. 0
      CQTY = MOD(JOB,10000) .NE. 0  
      CB = MOD(JOB,1000)/100 .NE. 0 
      CR = MOD(JOB,100)/10 .NE. 0
      CXB = MOD(JOB,10) .NE. 0  
      JU = MIN0(K,N-1)  
C
C     SPECIAL ACTION WHEN N=1.  
C
      IF (JU .NE. 0) GO TO 80
         IF (.NOT.CQY) GO TO 10 
            QYR(1) = YR(1)  
            QYI(1) = YI(1)  
   10    CONTINUE
         IF (.NOT.CQTY) GO TO 20
            QTYR(1) = YR(1) 
            QTYI(1) = YI(1) 
   20    CONTINUE
         IF (.NOT.CXB) GO TO 30 
            XBR(1) = YR(1)  
            XBI(1) = YI(1)  
   30    CONTINUE
         IF (.NOT.CB) GO TO 60  
            IF (CABS1(XR(1,1),XI(1,1)) .NE. 0.0D0) GO TO 40 
               INFO = 1 
            GO TO 50
   40       CONTINUE
               CALL WDIV(YR(1),YI(1),XR(1,1),XI(1,1),BR(1),BI(1))
   50       CONTINUE
   60    CONTINUE
         IF (.NOT.CR) GO TO 70  
            RSDR(1) = 0.0D0 
            RSDI(1) = 0.0D0 
   70    CONTINUE
      GO TO 290 
   80 CONTINUE  
C
C        SET UP TO COMPUTE QY OR QTY.
C
         IF (CQY) CALL WCOPY(N,YR,YI,1,QYR,QYI,1)
         IF (CQTY) CALL WCOPY(N,YR,YI,1,QTYR,QTYI,1)
         IF (.NOT.CQY) GO TO 110
C
C           COMPUTE QY. 
C
            DO 100 JJ = 1, JU
               J = JU - JJ + 1  
               IF (CABS1(QRAUXR(J),QRAUXI(J)) .EQ. 0.0D0)
     *            GO TO 90  
                  TEMPR = XR(J,J)
                  TEMPI = XI(J,J)
                  XR(J,J) = QRAUXR(J)
                  XI(J,J) = QRAUXI(J)
                  TR = -WDOTCR(N-J+1,XR(J,J),XI(J,J),1,QYR(J),QYI(J),1) 
                  TI = -WDOTCI(N-J+1,XR(J,J),XI(J,J),1,QYR(J),QYI(J),1) 
                  CALL WDIV(TR,TI,XR(J,J),XI(J,J),TR,TI)
                  CALL WAXPY(N-J+1,TR,TI,XR(J,J),XI(J,J),1,QYR(J),  
     *                       QYI(J),1)  
                  XR(J,J) = TEMPR
                  XI(J,J) = TEMPI
   90          CONTINUE 
  100       CONTINUE
  110    CONTINUE
         IF (.NOT.CQTY) GO TO 140
C
C           COMPUTE CTRANS(Q)*Y.
C
            DO 130 J = 1, JU
               IF (CABS1(QRAUXR(J),QRAUXI(J)) .EQ. 0.0D0)
     *            GO TO 120 
                  TEMPR = XR(J,J)
                  TEMPI = XI(J,J)
                  XR(J,J) = QRAUXR(J)
                  XI(J,J) = QRAUXI(J)
                  TR = -WDOTCR(N-J+1,XR(J,J),XI(J,J),1,QTYR(J), 
     *                         QTYI(J),1)
                  TI = -WDOTCI(N-J+1,XR(J,J),XI(J,J),1,QTYR(J), 
     *                         QTYI(J),1)
                  CALL WDIV(TR,TI,XR(J,J),XI(J,J),TR,TI)
                  CALL WAXPY(N-J+1,TR,TI,XR(J,J),XI(J,J),1,QTYR(J), 
     *                       QTYI(J),1) 
                  XR(J,J) = TEMPR
                  XI(J,J) = TEMPI
  120          CONTINUE 
  130       CONTINUE
  140    CONTINUE
C
C        SET UP TO COMPUTE B, RSD, OR XB.
C
         IF (CB) CALL WCOPY(K,QTYR,QTYI,1,BR,BI,1)  
         KP1 = K + 1
         IF (CXB) CALL WCOPY(K,QTYR,QTYI,1,XBR,XBI,1)
         IF (CR .AND. K .LT. N) 
     *      CALL WCOPY(N-K,QTYR(KP1),QTYI(KP1),1,RSDR(KP1),RSDI(KP1),1) 
         IF (.NOT.CXB .OR. KP1 .GT. N) GO TO 160
            DO 150 I = KP1, N
               XBR(I) = 0.0D0
               XBI(I) = 0.0D0
  150       CONTINUE
  160    CONTINUE
         IF (.NOT.CR) GO TO 180 
            DO 170 I = 1, K 
               RSDR(I) = 0.0D0  
               RSDI(I) = 0.0D0  
  170       CONTINUE
  180    CONTINUE
         IF (.NOT.CB) GO TO 230 
C
C           COMPUTE B.  
C
            DO 210 JJ = 1, K
               J = K - JJ + 1
               IF (CABS1(XR(J,J),XI(J,J)) .NE. 0.0D0) GO TO 190 
                  INFO = J  
C                 ......EXIT
C           ......EXIT  
                  GO TO 220 
  190          CONTINUE 
               CALL WDIV(BR(J),BI(J),XR(J,J),XI(J,J),BR(J),BI(J))
               IF (J .EQ. 1) GO TO 200  
                  TR = -BR(J)
                  TI = -BI(J)
                  CALL WAXPY(J-1,TR,TI,XR(1,J),XI(1,J),1,BR,BI,1)
  200          CONTINUE 
  210       CONTINUE
  220       CONTINUE
  230    CONTINUE
         IF (.NOT.CR .AND. .NOT.CXB) GO TO 280  
C
C           COMPUTE RSD OR XB AS REQUIRED.  
C
            DO 270 JJ = 1, JU
               J = JU - JJ + 1  
               IF (CABS1(QRAUXR(J),QRAUXI(J)) .EQ. 0.0D0)
     *            GO TO 260 
                  TEMPR = XR(J,J)
                  TEMPI = XI(J,J)
                  XR(J,J) = QRAUXR(J)
                  XI(J,J) = QRAUXI(J)
                  IF (.NOT.CR) GO TO 240
                     TR = -WDOTCR(N-J+1,XR(J,J),XI(J,J),1,RSDR(J),  
     *                            RSDI(J),1)
                     TI = -WDOTCI(N-J+1,XR(J,J),XI(J,J),1,RSDR(J),  
     *                            RSDI(J),1)
                     CALL WDIV(TR,TI,XR(J,J),XI(J,J),TR,TI) 
                     CALL WAXPY(N-J+1,TR,TI,XR(J,J),XI(J,J),1,RSDR(J),  
     *                          RSDI(J),1)  
  240             CONTINUE  
                  IF (.NOT.CXB) GO TO 250
                     TR = -WDOTCR(N-J+1,XR(J,J),XI(J,J),1,XBR(J),
     *                            XBI(J),1) 
                     TI = -WDOTCI(N-J+1,XR(J,J),XI(J,J),1,XBR(J),
     *                            XBI(J),1) 
                     CALL WDIV(TR,TI,XR(J,J),XI(J,J),TR,TI) 
                     CALL WAXPY(N-J+1,TR,TI,XR(J,J),XI(J,J),1,XBR(J),
     *                          XBI(J),1)
  250             CONTINUE  
                  XR(J,J) = TEMPR
                  XI(J,J) = TEMPI
  260          CONTINUE 
  270       CONTINUE
  280    CONTINUE
  290 CONTINUE  
      RETURN
      END
      SUBROUTINE MAGIC(A,LDA,N) 
C
C     ALGORITHMS FOR MAGIC SQUARES TAKEN FROM
C        MATHEMATICAL RECREATIONS AND ESSAYS, 12TH ED., 
C        BY W. W. ROUSE BALL AND H. S. M. COXETER
C
      DOUBLE PRECISION A(LDA,N),T
C
      IF (MOD(N,4) .EQ. 0) GO TO 100
      IF (MOD(N,2) .EQ. 0) M = N/2  
      IF (MOD(N,2) .NE. 0) M = N
C
C     ODD ORDER OR UPPER CORNER OF EVEN ORDER
C
      DO 20 J = 1,M 
         DO 10 I = 1,M  
            A(I,J) = 0  
   10    CONTINUE
   20 CONTINUE  
      I = 1 
      J = (M+1)/2
      MM = M*M  
      DO 40 K = 1, MM
         A(I,J) = K 
         I1 = I-1
         J1 = J+1
         IF(I1.LT.1) I1 = M 
         IF(J1.GT.M) J1 = 1 
         IF(IDINT(A(I1,J1)).EQ.0) GO TO 30  
            I1 = I+1
            J1 = J  
   30    I = I1 
         J = J1 
   40 CONTINUE  
      IF (MOD(N,2) .NE. 0) RETURN
C
C     REST OF EVEN ORDER
C
      T = M*M
      DO 60 I = 1, M
         DO 50 J = 1, M 
            IM = I+M
            JM = J+M
            A(I,JM) = A(I,J) + 2*T  
            A(IM,J) = A(I,J) + 3*T  
            A(IM,JM) = A(I,J) + T
   50    CONTINUE
   60 CONTINUE  
      M1 = (M-1)/2  
      IF (M1.EQ.0) RETURN
      DO 70 J = 1, M1
         CALL RSWAP(M,A(1,J),1,A(M+1,J),1)  
   70 CONTINUE  
      M1 = (M+1)/2  
      M2 = M1 + M
      CALL RSWAP(1,A(M1,1),1,A(M2,1),1) 
      CALL RSWAP(1,A(M1,M1),1,A(M2,M1),1)
      M1 = N+1-(M-3)/2  
      IF(M1.GT.N) RETURN
      DO 80 J = M1, N
         CALL RSWAP(M,A(1,J),1,A(M+1,J),1)  
   80 CONTINUE  
      RETURN
C
C     DOUBLE EVEN ORDER 
C
  100 K = 1 
      DO 120 I = 1, N
         DO 110 J = 1, N
            A(I,J) = K  
            IF (MOD(I,4)/2 .EQ. MOD(J,4)/2) A(I,J) = N*N+1 - K  
            K = K+1 
  110    CONTINUE
  120 CONTINUE  
      RETURN
      END
      SUBROUTINE BASE(X,B,EPS,S,N)  
      DOUBLE PRECISION X,B,EPS,S(1),T
C
C     STORE BASE B REPRESENTATION OF X IN S(1:N)
C
      INTEGER PLUS,MINUS,DOT,ZERO,COMMA 
      DATA PLUS/41/,MINUS/42/,DOT/47/,ZERO/0/,COMMA/48/ 
      L = 1 
      IF (X .GE. 0.0D0) S(L) = PLUS 
      IF (X .LT. 0.0D0) S(L) = MINUS
      S(L+1) = ZERO 
      S(L+2) = DOT  
      X = DABS(X)
      IF (X .NE. 0.0D0) K = DLOG(X)/DLOG(B) 
      IF (X .EQ. 0.0D0) K = 0
      IF (X .GT. 1.0D0) K = K + 1
      X = X/B**K
      IF (B*X .GE. B) K = K + 1 
      IF (B*X .GE. B) X = X/B
      IF (EPS .NE. 0.0D0) M = -DLOG(EPS)/DLOG(B) + 4
      IF (EPS .EQ. 0.0D0) M = 54
      DO 10 L = 4, M
      X = B*X
      J = IDINT(X)  
      S(L) = DFLOAT(J)  
      X = X - S(L)  
   10 CONTINUE  
      S(M+1) = COMMA
      IF (K .GE. 0) S(M+2) = PLUS
      IF (K .LT. 0) S(M+2) = MINUS  
      T = DABS(DFLOAT(K))
      N = M + 3 
      IF (T .GE. B) N = N + IDINT(DLOG(T)/DLOG(B))  
      L = N 
   20 J = IDINT(DMOD(T,B))  
      S(L) = DFLOAT(J)  
      L = L - 1 
      T = T/B
      IF (L .GE. M+3) GO TO 20  
      RETURN
      END
      DOUBLE PRECISION FUNCTION URAND(IY)
      INTEGER IY
C
C      URAND IS A UNIFORM RANDOM NUMBER GENERATOR BASED  ON  THEORY  AND
C  SUGGESTIONS  GIVEN  IN  D.E. KNUTH (1969),  VOL  2.   THE INTEGER  IY
C  SHOULD BE INITIALIZED TO AN ARBITRARY INTEGER PRIOR TO THE FIRST CALL
C  TO URAND.  THE CALLING PROGRAM SHOULD  NOT  ALTER  THE  VALUE  OF  IY
C  BETWEEN  SUBSEQUENT CALLS TO URAND.  VALUES OF URAND WILL BE RETURNED
C  IN THE INTERVAL (0,1).
C
      INTEGER IA,IC,ITWO,M2,M,MIC
      DOUBLE PRECISION HALFM,S  
      DOUBLE PRECISION DATAN,DSQRT  
      DATA M2/0/,ITWO/2/
      IF (M2 .NE. 0) GO TO 20
C
C  IF FIRST ENTRY, COMPUTE MACHINE INTEGER WORD LENGTH  
C
      M = 1 
   10 M2 = M
      M = ITWO*M2
      IF (M .GT. M2) GO TO 10
      HALFM = M2
C
C  COMPUTE MULTIPLIER AND INCREMENT FOR LINEAR CONGRUENTIAL METHOD  
C
      IA = 8*IDINT(HALFM*DATAN(1.D0)/8.D0) + 5  
      IC = 2*IDINT(HALFM*(0.5D0-DSQRT(3.D0)/6.D0)) + 1  
      MIC = (M2 - IC) + M2  
C
C  S IS THE SCALE FACTOR FOR CONVERTING TO FLOATING POINT
C
      S = 0.5D0/HALFM
C
C  COMPUTE NEXT RANDOM NUMBER
C
   20 IY = IY*IA
C
C  THE FOLLOWING STATEMENT IS FOR COMPUTERS WHICH DO NOT ALLOW  
C  INTEGER OVERFLOW ON ADDITION 
C
      IF (IY .GT. MIC) IY = (IY - M2) - M2  
C
      IY = IY + IC  
C
C  THE FOLLOWING STATEMENT IS FOR COMPUTERS WHERE THE
C  WORD LENGTH FOR ADDITION IS GREATER THAN FOR MULTIPLICATION  
C
      IF (IY/2 .GT. M2) IY = (IY - M2) - M2 
C
C  THE FOLLOWING STATEMENT IS FOR COMPUTERS WHERE INTEGER
C  OVERFLOW AFFECTS THE SIGN BIT
C
      IF (IY .LT. 0) IY = (IY + M2) + M2
      URAND = DFLOAT(IY)*S  
      RETURN
      END
      SUBROUTINE WMUL(AR,AI,BR,BI,CR,CI)
      DOUBLE PRECISION AR,AI,BR,BI,CR,CI,T,FLOP 
C     C = A*B
      T = AR*BI + AI*BR 
      IF (T .NE. 0.0D0) T = FLOP(T) 
      CR = FLOP(AR*BR - AI*BI)  
      CI = T
      RETURN
      END
      SUBROUTINE WDIV(AR,AI,BR,BI,CR,CI)
      DOUBLE PRECISION AR,AI,BR,BI,CR,CI
C     C = A/B
      DOUBLE PRECISION S,D,ARS,AIS,BRS,BIS,FLOP 
      S = DABS(BR) + DABS(BI)
      IF (S .EQ. 0.0D0) CALL ERROR(27)  
      IF (S .EQ. 0.0D0) RETURN  
      ARS = AR/S
      AIS = AI/S
      BRS = BR/S
      BIS = BI/S
      D = BRS**2 + BIS**2
      CR = FLOP((ARS*BRS + AIS*BIS)/D)  
      CI = (AIS*BRS - ARS*BIS)/D
      IF (CI .NE. 0.0D0) CI = FLOP(CI)  
      RETURN
      END
      SUBROUTINE WSIGN(XR,XI,YR,YI,ZR,ZI)
      DOUBLE PRECISION XR,XI,YR,YI,ZR,ZI,PYTHAG,T
C     IF Y .NE. 0, Z = X*Y/ABS(Y)
C     IF Y .EQ. 0, Z = X
      T = PYTHAG(YR,YI) 
      ZR = XR
      ZI = XI
      IF (T .NE. 0.0D0) CALL WMUL(YR/T,YI/T,ZR,ZI,ZR,ZI)
      RETURN
      END
      SUBROUTINE WSQRT(XR,XI,YR,YI) 
      DOUBLE PRECISION XR,XI,YR,YI,S,TR,TI,PYTHAG,FLOP  
C     Y = SQRT(X) WITH YR .GE. 0.0 AND SIGN(YI) .EQ. SIGN(XI)
C
      TR = XR
      TI = XI
      S = DSQRT(0.5D0*(PYTHAG(TR,TI) + DABS(TR)))
      IF (TR .GE. 0.0D0) YR = FLOP(S)
      IF (TI .LT. 0.0D0) S = -S 
      IF (TR .LE. 0.0D0) YI = FLOP(S)
      IF (TR .LT. 0.0D0) YR = FLOP(0.5D0*(TI/YI))
      IF (TR .GT. 0.0D0) YI = FLOP(0.5D0*(TI/YR))
      RETURN
      END
      SUBROUTINE WLOG(XR,XI,YR,YI)  
      DOUBLE PRECISION XR,XI,YR,YI,T,R,PYTHAG
C     Y = LOG(X)
      R = PYTHAG(XR,XI) 
      IF (R .EQ. 0.0D0) CALL ERROR(32)  
      IF (R .EQ. 0.0D0) RETURN  
      T = DATAN2(XI,XR) 
      IF (XI.EQ.0.0D0 .AND. XR.LT.0.0D0) T = DABS(T)
      YR = DLOG(R)  
      YI = T
      RETURN
      END
      SUBROUTINE WATAN(XR,XI,YR,YI) 
C     Y = ATAN(X) = (I/2)*LOG((I+X)/(I-X))  
      DOUBLE PRECISION XR,XI,YR,YI,TR,TI
      IF (XI .NE. 0.0D0) GO TO 10
         YR = DATAN2(XR,1.0D0)  
         YI = 0.0D0 
         RETURN 
   10 IF (XR.NE.0.0D0 .OR. DABS(XI).NE.1.0D0) GO TO 20  
         CALL ERROR(32) 
         RETURN 
   20 CALL WDIV(XR,1.0D0+XI,-XR,1.0D0-XI,TR,TI) 
      CALL WLOG(TR,TI,TR,TI)
      YR = -TI/2.0D0
      YI = TR/2.0D0 
      RETURN
      END
      DOUBLE PRECISION FUNCTION WNRM2(N,XR,XI,INCX) 
      DOUBLE PRECISION XR(1),XI(1),PYTHAG,S 
C     NORM2(X)  
      S = 0.0D0 
      IF (N .LE. 0) GO TO 20
      IX = 1
      DO 10 I = 1, N
         S = PYTHAG(S,XR(IX))
         S = PYTHAG(S,XI(IX))
         IX = IX + INCX 
   10 CONTINUE  
   20 WNRM2 = S 
      RETURN
      END
      DOUBLE PRECISION FUNCTION WASUM(N,XR,XI,INCX) 
      DOUBLE PRECISION XR(1),XI(1),S,FLOP
C     NORM1(X)  
      S = 0.0D0 
      IF (N .LE. 0) GO TO 20
      IX = 1
      DO 10 I = 1, N
         S = FLOP(S + DABS(XR(IX)) + DABS(XI(IX)))  
         IX = IX + INCX 
   10 CONTINUE  
   20 WASUM = S 
      RETURN
      END
      INTEGER FUNCTION IWAMAX(N,XR,XI,INCX) 
      DOUBLE PRECISION XR(1),XI(1),S,P  
C     INDEX OF NORMINF(X)
      K = 0 
      IF (N .LE. 0) GO TO 20
      K = 1 
      S = 0.0D0 
      IX = 1
      DO 10 I = 1, N
         P = DABS(XR(IX)) + DABS(XI(IX))
         IF (P .GT. S) K = I
         IF (P .GT. S) S = P
         IX = IX + INCX 
   10 CONTINUE  
   20 IWAMAX = K
      RETURN
      END
      SUBROUTINE WRSCAL(N,S,XR,XI,INCX) 
      DOUBLE PRECISION S,XR(1),XI(1),FLOP
      IF (N .LE. 0) RETURN  
      IX = 1
      DO 10 I = 1, N
         XR(IX) = FLOP(S*XR(IX))
         IF (XI(IX) .NE. 0.0D0) XI(IX) = FLOP(S*XI(IX)) 
         IX = IX + INCX 
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE WSCAL(N,SR,SI,XR,XI,INCX)  
      DOUBLE PRECISION SR,SI,XR(1),XI(1)
      IF (N .LE. 0) RETURN  
      IX = 1
      DO 10 I = 1, N
         CALL WMUL(SR,SI,XR(IX),XI(IX),XR(IX),XI(IX))
         IX = IX + INCX 
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE WAXPY(N,SR,SI,XR,XI,INCX,YR,YI,INCY)
      DOUBLE PRECISION SR,SI,XR(1),XI(1),YR(1),YI(1),FLOP
      IF (N .LE. 0) RETURN  
      IF (SR .EQ. 0.0D0 .AND. SI .EQ. 0.0D0) RETURN 
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1, N
         YR(IY) = FLOP(YR(IY) + SR*XR(IX) - SI*XI(IX))  
         YI(IY) = YI(IY) + SR*XI(IX) + SI*XR(IX)
         IF (YI(IY) .NE. 0.0D0) YI(IY) = FLOP(YI(IY))
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
      RETURN
      END
      DOUBLE PRECISION FUNCTION WDOTUR(N,XR,XI,INCX,YR,YI,INCY) 
      DOUBLE PRECISION XR(1),XI(1),YR(1),YI(1),S,FLOP
      S = 0.0D0 
      IF (N .LE. 0) GO TO 20
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1, N
         S = FLOP(S + XR(IX)*YR(IY) - XI(IX)*YI(IY))
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
   20 WDOTUR = S
      RETURN
      END
      DOUBLE PRECISION FUNCTION WDOTUI(N,XR,XI,INCX,YR,YI,INCY) 
      DOUBLE PRECISION XR(1),XI(1),YR(1),YI(1),S,FLOP
      S = 0.0D0 
      IF (N .LE. 0) GO TO 20
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1, N
         S = S + XR(IX)*YI(IY) + XI(IX)*YR(IY)  
         IF (S .NE. 0.0D0) S = FLOP(S)  
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
   20 WDOTUI = S
      RETURN
      END
      DOUBLE PRECISION FUNCTION WDOTCR(N,XR,XI,INCX,YR,YI,INCY) 
      DOUBLE PRECISION XR(1),XI(1),YR(1),YI(1),S,FLOP
      S = 0.0D0 
      IF (N .LE. 0) GO TO 20
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1, N
         S = FLOP(S + XR(IX)*YR(IY) + XI(IX)*YI(IY))
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
   20 WDOTCR = S
      RETURN
      END
      DOUBLE PRECISION FUNCTION WDOTCI(N,XR,XI,INCX,YR,YI,INCY) 
      DOUBLE PRECISION XR(1),XI(1),YR(1),YI(1),S,FLOP
      S = 0.0D0 
      IF (N .LE. 0) GO TO 20
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1, N
         S = S + XR(IX)*YI(IY) - XI(IX)*YR(IY)  
         IF (S .NE. 0.0D0) S = FLOP(S)  
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
   20 WDOTCI = S
      RETURN
      END
      SUBROUTINE WCOPY(N,XR,XI,INCX,YR,YI,INCY) 
      DOUBLE PRECISION XR(1),XI(1),YR(1),YI(1)  
      IF (N .LE. 0) RETURN  
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1, N
         YR(IY) = XR(IX)
         YI(IY) = XI(IX)
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE WSET(N,XR,XI,YR,YI,INCY)
      INTEGER N,INCY
      DOUBLE PRECISION XR,XI,YR(1),YI(1)
      IY = 1
      IF (N .LE. 0 ) RETURN 
      DO 10 I = 1,N 
         YR(IY) = XR
         YI(IY) = XI
         IY = IY + INCY 
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE WSWAP(N,XR,XI,INCX,YR,YI,INCY) 
      DOUBLE PRECISION XR(1),XI(1),YR(1),YI(1),T
      IF (N .LE. 0) RETURN  
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1, N
         T = XR(IX) 
         XR(IX) = YR(IY)
         YR(IY) = T 
         T = XI(IX) 
         XI(IX) = YI(IY)
         YI(IY) = T 
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE RSET(N,DX,DY,INCY) 
C
C     COPIES A SCALAR, X, TO A SCALAR, Y.
      DOUBLE PRECISION DX,DY(1) 
C
      IF (N.LE.0) RETURN
      IY = 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1,N 
        DY(IY) = DX 
        IY = IY + INCY  
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE RSWAP(N,X,INCX,Y,INCY) 
      DOUBLE PRECISION X(1),Y(1),T  
      IF (N .LE. 0) RETURN  
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX+1 
      IF (INCY.LT.0) IY = (-N+1)*INCY+1 
      DO 10 I = 1, N
         T = X(IX)  
         X(IX) = Y(IY)  
         Y(IY) = T  
         IX = IX + INCX 
         IY = IY + INCY 
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE RROT(N,DX,INCX,DY,INCY,C,S)
C
C     APPLIES A PLANE ROTATION. 
      DOUBLE PRECISION DX(1),DY(1),DTEMP,C,S,FLOP
      INTEGER I,INCX,INCY,IX,IY,N
C
      IF (N.LE.0) RETURN
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 10 I = 1,N 
        DTEMP = FLOP(C*DX(IX) + S*DY(IY))
        DY(IY) = FLOP(C*DY(IY) - S*DX(IX))  
        DX(IX) = DTEMP  
        IX = IX + INCX  
        IY = IY + INCY  
   10 CONTINUE  
      RETURN
      END
      SUBROUTINE RROTG(DA,DB,C,S)
C
C     CONSTRUCT GIVENS PLANE ROTATION.  
C
      DOUBLE PRECISION DA,DB,C,S,RHO,PYTHAG,FLOP,R,Z
C
      RHO = DB  
      IF ( DABS(DA) .GT. DABS(DB) ) RHO = DA
      C = 1.0D0 
      S = 0.0D0 
      Z = 1.0D0 
      R = FLOP(DSIGN(PYTHAG(DA,DB),RHO))
      IF (R .NE. 0.0D0) C = FLOP(DA/R)  
      IF (R .NE. 0.0D0) S = FLOP(DB/R)  
      IF ( DABS(DA) .GT. DABS(DB) ) Z = S
      IF ( DABS(DB) .GE. DABS(DA) .AND. C .NE. 0.0D0 ) Z = FLOP(1.0D0/C)
      DA = R
      DB = Z
      RETURN
      END
      LOGICAL FUNCTION EQID(X,Y)
C     CHECK FOR EQUALITY OF TWO NAMES
      INTEGER X(4),Y(4) 
      EQID = .TRUE. 
      DO 10 I = 1, 4
   10 EQID = EQID .AND. (X(I).EQ.Y(I))  
      RETURN
      END
      SUBROUTINE PUTID(X,Y) 
C     STORE A NAME  
      INTEGER X(4),Y(4) 
      DO 10 I = 1, 4
   10 X(I) = Y(I)
      RETURN
      END
      DOUBLE PRECISION FUNCTION ROUND(X)
      DOUBLE PRECISION X,Y,Z,E,H
      DATA H/1.0D9/ 
      Z = DABS(X)
      Y = Z + 1.0D0 
      IF (Y .EQ. Z) GO TO 40
      Y = 0.0D0 
      E = H 
   10 IF (E .GE. Z) GO TO 20
         E = 2.0D0*E
         GO TO 10
   20 IF (E .LE. H) GO TO 30
         IF (E .LE. Z) Y = Y + E
         IF (E .LE. Z) Z = Z - E
         E = E/2.0D0
         GO TO 20
   30 Z = IDINT(Z + 0.5D0)  
      Y = Y + Z 
      IF (X .LT. 0.0D0) Y = -Y  
      ROUND = Y 
      RETURN
   40 ROUND = X 
      RETURN
      END
