      SUBROUTINE SYSDAT(A,B,C,P,Q,G,K,D,W,V,NX,NU,NV,NY,NA,
     2                  IFLAG,NAME,IERR)
C-----------------------------------------------------------------C
C***BEGIN PROLOGUE SYSDAT.                                        C
C                                                                 C
C   PURPOSE                                                       C
C   SYSDAT is used to store or read the system matrixes from      C
C   data file. The matrixes are stored in asci format.            C
C                                                                 C
C   ON INPUT                                                      C
C   IFLAG = 1  Read matrixes from file.                           C
C   IFLAG = 2  Write matrixes to file.                            C  
C   NAME       Character name on file.                            C 
C                                                                 C
C   ON OUTPUT                                                     C
C   IERR  = 0  Succesfully operations                             C
C   IERR  = 1  Error in opening file                              C
C   IERR  = 4  Error in reading matrix A                          C
C   IERR  = 5  Error in reading matrix B                          C
C   IERR  = 6  Error in reading matrix C                          C
C   IERR  = 7  Error in reading matrix P                          C
C   IERR  = 8  Error in reading matrix Q                          C
C   IERR  = 9  Error in reading matrix G                          C
C   IERR  =10  Error in reading matrix K                          C
C   IERR  =11  Error in reading matrix D                          C
C   IERR  =12  Error in reading matrix W                          C
C   IERR  =13  Error in reading matrix V                          C
C                                                                 C
C   ROUTINES CALLED     : (NONE)                                  C
C***END PROLOGUE SYSDAT                                           C 
C-----------------------------------------------------------------C
      INTEGER I,J,NIN,NX,NU,NV,NY,IFLAG,IERR
      DOUBLE PRECISION A(NA,NA),B(NA,NA),C(NA,NA),P(NA,NA),Q(NA,NA)
      DOUBLE PRECISION G(NA,NA),K(NA,NA),D(NA,NA),W(NA,NA),V(NA,NA)
      CHARACTER NAME*(*),CFORM*6

      IERR  = 0
      CFORM = '(3X,A)'
      NIN   = 10

C1... Open file NAME                                     .........
      IF (IFLAG.EQ.1) THEN
       OPEN(NIN,FILE=NAME,STATUS='OLD',ERR=98)
      ELSEIF (IFLAG.EQ.2) THEN
       OPEN(NIN,FILE=NAME,STATUS='UNKNOWN',ERR=99)
      ENDIF

C2... Write to data file                                 .........
      IF (IFLAG.EQ.2) THEN
       WRITE (NIN,99979) NX, NU, NV, NY
       WRITE (NIN,99980) ((A(I,J),J=1,NX),I=1,NX)
       WRITE (NIN,99981) ((B(I,J),J=1,NU),I=1,NX)
       WRITE (NIN,99982) ((C(I,J),J=1,NV),I=1,NX)
       WRITE (NIN,99983) ((P(I,J),J=1,NU),I=1,NU)
       WRITE (NIN,99984) ((Q(I,J),J=1,NX),I=1,NX)
       WRITE (NIN,99985) ((G(I,J),J=1,NX),I=1,NU)
       WRITE (NIN,99986) ((K(I,J),J=1,NY),I=1,NX)
       WRITE (NIN,99987) ((D(I,J),J=1,NX),I=1,NY)
       WRITE (NIN,99988) ((W(I,J),J=1,NY),I=1,NY)
       WRITE (NIN,99989) ((V(I,J),J=1,NV),I=1,NV)

C3... Read from data file                               .........
      ELSEIF (IFLAG.EQ.1) THEN
       READ (NIN,99979,ERR=100) NX, NU, NV, NY
       READ (NIN,99980,ERR=101) ((A(I,J),J=1,NX),I=1,NX)
       READ (NIN,99981,ERR=102) ((B(I,J),J=1,NU),I=1,NX)
       READ (NIN,99982,ERR=103) ((C(I,J),J=1,NV),I=1,NX)
       READ (NIN,99983,ERR=104) ((P(I,J),J=1,NU),I=1,NU)
       READ (NIN,99984,ERR=105) ((Q(I,J),J=1,NX),I=1,NX)
       READ (NIN,99985,ERR=106) ((G(I,J),J=1,NX),I=1,NU)
       READ (NIN,99986,ERR=107) ((K(I,J),J=1,NY),I=1,NX)
       READ (NIN,99987,ERR=108) ((D(I,J),J=1,NX),I=1,NY)
       READ (NIN,99988,ERR=109) ((W(I,J),J=1,NY),I=1,NY)
       READ (NIN,99989,ERR=110) ((V(I,J),J=1,NV),I=1,NV)
      ENDIF
      CLOSE (NIN)
99979 FORMAT(4I4   )
99980 FORMAT(4E16.9)
99981 FORMAT(2E16.9)
99982 FORMAT(2E16.9)
99983 FORMAT(2E16.9)
99984 FORMAT(4E16.9)
99985 FORMAT(4E16.9)
99986 FORMAT(4E16.9)
99987 FORMAT(4E16.9)
99988 FORMAT(4E16.9)
99989 FORMAT(4E16.9)
      GO TO 1000

98    WRITE(*,CFORM)'* ---> File does not exist'
      IERR = 1
      GOTO 1000
99    WRITE(*,CFORM)'* ---> Error in opening file '
      IERR = 2
      GOTO 1000
100   WRITE(*,CFORM)'* ---> Error in reading dimensions NX, NU, NV'
      IERR = 3
      GO TO 1000
101   WRITE(*,CFORM)'* ---> Error in reading A '
      IERR = 4
      GOTO 1000
102   WRITE(*,CFORM)'* ---> Error in reading B '
      IERR = 5
      GOTO 1000
103   WRITE(*,CFORM)'* ---> Error in reading C '
      IERR = 6
      GOTO 1000
104   WRITE(*,CFORM)'* ---> Error in reading P '
      IERR = 7
      GOTO 1000
105   WRITE(*,CFORM)'* ---> Error in reading Q '
      IERR = 8
      GOTO 1000
106   WRITE(*,CFORM)'* ---> Error in reading G '
      IERR = 9
      GO TO 1000
107   WRITE(*,CFORM)'* ---> Error in reading K '
      IERR = 10
      GO TO 1000
108   WRITE(*,CFORM)'* ---> Error in reading D '
      IERR = 11
      GO TO 1000
109   WRITE(*,CFORM)'* ---> Error in reading W '
      IERR = 12
      GO TO 1000
110   WRITE(*,CFORM)'* ---> Error in reading V '
      IERR = 13

C... Last line of SYSDAT                .........................
1000  RETURN
      END

