
c/********************************************************************/
c/*                                                                  */
c/*  This function estimates the optimal perturbation intervals      */
c/*  to be used in any optimization algorithm based on approximation */
c/*  of derivatives by finite differences.                           */
c/*                                                                  */
c/********************************************************************/
      subroutine estint( fvalue,x,fx,epsa,xdim,xperturb,
     +                     grad,hessdiag,hfail )
      double precision x(xdim),xperturb(xdim),grad(xdim),
     +                   hessdiag(xdim),fx,epsa

      integer xdim,hfail(1) 

      double precision nyx(100),errgrad(100),h(20)
      double precision omega,etta,root,hs,fxplusshk,fxminhk,
     +                   fiFhk,fiBhk,fiChk,FIhk,ChatfiF,
     +                   ChatfiB,ChatFI,hFI,hF,fxplusshFI,
     +                   fxminhFI,fiChFI,EF,Estrek,fxplusshF,
     +                   fxminhF,fi,FIVAL 
      integer   KAA,i,k 
      external fvalue

        omega = 1.0 
        etta  = 1.0 
        KAA   = 7 
        do 2 i=1,xdim
 2         nyx(i) = x(i)

        root = dsqrt( epsa/(omega+dabs(fx)) ) 

        do 100 i=1,xdim
            k = 1 
            hs = -1.0 
            h(k) = 20.0*( etta+dabs(nyx(i)) )*root 

 1000       nyx(i) = x(i) + h(k) 
            call fvalue( fxplusshk,nyx,xdim )
            nyx(i) = x(i) - h(k) 
            call fvalue( fxminhk,nyx,xdim )

            fiFhk = (fxplusshk-fx)/h(k) 
            fiBhk = (fx-fxminhk)/h(k) 
            fiChk = (fxplusshk-fxminhk)/(2.0*h(k)) 
            FIhk  = (fxplusshk-2.0*fx+fxminhk)/h(k)**2.0
            if( (fiFhk.eq.0.0).or.(fiBhk.eq.0.0).or.(FIhk.eq.0.0) )
     +           goto 3000

            ChatfiF = 2.0*epsa/( h(k)*dabs(fiFhk) ) 
            ChatfiB = 2.0*epsa/( h(k)*dabs(fiBhk) ) 
            ChatFI = 4.0*epsa/( (h(k)**2.0)*dabs(FIhk) ) 

 2000       if( dmax1(ChatfiF,ChatfiB) .le. 0.1 ) hs = h(k)
            if( ChatFI .le. 0.1 .and. ChatFI .ge. 0.001 )then
                hFI = h(k) 
                goto 5000
            endif

            if( ChatFI .lt. 0.001 )goto 4000

 3000       k = k+1
            h(k) = h(k-1)*10.0 
            nyx(i) = x(i) + h(k) 
            call fvalue( fxplusshk,nyx,xdim )
            nyx(i) = x(i) - h(k) 
            call fvalue( fxminhk,nyx,xdim )

            fiFhk = (fxplusshk-fx)/h(k) 
            fiBhk = (fx-fxminhk)/h(k) 
            fiChk = (fxplusshk-fxminhk)/(2.0*h(k)) 
            FIhk  = (fxplusshk-2.0*fx+fxminhk)/(h(k)**2.0)
            if( (fiFhk.eq.0.0).or.(fiBhk.eq.0.0).or.(FIhk.eq.0.0) )
     +           goto 3000
            ChatfiF= 2.0*epsa/( h(k)*dabs(fiFhk) ) 
            ChatfiB = 2.0*epsa/( h(k)*dabs(fiBhk) ) 
            ChatFI  = 4.0*epsa/( (h(k)**2.0)*dabs(FIhk) ) 

            if( (hs.lt.0.0).and.(dmax1(ChatfiF,ChatfiB).le.0.1) )
     +           hs=h(k)
            if( ChatFI .le. 0.1 )then
                hFI = h(k) 
                goto 5000
            endif

            if( k .eq. KAA )goto 6000
            goto 3000

 4000       k = k+1
            h(k) = 0.1*h(k-1) 
            nyx(i) = x(i) + h(k) 
            call fvalue( fxplusshk,nyx,xdim )
            nyx(i) = x(i) - h(k) 
            call fvalue( fxminhk,nyx,xdim )

            fiFhk = (fxplusshk-fx)/h(k) 
            fiBhk = (fx-fxminhk)/h(k) 
            fiChk = (fxplusshk-fxminhk)/(2.0*h(k)) 
            FIhk  = (fxplusshk-2.0*fx+fxminhk)/(h(k)**2.0)
            ChatfiF = 2.0*epsa/( h(k)*dabs(fiFhk) ) 
            ChatfiB = 2.0*epsa/( h(k)*dabs(fiBhk) ) 
            ChatFI  = 4.0*epsa/( (h(k)**2.0)*dabs(FIhk) ) 

            if( ChatFI .gt. 0.1 )then
                hFI = h(k-1) 
                goto 5000
            endif

            if( dmax1(ChatfiF,ChatfiB) .le. 0.1 ) hs = h(k)
            if( ChatFI .le. 0.1 .and. ChatFI .ge. 0.001 )then
                hFI = h(k) 
                goto 5000
            endif

            if( k.eq.KAA )goto 6000
            goto 4000


 5000       hF = 2.0*dsqrt( epsa/dabs(FIhk) ) 
            nyx(i) = x(i) + hF 
            call fvalue( fxplusshF,nyx,xdim )
            fi = ( fxplusshF-fx )/hF 
            nyx(i) = x(i) - hF 
            call fvalue( fxminhF,nyx,xdim )
            FIVAL = (fxplusshF-2.0*fx+fxminhF)/(hF**2.0)
            EF = hF*dabs(FIVAL)/2.0 + 2.0*epsa/hF
            nyx(i) = x(i) + hFI 
            call fvalue( fxplusshFI,nyx,xdim )
            nyx(i) = x(i) - hFI 
            call fvalue( fxminhFI,nyx,xdim )
            fiChFI = (fxplusshFI-fxminhFI)/(2.0*hFI) 
            Estrek = dabs( (fi-fiChFI) ) 
            if( dmax1(EF,Estrek) .le. 0.5*dabs(fi) )then
                hfail(i) = 0 
            else
                hfail(i) = 1 
            endif
            goto 7000

 6000       if( hs .lt. 0.0 )then
                hF = 0.1*h(1) 
                fi = 0.0
                FIVAL = 0.0
                EF = 0.0
                hfail(i) = 2 
                goto 7000
            endif

            if( ChatFI .gt. 0.1 .and. hs .gt. 0.0 )then
               hF = hs 
               nyx(i) = x(i) + hF 
               call fvalue( fxplusshF,nyx,xdim )
               fi = (fxplusshF-fx)/hF 
               FIVAL = 0.0
               EF = hF*dabs(FIVAL)/2.0 + 2.0*epsa/hF
               hfail(i) = 3 
               goto 7000
            endif

            hF = h(k) 
            fi = fiFhk 
            FIVAL = FIhk
            EF = hF*dabs(FIVAL)/2.0 + 2.0*epsa/hF
            hfail(i) = 4 

 7000       xperturb(i) = hF 
            grad(i) = fi 
            hessdiag(i) = FIVAL 
            errgrad(i) = EF 
            nyx(i) = x(i) 
 100   continue
       return
       end

