

      SUBROUTINE NLPQL0(M,ME,N,X,F,XL,XU,IPRINT,IFAIL,WA,LWA,KWA,LKWA,

     *                  MAXIT,MAXFUN)

C

C***********************************************************************

C

C

C        AN IMPLEMENTATION OF A SEQUENTIAL QUADRATIC PROGRAMMING

C          METHOD FOR SOLVING NONLINEAR OPTIMIZATION PRPOBLEMS

C

C

C

C   THIS SUBROUTINE SOLVES THE GENERAL NONLINEAR PROGRAMMING PROBLEM

C

C             MINIMIZE    F(X)

C             SUBJECT TO  G(J,X)   =  0  ,  J=1,...,ME

C                         G(J,X)  >=  0  ,  J=ME+1,...,M

C                         XL  <=  X  <=  XU

C

C   NLPQL0 IS THE EASY-TO-USE SUBROUTINE TO SOLVE THE ABOVE NONLINEAR

C   OPTIMIZATION PROBLEM. IF SOME PARAMETERS ARE TO BE CHANGED, IF

C   MORE INITIAL INFORMATION SHALL BE PROVIDED, IF REVERSE COMMUNI-

C   CATION IS TO BE USED, OR IF PROGRAM MODULES ARE TO BE EXCHANGED,

C   PROCEED TO SUBROUTINE NLPQL1.

C

C

C   USAGE:

C

C      NLPQL0(M,ME,N,X,F,XL,XU,IPRINT,IFAIL,WA,LWA,KWA,LKWA)

C

C

C   DEFINITION OF THE PARAMETERS:

C

C   M :       TOTAL NUMBER OF CONSTRAINTS.

C   ME :      NUMBER OF EQUALITY CONSTRAINTS.

C   N :       NUMBER OF VARIABLES.

C   X(N) :    ON INPUT, THE ONE-DIMENSIONAL ARRAY X HAS TO CONTAIN AN

C             INITIAL GUESS FOR THE OPTIMAL SOLUTION. ON RETURN, X IS

C             REPLACED BY THE LAST COMPUTED ITERATE.

C   F :       ON RETURN, F CONTAINS THE OBJECTIVE FUNCTION VALUE OF X.

C   XL(N),XU(N) : ON INPUT, THE ONE-DIMENSIONAL ARRAYS XL AND XU MUST

C             CONTAIN THE UPPER AND LOWER BOUNDS OF THE VARIABLES.

C   IPRINT :  SPECIFICATION OF THE DESIRED OUTPUT LEVEL.

C      IPRINT = 0 :  NO OUTPUT OF THE PROGRAM.

C      IPRINT = 1 :  ONLY A FINAL CONVERGENCE ANALYSIS IS GIVEN.

C      IPRINT = 2 :  ONE LINE OF INTERMEDIATE RESULTS ARER PRINTED IN EACH

C                    ITERATION.

C      IPRINT = 3 :  MORE DETAILED INFORMATION IS PRINTED IN EACH ITERATION

C                    STEP, E.G. VARIABLE, CONSTRAINT AND MULTIPLIER VALUES.

C      IPRINT = 4 :  IN ADDITION TO 'IPRINT=3', MERIT FUNCTION AND STEPLENGTH

C                    VALUES ARE DISPLAYED DURING THE LINE-SEARCH.

C   IFAIL :   THE PARAMETER SHOWS THE REASON FOR TERMINATING A SOLUTION

C             PROCESS.

C      IFAIL = 0 :   THE OPTIMALITY CONDITIONS ARE SATISFIED.

C      IFAIL = 1 :   THE ALGORITHM HAS BEEN STOPPED AFTER 40 ITERATIONS.

C      IFAIL = 2 :   THE ALGORITHM COMPUTED AN UPHILL SEARCH DIRECTION.

C      IFAIL = 3 :   UNDERFLOW OCCURED WHEN DETERMINING A NEW APPROXI-

C                    MATION MATRIX FOR THE HESSIAN OF THE LAGRANGIAN.

C      IFAIL = 4 :   MORE THAN 10 FUNCTION CALLS ARE REQUIRED DURING THE

C                    LINE SEARCH.

C      IFAIL = 5 :   LENGTH OF A WORKING ARRAY IS TOO SHORT. MORE

C                    DETAILED ERROR INFORMATION IS OBTAINED WITH

C                    'IPRINT > 0' .

C      IFAIL = 7 :   THE SEARCH DIRECTION IS CLOSE TO ZERO, BUT THE

C                    CURRENT ITERATE IS STILL INFEASIBLE.

C      1000 > IFAIL > 10 :  THE SOLUTION OF THE QUADRATIC OR

C                    LEAST SQUARES SUBPROBLEM HAS BEEN TERMINATED WITH

C                    AN ERROR MESSAGE  IFQL > 0  AND IFAIL IS SET

C                    TO  IFAIL = IFQL + 10 .

C   WA(LWA),LWA :  WA IS A REAL WORKING ARRAY OF LENGTH LWA. LWA MUST BE

C             AT LEAST N*N + M*N + 11*M + 25*N + 60 AND HAS TO BE ENLARGED,

C             IF ADDITIONAL WORKING SPACE IS REQUIRED TO SOLVE THE

C             QUADRATIC PROGRAMMING OR LINEAR LEAST SQUARES SUBPROBLEM.

C   KWA(LKWA),LKWA :  THE USER HAS TO PROVIDE WORKING SPACE FOR AN

C             INTEGER ARRAY. LKWA SHOULD BE AT LEAST 18 OR GREATER,

C             IF ADDITIONAL SPACE IS REQUIRED TO SOLVE THE SUBPROBLEM.

C

C   A NAMED COMMON-BLOCK  /CMACHE/EPS  HAS TO BE DEFINED BY THE USER

C   CONTAINING A GUESS FOR THE UNDERLYING MACHINE PRECISION. ONE FURTHER

C   COMMON-BLOCK /NLPQL3/  IS USED INTERNALLY AND TRANSPORTS SOME DATA

C   FROM NLPQL1 TO NLPQL2.

C

C

C   THE USER HAS TO PROVIDE THE FOLLOWING SUBROUTINES:

C

C   A) CALCULATION OF PROBLEM FUNCTIONS:

C

C         NLFUNC(M,ME,MMAX,N,F,G,X,ACTIVE)

C

C      DEFINITION OF PARAMETERS:

C

C      M,ME,N :       CONTAIN SOME PROBLEM PARAMETERS AS DESCRIBED ABOVE. 

C      MMAX :         DIMENSION OF G AND ACTIVE. WHEN CALLING NLFUNC, MMAX IS

C                     GREATER THAN M.

C      F :            THE USER HAS TO EVALUATE THE OBJECTIVE FUNCTION AND TO

C                     STORE THE RESULT IN F.

C      G(MMAX) :      THE USER HAS TO EVALUATE THE ACTIVE CONSTRAINTS (I.E.

C                     ACTIVE(J) = .TRUE.) AND TO STORE THE RESULTS IN G

C                     (I.E. G(J)). IF ANALYTICAL DIFFERENTIATION IS USED,

C                     THEN ALL CONSTRAINTS MUST BE EVALUATED SIMULTANEOUSLY.

C      X(N) :         WHEN CALLING NLFUNC, X CONTAINS THE CURRENT ITERATE.

C      ACTIVE(MMAX) : LOGICAL ARRAY INDICATING THE ACTIVE CONSTRAINTS WHEN

C                     CALLING NLFUNC.

C 

C

C   B) CALCULATION OF THE GRADIENTS OF PROBLEM FUNCTIONS:

C

C         NLGRAD(M,ME,MMAX,N,F,G,DF,DG,X,ACTIVE,WA)

C

C      DEFINITION OF PARAMETERS:

C

C      M,ME,N :       CONTAIN SOME PROBLEM PARAMETERS AS DESCRIBED ABOVE.

C      MMAX :         DIMENSION OF G, ACTIVE, WA AND ROW DIMENSION OF DG.

C      F :            WHEN CALLING NLGRAD, F CONTAINS THE ACTUAL OBJECTIVE

C                     FUNCTION VALUE.

C      G(MMAX) :      WHEN CALLING NLGRAD, G CONTAINS THE ACTUAL CONSTRAINT

C                     FUNCTION VALUES.

C      DF(N) :        THE USER HAS TO EVALUATE THE GRADIENT OF THE OBJECTIVE

C                     FUNCTION AND TO STORE THE RESULTS IN DF.

C      DG(MMAX,N) :   THE USER HAS TO EVALUATE THE GRADIENTS OF ACTIVE

C                     RESTRICTIONS (ACTIVE(J) = .TRUE.) AND TO STORE THE

C                     RESULTS IN DG.

C      X(N) :         WHEN CALLING NLGRAD, X CONTAINS THE CURRENT ITERATE.

C      ACTIVE(MMAX) : LOGICAL ARRAY DETERMINING THE ACTIVE CONSTRAINTS.

C      WA(MMAX) :     WORKING ARRAY THAT CAN BE USED E.G. FOR IMPLEMENTING 

C                     A NUMERICAL DIFFERENTIATION RULE.

C      NOTE: WHENEVER NLGRAD IS CALLED, A PREVIOUS CALL OF NLFUNC WAS MADE

C            AT THE SAME ITERATE X AND F,G CONTAIN FUNCTION VALUES OF 

C            OBJECTIVE AND CONSTRAINTS AT X

C

C

C   AUTHOR:  K. SCHITTKOWSKI, 

C            MATHEMATISCHES INSTITUT,

C            UNIVERSITAET BAYREUTH,

C            8580 BAYREUTH,

C            GERMANY, F.R.

C

C

C   REFERENCE :  K. SCHITTKOWSKI. DESIGN, IMPLEMENTATION, AND TEST OF A

C                NONLINEAR PROGRAMMING ALGORITHM, REPORT, INSTITUT FUER

C                INFORMATIK, UNIVERSITAET STUTTGART, 1984.

C

C

C   VERSION:  2.3    (APRIL,1987)

C

C

C***********************************************************************

C

      INTEGER N,LWA,LKWA

      DIMENSION X(N),XL(N),XU(N),WA(LWA),KWA(LKWA)

      INTEGER M,ME,IPRINT,IFAIL,KWA

      INTEGER MAXFUN,MAXIT,MODE,IOUT,N1,M1,MNN1,MNN2,IG,IDF,IDG,IU,

     1     IDCL,ICD,IWA,LWA1,LACTIV,ITEST

      LOGICAL ACTIVE(200),LQL,LMERIT

      DOUBLE PRECISION X,F,XL,XU,WA,ACC,SCBOU,EPS

C

C   INTRINSIC FUNCTION:  DSQRT

C

      COMMON/CMACHE/EPS

C

C   PREPARE SOME DATA FOR CALLING NLPQL1

C

      ACC=DSQRT(EPS)

      SCBOU=1.D+3
c     MODE = 0 
      MODE=10

      IFAIL=0

      LQL=.FALSE.

      LMERIT=.TRUE.

      IF (N.LE.20) LQL=.TRUE.

      IOUT=6

C

C   INITIAL ADDRESSES IN WA

C

      N1=N+1

      M1=M

      IF (M.LE.0) M1=1

      MNN1=M1+N1+N1

      MNN2=M+N1+N1

      IG=1

      IDF=IG+M1

      IDG=IDF+N1

      IU=IDG+M1*N1

      IDCL=IU+MNN2

      ICD=IDCL+N1*N1

      IWA=ICD+N1

      LWA1=LWA-IWA+1

      LACTIV=200

      ITEST=IWA+6*M+20*N+60

      IF(ITEST.LE.LWA) GOTO1

      IFAIL=5

      IF(IPRINT.EQ.0) RETURN

      WRITE(IOUT,1000) ITEST

 1000 FORMAT(8X,44H**LENGTH OF WA TOO SHORT. SHOULD BE AT LEAST,I6)

      RETURN

    1 CONTINUE

C

C   CALL OF NLPQL1

C

      CALL NLPQL1(M,ME,M1,N,N1,MNN2,X,F,WA(IG),WA(IDF),WA(IDG),WA(IU),

     1     XL,XU,WA(IDCL),WA(ICD),ACC,SCBOU,MAXFUN,MAXIT,IPRINT,MODE,

     2     IOUT,IFAIL,WA(IWA),LWA1,KWA,LKWA,ACTIVE,LACTIV,LMERIT,LQL)

C

C   END OF NLPQL0

C

      RETURN

      END

      SUBROUTINE NLPQL1(M,ME,MMAX,N,NMAX,MNN2,X,F,G,DF,DG,U,XL,XU,

     1     C,D,ACC,SCBOU,MAXFUN,MAXIT,IPRINT,MODE,IOUT,IFAIL,WA,LWA,         

     2     KWA,LKWA,ACTIVE,LACTIV,LMERIT,LQL)

C

C***********************************************************************

C

C

C         AN IMPLEMENTATION OF A SEQUENTIAL QUADRATIC PROGRAMMING

C           METHOD FOR SOLVING NONLINEAR OPTIMIZATION PROBLEMS

C

C

C

C   THIS SUBROUTINE SOLVES THE GENERAL NONLINEAR PROGRAMMING PROBLEM

C

C             MINIMIZE    F(X)

C             SUBJECT TO  G(J,X)   =  0  ,  J=1,...,ME

C                         G(J,X)  >=  0  ,  J=ME+1,...,M

C                         XL  <=  X  <=  XU

C

C   THE ALGORITHM IS A MODIFICATION OF THE METHOD OF WILSON, HAN, AND

C   POWELL. IN EACH ITERATION STEP, A LINEARLY CONSTRAINED QUADRATIC

C   OR LINEAR LEAST SQUARES SUBPROBLEM IS FORMULATED BY APPROXIMATING THE

C   LAGRANGE FUNCTION QUADRATICALLY AND BY LINEARIZING THE CONSTRAINTS.

C   SUBSEQUENTLY, A ONE-DIMENSIONAL LINE SEARCH IS PERFORMED WITH

C   RESPECT TO AN AUGMENTED LAGRANGE MERIT FUNCTION TO OBTAIN THE NEW

C   ITERATE.

C

C   THE SUBROUTINE CONTAINS THE OPTIONS TO PREDETERMINE INITIAL

C   GUESSES FOR THE MULTIPLIERS OR THE HESSIAN OF THE LAGRANGE

C   FUNCTION AND TO PERFORM REVERSE COMMUNICATION. IT IS FURTHER-

C   MORE POSSIBLE TO EXCHANGE PROGRAM MODULES SUCH AS THE QUADRATIC

C   OR LINEAR LEAST SQUARES ALGORITHM, THE ONE-DIMENSIONAL MINIMI-

C   ZATION PROCEDURE, AND THE LINE-SEARCH OR MERIT FUNCTION. 

C

C

C   USAGE:

C

C      NLPQL1(M,ME,MMAX,N,NMAX,MNN2,X,F,G,DF,DG,U,XL,XU,C,D,ACC,

C             SCBOU,MAXFUN,MAXIT,IPRINT,MODE,IOUT,IFAIL,WA,LWA,KWA,LKWA,

C             ACTIVE,LACTIV,LMERIT,LQL)

C

C

C   DEFINITION OF THE PARAMETERS:

C

C   M :       NUMBER OF CONSTRAINTS.

C   ME :      NUMBER OF EQUALITY CONSTRAINTS.

C   MMAX :    ROW DIMENSION OF DG. MMAX MUST BE AT LEAST ONE AND

C             GREATER OR EQUAL TO M.

C   N :       NUMBER OF VARIABLES.

C   NMAX :    ROW DIMENSION OF C. NMAX MUST BE AT LEAST TWO AND GREATER

C             THAN N.

C   MNN2 :    MUST BE EQUAL TO M + N + N + 2.

C   X(N) :    ON INPUT, THE ONE-DIMENSIONAL ARRAY X HAS TO CONTAIN AN

C             INITIAL GUESS FOR THE OPTIMAL SOLUTION. ON RETURN, X IS

C             REPLACED BY THE LAST COMPUTED ITERATE.

C   F :       ON RETURN, F CONTAINS THE OBJECTIVE FUNCTION VALUE OF X.

C   G(MMAX) : ON RETURN, THE ARRAY G CONTAINS THE CONSTRAINT FUNCTION

C             VALUES FOR THE LAST ITERATE X.

C   DF(NMAX) : ON RETURN, DF CONTAINS THE LAST COMPUTED GRADIENT OF THE

C             OBJECTIVE FUNCTION.

C   DG(MMAX,NMAX) : ON RETURN, DG CONTAINS THE GRADIENTS OF THE ACTIVE

C             CONSTRAINTS (ACTIVE(J) = .TRUE.) OF THE LAST COMPUTED

C             ITERATE. THE REMAINING ROWS ARE FILLED WITH PREVIOUSLY

C             COMPUTED GRADIENTS. IN THE DRIVING PROGRAM, THE ROW

C             DIMENSION OF DG HAS TO BE EQUAL TO MMAX.

C   U(MNN2) : ON RETURN, U CONTAINS THE MULTIPLIERS WITH RESPECT TO THE

C             LAST COMPUTED ITERATE. THE FIRST M LOCATIONS CONTAIN THE

C             MULTIPLIERS OF THE M NONLINEAR CONSTRAINTS, THE SUBSEQUENT

C             N LOCATIONS THE MULTIPLIERS OF THE LOWER BOUNDS, AND THE

C             FOLLOWING N LOCATIONS THE MULTIPLIERS OF THE UPPER BOUNDS.

C             AT AN OPTIMAL SOLUTION, ALL MULTIPLIERS WITH RESPECT TO

C             INEQUALITY CONSTRAINTS SHOULD BE NONNEGATIVE.

C   XL(N),XU(N) : ON INPUT, THE ONE-DIMENSIONAL ARRAYS XL AND XU MUST

C             CONTAIN THE UPPER AND LOWER BOUNDS OF THE VARIABLES.

C   C(NMAX,NMAX),D(NMAX) : ON RETURN, C CONTAINS THE LAST COMPUTED

C             APPROXIMATION OF THE HESSIAN MATRIX OF THE LAGRANGE

C             FUNCTION. IF LINEAR LEAST SQUARES SUBPROBLEMS ARE FORMU-

C             LATED, C CONTAINS THE LOWER TRIANGULAR FACTOR OF AN LDL-

C             FACTORIZATION OF THE APPROXIMATION OF THE HESSIAN MATRIX

C             (WITHOUT DIAGONAL ELEMENTS, WHICH ARE ALWAYS ONE), AND

C             THE ELEMENTS OF THE DIAGONAL MATRIX ARE STORED IN THE ONE-

C             DIMENSIONAL ARRAY D. IN THE DRIVING PROGRAM, THE ROW

C             DIMENSION OF C HAS TO BE EQUAL TO NMAX.

C   ACC :     THE USER HAS TO SPECIFY THE DESIRED FINAL ACCURACY

C             (E.G. E-7).

C   SCBOU :   THE REAL VARIABLE ALLOWS AN AUTOMATIC SCALING OF THE

C             PROBLEM FUNCTIONS. IF AT THE STARTING POINT X, A FUNCTION

C             VALUE IS GREATER THAN SCBOU (E.G. E+3), THEN THIS FUNCTION

C             IS DIVIDED BY THE INVERSE OF THE SQUARE ROOT. IF SCBOU IS

C             SET TO ANY NEGATIVE REAL NUMBER, THEN THE OBJECTIVE

C             FUNCTION WILL BE MULTIPLIED BY THE VALUE STORED IN 

C             WA(MMAX+1) AND THE J-TH CONSTRAINT FUNCTION BY THE VALUE 

C             STORED IN WA(J), J=1,...,M. NOTE THAT WHEN LEAVING NLPQL1,

C             ONLY THE FUNCTION VALUES IN F AND G, RESPECTIVELY, ARE

C             RESCALED, NOT THE DERIVATIVE VALUES IN DF AND DG.

C   MAXFUN :  THE INTEGER VARIABLE ALLOWS TO GIVE AN UPPER BOUND FOR

C             THE NUMBER OF FUNC-CALLS DURING THE LINE SEARCH (E.G. 10).

C   MAXIT :   THE USER HAS TO SPECIFY AN UPPER BOUND FOR THE NUMBER OF

C             ITERATIONS (E.G. 40).

C   IPRINT :  SPECIFICATION OF THE DESIRED OUTPUT LEVEL.

C      IPRINT = 0 :  NO OUTPUT OF THE PROGRAM.

C      IPRINT = 1 :  ONLY A FINAL CONVERGENCE ANALYSIS IS GIVEN.

C      IPRINT = 2 :  ONE LINE OF INTERMEDIATE RESULTS IS PRINTED IN EACH

C                    ITERATION.

C      IPRINT = 3 :  MORE DETAILED INFORMATION IS PRINTED IN EACH ITERATION

C                    STEP, E.G. VARIABLE, CONSTRAINT AND MULTIPLIER VALUES.

C      IPRINT = 4 :  IN ADDITION TO 'IPRINT=3', MERIT FUNCTION AND STEPLENGTH

C                    VALUES ARE DISPLAYED DURING THE LINE-SEARCH.

C   MODE :    THE INTEGER VARIABLE SPECIFIES THE DESIRED VERSION OF

C             NLPQL1.

C      MODE = 0 :    NORMAL EXECUTION.

C      MODE = 1 :    THE USER WANTS TO PROVIDE AN INITIAL GUESS FOR THE

C                    MULTIPLIERS IN U AND FOR THE HESSIAN OF THE LAGRANGE

C                    FUNCTION IN C.

C      MODE = 2 :    THE USER WANTS TO PERFORM REVERSE COMMUNICATION.

C                    IN THIS CASE, IFAIL MUST BE SET TO ZERO INITIALLY

C                    AND F, G, DF, DG HAVE TO CONTAIN THE CORRESPONDING

C                    FUNCTION AND GRADIENT VALUES. IF NLPQL1 RETURNS TO

C                    THE MAIN PROGRAM WITH  IFAIL = -1 , THE USER HAS TO

C                    CALCULATE NEW OBJECTIVE AND CONSTRAINT FUNCTION

C                    VALUES WITH RESPECT TO THE VARIABLE STORED IN X, TO

C                    REPLACE F AND G BY THESE VALUES, AND TO EXECUTE

C                    NLPQL1 AGAIN. IF  IFAIL = -2 , THE USER IS TO

C                    CALCULATE NEW GRADIENTS FOR OBJECTIVE FUNCTION AND

C                    ACTIVE CONSTRAINTS (ACTIVE(J) = .TRUE.), TO STORE

C                    THEM IN DF AND DG, RESPECTIVELY, AND TO CALL NLPQL1

C                    AGAIN.

C      MODE = 3 :    REVERSE COMMUNICATION IS TO BE PERFORMED AND, IN ADDITION,

C                    INITIAL GUESSES FOR MULTIPLIERS AND HESSIAN MATRIX OF THE

C                    LAGRANGE FUNCTION ARE PROVIDED BY THE USER.

C      MODE = 5,6,7,8 : SAME AS THE VERSIONS CORRESPONDING TO MODE = 0,1,2,3,

C                    RESPECTIVELY, BUT AN EXPANDED SUBPROBLEM IS FORMU-

C                    LATED, SO THAT THE ALGORITHM FOR THE SUBPROBLEM IS

C                    ALWAYS PROVIDED WITH A FEASIBLE STARTING POINT.

C      MODE = 10,11,12,13,15,16,17,18 : SAME AS THE VERSIONS CORRESPONDING TO

C                    MODE = 0,1,2,3,5,6,7,8 , RESPECTIVELY, BUT RESTARTS ARE

C                    PERFORMED IN THE ERROR CASE  IFAIL > 0 , I.E.

C                    THE APPROXIMATION OF THE HESSIAN OF THE LAGRANGE

C                    FUNCTION IS REPLACED BY THE IDENTITY MATRIX AND

C                    THE MULTIPLIER ESTIMATES BY ZERO.

C   IOUT :    INTEGER INDICATING THE DESIRED OUTPUT UNIT NUMBER, I.E. ALL

C             WRITE-STATEMENTS START WITH 'WRITE(IOUT,... '.

C   IFAIL :   THE PARAMETER SHOWS THE REASON FOR TERMINATING A SOLUTION

C             PROCESS. INITIALLY IFAIL MUST BE SET TO ZERO. ON RETURN IFAIL

C             COULD CONTAIN THE FOLLOWING VALUES:

C      IFAIL = 0 :   THE OPTIMALITY CONDITIONS ARE SATISFIED.

C      IFAIL = 1 :   THE ALGORITHM HAS BEEN STOPPED AFTER MAXIT

C                    ITERATIONS.

C      IFAIL = 2 :   THE ALGORITHM COMPUTED AN UPHILL SEARCH DIRECTION.

C      IFAIL = 3 :   UNDERFLOW OCCURED WHEN DETERMINING A NEW APPROXI-

C                    MATION MATRIX FOR THE HESSIAN OF THE LAGRANGIAN.

C      IFAIL = 4 :   MORE THAN MAXFUN FUNCTION EVALUATIONS ARE REQUIRED

C                    DURING THE LINE SEARCH ALGORITHM.

C      IFAIL = 5 :   LENGTH OF A WORKING ARRAY IS TOO SHORT. MORE

C                    DETAILED ERROR INFORMATION IS OBTAINED WITH

C                    'IPRINT > 0' .

C      IFAIL = 6 :   THERE ARE FALSE DIMENSIONS, I.E.  M > MMAX ,

C                    N >= NMAX , OR  MNN2 <> M+N+N+2 .

C      IFAIL = 7 :   THE SEARCH DIRECTION IS CLOSE TO ZERO, BUT THE

C                    CURRENT ITERATE IS STILL INFEASIBLE.

C      1000 > IFAIL > 10 :  THE SOLUTION OF THE QUADRATIC OR

C                    LEAST SQUARES SUBPROBLEM HAS BEEN TERMINATED WITH

C                    AN ERROR MESSAGE  IFQL > 0  AND IFAIL IS SET

C                    TO  IFAIL = IFQL + 10 .

C   WA(LWA),LWA :  WA IS A REAL WORKING ARRAY OF LENGTH LWA. LWA MUST BE

C             AT LEAST 4*MMAX + 4*M + 19*N + 55, IF NO GRADIENT EVALU-

C             ATIONS ARE REQUIRED IN THE LINE SEARCH, AND

C             MMAX*N + 4*MMAX + 4*M + 18*N + 55 OTHERWISE. LWA HAS TO BE

C             ENLARGED, IF ADDITIONAL WORKING SPACE IS REQUIRED TO SOLVE

C             THE QUADRATIC PROGRAMMING OR LEAST SQUARES SUBPROBLEM.

C             THE INCLUDED QP-ALGORITHM REQUIRES ADDITIONAL WORKING SPACE

C             FOR 3/2*(N+1)*(N+1) + 10*N + 2*M + 10 REAL NUMBERS, IF

C             NMAX=N+1 AND MMAX=M, RESPECTIVELY.

C   KWA(LKWA),LKWA :  THE USER HAS TO PROVIDE WORKING SPACE FOR AN

C             INTEGER ARRAY. LKWA SHOULD BE AT LEAST 19 OR GREATER,

C             IF ADDITIONAL SPACE IS REQUIRED TO SOLVE THE SUBPROBLEM.

C             THE INCLUDED QP-ALGORITHM REQUIRES M+N+N ADDITIONAL WORKING

C             ELEMENTS OF KWA.

C   ACTIVE(LACTIV),LACTIV : THE LOGICAL ARRAY SHOWS A USER THE CON-

C             STRAINTS, WHICH NLPQL1 CONSIDERES TO BE ACTIVE AT THE LAST

C             COMPUTED ITERATE, I.E. G(J,X) IS ACTIVE, IF AND ONLY IF

C             ACTIVE(J) = .TRUE. , J=1,...,M. THE LENGTH LACTIV OF THE

C             LOGICAL ARRAY SHOULD BE AT LEAST 2*MMAX + 15.

C   LMERIT :  IF  LMERIT=.TRUE. , THE ALGORITHM USES THE L2-AUGMENTED

C             LAGRANGIAN PENALTY FUNCTION FOR THE LINE SEARCH. OTHER-

C             WISE THE L1-PENALTY FUNCTION ID APPLIED. IT IS RECOMMENDED

C             TO START WITH  LMERIT=.TRUE.  AND TO SWITCH TO THE L1-PENAL-

C             TY FUNCTION IN ERRROR CASES, E.G. IF  IFAIL=4 .

C   LQL :     IF  LQL = .TRUE. , THEN THE QUADRATIC PROGRAMMING SUBPROBLEM

C             IS TO BE SOLVED WITH A FULL POSITIVE DEFINITE QUASI-NEWTON

C             MATRIX. OTHERWISE, A CHOLESKY-DECOMPOSITION IS PERFORMED

C             AND UPDATED BY NLPQL1, SO THAT THE SUBPROBLEM MATRIX CONTAINS

C             AN UPPER TRIANGULAR FACTOR.

C

C   THE USER HAS TO PROVIDE A NAMED COMMON-BLOCK  /CMACHE/EPS  CONTAINING

C   A REAL NUMBER DEFINING THE UNDERLYING MACHINE PRECISION. ONE FURTHER

C   COMMON-BLOCK  /NLPQL3/  IS USED INTERNALLY AND TRANSPORTS SOME DATA 

C   FROM NLPQL1 TO NLPQL2. 

C

C

C   THE USER HAS TO PROVIDE THE FOLLOWING SUBROUTINES:

C

C   A) CALCULATION OF PROBLEM FUNCTIONS:

C

C         NLFUNC(M,ME,MMAX,N,F,G,X,ACTIVE)

C

C      DEFINITION OF PARAMETERS:

C

C      M,ME,N :       CONTAIN SOME PROBLEM PARAMETERS AS DESCRIBED ABOVE. 

C      MMAX :         DIMENSION OF G AND ACTIVE. WHEN CALLING NLFUNC, MMAX IS

C                     GREATER THAN M.

C      F :            THE USER HAS TO EVALUATE THE OBJECTIVE FUNCTION AND TO

C                     STORE THE RESULT IN F.

C      G(MMAX) :      THE USER HAS TO EVALUATE THE ACTIVE CONSTRAINTS (I.E.

C                     ACTIVE(J) = .TRUE.) AND TO STORE THE RESULTS IN G

C                     (I.E. G(J)). IF ANALYTICAL DIFFERENTIATION IS USED,

C                     THEN ALL CONSTRAINTS MUST BE EVALUATED SIMULTANEOUSLY.

C      X(N) :         WHEN CALLING NLFUNC, X CONTAINS THE CURRENT ITERATE.

C      ACTIVE(MMAX) : LOGICAL ARRAY INDICATING THE ACTIVE CONSTRAINTS WHEN

C                     CALLING NLFUNC.

C 

C

C   B) CALCULATION OF THE GRADIENTS OF PROBLEM FUNCTIONS:

C

C         NLGRAD(M,ME,MMAX,N,F,G,DF,DG,X,ACTIVE,WA)

C

C      DEFINITION OF PARAMETERS:

C

C      M,ME,N :       CONTAIN SOME PROBLEM PARAMETERS AS DESCRIBED ABOVE.

C      MMAX :         DIMENSION OF G, ACTIVE, WA AND ROW DIMENSION OF DG.

C      F :            WHEN CALLING NLGRAD, F CONTAINS THE ACTUAL OBJECTIVE

C                     FUNCTION VALUE.

C      G(MMAX) :      WHEN CALLING NLGRAD, G CONTAINS THE ACTUAL CONSTRAINT

C                     FUNCTION VALUES.

C      DF(N) :        THE USER HAS TO EVALUATE THE GRADIENT OF THE OBJECTIVE

C                     FUNCTION AND TO STORE THE RESULTS IN DF.

C      DG(MMAX,N) :   THE USER HAS TO EVALUATE THE GRADIENTS OF ACTIVE

C                     RESTRICTIONS (ACTIVE(J) = .TRUE.) AND TO STORE THE

C                     RESULTS IN DG.

C      X(N) :         WHEN CALLING NLGRAD, X CONTAINS THE CURRENT ITERATE.

C      ACTIVE(MMAX) : LOGICAL ARRAY DETERMINING THE ACTIVE CONSTRAINTS.

C      WA(MMAX) :     WORKING ARRAY THAT CAN BE USED E.G. FOR IMPLEMENTING 

C                     A NUMERICAL DIFFERENTIATION RULE.

C      NOTE: WHENEVER NLGRAD IS CALLED, A PREVIOUS CALL OF NLFUNC WAS MADE

C            AT THE SAME ITERATE X AND F,G CONTAIN FUNCTION VALUES OF 

C            OBJECTIVE AND CONSTRAINTS AT X

C   

C   AUTHOR:  K. SCHITTKOWSKI, 

C            MATHEMATISCHES INSTITUT,

C            UNIVERSITAET BAYREUTH,

C            8580 BAYREUTH,

C            GERMANY, F.R.

C

C

C   REFERENCE :  K. SCHITTKOWSKI. DESIGN, IMPLEMENTATION, AND TEST OF A

C                NONLINEAR PROGRAMMING ALGORITHM, REPORT, INSTITUT FUER

C                INFORMATIK, UNIVERSITAET STUTTGART, 1984.

C

C

C   VERSION:  2.3  (APRIL, 1987)

C

C

C***********************************************************************

C

      INTEGER N,MMAX,NMAX,MNN2,LWA,LKWA,LACTIV

      DIMENSION X(N),G(MMAX),DF(NMAX),DG(MMAX,NMAX),U(MNN2),

     1     XL(N),XU(N),C(NMAX,NMAX),D(NMAX),WA(LWA),

     2     KWA(LKWA),ACTIVE(LACTIV)

      INTEGER M,ME,MAXFUN,MAXIT,IPRINT,MODE,IOUT,IFAIL,KWA

      LOGICAL ACTIVE,REST,LQL,LMERIT,LMER,LLISE,LLISE1,LQL1

      DOUBLE PRECISION  X,G,DF,DG,U,XL,XU,C,D,WA,F,ACC,SCBOU

      DOUBLE PRECISION F1,ACC1,SCBOU1,DBDFAC,ZEFAC,RPENO,RPENS,RPENU,

     1     ZEFACU,DELTA,BETA,AMUE,ALM

C

C   INTRINSIC FUNCTION: DSQRT

C

      INTEGER MODE1,MAXIT1,MAXIT2,N1,LACT,M1,M2,M3,M4,M5,M6,M7,

     1     NO1,MNN,NMNN,ISCG,ISCF,I1,I2,I3,I4,I5,I6,IVMU,IDEL,IDLA,

     2     ICLF,IBDEL,IETA,IXOLD,IDLAOL,IV,IW,IVMUOL,IDPHI,IPEN,IFB,

     3     IDFB,IGB,IDGB,IWAQL,ITEST,LWAQL,LKWA0,I,MM1,ME1,IPRIN1,

     4     IFAIL1,MAXFU1,IOUT1

      COMMON/NLPQL3/F1,ACC1,SCBOU1,DBDFAC,ZEFAC,RPENO,RPENS,RPENU,     

     1     ZEFACU,DELTA,BETA,AMUE,ALM,MM1,ME1,MAXFU1,MAXIT1,IPRIN1,

     2     MODE1,IFAIL1,LLISE1,LQL1,LMER,IOUT1

C

C   PROVIDE CONSTANT PARAMETERS FOR CALLING NLPQL2

C

      DBDFAC=2.D-1

      ZEFAC=1.D+0

      IF(.NOT.LQL) ZEFAC=DSQRT(ZEFAC)

      RPENO=1.D+1

      RPENS=1.D-2

      ZEFACU=1.D+6

      IF(.NOT.LQL) ZEFACU=DSQRT(ZEFACU)

      RPENU=1.D+9

      DELTA=9.D-1

      BETA=1.D-1

      AMUE=1.D-4

      ALM=1.D+0

      IOUT1=IOUT

      LLISE=.TRUE.

C

C   PRINT PARAMETERS

C

      IF(IPRINT.EQ.0) GOTO50

      IF(IFAIL.LT.0) GOTO50

      WRITE(IOUT,1000)

 1000 FORMAT(////,3X,

     1 60H------------------------------------------------------------,

     2 15H---------------,

     3 /,5X,55HSTART OF THE SEQUENTIAL QUADRATIC PROGRAMMING ALGORITHM,

     4     /,3X, 

     5 60H------------------------------------------------------------,

     6 15H---------------)

      WRITE(IOUT,1100) MODE,ACC,SCBOU,MAXFUN,MAXIT,IPRINT

 1100 FORMAT(/,5X,11HPARAMETERS:,/,8X,6HMODE =,I3,/,8X,5HACC =,

     1     D13.4,/,8X,7HSCBOU =,D13.4,/,8X,8HMAXFUN =,I3,/,8X,7HMAXIT =,

     2     I5,/,8X,8HIPRINT =,I4)

      IF (IPRINT.NE.2) GOTO 50

      WRITE(IOUT,1600)

 1600 FORMAT(/,5X,30HOUTPUT IN THE FOLLOWING ORDER:,

     1   /,8X,24HIT    - ITERATION NUMBER,

     2   /,8X,32HF     - OBJECTIVE FUNCTION VALUE,

     3   /,8X,35HSCV   - SUM OF CONSTRAINT VIOLATION,

     4   /,8X,36HNA    - NUMBER OF ACTIVE CONSTRAINTS,

     4   /,8X,40HI     - NUMBER OF LINE SEARCH ITERATIONS,

     5   /,8X,28HALPHA - STEPLENGTH PARAMETER,

     6   /,8X,52HDELTA - ADDITIONAL VARIABLE TO PREVENT INCONSISTENCY,  

     7   /,8X,43HDLAN  - MAXIMUM NORM OF LAGRANGIAN GRADIENT,

     8   /,8X,40HKT    - KUHN-TUCKER OPTIMALITY CRITERION)

      WRITE(IOUT,1700)

 1700 FORMAT(//,2X,

     1 60HIT         F          SCV     NA  I    ALPHA    DELTA      D,

     2 15HLAN      KT    ,

     3 /,2X,

     4 60H------------------------------------------------------------,

     5 15H---------------)  

   50 CONTINUE

      MODE1=MODE

      MAXIT1=MAXIT

      MAXIT2=MAXIT/2

      REST=.FALSE.

      IF(MODE.LT.10) GOTO2

      REST=.TRUE.

      MODE1=MODE-10

    2 CONTINUE

C

C   INITIAL ADDRESSES IN WA

C

      N1=N+1

      LACT=2*MMAX+6

      M1=LACT+1

      M2=LACT+2

      M3=LACT+3

      M4=LACT+4

      M5=LACT+5

      M6=LACT+6

      M7=LACT+7

      NO1=N

      IF(LLISE) NO1=1

      MO1=MMAX

      IF(LLISE) MO1=1

      MNN=M+N+N

      NMNN=N+MNN

      IF(N.LT.NMAX.AND.M.LE.MMAX.AND.MNN2.EQ.MNN+2) GOTO10

      IFAIL=6

      IF(IPRINT.EQ.0) RETURN

      WRITE(IOUT,1010)

 1010 FORMAT(8X,18H**WRONG DIMENSIONS)

      RETURN

   10 CONTINUE

      ISCG=1

      ISCF=ISCG+MMAX

      I1=ISCF+1

      I2=I1+1

      I3=I2+1

      I4=I3+1

      I5=I4+1

      I6=I5+1

      IVMU=I6+1

      IDEL=IVMU+MNN

      IDLA=IDEL+N1

      ICLF=IDLA+N

      IBDEL=ICLF+N1

      IETA=IBDEL+N

      IXOLD=IETA+N

      IDLAOL=IXOLD+N

      IV=IDLAOL+N

      IW=IV+N1

      IVMUOL=IW+N1

      IDPHI=IVMUOL+MNN

      IPEN=IDPHI+MNN+N

      IFB=IPEN+MNN

      IDFB=IFB+1

      IGB=IDFB+NO1

      IDGB=IGB+MMAX

      IWAQL=IDGB+MO1*N

      ITEST=IWAQL+MMAX+39

      LWAQL=LWA-IWAQL+1

C

C   CHECK WORKING ARRAY LENGTHS

C

      IF(ITEST.LE.LWA) GOTO1

      IFAIL=5

      IF(IPRINT.EQ.0) RETURN

      WRITE(IOUT,1200) ITEST

 1200 FORMAT(8X,44H**LENGTH OF WA TOO SHORT. SHOULD BE AT LEAST,I6)

      RETURN

    1 IF(LKWA.GT.18) GOTO7

      IFAIL=5

      IF(IPRINT.EQ.0) RETURN

      WRITE(IOUT,1400)

 1400 FORMAT(8X,48H**LENGTH OF KWA TOO SHORT. SHOULD BE AT LEAST 19)

      RETURN

    7 IF(LACTIV.GT.2*MMAX+15) GOTO8

      IFAIL=5

      IF(IPRINT.EQ.0) RETURN

      WRITE(IOUT,1500)

 1500 FORMAT(8X,

     1   58H**LENGTH OF ACTIVE TOO SHORT. SHOULD BE AT LEAST 2*MMAX+15)

      RETURN

    8 CONTINUE

C

C   CHECK INITIAL POINT

C

      IF(IFAIL.LT.0) GOTO6

      IF (MODE1.EQ.2.OR.MODE1.EQ.3) GOTO 13

      IF (MODE1.EQ.7.OR.MODE1.EQ.8) GOTO 13

      F=ALM

   13 DO5 I=1,N

      IF(X(I).LT.XL(I)) X(I)=XL(I)

    5 IF(X(I).GT.XU(I)) X(I)=XU(I)

    6 CONTINUE

      LKWA0=LKWA-7

C

C   CALL OF NLPQL2

C

    4 CONTINUE

      MM1=M

      ME1=ME

      F1=F

      ACC1=ACC

      SCBOU1=SCBOU

      MAXFU1=MAXFUN

      IPRIN1=IPRINT

      IFAIL1=IFAIL

      LLISE1=LLISE

      LQL1=LQL

      LMER=LMERIT

      CALL NLPQL2(MMAX,N,NMAX,X,G,DF,DG,U,XL,XU,C,D,WA(IVMU),WA(IDEL),     

     1     WA(IDLA),WA(ICLF),WA(IBDEL),WA(IETA),WA(IXOLD),WA(IDLAOL),

     2     WA(IV),WA(IW),WA(IVMUOL),WA(IDPHI),WA(IPEN),WA(ISCG),

     3     WA(IFB),WA(IDFB),WA(IGB),WA(IDGB),WA(IWAQL),LWAQL,N1,MNN,

     4     MNN2,NMNN,NO1,MO1,KWA(1),KWA(2),KWA(3),KWA(4),KWA(5),

     5     KWA(6),KWA(7),KWA(8),LKWA0,WA(I1),WA(I2),WA(I3),WA(I4),

     6     WA(I5),WA(ISCF),WA(I6),ACTIVE,LACT,ACTIVE(M1),ACTIVE(M2),

     7     ACTIVE(M3),ACTIVE(M4),ACTIVE(M5),ACTIVE(M6),ACTIVE(M7))      

      F=F1

      IFAIL=IFAIL1

      IF(.NOT.REST.OR.IFAIL.LT.1.OR.MAXIT1.LT.MAXIT2) GOTO3

      MAXIT1=MAXIT1-10

      IF(MAXIT1.LE.0) GOTO3

      IF(IPRINT.EQ.0) GOTO4

      WRITE(IOUT,1300)

 1300 FORMAT(////,5X,

     1 60H------------------------------------------------------------,

     2 /,5X,7HRESTART,/,5X,

     3 60H------------------------------------------------------------)

      GOTO4

    3 CONTINUE

C

C   END OF NLPQL1

C

      RETURN

      END

      SUBROUTINE NLPQL2(MMAX,N,NMAX,X,G,DF,DG,U,XL,XU,DCL,CD,VMU,

     1     DEL,DLA,DCLF,BDEL,ETA,XOLD,DLAOLD,V,W,VMUOLD,DPHI,RPEN,

     2     SCG,FBEST,DFBEST,GBEST,DGBEST,WA,LWA,N1,MNN,MNN2,NMNN,

     3     NO1,MO1,NFUNC,NGRAD,ITER,NQL,ILINE,IFLISE,NOPT,IW,LIW,PHI,     

     4     DFDEL,DBD,ALPHAM,ALPHAO,SCF,PROD,ACTIVE,LACTIV,LSCF,LSCG,

     5     LLINE,LREST,LALSD,LRECO,LBEST)

C

C*********************************************************************

C

C

C   NONLINEAR PROGRAMMING BY SOLVING QUADRATIC OR LEAST SQUARES

C   SUBPROBLEMS.

C

C   THIS IS THE BASIC SUBROUTINE TO SOLVE THE NONLINEAR

C   PROGRAMMING PROBLEM.

C

C

C*********************************************************************

C

      INTEGER N,MNN2,MNN,NMAX,MMAX,N1,NMNN,NO1,MO1,LWA,LIW,LACTIV

      DIMENSION X(N),U(MNN2),VMU(MNN),DCL(NMAX,NMAX),DEL(N1),

     1     DLA(N),G(MMAX),DF(NMAX),DG(MMAX,NMAX),DCLF(N1),

     2     XOLD(N),DLAOLD(N),CD(NMAX),BDEL(N),ETA(N),

     3     XL(N),XU(N),V(N1),W(N1),VMUOLD(MNN),SCG(MMAX),

     4     DPHI(NMNN),RPEN(MNN),DFBEST(NO1),GBEST(MMAX),DGBEST(MO1,N),

     5     WA(LWA),IW(LIW)

      LOGICAL ACTIVE(LACTIV),LSCF,LSCG,LLINE,LREST,LALSD,LRECO,

     1     LQL,LLISE,LBEST,LMERIT

      DOUBLE PRECISION X,U,VMU,DCL,DEL,DLA,G,DF,DG,DCLF,XOLD,DLAOLD,

     1     CD,BDEL,ETA,XL,XU,V,W,VMUOLD,DPHI,RPEN,SCG,FBEST,DFBEST,

     2     GBEST,DGBEST,WA

      DOUBLE PRECISION F,ACC,SCBOU,DBDFAC,ZEFAC,RPENO,RPENS,RPENU,

     1     ZEFACU,DELTA,BETA,AMUE,ALM

      DOUBLE PRECISION SRES,PHI,SQACC,SDCL11,DCL11,DFDEL,SUM,DBD,

     1     FACT,UAD,PROD,ALPHAM,ALPHAO,EDEL,DBD1,THETA,PHIOLD,

     2     DLAN,THETA1,DBDI,EDELI,SQD,SCF,ZE,ON,TW,UF,OF,XNM,DELNM,

     3     RPMAX,FF,EPS0

C

C   INTRINSIC FUNCTIONS: DOUBLE PRECISION DBLE,DSQRT,DABS,DMIN1,DMAX1

C

      INTEGER M,ME,MAXFUN,MAXIT,IPRINT,IFAIL,MODE,

     1     NFUNC,NGRAD,ITER,NQL,ILINE,IFLISE,NOPT,IW,NACT

      INTEGER IOUT,IOUT1,MN,ME1,N2,LWAQL,LIWQL,ILWLS,IMERIT,IPR,IFAIL1,

     1     MN1,MNN1,I1,I,J,MMAX2

      COMMON/CMACHE/EPS0

      COMMON/NLPQL3/F,ACC,SCBOU,DBDFAC,ZEFAC,RPENO,RPENS,RPENU,

     1     ZEFACU,DELTA,BETA,AMUE,ALM,M,ME,MAXFUN,MAXIT,IPRINT,

     2     MODE,IFAIL,LLISE,LQL,LMERIT,IOUT1

C

C   CONSTANT DATA

C

      ZE=0.D+0

      ON=1.D+0

      TW=2.D+0

      UF=EPS0*EPS0

      OF=ON/UF

      IOUT=IOUT1

C

C   INITIAL DEFINITIONS

C

      MN=M+N

      ME1=ME+1

      N2=N+N

      LWAQL=LWA-MMAX-40

      LIWQL=LIW-10

      ILWLS=2*MMAX+1

      IMERIT=0

      IF(.NOT.LMERIT) IMERIT=4

      LRECO=.FALSE.

      LREST=.FALSE.

      LALSD=.FALSE.

      SQACC=DSQRT(ACC)

      IF(MODE.NE.2.AND.MODE.NE.7.AND.MODE.NE.3.AND.MODE.NE.8) GOTO78

      LRECO=.TRUE.

      IF(IFAIL.EQ.-1) GOTO72

      IF(IFAIL.EQ.-2) GOTO74

   78 ILINE=0

      ALPHAO=ZE

      NFUNC=0

      NGRAD=0

      ITER=0

      NQL=0

      NOPT=0

      IF(M.EQ.0) GOTO51

      MMAX2=MMAX+MMAX

      DO60 J=1,M

   60 ACTIVE(J)=.TRUE.

   51 CONTINUE

      IF(LRECO) GOTO77

      IF (M.EQ.0) GOTO206

      DO 205 J=1,M

  205 ACTIVE(M+J)=.TRUE.

  206 CONTINUE

      CALL NLFUNC(M,ME,MMAX,N,F,G,X,ACTIVE(M+1))

      CALL NLGRAD(M,ME,MMAX,N,F,G,DF,DG,X,ACTIVE,WA(41))

   77 CONTINUE

      LSCF=.FALSE.

      LSCG=.FALSE.

      IF(DABS(F).LT.SCBOU) GOTO160

      LSCF=.TRUE.

      IF(SCBOU.GT.ZE) SCF=ON/DSQRT(DABS(F))

      F=SCF*F

      DO161 I=1,N

  161 DF(I)=SCF*DF(I)

  160 IF(M.EQ.0) GOTO162

      DO163 J=1,M

      IF(DABS(G(J)).LT.SCBOU) GOTO163

      LSCG=.TRUE.

  163 CONTINUE

      IF(.NOT.LSCG) GOTO162

      DO181 J=1,M

      IF(SCBOU.GT.ZE) SCG(J)=ON/DMAX1(ON,DSQRT(DABS(G(J))))

      G(J)=SCG(J)*G(J)

      DO164 I=1,N

  164 DG(J,I)=SCG(J)*DG(J,I)

  181 CONTINUE

  162 CONTINUE

      IF(IPRINT.LT.3) GOTO182

      IF(LSCF.AND..NOT.LSCG) WRITE(IOUT,1410)

 1410 FORMAT(/,5X,33HOBJECTIVE FUNCTION WILL BE SCALED)

      IF(LSCF.AND.LSCG) WRITE(IOUT,1420)

 1420 FORMAT(/,5X,49HOBJECTIVE AND CONSTRAINT FUNCTIONS WILL BE SCALED)

      IF(.NOT.LSCF.AND.LSCG) WRITE(IOUT,1430)

 1430 FORMAT(/,5X,35HCONSTRAINT FUNCTIONS WILL BE SCALED)

  182 CONTINUE

      NFUNC=NFUNC+1

      NGRAD=NGRAD+1

      DCLF(N1)=ZE

      DF(N1)=ZE

      DO1 I=1,N

      DEL(I)=-DF(I)

      DCL(N1,I)=ZE

    1 DCL(I,N1)=ZE

      DCL(N1,N1)=ZEFAC

      IF(MODE.NE.1.AND.MODE.NE.6.AND.MODE.NE.3.AND.MODE.NE.8) GOTO80

      IF(LQL) GOTO25

      GOTO2

   80 CONTINUE

      DO23 I=1,N

      CD(I)=ON

      DO24 J=1,N

   24 DCL(I,J)=ZE

   23 DCL(I,I)=ON

   25 CONTINUE

      DO3 J=1,MNN

      RPEN(J)=RPENS

      VMU(J)=ZE

    3 IF(MODE.EQ.1.OR.MODE.EQ.6.OR.MODE.EQ.3.OR.MODE.EQ.8) VMU(J)=U(J)

      CALL MERIT1(IMERIT+3,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,        

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

C

C   START MAIN LOOP, PRINT INTERMEDIATE ITERATES

C

  100 CONTINUE

      LLINE=.FALSE.

      IF(IPRINT.LT.3) GOTO111

      IF(LSCF) F=F/SCF

      WRITE(IOUT,1000) ITER,F,(X(I),I=1,N)

 1000 FORMAT(//5X,9HITERATION,I3,//8X,23HFUNCTION VALUE:  F(X) =,

     1     D16.8,/8X,14HVARIABLE:  X =,/,(9X,4D16.8))

      IF(LSCF) F=F*SCF

      IF(M.EQ.0.OR.(.NOT.LSCF.AND..NOT.LSCG)) GOTO165

      DO166 J=1,M

      IF(LSCF) VMU(J)=VMU(J)/SCF

      IF(.NOT.LSCG) GOTO166

      VMU(J)=VMU(J)*SCG(J)

      G(J)=G(J)/SCG(J)

  166 CONTINUE

  165 CONTINUE

      WRITE(IOUT,1200) (VMU(J),J=1,MNN)

 1200 FORMAT(8X,17HMULTIPLIERS:  U =,/,(9X,4D16.8))

      IF(M.EQ.0) GOTO55

      WRITE(IOUT,1120)(G(J),J=1,M)

 1120 FORMAT(8X,19HCONSTRAINTS: G(X) =,/,(9X,4D16.8))

      IF(.NOT.LSCF.AND..NOT.LSCG) GOTO111

      DO168 J=1,M

      IF(LSCF) VMU(J)=VMU(J)*SCF

      IF(.NOT.LSCG) GOTO168

      VMU(J)=VMU(J)/SCG(J)

  168 G(J)=G(J)*SCG(J)

   55 CONTINUE

  111 CONTINUE

      ITER=ITER+1

      IF(ITER.LT.MAXIT) GOTO4

      IFAIL=1

      IF(IPRINT.EQ.0) GOTO90

      WRITE(IOUT,1140)

 1140 FORMAT(8X,28H**MORE THAN MAXIT ITERATIONS)

      GOTO90

    4 CONTINUE

C

C   SEARCH DIRECTION

C

      DO71 I=1,N

      DCLF(I)=DF(I)

      V(I)=XL(I) - X(I)

   71 W(I)=XU(I) - X(I)

      IPR=0

      IF(IPRINT.GT.10.AND.IPRINT.LT.1000) IPR=IPRINT-10

      IF(MODE.GE.5) GOTO47

      IFAIL1=ITER

      IF(LREST.OR.LALSD) IFAIL1=1

      IW(11)=0

      IF (LQL) IW(11)=1

      IW(12)=0

      CALL QL0001(M,ME,MMAX,N,NMAX,MNN,DCL,DCLF,DG,G,V,W,DEL,U,

     1     IOUT,IFAIL1,IPR,WA(MMAX+41),LWAQL,IW(11),LIWQL)

      DEL(N1)=ZE

      NQL=NQL+1

      LREST=.FALSE.

      IF(IFAIL1.EQ.0) GOTO126

   47 CONTINUE

      IF(ITER.EQ.1) GOTO154

      FACT=TW*DABS(DBD*DFDEL)/(DSQRT(DBD)*(ON-DEL(N1)))

      IF(LQL) FACT=FACT*FACT

      DCL11=DMAX1(ZEFAC,FACT)

      DCL(N1,N1)=DMIN1(ZEFACU,DCL11)

  154 CONTINUE

      DO41 I=1,N

   41 DEL(I)=ZE

      DEL(N1)=ON

      IF(M.EQ.0) GOTO56

      DO42 J=1,M

      DG(J,N1)=-G(J)

   42 IF(.NOT.ACTIVE(J)) DG(J,N1)=ZE

   56 CONTINUE

      V(N1)=ZE

      W(N1)=ON

      IFAIL1=-ITER

      IF(.NOT.LREST.OR.LALSD) IFAIL1=-1

      IW(11)=0

      IF (LQL) IW(11)=1

      IW(12)=1

      CALL QL0001(M,ME,MMAX,N1,NMAX,MNN2,DCL,DCLF,DG,G,V,W,DEL,U,

     1     IOUT,IFAIL1,IPR,WA(MMAX+41),LWAQL,IW(11),LIWQL)

      NQL=NQL+1

      MN1=M+N1+1

      MNN1=M+N1+N

      LREST=.TRUE.

      IF(IFAIL1.EQ.0) GOTO40

   39 IFAIL=10+IFAIL1

      IF(IPRINT.EQ.0) GOTO90

      WRITE(IOUT,1150) IFAIL1

 1150 FORMAT(8X,26H**ERROR IN QL. IFAIL(QL) =,I3)

      GOTO90

   40 CONTINUE

      DO89 I=MN1,MNN1

   89 U(I-1)=U(I)

      IF(IPRINT.LT.3) GOTO125

      WRITE(IOUT,1250) DEL(N1)

 1250 FORMAT( 8X,45HADDITIONAL VARIABLE TO PREVENT INCONSISTENCY:,

     1     9H  DELTA =,D13.4)

      SDCL11=DCL(N1,N1)

      IF(.NOT.LQL) SDCL11=DSQRT(SDCL11)

      WRITE(IOUT,1360) SDCL11

 1360 FORMAT(8X,35HPENALTY PARAMETER FOR DELTA:  RHO =,D13.4)

  125 CONTINUE

      DCL11=DCL(N1,N1)

      IF(DEL(N1).LT.DELTA) GOTO126

      DCL(N1,N1)=DCL11*RPENO

      IF(LQL) DCL(N1,N1)=DCL(N1,N1)*RPENO

      IF(DCL11.LT.ZEFACU) GOTO154

C

C   AUGMENTED LAGRANGIAN TYPE SEARCH DIRECTION

C

  151 LALSD=.TRUE.

      IF(IPRINT.LT.3) GOTO92

      WRITE(IOUT,1110)

 1110 FORMAT(8X,48H**WARNING: AUGMENTED LAGRANGIAN SEARCH DIRECTION)

   92 CALL MERIT1(4,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,VMU,U,

     1     X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      DO93 I=1,N

      WA(40+I)=DPHI(I)

   93 DCLF(I)=DPHI(I)

      IFAIL1=1

      IW(11)=0

      IF(LQL) IW(11)=1

      IW(12)=0

      CALL QL0001(0,0,MMAX,N,NMAX,MNN2,DCL,DCLF,DG,G,V,W,DEL,U,IOUT,

     1     IFAIL1,IPR,WA(MMAX+41),LWAQL,IW(11),LIWQL)

      IF(IFAIL1.GT.0) GOTO39

      IF(M.EQ.0) GOTO129

      DO96 I=1,N2

      I1=I-1

   96 U(MNN-I1)=U(N2-I1)

      DO94 J=1,M

   94 U(J)=VMU(J) - DPHI(N+J)

      GOTO129

  126 LALSD=.FALSE.

C

C   PROJECTION OF DEL, MAXIMAL STEPLENGTH, AND NORM OF X,DEL

C

  129 ALPHAM=OF

      XNM=ZE

      DELNM=ZE

      DO54 I=1,N

      IF(W(I).LT.DEL(I)) DEL(I)=W(I)

      IF(V(I).GT.DEL(I)) DEL(I)=V(I)

      UAD=DABS(DEL(I))

      IF(DEL(I).GT.UF) ALPHAM=DMIN1(ALPHAM,W(I)/DEL(I))

      IF(DEL(I).LT.-UF) ALPHAM=DMIN1(ALPHAM,V(I)/DEL(I))

      XNM=DMAX1(DABS(X(I)),XNM)

   54 DELNM=DMAX1(UAD,DELNM)

      ALPHAM=DMAX1(ON,ALPHAM)

      ALPHAM=DMIN1(ALPHAM,ALM)

C

C   GRADIENT OF LAGRANGIAN

C

   18 DO5 I=1,N

      UAD=DF(I)

      IF(LALSD) UAD=DPHI(I)

    5 DLA(I)=UAD - U(M+I) - U(MN+I)

      IF(M.EQ.0.OR.LALSD) GOTO57

      DO6 J=1,M

      IF(U(J).EQ.ZE) GOTO6

      DO7 I=1,N

    7 DLA(I)=DLA(I) - DG(J,I)*U(J)

    6 CONTINUE

   57 CONTINUE

      IF(LLINE) GOTO19

C

C   STORE SOME DATA

C

      DFDEL=ZE

      DLAN=ZE

      DO8 I=1,N

      DFDEL=DFDEL + DF(I)*DEL(I)

      DLAOLD(I)=DLA(I)

      DLAN=DMAX1(DLAN,DABS(DLA(I)))

      XOLD(I)=X(I)

    8 CONTINUE

      DO46 J=1,MNN

   46 VMUOLD(J)=VMU(J)

C

C   DETERMINE B*D AND D(T)*B*D

C

      DBD=ZE

      IF(LQL) GOTO81

      DO36 I=1,N

      SUM=ZE

      DO37 J=I,N

   37 SUM=SUM + DCL(I,J)*DEL(J)

   36 ETA(I)=SUM

      DO34 I=1,N

      DBD=DBD + ETA(I)*ETA(I)

      SUM=ZE

      DO35 J=1,I

   35 SUM=SUM + DCL(J,I)*ETA(J)

   34 BDEL(I)=SUM

      GOTO82

   81 DO83 I=1,N

      SUM=ZE

      DO84 J=1,N

   84 SUM=SUM + DCL(I,J)*DEL(J)

      DBD=DBD + SUM*DEL(I)

   83 BDEL(I)=SUM

   82 CONTINUE

C

C   TEST FOR OPTIMALITY AND FINAL OUTPUT

C

      SRES=ZE

      SUM=DABS(DFDEL)

      IF(LSCF) SUM=SUM/SCF

      NACT=0

      IF(M.EQ.0) GOTO58

      DO9 J=1,M

      IF (ACTIVE(J)) NACT=NACT + 1

      UAD=DABS(G(J))

      IF(LSCG) UAD=UAD/SCG(J)

      IF(J.LE.ME.OR.G(J).LT.ZE) SRES=SRES + UAD

    9 SUM=SUM + DABS(U(J)*G(J))

      IF (IPRINT.LT.3) GOTO 58

      WRITE(IOUT,1500) SRES

 1500 FORMAT(8X,30HSUM OF CONSTRAINT VIOLATIONS: ,19X,5HSCV =,

     1     D13.4) 

      WRITE(IOUT,1510) NACT

 1510 FORMAT(8X,30HNUMBER OF ACTIVE CONSTRAINTS: ,19X,5HNAC =,I4)

   58 CONTINUE

      DO70 I=1,N

   70 SUM=SUM + DABS(U(M+I)*V(I)) + DABS(U(MN+I)*W(I))

      IF (IPRINT.NE.2) GOTO 169

      FF=F

      IF (LSCF) FF=F/SCF

      WRITE(IOUT,1460) ITER,FF,SRES,NACT,ILINE,ALPHAO,DEL(N1),

     1     DLAN,SUM   

 1460 FORMAT(1X,I3,D16.8,D10.2,I4,I3,4D10.2)

  169 CONTINUE

      IF (IPRINT.LT.3) GOTO 200

      WRITE(IOUT,1520) SUM

 1520 FORMAT(8X,40HKUHN-TUCKER OPTIMALITY CONDITION:       ,

     1     9X,5HKTO =,D13.4)

      WRITE(IOUT,1530) DLAN

 1530 FORMAT(8X,40HNORM OF LAGRANGIAN GRADIENT:            ,

     1     9X,5HNLG =,D13.4)

  200 CONTINUE

      IF(DBD.GE.UF) GOTO43

      IF(SRES.LT.SQACC) GOTO59

      IF(DBD.GT.ZE) GOTO127

      IF(.NOT.LALSD) GOTO151

      IFAIL=7

      IF(IPRINT.EQ.0) GOTO90

      WRITE(IOUT,1440)

 1440 FORMAT(8X,48H**UNDERFLOW IN D(T)*B*D AND INFEASIBLE ITERATE X)

      GOTO90

   43 CONTINUE

      IF(SUM.GE.ACC.OR.SRES.GT.SQACC) GOTO127

      IF(DLAN.LE.DSQRT(SQACC).OR.DBD.LE.ACC) GOTO59

      NOPT=NOPT+1

      IF(NOPT.LT.3) GOTO127

   59 IFAIL=0

   90 CONTINUE

      IF(LSCF) F=F/SCF

      IF(M.EQ.0.OR.(.NOT.LSCF.AND..NOT.LSCG)) GOTO171

      DO170 J=1,M

      IF(LSCF) U(J)=U(J)/SCF

      IF(.NOT.LSCG) GOTO170

      U(J)=U(J)*SCG(J)

      G(J)=G(J)/SCG(J)

  170 CONTINUE

  171 CONTINUE

      IF(IPRINT.EQ.0) RETURN

      WRITE(IOUT,1270)

 1270 FORMAT(//,5X,28H* FINAL CONVERGENCE ANALYSIS,/)

      WRITE(IOUT,1280) F

 1280 FORMAT(8X,33HOBJECTIVE FUNCTION VALUE:  F(X) =,D16.8)

      WRITE(IOUT,1290) (X(I),I=1,N)

 1290 FORMAT(8X,31HAPPROXIMATION OF SOLUTION:  X =,/,(9X,4D16.8))

      WRITE(IOUT,1300) (U(J),J=1,MNN)

 1300 FORMAT(8X,34HAPPROXIMATION OF MULTIPLIERS:  U =,/,(9X,4D16.8))

      IF(M.EQ.0) GOTO131

      WRITE(IOUT,1310) (G(J),J=1,M)

 1310 FORMAT(8X,26HCONSTRAINT VALUES:  G(X) =,/,(9X,4D16.8))

  131 WRITE(IOUT,1320) (V(I),I=1,N)

 1320 FORMAT(8X,34HDISTANCE FROM LOWER BOUND:  XL-X =,/,(9X,4D16.8))

      WRITE(IOUT,1330) (W(I),I=1,N)

 1330 FORMAT(8X,34HDISTANCE FROM UPPER BOUND:  XU-X =,/,(9X,4D16.8))

      IF(.NOT.LLISE) WRITE(IOUT,1450) ITER

 1450 FORMAT(8X,29HNUMBER OF ITERATIONS:  ITER =,I4)

      WRITE(IOUT,1390) NFUNC

 1390 FORMAT(8X,30HNUMBER OF FUNC-CALLS:  NFUNC =,I4)

      WRITE(IOUT,1340) NGRAD

 1340 FORMAT(8X,30HNUMBER OF GRAD-CALLS:  NGRAD =,I4)

      WRITE(IOUT,1400) NQL

 1400 FORMAT(8X,30HNUMBER OF QL-CALLS:    NQL   =,I4,///)

      RETURN

  127 CONTINUE

C

C   CORRECT PENALTY PARAMETER

C

      IF(LALSD) GOTO87

      WA(1)=DBD

      WA(2)=DEL(N1)

      WA(3)=RPENU

      WA(4)=DBLE(ITER)

      CALL MERIT1(IMERIT+2,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      GOTO99

   87 SUM=ZE

      DO88 I=1,N

   88 SUM=SUM + DPHI(I)*DEL(I) + DABS(U(M+I)*V(I)) + DABS(U(MN+I)*W(I))

      IF(SUM.GT.DSQRT(SQACC)) GOTO99

      DO97 J=1,MNN

   97 RPEN(J)=DMIN1(ZEFACU,RPEN(J)*RPENO)

      CALL MERIT1(IMERIT+4,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

   99 IF(IPRINT.LT.3) GOTO123

      WRITE(IOUT,1260) DBD

 1260 FORMAT(8X,46HPRODUCT OF SEARCH DIRECTION WITH BFGS-MATRIX: ,

     1     8H   DBD =,D13.4)

      WRITE(IOUT,1230) (RPEN(J),J=1,MNN)

 1230 FORMAT(8X,23HPENALTY PARAMETER:  R =,/,(9X,4D16.8))

  123 CONTINUE

C

C   EVALUATION OF MERIT FUNCTION

C

   95 CALL MERIT1(IMERIT+3,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      IF(.NOT.LALSD) CALL MERIT1(IMERIT+4,M,ME,MMAX,N,MNN,NMNN,ACC,

     1     RPEN,F,DF,G,DG,VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      PROD=ZE

      DO44 I=1,N

   44 PROD=PROD + DPHI(I)*DEL(I)

      DO45 J=1,MNN

   45 PROD=PROD + DPHI(J+N)*(U(J) - VMU(J))

      PHIOLD=PHI

      IF(PROD.LT.ZE) GOTO28

      RPMAX=ZE

      DO91 J=1,MNN

      RPEN(J)=RPEN(J)*RPENO

   91 RPMAX=DMAX1(RPMAX,RPEN(J))

      IF(RPMAX.LT.RPENU) GOTO95

      IF(LALSD) GOTO98

      IF(.NOT.LREST.OR.DBD.LT.ACC) GOTO151

      DCL11=DCL(N1,N1)

      IF(DCL11.GE.ZEFACU) GOTO151

      DCL11=DCL11*RPENO

      IF(LQL) DCL11=DCL11*RPENO

      DCL(N1,N1)=DCL11

      GOTO154

   98 CONTINUE

      IFAIL=2

      IF(IPRINT.EQ.0) GOTO90

      WRITE(IOUT,1180) PROD

 1180 FORMAT(8X,44H**SEARCH DIRECTION NOT PROFITABLE:  DPHI*P =,D13.4)

      GOTO90

   28 CONTINUE

      IF(IPRINT.LT.3) GOTO135

      WRITE(IOUT,1350) PROD

 1350 FORMAT(8X,33HPRODUCT LAGRANGIAN GRADIENT WITH ,

     1     24HSEARCH DIRECTION:  DLP =,D13.4)

  135 CONTINUE

C

C   LINE SEARCH

C

      WA(6)=XNM

      WA(7)=DELNM

      LBEST=.FALSE.

      IFLISE=0

   14 IPR=0

      IF(IPRINT.GE.1000) IPR=IPRINT-1000

      IF ((IPRINT.NE.4).OR.(IPRINT.NE.2).OR.(IFLISE.GE.0)) GOTO 281

      WRITE(IOUT,2810) ILINE,ALPHAO,ALPHAM,PHI

  281 CONTINUE

 2810 FORMAT(7X,21H*** LINE SEARCH STEP ,I2,10H: ALPHA = ,D11.4, 

     1     11H: ALPHAM = ,D11.4, 14H, MERIT FCT = ,D13.6)

      CALL LINSEA(ALPHAO,ALPHAM,PHI,PROD,AMUE,BETA,ILINE,MAXFUN,IFLISE,

     1     IPR,WA(6),35,IW,10,ACTIVE(ILWLS),5)

      IF(IFLISE.GT.-2) GOTO192

      LBEST=.TRUE.

      FBEST=F

      IF(M.EQ.0) GOTO193

      DO194 J=1,M

  194 GBEST(J)=G(J)

  193 IF(LLISE) GOTO14

      DO195 I=1,N

      DFBEST(I)=DF(I)

      IF(M.EQ.0) GOTO195

      DO196 J=1,M

  196 DGBEST(J,I)=DG(J,I)

  195 CONTINUE

      GOTO14

  192 CONTINUE

      IF(IFLISE.EQ.1) GOTO10

      IF(IFLISE.GT.1) GOTO15

      DO11 I=1,N

   11 X(I)=XOLD(I) + ALPHAO*DEL(I)

      DO12 J=1,MNN

   12 VMU(J)=VMUOLD(J) + ALPHAO*(U(J) - VMUOLD(J))

      IF(IFLISE.EQ.0) GOTO13

      GOTO197

   15 IFAIL=1000+IFLISE

      IF(IPRINT.EQ.0) GOTO90

      WRITE(IOUT,1160) IFLISE

 1160 FORMAT(8X,32H**ERROR IN LINE SEARCH. IFLISE =,I4)

      GOTO90

   10 IFAIL=4

      IF(IPRINT.EQ.0) GOTO90

      WRITE(IOUT,1130)

 1130 FORMAT(8X,44H**MORE THAN MAXFUN FUNC-CALLS IN LINE SEARCH)

      GOTO90

   13 LLINE=.TRUE.

      IF(IPRINT.LT.3) GOTO117

      IF(ILINE.EQ.1) WRITE(IOUT,1090)

 1090 FORMAT(8X,50HLINE SEARCH SUCCESSFUL AFTER ONE STEP:  ALPHA = 1.)

      IF(ILINE.GT.1)WRITE(IOUT,1170) ILINE,ALPHAO

 1170 FORMAT(8X,28HLINE SEARCH SUCCESSFUL AFTER,I3,7H STEPS:,

     1     9H  ALPHA =,D13.4)

  117 CONTINUE

      IF(.NOT.LBEST.AND.LLISE) GOTO17

      IF(.NOT.LBEST.AND..NOT.LLISE) GOTO18

      F=FBEST

      IF(M.EQ.0) GOTO198

      DO199 J=1,M

  199 G(J)=GBEST(J)

  198 IF(LLISE) GOTO17

      DO201 I=1,N

      DF(I)=DFBEST(I)

      IF(M.EQ.0) GOTO201

      DO202 J=1,M

  202 DG(J,I)=DGBEST(J,I)

  201 CONTINUE

      CALL MERIT1(IMERIT+1,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      GOTO18

  197 CONTINUE

C

C   NEW FUNCTION AND GRADIENT VALUES

C

      IF(.NOT.LRECO) GOTO73

      IFAIL=-1

      RETURN

   73 CONTINUE

      IF (M.EQ.0) GOTO204

      DO 203 J=1,M

  203 ACTIVE(M+J)=.TRUE.

  204 CONTINUE

      CALL NLFUNC(M,ME,MMAX,N,F,G,X,ACTIVE(M+1))

   72 CONTINUE

      IF(LSCF) F=F*SCF

      IF(M.EQ.0.OR..NOT.LSCG) GOTO172

      DO173 J=1,M

  173 G(J)=SCG(J)*G(J)

  172 CONTINUE

      NFUNC=NFUNC+1

      CALL MERIT1(IMERIT+3,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      IF(LLISE.AND..NOT.LLINE) GOTO14

   17 CONTINUE

      CALL MERIT1(IMERIT+1,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      IF(LSCF) F=F/SCF

      IF(M.EQ.0.OR..NOT.LSCG) GOTO174

      DO175 J=1,M

  175 G(J)=G(J)/SCG(J)

  174 CONTINUE

      IF(.NOT.LRECO) GOTO75

      IFAIL=-2

      RETURN

   75 CONTINUE

      CALL NLGRAD(M,ME,MMAX,N,F,G,DF,DG,X,ACTIVE,WA(41))

   74 CONTINUE

      NGRAD=NGRAD+1

      IF(.NOT.LSCF) GOTO176

      F=F*SCF

      DO177 I=1,N

  177 DF(I)=SCF*DF(I)

  176 IF(M.EQ.0.OR..NOT.LSCG) GOTO178

      DO179 J=1,M

      G(J)=G(J)*SCG(J)

      IF(.NOT.ACTIVE(J)) GOTO179

      DO180 I=1,N

  180 DG(J,I)=DG(J,I)*SCG(J)

  179 CONTINUE

  178 CONTINUE

      IF(LLINE) GOTO18

      CALL MERIT1(IMERIT+4,M,ME,MMAX,N,MNN,NMNN,ACC,RPEN,F,DF,G,DG,

     1     VMU,U,X,XL,XU,PHI,DPHI,ACTIVE,WA,4)

      PROD=ZE

      DO190 I=1,N

  190 PROD=PROD + DPHI(I)*DEL(I)

      DO191 J=1,MNN

  191 PROD=PROD + DPHI(N+J)*(U(J) - VMUOLD(J))

      GOTO14

C

C   UPDATE HESSIAN OF LAGRANGIAN

C

   19 EDEL=ZE

      DBD=DBD*ALPHAO*ALPHAO

      DO20 I=1,N

      BDEL(I)=BDEL(I)*ALPHAO

      ETA(I)=DLA(I) - DLAOLD(I)

   20 EDEL=EDEL + ALPHAO*DEL(I)*ETA(I)

      DBD1=DBDFAC*DBD

      IF(EDEL.GE.DBD1) GOTO31

      THETA=(DBD - DBD1)/(DBD - EDEL)

      THETA1=ON - THETA

      DO32 I=1,N

   32 ETA(I)=THETA*ETA(I) + THETA1*BDEL(I)

   33 EDEL=DBD1

   31 CONTINUE

      DBDI=DSQRT(ON/DBD)

      EDELI=DSQRT(ON/EDEL)

C

C   UPDATE FACTORIZATION

C

      DO22 I=1,N

      BDEL(I)=DBDI*BDEL(I)

   22 ETA(I)=EDELI*ETA(I)

      IF(.NOT.LQL) GOTO85

      DO86 I=1,N

      DO86 J=I,N

      DCL(I,J)=DCL(I,J) + ETA(I)*ETA(J) - BDEL(I)*BDEL(J)

   86 DCL(J,I)=DCL(I,J)

      GOTO100

   85 CONTINUE

      CALL LDL001(N,NMAX,DCL,CD,ETA,BDEL)

C

C   CORRECT DATA FOR QL-SOLUTION

C

    2 DO48 I=1,N

      SQD=DSQRT(CD(I))

      IF(SQD.GT.UF) GOTO21

      IFAIL=3

      IF(IPRINT.EQ.0) GOTO90

      WRITE(IOUT,1370)

 1370 FORMAT(8X,26H**UNDERFLOW IN BFGS-UPDATE)

      GOTO90

   21 CONTINUE

      I1=I+1

      IF(I1.GT.N) GOTO48

      DO49 J=I1,N

   49 DCL(I,J)=SQD*DCL(J,I)

   48 DCL(I,I)=SQD

      IF(ITER.EQ.0) GOTO25

C

C   PERFORM NEXT ITERATION

C

      GOTO100

C

C   END OF NLPQL2

C

      END

      SUBROUTINE LDL001(N,NMAX,CL,D,U,V)

C

C********************************************************************

C

C

C   UPDATE CHOLESKEY FACTORIZATION

C

C     L*D*L(T) = L*D*L(T) + U*U(T) - V*V(T)

C

C

C********************************************************************

C

      INTEGER N,NMAX

      DIMENSION CL(NMAX,N),D(N),U(N),V(N)

      INTEGER R,J,J1,I,I1,N1

      DOUBLE PRECISION CL,D,U,V,EPS,T,ON,ZE,FO,PJ,DJ,TNEW,TT,TTI, 

     1     DJNEW,BETA,CLRJ,PDP,SUM,UI

      COMMON/CMACHE/EPS

C

C   CONSTANT DATA

C

      ON=1.D+0

      ZE=0.D+0

      FO=4.D+0

C

C   UPDATE L*D*L(T) + U*U(T)

C

      T=ON

      DO1 J=1,N

      PJ=U(J)

      DJ=D(J)

      TNEW=T + PJ*PJ/DJ

      TT=TNEW/T

      TTI=ON/TT

      DJNEW=DJ*TT

      BETA=PJ/(DJ*TNEW)

      J1=J+1

      IF(J1.GT.N) GOTO5

      IF(DJNEW.LE.FO*DJ) GOTO2

      DO3 R=J1,N

      CLRJ=CL(R,J)

      CL(R,J)=TTI*CLRJ + BETA*U(R)

    3 U(R)=U(R) - PJ*CLRJ

      GOTO5

    2 DO4 R=J1,N

      U(R)=U(R) - PJ*CL(R,J)

    4 CL(R,J)=CL(R,J) + BETA*U(R)

    5 T=TNEW

      D(J)=DJNEW

    1 CONTINUE

C

C   UPDATE L*D*L(T) - V*V(T)

C

      U(1)=V(1)

      PDP=U(1)*U(1)/D(1)

      DO6 I=2,N

      SUM=ZE

      I1=I-1

      DO7 J=1,I1

    7 SUM=SUM + CL(I,J)*U(J)

      UI=V(I) - SUM

      PDP=PDP + UI*UI/D(I)

    6 U(I)=UI

C

      T=ON - PDP

      IF(T.LE.EPS) T=EPS

C

      N1=N+1

      DO8 I=1,N

      J=N1-I

      PJ=U(J)

      DJ=D(J)

      TNEW=T + PJ*PJ/DJ

      TT=T/TNEW

      DJNEW=DJ*TT

      BETA=-PJ/(DJ*T)

      V(J)=PJ

      T=TNEW

      D(J)=DJNEW

      J1=J+1

      IF(J1.GT.N) GOTO8

      DO9 R=J1,N

      CLRJ=CL(R,J)

      CL(R,J)=CLRJ + BETA*V(R)

    9 V(R)=V(R) + PJ*CLRJ

    8 CONTINUE

C

C   END OF LDL001

C

      RETURN

      END

C

      SUBROUTINE LINSEA(ALPHA,ALPHAM,PHI,DPHI,AMUE,BETA,ILINE,MAXFUN,

     1     IFAIL,IPRINT,WA,LWA,KWA,LKWA,LOWA,LLOWA)

C

C*********************************************************************

C

C

C   LINE SEARCH ALGORITHM BASED ON QUADRATIC INTERPOLATION AND ARMIJO-

C   TYPE STOPPING CRITERION.

C

C

C*********************************************************************

C

      INTEGER LWA,LKWA,LLOWA

      DIMENSION WA(LWA),KWA(LKWA),LOWA(LLOWA)

      DOUBLE PRECISION ALPHA,ALPHAM,PHI,DPHI,AMUE,BETA,WA,HA,ON,DIFF 

C

C   INTRINSIC FUNCTION:  DMAX1

C

      INTEGER ILINE,MAXFUN,IFAIL,IPRINT,KWA   

      LOGICAL LOWA

C

C   CONSTANT PARAMETERS

C

      HA=5.D-1

      ON=1.D+0

C

C   START LINE SEARCH

C

      IF(IFAIL.EQ.-1) GOTO1

      WA(1)=PHI

      ILINE=0

      ALPHA=ALPHAM

      WA(2)=ALPHA

    2 ILINE=ILINE+1

      DPHI=DPHI*WA(2)

      IFAIL=-1

      RETURN

    1 IF(ILINE.LT.MAXFUN) GOTO3

      IFAIL=1

      RETURN

    3 DIFF=PHI - WA(1)

      IF(DIFF.GT.AMUE*DPHI) GOTO4

      IFAIL=0

      RETURN

    4 WA(2)=DMAX1(BETA,HA*DPHI/(DPHI - DIFF))

      ALPHA=WA(2)*ALPHA

      GOTO2

C

C   END OF LINSEA

C

      END

      SUBROUTINE MERIT1(MODE,M,ME,MMAX,N,MNN,NMNN,ACC,R,F,DF,G,DG,V,U,

     1     X,XL,XU,PHI,DPHI,ACTIVE,WA,LWA)

C

C*********************************************************************

C

C

C   FUNCTION AND GRADIENT EVALUATIONS OF THE AUGMENTED LAGRANGIAN

C   MERIT FUNCTION, PENALTY PARAMETERS, AND ACTIVE SET

C

C

C*********************************************************************

C

      INTEGER MNN,N,MMAX,NMNN,LWA

      DIMENSION R(MNN),DF(N),G(MMAX),DG(MMAX,N),V(MNN),U(MNN),X(N),

     1     XL(N),XU(N),DPHI(NMNN),ACTIVE(MMAX),WA(LWA)

      DOUBLE PRECISION ACC,R,F,DF,G,DG,V,U,X,XL,XU,PHI,DPHI,WA

      INTEGER MODE,M,ME

      LOGICAL ACTIVE

      DOUBLE PRECISION FACT,RNEW,UAD,PHI1,RMJ,RMI,RMNJ,RMNI,VMI,VMNI,

     1     XMXL,XUMX,SUM,ZE,UF,HA,ON,EPS

C

C INTRINSIC FUNCTIONS:  DSQRT,DBLE,DMIN1,DMAX1

C

      INTEGER ME1,J,MN,I

      COMMON/CMACHE/EPS

C

C   CONSTANT DATA

C

      ZE=0.D+0

      UF=EPS*EPS

      HA=5.D-1

      ON=1.D+0

      GOTO(1,2,3,3,5,6,7,8),MODE

C

C   EVALUATION OF ACTIVE SET

C

    1 IF(M.EQ.0.OR.ME.EQ.M) GOTO10

      ME1=ME+1

      DO11 J=ME1,M

      ACTIVE(J)=.TRUE.

   11 IF(V(J).EQ.ZE.AND.G(J).GT.ACC) ACTIVE(J)=.FALSE.

   10 RETURN

C

C   EVALUATION OF PENALTY PARAMETER

C

    2 FACT=WA(3)

      IF(WA(1).GT.UF) FACT=DBLE(2*M)/(WA(1) + (ON-WA(2)))

      DO20 J=1,MNN

      RNEW=DMIN1(WA(3),FACT*(U(J) - V(J))**2)

      UAD=DMIN1(R(J),WA(4)*DSQRT(R(J)))

   20 R(J)=DMAX1(RNEW,UAD)

      RETURN

C

C   PREPARE EVALUATION OF AUGMENTED LAGRANGIAN

C

    3 IF(M.EQ.0) GOTO30

      DO31 J=1,M

      ACTIVE(M+J)=.TRUE.

   31 IF(J.GT.ME.AND.G(J).GT.V(J)/R(J)) ACTIVE(M+J)=.FALSE.

   30 CONTINUE

      IF(MODE.EQ.4) GOTO4

C

C   EVALUATION OF AUGMENTED LAGRANGIAN VALUE

C

      PHI=F

      IF(M.EQ.0) GOTO32

      DO33 J=1,M

      IF(ACTIVE(M+J)) PHI1=(V(J) - HA*R(J)*G(J))*G(J)

      IF(.NOT.ACTIVE(M+J)) PHI1=HA*V(J)*V(J)/R(J)

   33 PHI=PHI - PHI1

   32 CONTINUE

      MN=M+N

      DO34 I=1,N

      RMJ=R(M+I)

      RMI=ON/RMJ

      RMNJ=R(MN+I)

      RMNI=ON/RMNJ

      VMI=V(M+I)

      VMNI=V(MN+I)

      XMXL=X(I) - XL(I)

      XUMX=XU(I) - X(I)

      IF(XMXL.LE.VMI*RMI) PHI=PHI - (VMI-HA*RMJ*XMXL)*XMXL

      IF(XMXL.GT.VMI*RMI) PHI=PHI - HA*RMI*VMI*VMI

      IF(XUMX.LE.VMNI*RMNI) PHI=PHI - (VMNI-HA*RMNJ*XUMX)*XUMX

      IF(XUMX.GT.VMNI*RMNI) PHI=PHI - HA*RMNI*VMNI*VMNI

   34 CONTINUE

      RETURN

C

C   GRADIENT EVALUATION OF AUGMENTED LAGRANGIAN

C

    4 DO42 I=1,N

      DPHI(I)=DF(I)

      SUM=ZE

      IF(M.EQ.0) GOTO42

      DO43 J=1,M

   43 IF(ACTIVE(M+J)) SUM=SUM + DG(J,I)*(V(J) - R(J)*G(J))

   42 DPHI(I)=DPHI(I) - SUM

      IF(M.EQ.0) GOTO44

      DO45 J=1,M

      IF(ACTIVE(M+J)) DPHI(N+J)=-G(J)

   45 IF(.NOT.ACTIVE(M+J)) DPHI(N+J)=-V(J)/R(J)

   44 CONTINUE

      MN=M+N

      DO46 I=1,N

      VMI=V(M+I)

      VMNI=V(MN+I)

      XMXL=X(I) - XL(I)

      XUMX=XU(I) - X(I)

      RMJ=R(M+I)

      RMI=ON/RMJ

      RMNJ=R(MN+I)

      RMNI=ON/RMNJ

      IF(XMXL.LE.VMI*RMI) DPHI(I)=DPHI(I) - VMI + RMJ*XMXL

      IF(XUMX.LE.VMNI*RMNI) DPHI(I)=DPHI(I) + VMNI - RMNJ*XUMX

      IF(XMXL.LE.VMI*RMI) DPHI(MN+I)=-XMXL

      IF(XMXL.GT.VMI*RMI) DPHI(MN+I)=-VMI*RMI

      IF(XUMX.LE.VMNI*RMNI) DPHI(MNN+I)=-XUMX

      IF(XUMX.GT.VMNI*RMNI) DPHI(MNN+I)=-VMNI*RMNI

   46 CONTINUE

      RETURN

C

C   EVALUATION OF L1-PENALTY FUNCTION

C

C   ACTIVE SET

C

    5 IF(M.EQ.0) GOTO9

      DO12 J=1,M

   12 ACTIVE(J)=.TRUE.

    9 RETURN

C

C   PENALTY PARAMETER

C

    6 DO13 J=1,MNN

      PHI1=DABS(U(J))

   13 R(J)=DMAX1(PHI1,HA*(R(J)+PHI1))

      RETURN

C

C   MERIT FUNCTION VALUE

C

    7 PHI=F

      IF(M.EQ.0) GOTO14

      DO15 J=1,M

      PHI1=DABS(G(J))

      IF(J.GT.ME.AND.G(J).GT.ZE) PHI1=ZE

   15 PHI=PHI + R(J)*PHI1

   14 RETURN

C

C   GRADIENT OF MERIT FUNCTION

C

    8 DO16 J=N,NMNN

   16 DPHI(J)=ZE

      DO17 I=1,N

      DPHI(I)=DF(I)

      IF(M.EQ.0) GOTO17

      PHI1=ZE

      DO18 J=1,M

      IF(G(J).GT.ZE.AND.J.LE.ME) PHI1=PHI1 + R(J)*DG(J,I)

   18 IF(G(J).LE.ZE) PHI1=PHI1 - R(J)*DG(J,I)

      DPHI(I)=DPHI(I) + PHI1

   17 CONTINUE

      RETURN

C

C   END OF MERIT1

C

      END

      SUBROUTINE QL0001(M,ME,MMAX,N,NMAX,MNN,C,D,A,B,XL,XU,

     1      X,U,IOUT,IFAIL,IPRINT,WAR,LWAR,IWAR,LIWAR)

C

C**************************************************************************

C

C

C             SOLUTION OF QUADRATIC PROGRAMMING PROBLEMS

C

C

C

C   QL0001 SOLVES THE QUADRATIC PROGRAMMING PROBLEM

C

C   MINIMIZE        .5*X'*C*X + D'*X

C   SUBJECT TO      A(J)*X  +  B(J)   =  0  ,  J=1,...,ME

C                   A(J)*X  +  B(J)  >=  0  ,  J=ME+1,...,M

C                   XL  <=  X  <=  XU

C   

C   HERE C MUST BE AN N BY N SYMMETRIC AND POSITIVE MATRIX, D AN N-DIMENSIONAL

C   VECTOR, A AN M BY N MATRIX AND B AN M-DIMENSIONAL VECTOR. THE ABOVE 

C   SITUATION IS INDICATED BY IWAR(1)=1. ALTERNATIVELY, I.E. IF IWAR(1)=0,

C   THE OBJECTIVE FUNCTION MATRIX CAN ALSO BE PROVIDED IN FACTORIZED FORM.

C   IN THIS CASE, C IS AN UPPER TRIANGULAR MATRIX.

C

C   THE SUBROUTINE REORGANIZES SOME DATA SO THAT THE PROBLEM CAN BE SOLVED

C   BY A MODIFICATION OF AN ALGORITHM PROPOSED BY POWELL (1983).

C

C

C   USAGE:

C

C      QL0001(M,ME,MMAX,N,NMAX,MNN,C,D,A,B,XL,XU,X,U,IOUT,IFAIL,IPRINT,

C             WAR,LWAR,IWAR,LIWAR)

C

C

C   DEFINITION OF THE PARAMETERS:

C

C   M :        TOTAL NUMBER OF CONSTRAINTS.

C   ME :       NUMBER OF EQUALITY CONSTRAINTS.

C   MMAX :     ROW DIMENSION OF A. MMAX MUST BE AT LEAST ONE AND GREATER

C              THAN M.

C   N :        NUMBER OF VARIABLES.

C   NMAX :     ROW DIMENSION OF C. NMAX MUST BE GREATER OR EQUAL TO N.

C   MNN :      MUST BE EQUAL TO M + N + N.

C   C(NMAX,NMAX): OBJECTIVE FUNCTION MATRIX WHICH SHOULD BE SYMMETRIC AND

C              POSITIVE DEFINITE. IF IWAR(1) = 0, C IS SUPPOSED TO BE THE

C              CHOLESKEY-FACTOR OF ANOTHER MATRIX, I.E. C IS UPPER

C              TRIANGULAR.

C   D(NMAX) :  CONTAINS THE CONSTANT VECTOR OF THE OBJECTIVE FUNCTION.

C   A(MMAX,NMAX): CONTAINS THE DATA MATRIX OF THE LINEAR CONSTRAINTS.

C   B(MMAX) :  CONTAINS THE CONSTANT DATA OF THE LINEAR CONSTRAINTS.

C   XL(N),XU(N): CONTAIN THE LOWER AND UPPER BOUNDS FOR THE VARIABLES.

C   X(N) :     ON RETURN, X CONTAINS THE OPTIMAL SOLUTION VECTOR.

C   U(MNN) :   ON RETURN, U CONTAINS THE LAGRANGE MULTIPLIERS. THE FIRST

C              M POSITIONS ARE RESERVED FOR THE MULTIPLIERS OF THE M

C              LINEAR CONSTRAINTS AND THE SUBSEQUENT ONES FOR THE 

C              MULTIPLIERS OF THE LOWER AND UPPER BOUNDS. ON SUCCESSFUL

C              TERMINATION, ALL VALUES OF U WITH RESPECT TO INEQUALITIES 

C              AND BOUNDS SHOULD BE GREATER OR EQUAL TO ZERO.

C   IOUT :     INTEGER INDICATING THE DESIRED OUTPUT UNIT NUMBER, I.E.

C              ALL WRITE-STATEMENTS START WITH 'WRITE(IOUT,... '.

C   IFAIL :    SHOWS THE TERMINATION REASON.

C      IFAIL = 0 :   SUCCESSFUL RETURN.

C      IFAIL = 1 :   TOO MANY ITERATIONS (MORE THAN 40*(N+M)).

C      IFAIL = 2 :   ACCURACY INSUFFICIENT TO SATISFY CONVERGENCE

C                    CRITERION.

C      IFAIL = 5 :   LENGTH OF A WORKING ARRAY IS TOO SHORT.

C      IFAIL > 10 :  THE CONSTRAINTS ARE INCONSISTENT.

C   IPRINT :   OUTPUT CONTROL.

C      IPRINT = 0 :  NO OUTPUT OF QL0001.

C      IPRINT > 0 :  BRIEF OUTPUT IN ERROR CASES.

C   WAR(LWAR) : REAL WORKING ARRAY. THE LENGTH LWAR SHOULD BE GRATER THAN 

C               3*NMAX*NMAX/2 + 10*NMAX + 2*MMAX.

C   IWAR(LIWAR): INTEGER WORKING ARRAY. THE LENGTH LIWAR SHOULD BE AT 

C               LEAST N.

C

C   A NAMED COMMON-BLOCK  /CMACHE/EPS   MUST BE PROVIDED BY THE USER,

C   WHERE EPS DEFINES A GUESS FOR THE UNDERLYING MACHINE PRECISION.

C

C

C   AUTHOR:    K. SCHITTKOWSKI,

C              MATHEMATISCHES INSTITUT,

C              UNIVERSITAET BAYREUTH,

C              8580 BAYREUTH,

C              GERMANY, F.R.

C

C

C   VERSION:   1.4  (MARCH, 1987)  

C

C

C*********************************************************************

C

C

      INTEGER NMAX,MMAX,N,MNN,LWAR,LIWAR

      DIMENSION C(NMAX,NMAX),D(NMAX),A(MMAX,NMAX),B(MMAX),

     1      XL(N),XU(N),X(N),U(MNN),WAR(LWAR),IWAR(LIWAR)

      DOUBLE PRECISION C,D,A,B,X,XL,XU,U,WAR,DIAG,ZERO,

     1      EPS,QPEPS,TEN

      INTEGER M,ME,IOUT,IFAIL,IPRINT,IWAR,INW1,INW2,IN,J,LW,MN,I,         

     1      IDIAG,INFO,NACT,MAXIT

      LOGICAL LQL

C

C     INTRINSIC FUNCTIONS:  DSQRT

C

      COMMON /CMACHE/EPS

C

C     CONSTANT DATA

C

      LQL=.FALSE.

      IF (IWAR(1).EQ.1) LQL=.TRUE.

      ZERO=0.0D+0

      TEN=1.D+1

      MAXIT=40*(M+N)

      QPEPS=EPS

C      IF (N.GT.100)  QPEPS=QPEPS*TEN

C      IF (N.GT.200)  QPEPS=QPEPS*TEN

C      IF (N.GT.300)  QPEPS=QPEPS*TEN

C      IF (N.GT.500)  QPEPS=QPEPS*TEN

C      IF (M.GT.200)  QPEPS=QPEPS*TEN

C      IF (M.GT.400)  QPEPS=QPEPS*TEN

C      IF (QPEPS.GT.DSQRT(EPS)) QPEPS=DSQRT(EPS)

      INW1=1

      INW2=INW1+MMAX

C

C     PREPARE PROBLEM DATA FOR EXECUTION

C

      IF (M.LE.0) GOTO 20

      IN=INW1

      DO 10 J=1,M

      WAR(IN)=-B(J)

   10 IN=IN+1

   20 LW=3*NMAX*NMAX/2 + 10*NMAX + M

      IF ((INW2+LW).GT.LWAR) GOTO 80

      IF (LIWAR.LT.N) GOTO 81

      IF (MNN.LT.M+N+N) GOTO 82

      MN=M+N

C

C     CALL OF QL0002

C

      CALL QL0002(N,M,ME,MMAX,MN,MNN,NMAX,LQL,A,WAR(INW1),

     1    D,C,XL,XU,X,NACT,IWAR,MAXIT,QPEPS,INFO,DIAG,

     2    WAR(INW2),LW)

C

C     TEST OF MATRIX CORRECTIONS

C

      IFAIL=0

      IF (INFO.EQ.1) GOTO 40

      IF (INFO.EQ.2) GOTO 90

      IDIAG=0

      IF ((DIAG.GT.ZERO).AND.(DIAG.LT.1000.0)) IDIAG=DIAG

      IF ((IPRINT.GT.0).AND.(IDIAG.GT.0))

     1   WRITE(IOUT,1000) IDIAG

      IF (INFO .LT. 0) GOTO  70

C

C     REORDER MULTIPLIER

C

      DO  50 J=1,MNN

   50 U(J)=ZERO

      IN=INW2-1

      IF (NACT.EQ.0) GOTO 30

      DO  60 I=1,NACT

      J=IWAR(I)

      U(J)=WAR(IN+I)

   60 CONTINUE

   30 CONTINUE

      RETURN

C

C     ERROR MESSAGES

C

   70 IFAIL=-INFO+10

      IF ((IPRINT.GT.0).AND.(NACT.GT.0))

     1     WRITE(IOUT,1100) -INFO,(IWAR(I),I=1,NACT)

      RETURN

   80 IFAIL=5

      IF (IPRINT .GT. 0) WRITE(IOUT,1200)

      RETURN

   81 IFAIL=5

      IF (IPRINT .GT. 0) WRITE(IOUT,1210)

      RETURN

   82 IFAIL=5

      IF (IPRINT .GT. 0) WRITE(IOUT,1220)

      RETURN

   40 IFAIL=1

      IF (IPRINT.GT.0) WRITE(IOUT,1300) MAXIT

      RETURN

   90 IFAIL=2

      IF (IPRINT.GT.0) WRITE(IOUT,1400) 

      RETURN

C

C     FORMAT-INSTRUCTIONS

C

 1000 FORMAT(/8X,28H***QL: MATRIX G WAS ENLARGED,I3,

     1        20H-TIMES BY UNITMATRIX)

 1100 FORMAT(/8X,18H***QL: CONSTRAINT ,I5,

     1        19H NOT CONSISTENT TO ,/,(10X,10I5))

 1200 FORMAT(/8X,21H***QL: LWAR TOO SMALL)

 1210 FORMAT(/8X,22H***QL: LIWAR TOO SMALL)

 1220 FORMAT(/8X,20H***QL: MNN TOO SMALL)

 1300 FORMAT(/8X,37H***QL: TOO MANY ITERATIONS (MORE THAN,I6,1H))

 1400 FORMAT(/8X,50H***QL: ACCURACY INSUFFICIENT TO ATTAIN CONVERGENCE) 

      END

C

      SUBROUTINE QL0002(N,M,MEQ,MMAX,MN,MNN,NMAX,LQL,A,B,GRAD,G,

     1      XL,XU,X,NACT,IACT,MAXIT,VSMALL,INFO,DIAG,W,LW)

C

C**************************************************************************

C

C

C   THIS SUBROUTINE SOLVES THE QUADRATIC PROGRAMMING PROBLEM 

C

C       MINIMIZE      GRAD'*X  +  0.5 * X*G*X

C       SUBJECT TO    A(K)*X  =  B(K)   K=1,2,...,MEQ,

C                     A(K)*X >=  B(K)   K=MEQ+1,...,M,

C                     XL  <=  X  <=  XU

C

C   THE QUADRATIC PROGRAMMING METHOD PROCEEDS FROM AN INITIAL CHOLESKY-

C   DECOMPOSITION OF THE OBJECTIVE FUNCTION MATRIX, TO CALCULATE THE

C   UNIQUELY DETERMINED MINIMIZER OF THE UNCONSTRAINED PROBLEM. 

C   SUCCESSIVELY ALL VIOLATED CONSTRAINTS ARE ADDED TO A WORKING SET 

C   AND A MINIMIZER OF THE OBJECTIVE FUNCTION SUBJECT TO ALL CONSTRAINTS 

C   IN THIS WORKING SET IS COMPUTED. IT IS POSSIBLE THAT CONSTRAINTS

C   HAVE TO LEAVE THE WORKING SET.

C

C

C   DESCRIPTION OF PARAMETERS:    

C

C     N        : IS THE NUMBER OF VARIABLES.

C     M        : TOTAL NUMBER OF CONSTRAINTS.

C     MEQ      : NUMBER OF EQUALITY CONTRAINTS.

C     MMAX     : ROW DIMENSION OF A, DIMENSION OF B. MMAX MUST BE AT

C                LEAST ONE AND GREATER OR EQUAL TO M.

C     MN       : MUST BE EQUAL M + N.

C     MNN      : MUST BE EQUAL M + N + N.

C     NMAX     : ROW DIEMSION OF G. MUST BE AT LEAST N.

C     LQL      : DETERMINES INITIAL DECOMPOSITION.

C        LQL = .FALSE.  : THE UPPER TRIANGULAR PART OF THE MATRIX G

C                         CONTAINS INITIALLY THE CHOLESKY-FACTOR OF A SUITABLE 

C                         DECOMPOSITION.

C        LQL = .TRUE.   : THE INITIAL CHOLESKY-FACTORISATION OF G IS TO BE

C                         PERFORMED BY THE ALGORITHM.

C     A(MMAX,NMAX) : A IS A MATRIX WHOSE COLUMNS ARE THE CONSTRAINTS NORMALS.

C     B(MMAX)  : CONTAINS THE RIGHT HAND SIDES OF THE CONSTRAINTS.

C     GRAD(N)  : CONTAINS THE OBJECTIVE FUNCTION VECTOR GRAD.

C     G(NMAX,N): CONTAINS THE SYMMETRIC OBJECTIVE FUNCTION MATRIX.

C     XL(N), XU(N): CONTAIN THE LOWER AND UPPER BOUNDS FOR X.

C     X(N)     : VECTOR OF VARIABLES.

C     NACT     : FINAL NUMBER OF ACTIVE CONSTRAINTS.

C     IACT(K) (K=1,2,...,NACT): INDICES OF THE FINAL ACTIVE CONSTRAINTS.

C     INFO     : REASON FOR THE RETURN FROM THE SUBROUTINE.

C         INFO = 0 : CALCULATION WAS TERMINATED SUCCESSFULLY.

C         INFO = 1 : MAXIMUM NUMBER OF ITERATIONS ATTAINED.

C         INFO = 2 : ACCURACY IS INSUFFICIENT TO MAINTAIN INCREASING

C                    FUNCTION VALUES.

C         INFO < 0 : THE CONSTRAINT WITH INDEX ABS(INFO) AND THE CON-

C                    STRAINTS WHOSE INDICES ARE IACT(K), K=1,2,...,NACT,

C                    ARE INCONSISTENT.

C     MAXIT    : MAXIMUM NUMBER OF ITERATIONS.

C     VSMALL   : REQUIRED ACCURACY TO BE ACHIEVED (E.G. IN THE ORDER OF THE 

C                MACHINE PRECISION FOR SMALL AND WELL-CONDITIONED PROBLEMS).

C     DIAG     : ON RETURN DIAG IS EQUAL TO THE MULTIPLE OF THE UNIT MATRIX

C                THAT WAS ADDED TO G TO ACHIEVE POSITIVE DEFINITENESS.

C     W(LW)    : THE ELEMENTS OF W(.) ARE USED FOR WORKING SPACE. THE LENGTH

C                OF W MUST NOT BE LESS THAN (1.5*NMAX*NMAX + 10*NMAX + M).

C                WHEN INFO = 0 ON RETURN, THE LAGRANGE MULTIPLIERS OF THE

C                FINAL ACTIVE CONSTRAINTS ARE HELD IN W(K), K=1,2,...,NACT.

C   THE VALUES OF N, M, MEQ, MMAX, MN, MNN AND NMAX AND THE ELEMENTS OF

C   A, B, GRAD AND G ARE NOT ALTERED.

C

C   THE FOLLOWING INTEGERS ARE USED TO PARTITION W:

C     THE FIRST N ELEMENTS OF W HOLD LAGRANGE MULTIPLIER ESTIMATES.

C     W(IWZ+I+(N-1)*J) HOLDS THE MATRIX ELEMENT Z(I,J).

C     W(IWR+I+0.5*J*(J-1)) HOLDS THE UPPER TRIANGULAR MATRIX

C       ELEMENT R(I,J). THE SUBSEQUENT N COMPONENTS OF W MAY BE

C       TREATED AS AN EXTRA COLUMN OF R(.,.).

C     W(IWW-N+I) (I=1,2,...,N) ARE USED FOR TEMPORARY STORAGE.

C     W(IWW+I) (I=1,2,...,N) ARE USED FOR TEMPORARY STORAGE.

C     W(IWD+I) (I=1,2,...,N) HOLDS G(I,I) DURING THE CALCULATION.

C     W(IWX+I) (I=1,2,...,N) HOLDS VARIABLES THAT WILL BE USED TO

C       TEST THAT THE ITERATIONS INCREASE THE OBJECTIVE FUNCTION.

C     W(IWA+K) (K=1,2,...,M) USUALLY HOLDS THE RECIPROCAL OF THE

C       LENGTH OF THE K-TH CONSTRAINT, BUT ITS SIGN INDICATES

C       WHETHER THE CONSTRAINT IS ACTIVE.

C

C   

C   AUTHOR:    K. SCHITTKOWSKI,

C              MATHEMATISCHES INSTITUT,

C              UNIVERSITAET BAYREUTH,

C              8580 BAYREUTH,

C              GERMANY, F.R.

C

C   AUTHOR OF ORIGINAL VERSION:

C              M.J.D. POWELL, DAMTP,

C              UNIVERSITY OF CAMBRIDGE, SILVER STREET

C              CAMBRIDGE,

C              ENGLAND

C

C

C   REFERENCE: M.J.D. POWELL: ZQPCVX, A FORTRAN SUBROUTINE FOR CONVEX

C              PROGRAMMING, REPORT DAMTP/1983/NA17, UNIVERSITY OF

C              CAMBRIDGE, ENGLAND, 1983.

C

C

C   VERSION :  2.0 (MARCH, 1987)

C

C

C*************************************************************************

C

      INTEGER MMAX,NMAX,N,LW

      DIMENSION A(MMAX,NMAX),B(MMAX),GRAD(N),G(NMAX,N),X(N),IACT(N),

     1      W(LW),XL(N),XU(N)

      INTEGER M,MEQ,MN,MNN,NACT,IACT,INFO,MAXIT

      DOUBLE PRECISION CVMAX,DIAG,DIAGR,FDIFF,FDIFFA,GA,GB,PARINC,PARNEW       

     1      ,RATIO,RES,STEP,SUM,SUMX,SUMY,SUMA,SUMB,SUMC,TEMP,TEMPA,

     2       VSMALL,XMAG,XMAGR,ZERO,ONE,TWO,ONHA,VFACT

      DOUBLE PRECISION A,B,G,GRAD,W,X,XL,XU

C

C   INTRINSIC FUNCTIONS:   DMAX1,DSQRT,DABS,DMIN1

C

      INTEGER IWZ,IWR,IWW,IWD,IWA,IFINC,KFINC,K,I,IA,ID,II,IR,IRA,

     1     IRB,J,NM,IZ,IZA,ITERC,ITREF,JFINC,IFLAG,IWS,IS,K1,IW,KK,IP,

     2     IPP,IL,IU,JU,KFLAG,LFLAG,JFLAG,KDROP,NU,MFLAG,KNEXT,IX,IWX,

     3     IWY,IY,JL

      LOGICAL LQL,LOWER

C

C   INITIAL ADDRESSES

C

      IWZ=NMAX

      IWR=IWZ+NMAX*NMAX

      IWW=IWR+(NMAX*(NMAX+3))/2

      IWD=IWW+NMAX

      IWX=IWD+NMAX

      IWA=IWX+NMAX

C

C     SET SOME CONSTANTS.

C

      ZERO=0.D+0

      ONE=1.D+0

      TWO=2.D+0

      ONHA=1.5D+0

      VFACT=1.D+0

C

C     SET SOME PARAMETERS.

C     NUMBER LESS THAN VSMALL ARE ASSUMED TO BE NEGLIGIBLE.

C     THE MULTIPLE OF I THAT IS ADDED TO G IS AT MOST DIAGR TIMES

C       THE LEAST MULTIPLE OF I THAT GIVES POSITIVE DEFINITENESS.

C     X IS RE-INITIALISED IF ITS MAGNITUDE IS REDUCED BY THE

C       FACTOR XMAGR.

C     A CHECK IS MADE FOR AN INCREASE IN F EVERY IFINC ITERATIONS,

C       AFTER KFINC ITERATIONS ARE COMPLETED.

C

      DIAGR=TWO

      XMAGR=1.0D-2

      IFINC=3

      KFINC=MAX0(10,N)

C

C     FIND THE RECIPROCALS OF THE LENGTHS OF THE CONSTRAINT NORMALS.

C     RETURN IF A CONSTRAINT IS INFEASIBLE DUE TO A ZERO NORMAL.

C

      NACT=0

      IF (M .LE. 0) GOTO 45

      DO 40 K=1,M

      SUM=ZERO

      DO 10 I=1,N

   10 SUM=SUM+A(K,I)**2

      IF (SUM .GT. ZERO) GOTO 20

      IF (B(K) .EQ. ZERO) GOTO 30

      INFO=-K

      IF (K .LE. MEQ) GOTO 730

      IF (B(K)) 30,30,730

   20 SUM=ONE/DSQRT(SUM)

   30 IA=IWA+K

   40 W(IA)=SUM

   45 DO 50 K=1,N

      IA=IWA+M+K

   50 W(IA)=ONE

C

C     IF NECESSARY INCREASE THE DIAGONAL ELEMENTS OF G.

C

      IF (.NOT. LQL) GOTO 165

      DIAG=ZERO

      DO 60 I=1,N

      ID=IWD+I

      W(ID)=G(I,I)

      DIAG=DMAX1(DIAG,VSMALL-W(ID))

      IF (I .EQ. N) GOTO 60

      II=I+1

      DO 55 J=II,N

      GA=-DMIN1(W(ID),G(J,J))

      GB=DABS(W(ID)-G(J,J))+DABS(G(I,J))

      IF (GB .GT. ZERO) GA=GA+G(I,J)**2/GB

   55 DIAG=DMAX1(DIAG,GA)

   60 CONTINUE

      IF (DIAG .LE. ZERO) GOTO 90

   70 DIAG=DIAGR*DIAG

      DO 80 I=1,N

      ID=IWD+I

   80 G(I,I)=DIAG+W(ID)

C

C     FORM THE CHOLESKY FACTORISATION OF G. THE TRANSPOSE

C     OF THE FACTOR WILL BE PLACED IN THE R-PARTITION OF W.

C

   90 IR=IWR

      DO 130 J=1,N

      IRA=IWR

      IRB=IR+1

      DO 120 I=1,J

      TEMP=G(I,J)

      IF (I .EQ. 1) GOTO 110

      DO 100 K=IRB,IR

      IRA=IRA+1

  100 TEMP=TEMP-W(K)*W(IRA)

  110 IR=IR+1

      IRA=IRA+1

      IF (I .LT. J) W(IR)=TEMP/W(IRA)

  120 CONTINUE

      IF (TEMP .LT. VSMALL) GOTO 140

  130 W(IR)=DSQRT(TEMP)

      GOTO 170

C

C     INCREASE FURTHER THE DIAGONAL ELEMENT OF G.

C

  140 W(J)=ONE

      SUMX=ONE

      K=J

  150 SUM=ZERO

      IRA=IR-1

      DO 160 I=K,J

      SUM=SUM-W(IRA)*W(I)

  160 IRA=IRA+I

      IR=IR-K

      K=K-1

      W(K)=SUM/W(IR)

      SUMX=SUMX+W(K)**2

      IF (K .GE. 2) GOTO 150

      DIAG=DIAG+VSMALL-TEMP/SUMX

      GOTO 70

C

C     STORE THE CHOLESKY FACTORISATION IN THE R-PARTITION

C     OF W.

C

  165 IR=IWR

      DO 166 I=1,N

      DO 166 J=1,I

      IR=IR+1

  166 W(IR)=G(J,I)

C

C     SET Z THE INVERSE OF THE MATRIX IN R.

C

  170 NM=N-1

      DO 220 I=1,N

      IZ=IWZ+I

      IF (I .EQ. 1) GOTO 190

      DO 180 J=2,I

      W(IZ)=ZERO

  180 IZ=IZ+N

  190 IR=IWR+(I+I*I)/2

      W(IZ)=ONE/W(IR)

      IF (I .EQ. N) GOTO 220

      IZA=IZ

      DO 210 J=I,NM

      IR=IR+I

      SUM=ZERO

      DO 200 K=IZA,IZ,N

      SUM=SUM+W(K)*W(IR)

  200 IR=IR+1

      IZ=IZ+N

  210 W(IZ)=-SUM/W(IR)

  220 CONTINUE

C

C     SET THE INITIAL VALUES OF SOME VARIABLES.

C     ITERC COUNTS THE NUMBER OF ITERATIONS.

C     ITREF IS SET TO ONE WHEN ITERATIVE REFINEMENT IS REQUIRED.

C     JFINC INDICATES WHEN TO TEST FOR AN INCREASE IN F.

C

      ITERC=1

      ITREF=0

      JFINC=-KFINC

C

C     SET X TO ZERO AND SET THE CORRESPONDING RESIDUALS OF THE

C     KUHN-TUCKER CONDITIONS.

C

  230 IFLAG=1

      IWS=IWW-N

      DO 240 I=1,N

      X(I)=ZERO

      IW=IWW+I

      W(IW)=GRAD(I)

      IF (I .GT. NACT) GOTO 240

      W(I)=ZERO

      IS=IWS+I

      K=IACT(I)

      IF (K .LE. M) GOTO 235

      IF (K .GT. MN) GOTO 234

      K1=K-M

      W(IS)=XL(K1)

      GOTO 240

  234 K1=K-MN

      W(IS)=-XU(K1)

      GOTO 240

  235 W(IS)=B(K)

  240 CONTINUE

      XMAG=ZERO

      VFACT=1.D+0

      IF (NACT) 340,340,280

C

C     SET THE RESIDUALS OF THE KUHN-TUCKER CONDITIONS FOR GENERAL X.

C

  250 IFLAG=2

      IWS=IWW-N

      DO 260 I=1,N

      IW=IWW+I

      W(IW)=GRAD(I)

      IF (LQL) GOTO 259

      ID=IWD+I

      W(ID)=ZERO

      DO 251 J=I,N

  251 W(ID)=W(ID)+G(I,J)*X(J)

      DO 252 J=1,I

      ID=IWD+J

  252 W(IW)=W(IW)+G(J,I)*W(ID)

      GOTO 260

  259 DO 261 J=1,N

  261 W(IW)=W(IW)+G(I,J)*X(J)

  260 CONTINUE

      IF (NACT .EQ. 0) GOTO 340

      DO 270 K=1,NACT

      KK=IACT(K)

      IS=IWS+K

      IF (KK .GT. M) GOTO 265

      W(IS)=B(KK)

      DO 264 I=1,N

      IW=IWW+I

      W(IW)=W(IW)-W(K)*A(KK,I)

  264 W(IS)=W(IS)-X(I)*A(KK,I)

      GOTO 270

  265 IF (KK .GT. MN) GOTO 266

      K1=KK-M

      IW=IWW+K1

      W(IW)=W(IW)-W(K)

      W(IS)=XL(K1)-X(K1)

      GOTO 270

  266 K1=KK-MN

      IW=IWW+K1

      W(IW)=W(IW)+W(K)

      W(IS)=-XU(K1)+X(K1)

  270 CONTINUE

C

C     PRE-MULTIPLY THE VECTOR IN THE S-PARTITION OF W BY THE

C     INVERS OF R TRANSPOSE.

C

  280 IR=IWR

      IP=IWW+1

      IPP=IWW+N

      IL=IWS+1

      IU=IWS+NACT

      DO 310 I=IL,IU

      SUM=ZERO

      IF (I .EQ. IL) GOTO 300

      JU=I-1

      DO 290 J=IL,JU

      IR=IR+1

  290 SUM=SUM+W(IR)*W(J)

  300 IR=IR+1

  310 W(I)=(W(I)-SUM)/W(IR)

C

C     SHIFT X TO SATISFY THE ACTIVE CONSTRAINTS AND MAKE THE

C     CORRESPONDING CHANGE TO THE GRADIENT RESIDUALS.

C

      DO 330 I=1,N

      IZ=IWZ+I

      SUM=ZERO

      DO 320 J=IL,IU

      SUM=SUM+W(J)*W(IZ)

  320 IZ=IZ+N

      X(I)=X(I)+SUM

      IF (LQL) GOTO 329

      ID=IWD+I

      W(ID)=ZERO

      DO 321 J=I,N

  321 W(ID)=W(ID)+G(I,J)*SUM

      IW=IWW+I

      DO 322 J=1,I

      ID=IWD+J

  322 W(IW)=W(IW)+G(J,I)*W(ID)

      GOTO 330

  329 DO 331 J=1,N

      IW=IWW+J

  331 W(IW)=W(IW)+SUM*G(I,J)

  330 CONTINUE

C

C     FORM THE SCALAR PRODUCT OF THE CURRENT GRADIENT RESIDUALS

C     WITH EACH COLUMN OF Z.

C

  340 KFLAG=1

      GOTO 930

  350 IF (NACT .EQ. N) GOTO 380

C

C     SHIFT X SO THAT IT SATISFIES THE REMAINING KUHN-TUCKER

C     CONDITIONS.

C

      IL=IWS+NACT+1

      IZA=IWZ+NACT*N

      DO 370 I=1,N

      SUM=ZERO

      IZ=IZA+I

      DO 360 J=IL,IWW

      SUM=SUM+W(IZ)*W(J)

  360 IZ=IZ+N

  370 X(I)=X(I)-SUM

      INFO=0

      IF (NACT .EQ. 0) GOTO 410

C

C     UPDATE THE LAGRANGE MULTIPLIERS.

C

  380 LFLAG=3

      GOTO 740

  390 DO 400 K=1,NACT

      IW=IWW+K

  400 W(K)=W(K)+W(IW)

C

C     REVISE THE VALUES OF XMAG.

C     BRANCH IF ITERATIVE REFINEMENT IS REQUIRED.

C

  410 JFLAG=1

      GOTO 910

  420 IF (IFLAG .EQ. ITREF) GOTO 250

C

C     DELETE A CONSTRAINT IF A LAGRANGE MULTIPLIER OF AN

C     INEQUALITY CONSTRAINT IS NEGATIVE.

C

      KDROP=0

      GOTO 440

  430 KDROP=KDROP+1

      IF (W(KDROP) .GE. ZERO) GOTO 440

      IF (IACT(KDROP) .LE. MEQ) GOTO 440

      NU=NACT

      MFLAG=1

      GOTO 800

  440 IF (KDROP .LT. NACT) GOTO 430

C

C     SEEK THE GREATEAST NORMALISED CONSTRAINT VIOLATION, DISREGARDING

C     ANY THAT MAY BE DUE TO COMPUTER ROUNDING ERRORS.

C

  450 CVMAX=ZERO

      IF (M .LE. 0) GOTO 481

      DO 480 K=1,M

      IA=IWA+K

      IF (W(IA) .LE. ZERO) GOTO 480

      SUM=-B(K)

      DO 460 I=1,N

  460 SUM=SUM+X(I)*A(K,I)

      SUMX=-SUM*W(IA)

      IF (K .LE. MEQ) SUMX=DABS(SUMX)

      IF (SUMX .LE. CVMAX) GOTO 480

      TEMP=DABS(B(K))

      DO 470 I=1,N

  470 TEMP=TEMP+DABS(X(I)*A(K,I))

      TEMPA=TEMP+DABS(SUM)

      IF (TEMPA .LE. TEMP) GOTO 480

      TEMP=TEMP+ONHA*DABS(SUM)

      IF (TEMP .LE. TEMPA) GOTO 480

      CVMAX=SUMX

      RES=SUM

      KNEXT=K

  480 CONTINUE

  481 DO 485 K=1,N

      LOWER=.TRUE.

      IA=IWA+M+K

      IF (W(IA) .LE. ZERO) GOTO 485

      SUM=XL(K)-X(K)

      IF (SUM) 482,485,483

  482 SUM=X(K)-XU(K)

      LOWER=.FALSE.

  483 IF (SUM .LE. CVMAX) GOTO 485

      CVMAX=SUM

      RES=-SUM

      KNEXT=K+M

      IF (LOWER) GOTO 485

      KNEXT=K+MN

  485 CONTINUE

C

C     TEST FOR CONVERGENCE

C

      INFO=0

      IF (CVMAX .LE. VSMALL) GOTO 700

C

C     RETURN IF, DUE TO ROUNDING ERRORS, THE ACTUAL CHANGE IN

C     X MAY NOT INCREASE THE OBJECTIVE FUNCTION

C

      JFINC=JFINC+1

      IF (JFINC .EQ. 0) GOTO 510

      IF (JFINC .NE. IFINC) GOTO 530

      FDIFF=ZERO

      FDIFFA=ZERO

      DO 500 I=1,N

      SUM=TWO*GRAD(I)

      SUMX=DABS(SUM)

      IF (LQL) GOTO 489

      ID=IWD+I

      W(ID)=ZERO

      DO 486 J=I,N

      IX=IWX+J

  486 W(ID)=W(ID)+G(I,J)*(W(IX)+X(J))

      DO 487 J=1,I

      ID=IWD+J

      TEMP=G(J,I)*W(ID)

      SUM=SUM+TEMP

  487 SUMX=SUMX+DABS(TEMP)

      GOTO 495

  489 DO 490 J=1,N

      IX=IWX+J

      TEMP=G(I,J)*(W(IX)+X(J))

      SUM=SUM+TEMP

  490 SUMX=SUMX+DABS(TEMP)

  495 IX=IWX+I

      FDIFF=FDIFF+SUM*(X(I)-W(IX))

  500 FDIFFA=FDIFFA+SUMX*DABS(X(I)-W(IX))

      INFO=2

      SUM=FDIFFA+FDIFF

      IF (SUM .LE. FDIFFA) GOTO 700

      TEMP=FDIFFA+ONHA*FDIFF

      IF (TEMP .LE. SUM) GOTO 700

      JFINC=0

      INFO=0

  510 DO 520 I=1,N

      IX=IWX+I

  520 W(IX)=X(I)

C

C     FORM THE SCALAR PRODUCT OF THE NEW CONSTRAINT NORMAL WITH EACH

C     COLUMN OF Z. PARNEW WILL BECOME THE LAGRANGE MULTIPLIER OF

C     THE NEW CONSTRAINT.

C

  530 ITERC=ITERC+1

      IF (ITERC.LE.MAXIT) GOTO 531

      INFO=1

      GOTO 710

  531 CONTINUE

      IWS=IWR+(NACT+NACT*NACT)/2

      IF (KNEXT .GT. M) GOTO 541

      DO 540 I=1,N

      IW=IWW+I

  540 W(IW)=A(KNEXT,I)

      GOTO 549

  541 DO 542 I=1,N

      IW=IWW+I

  542 W(IW)=ZERO

      K1=KNEXT-M

      IF (K1 .GT. N) GOTO 545

      IW=IWW+K1

      W(IW)=ONE

      IZ=IWZ+K1

      DO 543 I=1,N

      IS=IWS+I

      W(IS)=W(IZ)

  543 IZ=IZ+N

      GOTO 550

  545 K1=KNEXT-MN

      IW=IWW+K1

      W(IW)=-ONE

      IZ=IWZ+K1

      DO 546 I=1,N

      IS=IWS+I

      W(IS)=-W(IZ)

  546 IZ=IZ+N

      GOTO 550

  549 KFLAG=2

      GOTO 930

  550 PARNEW=ZERO

C

C     APPLY GIVENS ROTATIONS TO MAKE THE LAST (N-NACT-2) SCALAR

C     PRODUCTS EQUAL TO ZERO.

C

      IF (NACT .EQ. N) GOTO 570

      NU=N

      NFLAG=1

      GOTO 860

C

C     BRANCH IF THERE IS NO NEED TO DELETE A CONSTRAINT.

C

  560 IS=IWS+NACT

      IF (NACT .EQ. 0) GOTO 640

      SUMA=ZERO

      SUMB=ZERO

      SUMC=ZERO

      IZ=IWZ+NACT*N

      DO 563 I=1,N

      IZ=IZ+1

      IW=IWW+I

      SUMA=SUMA+W(IW)*W(IZ)

      SUMB=SUMB+DABS(W(IW)*W(IZ))

  563 SUMC=SUMC+W(IZ)**2

      TEMP=SUMB+.1D+0*DABS(SUMA)

      TEMPA=SUMB+.2D+0*DABS(SUMA)

      IF (TEMP .LE. SUMB) GOTO 570

      IF (TEMPA .LE. TEMP) GOTO 570

      IF (SUMB .GT. VSMALL) GOTO 5

      GOTO 570

    5 SUMC=DSQRT(SUMC)

      IA=IWA+KNEXT

      IF (KNEXT .LE. M) SUMC=SUMC/W(IA)

      TEMP=SUMC+.1D+0*DABS(SUMA)

      TEMPA=SUMC+.2D+0*DABS(SUMA)

      IF (TEMP .LE. SUMC) GOTO 567

      IF (TEMPA .LE. TEMP) GOTO 567

      GOTO 640

C

C     CALCULATE THE MULTIPLIERS FOR THE NEW CONSTRAINT NORMAL

C     EXPRESSED IN TERMS OF THE ACTIVE CONSTRAINT NORMALS.

C     THEN WORK OUT WHICH CONTRAINT TO DROP.

C

  567 LFLAG=4

      GOTO 740

  570 LFLAG=1

      GOTO 740

C

C     COMPLETE THE TEST FOR LINEARLY DEPENDENT CONSTRAINTS.

C

  571 IF (KNEXT .GT. M) GOTO 574

      DO 573 I=1,N

      SUMA=A(KNEXT,I)

      SUMB=DABS(SUMA)

      IF (NACT.EQ.0) GOTO 581

      DO 572 K=1,NACT

      KK=IACT(K)

      IW=IWW+K

      TEMP=W(IW)*A(KK,I)

      SUMA=SUMA-TEMP

  572 SUMB=SUMB+DABS(TEMP)

  581 IF (SUMA .LE. VSMALL) GOTO 573

      TEMP=SUMB+.1D+0*DABS(SUMA)

      TEMPA=SUMB+.2D+0*DABS(SUMA)

      IF (TEMP .LE. SUMB) GOTO 573

      IF (TEMPA .LE. TEMP) GOTO 573

      GOTO 630

  573 CONTINUE

      LFLAG=1

      GOTO 775

  574 K1=KNEXT-M

      IF (K1 .GT. N) K1=K1-N

      DO 578 I=1,N

      SUMA=ZERO

      IF (I .NE. K1) GOTO 575

      SUMA=ONE

      IF (KNEXT .GT. MN) SUMA=-ONE

  575 SUMB=DABS(SUMA)

      IF (NACT.EQ.0) GOTO 582

      DO 577 K=1,NACT

      KK=IACT(K)

      IF (KK .LE. M) GOTO 579

      KK=KK-M

      TEMP=ZERO

      IF (KK.EQ.I) TEMP=W(IWW+KK)

      KK=KK-N

      IF (KK.EQ.I) TEMP=-W(IWW+KK)

      GOTO 576

  579 IW=IWW+K

      TEMP=W(IW)*A(KK,I)

  576 SUMA=SUMA-TEMP

  577 SUMB=SUMB+DABS(TEMP)

  582 TEMP=SUMB+.1D+0*DABS(SUMA)

      TEMPA=SUMB+.2D+0*DABS(SUMA)

      IF (TEMP .LE. SUMB) GOTO 578

      IF (TEMPA .LE. TEMP) GOTO 578

      GOTO 630

  578 CONTINUE

      LFLAG=1

      GOTO 775

C

C     BRANCH IF THE CONTRAINTS ARE INCONSISTENT.

C

  580 INFO=-KNEXT

      IF (KDROP .EQ. 0) GOTO 700

      PARINC=RATIO

      PARNEW=PARINC

C

C     REVISE THE LAGRANGE MULTIPLIERS OF THE ACTIVE CONSTRAINTS.

C

  590 IF (NACT.EQ.0) GOTO 601

      DO 600 K=1,NACT

      IW=IWW+K

      W(K)=W(K)-PARINC*W(IW)

      IF (IACT(K) .GT. MEQ) W(K)=DMAX1(ZERO,W(K))

  600 CONTINUE

  601 IF (KDROP .EQ. 0) GOTO 680

C

C     DELETE THE CONSTRAINT TO BE DROPPED.

C     SHIFT THE VECTOR OF SCALAR PRODUCTS.

C     THEN, IF APPROPRIATE, MAKE ONE MORE SCALAR PRODUCT ZERO.

C

      NU=NACT+1

      MFLAG=2

      GOTO 800

  610 IWS=IWS-NACT-1

      NU=MIN0(N,NU)

      DO 620 I=1,NU

      IS=IWS+I

      J=IS+NACT

  620 W(IS)=W(J+1)

      NFLAG=2

      GOTO 860

C

C     CALCULATE THE STEP TO THE VIOLATED CONSTRAINT.

C

  630 IS=IWS+NACT

  640 SUMY=W(IS+1)

      STEP=-RES/SUMY

      PARINC=STEP/SUMY

      IF (NACT .EQ. 0) GOTO 660

C

C     CALCULATE THE CHANGES TO THE LAGRANGE MULTIPLIERS, AND REDUCE

C     THE STEP ALONG THE NEW SEARCH DIRECTION IF NECESSARY.

C

      LFLAG=2

      GOTO 740

  650 IF (KDROP .EQ. 0) GOTO 660

      TEMP=ONE-RATIO/PARINC

      IF (TEMP .LE. ZERO) KDROP=0

      IF (KDROP .EQ. 0) GOTO 660

      STEP=RATIO*SUMY

      PARINC=RATIO

      RES=TEMP*RES

C

C     UPDATE X AND THE LAGRANGE MULTIPIERS.

C     DROP A CONSTRAINT IF THE FULL STEP IS NOT TAKEN.

C

  660 IWY=IWZ+NACT*N

      DO 670 I=1,N

      IY=IWY+I

  670 X(I)=X(I)+STEP*W(IY)

      PARNEW=PARNEW+PARINC

      IF (NACT .GE. 1) GOTO 590

C

C     ADD THE NEW CONSTRAINT TO THE ACTIVE SET.

C

  680 NACT=NACT+1

      W(NACT)=PARNEW

      IACT(NACT)=KNEXT

      IA=IWA+KNEXT

      IF (KNEXT .GT. MN) IA=IA-N

      W(IA)=-W(IA)

C

C     ESTIMATE THE MAGNITUDE OF X. THEN BEGIN A NEW ITERATION,

C     RE-INITILISING X IF THIS MAGNITUDE IS SMALL.

C

      JFLAG=2

      GOTO 910

  690 IF (SUM .LT. (XMAGR*XMAG)) GOTO 230

      IF (ITREF) 450,450,250

C

C     INITIATE ITERATIVE REFINEMENT IF IT HAS NOT YET BEEN USED,

C     OR RETURN AFTER RESTORING THE DIAGONAL ELEMENTS OF G.

C

  700 IF (ITERC .EQ. 0) GOTO 710

      ITREF=ITREF+1

      JFINC=-1

      IF (ITREF .EQ. 1) GOTO 250

  710 IF (.NOT. LQL) RETURN

      DO 720 I=1,N

      ID=IWD+I

  720 G(I,I)=W(ID)

  730 RETURN

C

C

C     THE REMAINIG INSTRUCTIONS ARE USED AS SUBROUTINES.

C

C

C********************************************************************

C

C

C     CALCULATE THE LAGRANGE MULTIPLIERS BY PRE-MULTIPLYING THE

C     VECTOR IN THE S-PARTITION OF W BY THE INVERSE OF R.

C

  740 IR=IWR+(NACT+NACT*NACT)/2

      I=NACT

      SUM=ZERO

      GOTO 770

  750 IRA=IR-1

      SUM=ZERO

      IF (NACT.EQ.0) GOTO 761

      DO 760 J=I,NACT

      IW=IWW+J

      SUM=SUM+W(IRA)*W(IW)

  760 IRA=IRA+J

  761 IR=IR-I

      I=I-1

  770 IW=IWW+I

      IS=IWS+I

      W(IW)=(W(IS)-SUM)/W(IR)

      IF (I .GT. 1) GOTO 750

      IF (LFLAG .EQ. 3) GOTO 390

      IF (LFLAG .EQ. 4) GOTO 571

C

C     CALCULATE THE NEXT CONSTRAINT TO DROP.

C

  775 IP=IWW+1

      IPP=IWW+NACT

      KDROP=0

      IF (NACT.EQ.0) GOTO 791

      DO 790 K=1,NACT

      IF (IACT(K) .LE. MEQ) GOTO 790

      IW=IWW+K

      IF ((RES*W(IW)) .GE. ZERO) GOTO 790

      TEMP=W(K)/W(IW)

      IF (KDROP .EQ. 0) GOTO 780

      IF (DABS(TEMP) .GE. DABS(RATIO)) GOTO 790

  780 KDROP=K

      RATIO=TEMP

  790 CONTINUE

  791 GOTO (580,650), LFLAG

C

C

C********************************************************************

C

C

C     DROP THE CONSTRAINT IN POSITION KDROP IN THE ACTIVE SET.

C

  800 IA=IWA+IACT(KDROP)

      IF (IACT(KDROP) .GT. MN) IA=IA-N

      W(IA)=-W(IA)

      IF (KDROP .EQ. NACT) GOTO 850

C

C     SET SOME INDICES AND CALCULATE THE ELEMENTS OF THE NEXT

C     GIVENS ROTATION.

C

      IZ=IWZ+KDROP*N

      IR=IWR+(KDROP+KDROP*KDROP)/2

  810 IRA=IR

      IR=IR+KDROP+1

      TEMP=DMAX1(DABS(W(IR-1)),DABS(W(IR)))

      SUM=TEMP*DSQRT((W(IR-1)/TEMP)**2+(W(IR)/TEMP)**2)

      GA=W(IR-1)/SUM

      GB=W(IR)/SUM

C

C     EXCHANGE THE COLUMNS OF R.

C

      DO 820 I=1,KDROP

      IRA=IRA+1

      J=IRA-KDROP

      TEMP=W(IRA)

      W(IRA)=W(J)

  820 W(J)=TEMP

      W(IR)=ZERO

C

C     APPLY THE ROTATION TO THE ROWS OF R.

C

      W(J)=SUM

      KDROP=KDROP+1

      DO 830 I=KDROP,NU

      TEMP=GA*W(IRA)+GB*W(IRA+1)

      W(IRA+1)=GA*W(IRA+1)-GB*W(IRA)

      W(IRA)=TEMP

  830 IRA=IRA+I

C

C     APPLY THE ROTATION TO THE COLUMNS OF Z.

C

      DO 840 I=1,N

      IZ=IZ+1

      J=IZ-N

      TEMP=GA*W(J)+GB*W(IZ)

      W(IZ)=GA*W(IZ)-GB*W(J)

  840 W(J)=TEMP

C

C     REVISE IACT AND THE LAGRANGE MULTIPLIERS.

C

      IACT(KDROP-1)=IACT(KDROP)

      W(KDROP-1)=W(KDROP)

      IF (KDROP .LT. NACT) GOTO 810

  850 NACT=NACT-1

      GOTO (250,610), MFLAG

C

C

C********************************************************************

C

C

C     APPLY GIVENS ROTATION TO REDUCE SOME OF THE SCALAR

C     PRODUCTS IN THE S-PARTITION OF W TO ZERO.

C

  860 IZ=IWZ+NU*N

  870 IZ=IZ-N

  880 IS=IWS+NU

      NU=NU-1

      IF (NU .EQ. NACT) GOTO 900

      IF (W(IS) .EQ. ZERO) GOTO 870

      TEMP=DMAX1(DABS(W(IS-1)),DABS(W(IS)))

      SUM=TEMP*DSQRT((W(IS-1)/TEMP)**2+(W(IS)/TEMP)**2)

      GA=W(IS-1)/SUM

      GB=W(IS)/SUM

      W(IS-1)=SUM

      DO 890 I=1,N

      K=IZ+N

      TEMP=GA*W(IZ)+GB*W(K)

      W(K)=GA*W(K)-GB*W(IZ)

      W(IZ)=TEMP

  890 IZ=IZ-1

      GOTO 880

  900 GOTO (560,630), NFLAG

C

C

C********************************************************************

C

C

C     CALCULATE THE MAGNITUDE OF X AN REVISE XMAG.

C

  910 SUM=ZERO

      DO 920 I=1,N

      SUM=SUM+DABS(X(I))*VFACT*(DABS(GRAD(I))+DABS(G(I,I)*X(I)))

      IF (LQL) GOTO 920

      IF (SUM .LT. 1.D-30) GOTO 920

      VFACT=1.D-10*VFACT

      SUM=1.D-10*SUM

      XMAG=1.D-10*XMAG

  920 CONTINUE

  925 XMAG=DMAX1(XMAG,SUM)

      GOTO (420,690), JFLAG

C

C

C********************************************************************

C

C

C     PRE-MULTIPLY THE VECTOR IN THE W-PARTITION OF W BY Z TRANSPOSE.

C

  930 JL=IWW+1

      IZ=IWZ

      DO 940 I=1,N

      IS=IWS+I

      W(IS)=ZERO

      IWWN=IWW+N

      DO 940 J=JL,IWWN

      IZ=IZ+1

  940 W(IS)=W(IS)+W(IZ)*W(J)

      GOTO (350,550), KFLAG

      RETURN

      END

C

      SUBROUTINE DVERK (N,FCN,X,Y,XEND,TOL,IND,C,NW,W,IER)              JUN00010

      INTEGER            N,IND,NW,IER                                   JUN00020

      DOUBLE PRECISION   X,Y(N),XEND,TOL,C(100),W(NW,9)                 JUN00030

      INTEGER            K                                              JUN00040

      DOUBLE PRECISION   ZERO,ONE,TWO,THREE,FOUR,FIVE,SEVEN,TEN,HALF,P9 JUN00050

      DOUBLE PRECISION   C4D15,C2D3,C5D6,C1D6,C1D15,C2D96,TEMP          JUN00060

      DOUBLE PRECISION   RK(39),REPS,RTOL                               JUN00070

      DATA               ZERO/0.0D0/,ONE/1.0D0/,TWO/2.0D0/,THREE/3.0D0/ JUN00080

      DATA               FOUR/4.0D0/,FIVE/5.0D0/,SEVEN/7.0D0/           JUN00090

      DATA               TEN/10.0D0/,HALF/0.5D0/,P9/0.9D0/              JUN00100

      DATA               C4D15/.2666666666666667D0/                     JUN00110

      DATA               C2D3/.6666666666666667D0/                      JUN00120

      DATA               C5D6/.8333333333333333D0/                      JUN00130

      DATA               C1D6/.1666666666666667D0/                      JUN00140

      DATA               C1D15/.6666666666666667D-1/                    JUN00150

      DATA               C2D96/120.4272910821709D0/                     JUN00160

      DATA               REPS/2.775557562D-17/                          JUN00170

      DATA               RTOL/1.058791184D-22/                          JUN00180

      DATA               RK( 1)/.1666666666666667D+00/                  JUN00190

      DATA               RK( 2)/.5333333333333333D-01/                  JUN00200

      DATA               RK( 3)/.2133333333333333D+00/                  JUN00210

      DATA               RK( 4)/.8333333333333333D+00/                  JUN00220

      DATA               RK( 5)/.2666666666666667D+01/                  JUN00230

      DATA               RK( 6)/.2500000000000000D+01/                  JUN00240

      DATA               RK( 7)/.2578125000000000D+01/                  JUN00250

      DATA               RK( 8)/.9166666666666667D+01/                  JUN00260

      DATA               RK( 9)/.6640625000000000D+01/                  JUN00270

      DATA               RK(10)/.8854166666666667D+00/                  JUN00280

      DATA               RK(11)/.2400000000000000D+01/                  JUN00290

      DATA               RK(12)/.8000000000000000D+01/                  JUN00300

      DATA               RK(13)/.6560457516339869D+01/                  JUN00310

      DATA               RK(14)/.3055555555555556D+00/                  JUN00320

      DATA               RK(15)/.3450980392156863D+00/                  JUN00330

      DATA               RK(16)/.5508666666666667D+00/                  JUN00340

      DATA               RK(17)/.1653333333333333D+01/                  JUN00350

      DATA               RK(18)/.9455882352941176D+00/                  JUN00360

      DATA               RK(19)/.3240000000000000D+00/                  JUN00370

      DATA               RK(20)/.2337882352941176D+00/                  JUN00380

      DATA               RK(21)/.2035465116279070D+01/                  JUN00390

      DATA               RK(22)/.6976744186046512D+01/                  JUN00400

      DATA               RK(23)/.5648179814561484D+01/                  JUN00410

      DATA               RK(24)/.1373815676141258D+00/                  JUN00420

      DATA               RK(25)/.2863022661036103D+00/                  JUN00430

      DATA               RK(26)/.1441785567164738D+00/                  JUN00440

      DATA               RK(27)/.7500000000000000D-01/                  JUN00450

      DATA               RK(28)/.3899286987522282D+00/                  JUN00460

      DATA               RK(29)/.3194444444444444D+00/                  JUN00470

      DATA               RK(30)/.1350383631713555D+00/                  JUN00480

      DATA               RK(31)/.1078329882677709D-01/                  JUN00490

      DATA               RK(32)/.6980519480519481D-01/                  JUN00500

      DATA               RK(33)/.6250000000000000D-02/                  JUN00510

      DATA               RK(34)/.6963012477718360D-02/                  JUN00520

      DATA               RK(35)/.6944444444444444D-02/                  JUN00530

      DATA               RK(36)/.6138107416879795D-02/                  JUN00540

      DATA               RK(37)/.6818181818181818D-01/                  JUN00550

      DATA               RK(38)/.1078329882677709D-01/                  JUN00560

      DATA               RK(39)/.6980519480519481D-01/                  JUN00570

      IER = 0                                                           JUN00580

      IF (IND.LT.1.OR.IND.GT.6) GO TO 290                               JUN00590

      GO TO (5,5,40,145,265,265), IND                                   JUN00600

    5 IF (N.GT.NW.OR.TOL.LE.ZERO) GO TO 295                             JUN00610

      IF (IND.EQ.2) GO TO 15                                            JUN00620

      DO 10 K=1,9                                                       JUN00630

         C(K) = ZERO                                                    JUN00640

   10 CONTINUE                                                          JUN00650

      GO TO 30                                                          JUN00660

   15 CONTINUE                                                          JUN00670

      DO 20 K=1,9                                                       JUN00680

         C(K) = DABS(C(K))                                              JUN00690

   20 CONTINUE                                                          JUN00700

      IF (C(1).NE.FOUR.AND.C(1).NE.FIVE) GO TO 30                       JUN00710

      DO 25 K=1,N                                                       JUN00720

         C(K+30) = DABS(C(K+30))                                        JUN00730

   25 CONTINUE                                                          JUN00740

   30 CONTINUE                                                          JUN00750

      C(10) = REPS                                                      JUN00760

      C(11) = RTOL                                                      JUN00770

      C(20) = X                                                         JUN00780

      DO 35 K=21,24                                                     JUN00790

         C(K) = ZERO                                                    JUN00800

   35 CONTINUE                                                          JUN00810

      GO TO 45                                                          JUN00820

   40 IF (C(21).NE.ZERO.AND.(X.NE.C(20).OR.XEND.EQ.C(20))) GO TO 285    JUN00830

      C(21) = ZERO                                                      JUN00840

   45 CONTINUE                                                          JUN00850

   50 CONTINUE                                                          JUN00860

      IF (C(7).EQ.ZERO.OR.C(24).LT.C(7)) GO TO 55                       JUN00870

      IND = -1                                                          JUN00880

      GO TO 9005                                                        JUN00890

   55 CONTINUE                                                          JUN00900

      IF (IND.EQ.6) GO TO 60                                            JUN00910

      CALL FCN (N,X,Y,W(1,1))                                           JUN00920

      C(24) = C(24)+ONE                                                 JUN00930

   60 CONTINUE                                                          JUN00940

      C(13) = C(3)                                                      JUN00950

      IF (C(3).NE.ZERO) GO TO 120                                       JUN00960

      TEMP = ZERO                                                       JUN00970

      IF (C(1).NE.ONE) GO TO 70                                         JUN00980

      DO 65 K=1,N                                                       JUN00990

         TEMP = DMAX1(TEMP,DABS(Y(K)))                                  JUN01000

   65 CONTINUE                                                          JUN01010

      C(12) = TEMP                                                      JUN01020

      GO TO 115                                                         JUN01030

   70 IF (C(1).NE.TWO) GO TO 75                                         JUN01040

      C(12) = ONE                                                       JUN01050

      GO TO 115                                                         JUN01060

   75 IF (C(1).NE.THREE) GO TO 85                                       JUN01070

      DO 80 K=1,N                                                       JUN01080

         TEMP = DMAX1(TEMP,DABS(Y(K))/C(2))                             JUN01090

   80 CONTINUE                                                          JUN01100

      C(12) = DMIN1(TEMP,ONE)                                           JUN01110

      GO TO 115                                                         JUN01120

   85 IF (C(1).NE.FOUR) GO TO 95                                        JUN01130

      DO 90 K=1,N                                                       JUN01140

         TEMP = DMAX1(TEMP,DABS(Y(K))/C(K+30))                          JUN01150

   90 CONTINUE                                                          JUN01160

      C(12) = DMIN1(TEMP,ONE)                                           JUN01170

      GO TO 115                                                         JUN01180

   95 IF (C(1).NE.FIVE) GO TO 105                                       JUN01190

      DO 100 K=1,N                                                      JUN01200

         TEMP = DMAX1(TEMP,DABS(Y(K))/C(K+30))                          JUN01210

  100 CONTINUE                                                          JUN01220

      C(12) = TEMP                                                      JUN01230

      GO TO 115                                                         JUN01240

  105 CONTINUE                                                          JUN01250

      DO 110 K=1,N                                                      JUN01260

         TEMP = DMAX1(TEMP,DABS(Y(K)))                                  JUN01270

  110 CONTINUE                                                          JUN01280

      C(12) = DMIN1(TEMP,ONE)                                           JUN01290

  115 CONTINUE                                                          JUN01300

      C(13) = TEN*DMAX1(C(11),C(10)*DMAX1(C(12)/TOL,DABS(X)))           JUN01310

  120 CONTINUE                                                          JUN01320

      C(15) = C(5)                                                      JUN01330

      IF (C(5).EQ.ZERO) C(15) = ONE                                     JUN01340

      IF (C(6).NE.ZERO.AND.C(5).NE.ZERO) C(16) = DMIN1(C(6),TWO/C(5))   JUN01350

      IF (C(6).NE.ZERO.AND.C(5).EQ.ZERO) C(16) = C(6)                   JUN01360

      IF (C(6).EQ.ZERO.AND.C(5).NE.ZERO) C(16) = TWO/C(5)               JUN01370

      IF (C(6).EQ.ZERO.AND.C(5).EQ.ZERO) C(16) = TWO                    JUN01380

      IF (C(13).LE.C(16)) GO TO 125                                     JUN01390

      IND = -2                                                          JUN01400

      GO TO 9005                                                        JUN01410

  125 CONTINUE                                                          JUN01420

      IF (IND.GT.2) GO TO 130                                           JUN01430

      C(14) = C(4)                                                      JUN01440

      IF (C(4).EQ.ZERO) C(14) = C(16)*TOL**C1D6                         JUN01450

      GO TO 140                                                         JUN01460

  130 IF (C(23).GT.ONE) GO TO 135                                       JUN01470

      TEMP = TWO*C(14)                                                  JUN01480

      IF (TOL.LT.C2D96*C(19)) TEMP = P9*(TOL/C(19))**C1D6*C(14)         JUN01490

      C(14) = DMAX1(TEMP,HALF*C(14))                                    JUN01500

      GO TO 140                                                         JUN01510

  135 CONTINUE                                                          JUN01520

      C(14) = HALF*C(14)                                                JUN01530

  140 CONTINUE                                                          JUN01540

      C(14) = DMIN1(C(14),C(16))                                        JUN01550

      C(14) = DMAX1(C(14),C(13))                                        JUN01560

      IF (C(8).EQ.ZERO) GO TO 145                                       JUN01570

      IND = 4                                                           JUN01580

      GO TO 9005                                                        JUN01590

  145 CONTINUE                                                          JUN01600

      IF (C(14).GE.DABS(XEND-X)) GO TO 150                              JUN01610

      C(14) = DMIN1(C(14),HALF*DABS(XEND-X))                            JUN01620

      C(17) = X+DSIGN(C(14),XEND-X)                                     JUN01630

      GO TO 155                                                         JUN01640

  150 CONTINUE                                                          JUN01650

      C(14) = DABS(XEND-X)                                              JUN01660

      C(17) = XEND                                                      JUN01670

  155 CONTINUE                                                          JUN01680

      C(18) = C(17)-X                                                   JUN01690

      DO 160 K=1,N                                                      JUN01700

         W(K,9) = Y(K)+C(18)*W(K,1)*RK(1)                               JUN01710

  160 CONTINUE                                                          JUN01720

      CALL FCN (N,X+C(18)*C1D6,W(1,9),W(1,2))                           JUN01730

      DO 165 K=1,N                                                      JUN01740

         W(K,9) = Y(K)+C(18)*(W(K,1)*RK(2)+W(K,2)*RK(3))                JUN01750

  165 CONTINUE                                                          JUN01760

      CALL FCN (N,X+C(18)*C4D15,W(1,9),W(1,3))                          JUN01770

      DO 170 K=1,N                                                      JUN01780

         W(K,9) = Y(K)+C(18)*(W(K,1)*RK(4)-W(K,2)*RK(5)+W(K,3)*RK(6))   JUN01790

  170 CONTINUE                                                          JUN01800

      CALL FCN (N,X+C(18)*C2D3,W(1,9),W(1,4))                           JUN01810

      DO 175 K=1,N                                                      JUN01820

         W(K,9) = Y(K)+C(18)*(-W(K,1)*RK(7)+W(K,2)*RK(8)-W(K,3)*RK(9)   JUN01830

     1   +W(K,4)*RK(10))                                                JUN01840

  175 CONTINUE                                                          JUN01850

      CALL FCN (N,X+C(18)*C5D6,W(1,9),W(1,5))                           JUN01860

      DO 180 K=1,N                                                      JUN01870

         W(K,9) = Y(K)+C(18)*(W(K,1)*RK(11)-W(K,2)*RK(12)+W(K,3)*RK(13) JUN01880

     1   -W(K,4)*RK(14)+W(K,5)*RK(15))                                  JUN01890

  180 CONTINUE                                                          JUN01900

      CALL FCN (N,X+C(18),W(1,9),W(1,6))                                JUN01910

      DO 185 K=1,N                                                      JUN01920

         W(K,9) = Y(K)+C(18)*(-W(K,1)*RK(16)+W(K,2)*RK(17)-W(K,3)       JUN01930

     1   *RK(18)-W(K,4)*RK(19)+W(K,5)*RK(20))                           JUN01940

  185 CONTINUE                                                          JUN01950

      CALL FCN (N,X+C(18)*C1D15,W(1,9),W(1,7))                          JUN01960

      DO 190 K=1,N                                                      JUN01970

         W(K,9) = Y(K)+C(18)*(W(K,1)*RK(21)-W(K,2)*RK(22)+W(K,3)*RK(23) JUN01980

     1   -W(K,4)*RK(24)+W(K,5)*RK(25)+W(K,7)*RK(26))                    JUN01990

  190 CONTINUE                                                          JUN02000

      CALL FCN (N,X+C(18),W(1,9),W(1,8))                                JUN02010

      DO 195 K=1,N                                                      JUN02020

         W(K,9) = Y(K)+C(18)*(W(K,1)*RK(27)+W(K,3)*RK(28)+W(K,4)*RK(29) JUN02030

     1   +W(K,5)*RK(30)+W(K,7)*RK(31)+W(K,8)*RK(32))                    JUN02040

  195 CONTINUE                                                          JUN02050

      C(24) = C(24)+SEVEN                                               JUN02060

      DO 200 K=1,N                                                      JUN02070

         W(K,2) = W(K,1)*RK(33)+W(K,3)*RK(34)-W(K,4)*RK(35)+W(K,5)      JUN02080

     1   *RK(36)+W(K,6)*RK(37)-W(K,7)*RK(38)-W(K,8)*RK(39)              JUN02090

  200 CONTINUE                                                          JUN02100

      TEMP = ZERO                                                       JUN02110

      IF (C(1).NE.ONE) GO TO 210                                        JUN02120

      DO 205 K=1,N                                                      JUN02130

         TEMP = DMAX1(TEMP,DABS(W(K,2)))                                JUN02140

  205 CONTINUE                                                          JUN02150

      GO TO 260                                                         JUN02160

  210 IF (C(1).NE.TWO) GO TO 220                                        JUN02170

      DO 215 K=1,N                                                      JUN02180

         IF (Y(K).EQ.ZERO) GO TO 280                                    JUN02190

         TEMP = DMAX1(TEMP,DABS(W(K,2)/Y(K)))                           JUN02200

  215 CONTINUE                                                          JUN02210

      GO TO 260                                                         JUN02220

  220 IF (C(1).NE.THREE) GO TO 230                                      JUN02230

      DO 225 K=1,N                                                      JUN02240

         TEMP = DMAX1(TEMP,DABS(W(K,2))/DMAX1(C(2),DABS(Y(K))))         JUN02250

  225 CONTINUE                                                          JUN02260

      GO TO 260                                                         JUN02270

  230 IF (C(1).NE.FOUR) GO TO 240                                       JUN02280

      DO 235 K=1,N                                                      JUN02290

         TEMP = DMAX1(TEMP,DABS(W(K,2))/DMAX1(C(K+30),DABS(Y(K))))      JUN02300

  235 CONTINUE                                                          JUN02310

      GO TO 260                                                         JUN02320

  240 IF (C(1).NE.FIVE) GO TO 250                                       JUN02330

      DO 245 K=1,N                                                      JUN02340

         TEMP = DMAX1(TEMP,DABS(W(K,2)/C(K+30)))                        JUN02350

  245 CONTINUE                                                          JUN02360

      GO TO 260                                                         JUN02370

  250 CONTINUE                                                          JUN02380

      DO 255 K=1,N                                                      JUN02390

         TEMP = DMAX1(TEMP,DABS(W(K,2))/DMAX1(ONE,DABS(Y(K))))          JUN02400

  255 CONTINUE                                                          JUN02410

  260 CONTINUE                                                          JUN02420

      C(19) = TEMP*C(14)*C(15)                                          JUN02430

      IND = 5                                                           JUN02440

      IF (C(19).GT.TOL) IND = 6                                         JUN02450

      IF (C(9).NE.ZERO) GO TO 9005                                      JUN02460

  265 CONTINUE                                                          JUN02470

      IF (IND.EQ.6) GO TO 275                                           JUN02480

      X = C(17)                                                         JUN02490

      DO 270 K=1,N                                                      JUN02500

         Y(K) = W(K,9)                                                  JUN02510

  270 CONTINUE                                                          JUN02520

      C(22) = C(22)+ONE                                                 JUN02530

      C(23) = ZERO                                                      JUN02540

      IF (X.NE.XEND) GO TO 50                                           JUN02550

      IND = 3                                                           JUN02560

      C(20) = XEND                                                      JUN02570

      C(21) = ONE                                                       JUN02580

      GO TO 9005                                                        JUN02590

  275 CONTINUE                                                          JUN02600

      C(23) = C(23)+ONE                                                 JUN02610

      IF (C(14).GT.C(13)) GO TO 50                                      JUN02620

      IND = -3                                                          JUN02630

      GO TO 9005                                                        JUN02640

  280 CONTINUE                                                          JUN02650

      IER = 132                                                         JUN02660

      GO TO 9000                                                        JUN02670

  285 CONTINUE                                                          JUN02680

      IER = 131                                                         JUN02690

      GO TO 9000                                                        JUN02700

  290 CONTINUE                                                          JUN02710

      IER = 130                                                         JUN02720

      GO TO 9000                                                        JUN02730

  295 CONTINUE                                                          JUN02740

      IER = 129                                                         JUN02750

 9000 CONTINUE                                                          JUN02760

      CALL UERTST (IER,6HDVERK )                                        JUN02770

 9005 CONTINUE                                                          JUN02780

      RETURN                                                            JUN02790

      END                                                               JUN02800

      DOUBLE PRECISION FUNCTION DCADRE (F,A,B,AERR,RERR,ERROR,IER)      JUN02810

      INTEGER            IER                                            JUN02820

      DOUBLE PRECISION   F,A,B,AERR,RERR,ERROR                          JUN02830

      INTEGER            IBEGS(30),MAXTS,MAXTBL,MXSTGE,IBEG,II,NNLEFT   JUN02840

      INTEGER            I,N2,III,ISTEP2,IEND,ISTEP,L,LM1,IT,ISTAGE,N   JUN02850

      DOUBLE PRECISION   T(10,10),R(10),AIT(10),DIF(10),RN(4),TS(2049)  JUN02860

      DOUBLE PRECISION   BEGIN(30),FINIS(30),EST(30)                    JUN02870

      DOUBLE PRECISION   H2TOL,AITTOL,LENGTH,JUMPTL,ZERO,P1,HALF,ONE    JUN02880

      DOUBLE PRECISION   TWO,FOUR,FOURP5,TEN,HUN,CADRE,AITLOW           JUN02890

      DOUBLE PRECISION   STEPMN,STEPNM,STAGE,CUREST,FNSIZE,HRERR        JUN02900

      DOUBLE PRECISION   PREVER,BEG,FBEG,EDN,FEND,STEP,ASTEP,TABS,HOVN  JUN02910

      DOUBLE PRECISION   FN,SUM,SUMABS,VINT,TABTLM,ERGL,ERGOAL          JUN02920

      DOUBLE PRECISION   ERRA,ERRR,FEXTRP,ERRER,DIFF,SING,FEXTM1        JUN02930

      DOUBLE PRECISION   H2NEXT,SINGNX,SLOPE,FBEG2,ERRET,H2TFEX,FI      JUN02940

      LOGICAL            H2CONV,AITKEN,RIGHT,REGLAR,REGLSV(30)          JUN02950

      DATA               AITLOW,H2TOL,AITTOL,JUMPTL,MAXTS,MAXTBL,MXSTGE JUN02960

     1                   /1.1D0,.15D0,.1D0,.01D0,2049,10,30/            JUN02970

      DATA               RN(1),RN(2),RN(3),RN(4)/                       JUN02980

     1                   .7142005D0,.3466282D0,.843751D0,.1263305D0/    JUN02990

      DATA               ZERO,P1,HALF,ONE,TWO,FOUR,FOURP5,TEN,HUN       JUN03000

     1                   /0.0D0,0.1D0,0.5D0,1.0D0,2.0D0,4.0D0,          JUN03010

     2                   4.5D0,10.0D0,100.0D0/                          JUN03020

      IER = 0                                                           JUN03030

      CADRE = ZERO                                                      JUN03040

      ERROR = ZERO                                                      JUN03050

      CUREST = ZERO                                                     JUN03060

      VINT = ZERO                                                       JUN03070

      LENGTH = DABS(B-A)                                                JUN03080

      IF (LENGTH .EQ. ZERO) GO TO 215                                   JUN03090

      IF (RERR .GT. P1 .OR. RERR .LT. ZERO) GO TO 210                   JUN03100

      HRERR = RERR+HUN                                                  JUN03110

      IF (AERR .EQ. ZERO .AND. HRERR .LE. HUN) GO TO 210                JUN03120

      ERRR = RERR                                                       JUN03130

      ERRA = DABS(AERR)                                                 JUN03140

      STEPMN = LENGTH/(TWO**MXSTGE)                                     JUN03150

      STEPNM = DMAX1(LENGTH,DABS(A),DABS(B))*TEN                        JUN03160

      STAGE = HALF                                                      JUN03170

      ISTAGE = 1                                                        JUN03180

      FNSIZE = ZERO                                                     JUN03190

      PREVER = ZERO                                                     JUN03200

      REGLAR = .FALSE.                                                  JUN03210

      BEG = A                                                           JUN03220

      FBEG = F(BEG)*HALF                                                JUN03230

      TS(1) = FBEG                                                      JUN03240

      IBEG = 1                                                          JUN03250

      EDN = B                                                           JUN03260

      FEND = F(EDN)*HALF                                                JUN03270

      TS(2) = FEND                                                      JUN03280

      IEND = 2                                                          JUN03290

    5 RIGHT = .FALSE.                                                   JUN03300

   10 STEP = EDN - BEG                                                  JUN03310

      ASTEP =  DABS(STEP)                                               JUN03320

      IF (ASTEP .LT. STEPMN) GO TO 205                                  JUN03330

      HRERR = STEPNM+ASTEP                                              JUN03340

      IF (HRERR .EQ. STEPNM) GO TO 205                                  JUN03350

      T(1,1) = FBEG + FEND                                              JUN03360

      TABS = DABS(FBEG) + DABS(FEND)                                    JUN03370

      L = 1                                                             JUN03380

      N = 1                                                             JUN03390

      H2CONV = .FALSE.                                                  JUN03400

      AITKEN = .FALSE.                                                  JUN03410

   15 LM1 = L                                                           JUN03420

      L = L + 1                                                         JUN03430

      N2 = N+N                                                          JUN03440

      FN = N2                                                           JUN03450

      ISTEP = (IEND - IBEG)/N                                           JUN03460

      IF (ISTEP .GT. 1) GO TO 25                                        JUN03470

      II = IEND                                                         JUN03480

      IEND = IEND + N                                                   JUN03490

      IF (IEND .GT. MAXTS) GO TO 200                                    JUN03500

      HOVN = STEP/FN                                                    JUN03510

      III = IEND                                                        JUN03520

      FI = ONE                                                          JUN03530

      DO 20 I=1,N2,2                                                    JUN03540

         TS(III) = TS(II)                                               JUN03550

         TS(III-1) = F(EDN - FI * HOVN)                                 JUN03560

         FI = FI+TWO                                                    JUN03570

         III = III-2                                                    JUN03580

         II = II-1                                                      JUN03590

   20 CONTINUE                                                          JUN03600

      ISTEP = 2                                                         JUN03610

   25 ISTEP2 = IBEG + ISTEP/2                                           JUN03620

      SUM = ZERO                                                        JUN03630

      SUMABS = ZERO                                                     JUN03640

      DO 30 I=ISTEP2,IEND,ISTEP                                         JUN03650

         SUM = SUM + TS(I)                                              JUN03660

         SUMABS = SUMABS + DABS(TS(I))                                  JUN03670

   30 CONTINUE                                                          JUN03680

      T(L,1) = T(L-1,1)*HALF+SUM/FN                                     JUN03690

      TABS = TABS*HALF+SUMABS/FN                                        JUN03700

      N = N2                                                            JUN03710

      IT = 1                                                            JUN03720

      VINT = STEP*T(L,1)                                                JUN03730

      TABTLM = TABS*TEN                                                 JUN03740

      FNSIZE = DMAX1(FNSIZE,DABS(T(L,1)))                               JUN03750

      ERGL = ASTEP*FNSIZE*TEN                                           JUN03760

      ERGOAL = STAGE*DMAX1(ERRA,ERRR*DABS(CUREST+VINT))                 JUN03770

      FEXTRP = ONE                                                      JUN03780

      DO 35 I=1,LM1                                                     JUN03790

         FEXTRP = FEXTRP*FOUR                                           JUN03800

         T(I,L) = T(L,I) - T(L-1,I)                                     JUN03810

         T(L,I+1) = T(L,I) + T(I,L)/(FEXTRP-ONE)                        JUN03820

   35 CONTINUE                                                          JUN03830

      ERRER = ASTEP*DABS(T(1,L))                                        JUN03840

      IF (L .GT. 2) GO TO 40                                            JUN03850

      HRERR = TABS+P1*DABS(T(1,2))                                      JUN03860

      IF (HRERR .EQ. TABS) GO TO 135                                    JUN03870

      GO TO 15                                                          JUN03880

   40 DO 45 I=2,LM1                                                     JUN03890

         DIFF = ZERO                                                    JUN03900

         HRERR = TABTLM+DABS(T(I-1,L))                                  JUN03910

         IF (HRERR .NE. TABTLM) DIFF = T(I-1,LM1)/T(I-1,L)              JUN03920

         T(I-1,LM1) = DIFF                                              JUN03930

   45 CONTINUE                                                          JUN03940

      IF (DABS(FOUR-T(1,LM1)) .LE. H2TOL) GO TO 60                      JUN03950

      IF (T(1,LM1) .EQ. ZERO) GO TO 55                                  JUN03960

      IF (DABS(TWO-DABS(T(1,LM1))) .LT. JUMPTL) GO TO 130               JUN03970

      IF (L .EQ. 3) GO TO 15                                            JUN03980

      H2CONV = .FALSE.                                                  JUN03990

      IF (DABS((T(1,LM1)-T(1,L-2))/T(1,LM1)) .LE. AITTOL) GO TO 75      JUN04000

   50 IF (REGLAR) GO TO 55                                              JUN04010

      IF (L .EQ. 4) GO TO 15                                            JUN04020

      HRERR = ERGL+ERRER                                                JUN04030

   55 IF (ERRER .GT. ERGOAL .AND. HRERR .NE. ERGL) GO TO 175            JUN04040

      GO TO 145                                                         JUN04050

   60 IF (H2CONV) GO TO 65                                              JUN04060

      AITKEN = .FALSE.                                                  JUN04070

      H2CONV = .TRUE.                                                   JUN04080

   65 FEXTRP = FOUR                                                     JUN04090

   70 IT = IT + 1                                                       JUN04100

      VINT = STEP*T(L,IT)                                               JUN04110

      ERRER = DABS(STEP/(FEXTRP-ONE)*T(IT-1,L))                         JUN04120

      IF (ERRER .LE. ERGOAL) GO TO 160                                  JUN04130

      HRERR = ERGL+ERRER                                                JUN04140

      IF (HRERR .EQ. ERGL) GO TO 160                                    JUN04150

      IF (IT .EQ. LM1) GO TO 125                                        JUN04160

      IF (T(IT,LM1) .EQ. ZERO) GO TO 70                                 JUN04170

      IF (T(IT,LM1) .LE. FEXTRP) GO TO 125                              JUN04180

      IF (DABS(T(IT,LM1)/FOUR-FEXTRP)/FEXTRP .LT. AITTOL)               JUN04190

     1       FEXTRP = FEXTRP*FOUR                                       JUN04200

      GO TO 70                                                          JUN04210

   75 IF (T(1,LM1) .LT. AITLOW) GO TO 175                               JUN04220

      IF (AITKEN) GO TO 80                                              JUN04230

      H2CONV = .FALSE.                                                  JUN04240

      AITKEN = .TRUE.                                                   JUN04250

   80 FEXTRP = T(L-2,LM1)                                               JUN04260

      IF (FEXTRP .GT. FOURP5) GO TO 65                                  JUN04270

      IF (FEXTRP .LT. AITLOW) GO TO 175                                 JUN04280

      IF (DABS(FEXTRP-T(L-3,LM1))/T(1,LM1) .GT. H2TOL) GO TO 175        JUN04290

      SING = FEXTRP                                                     JUN04300

      FEXTM1 = ONE/(FEXTRP - ONE)                                       JUN04310

      AIT(1) = ZERO                                                     JUN04320

      DO 85 I=2,L                                                       JUN04330

         AIT(I) = T(I,1) + (T(I,1)-T(I-1,1))*FEXTM1                     JUN04340

         R(I) = T(1,I-1)                                                JUN04350

         DIF(I) = AIT(I) - AIT(I-1)                                     JUN04360

   85 CONTINUE                                                          JUN04370

      IT = 2                                                            JUN04380

   90 VINT = STEP*AIT(L)                                                JUN04390

      ERRER = ERRER*FEXTM1                                              JUN04400

      HRERR = ERGL+ERRER                                                JUN04410

      IF (ERRER .GT. ERGOAL .AND. HRERR .NE. ERGL) GO TO 95             JUN04420

      IER = MAX0(IER,65)                                                JUN04430

      GO TO 160                                                         JUN04440

   95 IT = IT + 1                                                       JUN04450

      IF (IT .EQ. LM1) GO TO 125                                        JUN04460

      IF (IT .GT. 3) GO TO 100                                          JUN04470

      H2NEXT = FOUR                                                     JUN04480

      SINGNX = SING+SING                                                JUN04490

  100 IF (H2NEXT .LT. SINGNX) GO TO 105                                 JUN04500

      FEXTRP = SINGNX                                                   JUN04510

      SINGNX = SINGNX+SINGNX                                            JUN04520

      GO TO 110                                                         JUN04530

  105 FEXTRP = H2NEXT                                                   JUN04540

      H2NEXT = FOUR*H2NEXT                                              JUN04550

  110 DO 115 I=IT,LM1                                                   JUN04560

         R(I+1) = ZERO                                                  JUN04570

         HRERR = TABTLM+DABS(DIF(I+1))                                  JUN04580

         IF (HRERR .NE. TABTLM) R(I+1) = DIF(I)/DIF(I+1)                JUN04590

  115 CONTINUE                                                          JUN04600

      H2TFEX = -H2TOL*FEXTRP                                            JUN04610

      IF (R(L) - FEXTRP .LT. H2TFEX) GO TO 125                          JUN04620

      IF (R(L-1)-FEXTRP .LT. H2TFEX) GO TO 125                          JUN04630

      ERRER = ASTEP*DABS(DIF(L))                                        JUN04640

      FEXTM1 = ONE/(FEXTRP - ONE)                                       JUN04650

      DO 120 I=IT,L                                                     JUN04660

         AIT(I) = AIT(I) + DIF(I)*FEXTM1                                JUN04670

         DIF(I) = AIT(I) - AIT(I-1)                                     JUN04680

  120 CONTINUE                                                          JUN04690

      GO TO 90                                                          JUN04700

  125 FEXTRP = DMAX1(PREVER/ERRER,AITLOW)                               JUN04710

      PREVER = ERRER                                                    JUN04720

      IF (L .LT. 5) GO TO 15                                            JUN04730

      IF (L-IT .GT. 2 .AND. ISTAGE .LT. MXSTGE) GO TO 170               JUN04740

      ERRET = ERRER/(FEXTRP**(MAXTBL-L))                                JUN04750

      HRERR = ERGL+ERRET                                                JUN04760

      IF (ERRET .GT. ERGOAL .AND. HRERR .NE. ERGL) GO TO 170            JUN04770

      GO TO 15                                                          JUN04780

  130 HRERR = ERGL+ERRER                                                JUN04790

      IF (ERRER .GT. ERGOAL .AND. HRERR .NE. ERGL) GO TO 170            JUN04800

      DIFF = DABS(T(1,L))*(FN+FN)                                       JUN04810

      GO TO 160                                                         JUN04820

  135 SLOPE = (FEND-FBEG)*TWO                                           JUN04830

      FBEG2 = FBEG+FBEG                                                 JUN04840

      DO 140 I=1,4                                                      JUN04850

         DIFF = DABS(F(BEG+RN(I)*STEP) - FBEG2-RN(I)*SLOPE)             JUN04860

         HRERR = TABTLM+DIFF                                            JUN04870

         IF(HRERR .NE. TABTLM) GO TO 155                                JUN04880

  140 CONTINUE                                                          JUN04890

      GO TO 160                                                         JUN04900

  145 SLOPE = (FEND-FBEG)*TWO                                           JUN04910

      FBEG2 = FBEG+FBEG                                                 JUN04920

      I = 1                                                             JUN04930

  150 DIFF = DABS(F(BEG+RN(I)*STEP) - FBEG2-RN(I)*SLOPE)                JUN04940

  155 ERRER = DMAX1(ERRER,ASTEP*DIFF)                                   JUN04950

      HRERR = ERGL+ERRER                                                JUN04960

      IF (ERRER .GT. ERGOAL .AND. HRERR .NE. ERGL) GO TO 175            JUN04970

      I = I+1                                                           JUN04980

      IF (I .LE. 4) GO TO 150                                           JUN04990

      IER = 66                                                          JUN05000

  160 CADRE = CADRE + VINT                                              JUN05010

      ERROR = ERROR + ERRER                                             JUN05020

      IF (RIGHT) GO TO 165                                              JUN05030

      ISTAGE = ISTAGE - 1                                               JUN05040

      IF (ISTAGE .EQ. 0) GO TO 220                                      JUN05050

      REGLAR = REGLSV(ISTAGE)                                           JUN05060

      BEG = BEGIN(ISTAGE)                                               JUN05070

      EDN = FINIS(ISTAGE)                                               JUN05080

      CUREST = CUREST - EST(ISTAGE+1) + VINT                            JUN05090

      IEND = IBEG - 1                                                   JUN05100

      FEND = TS(IEND)                                                   JUN05110

      IBEG = IBEGS(ISTAGE)                                              JUN05120

      GO TO 180                                                         JUN05130

  165 CUREST = CUREST + VINT                                            JUN05140

      STAGE = STAGE+STAGE                                               JUN05150

      IEND = IBEG                                                       JUN05160

      IBEG = IBEGS(ISTAGE)                                              JUN05170

      EDN = BEG                                                         JUN05180

      BEG = BEGIN(ISTAGE)                                               JUN05190

      FEND = FBEG                                                       JUN05200

      FBEG = TS(IBEG)                                                   JUN05210

      GO TO 5                                                           JUN05220

  170 REGLAR = .TRUE.                                                   JUN05230

  175 IF (ISTAGE .EQ. MXSTGE) GO TO 205                                 JUN05240

      IF (RIGHT) GO TO 185                                              JUN05250

      REGLSV(ISTAGE+1) = REGLAR                                         JUN05260

      BEGIN(ISTAGE) = BEG                                               JUN05270

      IBEGS(ISTAGE) = IBEG                                              JUN05280

      STAGE = STAGE*HALF                                                JUN05290

  180 RIGHT = .TRUE.                                                    JUN05300

      BEG = (BEG+EDN)*HALF                                              JUN05310

      IBEG = (IBEG+IEND)/2                                              JUN05320

      TS(IBEG) = TS(IBEG)*HALF                                          JUN05330

      FBEG = TS(IBEG)                                                   JUN05340

      GO TO 10                                                          JUN05350

  185 NNLEFT = IBEG - IBEGS(ISTAGE)                                     JUN05360

      IF (IEND+NNLEFT .GE. MAXTS) GO TO 200                             JUN05370

      III = IBEGS(ISTAGE)                                               JUN05380

      II = IEND                                                         JUN05390

      DO 190 I=III,IBEG                                                 JUN05400

         II = II + 1                                                    JUN05410

         TS(II) = TS(I)                                                 JUN05420

  190 CONTINUE                                                          JUN05430

      DO 195 I=IBEG,II                                                  JUN05440

         TS(III) = TS(I)                                                JUN05450

         III = III + 1                                                  JUN05460

  195 CONTINUE                                                          JUN05470

      IEND = IEND + 1                                                   JUN05480

      IBEG = IEND - NNLEFT                                              JUN05490

      FEND = FBEG                                                       JUN05500

      FBEG = TS(IBEG)                                                   JUN05510

      FINIS(ISTAGE) = EDN                                               JUN05520

      EDN = BEG                                                         JUN05530

      BEG = BEGIN(ISTAGE)                                               JUN05540

      BEGIN(ISTAGE) = EDN                                               JUN05550

      REGLSV(ISTAGE) = REGLAR                                           JUN05560

      ISTAGE = ISTAGE + 1                                               JUN05570

      REGLAR = REGLSV(ISTAGE)                                           JUN05580

      EST(ISTAGE) = VINT                                                JUN05590

      CUREST = CUREST + EST(ISTAGE)                                     JUN05600

      GO TO 5                                                           JUN05610

  200 IER = 131                                                         JUN05620

      GO TO 215                                                         JUN05630

  205 IER = 132                                                         JUN05640

      GO TO 215                                                         JUN05650

  210 IER = 133                                                         JUN05660

  215 CADRE = CUREST + VINT                                             JUN05670

  220 DCADRE = CADRE                                                    JUN05680

 9000 CONTINUE                                                          JUN05690

      IF (IER .NE. 0) CALL UERTST (IER,6HDCADRE)                        JUN05700

 9005 RETURN                                                            JUN05710

      END                                                               JUN05720

      SUBROUTINE IQHSCU (X,Y,NX,C,IC,IER)                               JUN05730

      INTEGER            NX,IC,IER                                      JUN05740

      DOUBLE PRECISION   X(NX),Y(NX),C(IC,3)                            JUN05750

      INTEGER            I,N0                                           JUN05760

      DOUBLE PRECISION   T1,T2,B,RM1,RM2,RM3,RM4,HALF,ONE,ZERO          JUN05770

      DATA               HALF/0.50D0/,ONE/1.0D0/,ZERO/0.0D0/            JUN05780

      IF (IC .GE. NX) GO TO 5                                           JUN05790

      IER = 129                                                         JUN05800

      GO TO 9000                                                        JUN05810

    5 IF (NX .GT. 3) GO TO 10                                           JUN05820

      IER = 130                                                         JUN05830

      GO TO 9000                                                        JUN05840

   10 DO 15 I=2,NX                                                      JUN05850

         IF (X(I) .GT. X(I-1)) GO TO 15                                 JUN05860

         IER = 131                                                      JUN05870

         GO TO 9000                                                     JUN05880

   15 CONTINUE                                                          JUN05890

      IER = 0                                                           JUN05900

      RM3 = (Y(2)-Y(1))/(X(2)-X(1))                                     JUN05910

      T1 = RM3-(Y(2)-Y(3))/(X(2)-X(3))                                  JUN05920

      RM2 = RM3+T1                                                      JUN05930

      RM1 = RM2+T1                                                      JUN05940

      N0 = NX-2                                                         JUN05950

      DO 40 I=1,NX                                                      JUN05960

         IF (I .GT. N0) GO TO 20                                        JUN05970

         RM4 = (Y(I+2)-Y(I+1))/(X(I+2)-X(I+1))                          JUN05980

         GO TO 25                                                       JUN05990

   20    RM4 = RM3-RM2+RM3                                              JUN06000

   25    T1 = DABS(RM4-RM3)                                             JUN06010

         T2 = DABS(RM2-RM1)                                             JUN06020

         B = T1+T2                                                      JUN06030

         IF (B .NE. ZERO) GO TO 30                                      JUN06040

         C(I,1) = HALF*(RM2+RM3)                                        JUN06050

         GO TO 35                                                       JUN06060

   30    C(I,1) = (T1*RM2+T2*RM3)/B                                     JUN06070

   35    RM1 = RM2                                                      JUN06080

         RM2 = RM3                                                      JUN06090

         RM3 = RM4                                                      JUN06100

   40 CONTINUE                                                          JUN06110

      N0 = NX-1                                                         JUN06120

      DO 45 I=1,N0                                                      JUN06130

         T1 = ONE/(X(I+1)-X(I))                                         JUN06140

         T2 = (Y(I+1)-Y(I))*T1                                          JUN06150

         B = (C(I,1)+C(I+1,1)-T2-T2)*T1                                 JUN06160

         C(I,3) = B*T1                                                  JUN06170

         C(I,2) = -B+(T2-C(I,1))*T1                                     JUN06180

   45 CONTINUE                                                          JUN06190

 9000 CONTINUE                                                          JUN06200

      IF (IER .NE. 0) CALL UERTST(IER,6HIQHSCU)                         JUN06210

 9005 RETURN                                                            JUN06220

      END                                                               JUN06230

      SUBROUTINE UERSET (LEVEL,LEVOLD)                                  JUN06240

      INTEGER            LEVEL,LEVOLD                                   JUN06250

      LEVOLD = LEVEL                                                    JUN06260

      CALL UERTST (LEVOLD,6HUERSET)                                     JUN06270

      RETURN                                                            JUN06280

      END                                                               JUN06290

      SUBROUTINE UERTST (IER,NAME)                                      JUN06300

      INTEGER            IER                                            JUN06310

      INTEGER            NAME(1)                                        JUN06320

      INTEGER            I,IEQ,IEQDF,IOUNIT,LEVEL,LEVOLD,NAMEQ(6),      JUN06330

     *                   NAMSET(6),NAMUPK(6),NIN,NMTB                   JUN06340

      DATA               NAMSET/1HU,1HE,1HR,1HS,1HE,1HT/                JUN06350

      DATA               NAMEQ/6*1H /                                   JUN06360

      DATA               LEVEL/4/,IEQDF/0/,IEQ/1H=/                     JUN06370

      CALL USPKD (NAME,6,NAMUPK,NMTB)                                   JUN06380

      CALL UGETIO(1,NIN,IOUNIT)                                         JUN06390

      IF (IER.GT.999) GO TO 25                                          JUN06400

      IF (IER.LT.-32) GO TO 55                                          JUN06410

      IF (IER.LE.128) GO TO 5                                           JUN06420

      IF (LEVEL.LT.1) GO TO 30                                          JUN06430

      IF (IEQDF.EQ.1) WRITE(IOUNIT,35) IER,NAMEQ,IEQ,NAMUPK             JUN06440

      IF (IEQDF.EQ.0) WRITE(IOUNIT,35) IER,NAMUPK                       JUN06450

      GO TO 30                                                          JUN06460

    5 IF (IER.LE.64) GO TO 10                                           JUN06470

      IF (LEVEL.LT.2) GO TO 30                                          JUN06480

      IF (IEQDF.EQ.1) WRITE(IOUNIT,40) IER,NAMEQ,IEQ,NAMUPK             JUN06490

      IF (IEQDF.EQ.0) WRITE(IOUNIT,40) IER,NAMUPK                       JUN06500

      GO TO 30                                                          JUN06510

   10 IF (IER.LE.32) GO TO 15                                           JUN06520

      IF (LEVEL.LT.3) GO TO 30                                          JUN06530

      IF (IEQDF.EQ.1) WRITE(IOUNIT,45) IER,NAMEQ,IEQ,NAMUPK             JUN06540

      IF (IEQDF.EQ.0) WRITE(IOUNIT,45) IER,NAMUPK                       JUN06550

      GO TO 30                                                          JUN06560

   15 CONTINUE                                                          JUN06570

      DO 20 I=1,6                                                       JUN06580

         IF (NAMUPK(I).NE.NAMSET(I)) GO TO 25                           JUN06590

   20 CONTINUE                                                          JUN06600

      LEVOLD = LEVEL                                                    JUN06610

      LEVEL = IER                                                       JUN06620

      IER = LEVOLD                                                      JUN06630

      IF (LEVEL.LT.0) LEVEL = 4                                         JUN06640

      IF (LEVEL.GT.4) LEVEL = 4                                         JUN06650

      GO TO 30                                                          JUN06660

   25 CONTINUE                                                          JUN06670

      IF (LEVEL.LT.4) GO TO 30                                          JUN06680

      IF (IEQDF.EQ.1) WRITE(IOUNIT,50) IER,NAMEQ,IEQ,NAMUPK             JUN06690

      IF (IEQDF.EQ.0) WRITE(IOUNIT,50) IER,NAMUPK                       JUN06700

   30 IEQDF = 0                                                         JUN06710

      RETURN                                                            JUN06720

   35 FORMAT(19H *** TERMINAL ERROR,10X,7H(IER = ,I3,                   JUN06730

     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        JUN06740

   40 FORMAT(27H *** WARNING WITH FIX ERROR,2X,7H(IER = ,I3,            JUN06750

     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        JUN06760

   45 FORMAT(18H *** WARNING ERROR,11X,7H(IER = ,I3,                    JUN06770

     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        JUN06780

   50 FORMAT(20H *** UNDEFINED ERROR,9X,7H(IER = ,I5,                   JUN06790

     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        JUN06800

   55 IEQDF = 1                                                         JUN06810

      DO 60 I=1,6                                                       JUN06820

   60 NAMEQ(I) = NAMUPK(I)                                              JUN06830

   65 RETURN                                                            JUN06840

      END                                                               JUN06850

      SUBROUTINE UGETIO(IOPT,NIN,NOUT)                                  JUN06860

      INTEGER            IOPT,NIN,NOUT                                  JUN06870

      INTEGER            NIND,NOUTD                                     JUN06880

      DATA               NIND/1/,NOUTD/2/                               JUN06890

      IF (IOPT.EQ.3) GO TO 10                                           JUN06900

      IF (IOPT.EQ.2) GO TO 5                                            JUN06910

      IF (IOPT.NE.1) GO TO 9005                                         JUN06920

      NIN = NIND                                                        JUN06930

      NOUT = NOUTD                                                      JUN06940

      GO TO 9005                                                        JUN06950

    5 NIND = NIN                                                        JUN06960

      GO TO 9005                                                        JUN06970

   10 NOUTD = NOUT                                                      JUN06980

 9005 RETURN                                                            JUN06990

      END                                                               JUN07000

      SUBROUTINE USPKD  (PACKED,NCHARS,UNPAKD,NCHMTB)                   JUN07010

      INTEGER            NC,NCHARS,NCHMTB                               JUN07020

      INTEGER            UNPAKD(1),IBLANK                               JUN07030

      INTEGER            PACKED(1)                                      JUN07040

      DATA               IBLANK /1H /                                   JUN07050

      NCHMTB = 0                                                        JUN07060

      IF(NCHARS.LE.0) RETURN                                            JUN07070

      NC = MIN0 (129,NCHARS)                                            JUN07080

C     DECODE (NC,150,PACKED) (UNPAKD(I),I=1,NC)                         JUN07090

  150 FORMAT (129A1)                                                    JUN07100

      DO 200 N = 1,NC                                                   JUN07110

         NN = NC - N + 1                                                JUN07120

         IF(UNPAKD(NN) .NE. IBLANK) GO TO 210                           JUN07130

  200 CONTINUE                                                          JUN07140

  210 NCHMTB = NN                                                       JUN07150

      RETURN                                                            JUN07160

      END                                                               JUN07170

