C    ONE LINE CHANGED 3/3/87
C        RHSM RECALCULATED TO TAKE ACCOUNT OF PETZ. ERR EST
C        WHEN ORDER STAYS THE SAME.
C    LINES CHANGED 13/3/87
C        ON THIRD ERROR TEST FAILURE THE ORDER IS DROPPED TO ONE AND
C        THE STEPSIZE REDUCED BY A FACTOR OF 0.2
C
C    LINE CHANGES TO TAKE ACCOUNT OF REVISED BACKSUB PROCEDURE
C    APRIL 88.
C
C    C1 INTERPOLATION INTRODUCED IN JULY 1988.
C
      SUBROUTINE BDFSET( NY2DIM, IRET, MAXORD, METH, PETZLD)
      INTEGER METH, MAXORD,  IRET, NY2DIM
      LOGICAL PETZLD
C----------------------------------------------------------------------
C   THIS ROUTINE INITIALISES THE LSODI IMPLENTATION OF ADAMS/B.D.F
C   METHODS AS USED IN THE SPRINT INTEGRATION PACKAGE
C   THE INPUT PARAMETERS ARE:
C   NY2DIM     THE SECOND DIMENSION OF THE YSAVE ARRAY , IT SHOULD BE
C              SET TO MAXORD + 1, UNLESS MAXORD IS ZERO ,SEE BELOW.
C              N.B. IN THE CASE WHEN NY2DIM IS SET TO MAXORD + 2 THEN
C                   C1 INTERPOLATION WILL BE USED TO CALCULATE SOLUTION
C                   VALUES AT NON-MESH POINTS.
C
C   MAXORD     THE MAXIMUM ORDER TO BE USED. THIS SHOULD BE POSITIVE AND
C              FOR THE ADAMS METHODS LESS THAN 12 ; FOR THE B.D.F.
C              METHODS THIS SHOULD BE LESS THAN 6 . WHEN MAXORD IS SET
C              TO ZERO THE DEFAULT VALUES OF 11 (ADAMS) AND 5 (B.D.F.)
C              WILL BE USED BY THE CODE (NY2DIM MUST BE LARGE ENOUGH
C              FOR THESE DEFAULT VALUES).
C
C          N.B. THE SIZE OF THE YSAVE ARRAY PASSED INTO SPRINT MUST BE
C              YSAVE(NYH, MAXORD + 1). WHERE NYH IS THE NUMBER OF
C              DIFFERENTIAL ALGEBRAIC EQUATIONS OR AN UPPER BOUND ON
C              THIS NUMBER .
C   METH     METH INDICATES THE BASIC LINEAR MULTISTEP METHOD..
C              METH = 1 MEANS THE ADAMS MOULTON METHOD.
C              METH = 2 MEANS THE METHOD BASED ON BACKWARD
C                       DIFFERENTIATION FORMULAS (BDF-S).
C                THE BDF METHOD IS STRONGLY PREFERRED FOR STIFF PROB-
C              LEMS, WHILE THE ADAMS METHOD IS PREFERRED WHEN THE PROB-
C              LEM IS NOT STIFF. IF THE MATRIX A(T,Y) IS NONSINGULAR,
C              STIFFNESS HERE CAN BE TAKEN TO MEAN THAT OF THE EXPLICIT
C              ODE SYSTEM DY/DT = A**(-1) * G.  IF A IS SINGULAR, THE
C              CONCEPT OF STIFFNESS IS NOT WELL DEFINED.
C              IF MAXIMUM EFFICIENCY IS IMPORTANT, SOME EXPERIMENTATION
C              WITH METH MAY BE NECESSARY.
C
C              METH = 10 , AS FOR METH = 1 BUT USING FUNCTIONAL
C                  ITERATION TO SOLVE THE SYSTEMS OF NONLINEAR EQUATIONS
C
C              METH = 20 , AS FOR METH = 2 BUT USING FUNCTIONAL
C                  ITERATION TO SOLVE THE SYSTEM OF NONLINEAR EQUATIONS.
C
C              NOTE : THE OPTIONS METH = 10 AND METH = 20 SHOULD BE USED
C                     WITH GREAT CARE , PARTICULARLY IF THE PROBLEM
C                     CONSISTS OF BOTH ALGEBRAIC AND DIFFERENTIAL EQNS.
C                     SPRINT ITSELF MUST BE CALLED WITH FULL BANDED OR
C                     SPARSE LINEAR ALGEBRA ROUTINES AS SPGEAR WILL USE
C                     A NEWTON METHOD IF IT ENCOUNTERS CONVERGENCE
C                     DIFFICULTIES WHEN USING METH = 10 OR 20.
C              IN ORDER TO CHANGE THE VALUES OF METH SIMPLY RECALL THIS
C              ROUTINE , EVEN IN MID-INTEGRATION.
C
C   PETZLD     LOGICAL INDICATOR TO DEFINE WHETHER OR NOT THE PETZOLD
C              FORM OF THE LOCAL ERROR TESTB IS USED . THIS TEST IS MORE
C              EXPENSIVE BUT MORE STABLE FOR DIFFERENTIAL ALGEBRAIC EQNS
C              THIS PARAMETER MUST BE SET BY THE USER.
C              = .TRUE. MEANS THAT THE TEST IS USED
C              = .FALSE. MEANS THAT THE TEST IS NOT USED .
C              THIS INDICATOR CANNOT BE SET = .TRUE. IF METH = 10 OR 20
C              IS USED ABOVE.
C
C  OUTPUT PARAMETER
C   IRET       INTEGER FLAG SET TO ZERO IF ALL INPUTS ARE O.K.
C              OTHERWISE SET TO -1  TO INDICATE AN ERROR EXIT.
C----------------------------------------------------------------------
      INTEGER IOWNS, NMETH, NMITER, NMXORD, IDM, I, MORD(2)
      LOGICAL TRUPTZ, FNITER, C1INTR
      DOUBLE PRECISION ROWNS, HHUSED, CCMAX
      CHARACTER*6 ODCODE
      COMMON /ODECHK/ ODCODE
      COMMON /LS0001/ ROWNS(211), IOWNS(8), NMETH, NMITER,NMXORD, IDM(3)
      COMMON /LS001A/ TRUPTZ, FNITER, HHUSED, C1INTR
      COMMON /SPCCMX/ CCMAX
      SAVE   /LS001A/, /LS0001/, /ODECHK/, /SPCCMX/
      DATA MORD(1),MORD(2)/12,5/
C
      IRET = 0
      IF(METH .EQ. 10 .OR. METH .EQ. 20)THEN
         FNITER = .TRUE.
         NMETH   = METH / 10
         IF(PETZLD)THEN
            CALL SERROR(' BDFSET ERROR - THE LOGICAL PARAMETER PETZLD
     1      HAS BEEN ILEGALLY SET TO .TRUE. WHEN THE ROUTINE HAS BEEN
     2      CALLED WITH METH = 10 OR METH = 20 AND HAS BEEN SET TO
     3      FALSE ', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
            TRUPTZ = .FALSE.
         END IF
       ELSE
         IF(METH.NE.1.AND.METH.NE.2.) THEN
           CALL SERROR(' BDFSET- VALUE OF METH (=I1) IS ILLEGAL AND HAS
     1            BEEN INTERNALLY RESET TO 1 ', 1, 1, METH,
     2            0, 0, 0.0D0, 0.0D0)
           NMETH = 1
         ELSE
           NMETH = METH
         END IF
         TRUPTZ = PETZLD
         FNITER = .FALSE.
       END IF
       NMITER = 2
       CCMAX  = 0.3D0
C
C       IDM(2) CONTAINS THE PARAMETER WHICH SPECIFIES HOW MANY
C       CONVERGENCE FAILURES ARE ALLOWED.
C
       IDM(2) = 5
       IF (MAXORD.LT.0)THEN
          CALL SERROR(' BDFSET - NEG VALUE OF MAXORD (=I1) IS ILLEGAL'
     1    , 1, 1, MAXORD, 0, 0, 0.0D0, 0.0D0)
          IRET = -1
       END IF
       IF ( MAXORD.EQ.0)THEN
            MAXORD = MORD(METH)
       ELSE
            IF(MAXORD .GT. MORD(METH))THEN
            CALL SERROR(' BDFSET WARNING USER SUPPLIED MAXORD (=I1) IS
     1 GREATER THAN ALLOWED (=I2) AND IS RESET TO THE ALLOWED MAX',
     2 1, 2, MAXORD, MORD(METH), 0, 0.0D0, 0.0D0)
            END IF
            MAXORD = MIN0(MAXORD,MORD(METH))
       END IF
       IF(NY2DIM .LT. (MAXORD+1)) THEN
          I = MAXORD + 1
          CALL SERROR(' BDFSET - ERROR USER DEFINED VALUE OF NY2DIM
     1                  (=I1) IS SMALLER THAN THE REQUIRED VALUE (=I2)
     2                  INCREASE THE SECOND DIMENSION TO THIS VALUE ', 1
     3                 ,2 , NY2DIM, I, 0, 0.0D0, 0.0D0)
          IRET = -1
       ELSE
          ODCODE ='SPGEAR'
          C1INTR =.FALSE.
          IF( NY2DIM .GE. (MAXORD+2))C1INTR = .TRUE.
       END IF
       NMXORD = MAXORD
       RETURN
C------END-OF-ROUTINE-BDFSET-------------------------------------------
       END
      SUBROUTINE SPGEAR (NEQ, Y, YH, NYH, EWT, YDOT, SAVR, ACOR, INLN,
     1                  ISTEP, EL0, H, TN, HMIN, HMXI, IDAE)
      INTEGER NEQ, NYH, INLN, ISTEP, IDAE(1)
      INTEGER IALTH, IPUP, LMAX, MEO,  NSLP, ICF, JCUR,
     1   L, METH, MITER, MAXORD, MSBP, MXNCF, N,
     2   NQ, NST, NRE, NJE, NQU, NITER, JSTART, KFLAG, NINTER, IOVFLO
      INTEGER I,JC,IREDO, IRET, J, JB, KGO, NCF, NEWQ,ITRACE,IDEV,IZ
      LOGICAL PETZLD, RAISE, FNITER, C1INTR
      DOUBLE PRECISION Y, YH, EWT, YDOT, SAVR, ACOR, EL0, H, TN, HMXI,
     1   HMIN, ROWND
      DOUBLE PRECISION CONIT, EL, ELCO, HOLD, RMAX, TESCO, CCMAX, RC
      DOUBLE PRECISION DDN, DSM, DUP, DSN, TERK, TERKM1, TERKM2, DDN1,
     1   ELJH,  EXDN, EXSM, EXUP, TERKP1, CCMX1,
     2   R, RH, RHDN, RHSM, RHUP, TOLD, VNORM, HHUSED, DUN, U
      CHARACTER*6 ODCODE
      DIMENSION NEQ(1),Y(1),YH(NYH,1),EWT(1),YDOT(1),SAVR(1),ACOR(1)
      COMMON /LS0001/ ROWND, CONIT, EL(13), ELCO(13,12),
     1   HOLD, RMAX, TESCO(3,12), CCMAX, RC,
     2   IALTH, IPUP, LMAX, MEO, NSLP,
     3   ICF, JCUR, L, METH, MITER, MAXORD, MSBP, MXNCF, N
      COMMON /ODECHK/ ODCODE
      COMMON /LS001A/ PETZLD, FNITER, HHUSED, C1INTR
      COMMON /LSTATS/ NQ, NQU, NST, NRE, NJE, NITER, NINTER
      COMMON /SPCCMX/ CCMX1
C
C      EXTRA COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /LSSAVE/  DDN, DSM, DUP, ELJH, TERK, TERKM1, TERKP1,
     1   EXDN, EXSM, EXUP, R, RH, RHDN, RHSM, RHUP, TOLD, TERKM2, DDN1,
     2   I, JC, IREDO, IRET, J, JB, KGO, NCF, IZ, JSTART, KFLAG, RAISE
      COMMON/SDEV2/ITRACE,IDEV
      COMMON /SCONS1/ DUN, U, IOVFLO
      SAVE   /LSSAVE/, /LSTATS/, /LS001A/, /ODECHK/, /LS0001/, /SCONS1/
     1      ,/SPCCMX/
C-----------------------------------------------------------------------
C SPGEAR PERFORMS ONE STEP OF THE INTEGRATION OF AN INITIAL VALUE
C PROBLEM FOR A SYSTEM OF ORDINARY DIFFERENTIAL EQUATIONS.
C NOTE.. SPGEAR IS INDEPENDENT OF THE VALUE OF THE ITERATION METHOD
C INDICATOR MITER, AND HENCE IS INDEPENDENT
C OF THE TYPE OF CHORD METHOD USED, OR THE JACOBIAN STRUCTURE.
C COMMUNICATION WITH SPGEAR IS DONE WITH THE FOLLOWING VARIABLES..
C
C NEQ    = INTEGER ARRAY CONTAINING PROBLEM SIZE IN NEQ(1), AND
C          NOT USED EXCEPT TO INITIALISE THE COMMON BLOCK VARIABLE N
C Y      = AN ARRAY OF LENGTH .GE. N USED AS THE Y ARGUMENT IN
C          ALL CALLS TO RES, JAC, AND ADDA. ON THE FIRST CALL THIS
C          ARRAY IS ASSUMED TO CONTAIN THE INITIAL SOLUTION VALUES.
C YH     = AN NYH BY LMAX ARRAY CONTAINING THE DEPENDENT VARIABLES
C          AND THEIR APPROXIMATE SCALED DERIVATIVES, WHERE
C          LMAX = MAXORD + 1.  YH(I,J+1) CONTAINS THE APPROXIMATE
C          J-TH DERIVATIVE OF Y(I), SCALED BY H**J/FACTORIAL(J)
C          (J = 0,1,...,NQ).  ON ENTRY FOR THE FIRST STEP, THE FIRST
C          TWO COLUMNS OF YH MUST BE SET FROM THE INITIAL VALUES.
C NYH    = A CONSTANT INTEGER .GE. N, THE FIRST DIMENSION OF YH.
C EWT    = AN ARRAY OF LENGTH N CONTAINING MULTIPLICATIVE WEIGHTS
C          FOR LOCAL ERROR MEASUREMENTS.  LOCAL ERRORS IN Y(I) ARE
C          COMPARED TO 1.0/EWT(I) IN VARIOUS ERROR TESTS.
C YDOT   = AN ARRAY OF WORKING STORAGE, OF LENGTH N. ALSO USED FOR
C          INPUT OF YH(*,MAXORD+2) WHEN JSTART = -1 AND MAXORD IS LESS
C          THAN THE CURRENT ORDER NQ. ON THE FIRST CALL THIS ARRAY IS
C          ASSUMED TO HOLD THE INITIAL VALUES OF THE TIME DERIVATIVE.
C SAVR   = AN ARRAY OF WORKING STORAGE, OF LENGTH N.
C          THIS ARRAY IS NOT USED IN THE PRESENT IMPLEMENTATION.
C ACOR   = A WORK ARRAY OF LENGTH N USED FOR THE ACCUMULATED
C          CORRECTIONS. ON A SUCCESFUL RETURN, ACOR(I) CONTAINS
C          THE ESTIMATED ONE-STEP LOCAL ERROR IN Y(I).
C CCMAX  = MAXIMUM RELATIVE CHANGE IN H*EL0 BEFORE PJAC IS CALLED.
C H      = THE STEP SIZE TO BE ATTEMPTED ON THE NEXT STEP.
C          H IS ALTERED BY THE ERROR CONTROL ALGORITHM DURING THE
C          PROBLEM.  H CAN BE EITHER POSITIVE OR NEGATIVE, BUT ITS
C          SIGN MUST REMAIN CONSTANT THROUGHOUT THE PROBLEM.
C HMIN   = THE MINIMUM ABSOLUTE VALUE OF THE STEP SIZE H TO BE USED.
C HMXI   = INVERSE OF THE MAXIMUM ABSOLUTE VALUE OF H TO BE USED.
C          HMXI = 0.0 IS ALLOWED AND CORRESPONDS TO AN INFINITE HMAX.
C          HMIN AND HMXI MAY BE CHANGED AT ANY TIME, BUT WILL NOT
C          TAKE EFFECT UNTIL THE NEXT CHANGE OF H IS CONSIDERED.
C IDAE     INDICATOR ARRAY OF DIMENSION NEQ(1) NOT USED IN THIS
C          MODULE.
C TN     = THE INDEPENDENT VARIABLE. TN IS UPDATED ON EACH STEP TAKEN.
C ISTEP    : INPUT AND OUTPUT ERROR INDICATOR
C          ON INPUT ITS VALUES ARE TRANSFERRED INTO JSTART
C          ON OUTPUT IT TAKES THE VALUES OF KFLAG . IN BOTH CASES
C          THE ACTUAL VALUES ARE MODIFIED SO AS TO CORRESSPOND TO
C          THE SPRINT INTERFACE.
C    INPUT:    -1  PERFORM THE FIRST STEP  (JSTART = 0).
C               0  REVERSE COMMUNICATION RETURN FROM NLSLVR ROUTINE.
C               1  TAKE A NEW STEP CONTINUING FROM THE LAST (JSTART=1)
C               2  TAKE THE NEXT STEP WITH A NEW VALUE OF H, MAXORD,
C                    N, METH, (JSTART = -1).
C               3  TAKE THE NEXT STEP WITH A NEW VALUE OF H,
C                    BUT WITH OTHER INPUTS UNCHANGED (JSTART = -2)
C          ON RETURN, ISTEP  IS SET TO 1 TO FACILITATE CONTINUATION.
C    OUTPUT:    1  THE STEP WAS SUCCESSFUL , KFLAG = 0.
C               0  REVERSE COMMUNICATION RETURN - CHECK INLN.
C              -1  THE REQUESTED ERROR COULD NOT BE ACHIEVED.
C              -2  CORRECTOR CONVERGENCE COULD NOT BE ACHIEVED.
C              -3  RES ORDERED IMMEDIATE RETURN.
C              -4  ERROR CONDITION FROM RES COULD NOT BE AVOIDED.
C              -5  FATAL ERROR IN JACOBIAN FORMING OR BACKSUBSTITUTION.
C              -6  INIT MODULE LSET WAS NOT CALLED PRIOR TO FIRST ENTRY.
C              -7  WORKSPACE ERROR IN LINEAR ALGEBRA SECTION .
C APART FROM WHEN ISTEP = 0 OR 1 , ISTEP = KFLAG ON OUTPUT.
C          A RETURN WITH ISTEP = -1, -2, OR -4 MEANS EITHER
C          ABS(H) = HMIN OR 10 CONSECUTIVE FAILURES OCCURRED.
C          ON A RETURN WITH ISTEP NEGATIVE, THE VALUES OF TN AND
C          THE YH ARRAY ARE AS OF THE BEGINNING OF THE LAST
C          STEP, AND H IS THE LAST STEP SIZE ATTEMPTED.
C MAXORD = THE MAXIMUM ORDER OF INTEGRATION METHOD TO BE ALLOWED.
C MSBP   = MAXIMUM NUMBER OF STEPS BETWEEN PJAC CALLS.
C MXNCF  = MAXIMUM NUMBER OF CONVERGENCE FAILURES ALLOWED.
C METH/MITER = THE METHOD FLAGS.  SEE DESCRIPTION IN ROUTINE LSET.
C N      = THE NUMBER OF FIRST-ORDER DIFFERENTIAL EQUATIONS.
C
C PETZLD IN COMMON /LS001A/ SET TRUE IF PETZOLD ERROR TEST IS USED .
C       OTHERWISE SET TO FALSE, INITIALISE IN LSET ROUTINE.
C
C INLN       = REVERSE COMMUNICATION INDICATOR
C
C       ON ENTRY   INLN  = 0  : NORMAL ENTRY
C                        = -1 : ERROR IN JACOBIAN FORMATION
C                        = -2 : RESID ORDERED RETURN TO CALLING PROG.
C                        = -3 : RESID DETECTED ILLEGAL T, Y OR YDOTI.
C                        = -4 : RESID ORDERED ENTRY TO MONITR.
C                        = -5 ; WORKSPACE ERROR IN NONLINEAR SOLVER.
C                        = 1  : NONLINEAR SYSTEM SOLVED
C                        = 2  : ITERATION FAILED TO CONVERGE IN
C                               SOLUTION OF NONLINEAR SYSTEM
C                        = 3,4  NOT USED IN THIS MODULE
C                        = 5    RETURN TO FORM THE PETZOLD ERROR EST
C                               I.E. J INVERSE * DF/DYDOT * ACOR
C                        = 6    NONLINEAR SYSTEM OF EQUATIONS SOLVED
C                               USING FUNCTIONAL ITERATION.
C
C
C       ON EXIT    INLN  = 0  : NORMAL EXIT
C                        = 1  : FORM JACOBIAN AND SOLVE NONLINEAR SYSTEM
C                        = 2  : AS FOR INLN = 1 BUT USING OLD JACOBIAN
C                        = 3,4 NOT USED  HERE.
C                        = 5   FORM PETZOLD ERROR ESTIMATE.
C                        = 6  SOLVE SYSTEM OF NONLINEAR EQUATIONS
C                             USING FUNCTIONAL ITERATION.
C                        = 7   NOT USED HERE
C
C
C-----------------------------------------------------------------------
C
C     JUMP IF IT IS REVERSE COMMUNICATION
C
      IZ = INLN  + 5
      INLN  = 0
      GOTO ( 430, 430, 430, 430, 10, 450, 410, 10, 10, 4501, 450),IZ
      GOTO   435
10    KFLAG = 0
      IF(ODCODE .NE. 'SPGEAR')THEN
         CALL SERROR(' SPGEAR-SETUP MODULE BDFSET WAS NOT CALLED BEFORE
     1   ENTRY TO SPRINT- ERROR', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         ISTEP = -6
         RETURN
      END IF
      TOLD = TN
      NCF = 0
      JCUR = 0
      ICF = 0
      IF(ISTEP .EQ. 1)THEN
         JSTART = 1
         GOTO 200
      ELSE IF(ISTEP .EQ. 2)THEN
              JSTART = -1
              GOTO 100
           ELSE IF(ISTEP .EQ. 3)THEN
                   JSTART = -2
                   GOTO 160
      END IF
      JSTART = 0
C
C-----------------------------------------------------------------------
C ON THE FIRST CALL, THE ORDER IS SET TO 1, AND OTHER VARIABLES ARE
C INITIALIZED.  RMAX IS THE MAXIMUM RATIO BY WHICH H CAN BE INCREASED
C IN A SINGLE STEP.  IT IS INITIALLY 1.E4 TO COMPENSATE FOR THE SMALL
C INITIAL H, BUT THEN IS NORMALLY EQUAL TO 10.  IF A FAILURE
C OCCURS (IN CORRECTOR CONVERGENCE OR ERROR TEST), RMAX IS SET AT 2
C FOR THE NEXT INCREASE.
C-----------------------------------------------------------------------
      LMAX = MAXORD + 1
      NINTER = LMAX
      N = NEQ(1)
      NQ = 1
      NQU = 0
      L = 2
      IALTH = 2
      RMAX = 10000.0D0
      RC = 0.0D0
      HHUSED = 0.0D0
      EL0 = 1.0D0
      MSBP = 20
      CCMAX = CCMX1
      HOLD = H
      MEO = METH
      NSLP = 0
      IPUP = MITER
      IRET = 3
C
C   LOAD THE INITIAL VALUES OF THE SOLUTION AND THE TIME DERIVATIVE
C   INTO THE NORDSIECK VECTOR.
C
      DO 20 I = 1,N
         YH(I,1) = Y(I)
         YH(I,2) = YDOT(I)*H
 20   CONTINUE
      GO TO 140
C-----------------------------------------------------------------------
C THE FOLLOWING BLOCK HANDLES PRELIMINARIES NEEDED WHEN JSTART = -1.
C IPUP IS SET TO MITER TO FORCE A MATRIX UPDATE.
C IF AN ORDER INCREASE IS ABOUT TO BE CONSIDERED (IALTH = 1),
C IALTH IS RESET TO 2 TO POSTPONE CONSIDERATION ONE MORE STEP.
C IF THE CALLER HAS CHANGED METH, DFODE IS CALLED TO RESET
C THE COEFFICIENTS OF THE METHOD.
C IF THE CALLER HAS CHANGED MAXORD TO A VALUE LESS THAN THE CURRENT
C ORDER NQ, NQ IS REDUCED TO MAXORD, AND A NEW H CHOSEN ACCORDINGLY.
C IF H IS TO BE CHANGED, YH MUST BE RESCALED.
C IF H OR METH IS BEING CHANGED, IALTH IS RESET TO L = NQ + 1
C TO PREVENT FURTHER CHANGES IN H FOR THAT MANY STEPS.
C-----------------------------------------------------------------------
C100  IPUP = MITER
 100  LMAX = MAXORD + 1
      IF( NEQ(1) .NE. N)IPUP = MITER
      NINTER = LMAX
      IF( C1INTR ) NINTER = NINTER + 1
      IF(NEQ(1) .LT. N)THEN
C        NUMBER OF O.D.E.S HAS BEEN REDUCED- ZERO PART OF MEMORY VECTOR
         J = NEQ(1) + 1
         DO 105 I = J,N
            DO 105 JB = 1,NQ+1
               YH(I,JB) = 0.0D0
105      CONTINUE
      ELSE IF(NEQ(1) .GT. N .AND. ITRACE.GE. 1)THEN
              CALL SERROR(' SPGEAR-VALUE OF NEQ(1) (=I1) GT N(=I2)', 1,
     1             2, NEQ(1), N, 0, 0.0D0, 0.0D0)
      END IF
      N = NEQ(1)
      IF (IALTH .EQ. 1) IALTH = 2
      IF (METH .EQ. MEO) GO TO 110
      CALL DFODE (METH, ELCO, TESCO)
      MEO = METH
      IF (NQ .GT. MAXORD) GO TO 120
      IALTH = L
      IRET = 1
      GO TO 150
 110  IF (NQ .LE. MAXORD) GO TO 160
 120  NQ = MAXORD
C
C   MAXORD HAS BEEN REDUCED TO BELOW NQ
C
      J = MAXORD + 2
      L = LMAX
      DO 125 I = 1,L
 125    EL(I) = ELCO(I,NQ)
      RC = RC*EL(1)/EL0
      EL0 = EL(1)
      CONIT = 0.5D0/DFLOAT(NQ+2)
      DDN = VNORM (N, YH(1,J), EWT)/TESCO(1,L)
      EXDN = 1.0D0/DFLOAT(L)
      RHDN = 1.0D0/(1.3D0*DDN**EXDN + 0.0000013D0)
      RH = DMIN1(RHDN,1.0D0)
      IREDO = 3
      IF (H .EQ. HOLD) GO TO 170
      RH = DMIN1(RH,DABS(H/HOLD))
      H = HOLD
      GO TO 175
C-----------------------------------------------------------------------
C DFODE IS CALLED TO GET ALL THE INTEGRATION COEFFICIENTS FOR THE
C CURRENT METH.  THEN THE EL VECTOR AND RELATED CONSTANTS ARE RESET
C WHENEVER THE ORDER NQ IS CHANGED, OR AT THE START OF THE PROBLEM.
C-----------------------------------------------------------------------
 140  CALL DFODE (METH, ELCO, TESCO)
 150  DO 155 I = 1,L
 155    EL(I) = ELCO(I,NQ)
      RC = RC*EL(1)/EL0
      EL0 = EL(1)
      CONIT = 0.5D0/DFLOAT(NQ+2)
      GO TO (160, 170, 200), IRET
C-----------------------------------------------------------------------
C IF H IS BEING CHANGED, THE H RATIO RH IS CHECKED AGAINST
C RMAX, HMIN, AND HMXI, AND THE YH ARRAY RESCALED.  IALTH IS SET TO
C L = NQ + 1 TO PREVENT A CHANGE OF H FOR THAT MANY STEPS, UNLESS
C FORCED BY A CONVERGENCE OR ERROR TEST FAILURE.
C-----------------------------------------------------------------------
 160  IF (H .EQ. HOLD) GO TO 200
      RH = H/HOLD
      H = HOLD
      IREDO = 3
      GO TO 175
 170  RH = DMAX1(RH,HMIN/DABS(H))
 175  RH = DMIN1(RH,RMAX)
      RH = RH/DMAX1(1.0D0,DABS(H)*HMXI*RH)
      R = 1.0D0
C SCALE THE NORDSIECK VECTOR INCLUDING THOSE COMPONENTS USED IN INTERP
      JB = L
      IF(NQU .GT. NQ)JB = NQU + 1
      DO 180 J = 2,JB
        R = R*RH
        DO 180 I = 1,N
 180      YH(I,J) = YH(I,J)*R
      H = H*RH
      RC = RC*RH
      IALTH = L
      IF (IREDO .EQ. 0) GO TO 690
C-----------------------------------------------------------------------
C THIS SECTION COMPUTES THE PREDICTED VALUES BY EFFECTIVELY
C MULTIPLYING THE YH ARRAY BY THE PASCAL TRIANGLE MATRIX.
C RC IS THE RATIO OF NEW TO OLD VALUES OF THE COEFFICIENT  H*EL(1).
C WHEN RC DIFFERS FROM 1 BY MORE THAN CCMAX, IPUP IS SET TO MITER
C TO FORCE PJAC TO BE CALLED.
C IN ANY CASE, PJAC IS CALLED AT LEAST EVERY MSBP STEPS.
C-----------------------------------------------------------------------
 200  IF (DABS(RC-1.0D0) .GT. CCMX1) IPUP = MITER
      IF (NST .GE. NSLP+MSBP) IPUP = MITER
      TN = TN + H
      DO 215 JB = NQ,1,-1
         DO 215 JC = JB,NQ
           DO 210 I = 1,N
 210          YH(I,JC) = YH(I,JC) + YH(I,JC+1)
 215  CONTINUE
C-----------------------------------------------------------------------
C UP TO MAXCOR CORRECTOR ITERATIONS ARE TAKEN.  A CONVERGENCE TEST IS
C MADE ON THE R.M.S. NORM OF EACH CORRECTION, WEIGHTED BY H AND THE
C ERROR WEIGHT VECTOR EWT.  THE SUM OF THE CORRECTIONS IS ACCUMULATED
C IN ACOR(I).  THE YH ARRAY IS NOT ALTERED IN THE CORRECTOR LOOP.
C-----------------------------------------------------------------------
 220  DO 230 I = 1,N
        YDOT(I) = YH(I,2) / H
        Y(I) = YH(I,1)
 230  CONTINUE
      IF (IPUP .LE. 0) GO TO 232
C
C        EXIT TO FORM JACOBIAN AND TO SOLVE THE NONLINEAR SYSTEM
C        UNLESS FNITER = .TRUE. AND THEN USE FUNCTIONAL ITERATION.
C
      IPUP = 0
      RC = 1.0D0
      NSLP = NST
      INLN = 1
      IF( FNITER ) INLN = 6
      JCUR = 1
      ISTEP = 0
      RETURN
232   INLN = 2
C
C        EXIT TO SOLVE NONLINEAR SYSTEM USING OLD JACOBIAN
C        UNLESS FNITER = .TRUE. AND THEN USE FUNCTIONAL ITERATION.
C
      IF( FNITER ) INLN = 6
      ISTEP = 0
      RETURN
C-----------------------------------------------------------------------
C THE CORRECTOR ITERATION FAILED TO CONVERGE IN MAXCOR TRIES, OR ELSE
C RES HAS RETURNED ABNORMALLY. IF INLN = -3 OR -4, RETRACT THE YH ARRAY
C TO ITS VALUES BEFORE PREDICTION AND RETURN. OTHERWISE--
C IF THE JACOBIAN IS OUT OF DATE, PJAC IS CALLED FOR
C THE NEXT TRY.  OTHERWISE THE YH ARRAY IS RETRACTED TO ITS VALUES
C BEFORE PREDICTION, AND H IS REDUCED, IF POSSIBLE.  IF H CANNOT BE
C REDUCED OR MXNCF FAILURES HAVE OCCURRED, EXIT WITH KFLAG = -2 OR -4.
C-----------------------------------------------------------------------
 410  ICF = 1
      IF (JCUR .EQ. 1) GO TO 430
      IPUP = MITER
      GO TO 220
 430  ICF = 2
      NCF = NCF + 1
      RMAX = 2.0D0
 435  TN = TOLD
      DO 445 JB = NQ,1,-1
        DO 445 JC = JB,NQ
           DO 440 I = 1,N
 440         YH(I,JC) = YH(I,JC) - YH(I,JC+1)
 445  CONTINUE
      IF (IZ .EQ. 1 .OR. IZ .EQ. 3) THEN
          KFLAG = -3
          GOTO 720
      ELSE IF(IZ .EQ. 0)THEN
          KFLAG = -7
C         WORKSPACE ERROR IN LINEAR ALGEBRA PART
          GOTO 720
      END IF
      IF (DABS(H) .LE. HMIN*1.00001D0) GO TO 675
      IF (NCF .EQ. MXNCF)THEN
          IF (IZ .EQ. 2)THEN
              KFLAG = -4
              GOTO 680
          ELSE IF( IZ .EQ. 4)THEN
              GOTO 685
          ELSE
              GOTO 670
          END IF
      END IF
      RH = 0.25D0
      IPUP = MITER
      IREDO = 1
      GO TO 170
C-----------------------------------------------------------------------
C THE CORRECTOR HAS CONVERGED.  JCUR IS SET TO 0
C TO SIGNAL THAT THE JACOBIAN INVOLVED MAY NEED UPDATING LATER.
C THE LOCAL ERROR TEST IS MADE AND CONTROL PASSES TO STATEMENT 500
C IF IT FAILS.
C-----------------------------------------------------------------------
 450  JCUR = 0
      IF(PETZLD)THEN
C        RETURN FOR PETZOLD TYPE ERROR ESTIMATE.
         ISTEP = 0
         INLN  = 5
         RETURN
      END IF
 4501 DSN = VNORM (N, ACOR, EWT)/(TESCO(2,NQ)*EL0)
      IF(PETZLD)THEN
         DSM = VNORM (N, SAVR, EWT)/(TESCO(2,NQ)*EL0*EL0*ABS(H))
      ELSE
         DSM = DSN
      END IF
      IF(ITRACE .GE. 1)WRITE(IDEV,451)DSM
 451  FORMAT(' SCALED LOCAL ERROR ESTIMATE =',D11.3)
      IF(DSM .GT. 1.0D0) GO TO 500
C
C-----------------------------------------------------------------------
C AFTER A SUCCESSFUL STEP, UPDATE THE YH ARRAY.
C CONSIDER CHANGING H IF IALTH = 1.  OTHERWISE DECREASE IALTH BY 1.
C IF IALTH IS THEN 1 AND NQ .LT. MAXORD, THEN ACOR IS SAVED FOR
C USE IN A POSSIBLE ORDER INCREASE ON THE NEXT STEP.
C IF A CHANGE IN H IS CONSIDERED, AN INCREASE OR DECREASE IN ORDER
C BY ONE IS CONSIDERED ALSO.  A CHANGE IN H IS MADE ONLY IF IT IS BY A
C FACTOR OF AT LEAST 1.1.  IF NOT, IALTH IS SET TO 3 TO PREVENT
C TESTING FOR THAT MANY STEPS.
C-----------------------------------------------------------------------
      IF(RMAX .EQ. 1.0D+4)THEN
C        FIRST STEP - SEE IF INIT STEP WAS TOO SMALL
         EXSM = 1.0D0/DFLOAT(L)
         RHSM = 1.0D0/(1.2D0*DSN**EXSM + 0.0000012D0)
         IF(RHSM .GE. RMAX .AND. (H*HMXI) .LT. (1.0D0-U*8.D0)) THEN
C           RETAKE THE STEP AS THE INITIAL STEP WAS FAR TOO SMALL
            RH = RMAX
            TN = TOLD
            DO 460 JB = NQ , 1 ,-1
               DO 460 JC = JB,NQ
                   DO 455 I = 1,N
 455                  YH(I,JC) = YH(I,JC) - YH(I,JC+1)
 460        CONTINUE
            IREDO = 3
            GOTO 170
         END IF
      END IF
C       OTHERWISE UPDATE THE SOLUTION  AND CONTINUE
      KFLAG = 0
      IREDO = 0
      NQU = NQ
      NCF = 0
      HHUSED = H
      DO 469 I = 1,N
          YH(I,1) = YH(I,1) + ACOR(I)
 469      YH(I,2) = YH(I,2) + ACOR(I)/EL0
      DO 470 J = 3,L
        ELJH = EL(J)/EL0
        DO 470 I = 1,N
 470      YH(I,J) = YH(I,J) + ELJH*ACOR(I)
      IF ( C1INTR ) THEN
           J = LMAX + 1
           DO 472 I = 1,N
 472          YH(I,J) = ACOR(I) /EL0
      END IF
      IALTH = IALTH - 1
      IF (IALTH .EQ. 0) GO TO 520
      IF (IALTH .GT. 1 .OR. L .EQ. LMAX) THEN
          RAISE = .FALSE.
          GOTO 540
      END IF
C         ELSE THE ORDER MAY BE RAISED ON THE NEXT STEP
      DO 490 I = 1,N
 490    YH(I,LMAX) = ACOR(I)/(H*EL0)
      GO TO 700
C-----------------------------------------------------------------------
C THE ERROR TEST FAILED.  KFLAG KEEPS TRACK OF MULTIPLE FAILURES.
C RESTORE TN AND THE YH ARRAY TO THEIR PREVIOUS VALUES, AND PREPARE
C TO TRY THE STEP AGAIN.  COMPUTE THE OPTIMUM STEP SIZE FOR THIS OR
C ONE LOWER ORDER.  AFTER 2 OR MORE FAILURES, H IS FORCED TO DECREASE
C BY A FACTOR OF 0.2 OR LESS.
C-----------------------------------------------------------------------
 500  KFLAG = KFLAG - 1
      TN = TOLD
      DO 515 JB = NQ , 1 ,-1
        DO 515 JC = JB,NQ
           DO 510 I = 1,N
 510          YH(I,JC) = YH(I,JC) - YH(I,JC+1)
 515    CONTINUE
      RMAX = 2.0D0
      IF (DABS(H) .LE. HMIN*1.00001D0) GO TO 650
      IF (KFLAG .LE. -4) GO TO 660
      IREDO = 2
      RAISE = .FALSE.
      GO TO 540
C-----------------------------------------------------------------------
C REGARDLESS OF THE SUCCESS OR FAILURE OF THE STEP, FACTORS
C RHDN, RHSM, AND RHUP ARE COMPUTED, BY WHICH H COULD BE MULTIPLIED
C AT ORDER NQ - 1, ORDER NQ, OR ORDER NQ + 1, RESPECTIVELY.
C IN THE CASE OF FAILURE, RHUP = 0.0 TO AVOID AN ORDER INCREASE.
C THE LARGEST OF THESE IS DETERMINED AND THE NEW ORDER CHOSEN
C ACCORDINGLY.  IF THE ORDER IS TO BE INCREASED, WE COMPUTE ONE
C ADDITIONAL SCALED DERIVATIVE.
C-----------------------------------------------------------------------
 520  RAISE = .FALSE.
      IF (L .LT. LMAX) THEN
          RAISE = .TRUE.
          DO 530 I = 1,N
 530         YDOT(I) = ACOR(I) - YH(I,LMAX)*H*EL0
          DUP =  VNORM (N, YDOT, EWT)/(TESCO(3,NQ)*EL0)
          EXUP = 1.0D0/DFLOAT(L+1)
          RHUP = 1.0D0/(1.4D0*DUP**EXUP + 0.0000014D0)
          TERKP1 = DUP * DFLOAT(NQ + 2)
      END IF
 540  EXSM = 1.0D0/DFLOAT(L)
      RHSM = 1.0D0/(1.2D0*DSN**EXSM + 0.0000012D0)
      TERK = DSN * DFLOAT(NQ+1)
C  DSN OR DSM IN THE LINES ABOVE ????
      IF (NQ .GT. 1) THEN
          DDN = VNORM (N, YH(1,L), EWT)/TESCO(1,NQ)
          EXDN = 1.0D0/DFLOAT(NQ)
          RHDN = 1.0D0/(1.3D0*DDN**EXDN + 0.0000013D0)
          TERKM1 = DDN * DFLOAT(NQ)
          IF(ITRACE .GE. 1)WRITE(IDEV,3333)TERKM1, TERK, TERKP1
3333      FORMAT(' BELOW SAME AND ABOVE = ',3D11.3)
C
C         DECIDE WHETHER TO LOWER THE ORDER
C
          IF(.NOT. RAISE) TERKP1 = TERK
          IF(TERKM1 .LT.  TERK )THEN
C            LOWER THE ORDER
             NEWQ = NQ - 1
             RH = RHDN
             IF (KFLAG.LT.0 .OR. IALTH.GT.0) RH = DMIN1(RH,1.0D0)
             IF (KFLAG      .LE.         -2) RH = DMIN1(RH,0.2D0)
C            DROP ORDER TO 1 ON THIRD ERROR TEST FAILURE.
             IF (KFLAG      .LE.         -3) NEWQ = 1
             GO TO 630
          ELSE IF(IALTH .GT. 0 .AND. KFLAG .EQ. 0)THEN
             GO TO 700
C                     AS STEPSIZE AND ORDER UNCHANGED FOR NEXT STEP
          END IF
      END IF
C
C   DECIDE WHETHER TO RAISE THE ORDER
C
      IF(RAISE) THEN
         RAISE = .FALSE.
         IF(NQ .EQ. 1)THEN
            IF(TERKP1 .LT. (0.5D0 * TERK) ) RAISE = .TRUE.
         ELSE
            IF(NQ .EQ. 2) THEN
               TERKM2 = TERKM1
            ELSE
               DDN1 = VNORM(N,YH(1,L-1),EWT)/TESCO(1,NQ-1)
               TERKM2 = DDN1 * DFLOAT(NQ-1)
            END IF
            TERKM1 = DMAX1 (TERKM1 , TERKM2 )
            IF( TERKP1 .LT. TERK .AND. TERK .LT. TERKM1) RAISE = .TRUE.
         END IF
C     LINE TO STOP UNNESS RAISING OF THE ORDER
         IF( RHSM .GE. RHUP .AND. RHUP .LT. 2.D0) RAISE = .FALSE.
      END IF
      IF(RAISE)THEN
C              RAISE THE ORDER
         NEWQ = L
         RH = RHUP
         IF (RH .LT. 1.1D0) THEN
             IALTH = 3
             GO TO 700
         END IF
         R = EL(L)/(DFLOAT(L)*EL0)
         DO 600 I = 1,N
 600        YH(I,NEWQ+1) = ACOR(I)*R
      ELSE
C        LEAVE THE ORDER AS IT IS
         NEWQ = NQ
         RHSM = 1.0D0 / (1.2D0 * DSM **EXSM + 0.0000012D0)
         RH = RHSM
         IF ((KFLAG .EQ. 0) .AND. (RH .LT. 1.1D0)) THEN
              IALTH = 3
              GO TO 700
C                      AS IT IS NOT WORTH CHANGING THE STEPSIZE
         END IF
         IF (KFLAG .LE. -2) RH = DMIN1(RH,0.2D0)
         IF (KFLAG .LE. -3) THEN
            NEWQ = 1
            GOTO 630
         END IF
         GO TO 170
      END IF
C-----------------------------------------------------------------------
C IF THERE IS A CHANGE OF ORDER, RESET NQ, L, AND THE COEFFICIENTS.
C IN ANY CASE H IS RESET ACCORDING TO RH AND THE YH ARRAY IS RESCALED.
C THEN EXIT FROM 690 IF THE STEP WAS OK, OR REDO THE STEP OTHERWISE.
C-----------------------------------------------------------------------
 630  NQ = NEWQ
      L = NQ + 1
      IRET = 2
      GO TO 150
C-----------------------------------------------------------------------
C ALL RETURNS ARE MADE THROUGH THIS SECTION.  H IS SAVED IN HOLD
C TO ALLOW THE CALLER TO CHANGE H ON THE NEXT STEP.
C-----------------------------------------------------------------------
C
C  KFLAG = -1 ; ERROR TEST FAILED REPEATEDLY WITH ABS(H) = HMIN
C
 650  KFLAG = -1
      CALL SERROR(' SPGEAR-AT T=(R1) AND STEP SIZE H =(R2) THE ERROR
     1 TEST FAILED REPEATEDLY WITH ABS(H) = HMIN', 1, 0, 0, 0, 2, TN, H)
      GO TO 720
C
C  KLAG = -1  ; ERROR TEST REPEATEDLY FAILED.
C
 660  KFLAG = -1
      CALL SERROR(' SPGEAR-AT T=(R1) AND STEP SIZE H =(R2) THE ERROR
     1 TEST FAILED REPEATEDLY', 1, 0, 0, 0, 2, TN, H)
C
C  KFLAG = -2 CONVERGANCE FAILURE
C
 670  KFLAG = -2
      CALL SERROR(' SPGEAR-AT T=(R1) AND STEP SIZE H=(R2) THE CORRECTOR
     1 CONVERGENCE FAILED REPEATEDLY ', 1, 0, 0, 0, 2, TN, H)
      IF( FNITER ) THEN
         CALL SERROR (' THE SPGEAR MODULE WAS BEING USED WITH
     1             FUNCTIONAL ITERATION A SWITCH HAS BEEN MADE TO A
     2             NEWTON METHOD TO TRY AND IMPROVE CONVERGENCE.',
     3             1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         FNITER = .FALSE.
      END IF
      GO TO 720
C
C  KLAG = -2  CONVERGENCE FAILURE AT MINIMUM STEP SIZE
C
 675  KFLAG = -2
      CALL SERROR(' SPGEAR-AT T=(R1) AND STEP SIZE H=(R2) THE CORRECTOR
     1 CONVERGENCE FAILED REPEATEDLY WITH ABS(H) = HMIN', 1, 0, 0,
     2  0, 2, TN, H)
      IF( FNITER ) THEN
          CALL SERROR (' THE SPGEAR MODULE WAS BEING USED WITH
     1             FUNCTIONAL ITERATION.A SWITCH HAS BEEN MADE TO A
     2             NEWTON METHOD TO TRY AND IMPROVE CONVERGENCE.',
     3             1, 0, 0, 0, 0, 0.0D0, 0.0D0)
           FNITER = .FALSE.
      END IF
      GO TO 720
C     KFLAG = -4  RES ORDERED RETURN
C
 680    CALL SERROR(' SPGEAR-AT T=(R1) RESIDUAL ROUTINE RETURNED IRES
     1  =3 REPEATEDLY', 1, 0, 0, 0, 1, TN, 0.0D0)
      GO TO 720
 685  KFLAG = -5
      CALL SERROR(' SPGEAR-AT T = (R1) A SINGULAR JACOBIAN MATRIX WAS
     1     REPEATEDLY ENCOUNTERED' , 1, 0, 0, 0, 1, TN, 0.0D0)
      GO TO 720
 690  RMAX = 10.0D0
 720  CONTINUE
 700  R = 1.D0 /(TESCO(2,NQU) *EL0)
      IF(KFLAG .LT. 0)THEN
C        THE STEP HAS FAILED AND BEEN RETRACTED - INDICATE THAT LOCAL
C        ERROR HAS BEEN LOST
         CALL SERROR(' SPGEAR-CURRENT STEP FAILURE MEANS THAT OLD LOCAL
     1   ERROR ESTIMATE CANNOT BE RETRIEVED AND SO ACOR IS SET TO ZERO',
     2   1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         DO 702 I = 1,N
 702        ACOR(I) = 0.0D0
      END IF
      DO 710 I = 1,N
C       RESET YDOT AND SCALE ACOR SO THAT IT IS LOCAL ERROR ESTIMATE.
        Y(I)    = YH(I,1)
        YDOT(I) = YH(I,2)/H
 710    ACOR(I) = -ACOR(I)*R
      HOLD = H
      ISTEP = KFLAG
      IF(ISTEP .EQ. 0)ISTEP = 1
      RETURN
C----------------------- END OF SUBROUTINE SPGEAR-----------------------
      END
      SUBROUTINE DFODE (METH, ELCO, TESCO)
      INTEGER METH
      INTEGER I, IB, NQ, NQM1, NQP1
      DOUBLE PRECISION ELCO, TESCO
      DOUBLE PRECISION AGAMQ, FNQ, FNQM1, PC, PINT, RAGQ,
     1   RQFAC, RQ1FAC, TSIGN, XPIN
      DIMENSION ELCO(13,12), TESCO(3,12)
C-----------------------------------------------------------------------
C DFODE IS CALLED BY THE INTEGRATOR ROUTINE TO SET COEFFICIENTS
C NEEDED THERE.  THE COEFFICIENTS FOR THE CURRENT METHOD, AS
C GIVEN BY THE VALUE OF METH, ARE SET FOR ALL ORDERS AND SAVED.
C THE MAXIMUM ORDER ASSUMED HERE IS 12 IF METH = 1 AND 5 IF METH = 2.
C (A SMALLER VALUE OF THE MAXIMUM ORDER IS ALSO ALLOWED.)
C DFODE IS CALLED ONCE AT THE BEGINNING OF THE PROBLEM,
C AND IS NOT CALLED AGAIN UNLESS AND UNTIL METH IS CHANGED.
C
C THE ELCO ARRAY CONTAINS THE BASIC METHOD COEFFICIENTS.
C THE COEFFICIENTS EL(I), 1 .LE. I .LE. NQ+1, FOR THE METHOD OF
C ORDER NQ ARE STORED IN ELCO(I,NQ).  THEY ARE GIVEN BY A GENETRATING
C POLYNOMIAL, I.E.,
C     L(X) = EL(1) + EL(2)*X + ... + EL(NQ+1)*X**NQ.
C FOR THE IMPLICIT ADAMS METHODS, L(X) IS GIVEN BY
C     DL/DX = (X+1)*(X+2)*...*(X+NQ-1)/FACTORIAL(NQ-1),    L(-1) = 0.
C FOR THE BDF METHODS, L(X) IS GIVEN BY
C     L(X) = (X+1)*(X+2)* ... *(X+NQ)/K,
C WHERE         K = FACTORIAL(NQ)*(1 + 1/2 + ... + 1/NQ).
C
C THE TESCO ARRAY CONTAINS TEST CONSTANTS USED FOR THE
C LOCAL ERROR TEST AND THE SELECTION OF STEP SIZE AND/OR ORDER.
C AT ORDER NQ, TESCO(K,NQ) IS USED FOR THE SELECTION OF STEP
C SIZE AT ORDER NQ - 1 IF K = 1, AT ORDER NQ IF K = 2, AND AT ORDER
C NQ + 1 IF K = 3.
C-----------------------------------------------------------------------
      DIMENSION PC(12)
C
      GO TO (100, 200), METH
C
 100  ELCO(1,1) = 1.0D0
      ELCO(2,1) = 1.0D0
      TESCO(1,1) = 0.0D0
      TESCO(2,1) = 2.0D0
      TESCO(1,2) = 1.0D0
      TESCO(3,12) = 0.0D0
      PC(1) = 1.0D0
      RQFAC = 1.0D0
      DO 140 NQ = 2,12
C-----------------------------------------------------------------------
C THE PC ARRAY WILL CONTAIN THE COEFFICIENTS OF THE POLYNOMIAL
C     P(X) = (X+1)*(X+2)*...*(X+NQ-1).
C INITIALLY, P(X) = 1.
C-----------------------------------------------------------------------
        RQ1FAC = RQFAC
        RQFAC = RQFAC/DFLOAT(NQ)
        NQM1 = NQ - 1
        FNQM1 = DFLOAT(NQM1)
        NQP1 = NQ + 1
C FORM COEFFICIENTS OF P(X)*(X+NQ-1). ----------------------------------
        PC(NQ) = 0.0D0
        DO 110 IB = 1,NQM1
          I = NQP1 - IB
 110      PC(I) = PC(I-1) + FNQM1*PC(I)
        PC(1) = FNQM1*PC(1)
C COMPUTE INTEGRAL, -1 TO 0, OF P(X) AND X*P(X). -----------------------
        PINT = PC(1)
        XPIN = PC(1)/2.0D0
        TSIGN = 1.0D0
        DO 120 I = 2,NQ
          TSIGN = -TSIGN
          PINT = PINT + TSIGN*PC(I)/DFLOAT(I)
 120      XPIN = XPIN + TSIGN*PC(I)/DFLOAT(I+1)
C STORE COEFFICIENTS IN ELCO AND TESCO. --------------------------------
        ELCO(1,NQ) = PINT*RQ1FAC
        ELCO(2,NQ) = 1.0D0
        DO 130 I = 2,NQ
 130      ELCO(I+1,NQ) = RQ1FAC*PC(I)/DFLOAT(I)
        AGAMQ = RQFAC*XPIN
        RAGQ = 1.0D0/AGAMQ
        TESCO(2,NQ) = RAGQ
        IF (NQ .LT. 12) TESCO(1,NQP1) = RAGQ*RQFAC/DFLOAT(NQP1)
        TESCO(3,NQM1) = RAGQ
 140    CONTINUE
      RETURN
C
 200  PC(1) = 1.0D0
      RQ1FAC = 1.0D0
      DO 230 NQ = 1,5
C-----------------------------------------------------------------------
C THE PC ARRAY WILL CONTAIN THE COEFFICIENTS OF THE POLYNOMIAL
C     P(X) = (X+1)*(X+2)*...*(X+NQ).
C INITIALLY, P(X) = 1.
C-----------------------------------------------------------------------
        FNQ = DFLOAT(NQ)
        NQP1 = NQ + 1
C FORM COEFFICIENTS OF P(X)*(X+NQ). ------------------------------------
        PC(NQP1) = 0.0D0
        DO 210 IB = 1,NQ
          I = NQ + 2 - IB
 210      PC(I) = PC(I-1) + FNQ*PC(I)
        PC(1) = FNQ*PC(1)
C STORE COEFFICIENTS IN ELCO AND TESCO. --------------------------------
        DO 220 I = 1,NQP1
 220      ELCO(I,NQ) = PC(I)/PC(2)
        ELCO(2,NQ) = 1.0D0
        TESCO(1,NQ) = RQ1FAC
        TESCO(2,NQ) = DFLOAT(NQP1)/ELCO(1,NQ)
        TESCO(3,NQ) = DFLOAT(NQ+2)/ELCO(1,NQ)
        RQ1FAC = RQ1FAC/FNQ
 230    CONTINUE
      RETURN
C----------------------- END OF SUBROUTINE DFODE -----------------------
      END
