C PRINT AT T=0 INSERTED
C ROUTINE SSSTAT CHANGED BACK TO LAMINGS SPSTAT SO GET SUMMMARY TABLE
C SUBROUTINE TABLE IMPROVED
C BUGS WITH TOUT(NT) LINES 59, 299-301 CHANGED
C NEW VERSION AS PER REVISED MANUAL (PART 2) WITH NIW(3)
C              ELSE IF (STEP.EQ.'THETA'.OR.MATZ.EQ.'THETB') THEN
C CHANGED TO
C              ELSE IF (STEP.EQ.'THETA'.OR.STEP.EQ.'THETB') THEN
C IN SETPDE  LINE 1165 UPDATED
C          ELSE IF (STEP.EQ.'THETA'.OR.STEP.EQ.'THETB') THEN
C
C****************************************************************
C             SPRITE TO WORK WITH USER'S PROGRAM                *
C****************************************************************
C             ROUTINE TO INTERFACE TO SPRINT                    *
C                CONSISTS OF TWO ROUTINES -                     *
C     SPRITE  SPLITS UP WORK ARRAYS INTO THEIR COMPONENTS       *
C                THESE ARE THEN PASSED TO                       *
C     SATYR   WHERE SPRINT INITIALISATION ROUTINES ARE          *
C             CALLED, FOLLOWED BY SPRINT ITSELF.                *
C                                                               *
C               WRITTEN BY  ... K.M. LAMING                     *
C               MODIFIED BY ... L.R.CAIRNIE                     *
C                               (29.03.85)                      *
C               MODIFIED FURTHER M.BERZINS NOVEMBER 1985.       *
C****************************************************************
C
      SUBROUTINE SPRITE(NEQ,T,TOUT,NT,OUTPUT,Y,RTOL,ATOL,ITRACE,
     -                   SNORM,MATZ,STEP,RESID,MONITR,W,NW,IW,NIW)
C**********************************************************************
C
C          PARAMETER LIST
C          **************
C  The following variables are used to communicate with SPRITE.
C  These variables should be declared in the calling program as follows
C
C      INTEGER NEQ, NT, ITRACE, NW, NIW, IW(NIW)
C      DOUBLE PRECISION T, TOUT(NT), Y(NEQ), W(NW), RTOL(NEQ),ATOL(NEQ)
C      CHARACTER*6 SNORM, MATZ, STEP
C
C  The values that should be assigned to these variables are
C
C  NEQ
C    Number of differential/algebraic equations (INPUT).
C    Must be less than 1001 due to SPRITE internal array .
C
C  T
C    Initial value of time (the independent variable) (INPUT).On output
C    it contains the time at which a computed solution is returned.
C
C  TOUT
C    Array of length NT containing an ordered list of output times
C    (INPUT). Within SPRITE, SPRINT is called in a loop to integrate
C    up to each value of TOUT in turn, and output maybe produced at
C    each time level (see RESULT). Unchanged on OUTPUT.
C    N.B. the NT array values must be strictly increasing and
C         T       < TOUT(1)
C         TOUT(I-1)<TOUT(I)    , I = 2 ,NT .
C
C  NT
C    Number of output times specified (INPUT).
C
C  RESULT
C   Character *6 variable specifying the type of output required at each
C   value of TOUT (INPUT). Possible values are
C    'BRIEF' Summary information on the solution at each of the TOUT(NT)
C            points is provided.
C    'LONG ' As for 'BRIEF' but including the solution values at the
C            end of every integrator time step.
C
C    N.B. ALL CHARACTER STRINGS SHOULD BE LEFT JUSTIFIED IN THE CALL TO
C         SPRITE E.G. 'LONG' OR 'LONG  ' IS O.K. BUT ' LONG' IS NOT.
C
C  Y
C    Array of length NEQ.
C    On INPUT it must contain the initial values of the NEQ variables;
C    on OUTPUT it contains the computed solution at the final output
C    time TOUT(NT). If the solution fails before TOUT(NT), or is
C    stopped by the user in subroutine RESID or MYMNTR (see below),
C    Y contains the last-computed solution, and T contains the time that
C    was reached .
C
C  RTOL
C   Array of length NEQ containing the relative error tolerances for eac
C   variable (INPUT). The error bounds used in controlling the error
C   are calculated as
C
C              EWT(i) = RTOL(i) * ABS(Y(i)) + ATOL(i)
C
C   Frequently the same value of RTOL can be used for all variables a
C   a value of 1D-4 should give moderate accuracy.
C   To check validity of results do a repeat run with different RTOL
C   values, and see how much the solution changes.
C
C  ATOL
C   Array of length NEQ containing the absolute error tolerances for
C   variable (INPUT) (see RTOL).
C   Different values are required for different variables
C   if there are large differences in scaling. For each variable,
C   a value of ATOL should be chosen that is a few orders
C   of magnitude smaller than a typical value for the variable.
C
C  ITRACE
C   Specifies level of trace information required from SPRINT, permitted
C   values  0,1,2 or 3 (INPUT). If problems occur
C   turn on the trace to obtain diagnostic information.
C   The output is automatically written to Channel 4 i.e. to
C   FILE FT04F001 A on the VM/CMS system
C   unless the channel has been allocated to another file using FILEDEF.
C
C  SNORM
C   Character variable specifying type of norm to be used by SPRINT
C   in controlling the local error (INPUT) .
C   Possible values are  'L2NORM', 'MAXMUM' and'MODMAX' . If in doubt
C   try 'L2NORM', and consult SPRINT documentation for more details.
C
C  MATZ
C   Character variable specifying type of matrix algebra
C   required (INPUT). The possible choices are
C      'FULL' The LINPACK full matrix routines are setup and called by
C             SPRITE .
C      'BANDED' The LINPACK banded matrix routines are setup and called
C             by SPRITE. In this case the lower and upper bandwidths
C             ML and MU  of the Jacobian matrix should be placed in
C             locations IW(15) and IW(16) i.e.
C                IW(15) = ML
C                IW(16) = MU.
C             See User Manual Part One for further details . Note that
C             if SETPDE is called first then the value of IBAND returned
C             may be used for both ML and MU.
C      'SPARSE' YALE sparse matrix routines are setup and called by
C             SPRITE with the sparsity pattern being numerically
C             estimated. Should the sparsity pattern change radically
C             in mid-integration SPRITE should be restarted with IW(1)=0
C   Use 'FULL' for small problems, and unless
C   the system is known to have a banded or sparse coupling pattern.
C
C   STEP
C   Character variable specifying type of integrator required (INPUT).
C   The possible choices are 'BLEND', 'GEAR', 'THETA' and 'ADAMS'.
C   'BLEND'
C   is a blended multistep code suitable for both stiff and non-stiff
C   problems, that is very robust . It is however more expensive than
C   the other methods and so may not be suitable for large or costly
C   problems.
C   'GEAR'
C   is a Gear method, suitable for stiff problems and less expensive
C   than 'BLEND'.
C   'THETA'
C   is a Theta-method, suitable for stiff problems at moderate accuracy
C   requirement (RTOL > &eqv.1D-3) only, but may be inexpensive for
C   small problems.
C   'ADAMS'
C   is an Adams method suitable for non-stiff problems .
C   (But note this may be much less efficient than Adams methods
C    appearing in future releases of NAG).
C
C   RESID
C   Name of a subroutine supplied by the user to define the equations
C   - see specification below.
C   It must be declared as EXTERNAL in the user's calling program.
C   The user can stop the integration from RESID by setting its
C   parameter IRES  to a particular value.
C
C   MONITR
C   Name of a subroutine needed if the user wishes to provide output
C   at each time step, to perform intermediate calculation or to control
C   (stop) the integration.
C   It must be declared as EXTERNAL in the user's calling program.
C   If the user has no requirements for output/action
C   at each time step he may select the system
C   'dummy' routine SPRMON. If he needs to produce some output
C   or make some calculations, and is a non-expert SPRINT user
C   requiring an easy interface,
C   he should select EZMNTR. This will then call a subroutine MYMNTR
C   which the user must provide, where he can provide his output etc
C   see below w. If he requires more flexibility he
C   should specify the name of his own MONITR routine, which must
C   then havethe interface defined in the SPRINT User Manual (Chpt 6).
C
C  RW
C   Real workspace of length NRW.
C
C  NRW
C   Size of RW (INPUT). The required size of workspace depends on the
C   type of matrix algebra selected :
C
C           FULL   :   NRW  >=.  (18*NEQ+23) + NEQ*NEQ + 2
C
C           BANDED :   NRW  >=.  (18*NEQ+23) + &lbt.(2*ML+MU+1)*NEQ + 2
C
C                      ML and MU are lower and upper bandwidths, set by
C                      the user in locations IW(5) and IW(6) - see
C                      optional inputs below.
C
C           SPARSE :   NRW  >=.  (18*NEQ+23) + 4*NEQ + 11*NEQ/2
C                      This is an initial estimate - the user may need t
C                      provide more storage in which case a message is
C                      written to the terminal.
C
C  IW
C   Integer workspace of length NIW. The first location of IW, IW(1), is
C   used to tell SPRINT whether calls to set-up routines and
C   initialisations are required. On
C   the first call to SPRITE, set IW(1) = 0. IF the integration is
C   successful, it will have been reset to 2 on output.
C   Any other value indicates either an error, or the
C   user has interrupted the integration see the list of outputs in
C   Part One of the User Manual. If the integration was
C   successful and a further continuation call to SPRITE is made,
C   IW(1) should be left equal to 2 so that initialisations are not
C   repeated. Any attempt to call SPRITE with illegal inputs should
C   result in IW(1) being set to -3.
C
C  NIW(3)
C   INTEGER ARRAY OF SIZE 3 CONTAINING WORKSPACE SIZE INFORMATION WHERE
C   NIW(1)  ; SIZE OF ARRAY IW . THE REQUIRED SIZE IS
C                       NIW(1)  =  2*NEQ + 16
C   NIW(2)  ; SIZE OF WORKSPACE WKRES NEEDED BY RESIDUAL ROUTINE
C
C   NIW(3)  ; SIZE OF WORKSPACE WKMON NEEDED BY MONITR ROUTINE
C
C   NOTE ; IF SPRITE IS USED WITH SETPDE THEN BOTH NIW(2) AND NIW(3)
C          MUST BE SET TO ZERO , OTHERWISE BOTH MUST BE OF SIZE AT
C          LEAST ONE. THE SIZES OF NIW(2) AND NIW(3) WILL BE USED IN
C          DETERMINING THE TOTAL WORKSPACE NEEDED BY SPRITE.
C
C  Optional Inputs and Outputs to SPRITE and SPRINT.
C  -------------------------------------------------
C   IW(3) is passed across as INFORM(3) to SPRINT and so can be set =1
C   to signify optional inputs to SPRINT. These may be passed through as
C   H0 : RW(1)  ,  HMAX : RW(2)  , HMIN  ; RW(3) .
C   Locations 1-14 of the array IW are passed across to SPRINT as the
C   first 14 locations of INFORM and so may be used to access  options
C   as in part one of the manual.
C  N.B. SPRITE will always set INFORM(2) = 1 in calling SPRINT;if other
C   values are required use the ordinary interface to SPRINT.
C**********************************************************************
       INTEGER I, INFPOS, IOUT, IPOSRW, IPOSWJ, IPOSWM, IPOSWR, IPOSYD,
     1         IPOSYS, ITOTIW, ITRACE, IW(*), JACPOS, LENINF, LENJP,
     2         LENRW, LENWT, LENWM, LENWR, LENYD, LENYS, LREQ, ML, MU,
     3         NEQ, NT, NW, NWKMON, NWKRES, NY2DIM, IOVFLO, NIW(*)
C
       DOUBLE PRECISION SAVE1, SAVE2, SAVE3, T, TOUT(NT), Y(NEQ), W(NW),
     1         RTOL(NEQ), ATOL(NEQ), DUNFLO, UROUND
C
       CHARACTER*6 SNORM, MATZ, STEP, OUTPUT
C
       EXTERNAL RESID, SPRJAC, MONITR
       EXTERNAL PREPJF, SOLSF, PREPJB, SOLSB, PREPJS, SOLSS
       EXTERNAL SPGEAR, STHETA, STHETB, SBLEND
C
CC--------------------- COMMON BLOCKS ---------------------------------
C ... TO COMMUNICATE WITH CALLING PROGRAM .....
C
      COMMON /MYWORK/ NWKRES,NWKMON
      COMMON /SIZE/   ITOTW,IFLAG
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
C ... TO COMMUNICATE WITH SATYR  ROUTINE .....
C
      COMMON /ARSIZ1/ LENRW,LENWR,LENWM,NY2DIM,LENWJ,LENINF,LENJP
      COMMON /BAND/   ML,MU
      COMMON /SPAR/   LREQ
      COMMON /OUTPT/  IOUT
      SAVE
C
C*** INCLUDED 6/8 - 93
C     READ MAXIMUM # OF ODEs, MAXSPR
      INCLUDE 'SPRITE.INC'
C**** END INCLUDED 6/8 - 93
 
C
C------------------- END OF COMMON BLOCKS ----------------------------
C
C ... INITIALISE THE FLAG ETC ...IFLAG SET TO 1 IF MORE STORAGE NEEDED
C                             ...ITOTW WILL CONTAIN REQUIRED AMOUNT
C                             ...IOUT IS OUTPUT CHANNEL FOR RESULTS,STAT
C
      IFLAG = 0
      IOUT = 4
C
C ... CHECK THE SIZE OF NEQ
C
CUT   IF( NEQ .GT. 999 .OR. NEQ .LT. 1)THEN
      IF( NEQ .GT. MAXSPR .OR. NEQ .LT. 1)THEN
          WRITE(IOUT,*)' NEQ MUST BE < 1000 AND > 0, BUT IS ',NEQ
          WRITE(6   ,*)' NEQ MUST BE < 1000 AND > 0, BUT IS ',NEQ
          GOTO 2000
      END IF
C
C ... CHECK THE TOUT ARRAY
C
      IF( NT .LT. 1)THEN
          WRITE(IOUT,*)' NT MUST BE GREATER THAN 0, BUT IS ',NT
          WRITE(6   ,*)' NT MUST BE GREATER THAN 0, BUT IS ',NT
          GOTO 2000
      END IF
      DT = DABS(TOUT(1) - T )
      DTT = DABS( UROUND * TOUT(1) * 1000 )
      IF( DT. LT. DTT .OR. T .GE. TOUT(1))THEN
          WRITE(IOUT,*)' TOUT IS NOT STRICTLY GREATER THEN T'
          WRITE(6   ,*)' TOUT IS NOT STRICTLY GREATER THEN T'
          GOTO 2000
      END IF
      IF( NT .GT. 1)THEN
          DO 10 I = 2, NT
             DT = DABS(TOUT(I-1) - TOUT(I) )
             DTT = DABS( UROUND * TOUT(I) * 1000 )
             IF( DT. LT. DTT .OR. TOUT(I-1) .GE. TOUT(I))THEN
              WRITE(IOUT,*)' THE ARRAY TOUT IS NOT STRICTLY INCREASING'
              WRITE(6   ,*)' THE ARRAY TOUT IS NOT STRICTLY INCREASING'
              GOTO 2000
             END IF
 10       CONTINUE
      END IF
C
C     IF IW(3) IS NOT ONE THERE ARE NO OPTIONAL INPUTS SO ZERO
C     THE APPROPRIATE PARTS OF IW(...) AND W( )
C
       IF(IW(3) .NE. 1)THEN
          DO 11 I = 4,14
             IW(I) = 0
 11       CONTINUE
          DO 12 I = 1,3
             W(I) = 0.0D0
 12       CONTINUE
       END IF
C
C
C ... WORK OUT THE POSITIONS OF THE SPRINT ARRAYS IN W AND IW ...
C ... THE INTEGER ARRAYS ...
C
C     ... THE INFORM ARRAY :
              INFPOS = 1
              LENINF = 16 + NEQ
C     ... THE JACPVT ARRAY :
              JACPOS = 17 + NEQ
              LENJP  = NEQ
C
C ... THE REAL ARRAYS ...
C
C    ... THE YDOTI ARRAY:
            IPOSYD = 1
            LENYD = NEQ
C    ... THE YSAVE ARRAY:
               IF (STEP.EQ.'ADAMS') THEN
                 NY2DIM = 13
               ELSE IF (STEP.EQ.'GEAR') THEN
                  NY2DIM = 6
               ELSE IF (STEP.EQ.'THETA'.OR.STEP.EQ.'THETB') THEN
                  NY2DIM = 4
               ELSE IF (STEP.EQ.'BLEND') THEN
                  NY2DIM = 14
               ELSE
                  WRITE(IOUT,*)'INCORRECT SPECIFICATION OF STEP - NOT ',
     *           'RECOGNIZED AS ADAMS GEAR BLEND THETA OR THETB'
                  WRITE(6,*)'INCORRECT SPECIFICATION OF STEP - NOT ',
     *           'RECOGNIZED AS ADAMS GEAR BLEND THETA OR THETB'
                  GOTO 2000
               END IF
            IPOSYS = IPOSYD + LENYD
            LENYS  = NY2DIM*NEQ
C    ... THE RWORK ARRAY:
            IPOSRW = IPOSYS + LENYS
            LENRW  = 21 + 3*NEQ
C    ... THE WKRES ARRAY:
            IPOSWR = IPOSRW + LENRW
            IF (NIW(2).NE.0) NWKRES=NIW(2)
            LENWR  = MAX0(NWKRES, 1)
C    ... THE WKMON ARRAY:
            IPOSWM = IPOSWR + LENWR
            IF (NIW(3).NE.0) NWKMON=NIW(3)
            LENWM  = MAX0(NWKMON, 1)
C    ... THE WKJAC ARRAY:
            IPOSWJ = IPOSWM + LENWM
               IF (MATZ.EQ.'FULL') THEN
                   LENWJ = NEQ*NEQ + 2
                ELSE IF (MATZ.EQ.'BANDED') THEN
                   ML = IW(15)
                   MU = IW(16)
                   LENWJ = (2*ML + MU + 1)*NEQ +2
                   IW(15) = IW(JACPOS)
                   IW(16) = IW(JACPOS+1)
                ELSE IF (MATZ.EQ.'SPARSE') THEN
                   LENWJ = 4*NEQ + 11*NEQ/2
                   IF (IW(1).NE.0) LENWJ = LREQ
                ELSE
                WRITE(IOUT,*)'INCORRECT SPECIFICATION OF MATZ  -NOT ',
     *          'RECOGNIZED AS FULL BANDED OR SPARSE '
                WRITE(6,*)'INCORRECT SPECIFICATION OF MATZ -NOT ',
     *          'RECOGNIZED AS FULL BANDED OR SPARSE '
                GOTO 2000
               END IF
C
 1001 CONTINUE
C
C ... CHECK THAT THE WORKSPACE IS BIG ENOUGH ...
C
      ITOTW  = LENYD + LENYS + LENRW + LENWJ + LENWR + LENWM
      ITOTIW = LENINF + LENJP
C
      IF (ITOTW.GT.NW) THEN
         IFLAG = 1
         WRITE(IOUT,*)'THE REAL WORKSPACE IS NOT BIG ENOUGH ... '
         WRITE(IOUT,*)'IT IS :',NW
         WRITE(IOUT,*)'IT NEEDS TO BE :',ITOTW
         WRITE(6,*)'THE REAL WORKSPACE IS NOT BIG ENOUGH ... '
         WRITE(6,*)'IT IS :',NW
         WRITE(6,*)'IT NEEDS TO BE :',ITOTW
         GOTO 2000
      ELSE
C
C      FIX BY M.B. /10/85 ASSIGN SPARE STORAGE TO JACOBIAN FORMING.
C
         LENWJ = LENWJ + NW - ITOTW
         ITOTW = NW
      END IF
      IF (ITOTIW.GT.NIW(1)) THEN
         WRITE(IOUT,*)'THE INTEGER WORKSPACE IS NOT BIG ENOUGH ... '
         WRITE(IOUT,*)'IT IS :',NIW(1)
         WRITE(IOUT,*)'IT NEEDS TO BE :',ITOTIW
         WRITE(6,*)'THE INTEGER WORKSPACE IS NOT BIG ENOUGH ... '
         WRITE(6,*)'IT IS :',NIW(1)
         WRITE(6,*)'IT NEEDS TO BE :',ITOTIW
         GOTO 2000
      END IF
C
C ... NEXT RE-ALLOCATE ANY OPTIONAL INPUTS ...
C
      IF (IW(3).EQ.1) THEN
                 SAVE1 = W(IPOSRW + 4)
                 SAVE2 = W(IPOSRW + 5)
                 SAVE3 = W(IPOSRW + 6)
         W(IPOSRW + 4) = W(1)
         W(IPOSRW + 5) = W(2)
         W(IPOSRW + 6) = W(3)
                  W(1) = SAVE1
                  W(2) = SAVE2
                  W(3) = SAVE3
      END IF
C
C ... CALL THE SPRINT SETUP ROUTINE SATYR  ...
C ... WITH THE WORKSPACES SPLIT UP
C
C
      CALL SATYR (NEQ,T,TOUT,NT,OUTPUT,Y,RTOL,ATOL,ITRACE,SNORM,MATZ,
     1            STEP,RESID,MONITR,W(IPOSRW), W(IPOSWR), W(IPOSWM),
     2            W(IPOSYS),W(IPOSWJ),W(IPOSYD),IW(INFPOS),IW(JACPOS))
C
C ... RE-SET THE OPTIONAL INPUTS ...
C
      IF (IW(3).EQ.1) THEN
                 SAVE1 = W(1)
                 SAVE2 = W(2)
                 SAVE3 = W(3)
                  W(1) = W(IPOSRW + 4)
                  W(2) = W(IPOSRW + 5)
                  W(3) = W(IPOSRW + 6)
         W(IPOSRW + 4) = SAVE1
         W(IPOSRW + 5) = SAVE2
         W(IPOSRW + 6) = SAVE3
      END IF
C
      IF (MATZ.EQ.'BANDED') THEN
         IW(JACPOS)   = IW(15)
         IW(JACPOS+1) = IW(16)
              IW(15)  = ML
              IW(16)  = MU
      END IF
C
C ... IF THE SPARSE WORKSPACE ISN'T BIG ENOUGH ...
C
        IF (MATZ.EQ.'SPARSE') THEN
         IF (LREQ.GT.LENWJ) THEN
          I = LREQ - LENWJ
          WRITE(IOUT,*)'SPARSE MATRIX ROUTINES NEED MORE STORAGE.......'
          WRITE(IOUT,*)'EVEN THOUGH ALL EXTRA STORAGE ASSIGNED  .......'
          WRITE(6,*)'SPARSE MATRIX ROUTINES NEED MORE STORAGE.......'
          WRITE(6,*)'EVEN THOUGH ALL EXTRA STORTAGE ASSIGNED.......'
          IW(1) = 0
          WRITE(6,*)' ESTIMATE OF EXTRA STORAGE IS',I
          LENWJ = LREQ
         END IF
        END IF
C
      RETURN
2000  IW(1) = -3
      WRITE(IOUT,*)' SPRITE RUN ABORTED BECAUSE OF ILLEGAL INPUT'
      WRITE( 6  ,*)' SPRITE RUN ABORTED BECAUSE OF ILLEGAL INPUT'
C
C       RETURN POINT FOR ILLEGAL INPUT
C
      RETURN
      END
      SUBROUTINE SATYR  (NEQ,T,TOUT,NT,OUTPUT,Y,RTOL,ATOL,JTRACE,
     1                   SNORM,MATZ,STEP,RESID,MONITR,RWORK,WKRES,
     2                   WKMON,YSAVE,WKJAC,YDOTI,INFORM,JACPVT)
C********************************************************************
C
C          LOWER LEVEL SETUP ROUTINE
C
C********************************************************************
       INTEGER I, IDEV, IND, INFORM(21), IOUT, IRET, IS, ITOL, ITRACE,
     1         LINALG, LREQ, MAXORD, METH, ML, MU, NELEMT, NEQ, NFCN,
     2         NINF, NJPVT, NOINV, NOSOLV, NOSTEP, NRW, NT, NTT, NYH,
     3         NWKJAC, NWKMON, NWKRES, NY2DIM, JSINFO(4), JACPVT(1)
C
       DOUBLE PRECISION  SCALER, T, TEND, TFINAL, TINIT, YSAVE(NEQ,1),
     1         Y(1), RWORK(21), WKRES(1), WKMON(1), WKJAC(1), YDOTI(1),
     2         TOUT(NT), ATOL(1), RTOL(1)
C
       CHARACTER*6 SNORM, MATZ, STEP, OUTPUT, START
C
       EXTERNAL RESID, SPRJAC, MONITR
       EXTERNAL PREPJF, SOLSF, PREPJB, SOLSB, PREPJS, SOLSS
       EXTERNAL SPGEAR, STHETA, STHETB, SBLEND
C
C  ... OPTIONAL COMMON BLOCK FOR SPRINT ERROR TRACING ...
C                 (ITRACE>0)
C ITRACE = TRACE LEVEL, IDEV = OUTPUT CHANNEL FOR WRITES
C
       COMMON /SDEV2/ ITRACE, IDEV
       COMMON /GOFLAG/ START
       COMMON /ARSIZ1/ NRW,NWKRES,NWKMON,NY2DIM,NWKJAC,NINF,NJPVT
       COMMON /BAND/   ML,MU
       COMMON /SPAR/   LREQ
C
C ... FOR SPSTAT
       COMMON /SPSTCM/ SCALER
       COMMON /OUTPT/  IOUT
       COMMON /KMON2/ TINIT,NTT
       COMMON /KMON5/ IND
       SAVE
C
C ... ENABLE NT TO BE PASSED VIA COMMON ...
      NTT = NT
C******** LATEST ADDITION FOR SPSTAT **********
C
C ... TRACE CHANNEL ...
C
       IDEV = 4
C
C ... AND NOW FOR THE SETUPS ...
C
       IF(INFORM(1).EQ.0)THEN
          START = 'SPTRUE'
       ELSE IF(INFORM(1) .EQ. 2 .AND. START .EQ. 'SPTRUE')THEN
          START = 'SFALSE'
       ELSE
C         ERROR EXIT
          IF(INFORM(1) .EQ. 2)THEN
           CALL SERROR(' SPRITE - ROUTINE WAS ENTERED WITH IW(1) =2 BUT
     1     WITHOUT FIRST CALLING SPRITE WITH IW(1)= 0.THE INTEGRATION
     2     HAS BEEN TERMINATED. ', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
          ELSE
           CALL SERROR(' SPRITE - ROUTINE WAS ENTERED WITH AN ILLEGAL
     1     VALUE OF IW(1), WHICH WAS (=I1) . THE RUN HAS BEEN STOPPED.',
     2     1, 1, INFORM(1), 0, 0, 0.0D0, 0.0D0)
          END IF
          GOTO 1000
       END IF
       INFORM(2) = 1
       ITOL      = 4
       ITRACE = JTRACE
       NYH = NEQ
       TINIT  = T
C
       IF(START .EQ. 'SPTRUE')THEN
C*******************+++++++++++++++++++++++++++++++*********************
C                                                                      *
C ... CALL THE SETUP ROUTINE  FOR  ALGEBRA                             *
C ... CALL THE SETUP ROUTINE  FOR STEP INTEGRATOR                      *
C ... CHOICES ARE :                                                    *
C                                    SETUP ROUTINE        MAXORD       *
C     SPGEAR:  NY2DIM = MAXORD + 1      BDFSET     ADAMS = 12 GEAR = 5 *
C     STHETA:  NY2DIM = 4               THESET               1         *
C     STHETB:  NY2DIM = 4               THBSET               1         *
C     SBLEND:  NY2DIM = MAXORD + 3      BLDSET              11         *
C                                                                      *
C ... TO CHOOSE BETWEEN THE TWO SPGEAR MEHODS :                        *
C ...                FOR ADAMS METH = 1                                *
C ...                  FOR BDF/GEAR METH =2                            *
C                                                                      *
C*******************+++++++++++++++++++++++++++++++*********************
C
      IF (MATZ.EQ.'FULL  ') THEN
C         FULL ALGEBRA  SETUP
        LINALG=2
        CALL MATSET( LINALG, NEQ, MU, ML, NWKJAC, NEQ, IRET)
        IF(IRET .LT. 0)THEN
           WRITE(IOUT,11)
           GOTO 1000
        END IF
      ELSE IF (MATZ.EQ.'BANDED') THEN
C       BANDED ALGEBRA SETUP
        LINALG=1
        CALL MATSET( LINALG, NEQ, MU, ML, NWKJAC, NEQ, IRET)
        IF(IRET .LT. 0)THEN
           WRITE(IOUT,11)
           GOTO 1000
        END IF
      ELSE IF (MATZ.EQ.'SPARSE') THEN
C       SPARSE ALGEBRA SETUP
        LINALG=0
        IRET = 0
        NELEMT = 4 * NEQ
        CALL SMTSET(NEQ,2,0.0D0,NWKJAC,NJPVT,NELEMT,IRET,JSINFO)
        IF(IRET .LT. 0)THEN
           WRITE(IOUT,12)
           GOTO 1000
        END IF
      ELSE
        WRITE(IOUT,13)
        GOTO 1000
      END IF
C
C   SETUP THE O.D.E. INTEGRATOR ***************************************
C
      IF (STEP.EQ.'ADAMS ') THEN
               METH = 1
               MAXORD = 12
         CALL BDFSET( NY2DIM, IRET, MAXORD, METH, .FALSE.)
               IF(IRET .LT. 0)THEN
                  WRITE(IOUT,31)
                  GOTO 1000
               END IF
       ELSE IF (STEP.EQ.'GEAR ') THEN
                METH = 2
                MAXORD = 5
         CALL BDFSET( NY2DIM, IRET, MAXORD, METH, .FALSE.)
                IF(IRET .LT. 0)THEN
                   WRITE(IOUT,31)
                   GOTO 1000
                END IF
       ELSE IF (STEP.EQ.'THETA ') THEN
                MAXORD = 1
         CALL THESET( NY2DIM, IRET)
                IF(IRET .LT. 0)THEN
                   WRITE(IOUT,41)
                   GOTO 1000
                END IF
       ELSE IF (STEP.EQ.'THETB ') THEN
                MAXORD = 1
         CALL THBSET( NY2DIM, IRET,.FALSE.,.FALSE.,0.55D0)
                IF(IRET .LT. 0)THEN
                   WRITE(IOUT,51)
                   GOTO 1000
                END IF
C
       ELSE IF (STEP.EQ.'BLEND ') THEN
                MAXORD = 11
         CALL BLDSET( NY2DIM, IRET, MAXORD)
               IF(IRET .LT. 0)THEN
                  WRITE(IOUT,61)
                  GOTO 1000
               END IF
C
       ELSE
         WRITE(IOUT,71)
         GOTO 1000
C   //// END OF STEP ROUTINE SETUPS ///////////////////////////////////
       END IF
      ELSE
C      RESET START TO HANDLE NEXT CALL
       START = 'SPTRUE'
      END IF
C/////////////////////////////////////////////////////////////////////
C
C PRINT AT T=0
      IND    = 0
      SCALER = RWORK(14)
      NFCN   = 0
      NOSTEP = 0
      NOINV  = 0
      NOSOLV = 0
      IDUMST = 2
C*** 
      IF (NEQ.LT.1000) THEN
       IF (ITRACE.NE.4) THEN
        CALL SPSTAT(OUTPUT,NEQ,T,Y,IOUT,NOSTEP,NFCN,NOINV,
     -              NOSOLV,IDUMST,ATOL,RTOL)
       ENDIF
      ENDIF
C
C    NOW CALL SPRINT FOR THE ACTUAL INTEGRATION
C
      DO 10 I = 1,NT
         TEND = TOUT(I)

         IF (MATZ.EQ.'FULL  ') THEN
C         ///////////// FULL ALGEBRA /////////////////////////////////
C
          IF (STEP.EQ.'ADAMS ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SPGEAR, YSAVE, NYH, PREPJF, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSF, MONITR,
     4                   WKMON, NWKMON)
C
       ELSE IF (STEP.EQ.'GEAR ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SPGEAR, YSAVE, NYH, PREPJF, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSF, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'THETA ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,STHETA, YSAVE, NYH, PREPJF, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSF, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'THETB ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,STHETB, YSAVE, NYH, PREPJF, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSF, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'BLEND ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SBLEND, YSAVE, NYH, PREPJF, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSF, MONITR,
     4                   WKMON, NWKMON)
       ELSE
         WRITE(IOUT,71)
         GOTO 1000
       END IF
C
      ELSE IF (MATZ.EQ.'BANDED') THEN
C
C         //////////////// BANDED ALGEBRA ////////////////////////////
       IF (STEP.EQ.'ADAMS ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SPGEAR, YSAVE, NYH, PREPJB, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSB, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'GEAR ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SPGEAR, YSAVE, NYH, PREPJB, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSB, MONITR,
     4                   WKMON, NWKMON)
C
       ELSE IF (STEP.EQ.'THETA ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,STHETA, YSAVE, NYH, PREPJB, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSB, MONITR,
     4                   WKMON, NWKMON)
C
       ELSE IF (STEP.EQ.'THETB ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,STHETB, YSAVE, NYH, PREPJB, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSB, MONITR,
     4                   WKMON, NWKMON)
C
       ELSE IF (STEP.EQ.'BLEND ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SBLEND, YSAVE, NYH, PREPJB, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSB, MONITR,
     4                   WKMON, NWKMON)
       ELSE
         WRITE(IOUT,71)
         GOTO 1000
       END IF
C
      ELSE IF (MATZ.EQ.'SPARSE') THEN
C/////////////////////////// SPARSE ALGEBRA //////////////////////////
C
       IF (STEP.EQ.'ADAMS ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SPGEAR, YSAVE, NYH, PREPJS, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSS, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'GEAR ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SPGEAR, YSAVE, NYH, PREPJS, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSS, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'THETA ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,STHETA, YSAVE, NYH, PREPJS, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSS, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'THETB ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,STHETB, YSAVE, NYH, PREPJS, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSS, MONITR,
     4                   WKMON, NWKMON)
       ELSE IF (STEP.EQ.'BLEND ') THEN
            CALL SPRINT( NEQ, T, TEND, Y, YDOTI, RWORK, NRW, RTOL,
     1                   ATOL, ITOL, INFORM, SNORM, RESID, WKRES,
     2                   NWKRES,SBLEND, YSAVE, NYH, PREPJS, WKJAC,
     3                   NWKJAC, JACPVT, SPRJAC, SOLSS, MONITR,
     4                   WKMON, NWKMON)
C
       ELSE
         WRITE(IOUT,71)
         GOTO 1000
       END IF
C
       IF (INFORM(1).NE.2) THEN
           IS= 2
           CALL SMTSET(NEQ,2,0.0D0,NWKJAC,NJPVT,NELEMT,IS,JSINFO)
           LREQ = JSINFO(1)
           IF (INFORM(1).EQ.-10 .AND. LREQ .GT. NWKJAC)
     1                                LREQ = MAX( LREQ,NWKJAC+50)
       END IF
C/////////////////////////////////////////////////////////////////////
C
      ELSE
         WRITE(IOUT,21)
         GOTO 1000
      END IF
      SCALER = RWORK(14)
      NFCN   = INFORM(7)
      NOSTEP = INFORM(6)
      NOINV  = INFORM(8)
      NOSOLV = INFORM(11)

C..
      IF (NEQ.LT.1000) THEN
       IF (ITRACE.LT.1000) THEN
       CALL SPSTAT(OUTPUT,NEQ,T,Y,IOUT,NOSTEP,NFCN,NOINV,
     -             NOSOLV,INFORM(1),ATOL,RTOL)
       ENDIF
      ENDIF
C..
      IF (INFORM(1).NE.2) THEN
          WRITE(IOUT,20)T, INFORM(1)
20        FORMAT(' AT TIME T = ',D11.3,' SPRITE SET INFORM(1) = ',I5)
          GOTO 900
      END IF
CTEST... 
      CALL CTEST1(I,T,WKMON,NWKMON,Y,NEQ)
C
10    CONTINUE
C
C ... RETURN POINT FOR CALL TO SPRINT
900   CONTINUE
      RETURN
C
C ... THE ESCAPE LABEL FOR ILLEGAL INPUT
C
1000  CONTINUE
      INFORM(1) = -3
C
C ... THE FORMAT STATEMENTS ...
C
 11      FORMAT(' SETUP FAILED IN LINEAR ALGEBRA INITIALISATION MATSET')
 12      FORMAT(' SETUP FAILED IN LINEAR ALGEBRA INITIALISATION SMTSET')
 13      FORMAT(' INTERNAL ERROR  ..13.. IN SATYR ROUTINE')
 21      FORMAT(' INTERNAL ERROR  ..21.. IN SATYR ROUTINE')
 31      FORMAT(' SETUP FAILED IN INTEGRATOR INITIALISATION BDFSET')
 41      FORMAT(' SETUP FAILED IN INTEGRATOR INITIALISATION THESET')
 51      FORMAT(' SETUP FAILED IN INTEGRATOR INITIALISATION THBSET')
 61      FORMAT(' SETUP FAILED IN INTEGRATOR INITIALISATION BLDSET')
 71      FORMAT(' INTERNAL ERROR  ..71.. IN SATYR ROUTINE')
C
      RETURN
      END
        SUBROUTINE EZMNTR( NEQ, T, HLAST, H, Y, YDOT, YSAVE, NYH, R,
     1       ACOR, RESWK, NRESWK, WKMON, NWKMON, IMON, INLN, HMIN, HMXI)
C******************* DRIVER MONITR ROUTINE  *************************
C ... INTERFACES SHORTER ARGUMENT LIST TO THE USER
C
        INTEGER NEQ, NYH, NRESWK, NWKMON, IMON, INLN, ITRACE, IDEV,I,J,N
        DOUBLE PRECISION T, HLAST, H, Y(1), YDOT(1), YSAVE(NYH,1), R(1),
     1          ACOR(1), RESWK(NRESWK), WKMON(NWKMON), HMIN, HMXI
        CHARACTER *6 OUTPUT
        COMMON /SDEV2/ ITRACE, IDEV
        COMMON /KMON4/ OUTPUT
        SAVE
        IF (IMON.NE.1) RETURN
C
        CALL MYMNTR( NEQ, T, Y, HLAST, IMON)
      IF (OUTPUT.EQ.'LONG') THEN
C ...   THIS WRITES THE VALUES OF Y(I) AT EACH TIME STEP TO FILE 16 ...
        J = 0
        WRITE(16,39)T
        N = NEQ/5 + 1
        DO 10 I = 1,N
          IF (J+5.LE.NEQ) THEN
            WRITE(16,40)Y(J+1),Y(J+2),Y(J+3),Y(J+4),Y(J+5)
          ELSE IF (J+4.LE.NEQ) THEN
            WRITE(16,40)Y(J+1),Y(J+2),Y(J+3),Y(J+4)
          ELSE IF (J+3.LE.NEQ) THEN
            WRITE(16,40)Y(J+1),Y(J+2),Y(J+3)
          ELSE IF (J+2.LE.NEQ) THEN
            WRITE(16,40)Y(J+1),Y(J+2)
          ELSE IF (J+1.LE.NEQ) THEN
            WRITE(16,40)Y(J+1)
          ELSE
          END IF
          J = J + 5
          IF (J.GE.NEQ) GO TO 20
 10     CONTINUE
 20     CONTINUE
      END IF
C
C
 39   FORMAT(1X,E15.9)
 40   FORMAT(1X,5(E15.9,1X))
      RETURN
      END
      SUBROUTINE MYMNTR( NEQ, T, Y, H, IMON)
C********** DUMMY MYMNTR ROUTINE  *************************
C
      INTEGER NEQ,IMON
      DOUBLE PRECISION T,Y(NEQ),H
C
      RETURN
      END
************************************************************************
*               SPSTAT7 THAT HAS 2 TYPES OF OUTPUT                     *
*             ROUTINE TO OUTPUT STATISTICS FROM SPRINT                 *
*                    WRITTEN BY : K M LAMING (UWIST)                   *
*                     COMPLETED : 3-6-85                               *
************************************************************************
*             VERSION FOR SPRITE ALONE - NO VARIABLE NAMES             *
************************************************************************
      SUBROUTINE SPSTAT(OUTPUT,NEQ,T,Y,IOUT,NSTEP,NFCN,NJAC,NSOLV,ISTATE
     -,ATOL,RTOL)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      DIMENSION Y(NEQ),NUMBER(1000,4),XTIME(1000),ATOL(NEQ),RTOL(NEQ)

      CHARACTER*72 VAR1,VAR2
      CHARACTER*6  EQNAME, OUTPUT, OUT
      CHARACTER*8  NAME

* ... to communicate with SPRITE etc ...

      COMMON/SPSTCM/TFINAL,SCALER
      COMMON/EQTN/EQNAME(1000)
      COMMON /KMON1/ XMXVAR(1000)
      COMMON /KMON2/ TINIT,NT
      COMMON /KMON3/ NAME
      COMMON /KMON5/ IND

      IF (ISTATE.EQ.0) THEN
        IF (T.GE.1000.0) THEN
          WRITE(IOUT,3011)T
        ELSE
          WRITE(IOUT,301)T
        END IF
        WRITE(IOUT,302)
        WRITE(IOUT,*)
        WRITE(IOUT,*)
      ELSE IF (ISTATE.EQ.1) THEN
        IF (T.GE.1000.0) THEN
          WRITE(IOUT,3011)T
        ELSE
          WRITE(IOUT,301)T
        END IF
        WRITE(IOUT,3022)
        WRITE(IOUT,*)
        WRITE(IOUT,*)
      ELSE IF (ISTATE.EQ.-1) THEN
        IF (T.GE.1000.0) THEN
          WRITE(IOUT,3033)T
        ELSE
          WRITE(IOUT,303)T
        END IF
        WRITE(IOUT,*)'ISTATE IS ',ISTATE
        WRITE(IOUT,*)
        WRITE(IOUT,*)
      ELSE IF (ISTATE.EQ.-2) THEN
        IF (T.GE.1000.0) THEN
          WRITE(IOUT,3044)T
        ELSE
          WRITE(IOUT,304)T
        END IF
        WRITE(IOUT,*)'ISTATE IS ',ISTATE

*** WRITE OUT VARIABLE NAMES
        DO 17 I = 1,NEQ
          WRITE(EQNAME(I),33)I
          WRITE(IOUT,*)
      WRITE(IOUT,*)'ATOL FOR ',EQNAME(I),' SHOULD BE : ',SCALER*ATOL(I)
      WRITE(IOUT,*)'RTOL FOR ',EQNAME(I),' SHOULD BE : ',SCALER*RTOL(I)
          WRITE(IOUT,*)
 17     CONTINUE
      ELSE IF (ISTATE.EQ.2) THEN

* ... indicate that there will be some results/statistics ...
* ... and how many to expect ...
      IND = IND + 1

* ...   this writes the values of y(i) at each time step to FILE 18 ...
        J = 0
        WRITE(18,39)T
        N = NEQ/4 + 1
        DO 10 I = 1,N
          IF (J+4.LE.NEQ) THEN
            WRITE(18,40)Y(J+1),Y(J+2),Y(J+3),Y(J+4)
          ELSE IF (J+3.LE.NEQ) THEN
            WRITE(18,40)Y(J+1),Y(J+2),Y(J+3)
          ELSE IF (J+2.LE.NEQ) THEN
            WRITE(18,40)Y(J+1),Y(J+2)
          ELSE IF (J+1.LE.NEQ) THEN
            WRITE(18,40)Y(J+1)
          ELSE
          END IF
          J = J + 4
          IF (J.GE.NEQ) GO TO 20
 10     CONTINUE
 20     CONTINUE

* ... write statistics to arrays ...

           XTIME(IND) = T
        NUMBER(IND,1) = NSTEP
        NUMBER(IND,2) = NFCN
        NUMBER(IND,3) = NJAC
        NUMBER(IND,4) = NSOLV

* ... end of write statistics to arrays ...

      ELSE
        VAR1 = 'THERE IS AN ERROR ...'
        WRITE(IOUT,100)VAR1
        WRITE(IOUT,305)ISTATE
        WRITE(IOUT,*)
        WRITE(IOUT,*)
      END IF

      IF (T.GT.TINIT) THEN
        IF ((ISTATE.EQ.2.AND.IND.EQ.NT+1).OR.(ISTATE.NE.2.AND.IND.NE.0))
     -  THEN
*** WRITE OUT VARIABLE NAMES
          DO 34 I=1,NEQ
 34       WRITE(EQNAME(I),33)I
* ... BRIEF, MEDIUM & LONG ...
          WRITE(IOUT,*)'RESULTS TABLE'
          WRITE(IOUT,*)'-------------'
          CALL TABLE(NEQ,EQNAME,IND,IOUT)
          WRITE(IOUT,*)
          WRITE(IOUT,*)
          WRITE(IOUT,*)'SPRINT STATISTICS'
          WRITE(IOUT,*)'-----------------'
          WRITE(IOUT,*)
          WRITE(IOUT,21)
          WRITE(IOUT,22)
          DO 12 I = 1, IND
            IF (T.GE.1000.0) THEN
              WRITE(IOUT,233)XTIME(I),(NUMBER(I,J),J=1,4)
            ELSE
              WRITE(IOUT,23)XTIME(I),(NUMBER(I,J),J=1,4)
            END IF
 12       CONTINUE
          WRITE(IOUT,*)
* ... end of brief ...
          IF (OUTPUT.NE.'BRIEF') THEN
* ... write out xmxvar ...
            WRITE(IOUT,*)
            DO 14 I = 1, NEQ
              WRITE(IOUT,15)EQNAME(I),XMXVAR(I),ATOL(I)
              TABS = ATOL(I) * 1.D-3
              IF (XMXVAR(I).LT.TABS) THEN
                WRITE(IOUT,16)EQNAME(I)
              END IF
 14         CONTINUE
            WRITE(IOUT,*)
          END IF
* ... extra results file ...
* ... end of long ...
        END IF
      END IF

 15   FORMAT(1X,'MAX ABS VALUE OF ',A6,' = ',D13.7,', ABS TOLERANCE',
     -' = ',D13.7)
 16   FORMAT(1X,'**** WARNING **** ABS TOL FOR ',A6,' DOES NOT ALLOW',
     - ' MUCH ACCURACY')

 39   FORMAT(1X,E13.7)
 40   FORMAT(1X,4(E13.7,1X))

21    FORMAT (1X,'  TIME  | NO OF STEPS | NO OF FN CALLS |',1X,
     - 'NO OF JAC EVALS | NO OF ITERS  ')
22    FORMAT(1X,72('-'))
23    FORMAT(1X,F8.4,'|',4X,I4,5X,'|',6X,I4,6X,
     -                          '|',6X,I4,7X,'|',4X,I4,7X)
 33       FORMAT('Y(',I3,')')
233   FORMAT(1X,E8.2,'|',4X,I4,5X,'|',6X,I4,6X,
     -                          '|',6X,I4,7X,'|',4X,I4,7X)
100   FORMAT(1X,A)
301   FORMAT(1X,'INTEGRATION FAILED AT TIME ',F6.2)
3011  FORMAT(1X,'INTEGRATION FAILED AT TIME ',E8.2)
302   FORMAT(1X,'NOTHING WAS DONE AS TOUT = T WITH ISTATE = 0 ',
     - 'ON INPUT')
3022  FORMAT(1X,'NOTHING WAS DONE AS TOUT = T WITH ISTATE = 1 ',
     - 'ON INPUT')
303   FORMAT(1X,'INTEGRATION FAILED AT TIME ',F6.2,' AS TOO MANY ',
     -   'INTEGRATION STEPS TAKEN')
3033  FORMAT(1X,'INTEGRATION FAILED AT TIME ',E8.2,' AS TOO MANY ',
     -   'INTEGRATION STEPS TAKEN')
304   FORMAT(1X,'INTEGRATION FAILED AT TIME ',F6.2,' AS TOO MUCH ',
     -   'ACCURACY WAS REQUESTED')
3044  FORMAT(1X,'INTEGRATION FAILED AT TIME ',E8.2,' AS TOO MUCH ',
     -   'ACCURACY WAS REQUESTED')
305   FORMAT(1X,'PLEASE LOOK UP ISTATE = ',I3,' IN THE MANUAL')

      RETURN
      END
      SUBROUTINE TABLE(NEQ,EQNAME,IND,IOUT)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

* ... to read y(i)'s and tout's from FILE 18 and output a table to
*     FILE 3
* ... U NEED :
* ...   1. NEQ       - ARG
* ...   2. N         - CALC
* ...   3. EQNAME    - ARG
* ...   4. TOUTS     - READ
* ...   5. NOF TOUTS - ARG
* ...   6. Y(I)'S    - READ

      CHARACTER*6 EQNAME
      CHARACTER*13 Y1,Y2,Y3,Y4
      DIMENSION EQNAME(NEQ)

* ... CLEAR OUT THE EQNAME ARRAY ...
C     DO 9 I = NEQ+1,1000
C9    EQNAME(I) = '      '

* ... first work out how many lines the y(i)'s are written on ...
* ... n = number of lines  & also number of tables to write out ...

      XN = NEQ/4.
       N = INT(XN)
      IF (INT(XN).NE.XN) N = N + 1
      II = 0
      IREM = NEQ
* ...
* ... do i = 1 to number of tables ...
      DO 10 I = 1,N
* ...   write headings ...
        WRITE(IOUT,*)
        IMIN = MIN0(IREM,4)
        WRITE(IOUT,50) (EQNAME(II+IDUM),IDUM=1,IMIN)
        WRITE(IOUT,51)
        IREM = IREM - 4
        II = II + 4
        REWIND 18
* ...   do j = 1 to number of lines per table ...
        DO 20 J = 1,IND
          READ(18,40)T
          IF (I.GT.1) THEN
            DO 30 K = 1,I-1
* ...     read in any y(i)'s not required ...
            READ(18,40)DUM1,DUM2,DUM3,DUM4
 30         CONTINUE
          END IF

* ...     read in the y(i)'s that are required ...
          READ(18,39)Y1,Y2,Y3,Y4
* ...     and write them to the table ...
          IF (T.LT.1000.0) THEN
            WRITE(IOUT,52)T,Y1,Y2,Y3,Y4
          ELSE
            WRITE(IOUT,53)T,Y1,Y2,Y3,Y4
          END IF
* ... check how many more to read in ...

          IF (I.LT.N) THEN
            DO 31 K = I+1,N
* ...     read in the rest of the y(i)'s not required ...
            READ(18,40)DUM1,DUM2,DUM3,DUM4
 31         CONTINUE
          END IF
 20     CONTINUE
 10   CONTINUE

 40   FORMAT(1X,4(E13.7,1X))
 39   FORMAT(1X,4(A13,1X))
 50   FORMAT(1X,2X,'TIME',2X,4('|',4X,A6,5X))
 51   FORMAT(1X,72('-'))
 52   FORMAT(1X,F8.4,4('|',1X,A13,1X))
 53   FORMAT(1X,E8.3,4('|',1X,A13,1X))

      RETURN
      END
      SUBROUTINE SETPDE(NEQN,NPDE,NPTS,X,Y,SPACE,STEP,WORK,NWK,M,TS,
     1                  IBAND,ITIME,REMESH,MAXNPT,XBK,IBK,NEL,NPTL,
     2                  NV,NXI,XI)
C**********************************************************************
C ROUTINE TO SETUP THE CALLS TO THE PDE INITIALISATION ROUTINES
C INITCC AND INITSK.
C KNOWN BUGS - WHAT ABOUT REPEAT AND INCORRECT CALLS
C              + CHANGE OF INTEGRATOR MIDWAY ????
C**********************************************************************
      INTEGER I, IBAND, IBK, IFIRST, ILAST, IRESWK, ITIME, M, MAXNPT,
     1        NEL, NEQN, NIPTS, NPDE, NPTL, NPTS, NV, NWKMON, NWKRES,
     2        NXI
      DOUBLE PRECISION X(1), Y(1), WORK(1), XBK(1), XI(1), TS
      LOGICAL REMESH
      CHARACTER*6 SPACE,STEP
C
      COMMON /MYWORK/ NWKRES,NWKMON
      NWKMON = 1
         IF (STEP.EQ.'BLEND') THEN
                                IRESWK = 22+18*(NPDE*NPTS+NV)
           ELSE IF (STEP.EQ.'GEAR')  THEN
                                IRESWK = 22+10*(NPDE*NPTS+NV)
           ELSE IF (STEP.EQ.'THETA'.OR.STEP.EQ.'THETB') THEN
                                IRESWK = 22+8*(NPDE*NPTS+NV)
           ELSE IF (STEP.EQ.'ADAMS') THEN
                                IRESWK = 22+17*(NPDE*NPTS+NV)
           ELSE
                WRITE(6,*)'SETPDE - STEP INTEGRATOR NOT RECOGNISED'
                ITIME = -1
                RETURN
         END IF
C
      IF (SPACE.EQ.'SKEEL') THEN
         NWKRES = NPDE*(NPTS*2 + NXI*6 + 17) + NXI + NV
     *               +2*NPDE*(MAXNPT-NPTS) + 4*MAXNPT
      IF (NWKRES.GT.NWK) THEN
        WRITE(6,*)'WARNING - INSUFFICIENT WORK SPACE FOR DISCRETISATION'
        WRITE(6,*)'REQUIRED SPACE IS ',NWKRES,'ACTUAL SPACE IS ',NWK
        ITIME = -1
        RETURN
      END IF
C
      NIPTS = NPTS
      CALL INITSK(NEQN,NPDE,NPTS,X,Y,WORK(IRESWK),NWKRES,M,TS,IBAND,1,
     *            REMESH,MAXNPT,NV,NXI,XI)
C
      IF (NEQN.NE.(NPDE*NIPTS+NV)) THEN
C         WRITE*,'MOVING STORES IN SETPDE - NEQN = ',NEQN
          IFIRST = IRESWK + (NEQN-NPDE*NIPTS-NV)
          ILAST  = IFIRST + NWKRES
          DO 10 I=ILAST,IFIRST,-1
10        WORK(I) = WORK(I-NEQN+NPDE*NIPTS+NV)
      END IF
C
      ELSE IF (SPACE.EQ.'COLL') THEN
         NWKRES = 2*NPTL*(NPTL+NEL+1) +2 +NV
     *               +NPDE*(7*(NPTL+NXI)+8)
      IF (NWKRES.GT.NWK) THEN
        WRITE(6,*)'WARNING - INSUFFICIENT WORK SPACE FOR DISCRETISATION'
        WRITE(6,*)'REQUIRED SPACE IS ',NWKRES,'ACTUAL SPACE IS ',NWK
        ITIME = -1
        RETURN
      END IF
C
      CALL INITCC(NEQN,NPDE,NPTS,X,Y,WORK(IRESWK),NWKRES,M,TS,IBAND,1,
     *            XBK,IBK,NEL,NPTL,NV,NXI,XI)
C
C FOLLOWING NOT NEEDED - NO REMESHING WITH COLLOCATION
C     IF (NEQN.NE.(NPDE*NIPTS+NV)) THEN
C         WRITE*,'MOVING STORES IN SETPDE - NEQN = ',NEQN
C         IFIRST = IRESWK + (NEQN-NPDE*NIPTS-NV)
C         ILAST  = IFIRST + NWKRES
C         DO 10 I=ILAST,IFIRST,-1
C0        WORK(I) = WORK(I-NEQN+NPDE*NIPTS+NV)
C     END IF
C
      ELSE
          WRITE(6,*) 'SPACE DISCRETISATION NOT RECOGNIZED IN SETPDE'
          ITIME = -1
      END IF
C
      RETURN
      END
C
C **********************************************************************
       SUBROUTINE SPDEF1( T, X, NPTL, NPDE, U, DUDX, C, Q, R, NV, V,VD)
C DUMMY ROUTINE - IN CASE USER IS ACCESSING INITCC
       INTEGER NPDE, NPTL, NV
       DOUBLE PRECISION T, X(1), U(NPDE), DUDX(NPDE),
     1         C(NPDE,NPDE), Q(NPDE), R(NPDE), V(1) ,VD(1)
      RETURN
      END
C
C***********************************************************************
      SUBROUTINE BNDR( T, BETA, GAMMA, U, UX, NPDE, LEFT, NV, V, VD, IR)
C DUMMY ROUTINE - IN CASE USER IS ACCESSING INITCC
      INTEGER NPDE, NV, IR
      DOUBLE PRECISION T, BETA(NPDE), GAMMA(NPDE), U(NPDE), UX(NPDE),
     1                 V(1), VD(1)
      LOGICAL LEFT
      RETURN
      END
C
C***********************************************************************
      SUBROUTINE SPDEFN(T, X, NPTL, NPDE, U, DUDX, UDOT, UTDX,
     *                  Q, R, NV, V, VD, IRES)
C DUMMY ROUTINE - IN CASE USER IS ACCESSING INITSK
       INTEGER NPDE, NPTL, NV, IRES
       DOUBLE PRECISION T, X(1), U(NPDE,1), DUDX(NPDE,1),
     1         UDOT(NPDE,1),Q(NPDE,1), R(NPDE,1), V(1),VD(1),UTDX(NPDE)
      RETURN
      END
C
C **********************************************************************
      SUBROUTINE SBNDR( T, BETA, GAMMA, U, UX, UDOT, UTDX,
     `                  NPDE, LEFT, NV, V, VD, IRES)
C DUMMY ROUTINE - IN CASE USER IS ACCESSING INITSK
      INTEGER NPDE, NV, IRES
      DOUBLE PRECISION T, BETA(NPDE), GAMMA(NPDE), U(NPDE), UX(NPDE),
     1       UDOT(NPDE),UTDX(NPDE),V(1), VD(1)
      LOGICAL LEFT
      RETURN
      END
C
C***********************************************************************
      SUBROUTINE SVINIT(V,NV)
      INTEGER NV
      DOUBLE PRECISION V(1)
C     WRITE*,'WARNING - YOU HAVE NOT PROVIDED ROUTINE SVINIT'
      RETURN
      END
C
C***********************************************************************
      SUBROUTINE SODEFN(T,NV,V,VDOT,NPDE,NXI,XI,UI,UXI,RI,
     *                  UTI,UTXI,VRES,IRES)
      INTEGER NV, NPDE, NXI, IRES
      DOUBLE PRECISION T, XI(NXI),UI(NPDE,NXI),UXI(NPDE,NXI),
     *          RI(NPDE,NXI),UTI(NPDE,NXI),UTXI(NPDE,NXI),VRES(NV),
     *          V(NV),VDOT(NV)
C     WRITE*,'WARNING - YOU HAVE NOT PROVIDED ROUTINE SODEFN'
      RETURN
      END
C
C***********************************************************************
      SUBROUTINE EVNRMF(TIME,NIP,NPDE,X,U,R,MAXNPT,FMON)
      INTEGER NIP, MAXNPT, NPDE
      DOUBLE PRECISION TIME, U(NPDE,1),X(NIP),FMON(NIP),R(NPDE,1)
      RETURN
      END

C***********************************************************************
      SUBROUTINE CTEST1(I,T,WKMON,NWKMON,Y,NEQ)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE CTEST1                                         C
C   IMPLEMENTED BY : David Di Ruscio  NTH/ITK  4/3-1989           C
C    PURPOSE                                                      C
C    To pass Y(I) to MONITR trough SPRINT at each call.           C
C    Information could be placed in COMMON blocks to comunicate   C
C    whit MONITR or calling PROGRAM. This allows user more        C
C    flexibility                                                  C
C***END PROLOGUE CTEST1                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I, II, NEQ, NWKMON
      DOUBLE PRECISION WKMON(1),Y(1),T

C1... Print info at each call to SPRINT (I=1,...,NT times)    .....
      WRITE(*,'(5X,A,I3,1X,A,D12.4)')'EXIT SPRINT NO. ',I,'T = ',T

C2... Store info to be passed to MONITR                       .....
      WKMON(1) = T
      DO 10 II=2,NEQ+1
         WKMON(II) = Y(II)
10    CONTINUE

      RETURN
      END

