C  JULY 1987 CHANGES.
C      (1) THE VARIABLE K IS SET TO 1 ON THE FIRST STEP
C      (2) K IS THEN SET TO 2 AT THE END OF A SUCCESSFUL STEP.
C
C  APRIL 1988 CHANGES ACOR DIVIDED BY (H*THETA) IN DO 1025 LOOP
C      SO AS TO TAKE ACCOUNT OF CHANGED BACKSUB
C
       SUBROUTINE THESET( NY2DIM, IRET)
       INTEGER NY2DIM, IRET
C**********************************************************************
C      SETUP ROUTINE FOR THE THETA METHOD INTEGRATOR STHETA
C      THIS ROUTINE MUST BE CORRECTLY CALLED BT THE USER BEFOPRE THE
C      MAIN SPRINT PACKAGE IS ENTERED WITH STHETA AS THE STEP (CORE)
C      INTERGRATOR.
C
C   INPUT PARAMETER
C       NY2DIM  : THE SECOND DIMENSION OF THE WORK ARRAY YSAVE USED BY
C                 STHETA - YSAVE(NEQ, NY2DIM); NY2DIM MUST BE >= 4
C                 (NEQ IS THE NUMBER OF ORDINARY DIFFERENTIAL/ALGEBRAIC
C                  EQUATIONS BEING SOLVED)
C
C   OUTPUT PARAMETER
C         IRET  ; SET TO -1 IF NY2DIM IS TOO SMALL , SET TO ZERO OTHER-
C                 WISE.
C***********************************************************************
       CHARACTER*6 ODCODE
       COMMON /ODECHK/ ODCODE
       SAVE   /ODECHK/
C
       IF(NY2DIM .LT. 4)THEN
          CALL SERROR(' SETUP ROUTINE FOR STHETA - PARAM NY2DIM (=I1)
     1    SHOULD BE (=I2)', 1, 2, NY2DIM, 4, 0, 0.0D0,0.0D0)
          IRET = -1
       ELSE
          IRET = 0
          ODCODE = 'STHETA'
       END IF
       RETURN
       END
C=======================================================================
C
C   INTEGRATOR STHETA ;THETA METHOD FOR SPRINT PACKAGE.
C
C=======================================================================
      SUBROUTINE STHETA( NEQN, Y, W, NY, WT, DY, DEL, ACOR, INLN, ISTEP,
     1                   THETA, H, T, HMIN, HMXI, IDAE)
C********************************************************************
C
C     THIS ROUTINE SOLVES THE SYSTEM OF DIFFERENTIAL/ALGEBRAIC EQUATIONS
C  OF THE FORM
C        E Y' = F(Y, T)                                            (1)
C  WHERE E IS A SINGULAR MATRIX WHEN THERE ARE ALGEBRAIC EQUATIONS
C  PRESENT IN THE SYSTEM.
C     THE INTEGRATION METHOD USED IS:
C  E*Y(N+1) = E*Y(N) + H(THETA*F(Y,T(N+1)) + (1-THETA)*F(Y,T(N)))  (2)
C     WITH LOCAL ERROR CONTROL.
C
C     THE ITERATION MATRIX G IS GIVEN BY:
C          G  =  E - H*THETA*DF/DY                                 (3)
C
C-----------------------------------------------------------------------
C DIFFERENCES FROM THE CHUA AND DEW CODE.
C----------------------------------------
C THERE ARE TWO MAIN DIFFERENCES FROM T.S. CHUA'S CODE.
C      FIRSTLY AS THE SYSTEM OF NONLINEAR EQUATIONS IS SOLVED BY THE
C ROUTINE NLSLVR THE CHUA STRATEGY OF RE-EVALUATING THE JACOBIAN MATRIX
C IN MID-ITERATION IS NONLONGER USED. IN ANY CASE AS RELAXATION IS USED
C TO CORRECTLY SCALE THE ALGABRAIC COMPONENTS IT IS NO-LONGER SO
C IMPORTANT TO AVOID REDUCING THE STEPSIZE AND RE-EVALUATING THE
C JACOBIAN MATRIX.
C    SECONDLY , A MAXIMUM OF THREE ITERATIONS
C IS ALLOWED IN THE SOLUTION OF THE NONLINEAR EQUATIONS. THIS COULD BE
C CHANGED BY INCLUDING THE SPRINT SYSTEM COMMON BLOCK /SSOLVR/  AND BY
C SETTING MAXIT TO 5
C-----------------------------------------------------------------------
C
C  THE VARIABLES USED HAVE THE FOLLOWING MEANINGS:
C *NEQN--NUMBER OF EQUATIONS TO BE SOLVED; BOTH DIFFERENTIAL AND
C        ALGEBRAIC.
C *T   --THE INDEPENDENT VARIABLE. ON FIRST CALL IT SHOULD BE SET
C        TO THE INITIAL CONDITION. ON RETURN IT CONTAINS THE VALUE
C        OF T FOR WHICH Y IS THE SOLUTION.
C *H   --PROPOSED STEPSIZE FOR THE STEP. ON FIRST CALL, IT SHOULD
C        CONTAIN THE INITIAL ESTIMATE OF THE STEPSIZE .
C *Y   --THE DEPENDENT VARIABLE. ON FIRST CALL SHOULD BE SET TO
C        THE INITIAL CONDITIONS. ON RETURN IT CONTAINS THE SOLUTION
C        AT T. DIMENSIONED AS Y(NEQN).
C *DY  --AN ARRAY RETURNS THE QUANTITY H*DY/DT AT THE NEW TIME LEVEL.
C        ON FIRST ENTRY, IT SHOULD CONTAIN THE DERIVATIVES DY/DT AT
C        TIME T. DIMENSIONED AS DY(NEQN).
C *DEL --RETURNS THE RESULT OF A BACK-SUBSTITUTION (SEE INLN = 5)
C        DIMENSIONED AS DEL(NEQN).
C *ACOR--WORKSPACE ON A CALL TO THE NONLINEAR EQUATIONS SOLVER
C        WITH INLN = 5 IT CONTAINS THE VECTOR WHICH IS PREMULTIPLIED BY
C        THE MATRIX E AND THEN FED TO THE BACKSUBSTITUTION ROUTINE.
C        DIMENSIONED AS ACOR(NEQN)
C  ISTEP -INDICATOR FOR THE INTEGRATOR
C         ON EXIT   >0 FOR SUCCESSFUL CALL
C                   <0 FOR STEP FAILURE.
C         ON ENTRY, IT HAS THE FOLLOWING MEANINGS:
C         =-1 INITIAL STEP.
C         = 0 REVERSE COMMUNICATION ENTRY.
C         = 1 TAKE A NEW STEP , CONTINUING FROM THE PREVIOUS STEP.
C         = 2 TAKE A CONTINUATION STEP BUT WITH NEW VALUES OF H OR NEQN
C         = 3 TAKE A NEW STEP WITH A NEW VALUE OF H
C         ON EXIT, IT HAS THE FOLLOWING MEANINGS:
C         = 1 SUCCESSFUL STEP.
C         = 0 REVERSE COMMUNICATION EXIT.
C         =-1 REQUESTED ERROR COULD NOT BE ACHIEVED.
C         =-2 CORRECTOR CONVERGENCE COULD NOT BE ACHIEVED.
C         =-3 RESIDUAL FORMING ROUTINE ORDERED A RETURN TO THE DRIVER.
C         =-4 ERROR CONDITION IN RESIDUAL ROUTINE COULD NOT BE AVOIDED.
C         =-5 ERROR IN FORMING THE JACOBIAN. SINGULAR MATRIX
C         =-6 NOT USED HERE
C         =-7 IMPOSSIBLE ERROR OCCURED IN LINEAR ALGEBRA ROUTINES.
C  INLN  -POINTER TO INFORM THE CALLING PROGRAM ABOUT THE TYPE OF
C         PROBLEM DEPENDENT INFORMATION REQUIRED BY THE INTEGRATOR.
C         ON EXIT.....
C         = 0 INTEGRATION HAS FINISHED, CHECK ISTEP FOR POSSIBLE
C             ERROR.
C         = 1 TO COMPUTE THE SOLUTION OF NON-LINEAR SYSTEM OF EQUATIONS
C             GIVEN BY (2). THE INITIAL PREDICTION FOR Y(N+1) IS GIVEN
C             IN VECTOR Y; THE VALUES OF Y AND H*Y' AT TIME TN ARE
C             STORED IN THE WORKSPACE W(.,1) AND W(.,2) RESPECTIVELY.
C             IF A SOLUTION CANNOT BE FOUND BECAUSE THE ITERATION DOES
C             NOT CONVERGE, THEN SET IND(2)=-1 TO REDUCE THE STEPSIZE.
C             OTHERWISE SET IND(2)=-2 TO TERMINATE THE INTEGRATION.
C         = 5 TO COMPUTE THE SOLUTION OF G*DEL = DYY, IE A BACK-
C             SUBSTITUTION.
C         = 3 EVALUATE A NEW RIGHT-HAND FUNCTION F(Y,T) IN EQN.(1) AND
C             AND RETURN THE RESULTS IN VECTOR DEL.
C         ON ENTRY.....
C         = 0 NORMAL ENTRY - ACTION TO BE TAKEN IS GOVERNED BY ISTEP.
C         = 1 THE NONLINEAR SYSTEM OF EQUATIONS HAS BEEN SOLVED.
C         = 2 CONVERGENCE FAILURE OCCURED IN THE NONLINEAR EQUATIONS
C             SOLVER.
C         = 3, 5 THE TASKS SET BY EXIT VALUES OF ISAVE = 3  OR 5 WERE
C             SUCCESSFULLY PERFORMED.
C         = 4,6,7 NOT USED HERE AT PRESENT.
C         = -1 SINGULAR JACOBIAN MATRIX WAS FOUND .
C         = -2 ERROR IN RESIDUAL ROUTINE FORCES AN IMMEDIATE RETURN.
C         = -3 ILLEGAL VALUES ENCOUNTERED IN THE RESIDUAL ROUTINE.
C              REDUCE THE STEP SIZE BY A FACTOR OF TWO AND TRY AGAIN.
C         = -4 CONDITION ENCOUNTERED IN THE RESIDUAL ROUTINE FORCES
C              THE STEP TO BE FAILED AND A RETURN TO BE MADE TO THE
C              MONITOR ROUTINE.
C         = -5 IMPOSSIBLE ERROR OCCURED IN LINEAR ALGEBRA .
C
C ISAVE  -INTERNAL INDICATOR TO INFORM THE INTEGRATOR THE EXACT POINT
C         TO RETURN TO.
C *WT  --VECTOR OF WEIGHTS FOR ERROR CRITERION.
C        DIMENSIONED AS WT(NEQN)
C *W   --ARRAY USED AS WORKSPACE. DIMENSIONED AS W(NEQN,4), WHERE :
C        W(J,1) HOLDS THE VALUES OF Y AT PREVIOUS TIME LEVEL. IE. Y(N).
C        W(J,2) HOLDS THE VALUES OF H*DY AT PREVIOUS TIME LEVEL.
C        W(J,3) HOLDS BETA(N) FROM THE PREVIOUS TIME LEVEL.
C        W(J,4) CONTAINS THE VALUES OF Y(N)-Y(N-1).
C HMXI  : 1.0D0/HMAX  WHERE HMAX IS
C         THE MAXIMUM STEPSIZE THE INTEGRATOR IS ALLOWED TO TAKE
C         IN THE CASE WHEN THIS IS ZERO THE POSSIBLE STEPSIZE IS
C         RESTRICTED TO 1.0D0/TWOU WHERE TWOU IS THE UNIT ROUNDOFF ERR.
C HMIN  : THE MINUMUM STEPSIZE THE INTEGRATOR IS ALLOWED TO USE.
C T     : THE CURRENT VALUE OF THE INDEPENDENT VARIABLE.
C H     : THE CURRENT VALUE OF THE STEP SIZE.
C RESERR - LOGICAL PARAMETER THAT WHEN TRUE INDICATES THAT AN ERROR
C         HAS BEEN RETURNED FROM THE RESIDUAL ROUTINE.
C
C=======================================================================
C
C  NOTE--WHEN THE ROUTINE IS FIRST CALLED,
C        ISTEP , INLN  MUST BE SET TO -1 AND 0 RESPECTIVELY.
C
C***********************************************************************
      LOGICAL RESERR
      CHARACTER*6 ODCODE
      DOUBLE PRECISION T,H,FAC,FACTOR,HALF,ONE,ZERO,THETA,ERRL,COEF,
     *                 TWOU,TENP,DIF,FAC2,COEF1,COEF2,SUM
      DOUBLE PRECISION Y(NEQN),DY(NEQN),DEL(NEQN),W(NY,4),ACOR(NEQN),
     *                 R,WT(NEQN), HMIN,HMXI, VNORM
      INTEGER ISTEP, INLN, ISAVE, IOVFLO, K, NRE, NJE, NITER, JK, NEQN,
     1        JKOLD, N, I, J, IDAE(1), NINTER, JSTEP, NHALF, JCUR, MSBP,
     2        ITRACE, IDEV, NST, NY, IJ
      COMMON /ODECHK/ ODCODE
      COMMON /SCONS1/ TENP, TWOU, IOVFLO
      COMMON /SDEV2/ ITRACE, IDEV
      COMMON /LSTATS/ JK,JKOLD, NST,NRE,NJE,NITER , NINTER
      COMMON /STHESV/ COEF1,  COEF2, DIF, FAC, FACTOR, FAC2, ONE , SUM,
     1                 ZERO, HALF, R(4), COEF(3), ERRL, ISAVE, JSTEP,
     2                 NHALF, I,J, JCUR, MSBP, N, RESERR
      SAVE   /ODECHK/, /STHESV/, /SCONS1/, /LSTATS/
C
C**********************************************************************
C
C  THE COMMON BLOCKS USED ARE:
C /STHESV/-CONTAINS THE STATISTICS REQUIRED AT THE END OF THE SOLUTION
C      *R(4)-VECTOR USED AS A WORKSPACE.
C         `` (1)-HOLDS THE OLD STEPSIZE.
C         `` (2)-HOLDS THE OLD VALUE OF TIME ,T, USED IN RESTARTING.
C         `` (3)-HOLDS THE MAXIMUM (INTEGRATOR DEPENDENT) STEPSIZE.
C         `` (4)-HODS THE TIME AT WHICH A RESTART PHASE IS INITIATED.
C        *NHALF -COUNTS THE NO. OF STEP REDUCTIONS IN A STEP. ONLY
C                3 STEP REDUCTIONS IS ALLOWED.
C        *COEF(3)-WORKSPACE USED FOR STORING THE COEFFICIENTS REQUIRED
C                 IN THE ERROR ESTIMATION.
C        *JSTEP -THE NUMBER OF STEPS TO BE PERFORMED BEFORE STEPSIZE
C                CHANGES IS CONSIDERED. A MIN OF 4 STEPS IS NEEDED FOR
C                A PREVIOUS STEPSIZE INCREMENT AND A MIN OF K+1 STEPS
C                IS REQUIRED FOR A PREVIOUS STEPSIZE REDUCTION.
C        *ERRL  -HOLDS THE WEIGHTED LOCAL ERROR NORM.
C /SDEV2/-TO TRACE THE INTEGRATOR FOR DEBUGGING PURPOSES:
C        *IDEVO -OUTPUT DEVICE NUMBER.
C        *ITRACE-TRACE LEVEL REQUIRED.
C                =0 FOR NO TRACE
C                =1 TO OUTPUT INTERMEDIATE RESULTS AT APPROPRIATE
C                   POINTS.
C**********************************************************************
C NINTER SPECIFIES HOW MUCH OF THE MEMORY VECTOR IS TO BE USED IF
C ITERPOLATION IS USED E.G. IN P.D.E. REMESHING.
C
      ZERO=0.0D+0
      HALF=0.5D+0
      ONE=1.0D+0
      THETA = 0.55D0
      JK = 1
      JKOLD = 1
      IF(DABS(HMXI) .LE. TWOU)THEN
         R(3) = 1.0D0 / TWOU
      ELSE
         R(3) = 1.0D0/HMXI
      END IF
C
C  SET UP THE STRATEGY PARAMETERS FOR THE INITIAL STEP
C
      IF (ISTEP .LT. 0) THEN
          IF(ODCODE .NE. 'STHETA')THEN
            CALL SERROR(' STHETA THE SETUP ROUTINE WAS NOT CALLED FOR
     1      THIS ROUTINE BEFORE SPRINT WAS ENTERED', 1, 0, 0, 0, 0,
     2      0.0D0, 0.0D0)
            ISTEP = -6
            RETURN
          END IF
          NINTER = 4
          COEF(1)=HALF - THETA
          COEF(2)=THETA*THETA - THETA + ONE/6.0D+0
          COEF(3)=2.0D+0*THETA
          R(1)=H
          N = NEQN
          NHALF = 0
          MSBP = 0
          K = 1
          JCUR = 0
          ISAVE = -1
          JSTEP = 0
          RESERR = .FALSE.
          FAC   = 1.0D0
      END IF
C
C**********************************************************************
C
C  STAGE--1
C
C***********************************************************************
C
C     IF INLN >0 JUMP TO THE APPROPRIATE POINTS IN THE INTEGRATOR
C     OTHERWISE EITHER IT IS A FRESH CALL FOR THE PRESENT TIME LEVEL
C     OR AN ERROR IN THE NON-LINEAR EQUATIONS SOLVER HAS BEEN FOUND
C
      I=INLN + 5
      INLN = 0
      GO TO ( 9, 9,  9,  9, 14, 40, 9, 9, 9, 70), I
C      INLN= -4 -3  -2  -1  0   1   2   3  4   5
 9    CONTINUE
      IF(I .EQ. 1 .OR. I .EQ. 3)THEN
         ISTEP = -3
      ELSE IF(I .EQ. 4)THEN
         ISTEP = -5
      ELSE IF(I.EQ. 7 .AND. JCUR .EQ. 2)THEN
C        FORM THE JACOBIAN FOR THE FIRST TIME ON THIS STEP
         JCUR = 0
         GOTO 30
      ELSE IF(I.EQ. 0)THEN
         ISTEP = -7
      END IF
C-----------------------------------------------------------------------
C  RETRACT THE SOLUTION TO THAT AT THE PREVIOUS TIME STEP AND RETURN.
C-----------------------------------------------------------------------
        IF(K .EQ.1)THEN
           DO 11 J = 1,NEQN
              W(J,1) = W(J,1) - W(J,2)
              Y(J)   = W(J,1)
              W(J,2) = W(J,2)/FAC
              DY(J) = W(J,2) / R(1)
 11        CONTINUE
        ELSE
           FAC = H/R(1)
           DIF = ONE + THETA * (FAC - ONE)
           FAC2 = -(ONE -THETA)*FAC
           DO 12 J = 1,NEQN
              W(J,1) = W(J,1) - (DIF* W(J,3) + W(J,4))*FAC
              Y(J)   = W(J,1)
              W(J,2) = (W(J,2)*THETA - (DIF*W(J,3)+ W(J,4))*FAC)/FAC2
              DY(J)  = W(J,2)/R(1)
 12        CONTINUE
        END IF
        IF(I .EQ. 2)THEN
C          RESIDUAL ROUTINE HAS RETURNED IRES = INLN = 3
           RESERR = .TRUE.
           GOTO 80
        ELSE IF (I .EQ. 7)THEN
C             CONVERGENCE FAILURE IN NONLINEAR SOLVER WITH A NEW
C             JACOBIAN FOR THIS STEP - TRY REDUCING STEPSIZE A
C             MAXIMUM OF THREE TIMES.
              GOTO 80
        END IF
        H = R(1)
        T = R(2)
        RETURN
14    RESERR = .FALSE.
C
C   CHECK IF NEQN HAS CHANGED , POSSIBLY ON A RESTART WITH ISTEP = 1
C
      IF(ISTEP .EQ. 1)GOTO 20
      IF(ISTEP .EQ. -1)THEN
C        PUT INITIAL Y AND YDOT INTO MEMORY VECTOR
         DO 142 I = 1,NEQN
            W(I,1) = Y(I)
 142        W(I,2) = DY(I)*H
      END IF
      IF(ISTEP .EQ. 2 .AND. FAC .GT. 1.0D0)FAC = 1.0D0
      IF(R(1) .NE. H)THEN
         IF(DABS(H) .GT. R(3))H = R(3) * DSIGN(1.0D0,H)
         IF(DABS(H) .LT. HMIN) H = HMIN* DSIGN(1.0D0,H)
         JCUR = 0
         FAC  = H/R(1)
         H    = R(1)
      END IF
      IF(ISTEP .NE. 2) GOTO 20
      JSTEP= 3
C     JCUR = 0
CZ           TO FORCE A JACOBIAN EVALUATION AS MEMORY MAY HAVE CHANGED.
      IF(N .NE. NEQN)THEN
C        NEQN HAS BEEN CHANGED BY MONITR OR BY THE USER. FLAG AN
C        INCREASE AND IF A DECREASE ZERO UNUSED PARTS OF WORKSPACES
         IJ = N - NEQN
         IF(IJ .LT. 0)THEN
           CALL SERROR('STHETA - THE VALUE OF NEQN HAS BEEN INCREASED
     1     FROM(=I1) TO (=I2) ', 1, 2, N, NEQN, 0, 0.0D0, 0.0D0)
         END IF
         DO 141 I = NEQN+1,N
           DO 141 J = 1,4
 141          W(I,J) = 0.0D0
         N = NEQN
       END IF
C***********************************************************************
C
C  STAGE--2  PREDICT THE SOLUTION PRESERVING OLD T VALUE IN R(2).
C
C***********************************************************************
 20   R(2)=T
      H=H*FAC
      T=T+H
      FAC=H/R(1)
      IF (ISTEP .EQ. -1 .OR. ISTEP .EQ. 2 .OR. K .EQ. 1)THEN
C         INITIAL STEP OR RESTART FROM ORDER 1
          K=1
          IF(ISTEP.EQ. -1)ISTEP = 1
C         STORE THE INITIAL VALUES OF Y & DY IN W(1,J) &W(2,J)
          DO 1000 J=1,NEQN
             W(J,2)= W(J,2) * FAC
             W(J,1)= W(J,1) + W(J,2)
1000       CONTINUE
      ELSE
C          NON-INITIAL STEP, PREDICT THE SOLUTION TO 2ND ORDER ACCURACY
           K=2
           DIF = ONE + THETA * (FAC - ONE)
           DO 1010 J=1,NEQN
              W(J,1) = W(J,1) + (DIF * W(J,3) + W(J,4)) * FAC
              W(J,2) = (W(J,2)+(DIF*W(J,3)+W(J,4)-W(J,2))/THETA)*FAC
1010       CONTINUE
      END IF
      NHALF=0
C***********************************************************************
C
C  STAGE--3  NON-LINEAR EQUATIONS SOLVER
C
C***********************************************************************
30    CONTINUE
C
C  SET Y AND YDOT TO PREDICTED VALUES AND DECIDE WHETHER TO UPDATE JAC.
C
       DO 35 I = 1, NEQN
          Y(I) = W(I,1)
          DY(I) = W(I,2)/H
35     CONTINUE
       IF(JCUR .EQ. 0) THEN
          JCUR = 1
          INLN = 1
          MSBP = 0
       ELSE
          JCUR = 2
          INLN = 2
       END IF
C
C  EXIT TO SOLVE THE NON-LINEAR EQUATIONS; RETURN TO LABEL 40 IF SUCCESS
C
      RETURN
C***********************************************************************
C
C  STAGE--4 ERROR ESTIMATION
C
C***********************************************************************
C     A SOLUTION OF THE NON-LINEAR EQUATIONS HAS BEEN FOUND CHECK THE
C     NEW DERIVS Y' IN VECTOR DY ANDEXIT TO COMPUTE BETA(N+1) TO BE
C     USED IN THE LOCAL ERROR ESTIMATION. RETRACT MEMORY VECTOR FOR
C     ESTIMATING BETA(N+1) AND W(J,4)
40    FAC  = H/R(1)
      IF(K .EQ. 1)THEN
        DO 1023 J = 1,NEQN
           W(J,1) = W(J,1) - W(J,2)
 1023      W(J,2) = W(J,2)/FAC
      ELSE
        FAC2 = THETA - ONE
        DIF  = ONE + THETA*(FAC - ONE)
        DO 1024 J = 1,NEQN
           W(J,1) = W(J,1) - (DIF*W(J,3)+W(J,4))*FAC
 1024      W(J,2) =(W(J,2)/FAC*THETA - (DIF*W(J,3) + W(J,4)))/FAC2
      END IF
      DO 1025 J=1,NEQN
        DEL (J)=  DY(J)*H - W(J,2)*FAC
        ACOR(J)=(-DY(J)*H + W(J,2)*FAC) / ( H *THETA)
1025  CONTINUE
      INLN = 5
      ISAVE = 2
      RETURN
70    CONTINUE
C
C     EVALUATE THE LOCAL ERROR CORRECTIONS - HOPKIN'S ERROR ESTIMATE
C
      IF (K . EQ. 1) THEN
C         1ST ORDER METHOD
          DO 1030 J=1,NEQN
             ACOR(J)=COEF(1)*DEL(J)
1030      CONTINUE
      ELSE
C         2ND ORDER METHOD
          FAC=H/R(1)
          FAC2=FAC*FAC
          DIF=FAC*COEF(2)/(ONE + COEF(3)*(FAC-ONE))
          COEF1=COEF(1) + DIF
          COEF2=FAC2*DIF
          DO 1040 J=1,NEQN
             ACOR(J)=COEF1*DEL(J) - COEF2*W(J,3)
1040       CONTINUE
      END IF
      ERRL   = VNORM( NEQN, WT, ACOR)
      IF(ITRACE .GE. 1)WRITE(IDEV,8005)ERRL
      IF (ERRL.LE.ONE) GO TO 90
8005  FORMAT(/' SCALED LOCAL ERROR IS =',D15.6)
C***********************************************************************
C
C  STAGE--5  REDUCE H BY FACTOR AND RETURN TO THE PREDICTION STAGE
C            AS THE SOLUTION DOES NOT SATISFY THE LOCAL ERROR TOLERANCE
C
C***********************************************************************
80    FAC=HALF
      JSTEP=3
C           SHOULD POSSIBLY BE K + 1 ??
      NHALF=NHALF+1
      IF(ERRL.GT.2.0D0)FAC=FAC*2.0D0/DMIN1(100.0D0,ERRL)
      JCUR = 0
      T = R(2)
      IF (NHALF . GT. 3 .OR. (DABS(H)*FAC) .LT. HMIN) THEN
C         ERROR-- TOO MANY STEP REDUCTIONS
          ISTEP = -1
          H = R(1)
          IF(RESERR)ISTEP = -4
          CALL SERROR(' STHETA - FAILED DUE TO REPEATED ERROR TEST OR
     1    CONVERGENCE FAILURES ON THE CURRENT STEP (H=R1) AT TIME (=R2)'
     2    , 1, 0, 0, 0, 2, H, T)
          RETURN
      END IF
      GOTO 20
C***********************************************************************
C
C  STAGE--6  SUCCESSFUL STEP
C
C***********************************************************************
90    MSBP = MSBP + 1
      IF(MSBP .EQ. 21)JCUR = 0
      ISTEP = 1
      K = 2
C
C     UPDATE THE MEMORISED VALUES AND PUT THE LOCAL ERROR IN ACOR
C
      DO 1070 J=1,NEQN
         W(J,4)=Y(J) - W(J,1)
         W(J,3)=DEL(J)
         W(J,1)=Y(J)
         W(J,2)=DY(J)* H
1070  CONTINUE
      JSTEP=JSTEP-1
      FAC=ONE
      R(1)=H
C     IF (DABS(H).LT.R(3).AND.ERRL.LT.0.15D+0.AND.JSTEP.LE.0)THEN
      IF (DABS(H).LT.R(3).AND.ERRL.LT.0.25D+0.AND.JSTEP.LE.0)THEN
C        DOUBLE THE STEPSIZE TO BE USED NEXT STEP
         JCUR = 0
         FAC=2.0D+0
C
C         MORE THAN DOUBLE THE STEP IF ERRL IS VERY SMALL
C
         DO 1077 I = 1,4
           IF(ERRL .LE. 0.5* 10.0D0**(-I))FAC = FAC * 2.0
 1077    CONTINUE
         JSTEP=K+1
         IF (DABS(H)*FAC.GT.R(3)) FAC=R(3)/DABS(H)
      END IF
      NHALF = 0
      RETURN
C                       END OF SUBROUTINE RTHETA
      END
