C 
C   CHANGES - APRIL 1988 - NEW LOOP DO 1026 INTRODUCED TO TAKE
C                          ACCOUNT OF CHANGED BACKSUB IN SPRINT
C             AUGUST 1988 - BUG FIXED IN RETRAC SUBROUTINE.
C
C
       SUBROUTINE THBSET( NY2DIM, IRET, NEWTON, NOSWCH, THETA)
       INTEGER NY2DIM, IRET
       LOGICAL NEWTON, NOSWCH
       DOUBLE PRECISION THETA
C**********************************************************************
C      SETUP ROUTINE FOR THE THETA METHOD INTEGRATOR STHETB
C      THIS ROUTINE MUST BE CORRECTLY CALLED BT THE USER BEFOPRE THE
C      MAIN SPRINT PACKAGE IS ENTERED WITH STHETB AS THE STEP (CORE)
C      INTEGRATOR.
C      THIS MODULE IS AN ENHANCED VERSION OF THE STHETA CODE. THERE ARE
C      TWO MAIN IMPROVEMENTS. FIRST THE USER CAN CHANGE THE VALUE OF
C      THETA AND SECOND ,THE USER CAN ASK THE CODE TO SWITCH BETWEEN
C      USING A NEWTON METHOD IN THE NONLINEAR EQUATIONS SOLVER AND
C      BETWEEN USING FUNCTIONAL ITERATION.
C
C   INPUT PARAMETERS
C         NY2DIM: THE SECOND DIMENSION OF THE WORK ARRAY YSAVE USED BY
C                 STHETB - YSAVE(NEQN, NY2DIM); NY2DIM MUST BE > = 4
C         NEWTON ; SET .TRUE. IF A NEWTON METHOD IS TO BE USED AT THE
C                             START OF INTEGRATION.
C                      .FALSE. IF FUNCTIONAL ITERATION IS TO BE USED AT
C                              THE START OF INTEGRATION.
C         NOSWCH ; SET .TRUE. IF NO ATTEMPT TO SWITCH WILL BE MADE- THE
C                             CODE WILL STAY WITH THE METHOD DEFINED BY
C                             THE PARAMETER NEWTON.
C                      .FALSE. FOR SWITCHING BETWEEN NEWTON METHOD AND
C                              FUNCTIONAL ITERATION TO BE USED TO
C                              TRY AND IMPROVE EFFICIENCY.
C        THETA  ;              A DOUBLE PRECISION VARIABLE THAT CONTAINS
C                              THE VALUE OF THETA TO BE USED BY THE CODE
C                              THE RECOMMENDED VALUE OF THETA IS
C                      0.55    BUT ANY VALUE BETWEEN 0.51 AND 0.90 MAY
C                              BE USED.
C         N.B. IN ORDER TO USE INTEGRATION FORMULAE WITH THETA = 1.0
C              USE THE BACKWARD DIFFERENCE FORMULA OF ORDER 1 AND TO
C              USE THETA = 0.5 USE THE ADAMS FORMULA OF ORDER 2 -SEE
C              THE DOCUMENTATION OF MODULE SPGEAR FOR FURTHER DETAILS.
C
C   OUTPUT PARAMETER
C         IRET  ; SET TO -1 IF NY2DIM IS TOO SMALL , SET TO ZERO OTHER-
C                 WISE.
C
C   IMPORTANT NOTE
C   --------------
C     WHEN THE USE OF FUNCTIONAL ITERATION IS SPECIFIED , EITHER
C   THROUGH SWITCHING OR DIRECTLY, GREAT CARE MUST BE TAKEN TO WRITE
C   THE ORDINARY DIFFERENTIAL AND ALGEBRAIC EQUATIONS  IN A SUITABLE
C   FORM FOR FUNCTIONAL ITERATION IN SPRINT. THE RESIDUAL OF THE I TH
C   DIFFERENTIAL EQUATION MUST BE WRITTEN AS
C
C    R(I) = F ( Y,T) - YDOTI(I) (= 0 FOR EXACT Y AND YDOTI(I) )
C            I  -                              -
C                   AND THE RESIDUAL OF THE J TH ALGEBRAIC EQUATION
C   MUST BE WRITTEN AS
C
C    R(J) = G ( Y,T) - Y(J)   (  = 0  FOR EXACT Y(J) . )
C            J  -
C   IN PARTICULAR CARE MUST BE TAKEN WHEN SOLVING P.D.E. PROBLEMS TO
C   SEE THAT DIRICHLET BOUNDARY CONDITIONS ARE IN THIS LAST FORM.
C     FURTHERMORE IN ORDER TO ENSURE THAT FUNCTIONAL ITERATION IS
C   CONVERGING ON ALL THE SOLUTION COMPONENTS IT MAY BE ADVISABLE TO
C   USE THE MAXIMUM NORM OPTION IN SPRINT , SNORM = 'MAXMUM'.
C
C   OPTIONAL INPUT : THE COMMON BLOCK /ST002A/ CONTAINS THE
C   DOUBLE PRECISION VARIABLE SFNOFC WHICH IS A RELATIVE MEASURE
C   OF THE COST OF A NEWTON STEP AGAINST THE COST OF A FUNCTIONAL
C   ITERATION STEP. YOU MAY SUPPLY THIS RATIO IF YOU WISH
C   PROVIDING THAT
C                    1.0 < SFNOFC < 10.0D0
C
C   THE DEFAULT VALUE OF SFNOFC USED IS    SFNOFC = 4.0D0
C
C
C***********************************************************************
       CHARACTER*6 ODCODE
       LOGICAL NEWT, NOSW
       DOUBLE PRECISION THETB, RFNOFC, SFNOFC
       COMMON /ODECHK/ ODCODE
       COMMON /ST001A/ THETB, RFNOFC, NEWT, NOSW
       COMMON /ST002A/ SFNOFC
       SAVE   /ODECHK/, /ST001A/
C
       IF(NY2DIM .LT. 4)THEN
          CALL SERROR(' SETUP ROUTINE FOR STHETB - PARAM NY2DIM (=I1)
     1    SHOULD BE (>=I2)', 1, 2, NY2DIM, 4, 0, 0.0D0,0.0D0)
          IRET = -1
       ELSE IF(THETA .LT. 0.51D0 .OR. THETA .GT. 1.0D0)THEN
          CALL SERROR(' SETUP ROUTINE FOR STHETB - VALUE OF THETA (=R1)
     1    DOES NOT LIE IN THE ALLOWED RANGE (0.51 , 1.0) AND HAS BEEN
     2    SET TO THE DEFAULT VALUE 0.55', 1, 0, 0, 0, 1, THETA, 0.0D0)
          THETA = 0.55D0
          IRET = 0
          ODCODE = 'STHETB'
       ELSE
          IRET = 0
          ODCODE = 'STHETB'
       END IF
       IF(IRET .EQ. -1)RETURN
       NEWT = NEWTON
       NOSW = NOSWCH
       THETB = THETA
       IF( SFNOFC .GT. 1.0D0 .AND. SFNOFC .LT. 10.0D0)THEN
           RFNOFC = SFNOFC
       ELSE
           RFNOFC = 4.0D0
       END IF
       RETURN
       END
C=======================================================================
C
C   INTEGRATOR STHETB ;THETA METHOD FOR SPRINT PACKAGE WITH SWITCHING
C                      BETWEEN NEWTON METHOD AND FUINCTIONAL ITERATION.
C
C=======================================================================
      SUBROUTINE STHETB( NEQN, Y, W, NY, WT, DY, DEL, ACOR, INLN, ISTEP,
     1                   THETA, H, T, HMIN, HMXI, IDAE)
C********************************************************************
C
C     THIS ROUTINE SOLVES THE SYSTEM OF DIFFERENTIAL/ALGEBRAIC EQUATIONS
C  OF THE FORM
C        E Y' = F(Y, T)                                            (1)
C  WHERE E IS A SINGULAR MATRIX WHEN THERE ARE ALGEBRAIC EQUATIONS
C  PRESENT IN THE SYSTEM.
C     THE INTEGRATION METHOD USED IS:
C  E*Y(N+1) = E*Y(N) + H(THETA*F(Y,T(N+1)) + (1-THETA)*F(Y,T(N)))  (2)
C     WITH LOCAL ERROR CONTROL.
C
C     THE ITERATION MATRIX G IS GIVEN BY:
C          G  =  E - H*THETA*DF/DY                                 (3)
C
C-----------------------------------------------------------------------
C DIFFERENCES FROM THE CHUA AND DEW CODE.
C----------------------------------------
C THERE ARE TWO MAIN DIFFERENCES FROM T.S. CHUA'S CODE.
C      FIRSTLY AS THE SYSTEM OF NONLINEAR EQUATIONS IS SOLVED BY THE
C ROUTINE NLSLVR THE CHUA STRATEGY OF RE-EVALUATING THE JACOBIAN MATRIX
C IN MID-ITERATION IS NONLONGER USED. IN ANY CASE AS RELAXATION IS USED
C TO CORRECTLY SCALE THE ALGABRAIC COMPONENTS IT IS NO-LONGER SO
C IMPORTANT TO AVOID REDUCING THE STEPSIZE AND RE-EVALUATING THE
C JACOBIAN MATRIX.
C    SECONDLY , A MAXIMUM OF THREE ITERATIONS
C IS ALLOWED IN THE SOLUTION OF THE NONLINEAR EQUATIONS. THIS COULD BE
C CHANGED BY INCLUDING THE SPRINT SYSTEM COMMON BLOCK /SSOLVR/  AND BY
C SETTING MAXIT TO SAY 5.
C-----------------------------------------------------------------------
C
C  THE VARIABLES USED HAVE THE FOLLOWING MEANINGS:
C *NEQN--NUMBER OF EQUATIONS TO BE SOLVED; BOTH DIFFERENTIAL AND
C        ALGEBRAIC.
C *T   --THE INDEPENDENT VARIABLE. ON FIRST CALL IT SHOULD BE SET
C        TO THE INITIAL CONDITION. ON RETURN IT CONTAINS THE VALUE
C        OF T FOR WHICH Y IS THE SOLUTION.
C *H   --PROPOSED STEPSIZE FOR THE STEP. ON FIRST CALL, IT SHOULD
C        CONTAIN THE INITIAL ESTIMATE OF THE STEPSIZE .
C *Y   --THE DEPENDENT VARIABLE. ON FIRST CALL SHOULD BE SET TO
C        THE INITIAL CONDITIONS. ON RETURN IT CONTAINS THE SOLUTION
C        AT T. DIMENSIONED AS Y(NEQN).
C *DY  --AN ARRAY RETURNS THE QUANTITY DY/DT AT THE NEW TIME LEVEL.
C        ON FIRST ENTRY, IT SHOULD CONTAIN THE DERIVATIVES DY/DT AT
C        TIME T. DIMENSIONED AS DY(NEQN).
C *DEL --RETURNS THE RESULT OF A BACK-SUBSTITUTION (SEE INLN = 5)
C        DIMENSIONED AS DEL(NEQN).
C *ACOR--WORKSPACE ON A CALL TO THE NONLINEAR EQUATIONS SOLVER
C        WITH INLN = 5 IT CONTAINS THE VECTOR WHICH IS PREMULTIPLIED BY
C        THE MATRIX E AND THEN FED TO THE BACKSUBSTITUTION ROUTINE.
C        DIMENSIONED AS ACOR(NEQN)
C  ISTEP -INDICATOR FOR THE INTEGRATOR
C         ON EXIT   >0 FOR SUCCESSFUL CALL
C                   <0 FOR STEP FAILURE.
C         ON ENTRY, IT HAS THE FOLLOWING MEANINGS:
C         =-1 INITIAL STEP.
C         = 0 REVERSE COMMUNICATION ENTRY.
C         = 1 TAKE A NEW STEP , CONTINUING FROM THE PREVIOUS STEP.
C         = 2 TAKE A CONTINUATION STEP BUT WITH NEW VALUES OF H OR NEQN
C         = 3 TAKE A NEW STEP WITH A NEW VALUE OF H
C         ON EXIT, IT HAS THE FOLLOWING MEANINGS:
C         = 1 SUCCESSFUL STEP.
C         = 0 REVERSE COMMUNICATION EXIT.
C         =-1 REQUESTED ERROR COULD NOT BE ACHIEVED.
C         =-2 CORRECTOR CONVERGENCE COULD NOT BE ACHIEVED.
C         =-3 RESIDUAL FORMING ROUTINE ORDERED A RETURN TO THE DRIVER.
C         =-4 ERROR CONDITION IN RESIDUAL ROUTINE COULD NOT BE AVOIDED.
C         =-5 ERROR IN FORMING THE JACOBIAN. SINGULAR MATRIX
C         =-6 INITIALISATION ROUTINE THESET NOT CALLED.
C         =-7 IMPOSSIBLE ERROR IN LINEAR ALGEBRA ROUTINES.
C  INLN  -POINTER TO INFORM THE CALLING PROGRAM ABOUT THE TYPE OF
C         PROBLEM DEPENDENT INFORMATION REQUIRED BY THE INTEGRATOR.
C         ON EXIT.....
C         = 0 INTEGRATION HAS FINISHED, CHECK ISTEP FOR POSSIBLE
C             ERROR.
C         = 1 TO COMPUTE THE SOLUTION OF NON-LINEAR SYSTEM OF EQUATIONS
C             GIVEN BY (2). THE INITIAL PREDICTION FOR Y(N+1) IS GIVEN
C             IN VECTOR Y; THE VALUES OF Y AND H*Y' AT TIME TN ARE
C             STORED IN THE WORKSPACE W(.,1) AND W(.,2) RESPECTIVELY.
C             IF A SOLUTION CANNOT BE FOUND BECAUSE THE ITERATION DOES
C             NOT CONVERGE, THEN SET IND(2)=-1 TO REDUCE THE STEPSIZE.
C             OTHERWISE SET IND(2)=-2 TO TERMINATE THE INTEGRATION.
C         = 5 TO COMPUTE THE SOLUTION OF G*DEL = DYY, IE A BACK-
C             SUBSTITUTION.
C         = 3 EVALUATE A NEW RIGHT-HAND FUNCTION F(Y,T) IN EQN.(1) AND
C             AND RETURN THE RESULTS IN VECTOR DEL.
C         ON ENTRY.....
C         = 0 NORMAL ENTRY - ACTION TO BE TAKEN IS GOVERNED BY ISTEP.
C         = 1 THE NONLINEAR SYSTEM OF EQUATIONS HAS BEEN SOLVED.
C         = 2 CONVERGENCE FAILURE OCCURED IN THE NONLINEAR EQUATIONS
C             SOLVER.
C         = 3, 5 THE TASKS SET BY EXIT VALUES OF ISAVE = 3  OR 5 WERE
C             SUCCESSFULLY PERFORMED.
C         = 4,6,7 NOT USED HERE AT PRESENT.
C         = -1 SINGULAR JACOBIAN MATRIX WAS FOUND .
C         = -2 ERROR IN RESIDUAL ROUTINE FORCES AN IMMEDIATE RETURN.
C         = -3 ILLEGAL VALUES ENCOUNTERED IN THE RESIDUAL ROUTINE.
C              REDUCE THE STEP SIZE BY A FACTOR OF TWO AND TRY AGAIN.
C         = -4 CONDITION ENCOUNTERED IN THE RESIDUAL ROUTINE FORCES
C              THE STEP TO BE FAILED AND A RETURN TO BE MADE TO THE
C              MONITOR ROUTINE.
C         = -5 IMPOSSIBLE ERROR OCCURRED IN THE LINEAR ALGEBRA ROUTINES.
C
C
C ISAVE  -INTERNAL INDICATOR TO INFORM THE INTEGRATOR THE EXACT POINT
C         TO RETURN TO.
C *WT  --VECTOR OF WEIGHTS FOR ERROR CRITERION.
C        DIMENSIONED AS WT(NEQN)
C *W   --ARRAY USED AS WORKSPACE. DIMENSIONED AS W(NEQN,4).
C        WHERE:
C        W(J,1) HOLDS THE VALUES OF Y AT PREVIOUS TIME LEVEL. IE. Y(N).
C        W(J,2) HOLDS THE VALUES OF H*DY AT PREVIOUS TIME LEVEL.
C        W(J,3) HOLDS BETA(N) FROM THE PREVIOUS TIME LEVEL.
C        W(J,4) CONTAINS THE VALUES OF Y(N)-Y(N-1).
C
C HMXI  : 1.0D0/HMAX  WHERE HMAX IS
C         THE MAXIMUM STEPSIZE THE INTEGRATOR IS ALLOWED TO TAKE
C         IN THE CASE WHEN THIS IS ZERO THE POSSIBLE STEPSIZE IS
C         RESTRICTED TO 1.0D0/TWOU WHERE TWOU IS THE UNIT ROUNDOFF ERR.
C HMIN  : THE MINUMUM STEPSIZE THE INTEGRATOR IS ALLOWED TO USE.
C T     : THE CURRENT VALUE OF THE INDEPENDENT VARIABLE.
C H     : THE CURRENT VALUE OF THE STEP SIZE.
C
C RESERR - LOGICAL PARAMETER THAT WHEN TRUE INDICATES THAT AN ERROR
C         HAS BEEN RETURNED FROM THE RESIDUAL ROUTINE.
C
C=======================================================================
C
C  NOTE--WHEN THE ROUTINE IS FIRST CALLED,
C        ISTEP , INLN  MUST BE SET TO -1 AND 0 RESPECTIVELY.
C
C***********************************************************************
      LOGICAL RESERR, NEWTON, NOSWCH
      CHARACTER*6 ODCODE
      DOUBLE PRECISION T,H,FAC,HALF,ONE,ZERO,THETA,ERRL,COEF,THETB,
     *                 TWOU,TENP,DIF,FAC2,COEF1,COEF2, SUM, H4
      DOUBLE PRECISION Y(NEQN),DY(NEQN),DEL(NEQN),W(NY,4),ACOR(NEQN),
     *                 R,WT(NEQN), HMIN,HMXI, VNORM, HNEWTN, DEL1, CR,
     *                 DUMMY, CRATE, HITER, ERRN, DCON, ERRF, RT,
     *                 RFNOFC
      INTEGER ISTEP, INLN, ISAVE, IOVFLO, K, NRE, NJE, NITER, JK,
     1        JKOLD, N, I, J, IDAE(1), NINTER, ISTAGE, NJSTEP, NOCHST,
     2        NEFAIL, NCFAIL, NFSTEP, NWSTEP, NST, MAXIT, ITRACE, IDEV,
     3        JSTEP, NY, IJ, JCUR, MSBP, NSTEP, NEQN
      COMMON /ODECHK/ ODCODE
      COMMON /SCONS1/ TENP, TWOU, IOVFLO
      COMMON /SDEV2/ ITRACE, IDEV
      COMMON /LSTATS/ JK,JKOLD, NST,NRE,NJE,NITER , NINTER
      COMMON /ST001A/ THETB, RFNOFC,NEWTON, NOSWCH
      COMMON /STSAVE/ R(4), COEF(3), ERRL, COEF1, COEF2, DIF, FAC, FAC2,
     1       ONE, SUM, ZERO, HALF, I,J, JCUR, MSBP, N, RESERR,
     2       NCFAIL, NEFAIL, ISAVE, JSTEP, K, NJSTEP, NWSTEP, NOCHST
      COMMON /SSOLVR/ DUMMY(2), CRATE, MAXIT
      COMMON /ST001B/ NFSTEP, NSTEP, HITER, HNEWTN, ERRN,DEL1,CR,ISTAGE
      SAVE   /ST001A/, /ODECHK/, /STSAVE/, /SCONS1/, /LSTATS/, /ST001B/
C
C**********************************************************************
C
C  THE COMMON BLOCKS USED ARE:
C /STSAVE/-CONTAINS THE STATISTICS REQUIRED AT THE END OF THE SOLUTION
C      *R(4)-VECTOR USED AS A WORKSPACE.
C         `` (1)-HOLDS THE OLD STEPSIZE.
C         `` (2)-HOLDS THE OLD VALUE OF TIME ,T, USED IN RESTARTING.
C         `` (3)-HOLDS THE MAXIMUM (INTEGRATOR DEPENDENT) STEPSIZE.
C         `` (4)-HODS THE TIME AT WHICH A RESTART PHASE IS INITIATED.
C        *COEF(3)-WORKSPACE USED FOR STORING THE COEFFICIENTS REQUIRED
C                 IN THE ERROR ESTIMATION.
C        *JSTEP -THE NUMBER OF STEPS TO BE PERFORMED BEFORE STEPSIZE
C                CHANGES IS CONSIDERED. A MIN OF 4 STEPS IS NEEDED FOR
C                A PREVIOUS STEPSIZE INCREMENT AND A MIN OF K+1 STEPS
C                IS REQUIRED FOR A PREVIOUS STEPSIZE REDUCTION.
C        *ERRL  -HOLDS THE WEIGHTED LOCAL ERROR NORM.
C        *NOCHST-NUMBER OF SUCCESS STEPS BEFORE STEPSIZE CHANGE O.K.
C /SDEV2/-TO TRACE THE INTEGRATOR FOR DEBUGGING PURPOSES:
C        *IDEVO -OUTPUT DEVICE NUMBER.
C        *ITRACE-TRACE LEVEL REQUIRED.
C                =0 FOR NO TRACE
C                =1 TO OUTPUT INTERMEDIATE RESULTS AT APPROPRIATE
C                   POINTS.
C   IMPORTANT VARIABLES;
C   -------------------
C   NEWTON, NOSWCH  --- SEE THE SETUP ROUTINE FOR DESCRIPTIONS.
C   NCFAIL          --- COUNTS NUMBER OF CONVERGENCE FAILURES PER STEP
C   NEFAIL          --- COUNTS NUMBER OF ERROR TEST FAILURES PER STEP
C   NFSTEP          --- COUNTS NUMBER OF FUNCTIONAL ITERATION STEPS
C                       SINCE FUNCTIONAL ITERATION WAS INTRODUCED.
C**********************************************************************
C NINTER SPECIFIES HOW MUCH OF THE MEMORY VECTOR IS TO BE USED IF
C ITERPOLATION IS USED E.G. IN P.D.E. REMESHING.
C
      ZERO=0.0D+0
      HALF=0.5D+0
      ONE=1.0D+0
      THETA = THETB
      JK = 1
      JKOLD = 1
      IF(DABS(HMXI) .LE. TWOU)THEN
         R(3) = 1.0D0 / TWOU
      ELSE
         R(3) = 1.0D0/HMXI
      END IF
C          -----------------------------------------------------
C          |SET UP THE STRATEGY PARAMETERS FOR THE INITIAL STEP|
C          -----------------------------------------------------
      IF (ISTEP .EQ. -1)THEN
          IF(ODCODE .NE. 'STHETB')THEN
            CALL SERROR(' STHETB THE SETUP ROUTINE WAS NOT CALLED FOR
     1      THIS ROUTINE BEFORE SPRINT WAS ENTERED', 1, 0, 0, 0, 0,
     2      0.0D0, 0.0D0)
            ISTEP = -6
            RETURN
          END IF
          K = 1
          NINTER = 4
          NOCHST = 4
          COEF(1)=HALF - THETA
          COEF(2)=THETA*THETA - THETA + ONE/6.0D+0
          COEF(3)=2.0D+0*THETA
          R(1)=H
          R(2)=T
          FAC =1.0D0
          N = NEQN
          HNEWTN = H
          MSBP = 0
          JCUR = 0
          NJSTEP = NOCHST
          NWSTEP = 0
          JSTEP = 0
          NFSTEP = 0
          NCFAIL = 0
          NEFAIL = 0
          ISTAGE = 0
          ISAVE = -1
          RESERR = .FALSE.
          ISTEP = 1
          INLN  = 0
          DO 5 I = 1,N
            W(I,2) = DY(I)*H
            W(I,1) =  Y(I)
            W(I,3) = 0.0D0
            W(I,4) = 0.0D0
 5        CONTINUE
      END IF
C
C**********************************************************************
C
C  STAGE--0 TAKE APPROPRIATE ACTION ON VALUE OF INLN
C
C***********************************************************************
C
C     IF INLN >0 JUMP TO THE APPROPRIATE POINTS IN THE INTEGRATOR
C     OTHERWISE EITHER IT IS A FRESH CALL FOR THE PRESENT TIME LEVEL
C     OR AN ERROR IN THE NON-LINEAR EQUATIONS SOLVER HAS BEEN FOUND
C
      I=INLN + 5
      INLN = 0
      GO TO ( 9, 9,  9,  9, 14, 40, 9, 36, 9, 70), I
C      INLN= -4 -3  -2  -1  0   1   2   3  4   5
 9    CONTINUE
      IF(I .EQ. 1 .OR. I .EQ. 3)THEN
         ISTEP = -3
      ELSE IF(I .EQ. 4)THEN
         ISTEP = -5
      ELSE IF(I.EQ. 7 .AND. JCUR .EQ. 2)THEN
         GOTO 10
      ELSE IF(I .EQ. 0)THEN
         ISTEP = -7
      END IF
C   -------------------------------------------------------------------
C   |RETRACT THE SOLUTION TO THAT AT THE PREVIOUS TIME STEP AND RETURN|
C   -------------------------------------------------------------------
808     J = 0
        CALL RETRAC( N, H, R(1), K, NEWTON, Y, DY, NY, W, THETA, J)
        IF(I .NE. 7 .AND. I .NE. 2)THEN
           H = R(1)
           T = R(2)
C         --------------------------------------------------------
C         | ERROR RETURN POINT EXCEPT IF REPEATED CONVERGENCE OR |
C         | ERROR TEST FAILURES.                                 |
C         --------------------------------------------------------
           RETURN
        END IF
        IF(I .EQ. 2)RESERR = .TRUE.
C          RESIDUAL ROUTINE HAS RETURNED IRES = INLN = 3
C***********************************************************************
C  STAGE --1
C  NONLINEAR EQUATIONS SOLVER FAILED TO CONVERGE -TRY REDUCING STEPSIZE
C  A MAXIMUM OF 3 TIMES
C***********************************************************************
10    NCFAIL = NCFAIL + 1
      IF(JCUR .EQ. 2 .AND. NEWTON)THEN
C        TRY RE-EVALUATING THE JACOBIAN MATRIX .
         JCUR = 0
         ISTAGE = 0
         GOTO 30
      ELSE
C        TO HANDLE  REPEATED CONVERGENCE FAILURES TRY REDUCING THE STEP
C        SIZE. ALLOW SIX REDUCTIONS ON THE FIRST STEP AND 3 THEREAFTER
80       JSTEP=K+2
         H4 = H * (2.0D0/K + (2-K)*  30.0D0) * RFNOFC
         J  = NCFAIL*K
         IF(ITRACE .GE. 1)WRITE(IDEV,8001)
8001     FORMAT(' CONVERGENCE FAILURE ')
         IF (NEWTON .OR. NOSWCH) THEN
            IF (J .GT. 8 ) THEN
C               TOO MANY STEP REDUCTIONS DUE TO CONVERGENCE FAILURE
                ISTEP = -2
             END IF
             H4  = DMAX1( CRATE, 1.0D0)
             FAC = HALF / H4
             IF( J .GT. 6)K = 1
             JCUR = 0
          ELSE IF ( J .GT. 6  .OR.  HNEWTN .GT. H4) THEN
             NEWTON = .TRUE.
             K = 1
             NWSTEP = 0
             JCUR   = 0
             NCFAIL = 0
             FAC    = HNEWTN / H * 0.5D0
             IF( (H*FAC) .GT. R(3) )FAC = R(3) / H * 0.9D0
             NFSTEP = 0
             IF(ITRACE .GE. 0)WRITE(IDEV,85)R(2)
          ELSE
             H4  = DMAX1( CRATE, 1.0D0)
             FAC = HALF/H4
          END IF
          IF( H .LT. HMIN)THEN
             IF( NOSWCH .OR. NEWTON)THEN
                 ISTEP = -2
             ELSE
                 NCFAIL = 4
                 GOTO 80
             END IF
        END IF
        T = R(2)
        IF(ISTEP.EQ. -2)THEN
           IF(RESERR)ISTEP = -4
           H = R(1)
           IF(NEWTON)THEN
            CALL SERROR(' STHETB WITH NEWTON METH-REPEATED CONVERGENCE
     1      FAILURES WERE ENCOUNTERED AT T (=R1) WITH STEP H (=R2) ',1,
     2      0, 0, 0, 2, T, H)
           ELSE
            CALL SERROR(' STHETB WITH F/ITER METH-REPEATED CONVERGENCE
     1      FAILURES WERE ENCOUNTERED AT T (=R1) WITH STEP H (=R2) ',1,
     2      0, 0, 0, 2, T, H)
           END IF
           RETURN
        END IF
        ISTAGE = 0
        GOTO 20
85      FORMAT(' SWITCH TO NEWTON AS CONVERGENCE FAILURE AT T=',D11.3)
      END IF
14    RESERR = .FALSE.
C    ----------------------------
C    |CHECK IF NEQN HAS CHANGED |
C    ----------------------------
      IF(ISTEP .EQ. 1)GOTO 20
      IF(R(1) .NE. H)THEN
C        H HAS BEEN CHANGED BY THE USER.
         IF(DABS(H) .GT. R(3)) H = R(3)* DSIGN(1.0D0,H)
         IF(DABS(H) .LT. HMIN) H = HMIN* DSIGN(1.0D0,H)
         FAC = H/R(1)
         H   = R(1)
      END IF
      IF(ISTEP .NE. 2) GOTO 20
C  BELOW TWO LINES NOT IN STHETB SAVE
C     FAC   = DMIN1(1.0D0,FAC)
C     JSTEP = 3
      IF(NEWTON)THEN
CZ       FORCE A JACOBIAN EVALUATION BUT NO ATTEMPT AT A A SWITCH
C        JCUR = 0
         ISTAGE = 0
      END IF
      IF(N .NE. NEQN)THEN
C        NEQN HAS BEEN CHANGED BY MONITR OR BY THE USER. FLAG AN
C        INCREASE AND IF A DECREASE ZERO UNUSED PARTS OF WORKSPACES
         IJ = N - NEQN
         IF(IJ .LT. 0)THEN
           CALL SERROR('STHETB - THE VALUE OF NEQN HAS BEEN INCREASED
     1     FROM(=I1) TO (=I2) ', 1, 2, N, NEQN, 0, 0.0D0, 0.0D0)
         END IF
         DO 141 I = NEQN+1,N
           DO 141 J = 1,4
 141          W(I,J) = 0.0D0
         N = NEQN
       END IF
C***********************************************************************
C
C  STAGE--2  PREDICT THE SOLUTION PRESERVING OLD T VALUE IN R(2).
C            IF NECESS TRY TO USE FUNC ITERATION.
C***********************************************************************
 20   R(2)= T
      H   = H * FAC
      T   = T + H
      FAC = H / R(1)
      ISTEP = 0
      IF(NEWTON)HNEWTN = H
       IF(ISTAGE .EQ. 0 .OR. NOSWCH)GOTO 39
       IF(NEWTON .AND. JCUR .EQ. 0)THEN
C         SEE IF FUNCTIONAL ITERATION CAN BE USED FOR THIS STEP. PREDICT
C         THE FUNCTIONAL ITERATION STARTING VALUES AND TRY  TWO ITERS.
          J = 1
          CALL PREDIC( N, H, R(1), K, .FALSE., Y, DY, NY,W, THETA, J)
          DO 34 J = 1,N
 34          ACOR(J) = 0.0D0
          ISTAGE = 1
          INLN   = 3
          RETURN
       ELSE
          ISTAGE = 0
          GO TO 39
       END IF
 36    RT = 0.9D0 / (H*THETA)
       DO 37 I = 1,N
          ACOR(I) = ACOR(I)+DEL(I)*((1.D0-IDAE(I)) * RT + IDAE(I))
          Y(I) = Y(I) + THETA * H * DEL(I)
 37       DY(I) = DY(I) + DEL(I)
       IF(ISTAGE .EQ. 3)GOTO 38
       IF(ISTAGE .EQ. 1)THEN
          DEL1 = VNORM( N, WT, DEL)
       ELSE
          CR   = VNORM( N, WT, DEL)/DEL1
          IF(ITRACE .GE. 1)WRITE(IDEV,3701)CR
3701      FORMAT(' F/ITER TRY ; STAGE ONE CRATE=',D12.4)
          IF(CR .GT. 0.90D0) GOTO 39
       END IF
       ISTAGE = ISTAGE + 1
       INLN = 3
       RETURN
C            FOR ANOTHER RESIDUAL EVAL TO EST CONV RATE
 38    DEL1 = VNORM (N, WT, DEL) /( DEL1* CR)
       CR   = DMAX1(CR,DEL1)
       ISTAGE = 0
CZ     HITER = H * 0.3D0 /CR
CZ                 0.7 IN THETB
       HITER = H * 0.7D0 /CR
       DCON = DMIN1(1.0D0,1.5D0*CR) * THETA*H * 2.5D0* CR * DEL1
       IF(ITRACE .GE. 1)WRITE(IDEV,3801)DCON,CR
3801   FORMAT(' F/ITER TRY ; DCON= ',D12.4,' CRATE=',D12.4)
C        SEE IF FUNC ITER CAN BE USED
CZ                     1.0                  0.7 OIN THETB
CZ       IF (DCON .LT. 1.0D0 .AND.  CR .LT. 0.7D0)THEN
         IF (DCON .LT. 0.5D0 .AND.  CR .LT. 0.7D0)THEN
            NEWTON = .FALSE.
            NWSTEP = 0
            IF(ITRACE .GE. 1)WRITE(IDEV,95)T
95          FORMAT(' SWITCH TO FUNCTIONAL ITERATION AT  T=',D11.3)
            HNEWTN = H
            NFSTEP = 0
            GOTO 46
          END IF
C    END OF SWITCHING SECTION.
C
C***********************************************************************
C
C  STAGE--3  NON-LINEAR EQUATIONS SOLVER
C  SET Y AND YDOT TO PREDICTED VALUES AND DECIDE WHETHER TO UPDATE JAC.
C
C***********************************************************************
39     J = 0
       CALL PREDIC( N, H, R(1), K, NEWTON, Y, DY, NY, W, THETA, J)
30     DO 35 I = 1, N
          Y(I) = W(I,1)
35        DY(I) = W(I,2)/H
       IF (NEWTON) THEN
          IF(JCUR .EQ. 0) THEN
             JCUR = 1
             INLN = 1
             MSBP = 0
          ELSE
             JCUR = 2
             INLN = 2
          END IF
          HNEWTN = H
       ELSE
          JCUR = 1
          INLN = 6
       ENDIF
C
C  EXIT TO SOLVE THE NON-LINEAR EQUATIONS; RETURN TO LABEL 40 IF SUCCESS
C
      RETURN
40    CONTINUE
      IF(.NOT. NEWTON)THEN
C         ESTIMATE THE FUNCTIONAL ITERATION STEPSIZE
          CR = DMAX1(0.1D-4,CRATE)
          HITER = H * 0.5D0 / CR
      END IF
C     IF (ITRACE.GE.1) WRITE(IDEV,45) HITER,H,HNEWTN,CR
C45   FORMAT(' HITER=',D11.3,' H=',D11.3,' HNEW=',D11.3,' CRATE=',D11.3)
C***********************************************************************
C
C  STAGE--4 BUILD UP THE ERROR ESTIMATES
C
C***********************************************************************
C     A SOLUTION OF THE NON-LINEAR EQUATIONS HAS BEEN FOUND CHECK THE
C     NEW DERIVS Y' IN VECTOR DY ANDEXIT TO COMPUTE BETA(N+1) TO BE
C     USED IN THE LOCAL ERROR ESTIMATION. RETRACT MEMORY VECTOR FOR
C     ESTIMATING BETA(N+1) AND W(J,4)
      J = 1
      CALL RETRAC( N, H, R(1), K, NEWTON, Y, DY, NY, W, THETA, J)
      FAC = H/R(1)
46    DO 1025 J=1,N
        DEL (J)= DY(J)*H - W(J,2)*FAC
1025    ACOR(J)=-DY(J)*H + W(J,2)*FAC
      IF(NEWTON)THEN
         DO 1026 I = 1,N
1026        ACOR(I) = ACOR(I) / (H * THETA)
         INLN = 5
         ISAVE = 2
         RETURN
      ELSE
         DO 69 J = 1,N
            DEL(J) = DEL(J) * IDAE(J)
69       CONTINUE
      END IF
70    CONTINUE
C   --------------------------------------------------------------------
C   | EVALUATE THE LOCAL ERROR CORRECTIONS - PROTHERO'S ERROR ESTIMATE |
C   --------------------------------------------------------------------
      IF (K . EQ. 1) THEN
C         2ND ORDER TERMS ONLY
          DO 1030 J=1,N
             ACOR(J)=COEF(1)*DEL(J)
1030      CONTINUE
      ELSE
C         2ND ORDER AND 3RD ORDER TERMS
          FAC=H/R(1)
          FAC2=FAC*FAC
          DIF=FAC*COEF(2)/(ONE + COEF(3)*(FAC-ONE))
          COEF1=COEF(1) + DIF
          COEF2=FAC2*DIF
          DO 1040 J=1,N
             ACOR(J)=COEF1*DEL(J) - COEF2*W(J,3)
1040       CONTINUE
      END IF
      ERRL   = VNORM( N, WT, ACOR)
      IF (NEWTON) THEN
         ERRN = ERRL
      ELSE
         ERRF   = ERRL
         ERRN = ERRL *(HNEWTN / H)**2
         IF(ERRN .GT. ONE)THEN
           IF(NFSTEP.GT.0 .AND. MSBP.GT.0)HNEWTN = HNEWTN * HALF
         ELSE
           NJSTEP = NJSTEP - 1
         END IF
         IF (ERRN .LT. 0.25 .AND. NJSTEP .LE. 0)THEN
           HNEWTN = HNEWTN * 2.0D0
           DO 1060 I = 1,4
             IF(ERRN .LE. 0.5*10.0**(-I))ERRN = ERRN* 2.0D0
 1060      CONTINUE
           NJSTEP = NOCHST
         END IF
      END IF
      IF(ITRACE .GE. 1)WRITE(IDEV,8005)ERRL
8005            FORMAT(' SCALED LOCAL ERROR ESTIMATE = ',D15.6)
C**********************************************************************
C  STAGE--5   LOCAL ERROR TEST
C  SEE IF THE WEIGHTED NORM OF THE LOCAL ERROR IS LESS THAN 1
C  ACCEPT THE STEP IF IT IS .
C**********************************************************************
C
      IF (ERRL .GT.ONE ) THEN
          NEFAIL = NEFAIL + 1
C         THE SOLUTION DOES NOT SATISFY THE LOCAL ERROR TOLERANCE
C         RETRACT THE SOLUTION TO THAT AT THE PREVIOUS TIME STEP
C         REDUCE H BY FACTOR AND RETURN TO THE PREDICTION STAGE
          FAC = HALF
          IF(ERRL .GT. 2.0)FAC = 1.0D0/DMIN1(100.0D0,ERRL)
          JSTEP=K+2
          T = R(2)
          IF (NEFAIL .GT. 3 .OR. DABS(H) .LT. HMIN) THEN
C            TOO MANY STEP REDUCTIONS DUE TO ERROR TEST FAILURES.
             ISTEP = -1
             H = R(1)
             RETURN
           END IF
           JCUR = 0
           ISTAGE = 0
           GOTO 20
       ELSE
C           -----------------
C           |SUCCESSFUL STEP|
C           -----------------
           MSBP = MSBP + 1
           ISTAGE = 0
           IF(MSBP .EQ. 21)THEN
              JCUR = 0
              IF(NEWTON .AND. .NOT. NOSWCH)ISTAGE = 1
C                                  TO SEE IF THE NEXT JACOBIAN EVAL CAN
C                                  REPLACED BY FUNCTIONAL ITERATION.
           END IF
           NCFAIL = 0
           ISTEP = 1
           IF(THETB .LT. 0.99D0)K = 2
C          -------------------------------------------------------------
C          |UPDATE THE MEMORISED VALUES AND PUT THE LOCAL ERROR IN ACOR|
C          -------------------------------------------------------------
           IF(NEWTON)THEN
              NWSTEP = NWSTEP + 1
           ELSE
              NFSTEP = NFSTEP + 1
           END IF
           DO 1070 J=1,N
              W(J,4)=Y(J) - W(J,1)
              W(J,1)=Y(J)
              W(J,2)=DY(J)* H
              W(J,3)  = DEL(J)
              ACOR(J) = DEL(J)
1070      CONTINUE
          JSTEP=JSTEP-1
          FAC=ONE
          R(1)=H
          H4  =H * 2.0D0 * RFNOFC
          IF (DABS(H).LT.R(3).AND.ERRL.LT.0.25D+0.AND.JSTEP.LE.0) THEN
C             -------------------------------------------
C             |DOUBLE THE STEPSIZE TO BE USED NEXT STEP|
C             -------------------------------------------
              FAC = 2.0D0
C
C             IF LOCAL ERR VERY SMALL MORE THAN DOUBLE STEPSIZE
C
              DO 1077 I = 1,4
               IF(ERRL .LE. 0.5*10.0**(-I))FAC = FAC* 2.0D0
 1077         CONTINUE
              IF( DABS(H) * FAC .GT. R(3) )FAC = R(3)/ DABS(H)
              IF(ITRACE .GE. 1)WRITE(IDEV,1043)H,FAC,R(3)
 1043         FORMAT(' H =',D12.4,' FAC =',D12.4,' HMAX= ',D12.4)
              JSTEP = NOCHST
              IF (NEWTON) THEN
                  JCUR = 0
                  IF(.NOT. NOSWCH .AND. NWSTEP .GE. 10)ISTAGE = 1
              ELSE
                  IF(FAC .GE. (HITER/H))FAC = ONE
                  IF (HNEWTN .GE. H4 .AND. .NOT. NOSWCH ) THEN
                      NEWTON = .TRUE.
                      NWSTEP = 0
                      FAC    = 0.9D0 * HNEWTN /H
                      IF(ITRACE .GE. 1)WRITE(IDEV,87)T,HNEWTN,H
 87                   FORMAT(' SWITCH TO NEWTON AT T = ',D11.3/'  AS ',
     1                'H NEWTON =',D11.3,' WHILE H FUNC ITER =',D11.3)
                      NFSTEP = 0
                      JCUR = 0
                      K = 1
                  END IF
              END IF
           END IF
           J = NFSTEP * MIN0 ( 1 ,NEFAIL)
           IF((NFSTEP.GT.12 .OR. J .EQ. 1).AND. .NOT. NEWTON)THEN
C             EITHER 12 STEPS OF FUNCTIONAL ITERATION HAVE BEEN TAKEN OR
C             FUNCTIONAL ITERATION IS HAVING TROUBLE WITH THE ERROR TEST
C             AT THE END OF THE FIRST STEP.
              IF(HNEWTN.GE.(H4*0.5D0) .AND. .NOT.NOSWCH) THEN
                 NEWTON = .TRUE.
                 NWSTEP = 0
                 FAC    =  HNEWTN /H * 0.9D0
                 IF(ITRACE .GE. 1)WRITE(IDEV,87)T,HNEWTN,H
                 NFSTEP = 0
                 JCUR = 0
                 K = 1
               END IF
           END IF
           NEFAIL = 0
      END IF
      RETURN
C-----------------------------------------------------------------------
C     END OF SUBROUTINE RTHETA
C-----------------------------------------------------------------------
      END
      SUBROUTINE RETRAC( N, H, HOLD, K, NEWTON, Y, DY, NY,W, THETA, IND)
      INTEGER N, K, IND, J, NY
      DOUBLE PRECISION H, HOLD, Y(N), DY(N), W(NY,4), THETA
      DOUBLE PRECISION FAC, DIF, FAC2
      LOGICAL NEWTON
C   -------------------------------------------------------------------
C   | ROUTINE TO RETRACT THE SOLUTION AND ITS TIME DERIV TO THEIR     |
C   | VALUES  AT THE PREVIOUS TIME STEP.                              |
C   -------------------------------------------------------------------
        FAC = H/HOLD
        IF(K .EQ.1 )THEN
           DO 11 J = 1,N
              W(J,1) = W(J,1) - W(J,2)
              W(J,2) = W(J,2)/FAC
 11        CONTINUE
        ELSE IF(NEWTON)THEN
           DIF = 1.0D0 + THETA * (FAC - 1.0D0)
           FAC2 = -(1.0D0 -THETA)*FAC
           DO 12 J = 1,N
              W(J,1) = W(J,1) -(DIF* W(J,3) + W(J,4))*FAC
              W(J,2) = (W(J,2)*THETA - (DIF*W(J,3)+ W(J,4))*FAC)/FAC2
 12        CONTINUE
        ELSE
           DO 13 J = 1,N
              W(J,2) = ( W(J,2) + FAC**2/THETA * W(J,4)) /
     1                 ( FAC +  FAC**2 /THETA )
              W(J,1) = W(J,1) - FAC * W(J,2) - FAC**2* (W(J,2) - W(J,4))
 13        CONTINUE
        END IF
        IF(IND .EQ. 1)RETURN
        DO 14 J = 1, N
           Y(J) = W(J,1)
 14       DY(J) = W(J,2) / HOLD
        RETURN
        END
      SUBROUTINE PREDIC( N, H, HOLD, K, NEWTON, Y, DY, NY,W, THETA, IND)
      INTEGER N, K, IND, NY, J
      DOUBLE PRECISION H, HOLD, Y(N), DY(N), W(NY,4), THETA
      DOUBLE PRECISION FAC, DIF
      LOGICAL NEWTON
C***********************************************************************
C  ROUTINE TO PREDICT THE NEW SOLUTION AND ITS TIME DERIVATIVE.
C***********************************************************************
      FAC=H/HOLD
      IF (K .EQ. 1 )THEN
C         INITIAL STEP OR RESTART FROM ORDER 1
          DO 1000 J=1,N
             DY(J) =W(J,2)/HOLD
             Y(J)  =W(J,1)+DY(J)* H
1000       CONTINUE
      ELSE IF(NEWTON) THEN
C          NON-INITIAL STEP, PREDICT THE SOLUTION TO 2ND ORDER ACCURACY
C          USING INFORMATION FROM NEWTON ITERATION.
           DIF = 1.0D0 + THETA * (FAC - 1.0D0)
           DO 1010 J=1,N
              Y(J) = W(J,1) + (DIF * W(J,3) + W(J,4)) * FAC
             DY(J) = (W(J,2)+(DIF*W(J,3)+W(J,4)-W(J,2))/THETA)/HOLD
1010       CONTINUE
      ELSE
C          NON INITIAL STEP USE INFERIOR PREDICTOR BASED ON TRAPEZOIDAL
C          RULE AS FUNCTIONAL ITERATION IS BEING USED.
           DO 1020 J = 1,N
              Y(J) = W(J,1) + FAC*W(J,2) + FAC**2*(W(J,2) - W(J,4))
             DY(J) = W(J,2)/HOLD + FAC/(THETA*HOLD) * (W(J,2)-W(J,4))
 1020      CONTINUE
      END IF
      IF(IND .EQ. 1)RETURN
      DO 1030 J = 1,N
         W(J,1) = Y(J)
         W(J,2) = DY(J)*H
 1030 CONTINUE
      RETURN
      END
