C    
C SPRINT VERSION OF DASSL.
C
      SUBROUTINE DASSET( NY2DIM, IRET, MAXORD)
      INTEGER MAXORD,  IRET, NY2DIM
C----------------------------------------------------------------------
C   THIS ROUTINE INITIALISES THE DASSL IMPLENTATION OF B.D.F
C   METHODS AS USED IN THE SPRINT INTEGRATION PACKAGE
C   THE INPUT PARAMETERS ARE:
C   NY2DIM     THE SECOND DIMENSION OF THE YSAVE ARRAY , IT SHOULD BE
C              SET TO MAXORD + 3, UNLESS MAXORD IS ZERO ,SEE BELOW.
C   MAXORD     THE MAXIMUM ORDER TO BE USED. THIS SHOULD BE POSITIVE AND
C              THIS SHOULD BE LESS THAN 6 . WHEN MAXORD IS SET
C              TO ZERO THE DEFAULT VALUES OF  5
C              WILL BE USED BY THE CODE (NY2DIM MUST BE LARGE ENOUGH
C              FOR THESE DEFAULT VALUES).
C
C
C          N.B. THE SIZE OF THE YSAVE ARRAY PASSED INTO SPRINT MUST BE
C              YSAVE(NYH, MAXORD + 3). WHERE NYH IS THE NUMBER OF
C              DIFFERENTIAL ALGEBRAIC EQUATIONS OR AN UPPER BOUND ON
C              THIS NUMBER .
C
C  OUTPUT PARAMETER
C   IRET       INTEGER FLAG SET TO ZERO IF ALL INPUTS ARE O.K.
C              OTHERWISE SET TO -1  TO INDICATE AN ERROR EXIT.
C----------------------------------------------------------------------
      INTEGER IOWNS, NMXORD, MORD, I, IERR, IOWNS2
      DOUBLE PRECISION ROWNS, CCMAX
      CHARACTER*6 ODCODE
      LOGICAL PETZ
      COMMON /ODECHK/ ODCODE
      COMMON /DDA003/ ROWNS(3), IOWNS(5), NMXORD, IOWNS2
      COMMON /DDA004/ IERR, PETZ
      COMMON /SPCCMX/ CCMAX
      SAVE   /DDA003/, /ODECHK/, /SPCCMX/
      DATA MORD /5/
C
      IRET = 0
      IERR = 0
       IF (MAXORD.LT.0)THEN
          CALL SERROR(' DASSET - NEG VALUE OF MAXORD (=I1) IS ILLEGAL'
     1    , 1, 1, MAXORD, 0, 0, 0.0D0, 0.0D0)
          IRET = -1
       END IF
       PETZ = .TRUE.
       CCMAX  = 0.3D0
       IF ( MAXORD.EQ.0)THEN
            MAXORD = MORD
            NMXORD = MORD
       ELSE
            IF(MAXORD .GT. MORD)THEN
            CALL SERROR(' DASSET WARNING USER SUPPLIED MAXORD (=I1) IS
     1 GREATER THAN ALLOWED (=I2) AND IS RESET TO THE ALLOWED MAX',
     2 1, 2, MAXORD, MORD, 0, 0.0D0, 0.0D0)
            END IF
            NMXORD = MIN0(MAXORD,MORD)
            MAXORD = NMXORD
       END IF
       IF(NY2DIM .LT. (MAXORD+3)) THEN
          I = MAXORD + 3
          CALL SERROR(' DASSET - ERROR USER DEFINED VALUE OF NY2DIM
     1                  (=I1) IS SMALLER THAN THE REQUIRED VALUE (=I2)
     2                  INCREASE THE SECOND DIMENSION TO THIS VALUE ', 1
     3                 ,2 , NY2DIM, I, 0, 0.0D0, 0.0D0)
          IRET = -1
       ELSE
          ODCODE ='SPDASL'
       END IF
       RETURN
C------END-OF-ROUTINE-DASSET-------------------------------------------
       END
      SUBROUTINE SPDASL (NEQN, Y,PHI, NYH, WT, YPRIME, DELTA, E, INLN,
     1                  IDID, EL0, H, X, HMIN, HMXI, IDAE)
      INTEGER NEQ, NYH, INLN, IDID, IDAE(1)
      DOUBLE PRECISION Y, PHI, WT, YPRIME, DELTA, E, EL0, H, X, HMIN,
     1        HMXI
      DIMENSION NEQN(1),Y(1),PHI(NYH,1),WT(1),YPRIME(1),DELTA(1),E(1)
C-----------------------------------------------------------------------
C SPDASL SOLVES A SYSTEM OF DIFFERENTIAL/ALGEBRAIC EQUATIONS OF THE FORM
C G(X,Y,YPRIME) = 0,  FOR ONE STEP (NORMALLY FROM X TO X+H).
C THE METHODS USED ARE MODIFIED DIVIDED DIFFERENCE,FIXED LEADING
C COEFFICIENT FORMS OF BACKWARD DIFFERENTIATION FORMULAS. THE CODE
C ADJUSTS THE STEPSIZE AND ORDER TO CONTROL THE LOCAL ERROR PER STEP.
C
C PARAMETERS
C **********
C NEQN   = INTEGER ARRAY CONTAINING PROBLEM SIZE IN NEQN(1), AND
C          NOT USED EXCEPT TO INITIALISE THE COMMON BLOCK VARIABLE NEQ
C Y      = AN ARRAY OF LENGTH .GE. N USED AS THE Y ARGUMENT IN
C          ALL CALLS TO OTHER SUBROUTINES. ON THE FIRST CALL THIS
C          ARRAY IS ASSUMED TO CONTAIN THE INITIAL SOLUTION VALUES.
C PHI    = AN NYH BY LMAX ARRAY CONTAINING THE DEPENDENT VARIABLES
C          AND THEIR MODIFIED DIVIDED DIFFERENCES, WHERE
C          LMAX = MAXORD + 1.
C              ON ENTRY FOR THE FIRST STEP, THE FIRST
C          TWO COLUMNS OF YH MUST BE SET FROM THE INITIAL VALUES.
C NYH    = A CONSTANT INTEGER .GE. N, THE FIRST DIMENSION OF YH.
C WT     = AN ARRAY OF LENGTH N CONTAINING MULTIPLICATIVE WEIGHTS
C          FOR LOCAL ERROR MEASUREMENTS.  LOCAL ERRORS IN Y(I) ARE
C          COMPARED TO 1.0/EWT(I) IN VARIOUS ERROR TESTS.
C YPRIME = AN ARRAY OF WORKING STORAGE, OF LENGTH N USED TO HOLD THE
C          APPROXIMATE VALUES OF THE TIME DERIVATIVE.
C DELTA  = AN ARRAY OF WORKING STORAGE, OF LENGTH N.
C          THIS ARRAY IS NOT USED IN THE PRESENT IMPLEMENTATION.
C E      = A WORK ARRAY OF LENGTH N USED FOR THE ACCUMULATED
C          CORRECTIONS. ON A SUCCESFUL RETURN, E(I) CONTAINS
C          THE ESTIMATED ONE-STEP LOCAL ERROR IN Y(I).
C H      = THE STEP SIZE TO BE ATTEMPTED ON THE NEXT STEP.
C          H IS ALTERED BY THE ERROR CONTROL ALGORITHM DURING THE
C          PROBLEM.  H CAN BE EITHER POSITIVE OR NEGATIVE, BUT ITS
C          SIGN MUST REMAIN CONSTANT THROUGHOUT THE PROBLEM.
C HMIN   = THE MINIMUM ABSOLUTE VALUE OF THE STEP SIZE H TO BE USED.
C HMXI   = INVERSE OF THE MAXIMUM ABSOLUTE VALUE OF H TO BE USED.
C          HMXI = 0.0 IS ALLOWED AND CORRESPONDS TO AN INFINITE HMAX.
C          HMIN AND HMXI MAY BE CHANGED AT ANY TIME, BUT WILL NOT
C          TAKE EFFECT UNTIL THE NEXT CHANGE OF H IS CONSIDERED.
C IDAE     INDICATOR ARRAY OF DIMENSION NEQ(1) NOT USED IN THIS
C          MODULE.
C X      = THE INDEPENDENT VARIABLE. TN IS UPDATED ON EACH STEP TAKEN.
C IDID     : INPUT AND OUTPUT ERROR INDICATOR
C    INPUT:    -1  PERFORM THE FIRST STEP  .
C               0  REVERSE COMMUNICATION RETURN FROM NLSLVR ROUTINE.
C               1  TAKE A NEW STEP CONTINUING FROM THE LAST.
C               2  TAKE THE NEXT STEP WITH A NEW VALUE OF H, MAXORD,
C                    N, METH.
C               3  TAKE THE NEXT STEP WITH A NEW VALUE OF H,
C                    BUT WITH OTHER INPUTS UNCHANGED .
C          ON RETURN, IDID   IS SET TO 1 TO FACILITATE CONTINUATION.
C    OUTPUT:    1  THE STEP WAS SUCCESSFUL.
C               0  REVERSE COMMUNICATION RETURN - CHECK INLN.
C              -1  THE REQUESTED ERROR COULD NOT BE ACHIEVED.
C              -2  CORRECTOR CONVERGENCE COULD NOT BE ACHIEVED.
C              -3  RES ORDERED IMMEDIATE RETURN.
C              -4  ERROR CONDITION FROM RES COULD NOT BE AVOIDED.
C              -5  FATAL ERROR IN JACOBIAN FORMING OR BACKSUBSTITUTION.
C              -6  INIT MODULE WAS NOT CALLED PRIOR TO FIRST ENTRY.
C              -7  WORKSPACE ERROR IN LINEAR ALGEBRA SECTION .
C          A RETURN WITH ISTEP = -1, -2, OR -4 MEANS EITHER
C          ABS(H) = HMIN OR 10 CONSECUTIVE FAILURES OCCURRED.
C          ON A RETURN WITH IDID NEGATIVE, THE VALUES OF X AND
C          THE PHI ARRAY ARE AS OF THE BEGINNING OF THE LAST
C          STEP, AND H IS THE LAST STEP SIZE ATTEMPTED.
C
C INLN       = REVERSE COMMUNICATION INDICATOR
C
C       ON ENTRY   INLN  = 0  : NORMAL ENTRY
C                        = -1 : ERROR IN JACOBIAN FORMATION
C                        = -2 : RESID ORDERED RETURN TO CALLING PROG.
C                        = -3 : RESID DETECTED ILLEGAL T, Y OR YDOTI.
C                        = -4 : RESID ORDERED ENTRY TO MONITR.
C                        = -5 ; WORKSPACE ERROR IN NONLINEAR SOLVER.
C                        = 1  : NONLINEAR SYSTEM SOLVED
C                        = 2  : ITERATION FAILED TO CONVERGE IN
C                               SOLUTION OF NONLINEAR SYSTEM
C                        = 3,4,5,6  NOT USED IN THIS MODULE
C
C       ON EXIT    INLN  = 0  : NORMAL EXIT
C                        = 1  : FORM JACOBIAN AND SOLVE NONLINEAR SYSTEM
C                        = 2  : AS FOR INLN = 1 BUT USING OLD JACOBIAN
C                        = 3,4,5,6,7 NOT USED  HERE.
C
C
C   KNOWN BUGS
C   **********
C     DOES NOT CATER FOR FLYING RESTART POSSIBLY WITH CHANGED EQN NUM.
C-----------------------------------------------------------------------
C
      DOUBLE PRECISION ALPHA, ALPHAS, ALPHA0, BETA, CCMX1, CJ,
     1        CJOLD, CK, DUN, DUMMY, ENORM, ERK, ERKM1, ERKM2,
     2        ERKP1, ERR, EST, GAMMA, HNEW, HOLD, PSI, R, SERKM1,
     3        SIGMA, TEMP1, TEMP2, TERK, TERKM1, TERKM2, TERKP1, RMAX,
     4        U, XOLD, XRATE, VNORM, S, SERK, SERKP1,SETEMP, ERR1,
     5        RATE, PNORM, OLDNRM, SERKM2
C
      INTEGER I, IDEV, IERR, IK, IPHASE, ITRACE, IZ, J, JCALC, J1, K,
     1        KDIFF, KM1, KNEW, KOLD, KP1, KP2, LMXORD, MAXIT, IOVFLO,
     2        NS
      LOGICAL CONVGD, PETZ
      CHARACTER*6 ODCODE
      COMMON /SDEV2/ ITRACE, IDEV
      COMMON /SDDTR/ TERKP1, TERK, TERKM1, TERKM2
      COMMON /DDA002/ PSI(7),ALPHA(7),BETA(7),GAMMA(7),SIGMA(7)
      COMMON /DDA003/ CJOLD,HOLD,S,IPHASE,JCALC,K,KOLD,NS,LMXORD, NEQ
      COMMON /DDA004/ IERR, PETZ
      COMMON /LSTATS/ NQ, NQU, NST, NRE, NJE, NITER, NINTER
      COMMON /SPCCMX/ CCMX1
      COMMON /ODECHK/ ODCODE
      COMMON /SSOLVR/ DUMMY(3), MAXIT
      COMMON /SCONS1/ DUN, U, IOVFLO
      COMMON /ERRCNT/ IEFAIL
      SAVE
C
C-----------------------------------------------------------------------
C     block 1.
C     initialize. on the first call,set
C     the order to 1 and initialize
C     other variables.
C-----------------------------------------------------------------------
C
C     JUMP IF IT IS REVERSE COMMUNICATION
C
      IZ = INLN  + 5
      INLN  = 0
C     INLN  = -4   -3   -2   -1   0   1    2   3  4   5
      GOTO ( 370, 370, 370, 370, 10, 343, 370, 8, 9, 10),IZ
      IF(IZ .EQ. 13 .AND. JJ .EQ. 1) GOTO 422
      IF(IZ .EQ. 13 .AND. JJ .EQ. 2) GOTO 512
      IF(IZ .EQ. 13 .AND. JJ .EQ. 3) GOTO 406
      IF(IZ .EQ. 13 .AND. JJ .EQ. 4) GOTO 408
      GOTO   370
 8    CONTINUE
C     INLN = 3
      IF(JJ .EQ. 1)GOTO 343
      IF(JJ .EQ. 2)GOTO 350
 9    CONTINUE
C     INLN = 4
      IF(JJ .EQ. 1)GOTO 356
      IF(JJ .EQ. 2)GOTO 424
10    CONTINUE
      IF(ODCODE .NE. 'SPDASL')THEN
         CALL SERROR(' SPDASL-SETUP MODULE DASSET WAS NOT CALLED BEFORE
     1   ENTRY TO SPRINT- ERROR', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         IDID  = -6
         RETURN
      END IF
C
C     initializations for all calls
      I   = IDID
      IDID=1
      XOLD=X
      NCF=0
      NSF=0
      NEF=0
      IF(I .EQ. 1) THEN
         NEQ  = NEQN(1)
         GO TO 120
      END IF
      IF( I.EQ. 2 .OR. I .EQ. 3)THEN
        IF(NEQN(1) .LT. NEQ)THEN
C         SIZE OF D.A.E. SYSTEM HAS BEEN REDUCED.
          J = NEQN(1) + 1
          DO 30 I = J,NEQ
             DO 30 IK = 1,KP1
 30             PHI(I,IK) = 0.0D0
        END IF
        K = MIN( K, LMXORD)
        H = MAX( H, HMIN)
        IF(H * HMXI .GT. 1.D0)H = 1.D0 / HMXI
        NEQ = NEQN(1)
        NINTER = K + 1
        IF(NEQ .GT. NYH)THEN
           WRITE(IDEV,31)
 31       FORMAT(' NEW VALUE OF NEQ EXCEEDS DECLARED SPACE IN YH ARRAY')
           IDID = -6
           RETURN
        END IF
        JCALC = -1
        GOTO 200
      END IF
C
C     IF THIS IS THE FIRST STEP,PERFORM OTHER INITIALIZATIONS
C
      NEQ   = NEQN(1)
      K     = 1
      KOLD  = 0
      HOLD  = 0.0D0
      PSI(1)= H
      CJOLD = 1.0D0/H
      CJ    = CJOLD
      RMAX  = 1.0D+2
      JCALC = -1
      MAXIT = 4
      XRATE = 1.D0 / 4.D0
      NQU   = 1
      IPHASE = 0
      NS=0
      DO 100 I = 1,NEQ
         PHI(I,1) = Y(I)
         PHI(I,2) = YPRIME(I) * H
100   CONTINUE
C
120   CONTINUE
C
C-----------------------------------------------------------------------
C     block 2
C     compute coefficients of formulas for
C     this step.
C-----------------------------------------------------------------------
200   CONTINUE
      KP1=K+1
      KP2=K+2
      KM1=K-1
      XOLD=X
      IF(H.NE.HOLD.OR.K .NE. KOLD) NS = 0
      NS=MIN(NS+1,KOLD+2)
      NSP1=NS+1
      IF(KP1 .LT. NS)GO TO 230
C
      BETA(1)=1.0D0
      ALPHA(1)=1.0D0
      TEMP1=H
      GAMMA(1)=0.0D0
      SIGMA(1)=1.0D0
      DO 210 I=2,KP1
         TEMP2=PSI(I-1)
         PSI(I-1)=TEMP1
         BETA(I)=BETA(I-1)*PSI(I-1)/TEMP2
         TEMP1=TEMP2+H
         ALPHA(I)=H/TEMP1
         SIGMA(I)=(I-1.D0)*SIGMA(I-1)*ALPHA(I)
         GAMMA(I)=GAMMA(I-1)+ALPHA(I-1)/H
210      CONTINUE
      PSI(KP1)=TEMP1
230   CONTINUE
C
C     compute alphas, alpha0
      ALPHAS = 0.0D0
      ALPHA0 = 0.0D0
      DO 240 I = 1,K
        ALPHAS = ALPHAS - 1.0D0/(I)
        ALPHA0 = ALPHA0 - ALPHA(I)
240     CONTINUE
C
C     compute leading coefficient cj
      CJ = -ALPHAS/H
C
C     compute variable stepsize error coefficient ck
      CK = ABS(ALPHA(KP1) + ALPHAS - ALPHA0)
      CK = MAX(CK,ALPHA(KP1))
C
C     decide whether new jacobian is needed
      TEMP1 = (1.0D0 - XRATE)/(1.0D0 + XRATE)
      TEMP2 = 1.0D0/TEMP1
      IF (CJ/CJOLD .LT. TEMP1 .OR. CJ/CJOLD .GT. TEMP2) JCALC = -1
C
C     change phi to phi star
      IF(KP1 .GE. NSP1) THEN
        DO 270 J=NSP1,KP1
           DO 260 I=1,NEQ
260           PHI(I,J)=BETA(J)*PHI(I,J)
270     CONTINUE
      END IF
C
C     update time
      X=X+H
      NINTER = KP1
C-----------------------------------------------------------------------
C   BLOCK 3
C   PREDICT THE SOLUTION AND DERIVATIVE,AND SOLVE THE CORRECTOR EQUATION
C-----------------------------------------------------------------------
C
C     PREDICT THE SOLUTION AND DERIVATIVE AND UPDATE FIRST TWO
C     COMPONENTS OF THE PHI ARRAY SO THAT THEY HOLD THE VALUES OF
C     Y AND H*YPRIME FOR THE DURATION OF THE CALL TO THE NONLINEAR
C     EQUATIONS SOLVER . THIS IS THE ONLY ALGORITHMIC DEPARTURE FROM
C     THE DASSL CODE OF PETZOLD.
      DO 310 I=1,NEQ
         PHI(I,LMXORD+2) = PHI(I,1)
         PHI(I,LMXORD+3) = PHI(I,2)
         PHI(I,1) = PHI(I,1) + PHI(I,2)
310      PHI(I,2) = PHI(I,2) * GAMMA(2)
      DO 330 J=3,KP1
         DO 320 I=1,NEQ
            PHI(I,1) = PHI(I,1) + PHI(I,J)
320         PHI(I,2) = PHI(I,2) + PHI(I,J) * GAMMA(J)
330   CONTINUE
      DO 340 I = 1,NEQ
         Y(I)      = PHI(I,1)
         YPRIME(I) = PHI(I,2)
         PHI(I,2)  = PHI(I,2) * H
340   CONTINUE
C
      PNORM = VNORM( NEQ, Y, WT)
C
C   SOLVE THE CORRECTOR EQUATION USING A MODIFIED NEWTON SCHEME.
C
341   CONVGD= .TRUE.
C
C IF INDICATED,REEVALUATE THE ITERATION MATRIX PD=DG/DY+CJ*DG/DYPRIME
C (WHERE G(X,Y,YPRIME)=0). SET JCALC TO 0 AS AN INDICATOR THAT
C THIS HAS BEEN DONE.
      IF(JCALC .EQ. -1)THEN
        INLN = 1
        JCALC= 0
        CJOLD= CJ
        MAXIT= 0
        EL0    =-1.D0/ALPHAS
        IDID   = 0
        JJ     = 0
        RETURN
      ELSE
        JJ     = 1
        INLN   = 3
        ISTEP  = 0
        RETURN
      END IF
C
343   M=0
      S = 100.E0
C
C     initialize the error accumulation vector e.
      DO 345 I=1,NEQ
345      E(I)=0.0D0
C
C     corrector loop.
350   CONTINUE
C
      IF( JJ .GT. 0)THEN
C       EXIT FOR BACK SUBSTITUTION.
        JJ = 1
        INLN = 4
        ISTEP= 0
        RETURN
      END IF
356   CONTINUE
C
C     update y,e,and yprime
      DO 360 I=1,NEQ
         Y(I)=Y(I)+DELTA(I)
         E(I)=E(I)+DELTA(I)
360      YPRIME(I)=YPRIME(I)+CJ*DELTA(I)
      IF(ITRACE.GE.2)THEN
         WRITE(IDEV,361)
361      FORMAT(' SOLUTION INCREMENTS ARE')
         WRITE(IDEV,362)(DELTA(I),I = 1,NEQ)
         WRITE(IDEV,363)
363      FORMAT(' SOLUTION IS ')
         WRITE(IDEV,362)(Y(I),I = 1,NEQ)
362      FORMAT(1X,5D11.3)
      END IF
C
C     test for convergence of the iteration
      DELNRM = VNORM( NEQ, DELTA, WT)
      IF (DELNRM .LE. 100.D0 * U * PNORM) GO TO 390
      IF (M .GT. 0) GO TO 365
         OLDNRM = DELNRM
         GO TO 367
365   RATE = (DELNRM/OLDNRM)**(1.0D0/DFLOAT(M))
      IF (RATE .GT. 0.90D0) GO TO 370
      S = RATE/(1.0D0 - RATE)
367   IF (S*DELNRM .LE. 0.33D0) GO TO 390
C
C     THE CORRECTOR HAS NOT YET CONVERGED. UPDATE M AND TEST WHETHER
C     MAXIMUM NUMBER OF ITERATIONS , SET HERE TO FOUR, HAVE BEEN TRIED.
      M=M+1
      IF(M.GE. 4  )GO TO 370
C
C     EVALUATE THE RESIDUAL  AND GO BACK TO DO ANOTHER ITERATION AT 350
C
      JJ    = 2
      INLN  = 3
      ISTEP = 0
      RETURN
C
370   CONTINUE
C
C     THE CORRECTOR FAILED TO CONVERGE IN MAXIT ITERATIONS. IF THE
C     ITERATION MATRIX IS NOT CURRENT,RE-DO THE STEP WITH  A NEW
C     ITERATION MATRIX.
      IF(ITRACE .GE. 1)WRITE(IDEV,371)
371   FORMAT(' CONVERGENCE FAILURE 1')
      IF(JCALC.EQ.0)GO TO 380
      JCALC=-1
C       RESET Y AND YPRIME
      DO 375 I = 1,NEQ
         Y(I) = PHI(I,1)
375      YPRIME(I) = PHI(I,2)/H
      GO TO 341
C
C     exits from block 3
C     NO CONVERGENCE WITH CURRENT ITERATION MATRIX,OR SINGULAR MATRIX
380   CONVGD= .FALSE.
      IF(ITRACE .GE. 1)WRITE(IDEV,386)
386   FORMAT(' CONVERGENCE FAILURE 2')
390   JCALC = 1
C
C       RESTORE THE PHI VECTOR TO WHAT IT SHOULD BE
C
       DO 392 I = 1,NEQ
          PHI(I,1) = PHI(I,LMXORD+2)
          PHI(I,2) = PHI(I,LMXORD+3)
 392   CONTINUE
      IF(.NOT.CONVGD)GO TO 600
C
C-----------------------------------------------------------------------
C     block 4
C     ESTIMATE THE ERRORS AT ORDERS K,K-1,K-2 AS IF CONSTANT STEPSIZE
C     WAS USED. ESTIMATE THE LOCAL ERROR AT ORDER K AND TEST
C     WHETHER THE CURRENT STEP IS SUCCESSFUL.
C-----------------------------------------------------------------------
C
C     estimate errors at orders k,k-1,k-2
      ENORM = VNORM(NEQ,E,WT)
      ERR1   = CK * ENORM
      ERK    = SIGMA(K+1)*ENORM
      TERK   = (K+1.D0)*ERK
      EST    = ERK
      KNEW   = K
      DO 421 I = 1,NEQ
        PHI(I,LMXORD+3) = YPRIME(I)
        YPRIME(I)       = E(I)
421   CONTINUE
      JJ = 1
      INLN = 8
      RETURN
422   SERK = VNORM(NEQ,DELTA,WT) *SIGMA(K+1) * (1.D0 + K)
      JJ   = 2
      INLN = 4
      RETURN
424   ENORM = VNORM(NEQ,DELTA,WT) * ABS(ALPHAS/H)
      ERR   = CK * ENORM *SIGMA(K+1)
      EST   = ENORM *SIGMA(K+1)
      DO 423 I = 1,NEQ
423     YPRIME(I) = PHI(I,LMXORD+3)
      IF(K .EQ. 1)GO TO 430
      DO 405 I = 1,NEQ
405     YPRIME(I) = PHI(I,KP1) + E(I)
      ERKM1=SIGMA(K)* VNORM(NEQ,YPRIME,WT)
      TERKM1 = K * ERKM1
      JJ = 3
      INLN = 8
      RETURN
406   SERKM1 = VNORM(NEQ,DELTA,WT) * K
      IF(((K.EQ.2) .AND. (SERKM1 .LE. 0.5*SERK))  .OR.
     1   ((K.GT.2) .AND. (SERKM1 .LE.     SERK))) THEN
C         LOWER THE ORDER
          KNEW=K-1
          EST = ERKM1
          GOTO 430
      END IF
C  ESTIMATE ERROR AT ORDER K-2
      IF( K .GT. 2)THEN
         DO 407 I = 1,NEQ
407         YPRIME(I) = PHI(I,K) + YPRIME(I)
         ERKM2  = SIGMA(K-1)* VNORM(NEQ, YPRIME ,WT)
         TERKM2 = ERKM2 * ( K - 1.D0)
         IF (TERKM2 .LE. TERK )THEN
            JJ = 4
            INLN = 8
            RETURN
         ELSE
            GOTO 430
         END IF
      END IF
408   IF ( K .GT. 2)THEN
         ERKM2  = SIGMA(K-1)* VNORM(NEQ,DELTA,WT)
         SERKM2 = ERKM2 * ( K - 1.D0)
         IF ( SERKM2 .LE. SERK )THEN
C           LOWER THE ORDER
            KNEW=K-1
            EST = ERKM1
         END IF
      END IF
C
C     calculate the local error for the current step
C     to see if the step was successful
430   CONTINUE
      DO 427 I = 1,NEQ
427     YPRIME(I) = PHI(I,LMXORD+3)
      IF(ITRACE .GE. 1)WRITE(IDEV,431)ERR,ERR1
431   FORMAT(' SCALED LOCAL  ERRORS=',2D12.3)
      IF(ERR .GT. 1.0D0)GO TO 600
C
      IF(RMAX .EQ. 1.0D+2)THEN
C        FIRST STEP - SEE IF INIT STEP WAS TOO SMALL
         TEMP2 = K + 1.D0
         R = (2.D0 * EST + 0.0001D0) ** (-1.D0 / TEMP2 )
         IF(ITRACE .GE. 1)WRITE(IDEV,4301)R
4301     FORMAT(' R ON INITIAL STEP IS ',D12.4)
         IF(R .GE. RMAX .AND. (H*HMXI) .LT. (1.0D0-U*8.D0)) THEN
C           RETAKE THE STEP AS THE INITIAL STEP WAS FAR TOO SMALL
            R = RMAX
            IF( (R * H * HMXI) .GT. 1.D0) R= 1.D0 /(H * HMXI)
            X = XOLD
            IF(KP1.GE.NSP1) THEN
               DO 433 J=NSP1,KP1
                  TEMP1=1.0D0/BETA(J)
                  DO 432 I=1,NEQ
432                   PHI(I,J)=TEMP1*PHI(I,J)
433               CONTINUE
            END IF
            DO 434 I=2,KP1
434            PSI(I-1)=PSI(I)-H
            H=H * R
            GOTO 200
         END IF
      END IF
C-----------------------------------------------------------------------
C     BLOCK 5
C     THE STEP IS SUCCESSFUL. DETERMINE THE BEST ORDER AND STEPSIZE FOR
C     THE NEXT STEP. UPDATE THE DIFFERENCES FOR THE NEXT STEP.
C-----------------------------------------------------------------------
      IDID=1
      KDIFF=K-KOLD
      KOLD=K
      NQU =K
      HOLD=H
C
C   ESTIMATE THE ERROR AT ORDER K+1 UNLESS ALREADY DECIDED TO LOWER
C   ORDER OR ALREADY USING MAXIMUM ORDER, OR STEPSIZE NOT CONSTANT, OR
C   ORDER RAISED IN PREVIOUS STEP
C
      IF(KNEW.EQ.KM1 .OR. K.EQ. LMXORD)IPHASE=1
      IF(IPHASE .EQ. 0)GO TO 545
      IF(KNEW.EQ.KM1)GO TO 540
      IF(K.EQ. LMXORD) GOTO 550
      IF(KP1.GE.NS.OR.KDIFF.EQ.1)GO TO 550
      DO 510 I=1,NEQ
510      DELTA(I)=E(I)-PHI(I,KP2)
      ERKP1 = (1.0D0/(K+2.D0))* VNORM(NEQ,DELTA,WT)
      TERKP1 = (K+2.D0)*ERKP1
      DO 511 I = 1,NEQ
        PHI(I,LMXORD+3) = YPRIME(I)
        YPRIME(I)       = DELTA(I)
511   CONTINUE
      JJ = 2
      INLN = 8
      RETURN
512   SERKP1 = VNORM(NEQ,DELTA,WT)
      IF(ITRACE .GE. 1)WRITE(IDEV,513)TERKM1,TERK,TERKP1,SERKM1,SERK,
     1                 SERKP1
513   FORMAT(' TERKMI=',D11.3,'TERK=',D11.3,' TERKP1=',D11.3/
     1       ' SERKM1=',D11.3,'SERK=',D11.3,' SERKP1=',D11.3)
      DO 514 I = 1,NEQ
514     YPRIME(I) = PHI(I,LMXORD+3)
      IF(K.GT.1)GO TO 520
      IF(SERKP1.GE.0.5D0*SERK)GO TO 550
      GO TO 530
520   IF(SERKM1.LE.MIN(SERK,SERKP1))GO TO 540
      IF(SERKP1.GE.SERK.OR.K.EQ. LMXORD)GO TO 550
C
C     raise order
530   K=KP1
CMBZ
      IF(ITRACE .GE. 1)WRITE(IDEV,531)
 531  FORMAT(' ORDER RAISE BEING ATTEMPTED')
      EST = ERKP1
      GO TO 550
C
C     lower order
540   K   = K-1
      EST = ERKM1
      GO TO 550
C
C     if iphase = 0, increase order by one and multiply stepsize by
C     factor two
545   K = KP1
      IF(ITRACE .GE. 1)WRITE(IDEV,546)
 546  FORMAT(' ORDER RAISE BEING ATTEMPTED IPHASE = 0')
      HNEW = H*2.0D0
      IF(HNEW*HMXI .GT. 1.D0)HNEW = 1.D0/HMXI
      H = HNEW
      GO TO 575
C
C     determine the appropriate stepsize for
C     the next step.
550   HNEW=H
      NQ  = K
      TEMP2=K+1
      R=(2.0D0*EST+0.0001D0)**(-1.0D0/TEMP2)
      IF( (R * H * HMXI) .GT. 1.D0) THEN
C  NEXT TWO LINES ARE MB CHANGES TO DASSL STEPSIZE STRATEGY 3/7/87
         R= 1.D0 /(H * HMXI)
         HNEW = H * R
      ELSE IF(R .GE. 2.0D0) THEN
         HNEW = 2.0D0*H
C     ELSE IF(R .LE. 1.D0 .AND. ERR .LE. 0.5D0 .AND. KDIFF .EQ. 0)THEN
C  NEXT TWO LINES ARE MB CHANGES TO DASSL STEPSIZE STRATEGY 15/4/88
C        R = 1.0D0
C        HNEW = H
      ELSE IF(R .LE. 1.0D0) THEN
         R = MAX(0.5D0,MIN(0.9D0,R))
         HNEW = H*R
      END IF
      H=HNEW
C
C
C     update differences for next step
575   CONTINUE
      IF(KOLD.EQ.LMXORD)GO TO 585
      DO 580 I=1,NEQ
580      PHI(I,KP2)=E(I)
585   CONTINUE
      DO 590 I=1,NEQ
590      PHI(I,KP1)=PHI(I,KP1)+E(I)
      DO 595 J1=2,KP1
         J=KP1-J1+1
         DO 595 I=1,NEQ
595      PHI(I,J)=PHI(I,J)+PHI(I,J+1)
      DO 596 I = 1,NEQ
         E (I)   = E(I) * CK
596   CONTINUE
      RMAX = 10.0D0
      RETURN
C
C-----------------------------------------------------------------------
C     block 6
C     the step is unsuccessful. restore x,psi,phi
C     determine appropriate stepsize for
C     continuing the integration, or exit with
C     an error flag if there have been many
C     failures.
C-----------------------------------------------------------------------
600   IPHASE = 1
C
C     restore x,phi,psi
      X=XOLD
      IF(KP1.GE.NSP1)THEN
         DO 620 J=NSP1,KP1
            TEMP1=1.0D0/BETA(J)
            DO 610 I=1,NEQ
610            PHI(I,J)=TEMP1*PHI(I,J)
620      CONTINUE
      END IF
      DO 640 I=2,KP1
640      PSI(I-1)=PSI(I)-H
C
C    SEE IF RESID ROUTINE ORDERED RETURN OR IF ENTRY INTO MONITR REQD.
CC
      IF(IZ .EQ. 1)THEN
         IDID = -7
         GOTO 675
      ELSE IF(IZ .EQ. 3)THEN
           IDID = -4
           GOTO 675
      END IF

C
C     test whether failure is due to corrector iteration
C     or error test
      IF(CONVGD)GO TO 660
      IF( IZ .EQ. 4)THEN
C        THE ITERATION MATRIX IS SINGULAR. REDUCE
C        THE STEPSIZE BY A FACTOR OF 4. IF
C        THIS HAPPENS THREE TIMES IN A ROW ON
C        THE SAME STEP, RETURN WITH AN ERROR FLAG
         NSF=NSF+1
         R = 0.25D0
         H=H*R
         IF (NSF .LT. 3 .AND. ABS(H) .GE. HMIN) GO TO 690
         IDID=-5
      ELSE
C        THE NEWTON ITERATION FAILED TO CONVERGE FOR A REASON
C        OTHER THAN A SINGULAR ITERATION MATRIX. REDUCE THE STEP SIZE
C        AND TRY AGAIN, UNLESS TOO MANY FAILURES HAVE OCCURED.
         NCF = NCF + 1
         R = 0.25D0
         H = H*R
         IF (NCF .LT. 10 .AND. ABS(H) .GE. HMIN) GO TO 690
         IDID = -2
      END IF
      GO TO 675
C
C     the newton scheme converged,and the cause
C     of the failure was the error estimate
C     exceeding the tolerance.
660   NEF=NEF+1
      IEFAIL = IEFAIL + 1
      IF(ITRACE .GE.1)WRITE(IDEV,661)
661   FORMAT(' ERROR TEST FAILURE')
      IF (NEF .GT. 1) GO TO 665
C
C     on first error test failure, keep current order or lower
C     order by one.  compute new stepsize based on differences
C     of the solution.
      K = KNEW
      TEMP2 = K + 1
      R = 0.90D0*(2.0D0*EST+0.0001D0)**(-1.0D0/TEMP2)
      R = MAX(0.25D0,MIN(0.9D0,R))
      H = H*R
      IF (ABS(H) .GE. HMIN) GO TO 690
      IDID = -1
      GO TO 675
C
C     on second error test failure, use the current order or
C     decrease order by one.  reduce the stepsize by a factor of
C     one quarter.
665   IF (NEF .GT. 2) GO TO 670
      K = KNEW
      H = 0.25D0*H
      IF (ABS(H) .GE. HMIN) GO TO 690
      IDID = -1
      GO TO 675
C
C     on third and subsequent error test failures, set the order to
C     one and reduce the stepsize by a factor of one quarter
670   K = 1
      H = 0.25D0*H
      IF (ABS(H) .GE. HMIN) GO TO 690
      IDID = -1
C
C     for all crashes, restore y to its last value,
C     INTERPOLATE TO FIND YPRIME AT LAST X, AND RETURN AFTER PUTTING
C     THE LOST ERROR ESTIMATE FOR THE PREVIOUS STEP TO ZERO
675   CONTINUE
      CALL SDATRP( X, X, Y, YPRIME, NEQ, K, PHI, NYH)
      DO 680 I = 1,NEQ
          E(I)   = 0.0D0
680   CONTINUE
      WRITE(IDEV,685)
685   FORMAT(' STEP FAILED - ERROR ESTIMATE LOST FOR PREVIOUS STEP')
      RETURN
C
C     go back and try this step again
690   GO TO 200
C
C------end of subroutine dastep------
      END
      SUBROUTINE SDATRP(X,XOUT,YOUT,YPOUT,NEQ,KOLD,PHI,NYH)
C
C-----------------------------------------------------------------------
C     the methods in subroutine dastep use polynomials
C     TO APPROXIMATE THE SOLUTION. SDATRP APPROXIMATES THE
C     solution and its derivative at time xout by evaluating
C     one of these polynomials,and its derivative,there.
C     information defining this polynomial is passed from
C     dastep, so ddatrp cannot be used alone.
C
C     the parameters are%
C     x     the current time in the integration.
C     xout  the time at which the solution is desired
C     yout  the interpolated approximation to y at xout
C           (this is output)
C     ypout the interpolated approximation to yprime at xout
C           (this is output)
C     neq   number of equations
C     kold  order used on last successful step
C     phi   array of scaled divided differences of y
C           DASSL PHI STARTS AT PHI(1,3) PLEASE NOTE
C   VIA COMMON....
C     psi   array of past stepsize history
C-----------------------------------------------------------------------
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION YOUT(1),YPOUT(1)
      DIMENSION PHI(NYH,1)
      COMMON /DDA002/ PSI(7),DUMMY(28)
      KOLDP1=KOLD+1
      TEMP1=XOUT-X
      DO 10 I=1,NEQ
         YOUT(I)=PHI(I,1)
10       YPOUT(I)=0.0D0
      C=1.0D0
      D=0.0D0
      GAMMA=TEMP1/PSI(1)
      DO 30 J=2,KOLDP1
         D=D*GAMMA+C/PSI(J-1)
         C=C*GAMMA
         GAMMA=(TEMP1+PSI(J-1))/PSI(J)
         DO 20 I=1,NEQ
            YOUT(I)=YOUT(I)+C*PHI(I,J)
20          YPOUT(I)=YPOUT(I)+D*PHI(I,J)
30       CONTINUE
      RETURN
C------END OF SUBROUTINE SDATRP------
      END
