      SUBROUTINE SPANUF( NEQ, NEQMAX, JCEVAL, NWKJAC, IA, NIA, JA, NJA,
     1                   JACPVT, NJCPVT, SENS, U, ETA, LBLOCK,
     2                   ISPLIT, ISET )

C NB: THIS ROUTINE NEEDS TO HAVE THE ERROR HANDLING CONVERTED TO
C     ONE SUCH THAT EXECUTION CONTINUES, WITH AN ERROR FLAG SET.
C***********************************************************************
C
C THIS IS THE SETUP ROUTINE FOR THE SPRINT (NAG VERSION) PACKAGE. THE
C ROUTINE SETS UP THE DATA USED BY THE SPRINT INTERFACE TO THE NAG
C SPARSE MATRIX PACKAGE.
C
C INPUT PARAMETERS:
C
C NEQ    : THE NUMBER OF DIFFERENTIAL EQUATIONS TO BE SOLVED.
C NEQMAX : THE MAXIMUM NUMBER OF DIFFERENTIAL EQUATIONS TO BE SOLVED
C          DURING THE INTEGRATION.
C JCEVAL : A STRING SPECIFYING THE METHOD WHICH THE USER WISHES TO
C          USE TO DETERMINE THE JACOBIAN R/Y. VALUES ACCEPTED ARE:
C          'NUMERICAL'  - SPARSITY STRUCTURE & VALUES OF R/Y ARE TO
C                         BE DETERMINED NUMERICALLY.
C          'STRUCTURAL' - SPARSITY STRUCTURE OF R/Y IS SUPPLIED IN
C                         THE ARRAYS IA AND JA, BUT THE VALUES OF
C                         R/Y ARE TO BE EVALUATED NUMERICALLY.
C                         (THIS IS THE RECOMMENDED MODE OF OPERATION
C                         UNLESS IT IS A SIMPLE MATTER TO SUPPLY THE
C                         JACOBIAN.)
C          'ANALYTICAL' - THE VALUES OF R/Y WILL BE CALCULATED BY
C                         CALLS TO A SUBROUTINE JAC SUPPLIED BY THE
C                         USER. THE SPARSITY STRUCTURE IS ESTIMATED
C                         BY CALLS TO JAC ALSO.
C          'FULL INFO'  - SPARSITY STRUCTURE IS SUPPLIED IN THE IA
C                         AND JA ARRAYS, AND THE VALUES OF R/Y WILL
C                         BE CALCULATED BY CALLS TO A USER PROVIDED
C                         SUBROUTINE JAC. (THIS IS THE RECOMMENDED
C                         MODE OF OPERATION IF THE SUBROUTINE JAC
C                         IS SIMPLE TO FORM.
C          'DEFAULT'    - THE SAME AS 'STRUCTURAL'.
C
C NB. IF THE SPARSITY STRUCTURE IS SUPPLIED IN IA AND JA THEN ANY
C     EVIDENCE FROM THE NUMERICAL OR ANALYTICAL FORMATION OF THE
C     JACOBIAN THAT THIS STRUCTURE IS NOT CORRECT IS IGNORED!
C
C     ONLY THE FIRST CHARACTER OF THE ABOVE "MODES" IS ACTUALLY
C     REQUIRED, THUS 'N', 'S', 'A', 'F' AND 'D' WILL SUFFICE.
C
C     IF A NUMERICAL JACOBIAN IS TO BE FORMED (JCEVAL IS 'N', 'S'
C     OR 'D') THEN THE DUMMY ROUTINE SPANDZ MUST BE SUPPLIED AS THE
C     ACTUAL ARGUMENT FOR JAC AS SPECIFIED FOR THE CALL TO THE
C     INTEGRATOR.
C
C NWKJAC : SPECIFIES THE SIZE OF THE WORKSPACE ARRAY WKJAC TO BE
C          SUPPLIED TO THE INTEGRATOR. AS THE SIZE REQUIRED FOR
C          THIS WORKSPACE CANNOT BE DETERMINED PRECISELY IN ADVANCE,
C          AS IT DEPENDS ON THE NUMBER OF NON-ZERO ELEMENTS IN THE
C          JACOBIAN AND THEIR SPARSITY PATTERN, THEN IF THE
C          SPARSITY STRUCTURE IS NOT SUPPLIED, THE STARTING ESTIMATE
C          IS 4 * NEQMAX. IF THIS IS GREATER THAN NWKJAC A WARNING
C          IS GIVEN, BUT EXECUTION CONTINUES.
C
C IA, JA : ARRAYS OF LENGTHS  OF (AT LEAST) NIA AND NJA RESP.
C          IF JCEVAL = 'S', 'F' OR 'D' THEN THESE TWO ARRAYS MUST
C          DESCRIBE THE SPARSITY PATTERN TO BE USED FOR THE JACOBIAN.
C          JA CONTAINS THE ROW INDICES OF THE NON-ZERO ELEMENTS, IN
C          COLUMN-WISE ORDER, AND IA CONTAINS THE STARTING LOCATIONS
C          IN JA OF THE DESCRIPTIONS OF COLUMNS 1,2,...,NEQ IN THAT
C          ORDER, WITH IA(1) = 1. THUS FOR EACH COLUMN INDEX J = 1,
C          2,...,NEQ, THE VALUES OF THE ROW INDEX I IN COLUMN J WHERE
C          A NON-ZERO ELEMENT MAY OCCUR ARE GIVEN BY:
C            I = JA(K), WHERE IA(J) <= K <= IA(J+1).
C
C          THUS FOR NELEMENT, THE TOTAL NUMBER OF NON-ZEROS,
C          IA(NEQ+1) MUST BE NELEMENT+1.
C          DUPLICATE ENTRIES ARE NOT ALLOWED.
C
C NIA,NJA: THESE DEFINE THE LENGTHS OF THE ARRAYS IA AND JA RESPEC.
C          THEY MUST BE OF A SIZE LARGE ENOUGH TO DEFINE THE STRUCTURE
C          OF THE JACOBIAN IF IA AND JA ARE USED. IN THIS CASE
C          NIA >= NEQ+1 & NJA >= IA(NEQ+1).
C
C NJCPVT : SPECIFIES THE SIZE OF THE WORKSPACE ARRAY JACPVT, WHICH IS
C          SUPPLIED BY THE USER TO THE INTEGRATOR AS DIMENSIONED IN
C          THE USERS CALLING PROGRAM.
C          AS FOR NWKJAC, THE SIZE REQUIRED CANNOT BE PRECISELY
C          DETERMINED IN ADVANCE. THE STARTING ESTIMATE SHOULD BE AT
C          LEAST 20 * NEQMAX. IF JCEVAL = 'S', 'F' OR 'D', THEN
C          NJCPVT MUST BE GREATER THAN NEQ+NELEMENT+1. SEE OUTPUT
C          PARAMETERS: JACPVT.
C
C SENS   : IS A THRESHOLD PARAMETER USED TO DETERMINE WHETHER OR NOT
C          A MATRIX ELEMENT IS ZERO :- WHEN SENS IS SET TO 0.0 ON
C          ENTRY, THE ROUTINE WILL USE SENS = 100.0 * MACHEPS (SEE
C          X02AAF), AND THIS VALUE IS OUTPUT. OTHERWISE THE ABSOLUTE
C          VALUE OF SENS IS USED.
C
C U      : ON ENTRY U SHOULD HAVE A VALUE BETWEEN 0.0 AND 0.9999. IF
C          NOT A DEFAULT VALUE OF 0.1 IS SET. WHEN THE SPARSITY
C          PATTERN HAS BEEN EVALUATED THEN THE FIRST JACOBIAN COMPUTED
C          IS DECOMPOSED WITH U GOVERNING THE CHOICE OF PIVOTS. ALL
C          SUBSEQUENT JACOBIAN DECOMPOSITIONS USE THE SAME PATTERN OF
C          DECOMPOSITION UNTIL THE SPARSITY PATTERN IS RE-EVALUATED.
C          WHEN SEARCHING A ROW FOR A PIVOT THEN ANY ELEMENT IS
C          EXCLUDED FROM THE SEARCH WHICH IS LESS THAN U TIMES THE
C          LARGEST OF THOSE ELEMENTS IN THE ROW AVAILABLE AS PIVOTS.
C          THUS DECREASING U BIASES THE ALGORITHM TOWARDS MAINTAINING
C          SPARSITY AST THE EXPENSE OF NUMERICAL STABILITY.
C
C ETA    : ON SUBSEQUENT DECOMPOSITIONS (SEE U ABOVE) ETA IS USED AS
C          A RELATIVE PIVOT THRESHOLD, BELOW WHICH AN INTERNAL ERROR
C          IS PROVOKED. IF ETA <= 0.0 THEN THE DEFAULT 10.0 VALUE IS
C          USED INSTEAD.
C
C LBLOCK : IF LBLOCK = .TRUE. ON ENTRY THEN THE JACOBIAN MATRIX IS
C          PREORDERED TO BLOCK LOWER TRIANGULAR FORM BEFORE A
C          DECOMPOSITION IS PERFORMED, (THIS IS THE RECOMMENDED
C          MODE). IF THE USER KNOWS THAT THE STRUCTURE OF THE JACOBIAN
C          IS IRREDUCIBLE, IE: IT IS NOT PERMUTABLE TO BLOCK LOWER
C          TRIANGULAR FORM, THEN THE USER SHOULD SET LBLOCK = .FALSE.
C
C ISPLIT : IS USED FOR SPLITTING THE INEGER WORKSPACE JACPVT TO EFFECT
C          AN EFFICIENT DECOMPOSITION. IT MUST SATISFY 1 < ISPLIT <
C          99. IF ISPLIT LIES OUTSIDE THIS RANGE ON ENTRY THEN THE
C          DEFAULT VALUE OF 73 IS USED.
C              ISPLIT PERCENT OF PART OF THE WORKSPACE IS USED TO STORE
C          THE COLUMN INDICES AND ROW START POSITIONS OF THE ORIGINAL
C          UNDECOMPOSED JACOBIAN MATRIX WHILE 100-ISPLIT PERCENT OF THE
C          WORKSPACE IS USED TO STORE THE COLUMN INDICES AND ROW START
C          POSITIONS OF THE DECOMPOSED MATRIX. AS NIETHER THE NUMBER
C          OF NON-ZEROES OR THE FILL-IN GENERATED WILL BE KNOWN BEFORE
C          INTEGRATION IN GENERAL IT WILL BE NECESSARY TO GUESS ISPLIT.
C              AN APPROPRIATE VALUE FOR ISPLIT FOR SUBSEQUENT RUNS ON A
C          SIMILAR PROBLEM IS AVAILABLE VIA  THE OPTIONAL OUTPUT
C          ROUTINE SPANXF.
C
C OUTPUT PARAMETERS:
C
C JACPVT : A REAL ARRAY OF LENGTH AT LEAST NJCPVT. ON OUTPUT FROM
C          SPANUF THE ARRAYS IA AND JA ARE PACKED INTO JACPVT IF IA
C          AND JA ARE BEING USED, OTHERWISE JACPVT IS NOT USED.
C          SEE NJCPVT ABOVE.
C
C ISET   : THIS INTEGER VARIABLE IS SET TO -1 IF AN ERROR CONDITION
C          SHOULD OCCUR WHILE THE SETUP ROUTINE IS IN OPERATION.
C
C OTHER INFORMATION:
C
C RWORK  : THIS IS THE ARRAY USED BY THE INTEGRATOR OF THE SAME NAME.
C          IT CONTAINS INFO FROM THE SETUP ROUTINES.
C
C UNLIKE SPANUF, THERE ARE ERROR RETURNS FROM SPANUF. IF AN ERROR SHOULD
C OCCUR THEN AN ERROR MESSAGE IS SENT TO THE CURRENT ERROR MESSAGE UNIT,
C AN ERROR FLAG (ISET) IS SET AND CONTROL IS RETURNED TO THE CALLING
C PROGRAM.
C
C***********************************************************************
      INTEGER NEQ, NEQMAX, NWKJAC, IA(NIA), NIA, JA(NJA), NJA,
     1        JACPVT(NJCPVT), NCJPVT, ISPLIT
      DOUBLE PRECISION  SENS, U, ETA, RWORK
      LOGICAL LBLOCK
      CHARACTER*(*) JCEVAL
C
      INTEGER IDEV, NMOSS, I, IAPI, IAPIM1, IFAIL, IPIAN, IPJAN, J, K,
     1        KCOL, KMAX, KMIN, L, LNJCRQ, LNJREQ, ISET, ITRACE
      LOGICAL LFULL
      CHARACTER*1 JCEVL1
C
      CHARACTER*6 LAOPTN, ALCODE, SINGLR
      COMMON /SPHIL/ RWORK(50)
      COMMON /SDEV2/ ITRACE, IDEV
      COMMON /ALGCHK/ ALCODE, SINGLR
      COMMON /OPTNLA/ LAOPTN
      SAVE   /OPTNLA/, /SPHIL/, /ALGCHK/, /SDEV2/
C
      LAOPTN = 'SPARSE'
      ALCODE = 'NAGSPA'
      SINGLR = 'SINGLR'
      CALL X04AAF( 0, IDEV)
      JCEVL1 = JCEVAL(1:1)
      IF (JCEVL1 .EQ. 'S' .OR. JCEVL1 .EQ. 'D') THEN
         NMOSS = 0
      ELSEIF (JCEVL1 .EQ. 'A') THEN
         NMOSS = 1
      ELSEIF (JCEVL1 .EQ. 'N') THEN
         NMOSS = 2
      ELSEIF (JCEVL1 .EQ. 'F') THEN
         NMOSS = 3
      ELSE
         WRITE(IDEV,99999) JCEVL1
         GOTO 920
      ENDIF
      RWORK(48) = DBLE(NMOSS)
      RWORK(34) = 3.0D0
      RWORK(35) = 1.0D0
      IF (NEQ .LT. 1) THEN
         WRITE(IDEV,99998) NEQ
         GOTO 920
      ENDIF
      IF (NEQMAX .LT. 1) THEN
         WRITE(IDEV,99997) NEQMAX
         GOTO 920
      ENDIF
      IF (NEQ .GT. NEQMAX) THEN
         WRITE(IDEV,99996) NEQ, NEQMAX
         GOTO 920
      ENDIF
      IF (SENS .NE. 0.0D0) THEN
         RWORK(43) = DABS(SENS)
      ELSE
         RWORK(43) = 100.0D0 * X02AAF(1.0D0)
      ENDIF
      IF (ISPLIT .LE. 1 .OR. ISPLIT .GE. 99) THEN
         RWORK(36) = 8.0D0/11.0D0
         RWORK(37) = 0.0D0
      ELSE
         RWORK(36) = DBLE(ISPLIT)/100.0D0
         RWORK(37) = 1.0D0
      ENDIF
      IF (U .LT. 0.0D0 .OR. U .GT. 0.9999D0) THEN
         RWORK(38) = 0.1D0
      ELSE
         RWORK(38) = U
      ENDIF
      IF (ETA .GT. 1.0D0) THEN
         RWORK(39) = 2.0D0
      ELSEIF (ETA .LE. 0.0D0) THEN
         RWORK(39) = 1.0D-4
      ELSE
         RWORK(39) = ETA
      ENDIF
C     IF (GROW) THEN
         RWORK(41) = 1.0D0
C     ELSE
C        RWORK(41) = 0.0D0
C     ENDIF
      IF (LBLOCK) THEN
         RWORK(42) = 1.0D0
      ELSE
         RWORK(42) = 0.0D0
      ENDIF
C
      IF (NMOSS .EQ. 1 .OR. NMOSS .EQ. 2) GOTO 1000
C
      IFAIL = 0
      IF (NIA .LT. NEQ+1) THEN
         IFAIL = 1
         WRITE(IDEV,99995) NIA, NEQ+1
      ENDIF
      LNJREQ = IA(NEQ+1) - 1
      IF (LNJREQ .LT. NEQ) THEN
         WRITE(IDEV,99994) LNJREQ, NEQ
         GOTO 920
      ENDIF
      IF (NJA .LT. LNJREQ) THEN
         IFAIL = 1
         WRITE(IDEV,99993) NJA, LNJREQ
      ENDIF
      IF (IFAIL .GT. 0) GOTO 920
      LNJCRQ = 3*LNJREQ + 14*NEQ + 1
      IF (LNJCRQ .GT. NJCPVT) THEN
         WRITE(IDEV,99992) NJCPVT, LNJCRQ
         GOTO 920
      ENDIF
      IF (IA(1) .NE. 1) THEN
         WRITE(IDEV,99989) IA(1)
         GOTO 920
      ENDIF
      JACPVT(1) = 1
      DO 100 I = 2,NEQ+1
         IAPI = IA(I)
         IAPIM1 = IA(I-1)
         IF (IAPI .LE. IAPIM1) THEN
            WRITE(IDEV,99988) I,IAPI,I-1,IAPIM1
            GOTO 920
         ELSEIF (IAPI .GT. IAPIM1+NEQ) THEN
            WRITE(IDEV,99987) I,IAPI,I-1,IAPIM1,NEQ
            GOTO 920
         ELSE
            JACPVT(I) = IA(I)
         ENDIF
  100 CONTINUE
      KCOL = NJCPVT - NEQ
      IPIAN = 1
      IPJAN = IPIAN + NEQ + 1
      KMIN = 1
      IFAIL = 0
      DO 200 I =1,NEQ
         KMAX = IA(I+1) - 1
         DO 180 J = 1,NEQ
            JACPVT(KCOL+J) = 0
  180    CONTINUE
         DO 190 K = KMIN,KMAX
            L = JA(K)
            IF (L .LT. 1 .OR. L .GT. NEQ) THEN
               WRITE(IDEV,99986) K,L
               IFAIL = 1
               GOTO 190
            ENDIF
            LFULL = JACPVT(KCOL+L) .EQ. 1
            IF (LFULL) THEN
               WRITE(IDEV,99985) I, L
               IFAIL = 1
            ELSE
               JACPVT(KCOL+L) = 1
               JACPVT(K+IPJAN-1) = L
            ENDIF
  190    CONTINUE
         KMIN = KMAX + 1
  200 CONTINUE
      IF (IFAIL .GT. 0) GOTO 920
 1000 CONTINUE
      RWORK(46) = DBLE(NWKJAC)
      RWORK(47) = DBLE(NJCPVT)
      RETURN

  920 CONTINUE
      ISET = -1
      RETURN

99999 FORMAT(' *** SPANUF - JCEVAL(1:1)(=',A1,') IS NOT',
     1       ' ''N'' OR ''S'' OR ''A'' OR ''F'' OR ''D'' *** ')
99998 FORMAT(' *** SPANUF - NEQ(=',I8,') < 1 *** ')
99997 FORMAT(' *** SPANUF - NEQMAX(=',I8,') < 1 *** ')
99996 FORMAT(' *** SPANUF - NEQ(=',I8,') > NEQMAX(=',I8,') *** ')
99995 FORMAT(' *** SPANUF - NIA(=',I8,') < NEQ+1(=',I8,') *** ')
99994 FORMAT(' *** SPANUF - IA(NEQ+1)-1(=',I8,') < NEQ(=',I8,' *** ')
99993 FORMAT(' *** SPANUF - NJA(=',I8,') < IA(NEQ+1)-1(=',I8,') *** ')
99992 FORMAT(' *** SPANUF - NJCPVT(=',I8,') < LENGTH REQUIRED(=',I8,
     1       ') *** ')
99989 FORMAT(' *** SPANUF - IA(1)(=',I8,') IS NOT EQUAL TO 1 *** ')
99988 FORMAT(' *** SPANUF - IA(',I4,')(=',I8,') .LT. IA(',I4,')(=',I8,
     1       ') *** ')
99987 FORMAT(' *** SPANUF - IA(',I4,')(=',I8,') - IA(',I4,')(=',I8,
     1       ') > NEQ(=',I8,') *** ')
99986 FORMAT(' *** SPANUF - JA(',I4,')(=',I8,') < I OR > NEQ(=',I8,
     1       ') *** ')
99985 FORMAT(' *** SPANUF - DUPLICATE ENTRY IN ROW ',I4,',COL',I4,
     1       ' *** ')
      END
C=-=-=-=-=-=-=-=-= END OF SUBROUTINE SPANUF =-=-=-=-=-=-=-=-=-=-=-=-=-=-

      SUBROUTINE SPANXF( NEQMAX, ICALL, LIWREQ, LIWUSD, LRWREQ, LRWUSD,
     1                   NLU, NNZ, NGP, ISPLIT, IGROW, LBLOCK,
     2                   NBLOCK )

      INTEGER NEQMAX, ICALL, LIWREQ, LIWUSD, LRWREQ, LRWUSD, NLU, NNZ,
     1        NGP, ISPLIT, IGROW, NBLOCK, JSINFO
      LOGICAL LBLOCK
      INTEGER IDEV
      COMMON /INPHIL/ JSINFO(10)

      IF (NEQMAX .LT. 1) THEN
         CALL X04AAF( 0, IDEV)
         WRITE (IDEV,99999) NEQMAX
         STOP
      ENDIF
      LIWREQ = JSINFO(1)
      LIWUSD = JSINFO(2)
      LRWREQ = JSINFO(3)
      LRWUSD = JSINFO(4)
      IF (ICALL .EQ. 1) RETURN
      NLU = JSINFO(5)
      NNZ = JSINFO(6)
      NGP = JSINFO(7)
      ISPLIT = JSINFO(8)
      IGROW = JSINFO(9)
      IF (LBLOCK) NBLOCK = JSINFO(10)
      RETURN
99999 FORMAT(' *** SPANXF - NEQMAX(=',I8,' < 1 ***')
C-=-=-=-=-=-=-=-=-=- END OF SUBROUTINE SPANXF =-=-=-=-=-=-=-=-=-=-=-=-=-
      END
      SUBROUTINE SPANML (NEQ, Y, YDOT, YH, NYH, SAVF, EWT, FTEM,
     1                   WK, IWK, IPPER, H, EL0, TN, JAC )
C********************************************************************
C
C     OLD NAME PREP
C
C********************************************************************
      INTEGER NEQ, IWK, IPPER, NYH
      INTEGER IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     1   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     2   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     3   NSLJ, NGP, NLU, NNZ, NSP
      INTEGER I, IBR, IER, IPTT1, IPTT2, J, K, KNEW,
     1   KMAX, KMIN, LDIF, LENIGP, LIWK, MAXG, MOSSP1, NP1, N
      INTEGER ITRACE, IDEV, IROW, ICOL
      INTEGER IKEEP,IPIWK,IPRWK,IRN,LIRN,ICN,LICN
      INTEGER LIWUSD, LIWREQ, LRWREQ, LRWUSD, ISPSET
      INTEGER ISPIWK, ISPRWK, IXTRAR, IXTRAI, IBICN
      INTEGER LENIWK, LENRWK
      INTEGER MM4
      DOUBLE PRECISION  Y, YH, SAVF, EWT, FTEM, WK, EL0, H, TN, YDOT
      DOUBLE PRECISION  CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH
      DOUBLE PRECISION  DQ, DYJ, ERWT, FAC, YJ, ELH, YDOTJ
      DOUBLE PRECISION  RSPLIT
      CHARACTER*6 ALCODE, SINGLR
      DIMENSION NEQ(*), Y(*), YH(NYH,*), SAVF(*), EWT(*), FTEM(*),
     *                 WK(*), IWK(*), YDOT(*)
      EXTERNAL JAC
      COMMON /LSS001/ CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH,
     1   IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     2   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     3   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     4   NSLJ, NGP, NLU, NNZ, NSP
      COMMON /RWB002/ IKEEP,IPIWK,IPRWK,IRN,LIRN,ICN,LICN,ISPRWK,ISPIWK
C RWB002
C           HAS BEEN DIVIDED UP FOR NROUTINE SPANNU
C   IPRWK   POINTER FOR PART OF RWK TO BE USED AS WORKSPACE BY SPANNU
C   ISPIWK  LENGTH OF AVAILABLE SPACE OF IWK TO HOLD THE ROW AND COL
C           INDICES FOR DECOMPOSITION
C   ISPRWK  LENGTH OF AVAILABLE SPACE OF RWK TO HOLD THE MATRIX AND
C           THEN ITS DECOMPOSITION
      COMMON /RWB005/ LENIWK,LENRWK
      COMMON /RWB007/ LIWUSD, LIWREQ, LRWREQ, LRWUSD, RSPLIT, ISPSET
      COMMON /JACVAL/ J, MM4
      COMMON /JACPTR/ IROW, ICOL
       COMMON /ALGCHK/ ALCODE, SINGLR
       COMMON /SDEV2/ ITRACE, IDEV
C
C     COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /PRPSAV/ DQ, DYJ, ELH, ERWT, FAC, YJ, YDOTJ,
     1   I, IBR, IPTT2,    K, KMAX, KMIN, KNEW, LIWK, LDIF, LENIGP,
     2   MAXG, MOSSP1, N, NP1
      SAVE   /ALGCHK/, /PRPSAV/, /LSS001/, /RWB002/, /RWB007/,
     1       /JACVAL/, /JACPTR/, /SDEV2/
C-----------------------------------------------------------------------
C THIS ROUTINE PERFORMS PREPROCESSING RELATED TO THE SPARSE LINEAR
C SYSTEMS THAT MUST BE SOLVED IF MITER = 1 OR 2.
C THE OPERATIONS THAT ARE PERFORMED HERE ARE..
C  * COMPUTE SPARSENESS STRUCTURE OF JACOBIAN ACCORDING TO MOSS,
C  * COMPUTE GROUPING OF COLUMN INDICES (MITER = 2),
C  * SET POINTERS FOR SEGMENTS OF THE IWK/WK ARRAYS.
C IN ADDITION TO VARIABLES DESCRIBED PREVIOUSLY, SPANML USES THE
C FOLLOWING FOR COMMUNICATION..
C Y      = CURRENT SOLUTION VECTOR AT TIME TN
C YDOT   = CURRENT VALUES OF THE TIME DERIV AT TIME TN
C H,TN, EL0  CURRENT STEPSIZE ,TIME AND CONST IN YDOT.
C YH     = THE HISTORY ARRAY.  ONLY THE FIRST COLUMN, CONTAINING THE
C          CURRENT Y VECTOR, IS USED AND THE SECOND COLUMN CONTAINING
C          ITS CURRENT TIME DERIVATIVE MULTIPLIED BY H ARE USED.
C          OF DIMENSION (NYH, 1)
C SAVF   = A WORK ARRAY OF LENGTH NEQ, USED ONLY IF MOSS .NE. 0.
C          ON ENTRY IS ASSUMED TO CONTAIN THE CURRENT RESIDUAL.
C EWT    = ARRAY OF LENGTH NEQ CONTAINING (INVERTED) ERROR WEIGHTS.
C          USED ONLY IF MOSS = 2 OR IF ISTATE = MOSS = 1.
C FTEM   = A WORK ARRAY OF LENGTH NEQ, IDENTICAL TO ACOR IN THE DRIVER,
C          USED ONLY IF MOSS = 2.
C WK     = A REAL WORK ARRAY OF LENGTH LENWK, IDENTICAL TO WM IN
C          THE DRIVER.
C IWK    = INTEGER WORK ARRAY, ASSUMED TO OCCUPY THE SAME SPACE AS WK.
C LENWK  = THE LENGTH OF THE WORK ARRAYS WK AND IWK.
C ISTATC = A COPY OF THE DRIVER INPUT ARGUMENT ISTATE (= 1 ON THE
C          FIRST CALL, = 3 ON A CONTINUATION CALL).
C IYS    = FLAG VALUE FROM ODRV OR CDRV.
C IPPER  = OUTPUT ERROR FLAG WITH THE FOLLOWING VALUES AND MEANINGS..
C          0  NO ERROR.
C        -1  INSUFFICIENT INTEGER STORAGE FOR INTERNAL STRUCTURE
C            POINTERS
C        -2  INSUFFICIENT INTEGER STORAGE FOR SPANMK
C        -3  INSUFFICIENT INTEGER STORAGE TO ENTER SPANNU
C        -4  INSUFFICIENT REAL STORAGE TO ENTER SPANNU
C        -6 OTHER ERROR RETURN (SEE 1ST FEW LINES OF CODE)
C-----------------------------------------------------------------------
C
      IF(IPPER .GT. 0) GOTO 108
      IF(ALCODE .NE. 'NAGSPA')THEN
         CALL SERROR('   SPANML    ERROR-SETUP ROUTINE SPANUF WAS NOT
     1   CALLED PRIOR TO THE SPRINT ENTRY', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         IPPER = -6
         RETURN
      END IF
      IBIAN = 0
      IPIAN = IBIAN + 1
      LIWREQ = LENIWK
      LIWUSD = LENIWK
      LRWUSD = LENRWK
      LRWREQ = LENRWK
      N = NEQ(1)
      ELH = H * EL0
      NP1 = N + 1
      IPJAN = IPIAN + NP1
      IBJAN = IPJAN - 1
      IF (IPJAN+N-1 .GT. LENIWK) GO TO 260
      MOSSP1 = MOSS + 1
      GOTO (140, 70, 100, 140),MOSSP1
C
C NB: THIS COMMENTED CODE IS NOW INCLUDED IN THE SETUP ROUTINE 'SPANUF'
C
C     GOTO (30, 70, 100),MOSSP1
C
C MOSS = 0.  PROCESS USER-S IA,JA.
C
C30   KNEW = IPJAN
C     KMIN = IA(1)
C     IWK(IPIAN) = 1
C     DO 60 J = 1,N
C       KMAX = IA(J+1) - 1
C       IF (KMIN .GT. KMAX) GO TO 45
C       DO 40 K = KMIN,KMAX
C         I = JA(K)
C         SEE IF THE WORKSPACE IS LARGE ENOUGH
C         IF (KNEW .GT. LENIWK) GO TO 260
C         IWK(KNEW) = I
C         KNEW = KNEW + 1
C40       CONTINUE
C45     IF (KNEW .GT. LENIWK) GO TO 260
C50     IWK(IPIAN+J) = KNEW + 1 - IPJAN
C       KMIN = KMAX + 1
C60     CONTINUE
C     GO TO 140
C
C MOSS = 1.  COMPUTE STRUCTURE FROM USER-SUPPLIED JACOBIAN ROUTINE JAC.
C
C
 70    K = IPJAN
       IWK(IPIAN) = 1
      J = 0
 90   J = J + 1
          IF(K .GT. LENIWK) GOTO 260
          DO 75 I = 1,N
             Y(I)    = YH(I,1)
             YDOT(I) = YH(I,2)/H
 75          SAVF(I) = 0.0D0
C
          CALL JAC( NEQ, TN, Y,YDOT,H,EL0,J,IWK(IPIAN),IWK(IPJAN),SAVF)
C
          DO 80 I = 1,N
             IF(  DABS(SAVF(I)) .LE. SETH) GOTO 80
             IF (ITRACE .GT. 2) THEN
                IROW = I
                ICOL = J
                IF(ITRACE .EQ. 3)WRITE(IDEV,751)I,J
 751            FORMAT(' ROW',I8,'  COL',I8,' IS FULL')
             ENDIF
             IF(K .GT. LENIWK) GOTO 260
             IWK(K) = I
             K = K + 1
 80       CONTINUE
          IWK(IPIAN+J) = K+1 - IPJAN
       IF (J .EQ. N) GOTO 140
       GOTO 90
C
C MOSS = 2.  COMPUTE STRUCTURE FROM RESULTS OF N + 1 CALLS TO F. -------
C
 100  K = IPJAN
      IWK(IPIAN) = 1
      IPPER = 0
 105  IPPER = IPPER + 1
      J = IPPER
        IF (K .GT. LENIWK) GO TO 260
        YJ     = YH(J,1)
        YDOTJ  = YH(J,2)/H
        ERWT   = 1.0D0/EWT(J)
        DYJ    =  DSIGN(ERWT,YJ)
        Y(J)   = YJ + DYJ
        YDOT(J)= YDOT(J) + DYJ/ELH
        RETURN
C
C   FUNCTION CALL - REVERSE COMMUNICATIONS EXIT
C
108     J = IPPER
        DYJ = Y(J) - YJ
        Y(J)    = YJ
        YDOT(J) = YDOTJ
        DO 110 I = 1,N
          DQ = (FTEM(I) - SAVF(I))/DYJ
          IF ( DABS(DQ) .LE. SETH) GO TO 110
          IF (ITRACE .GT. 2) THEN
             IROW = I
             ICOL = J
             IF(ITRACE.GE.3)WRITE(IDEV,109)I,J
          ENDIF
 109      FORMAT(' ROW',I8,'  COL',I8,' IS FULL')
C         CHECK IF THE WORKSPACE IS LARGE ENOUGH
          IF (K .GT. LENIWK) GO TO 260
          IWK(K) = I
          K = K + 1
 110      CONTINUE
        IWK(IPIAN+J) = K + 1 - IPJAN
        IF(IPPER .LT. N)GOTO 105
        IPPER = 0
C
 140  CONTINUE
      NNZ = IWK(IPIAN + N) -1
      IF(ITRACE.GE.3)WRITE(IDEV,141)NNZ
 141  FORMAT(' NUMBER OF ELEMENTS IN JACOBIAN IS',I8)
      NGP = 0
      LENIGP = 0
      IPIGP = IPJAN + NNZ
      IF (MOSS .EQ. 1 .OR. MOSS .EQ. 3) GOTO 160
C
C COMPUTE GROUPING OF COLUMN INDICES (MITER = 2). ----------------------
C
      MAXG = NP1
      IPJGP = IPJAN + NNZ
      IBJGP = IPJGP - 1
      IPIGP = IPJGP + N
      IPTT1 = IPIGP + NP1
      IPTT2 = IPTT1 + N
      LREQ = IPTT2 + N - 1
      IF (LREQ .GT. LENIWK) GO TO 210
      CALL SPANMK (N, IWK(IPIAN), IWK(IPJAN), MAXG, NGP, IWK(IPIGP),
     1   IWK(IPJGP), IWK(IPTT1), IWK(IPTT2), IER)
      IF (IER .NE. 0) GO TO 210
      LENIGP = NGP + 1
 160  IPR = IPIGP + LENIGP
C
C  SET POINTERS FOR WORK ARRAYS
C
C            IPIGP                                                 L
C I     I     |                                     I       I      E
C P     P     |                                     P       K      N
C I     J      I             I                      I       E      I
C A     A      R             C                      W       E      W
C N     N      N             N                      K       P      K
C   N+1   NNZ      LIRN               LICN             8*N    5*N
C              ------------ISPIWK------------------
C
C                              SINCE IWK IS THE LAST PART OF JACPVT
      IKEEP = LENIWK + 1 - 5*N
      IPIWK = IKEEP - 8*N
      IPRWK = LENRWK + 1 - N
      ISPRWK = LENRWK - N
      ISPIWK = IPIWK - IPR
C
C WARNING : IF THE USER HAS SUPPLIED ISPLIT THEN NO CHECK IS MADE ON
C           ON THE LENGTH WORK SPACE SUPPLIED . THEREFORE IF THE USER
C           HAS NOT SUPPLIED ENOUGH THE PROGRAM IS LIKELY TO OVERWRITE
C           ITSELF.
C
      IF (ISPSET.EQ.1) THEN
         LICN = MIN (NINT(RSPLIT*ISPIWK), ISPRWK)
         LIRN = ISPIWK - LICN
         GOTO 167
      ENDIF
C
C WE CHOOSE AS MINIMUM LENGTHS FOR SPLITTING UP THE WORK ARRAYS
C             LIRN >= NNZ + N
C             LICN >= 2*NNZ
C
      IXTRAI = 3*NNZ + N - ISPIWK
      IXTRAR = 2*NNZ - ISPRWK
      IF ( IXTRAI .GT. 0 ) GOTO 220
  165 CONTINUE
      IF ( IXTRAR .GT. 0 ) GOTO 230
      IF ( IPPER .LT. 0 ) RETURN
      LICN = MIN (NINT(RSPLIT*ISPIWK), ISPRWK)
      LIRN = ISPIWK - LICN
      IF ( LICN .LT. 4*NNZ) THEN
C        LARGER REAL AND/OR REAL WORKSPACES MAY BE NEEDED
C        BUT WILL CARRY ON WITH DECOMPOSITION
         LIRN = MIN (NNZ+2*N,3*NNZ/2)
         LICN = MIN (ISPIWK-LIRN,ISPRWK)
      ENDIF
      RSPLIT = DBLE (LICN)/DBLE (ISPIWK)
  167 CONTINUE
C     WRITE(6,2121)ISPSET,LICN,NNZ
C2121  FORMAT(' ISPSET=',I3,' LICN=',I7,' NNZ=',I7)
      IRN = IPR
      ICN = IRN + LIRN
      IPA = 1
      IBA = IPA - 1
      IBICN = ICN - 1
      LRWUSD = LICN + N
      LIWUSD = LENIWK
      LRWREQ = LRWUSD
      LIWREQ = LIWUSD
C
C  COPY COLUMN POINTERS
C      THE COPY STARTING AT IPJAN WILL BE USED IN ROUTINE F01BSF
C      THE COPY STARTING AT IPICN WILL BE USED IN ROUTINE F01BRF
C
      DO 170 I=1,NNZ
        IWK(IBICN+I) = IWK(IBJAN+I)
  170 CONTINUE
      IPPER = 0
      RETURN
C
 210  IPPER = -2
      LIWREQ = LREQ
      CALL SERROR('   SPANML    LARGER WORKSPACE NEEDED FOR SPANMK USER
     1 PROVIDED (=I1) AMOUNT NEEDED IS (=I2)', 1, 2, LENIWK, LREQ, 0,
     2 0.0D0, 0.0D0)
      RETURN
C
  220 IPPER = -3
      LREQ = LENIWK + IXTRAI
      LIWREQ = LREQ
      CALL SERROR('   SPANML    - NOT ENOUGH INTEGER STORAGE PROVIDED TO
     1  ENTER SPARSE MATRIX ROUTINES , AT LEAST (=I1) UNITS OF STORE
     2  NEEDED , AMOUNT PROVIDED WAS (=I2)', 1, 2, LREQ, LENIWK, 0,
     3  0.0D0, 0.0D0)
      RETURN
  230 IPPER = -4
      LREQ = LENRWK + IXTRAR
      LRWREQ = LREQ
      CALL SERROR('   SPANML    - NOT ENOUGH REAL STORAGE PROVIDED TO
     1  ENTER SPARSE MATRIX ROUTINES , AT LEAST (=I1) UNITS OF STORE
     2  NEEDED , AMOUNT PROVIDED WAS (=I2)', 1, 2, LREQ, LENRWK, 0,
     3  0.0D0, 0.0D0)
      RETURN
C
 260  IPPER = -1
      LREQ = 2*N + 2
      LREQ = MAX (LENIWK+1,LREQ)
      LRWREQ = LREQ
      CALL SERROR('   SPANML   - NOT ENOUGH STORE PROVIDED FOR INTERNAL
     1  STORAGE POINTERS, (=I1) UNITS OF STORE NEEDED , AMOUNT PROVIDED
     2  IS (=I2)', 1, 2, LREQ, LENIWK, 0, 0.0D0, 0.0D0)
      RETURN
C=-=-=-=-=-=-=-=-=-=-=-= END OF SUBROUTINE SPANML =-=-=-=-=-=-=-=-=-=-=-
      END
      SUBROUTINE SPANMK (N,IA,JA,MAXG,NGRP,IGP,JGP,INCL,JDONE,IER)
C***********************************************************************
C
C     OLD NAME JGROUP
C
C***********************************************************************
      INTEGER N, IA, JA, MAXG, NGRP, IGP, JGP, INCL, JDONE, IER
      DIMENSION IA(*), JA(*), IGP(*), JGP(N), INCL(N), JDONE(N)
C-----------------------------------------------------------------------
C THIS SUBROUTINE CONSTRUCTS GROUPINGS OF THE COLUMN INDICES OF
C THE JACOBIAN MATRIX, USED IN THE NUMERICAL EVALUATION OF THE
C JACOBIAN BY FINITE DIFFERENCES.
C
C INPUT..
C N      = THE ORDER OF THE MATRIX.
C IA,JA  = SPARSE STRUCTURE DESCRIPTORS OF THE MATRIX BY ROWS.
C MAXG   = LENGTH OF AVAILABLE STORATE IN THE IGP ARRAY.
C
C OUTPUT..
C NGRP   = NUMBER OF GROUPS.
C JGP    = ARRAY OF LENGTH N CONTAINING THE COLUMN INDICES BY GROUPS.
C IGP    = POINTER ARRAY OF LENGTH NGRP + 1 TO THE LOCATIONS IN JGP
C          OF THE BEGINNING OF EACH GROUP.
C IER    = ERROR INDICATOR.  IER = 0 IF NO ERROR OCCURRED, OR 1 IF
C          MAXG WAS INSUFFICIENT.
C
C INCL AND JDONE ARE WORKING ARRAYS OF LENGTH N.
C-----------------------------------------------------------------------
C
      INTEGER I, J, K, KMIN, KMAX, NCOL, NG
C
      IER = 0
      DO 10 J = 1,N
 10     JDONE(J) = 0
      NCOL = 1
      DO 60 NG = 1,MAXG
        IGP(NG) = NCOL
        DO 20 I = 1,N
 20       INCL(I) = 0
        DO 50 J = 1,N
C REJECT COLUMN J IF IT IS ALREADY IN A GROUP.--------------------------
          IF (JDONE(J) .EQ. 1) GO TO 50
          KMIN = IA(J)
          KMAX = IA(J+1) - 1
          DO 30 K = KMIN,KMAX
C REJECT COLUMN J IF IT OVERLAPS ANY COLUMN ALREADY IN THIS GROUP.------
            I = JA(K)
            IF (INCL(I) .EQ. 1) GO TO 50
 30         CONTINUE
C ACCEPT COLUMN J INTO GROUP NG.----------------------------------------
          JGP(NCOL) = J
          NCOL = NCOL + 1
          JDONE(J) = 1
          DO 40 K = KMIN,KMAX
            I = JA(K)
 40         INCL(I) = 1
 50       CONTINUE
C STOP IF THIS GROUP IS EMPTY (GROUPING IS COMPLETE).-------------------
        IF (NCOL .EQ. IGP(NG)) GO TO 70
 60     CONTINUE
C ERROR RETURN IF NOT ALL COLUMNS WERE CHOSEN (MAXG TOO SMALL).---------
      IF (NCOL .LE. N) GO TO 80
      NG = MAXG
 70   NGRP = NG - 1
      RETURN
 80   IER = 1
      RETURN
      END
C=-=-=-=-=-=-=-=-=-=-=-= END OF SUBROUTINE SPANMK =-=-=-=-=-=-=-=-=-=-=-

      SUBROUTINE SPANMM ( NEQ, Y, YH, NYH, EWT, FTEM, SAVF, YDOT, WK,
     1                    JACPVT, IFJ, H, EL0, TN, IFUNC, JAC, IDAE )
C***********************************************************************
C
C     OLD NAME PREPJS
C
C***********************************************************************
      INTEGER NEQ, NYH, IFJ, IFUNC, JACPVT(*), IDAE(*), JSINFO
      DOUBLE PRECISION  Y, YH, EWT, FTEM, SAVF, YDOT, WK, H, EL0, TN,
     1                  RWORK
      INTEGER ICURR, IFSAVE, NP1, IJCNT, N, IPREP, IDSTRC
      DIMENSION NEQ(*), Y(*), YH(NYH,*), EWT(*), FTEM(*), SAVF(*),
     *   WK(*), YDOT(*)
      INTEGER IDUM1, IDUM2, IDUM3, IDUM4, IRBWRK, ISEND, LIW, MOSS,
     1        LIWUSD, LIWREQ, LRWUSD, LRWREQ, NBLOCK, NGP, NNZ
      INTEGER  MM1, NLU
      DOUBLE PRECISION  RDUM1, RDUM2
      INTEGER LENIWK, LENRWK, ISPSET
      DOUBLE PRECISION  RSPLIT, U, ETA, RGROW
      LOGICAL LBLOCK, GROW, ABORT, SETINF, COPYPT
      DOUBLE PRECISION  CCMXJ, SENS
      INTEGER MSBJ, NSLJ
      EXTERNAL JAC
C----------------------------------------------------------------------
C       DRIVING ROUTINE TO CALL EITHER THE ROUTINE SPANMLP TO DETERMINE
C       THE SPARSITRY PATTERN OF THE JACOBIAN (SPANMLP) AN /OR TO CALL
C       THE ROUTINE TO ACTUALLY FORM THE JACOBIAN.
C----------------------------------------------------------------------
      COMMON /INPHIL/ JSINFO(10)
      COMMON /SPHIL/ RWORK(50)
      COMMON /SPJCNT/ IJCNT, N
      COMMON /SPJSVE/ IFSAVE, ICURR, NP1, IPREP, SETINF
      COMMON /LSS001/ RDUM1(2), CCMXJ, RDUM2(2), SENS, IDUM1(23), MOSS,
     1                MSBJ, NSLJ, NGP, NLU, NNZ, IDUM2
      COMMON /RWB007/ LIWUSD, LIWREQ, LRWREQ, LRWUSD, RSPLIT, ISPSET
      COMMON /RWB006/ LBLOCK, GROW, ABORT(4), U, ETA, RGROW
      COMMON/ RWB005/ LENIWK, LENRWK
      COMMON /RWB003/ IDUM3(8), NBLOCK, IDUM4
      COMMON /RWB001/ IDSTRC
C RWB001
C   IDSTRC  THIS IS SET TO 0 EVERY TIME THE SPARSITY PATTERN IS
C           RECOMPUTED SO THAT SPANNU CAN USE F01BRF TO DECOMPOSE THE
C           MATRIX WHEN IT HAS BEEN FORMED. IDSTRC SET TO 1 AFTER A
C           CALL TO F01BRF SO THAT F01BSF CAN BE USED TO DECOMPOSE
C           MATRICES OF THE SAME STRUCTURE USNG INFORMATION ALREADY
C           GENERATED BY F01BRF.
         COMMON /JACVAL/ MM1 ,ISEND
         COMMON /RWB008/ COPYPT
      SAVE   /SPJCNT/, /SPJSVE/, /RWB001/, /JACVAL/, /RWB007/,
     1       /LSS001/, /RWB006/, /RWB005/, /RWB003/, /RWB008/,
     2       /SPHIL/, /INPHIL/
C
      IF (RWORK(35) .EQ. 1.0D0) THEN
         RWORK(35) = 2.0D0
         MOSS = INT (RWORK(48))
         RSPLIT = RWORK(36)
         ISPSET = INT (RWORK(37))
         U = RWORK(38)
         ETA = RWORK(39)
         LBLOCK = RWORK(42) .EQ. 1.0D0
         GROW = RWORK(41) .EQ. 1.0D0
         COPYPT = .TRUE.
         SENS = RWORK(43)
         ABORT(1) = .TRUE.
         ABORT(2) = .TRUE.
         ABORT(3) = .FALSE.
         ABORT(4) = .TRUE.
         IJCNT = 0
         SETINF = .TRUE.
         N = NEQ(1)
         MSBJ = 50
         NLU = 0
         NSLJ = 0
         CCMXJ = 0.2D0
         LENIWK = INT (RWORK(47))
         LENRWK = INT (RWORK(46))
      ENDIF
C
      IF(N .NE. NEQ(1))THEN
C        O.D.E. PROBLEM HAS CHANGED DIMENSION RECOMPUTE SPARSITY
         N = NEQ(1)
         IJCNT = 0
         SETINF = .TRUE.
      END IF
      NP1 = NEQ(1) + 2
      IF(IFUNC .NE. 0)IFSAVE = IFUNC
      IF(IFUNC .EQ. 0)THEN
         IF(ICURR .EQ. 1)GOTO 20
      ELSE
         IFJ = 0
         IF(IJCNT .GT. 0)GOTO 20
      END IF
      IDSTRC = 0
99998 CALL SPANML( NEQ, Y, YDOT, YH, NYH, SAVF, EWT, FTEM,
     1             WK, JACPVT, IFJ, H, EL0, TN, JAC )
      IPREP = 0
C           IS SET TO 1 BELOW WHEN SYMBOLIC FACTOR FAILS.
      IF(IFJ .GT. 0)THEN
         ICURR = 0
         IFUNC  = 0
         RETURN
      ELSE
         IFUNC = IFSAVE
C      IJCNT COULD BE USED TO FORCE A PERIODIC RE-EVALUATION OF SPARSITY
         IJCNT = 1
         IF(IFJ .LT. 0)THEN
            ICURR = 1
            IPREP = 1
            IFJ = -2
            RETURN
         END IF
      END IF
20    IF(IPREP .EQ. 1)THEN
C        SYMBOLIC FACTOR FAILED
         IFJ = -1
         RETURN
      END IF
C
C       CALL ROUTINE TO FORM THE JACOBIAN
C
99999 CALL SPANMJ( NEQ, Y, YH, NYH, EWT, FTEM, SAVF, YDOT, WK,
     1             JACPVT, IFJ, H, EL0, TN, IFUNC, JAC, IDAE )
      IF(IFJ .GT. 0)THEN
         ICURR = 1
         IFUNC = 0
      ELSE
         IFUNC = IFSAVE
      END IF
      IF (SETINF .AND. IFJ .LE. 0) THEN
         SETINF = .FALSE.
         JSINFO(1) = LIWREQ
         JSINFO(2) = LIWUSD
         JSINFO(3) = LRWREQ
         JSINFO(4) = LRWUSD
         IF (IFJ .EQ. 0) THEN
            JSINFO(6) = NNZ
            JSINFO(7) = NGP
            JSINFO(8) = NINT(RSPLIT*100.0D0)
            IF (LBLOCK) JSINFO(10) = NBLOCK
         ENDIF
      ENDIF
      JSINFO(5) = NLU
      IF (IFJ .EQ. 0 .AND. GROW) JSINFO(9) = NINT(RGROW)
      IF (MOSS.GT.0 .AND. MOSS.LT.3 .AND. NLU.EQ.1 .AND. IFJ.EQ.0) THEN
         IJCNT = 0
         SETINF = .TRUE.
         COPYPT = .TRUE.
C        FORCES RE-EVALUATION OF SPARSITY PATTERN
      ENDIF
      RETURN
C=-=-=-=-=-=-=-=-=-=-=-END OF SUBROUTINE SPANMM-=-=-=-=-=-=-=-=-=-=-=-=
      END
      SUBROUTINE SPANMJ ( NEQ, Y, YH, NYH, EWT, FTEM, SAVF, YDOT, WK,
     1                    IWK, IFJ, H, EL0, TN, IFUNC, JAC, IDAE )
C***********************************************************************
C
C     OLD NAME PRJS
C
C***********************************************************************
C
      INTEGER NEQ, NYH, IWK, IFJ, IFUNC, IDAE(*)
      INTEGER IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     1   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     2   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     3   NSLJ, NGP, NLU, NNZ, NSP, JPFLAG, NST
      INTEGER I, IMUL, J, JJ, JOK, JMAX, JMIN, K, KMAX, KMIN, NG, N
      INTEGER ITRACE, IDEV, IOVFLO, IROW, ICOL
      INTEGER IDSTRC
      INTEGER MM4
      DOUBLE PRECISION  Y, YH, EWT, FTEM, SAVF, WK, YDOT, H, EL0, TN
      DOUBLE PRECISION  CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH
      DOUBLE PRECISION  CON, FAC, HL0, PIJ, R, R0, RCON, RCONT,
     1   SRUR, VNORM, DUNFLO, UROUND, HL0OLD, FAC1
      CHARACTER*6 ALCODE, SINGLR
      DIMENSION NEQ(*), Y(*), YH(NYH,*), EWT(*), FTEM(*), SAVF(*),
     *   WK(*), IWK(*), YDOT(*)
      EXTERNAL JAC
      COMMON /ALGCHK/ ALCODE, SINGLR
      COMMON /SDEV2/  ITRACE, IDEV
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
      COMMON /LSS001/ CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH,
     1   IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     2   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     3   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     4   NSLJ, NGP, NLU, NNZ, NSP
C
C    COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /PRJSAV/ CON, FAC, HL0, HL0OLD, PIJ, R, R0, RCONT, SRUR,
     1   RCON, I,    JJ, JMAX, JMIN, JOK, JPFLAG, K, KMAX, KMIN, N, NG
      COMMON /RWB001/ IDSTRC
      COMMON /JACVAL/ J,MM4
      COMMON /JACPTR/ IROW, ICOL
      SAVE   /PRJSAV/, /LSS001/, /SCONS1/, /ALGCHK/, /RWB001/, /JACVAL/,
     1       /JACPTR/, /SDEV2/
C
C-----------------------------------------------------------------------
C SPANMJ IS CALLED TO COMPUTE AND PROCESS THE MATRIX DF/DYDOT - H*EL0*J
C WHERE J = DF/DY AND BOTH J AND DF/DYDOT ARE
C COMPUTED BY COLUMNS, EITHER BY THE USER-SUPPLIED ROUTINE JAC
C IF MITER = 1, OR BY FINITE DIFFERENCING IF MITER = 2.
C IF MITER = 1 OR 2, AND IF THE EXISTING VALUE OF THE JACOBIAN
C (AS CONTAINED IN P) IS CONSIDERED ACCEPTABLE, THEN A NEW VALUE OF
C P IS RECONSTRUCTED FROM THE OLD VALUE.  IN ANY CASE, WHEN MITER
C IS 1 OR 2, THE P MATRIX IS SUBJECTED TO LU DECOMPOSITION IN CDRV.
C P AND ITS LU DECOMPOSITION ARE STORED (SEPARATELY) IN WK.
C
C IN ADDITION TO VARIABLES DESCRIBED PREVIOUSLY, COMMUNICATION
C WITH SPANMJ USES THE FOLLOWING..
C Y     = ARRAY CONTAINING PREDICTED VALUES ON ENTRY.
C YDOT  = ARRAY CONTAINING PREDICTED VALUES OF TIME DERIVATIVE ON ENTRY
C FTEM  = WORK ARRAY OF LENGTH N (ACOR IN STODE).
C SAVF  = ARRAY CONTAINING F EVALUATED AT PREDICTED Y.
C WK    = REAL WORK SPACE FOR MATRICES.  ON OUTPUT IT CONTAINS THE
C         THE SPARSE LU DECOMPOSITION IF MITER IS 1 OR 2.
C         STORAGE OF MATRIX ELEMENTS STARTS AT WK(1).
C IWK   = INTEGER WORK SPACE FOR MATRIX-RELATED DATA, ASSUMED TO
C         BE EQUIVALENCED TO WK.  IN ADDITION, WK(IPRSP) AND IWK(IPISP)
C         ARE ASSUMED TO HAVE IDENTICAL LOCATIONS.
C EL0   = EL(1) (INPUT).
C IERPJ = OUTPUT ERROR FLAG (IN COMMON).
C       = 0 IF NO ERROR.
C       = 1  IF ZERO PIVOT FOUND IN CDRV.
C       = 2  IF A SINGULAR MATRIX AROSE WITH MITER = 3.
C       = -1 IF INSUFFICIENT STORAGE FOR CDRV (SHOULD NOT OCCUR HERE).
C       = -2 IF OTHER ERROR FOUND IN CDRV (SHOULD NOT OCCUR HERE).
C H     =  CURRENT STEPSIZE IN USE
C TN    =  CURRENT VALUE OF THE TIME STEP
C IFJ   =  OUTPUT ERROR FLAG
C          < 0 IF THE JACOBIAN MATRIX IS SINGULAR
C          = 0 IF THE DECOMPOSTION WENT 0.K.
C          > 0 IF THIS IS A REVERSE COMMUNICATION EXIT.
C IFUNC = INDICATOR TO DETERMINE THE MODE OF OPERATION OF THE ROUTINE
C          = 0 MEANS REVERSE COMMUNICATION ENTRY.
C          = 1 MEANS CALCULATE THE NORMAL JACOBIAN MATRIX
C THIS ROUTINE ALSO USES UROUND IN COMMON/SCONS1/
C-----------------------------------------------------------------------
      IF(IFJ .GT. 0)GOTO 215
      N = NEQ(1)
      HL0 = H*EL0
      CON = -HL0
C
C  THE JACOBIAN IS TO BE REEVALUATED. --------------------------------
C
 20   NSLJ = 0
      IPLOST = 0
      HL0OLD = HL0
      CONMIN =  DABS(CON)
      IF (MOSS .EQ. 0 .OR. MOSS .EQ. 2) GOTO 200
C
C     IF(MOSS .NE. 1)GOTO 200
C
C  MOSS = 1 - THE JACOBIAN IS TO BE FORMED FROM THE USERS ROUTINE.
C
      KMIN = IWK(IPIAN)
      J = 0
130   J = J + 1
         KMAX = IWK(IPIAN+J) -1
         DO 110 I = 1,N
            Y(I)    = YH(I,1)
            YDOT(I) = YH(I,2)/H
 110        FTEM(I) = 0.0D0
C
         CALL JAC( NEQ, TN, Y, YDOT, H, EL0, J, IWK(IPIAN), IWK(IPJAN),
     1             FTEM)
C
         DO 120 K = KMIN,KMAX
            I = IWK(IBJAN+K)
 120        WK(IBA + K ) = -FTEM(I)*(IDAE(I) + (1-IDAE(I))/(EL0*H))
         KMIN = KMAX + 1
       IF (J .EQ. N) GOTO 290
       GOTO 130
C
C IF MITER = 2, MAKE NGP CALLS TO F TO APPROXIMATE J AND P. ------------
C
 200  CONTINUE
      FAC = VNORM(N, SAVF, EWT)
      R0 = 1000.0D0 *  DABS(H) * UROUND * DBLE (N) * FAC
      IF (R0 .EQ. 0.0D0) R0 = 1.0D0
      SRUR =  DSQRT( UROUND)
      JMIN = IWK(IPIGP)
      IFJ = 0
 205  IFJ = IFJ + 1
        NG = IFJ
        JMAX = IWK(IPIGP+NG) - 1
        DO 210 J = JMIN,JMAX
          JJ = IWK(IBJGP+J)
          R =  DMAX1 (SRUR* DABS(Y(JJ)),R0/EWT(JJ))
C   THE FOLLOWING LINE BY M.BERZINS 3/12/83. REMOVE FOR HINDMARSH CODE
          R =  DMAX1 (R, UROUND)
          YDOT(JJ) = YH(JJ,2)/H + R/HL0
          Y(JJ)    = YH(JJ,1)   + R
 210    CONTINUE
        RETURN
C
C    REVERSE COMMUNICATION EXIT FOR A FUNCTION CALL
C
 215    CONTINUE
        DO 230 J = JMIN,JMAX
          JJ = IWK(IBJGP+J)
          R  = Y(JJ) - YH(JJ,1)
          Y(JJ)    = YH(JJ,1)
          YDOT(JJ) = YH(JJ,2)/H
CRWB      R =  DMAX1 (SRUR* DABS(Y(JJ)),R0/EWT(JJ))
C    THE FOLLOWING LINE BY M.BERZINS 3/12/83. REMOVE FOR HINDMARSH CODE
CRWB      R =  DMAX1 (R, UROUND)
          FAC = -HL0/R
          FAC1= -1.0D0/R
          KMIN =IWK(IBIAN+JJ)
          KMAX =IWK(IBIAN+JJ+1) - 1
CX      WRITE(6,9994) IBJAN,KMIN,KMAX,(IWK(IBJAN+KK),KK=KMIN,KMAX)
CX 9994 FORMAT(' IBJAN = ',I4,4X,'KMIN = ',I2,4X,'KMAX = ',I2/(10I5))
          DO 220 K = KMIN,KMAX
            I = IWK(IBJAN+K)
CX      WRITE(6,9995) I
CX 9995 FORMAT( ' I = ',I3)
            WK(IBA+K) = (FTEM(I)-SAVF(I))*(FAC*IDAE(I)+(1-IDAE(I))*FAC1)
 220        CONTINUE
          IF(ITRACE.GE. 3)THEN
          DO 235 K = KMIN,KMAX
            I = IWK(IBJAN+K)
            IROW = I
            ICOL = JJ
            WRITE(IDEV,219)I,JJ,WK(IBA+K)
 219        FORMAT(' DF',I3,'/DY',I3,' IS',D11.3)
 235        CONTINUE
          END IF
 230      CONTINUE
        JMIN = JMAX + 1
      IF(NG .EQ. NGP)THEN
         IFJ = 0
      ELSE
         GOTO 205
      END IF
C
C DO NUMERICAL FACTORIZATION OF P MATRIX. ------------------------------
C
 290  NLU = NLU + 1
      CON0 = CON
      IFJ = 0
      DO 295 I = 1,N
 295    FTEM(I) = 0.0D0
      SINGLR = 'NSING4'
      IF (IDSTRC.EQ.0) THEN
C        FIRST TIME THIS SPARSITY STRUCTURE HAS BEEN USED
         CALL SPANNU(N,IWK,WK,WK,1,IFJ)
      ELSEIF (IDSTRC.EQ.1) THEN
C        DECOMPOSE NEW MATRIX WITH OLD SPARSITY STRUCTURE
         CALL SPANNU(N,IWK,WK,WK,2,IFJ)
      ENDIF
      IF ( IFJ .EQ. 0 ) THEN
         IDSTRC = 1
         RETURN
      ENDIF
      CALL SPANMH(IFJ, IDSTRC, N)
      IF (IFJ .EQ. -1) SINGLR = 'SINGLR'
      IDSTRC = 0
C                 TO HANDLE ERROR MESSAGE NUMBER FROM SPANNU
      RETURN
C=-=-=-=-=-=-=-=-=-=-=-= END OF SUBROUTINE SPANMJ =-=-=-=-=-=-=-=-=-=-=-
      END
      SUBROUTINE SPANNV (WK, IWK, X, N, IER)
C***********************************************************************
C
C     OLD NAME SOLSS
C
C*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
C THIS ROUTINE MANAGES THE SOLUTION OF THE LINEAR SYSTEM ARISING FROM
C A CHORD ITERATION.
C IT CALLS SPANNU TO ACCOMPLISH THIS.
C COMMUNICATION WITH SLSS USES THE FOLLOWING VARIABLES..
C WK    = REAL WORK SPACE CONTAINING THE
C         LU DECOMPOSITION OF THE MATRIX OTHERWISE.
C         STORAGE OF MATRIX ELEMENTS STARTS AT WK(1).
C IWK   = INTEGER WORK SPACE FOR MATRIX-RELATED DATA
C X     = THE RIGHT-HAND SIDE VECTOR ON INPUT, AND THE SOLUTION VECTOR
C         ON OUTPUT, OF LENGTH N.
C IER   = OUTPUT FLAG
C         IER   = 0  IF NO TROUBLE OCCURRED.
C         IER   = -1 IF CDRV RETURNED AN ERROR FLAG (MITER = 1 OR 2).
C                    THIS SHOULD NEVER OCCUR AND IS CONSIDERED FATAL.
C N     = THE DIMENSION OF THE O.D.E. SYSTEM
C THIS ROUTINE ALSO USES OTHER VARIABLES IN COMMON.
C-----------------------------------------------------------------------
      INTEGER IWK(*), N, IER
      DOUBLE PRECISION  WK(*), X(*)
      CHARACTER*6 ALCODE, SINGLR
      COMMON /ALGCHK/ ALCODE, SINGLR
      SAVE   /ALGCHK/
      IF(SINGLR .NE. 'NSING4')THEN
         CALL SERROR(' SPANNV - ERROR BACKSUBSTITUTION TRIED WITH EITHER
     1                 SINGULAR JACOBIAN MATRIX OR NO JACOBIAN MATRIX AT
     2                 ALL OR WITH SPRINT HAVING BEEN CALLED WITH A
     3                 JACOBIAN FORMING ROUTINE OTHER THAN SPANMM', 1,
     4                 0, 0, 0, 0, 0.0D0, 0.0D0)
         IER = -1
         RETURN
      END IF
      IER = 0
      CALL SPANNU( N, IWK, WK, X, 3, IER)
      IF (IER .NE. 0) IER = -1
      RETURN
C----------------------- END OF SUBROUTINE SPANNV ----------------------
      END
      SUBROUTINE SPANNU(N,IWK,WK,RHS,PATH,IFAIL)
C-----------------------------------------------------------------------
C      THIS ROUTINE INTERFACES TO THE NAG ROUTINES :
C         F01BRF - LU DECOMPOSITION OF SPARSE MATRIX , PERMUTING TO
C                  BLOCK LOWER TRIANGULAR FORM AND USING  PARTIAL
C                  PIVOTING
C         F01BSF - USES DECOMPOSITION FROM A PREVIOUS CALL TO F01BSF
C                  TO DECOMPOSE A MATRIX OF THE SAME SPARSITY PATTERN
C         F04AXF - SOLVES THE LINEAR SYSTEMS A X = B , A(TR) X = B
C                  USING A DECOMPOSITION SUPPLIED BY THE PREVIOUS
C                  CALL TO F01BRF OR F01BSF
C
C      INPUTS :
C   N     THE NUMBER OF LINEAR EQUATIONS
C   WK    ARRAY CONTAINING SPARSE MATRIX , OR ITS DECOMPOSITION , AND
C         ALSO WORKSPACE
C   IWK   INTEGER ARRAY CONTAINING POINTERS AND OTHER PERTINENT
C         INFORMATION
C   RHS   RIGHTHAND SIDE OF LINEAR SYSTEM A X = B
C   PATH  INTEGER WHICH INDICATES WHAT ACTION IS TO BE TAKEN
C            = 1 ; DECOMPOSE A NEW SPARSITY STRUCTURE
C            = 2 ; DECOMPOSE A NEW MATRIX USING OLD SPARSITY STRUCTURE
C            = 3 ; SOLVE A(TR) X = B
C  ABORT LOGICAL ARRAY TO CONTROL RETURNS FROM F01BRF/F01BSF
C    FOR F01BRF :
C        ABORT(1) = .TRUE. ; THE ROUTINE EXITS IMMEADIATELY ON DETECTING
C                            STRUCTURAL SINGULARITY
C        ABORT(2) = .TRUE. ; THE ROUTINE EXITS IMMEADIATELY ON DETECTING
C                            NUMERICAL SINGULARITY
C        ABORT(3) = .FALSE.; ROUTINE CONTINUES IF NOT ENOUGH STORE BY
C                            DROPPING ELEMENTS FROM THE DECOMPOSITION.
C                            THEN IDISP(6:7) GIVE A GUIDE AS TO HOW MUCH
C                            STORE IS ACTUALLY REQUIRED.
C    FOR ROUTINES F01BRF AND F01BSF
C        ABORT(4) = .TRUE. ; THE ROUTINE EXITS IMMEADIATELY ON DETECTING
C                            DUPLICATE ELEMENTS
C  IFAIL ERROR MESSAGE INDICATOR
C
C  THIS ROUTINE ALSO USES OTHER VARIABLES IN COMMON
C-----------------------------------------------------------------------
      INTEGER IKEEP,IPIWK,IPRWK,IRN,LIRN,ICN,LICN
      INTEGER IWK, IER, N, PATH
      INTEGER I, LSTNTR, NENTRS
      INTEGER IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     1   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     2   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     3   NSLJ, NGP, NLU, NNZ, NSP
      DOUBLE PRECISION  WK, RHS, RLSS, U
      INTEGER IDISP(10)
      INTEGER ISPIWK, ISPRWK
      DOUBLE PRECISION  ETA, RPMIN, RESID, RGROW
      INTEGER ITRACE,IDEV
      LOGICAL LBLOCK,GROW,ABORT(4),COPYPT
      DIMENSION WK(*), IWK(*), RHS(*)
      COMMON /LSS001/ RLSS(6),
     1   IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     2   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     3   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     4   NSLJ, NGP, NLU, NNZ, NSP
      COMMON /RWB002/ IKEEP,IPIWK,IPRWK,IRN,LIRN,ICN,LICN,ISPRWK,ISPIWK
      COMMON /RWB003/ IDISP
      COMMON /RWB006/ LBLOCK, GROW, ABORT, U, ETA, RGROW
C RWB003
C   IDISP(1) AND IDISP(2) INDICATE THE FIRST AND LAST ELEMENTS OF THE
C   DECOMPOSITION OF THE DIAGONAL BLOCKS (ICN+IDISP(1) AND ICN+IDISP(2))
C   IN IWK). THEY MUST NOT BE CHANGED BETWEEN A CALL OF F01BRF AND CALLS
C   TO F01BSF AND F04AXF.
C   IDIDP(3) AND IDISP(4) GIVE THE NUMBER IF TIMES THE DATA HAS BEEN
C   COMPRESSED DURING THE DECOMPOSITION TO RELEASE TO MORE STORE.
C   IDISP(5) IS AN UPPER BOUND OF THE MATRIX
C   IDISP(6) AND IDISP(7) GIVE THE MINIMUM SIZES OF LIRN AND LICN
C   RESPECTIVELY  (NOTE WITHOUT 'ELBOW ROOM').
C   IF (LBLOCK) THEN IDISP(8:10) USED
C   IDISP(8) IS STRUCTURAL RANK OF THE MATRIX
C   IDISP(9) GIVES THE THE NUMBER OF DIAGONAL BLOCKS.
C   IDISP(10) IS THE SIZE OF THE LARGEST DIAGONAL BLOCK.
      COMMON /SDEV2/ ITRACE,IDEV
      COMMON /RWB008/ COPYPT
      SAVE   /LSS001/, /RWB002/, /RWB003/, /RWB006/, /SDEV2/, /RWB008/

C     CHECK FOR VALID PATH SPECIFICATION
      IF (PATH.LT.1 .OR. PATH.GT.3) THEN
         IFAIL = -3
         RETURN
      ENDIF
      IF (PATH.EQ.3) GOTO 40

      IF (COPYPT) THEN
C        WRITE(6,99999)
C9999    FORMAT(' ***  EXPANDING ROW POINTERS  ***')
C
C EXPAND ROW POINTERS
C     INTO IWK(IRN,...,IRN+NNZ-1)  ( FROM IWK(IPJAN,...,IPJAN+N) )
C       THIS ORDER IS DESTROYED BY F01BRF AND IS NOT REQUIRED AFTER
C       HENCE THIS OPERATION IS REPEATED BEFORE CALLING F01BSF
C
         LSTNTR = IRN + NNZ
         IBIAN = IPIAN - 1
         DO 20 I=N,1,-1
           NENTRS = IWK(IPIAN+I) - IWK(IBIAN+I)
           DO 10 J=1,NENTRS
             IWK(LSTNTR-J) = I
   10        CONTINUE
           LSTNTR = LSTNTR-NENTRS
   20    CONTINUE
      ENDIF
C
C  WE SUPPRESS NAG ERROR DIAGNOSTICS FROM F01BRF/F01BSF AND THEREFORE
C     MAKE OUR OWN CHECKS BEFORE ENTERING THESE ROUTINES :
C      N <= 0 ?
C      NNZ <= 0 ?
C      LIRN < NNZ ?
C      LICN < NNZ ?
C      ROW/COL POINTERS OUTSIDE RANGE 1,N ?
C      DUPLICATE ELEMNTS ?
C
      IFAIL = 0
      CALL SPANNT(N, NNZ, LICN, LIRN, IWK(IRN), IWK(IPJAN), IWK(IPIWK),
     1            IWK(IPIAN), WK(IPA), IFAIL)
C
      IF (IFAIL.GT.0) RETURN
      IFAIL = 1
      IF (PATH.EQ.2) GOTO 30
C
C**** LU DECOMP WITH NEW SPARSITY USING F01BRF
C
      CALL F01BRF ( N, NNZ, WK(IPA), LICN, IWK(IRN), LIRN, IWK(ICN),
     1              U, IWK(IKEEP), IWK(IPIWK), WK(IPRWK), LBLOCK,
     2              GROW, ABORT, IDISP, IFAIL)

      IF (GROW) RGROW = WK(IPRWK)
      IF (IFAIL .EQ. 0) CALL SPANNS
      RETURN
C
C**** LU DECOMP WITH OLD SPARSITY USING F01BSF
C
   30 CONTINUE
      WK(IPRWK) = 0.0D0
      CALL F01BSF ( N, NNZ, WK(IPA), LICN, IWK(IRN), IWK(IPJAN),
     1              IWK(ICN), IWK(IKEEP), IWK(IPIWK), WK(IPRWK), GROW,
     2              ETA, RPMIN, ABORT(4), IDISP, IFAIL)
C
      IF (GROW) RGROW = WK(IPRWK)
      COPYPT = .FALSE.
      RETURN
C
C**** SOLVE A(TR) X = B  USING F04AXF
C
   40 CONTINUE
      IFAIL = 0
C               SINCE THERE ARE NO ERROR RETURNS FROM F04AXF
      CALL F04AXF ( N, WK(IPA), LICN, IWK(ICN), IWK(IKEEP), RHS,
     1              WK(IPRWK), 0, IDISP, RESID)
C
      RETURN
C=-=-=-=-=-=-=-=-=-=- END OF SUBROUTINE SPANNU =-=-=-=-=-=-=-=-=-=-=-=-
      END
      SUBROUTINE SPANMH (IER, IDSTRC, N)
C**********************************************************************
      INTEGER IER, IDSTRC, IDISP(10), NEWIWK, LIRN, LICN
      INTEGER LRWREQ, LIWREQ, LRWUSD, LIWUSD, LENIWK, LENRWK, ISPSET
      INTEGER ICN, IKEEP, IPIWK, IPRWK, IRN, ISPIWK, ISPRWK, I1, N
      DOUBLE PRECISION  RSPLIT
      COMMON /RWB002/ IKEEP,IPIWK,IPRWK,IRN,LIRN,ICN,LICN,ISPRWK,ISPIWK
      COMMON /RWB007/ LIWUSD, LIWREQ, LRWREQ, LRWUSD, RSPLIT, ISPSET
      COMMON /RWB005/ LENIWK, LENRWK
      COMMON /RWB003/ IDISP
      COMMON /RWB004/ I1
C RWB004
C   I1  AN INTEGER SUPPLIED ROUTINE SPANNT
C       THE I1TH ELEMENT HAS ROW/COL INDICES OUT OF RANGE
C    OR THE I1TH ROW CONTAINS A DUPLICATE ELEMENT
      SAVE /RWB002/, /RWB003/, /RWB004/, /RWB005/, /RWB007/
C***********************************************************************
C     ROUTINE TO PRINT ERROR MESSAGE CORRESPONDING TO THE ERROR NUMBER
C     RETURNED BY THE SPARSE MATRIX DRIVER SPANNU.
C     INPUTS :
C        IDSTRC = 0 INDICATES ERROR FROM F01BRF
C               = 1 INDICATES ERROR FROM F01BSF
C        IER THE ERROR MESSAGE NUMBER
C
C     IER = -3 ; ILLEGAL PATH SPECIFICATION
C     IDSTRC = 0  IE. ERROR FROM F01BRF
C      IER = -2 ; SUCCESSFUL DECOMPOSITION OF NUMERICALLY SINGULAR
C                 MATRIX. THIS SHOULD NOT HAPPEN SINCE WE SET ABORT(2)O
C                 TO .TRUE. . ON DETECTION OF NUMERICAL SINGULARITY
C                 F01BRF SHOULD EXIT WITH IER = 2.
C          = -1 ; SUCCESSFUL DECOMPOSITION OF STRUCTURALLY SINGULAR
C                 MATRIX. THIS SHOULD NOT HAPPEN SINCE WE SET ABORT(1)
C                 TO .TRUE. . ON DETECTION OF STRUCTURAL SINGULARITY
C                 F01BRF SHOULD EXIT WITH IER = 1.
C          =  0 ; NO ERRORS HAVE BEEN DETECTED. THIS ROUTINE SHOULD NOT
C                 BE ENTERED WITH THIS VALUE , HENCE THIS IS CONSIDERED
C                 FATAL.
C          =  1 ; THE MATRIX IS STRUCTURALLY SINGULAR AND THE DECOMPOSI-
C                 TION HAS BEEN ABANDONED.
C          =  2 ; THE MATRIX IS NUMERICALLY SINGULAR AND THE DECOMPOSIT-
C                 ION HAS BEEN ABANDONED.
C          =  3 ; LIRN IS TO SMALL: THERE IS NOT ENOUGH SPACE TO
C                 CONTINUE THE DECOMPOSITION. THE USER IS RECOMMENDED TO
C                 TRY AGAIN WITH LIRN = IDISP(6)+N/2+1, IE. WE ADVISE
C                 THE USER TO EXTEND THE INTEGER WORKSPACE AND MAYBE THE
C                 REAL WORKSPACE AS A RESULT.
C          =  4 ; LICN IS MUCH TOO SMALL: USER MUST EXTEND INTEGER AND/O
C                 REAL SPACE(S).
C          =  5 ; LICN IS TOO SMALL; USER MUST EXTEND THE INTEGER AND/OR
C                 REAL SPACE(S).
C ( THE SIZE OF LICN MAY BE GOVERNED BY THE FACT THE USER SUPPLIED TOO
C   LITTLE REAL WORKSPACE , HENCE FOR THE ABOVE TWO ERROR RETURNS THE
C   USER MAY HAVE TO EXTEND BOTH WORKSPACES )
C          =  6 ; LICN AND LIRN ARE BOTH TOO SMALL.
C          =  7 ; LICN IS TOO SMALL: THERE IS NOT ENOUGH SPACE TO PERMUT
C                 PERMUTE TO BLOCK TRIANGULAR FORM. THE USER MUST SUPPLY
C                 MORE INTEGER AND/OR REAL SPACE(S).
C          =  8 ; ON ENTRY , N <= 0 .
C          =  9 ; ON ENTRY , NNZ <= 0.
C          = 10 ; ON ENTRY , LICN < NNZ.
C          = 11 ; ON ENTRY , LIRN < NNZ.
C          = 12 ; ON ENTRY , AN ELEMENT OF THE INPUT MATRIX HAS A ROW
C                 OR COLUMN INDEX OUTSIDE THE RANGE 1 TO N.
C          = 13 ; ON ENTRY , DUPLICATE ELEMENTS HAVE BEEN FOUND IN THE
C                 INPUT MATRIX.
C ( THE ABOVE SIX ERROR RETURNS ARE CHECKED FOR IN THE ROUTINE SPANNT.
C   THIS IS SO AS TO ENABLE US TO SUPPRESS NAG ERROR DIAGNOSTICS BUT TO
C   SUPPLY THE SAME INFORMATION IN OUR OWN FORMAT )
C
C     IDSTRC = 1 ; IE. ERROR RETURN FROM F01BSF
C      IER =  0 ; SAME AS FOR IDSTRC = 0
C          =  1 ; SAME AS FOR IDSTRC = 0 , IER = 8
C          =  2 ; SAME AS FOR IDSTRC = 0 , IER = 9
C          =  3 ; SAME AS FOR IDSTRC = 0 , IER = 10
C          =  4 ; SAME AS FOR IDSTRC = 0 , IER = 12
C          =  5 ; THE INPUT MATRIX IS INCOMPATIBLE WITH THE MATRIX
C                 DECOMPOSED BY THE PREVIOUS CALL TO F01BRF.
C          =  6 ; THE INPUT MATRIX IS NUMERICALLY SINGULAR.
C          =  7 ; A VERY SMALL PIVOT HAS BEEN DETECTED ( |.| < ETA )
C                 THE DECOMPOSITION HAS BEEN COMPLETED BUT IS
C                 POTENTIALLY UNSTABLE.
C          =  8 ; SAME AS FOR IDSTRC = 0 , IER = 13
C
C
C***********************************************************************
C
C     WRITE(6,31)IDSTRC,IER
C31   FORMAT(' IDSTRC=',I3,' IER=',I3)
      IF (IER.EQ. -3) THEN
      CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     SPANNU WAS ENTERED WITH AN ILLEGAL PATH SPECIFICATION',
     2     1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1020
      ENDIF
      IF (IDSTRC .EQ. 1) GOTO 32
      IER = IER + 3
C     IER  -2 -1  0  1  2  3  4  5  6  7  8  9  10  11 12 13
      GOTO(10,10,10,20,30,40,50,50,50,50,10,10,100,110,60,70),IER
  32  IER = IER + 1
      GOTO(10,10,10,10,60,10,30,90,70),IER
   10 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU-
     1     BUT THE ERROR MESSAGE PARAMETER HAD AN ILLEGAL VALUE - THE
     2     FORM OF THE ERROR IS NOT KNOWN-TRY INCREASING THE WORKSPACES
     3     OR CHANGING THE VALUE OF ISPLIT',1,0, 0, 0, 0, 0.0D0, 0.0D0)
      WRITE(IDEV,21)IDSTRC
 21   FORMAT(' ILLEGAL ERROR-PROBABLY WORK AREA OVERWRITE IDSTRC=',I3)
      GOTO 1020
   20 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE JACOBIAN MATRIX WAS FOUND TO BE STRUCTURALLY SINGULAR',
     2     1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1000
   30 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE JACOBIAN MATRIX WAS FOUND TO BE NUMERICALLY SINGULAR',
     2     1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1000
   40 NEWIWK = NINT(1.0/(1.0-RSPLIT)*(IDISP(6)+N/2+1))
      LIWREQ = LENIWK - ISPIWK + NEWIWK
      LRWREQ = N + NINT(RSPLIT*NEWIWK)
      CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE ROUTINE F01BRF DID NOT HAVE ENOUGH STORAGE - USEFUL
     2     INFORMATION SUPPLIED BY CALLING ROUTINE SPANYF',
     3     1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1020
   50 LIWREQ = MAX (NINT(1.1*ISPIWK), NINT(1.5*(IDISP(6)+IDISP(7))))
      LIWREQ = LIWREQ + LENIWK - ISPIWK
      LRWREQ = N + MAX (NINT(1.1*LICN), NINT(1.5*IDISP(7)))
      CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE ROUTINE F01BRF DID NOT HAVE ENOUGH STORAGE - INFORMATION
     2     SUPPLIED BY A CALL TO ROUTINE SPANYF NOT USEFUL',
     3     1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1020
   60 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE F01BRF/F01BSF FOUND THE (=I1)TH ELEMENT OF THE JACOBIAN
     2     WITH A ROW/COL INDEX OUTSIDE SPECIFIED RANGE OF 1 TO N',
     3     1, 1, I1, 0, 0, 0.0D0, 0.0D0)
      GOTO 1020
   70 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE ROUTINE FO1BRF/F01BSF FOUND DUPLICATE ELEMENT IN THE
     2     (=I1)TH ROW OF THE JACOBIAN MATRIX - RUN ABANDONED',
     3     1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1020
   90 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE ROUTINE F01BSF HAS DETECTED A VERY SMALL PIVOT -
     2     ASSUMING SINGULARITY',
     3     1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1000
  100 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE VALUE OF THE USER INPUT - ISPLIT - WAS TOO LARGE TRY
     2     DECREASING ISPLIT AND PERHAPS INCREASING THE SIZE OF THE
     3     INTEGER WORKSPACE ', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      GOTO 1020
  110 CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED BY ROUTINE SPANNU -
     1     THE VALUE OF THE USER INPUT - ISPLIT - WAS TOO SMALL TRY
     2     INCREASING ISPLIT AND PERHAPS INCREASING THE SIZE OF THE
     3     INTEGER WORKSPACE ', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
 1020 CONTINUE
      IER = -2
      RETURN
 1000 CONTINUE
      IER = -1
      RETURN
      END
C=-=-=-=-=-=-=-=-=-=- END OF SUBROUTINE SPANMH =-=-=-=-=-=-=-=-=-=-=-=-=
C
      SUBROUTINE SPANNS
C
C-----------------------------------------------------------------------
C     THIS ROUTINE, AFTER A CALL TO F01BRF, GIVES THE USER ADVICE ON HOW
C     HE MIGHT AID BETTER USE OF THE WORKSPACES PROVIDED, OR IMPROVE
C     PERFORMANCE BY INCREASING THE SIZE OF THE WORKSPACES. TO DO THIS W
C     WE LOOK AT THE NUMBER OF COMPRESSIONS CARRIED OUT DURING THE
C     DECOMPOSITION. FOR THE ACTUAL SIZES OF THE SPACES SUGGESTED
C     THE USER IS ADVISED TO CALL ROUTINE SPANYF ON EXITING SPRINT AND
C     EXAMINE THE ARRAY JSINFO.
C     ISPIWK LENGTH OF INTEGER STORE AVAILABLE FOR THE ARRAYS IRN AND
C            ICN IN THE CALL TO F01BRF
C     ISPRWK LENGTH OF REAL STORE AVAILABLE FOR THE ARRAY A IN THE CALL
C            TO F01BRF
C     LICN   LENGTH OF ICN AND A
C     LIRN   LENGTH OF IRN
C     MINICN MIN. LENGTHS OF A AND ICN (IE. NO 'ELBOW ROOM')
C     MINIRN MIN. LENGTH OF IRN (IE. NO 'ELBOW ROOM')
C     LENIWK LENGTH OF INTEGER STORE SUPPLIED
C     LENRWK LENGTH OF REAL STORE SUPPLIED
C
C
C     COMPRESSIONS  ADVICE
C     ROW   COL
C     =0    =0      IF 5*(MINIRN+MINICN)/2 < ISPIWK THEN DECREASE LENIWK
C                   IF 5*MINICN/2 < ISPRWK THEN DECREASE LENRWK
C     >0    >0      INCREASE LENIWK
C                   AND MAYBE INCREASE LENRWK AS A RESULT OF ABOVE
C     >4     0      INCREASE LENIWK
C                   AND ALTER ISPLIT TO RETAIN SAME LICN
C     <=4    0      WE WUOLD LIKE TO INCREASE LIRN BY 10%
C                   IF THIS WOULD NOT TAKE US BELOW 150% OF MINICN THEN
C                      IF USER DIDN'T SET ISPLIT THEN ALTER ISPLIT
C                      ELSE INCREASE LENIWK
C                   ELSE INCREASE LENIWK AND ALTER ISPLIT
C      0    >4      IF LICN RESTRICTED BY ISPRWK
C                      INCREASE LENRWK
C                      AND MAYBE INCREASE LENIWK AS A RESULT OF ABOVE
C                      ALTER ISPLIT
C                   ELSE INCREASE LENIWK
C                        AND MAYBE INCREASE LENRWK AS A RESULT OF ABOVE
C                        ALTER ISPLIT
C      0    <=4     WE WOULD LIKE TO INCREASE LICN BY 10%
C                   IF THIS WOULD NOT TAKE US BELOW 150% OF MINIRN THEN
C                      IF USER DIDN'T SET ISPLIT THEN ALTER ISPLIT
C                      ELSE INCREASE LENIWK
C                   ELSE INCREASE LENIWK
C                        AND MAYBE INCREASE LENRWK AS A RESULT OF ABOVE
C                        ALTER ISPLIT
C
C     ANOTHER CHECK IS MADE ON THE NUMBER OF DIAGONAL BLOCKS. IF THIS IS
C      ONE THEN F01BRF WASTED TIME TO PERMUTE AN IRREDUCIBLE MATRIX TO
C      BLOCK LOWER TRIANGULAR FORM AND THE USER IS ADVISED TO SET LBLOCK
C      TO .FALSE. FOR RUNS ON A SIMILAR PROBLEM
C
C-----------------------------------------------------------------------
      INTEGER LENIWK, LENRWK, LIWUSD, LIWREQ, LRWUSD, LRWREQ, ISPSET,
     1        IKEEP, IPIWK, IPRWK, IRN, LIRN, LICN, ICN, ISPRWK, ISPIWK
      INTEGER IXTRAI, IXTRAR, ITRACE, IDEV
      DOUBLE PRECISION  U, ETA, RGROW, RSPLIT
      LOGICAL ABORT(4), LBLOCK, GROW, OKPRNT
      INTEGER I1,I2,IRCMPS,ICCMPS,I5,MINIRN,MINICN,I8,NBLOKS,I10
C
      COMMON /RWB005/ LENIWK, LENRWK
      COMMON /RWB006/ LBLOCK, GROW, ABORT, U, ETA, RGROW
      COMMON /RWB007/ LIWUSD, LIWREQ, LRWREQ, LRWUSD, RSPLIT, ISPSET
      COMMON /RWB002/ IKEEP, IPIWK, IPRWK, IRN, LIRN, ICN, LICN,
     1                ISPRWK, ISPIWK
      COMMON /RWB003/ I1, I2, IRCMPS, ICCMPS, I5, MINIRN, MINICN,
     1                I8, NBLOKS, I10
      COMMON /SDEV2/  ITRACE, IDEV
C
      SAVE /RWB005/, /RWB006/, /RWB007/, /RWB002/, /RWB003/, /SDEV2/
C
      OKPRNT = ITRACE .GT. 2
C
      IF (IRCMPS.EQ.0 .AND. ICCMPS.EQ.0) THEN
C
         IXTRAI= MIN (0,5*(MINICN+MINIRN)/2+1-ISPIWK)
         IF (IXTRAI.LT.0 .AND. OKPRNT) CALL SPANNR(1)
         IXTRAR= MIN (0,5*MINICN/2+1-ISPRWK)
         IF (IXTRAR.LT.0 .AND. OKPRNT) CALL SPANNR(2)
         LRWREQ = LENRWK + IXTRAR
         LIWREQ = LENIWK + IXTRAI
         GOTO 100
C
      ELSEIF (IRCMPS.GT.0 .AND. ICCMPS.GT.0) THEN
C
         IXTRAI= MAX (NINT(1.1*ISPIWK),NINT(1.5*(MINIRN+MINICN)))-ISPIW
         IF (OKPRNT) CALL SPANNR(3)
         IXTRAR= MAX (0,LICN+NINT(RSPLIT*IXTRAI)-ISPRWK)
         IF (IXTRAR.GT.0 .AND. OKPRNT) CALL SPANNR(4)
C
      ELSEIF (IRCMPS.GT.4 .AND. ICCMPS.EQ.0) THEN
C
         IXTRAI= MAX (NINT(1.1*LIRN),NINT(1.5*MINIRN))-LIRN
         IF (OKPRNT) CALL SPANNR(3)
         RSPLIT=DBLE (LICN)/(ISPIWK+IXTRAI)
         IF (OKPRNT) CALL SPANNR(5)
C
      ELSEIF (IRCMPS.LE.4 .AND. ICCMPS.EQ.0) THEN
C
         IXTRAI=NINT(0.1*LIRN)
         IF (LICN-IXTRAI.GT.3*MINICN/2+1) THEN
            IF (ISPSET.EQ.0) THEN
               RSPLIT=DBLE (LICN-IXTRAI)/ISPIWK
               IF (OKPRNT) CALL SPANNR(6)
               IXTRAI = 0
            ELSE
               IXTRAI=NINT(0.1*(LICN+LIRN))
               IF (OKPRNT) CALL SPANNR(3)
            ENDIF
         ELSE
            IF (OKPRNT) CALL SPANNR(3)
            RSPLIT=DBLE (LICN)/(ISPIWK+IXTRAI)
            IF (OKPRNT) CALL SPANNR(5)
         ENDIF
C
      ELSEIF (IRCMPS.EQ.0 .AND. ICCMPS.GT.4) THEN
C
         IF (LICN.EQ.ISPRWK) THEN
            IXTRAR= MAX (NINT(1.1*ISPRWK),NINT(1.5*MINICN))-ISPRWK
            IF (OKPRNT) CALL SPANNR(7)
            IXTRAI= MAX (0,LICN+IXTRAR+3*MINIRN/2+1-ISPIWK)
            IF (IXTRAI.GT.0 .AND. OKPRNT) CALL SPANNR(8)
            RSPLIT = DBLE (LICN+IXTRAR)/(ISPIWK+MAX (0,IXTRAI))
            IF (OKPRNT) CALL SPANNR(5)
         ELSE
            IXTRAI= MAX (NINT(1.1*LICN),NINT(1.5*MINICN))-LICN
            IF (OKPRNT) CALL SPANNR(3)
            IXTRAR= MAX (0,LICN+IXTRAI-ISPRWK)
            IF (IXTRAR.GT.0 .AND. OKPRNT) CALL SPANNR(4)
            RSPLIT=DBLE (LICN+IXTRAI)/(ISPIWK+IXTRAI)
            IF (OKPRNT) CALL SPANNR(5)
         ENDIF
C
      ELSEIF (IRCMPS.EQ.0 .AND. ICCMPS.LE.4) THEN
C
         IXTRAI=NINT(0.1*LICN)
         IF (LIRN-IXTRAI.GT.3*MINIRN/2+1) THEN
            IF (ISPSET.EQ.0) THEN
               RSPLIT = DBLE (IXTRAI+LICN)/ISPIWK
               IF (OKPRNT) CALL SPANNR(6)
               IXTRAI = 0
            ELSE
               IXTRAI=NINT(0.1*(LIRN+LICN))
               IF (OKPRNT) CALL SPANNR(3)
            ENDIF
         ELSE
            IF (OKPRNT) CALL SPANNR(3)
            RSPLIT = DBLE (IXTRAI+LICN)/(ISPIWK+IXTRAI)
            IF (OKPRNT) CALL SPANNR(5)
            IXTRAR= MAX (0,LICN+IXTRAI-ISPRWK)
            IF (IXTRAR.GT.0 .AND. OKPRNT) CALL SPANNR(4)
         ENDIF
      ENDIF
C
      LIWREQ = LIWUSD + IXTRAI
      LRWREQ = LRWUSD + IXTRAR
C
  100 IF (LBLOCK .AND. NBLOKS.EQ.1 .AND. OKPRNT) CALL SPANNR(9)
C
      RETURN
C=-=-=-=-=-=-=-=-=-=-= END OF SUBROUTINE SPANNS =-=-=-=-=-=-=-=-=-=-=-=-
      END
      SUBROUTINE SPANNR(ITHINK)
C***********************************************************************
      INTEGER ITHINK, ITRACE, IDEV
      COMMON /SDEV2/ ITRACE, IDEV
      SAVE   /SDEV2/
      GOTO(1, 2, 3, 4, 5, 6, 7, 8, 9), ITHINK
    1 WRITE(IDEV,10)
      GOTO 100
    2 WRITE(IDEV,20)
      GOTO 100
    3 WRITE(IDEV,30)
      GOTO 100
    4 WRITE(IDEV,40)
      GOTO 100
    5 WRITE(IDEV,50)
      GOTO 100
    6 WRITE(IDEV,60)
      GOTO 100
    7 WRITE(IDEV,70)
      GOTO 100
    8 WRITE(IDEV,80)
      GOTO 100
    9 WRITE(IDEV,90)
  100 CONTINUE
      RETURN
   10 FORMAT('    SPANNS     - THE USER COULD SUPPLY LESS INTEGER'/
     1'     WORKSPACE , PROBABLY WITHOUT AFFECTING THE PERFORMANCE')
   20 FORMAT('    SPANNS     - THE USER COULD SUPPLY LESS REAL'/
     1'     WORKSPACE , PROBABLY WITHOUT AFFECTING THE PERFORMANCE')
   30 FORMAT('    SPANNS     - PERFORMANCE WOULD PROBABLY IMPROVE'/
     1'     IF MORE INTEGER WORKSPACE WERE PROVIDED')
   40 FORMAT('    SPANNS     - AND MORE REAL WORKSPACE WERE'/
     1'     PROVIDED')
   50 FORMAT('    SPANNS     - AND THE STORAGE DIVISION PARAMETER'/
     1'     ISPLIT WERE ALTERED')
   60 FORMAT('    SPANNS     - PERFORMANCE WOULD PROBABLY IMPROVE'/
     1'     IF ISPLIT WERE ALTERED')
   70 FORMAT('    SPANNS     - PERFORMANCE WOULD PROBABLY IMPROVE'/
     1'     IF MORE REAL WORKSPACE WERE PROVIDED')
   80 FORMAT('    SPANNS     - AND MORE INTEGER WORKSPACE WERE'/
     1'     PROVIDED')
   90 FORMAT('    SPANNS     - THE JACOBIAN MATRIX IS IRREDUCIBLE ,'/
     1'     IT WOULD PROBABLY BE MORE EFFICIENT ON RUNS OF A SIMILAR'/
     2'     PROBLEM TO SET LBLOCK = .FALSE.')
C=-=-=-=-=-=-=-=-=-=- END OF SUBROUTINE SPANNR =-=-=-=-=-=-=-=-=-=-=-=-=
      END
      SUBROUTINE SPANNT(N,NZ,LICN,LIRN,IA,JA,IWK,IAMY,A,IFAIL)
C-----------------------------------------------------------------------
C     IA    ROW POINTERS
C     IAMY  COMPACT ROW POINTERS
C     JA    COLUMN POINTERS
C-----------------------------------------------------------------------
      INTEGER N,NZ,LICN,LIRN,IA(NZ),JA(NZ),IAMY(N+1),IWK(N)
      DOUBLE PRECISION  A(NZ)
      INTEGER I, ICOL, I1, J, K, KEND, IDSTRC
      COMMON /RWB001/ IDSTRC
      COMMON /RWB004/ I1
      SAVE /RWB001/, /RWB004/
C
      IF (N.GT.0) GOTO 20
      IFAIL = 1 + (1-IDSTRC)*7
      RETURN
C
   20 IF (NZ.GT.0) GOTO 30
      IFAIL = 2 + (1-IDSTRC)*7
      RETURN
C
   30 IF (LIRN.GE.NZ) GOTO 40
      IFAIL = 3 + (1-IDSTRC)*7
      RETURN
C
   40 IF (LICN.GE.NZ) GOTO 45
      IFAIL = 11
      RETURN
C
C CHECK THAT ROW/COL INDICES ARE WITHIN THE RANGE 1 TO N
C
   45 DO 50 I = 1,NZ
        IF (IA(I).GT.0.AND.IA(I).LE.N.AND.JA(I).GT.0.AND.JA(I).LE.N)
     1                              GOTO 50
           I1 = I
           IFAIL = 4 + (1-IDSTRC)*8
           RETURN
   50 CONTINUE
C
C CHECK FOR DUPLICATE ELEMENTS
C
      K = IAMY(1)
      DO 100 I = 1,N
        KEND = IAMY(I+1)
        DO 120 J = 1,N
          IWK(J) = 0
  120   CONTINUE
        DO 130 J = K,KEND-1
          ICOL = JA(J)
          IF (IWK(ICOL).GT.0) THEN
             I1 = I
             IFAIL = 8 + (1-IDSTRC)*5
             RETURN
          ELSE
             IWK(ICOL) = 1
          ENDIF
  130  CONTINUE
       K = KEND
  100 CONTINUE
      RETURN
C=-=-=-=-=-=-=-=-=-=- END OF SUBROUTINE SPANNT -=-=-=-=-=-=-=-=-=-=-=-=-
      END
