      SUBROUTINE BLDSET( NY2DIM, IRET, MAXORD)
      INTEGER NY2DIM, MAXORD,  IRET
C----------------------------------------------------------------------
C   THIS ROUTINE INITIALISES THE LSODI IMPLENTATION OF ADAMS/B.D.F
C   BLEND METHOD AS USED IN TH SPRINT INTEGRATION PACKAGE
C   THE INPUT PARAMETERS ARE:
C   NY2DIM , THE SECOND DIMENSION OF THE ARRAY YSAVE(NYH,NY2DIM)
C            THIS VALUE MUST BE SET TO MAXORD + 3, UNLESS MAXORD = 0
C            (SEE BELOW ).
C   MAXORD   THE MAXIMUM ORDER TO BE USED BY THE BLEND METHOD.
C            THIS VALUE MUST BE POSITIVE AND LESS THAN 12 FOR BLEND.
C            IF MAXORD IS ZERO THE DEFAULT VALUE OF 11 IS USED, AND
C            NY2DIM MUST BE LARGE ENOUGH.
C   OUTPUT PARAMETER;
C   IRET       INTEGER FLAG SET TO ZERO IF ALL INPUTS ARE O.K.
C              OTHERWISE SET TO -1  TO INDICATE AN ERROR EXIT
C----------------------------------------------------------------------
C
      INTEGER IOWNS, NMETH, NMXORD, IDM, I
      DOUBLE PRECISION ROWNS,CCMAX
      CHARACTER*6 ODCODE
C
      COMMON /ODECHK/ ODCODE
      COMMON /BL0001/ ROWNS(355), IOWNS(8), NMETH, NMXORD, IDM(3)
      COMMON /SPCCMX/ CCMAX
      SAVE   /ODECHK/, /BL0001/, /SPCCMX/
      IRET = 0
      NMETH = 2
      CCMAX = 0.3D0
C       IDM(2) CONTAINS THE PARAMETER WHICH SPECIFIES HOW MANY
C       CONVERGENCE FAILURES ARE ALLOWED.
C
       IDM(2) = 5
       IF (MAXORD.LT.0)THEN
          CALL SERROR(' BLDSET NEGTIVE VALUE OF MAXORD (=I1) IS ILLEGAL'
     1    , 1, 1, MAXORD, 0, 0, 0.0D0, 0.0D0)
          IRET = -1
       END IF
       IF ( MAXORD.EQ.0)THEN
            MAXORD = 11
       ELSE
            IF(MAXORD .GT. 11)THEN
            CALL SERROR(' BLDSET WARNING USER SUPPLIED MAXORD (=I1) IS
     1 GREATER THAN ALLOWED (=I2) AND IS RESET TO THE ALLOWED MAX',
     2 1, 2, MAXORD, 11, 0, 0.0D0, 0.0D0)
            END IF
            MAXORD = MIN0(MAXORD, 11)
       END IF
       NMXORD = MAXORD
       IF( NY2DIM .LT. (MAXORD+3))THEN
          I = MAXORD + 3
          CALL SERROR(' BLDSET - ERROR SECOND DIMENSION OF YSAVE ARRAY
     1                  ,NY2DIM (=I1) IS SMALLER THAN REQUIRED (=I2),
     2                  INCREASE THE ARRAY DIMENSION ', 1, 2, NY2DIM,
     3                  I, 0 , 0.0D0, 0.0D0)
          IRET = -1
       ELSE
          ODCODE = 'SBLEND'
       END IF
       RETURN
C------END-OF-ROUTINE-BLDSET-------------------------------------------
       END
      SUBROUTINE SBLEND(NEQ, Y, YH, NYH, EWT, YDOT, SAVR, ACOR, INLN,
     1                  ISTEP, EL0, H, TN, HMIN, HMXI, IDAE)
      INTEGER NEQ, NYH, INLN, ISTEP, IDAE(1)
      INTEGER IALTH, IPUP, LMAX, MEO,  NSLP, ICF, JCUR,
     1   L, METH, MAXORD, MSBP, MXNCF, N, IOVFLO,
     2   NQ, NST, NRE, NJE, NQU, NITER, JSTART, KFLAG
      INTEGER I,JC,IREDO, IRET, J, JB, KGO, NCF, NEWQ,ITRACE,IDEV,IZ
      INTEGER MAXIT, NT, NINTER, IOMEGA, IOMEGB, M, MAXCOR, IFLAG
      DOUBLE PRECISION Y, YH, EWT, YDOT, SAVR, ACOR, EL0, H, TN, HMXI,
     1   HMIN, ROWND, DUN, U
      DOUBLE PRECISION CONIT, AL, ALCO, HOLD, RMAX, TESCO, CCMAX, RC
      DOUBLE PRECISION DDN,  DUP, DSN, BL, BLCO, BLJH,
     1   ALJH,  EXDN, EXSM, EXUP, C, GAMMA,
     2   R, RH, RHDN, RHSM, RHUP, TOLD, VNORM, WB, RMWB
      DOUBLE PRECISION DAMP, RJNORM, CCMX1
      DOUBLE PRECISION ELSAVE, DEL, DELP, CRATE, EL1H, EL, DCON, BND
      CHARACTER*6 ODCODE
      DIMENSION NEQ(1),Y(1),YH(NYH,1),EWT(1),YDOT(1),SAVR(1),ACOR(1)
      COMMON /BL0001/ ROWND, CONIT, AL(13), ALCO(13,12), BL(12),
     1   BLCO(12,11), HOLD, RMAX, TESCO(3,12), CCMAX, RC,
     2   IALTH, IPUP, LMAX, MEO, NSLP,
     3   ICF, JCUR, L, METH, MAXORD, MSBP, MXNCF, N
      COMMON /ODECHK/ ODCODE
      COMMON /SCONS1/ DUN, U, IOVFLO
      COMMON /LSTATS/ NQ, NQU, NST, NRE, NJE, NITER, NINTER
      COMMON /SSOLVR/ DAMP, RJNORM, CRATE, MAXIT
      COMMON /SPCCMX/ CCMX1
C
C      EXTRA COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /BLSAVE/  DDN, DSN,  DUP, ALJH, BLJH, ELSAVE, DEL,
     1   DELP, EL1H, EL, DCON, C(11), GAMMA(11), RMWB, WB, BND,
     2   EXDN, EXSM, EXUP, R, RH, RHDN, RHSM, RHUP, TOLD,
     3   I, JC, IREDO, IRET, J, JB, KGO, NCF, IZ, JSTART, KFLAG,
     4   IOMEGA, IOMEGB, M ,MAXCOR, IFLAG, NT
      COMMON /SDEV2/  ITRACE,IDEV
      SAVE   /BLSAVE/, /ODECHK/, /LSTATS/, /SSOLVR/, /BL0001/,
     1       /SCONS1/, /SPCCMX/
C-----------------------------------------------------------------------
C SBLEND PERFORMS ONE STEP OF THE INTEGRATION OF AN INITIAL VALUE
C PROBLEM FOR A SYSTEM OF ORDINARY DIFFERENTIAL EQUATIONS.
C NOTE..SBLEND IS INDEPENDENT
C OF THE TYPE OF CHORD METHOD USED, OR THE JACOBIAN STRUCTURE.
C COMMUNICATION WITH SBLEND IS DONE WITH THE FOLLOWING VARIABLES..
C
C NEQ    = INTEGER ARRAY CONTAINING PROBLEM SIZE IN NEQ(1), AND
C          NOT USED EXCEPT TO INITIALISE THE COMMON BLOCK VARIABLE N
C Y      = AN ARRAY OF LENGTH .GE. N USED AS THE Y ARGUMENT IN
C          ALL CALLS TO RES, JAC, AND ADDA. ON THE FIRST CALL THIS
C          ARRAY IS ASSUMED TO CONTAIN THE INITIAL SOLUTION VALUES.
C YH     = AN NYH BY LMAX ARRAY CONTAINING THE DEPENDENT VARIABLES
C          AND THEIR APPROXIMATE SCALED DERIVATIVES, WHERE
C          LMAX = MAXORD + 3.  YH(I,J+1) CONTAINS THE APPROXIMATE
C          J-TH DERIVATIVE OF Y(I), SCALED BY H**J/FACTORIAL(J)
C          (J = 0,1,...,NQ).  YH(I,LMAX+2) AND YH(I,LMAX+3) ARE WORK
C          ARRAYS USED IN SOLVING THE NONLINEAR SYSTEM OF EQUATIONS.
C NYH    = A CONSTANT INTEGER .GE. N, THE FIRST DIMENSION OF YH.
C EWT    = AN ARRAY OF LENGTH N CONTAINING MULTIPLICATIVE WEIGHTS
C          FOR LOCAL ERROR MEASUREMENTS.  LOCAL ERRORS IN Y(I) ARE
C          COMPARED TO 1.0/EWT(I) IN VARIOUS ERROR TESTS.
C YDOT   = AN ARRAY OF WORKING STORAGE, OF LENGTH N. ALSO USED FOR
C          INPUT OF YH(*,MAXORD+2) WHEN JSTART = -1 AND MAXORD IS LESS
C          THAN THE CURRENT ORDER NQ. ON THE FIRST CALL THIS ARRAY IS
C          ASSUMED TO HOLD THE INITIAL VALUES OF THE TIME DERIVATIVE.
C SAVR   = AN ARRAY OF WORKING STORAGE, OF LENGTH N.
C          THIS ARRAY IS NOT USED IN THE PRESENT IMPLEMENTATION.
C ACOR   = A WORK ARRAY OF LENGTH N USED FOR THE ACCUMULATED
C          CORRECTIONS. ON A SUCCESFUL RETURN, ACOR(I) CONTAINS
C          THE ESTIMATED ONE-STEP LOCAL ERROR IN Y(I).
C CCMAX  = MAXIMUM RELATIVE CHANGE IN H*EL0 BEFORE PJAC IS CALLED.
C H      = THE STEP SIZE TO BE ATTEMPTED ON THE NEXT STEP.
C          H IS ALTERED BY THE ERROR CONTROL ALGORITHM DURING THE
C          PROBLEM.  H CAN BE EITHER POSITIVE OR NEGATIVE, BUT ITS
C          SIGN MUST REMAIN CONSTANT THROUGHOUT THE PROBLEM.
C HMIN   = THE MINIMUM ABSOLUTE VALUE OF THE STEP SIZE H TO BE USED.
C HMXI   = INVERSE OF THE MAXIMUM ABSOLUTE VALUE OF H TO BE USED.
C          HMXI = 0.0 IS ALLOWED AND CORRESPONDS TO AN INFINITE HMAX.
C          HMIN AND HMXI MAY BE CHANGED AT ANY TIME, BUT WILL NOT
C          TAKE EFFECT UNTIL THE NEXT CHANGE OF H IS CONSIDERED.
C IDAE   = INDICATOR ARRAY OF DIMENSION NEQ(1). IDAE(J) = 0 IF THE JTH
C          EQUTAION IS ALGEBRAIC AND 1 OTHERWISE.
C TN     = THE INDEPENDENT VARIABLE. TN IS UPDATED ON EACH STEP TAKEN.
C ISTEP    : INPUT AND OUTPUT ERROR INDICATOR
C          ON INPUT ITS VALUES ARE TRANSFERRED INTO JSTART
C          ON OUTPUT IT TAKES THE VALUES OF KFLAG . IN BOTH CASES
C          THE ACTUAL VALUES ARE MODIFIED SO AS TO CORRESSPOND TO
C          THE SPRINT INTERFACE.
C    INPUT:    -1  PERFORM THE FIRST STEP  (JSTART = 0).
C               0  REVERSE COMMUNICATION RETURN FROM NLSLVR ROUTINE.
C               1  TAKE A NEW STEP CONTINUING FROM THE LAST (JSTART=1)
C               2  TAKE THE NEXT STEP WITH A NEW VALUE OF H, MAXORD,
C                    N, METH, (JSTART = -1).
C               3  TAKE THE NEXT STEP WITH A NEW VALUE OF H,
C                    BUT WITH OTHER INPUTS UNCHANGED (JSTART = -2)
C          ON RETURN, ISTEP  IS SET TO 1 TO FACILITATE CONTINUATION.
C    OUTPUT:    1  THE STEP WAS SUCCESSFUL , KFLAG = 0.
C               0  REVERSE COMMUNICATION RETURN - CHECK INLN.
C              -1  THE REQUESTED ERROR COULD NOT BE ACHIEVED.
C              -2  CORRECTOR CONVERGENCE COULD NOT BE ACHIEVED.
C              -3  RES ORDERED IMMEDIATE RETURN.
C              -4  ERROR CONDITION FROM RES COULD NOT BE AVOIDED.
C              -5  FATAL ERROR IN JACOBIAN FORMING OR BACKSUBSTITUTION.
C              -6  INIT MODULE LSET WAS NOT CALLED PRIOR TO FIRST ENTRY.
C              -7  WORKSPACE ERROR OCCURED IN THE NONLINEAR SOLVER .
C APART FROM WHEN ISTEP = 0 OR 1 , ISTEP = KFLAG ON OUTPUT.
C          A RETURN WITH ISTEP = -1, -2, OR -4 MEANS EITHER
C          ABS(H) = HMIN OR 10 CONSECUTIVE FAILURES OCCURRED.
C          ON A RETURN WITH ISTEP NEGATIVE, THE VALUES OF TN AND
C          THE YH ARRAY ARE AS OF THE BEGINNING OF THE LAST
C          STEP, AND H IS THE LAST STEP SIZE ATTEMPTED.
C MAXORD = THE MAXIMUM ORDER OF INTEGRATION METHOD TO BE ALLOWED.
C MSBP   = MAXIMUM NUMBER OF STEPS BETWEEN PJAC CALLS.
C MXNCF  = MAXIMUM NUMBER OF CONVERGENCE FAILURES ALLOWED.
C METH   = THE METHOD FLAG.  SEE DESCRIPTION IN ROUTINE BLSET.
C N      = THE NUMBER OF FIRST-ORDER DIFFERENTIAL EQUATIONS.
C
C INLN       = REVERSE COMMUNICATION INDICATOR
C
C       ON ENTRY   INLN  = 0  : NORMAL ENTRY
C                        = -1 : ERROR IN JACOBIAN FORMATION
C                        = -2 : RESID ORDERED RETURN TO USER
C                        = -3 : RESID FOUND ILLEGAL T, Y, OR YDOTI.
C                        = -4 : RESID ORDERED ENTRY TO MONITR.
C                        = -5 : WORKSPACE ERROR IN LINEAR ALGEBRA.
C                        = 1  : NONLINEAR SYSTEM SOLVED
C                        = 2  : ITERATION FAILED TO CONVERGE IN
C                               SOLUTION OF NONLINEAR SYSTEM
C                        = 3,4  NOT USED IN THIS MODULE
C                        = 5    RETURN TO FORM THE PETZOLD ERROR EST
C                               I.E. J INVERSE * DF/DYDOT * ACOR
C
C       ON EXIT    INLN  = 0  : NORMAL EXIT
C                        = 1  : FORM JACOBIAN AND SOLVE NONLINEAR SYSTEM
C                        = 2  : AS FOR INLN = 1 BUT USING OLD JACOBIAN
C                        = 3,4 NOT USED  HERE.
C                        = 5   FORM PETZOLD ERROR ESTIMATE.
C                        = 6,7 NOT USED HERE.
C-----------------------------------------------------------------------
C
C     JUMP IF IT IS REVERSE COMMUNICATION
C
      IZ = INLN  + 5
      INLN  = 0
      GOTO ( 430, 430, 430, 430, 10, 232, 410, 240, 245, 247),IZ
      GOTO 435
10    KFLAG = 0
      IF(ODCODE .NE. 'SBLEND')THEN
         CALL SERROR(' SBLEND- SETUP MODULE LSET WAS NOT CALLED PRIOR TO
     1   ENTRY TO SPRINT- ERROR', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         ISTEP = -6
         RETURN
      END IF
      TOLD = TN
      NCF = 0
      JCUR = 0
      ICF = 0
      IF(ISTEP .EQ. 1)THEN
         JSTART = 1
         GOTO 200
      ELSE IF(ISTEP .EQ. 2)THEN
              JSTART = -1
              GOTO 100
           ELSE IF(ISTEP .EQ. 3)THEN
                   JSTART = -2
                   GOTO 160
      END IF
      JSTART = 0
C-----------------------------------------------------------------------
C ON THE FIRST CALL, THE ORDER IS SET TO 1, AND OTHER VARIABLES ARE
C INITIALIZED.  RMAX IS THE MAXIMUM RATIO BY WHICH H CAN BE INCREASED
C IN A SINGLE STEP.  IT IS INITIALLY 1.E4 TO COMPENSATE FOR THE SMALL
C INITIAL H, BUT THEN IS NORMALLY EQUAL TO 10.  IF A FAILURE
C OCCURS (IN CORRECTOR CONVERGENCE OR ERROR TEST), RMAX IS SET AT 2
C FOR THE NEXT INCREASE.
C-----------------------------------------------------------------------
      LMAX = MAXORD + 1
      NINTER = LMAX
      IOMEGA = LMAX + 1
      IOMEGB = LMAX + 2
      N = NEQ(1)
      NQ = 1
      L = 2
      IALTH = 2
      RMAX = 10000.0D0
      RC = 0.0D0
      EL0 = 1.0D0
      MSBP = 20
      CCMAX = 0.3D0
      HOLD = H
      MEO = METH
      NSLP = 0
      IPUP = 1
      IRET = 3
C
C   LOAD THE INITIAL VALUES OF Y AND YDOT INTO THE NORDSIECK VECTOR.
C
      DO 20 I = 1,N
         YH(I,1) = Y(I)
         YH(I,2) = YDOT(I)*H
 20   CONTINUE
      GO TO 140
C-----------------------------------------------------------------------
C THE FOLLOWING BLOCK HANDLES PRELIMINARIES NEEDED WHEN JSTART = -1.
C IPUP IS SET TO 1 TO FORCE A MATRIX UPDATE.
C IF AN ORDER INCREASE IS ABOUT TO BE CONSIDERED (IALTH = 1),
C IALTH IS RESET TO 2 TO POSTPONE CONSIDERATION ONE MORE STEP.
C IF THE CALLER HAS CHANGED METH, CFODE IS CALLED TO RESET
C THE COEFFICIENTS OF THE METHOD.
C IF THE CALLER HAS CHANGED MAXORD TO A VALUE LESS THAN THE CURRENT
C ORDER NQ, NQ IS REDUCED TO MAXORD, AND A NEW H CHOSEN ACCORDINGLY.
C IF H IS TO BE CHANGED, YH MUST BE RESCALED.
C IF H OR METH IS BEING CHANGED, IALTH IS RESET TO L = NQ + 1
C TO PREVENT FURTHER CHANGES IN H FOR THAT MANY STEPS.
C-----------------------------------------------------------------------
 100  IPUP = 1
      LMAX = MAXORD + 1
      NINTER = LMAX
      IOMEGA = LMAX + 1
      IOMEGB = LMAX + 2
      IF(NEQ(1) .LT. N)THEN
C        NUMBER OF O.D.E.S HAS BEEN REDUCED- ZERO PART OF MEMORY VECTOR
         J = NEQ(1) + 1
         DO 105 I = J,N
            DO 105 JB = 1,NQ+1
               YH(I,JB) = 0.0D0
105      CONTINUE
      ELSE IF(NEQ(1) .GT. N .AND. ITRACE .GE. 1) THEN
              CALL SERROR(' SBLEND- VALUE OF NEQ(1) (=I1) GT N(=I2)', 1,
     1             2, NEQ(1), N, 0, 0.0D0, 0.0D0)
      END IF
      N = NEQ(1)
      IF (IALTH .EQ. 1) IALTH = 2
      IF (METH .EQ. MEO) GO TO 110
      CALL CFODE (METH, ALCO, BLCO, TESCO, GAMMA, C)
      MEO = METH
      IF (NQ .GT. MAXORD) GO TO 120
      IALTH = L
      IRET = 1
      GO TO 150
 110  IF (NQ .LE. MAXORD) GO TO 160
 120  NQ = MAXORD
C
C   MAXORD HAS BEEN REDUCED TO BELOW NQ
C
      J = MAXORD + 2
      L = LMAX
      DO 125 I = 1,L
        BL(I) = BLCO(I,NQ)
 125    AL(I) = ALCO(I,NQ)
      RC = RC*C(NQ)/EL0
      EL0 = C(NQ)
      CONIT = 0.5D0/DFLOAT(NQ+2)
      DDN = VNORM (N, YH(1,J), EWT)/TESCO(1,L)
      EXDN = 1.0D0/DFLOAT(L)
      RHDN = 1.0D0/(1.3D0*DDN**EXDN + 0.0000013D0)
      RH = DMIN1(RHDN,1.0D0)
      IREDO = 3
      IF (H .EQ. HOLD) GO TO 170
      RH = DMIN1(RH,DABS(H/HOLD))
      H = HOLD
      GO TO 175
C-----------------------------------------------------------------------
C CFODE IS CALLED TO GET ALL THE INTEGRATION COEFFICIENTS FOR THE
C CURRENT METH.  THEN THE EL VECTOR AND RELATED CONSTANTS ARE RESET
C WHENEVER THE ORDER NQ IS CHANGED, OR AT THE START OF THE PROBLEM.
C-----------------------------------------------------------------------
 140  CALL CFODE (METH, ALCO, BLCO, TESCO, GAMMA, C)
 150  DO 155 I = 1,L
        BL(I) = BLCO(I,NQ)
        AL(I) = ALCO(I,NQ)
 155  CONTINUE
      RC = RC*C(NQ)/EL0
      EL0 = C(NQ)
      CONIT = 0.5D0/DFLOAT(NQ+2)
      GO TO (160, 170, 200), IRET
C-----------------------------------------------------------------------
C IF H IS BEING CHANGED, THE H RATIO RH IS CHECKED AGAINST
C RMAX, HMIN, AND HMXI, AND THE YH ARRAY RESCALED.  IALTH IS SET TO
C L = NQ + 1 TO PREVENT A CHANGE OF H FOR THAT MANY STEPS, UNLESS
C FORCED BY A CONVERGENCE OR ERROR TEST FAILURE.
C-----------------------------------------------------------------------
 160  IF (H .EQ. HOLD) GO TO 200
      RH = H/HOLD
      H = HOLD
      IREDO = 3
      GO TO 175
 170  RH = DMAX1(RH,HMIN/DABS(H))
 175  RH = DMIN1(RH,RMAX)
      RH = RH/DMAX1(1.0D0,DABS(H)*HMXI*RH)
      R = 1.0D0
      DO 180 J = 2,L
        R = R*RH
        DO 180 I = 1,N
 180      YH(I,J) = YH(I,J)*R
      H = H*RH
      RC = RC*RH
      IALTH = L
      IF (IREDO .EQ. 0) GO TO 690
C-----------------------------------------------------------------------
C THIS SECTION COMPUTES THE PREDICTED VALUES BY EFFECTIVELY
C MULTIPLYING THE YH ARRAY BY THE PASCAL TRIANGLE MATRIX.
C RC IS THE RATIO OF NEW TO OLD VALUES OF THE COEFFICIENT  H*EL(1).
C WHEN RC DIFFERS FROM 1 BY MORE THAN CCMAX, IPUP IS SET TO 1
C TO FORCE PJAC TO BE CALLED.
C IN ANY CASE, PJAC IS CALLED AT LEAST EVERY MSBP STEPS.
C-----------------------------------------------------------------------
 200  IF(DABS(RC-1.0D0) .GT. CCMX1 .OR. NST .GE. (NSLP+MSBP)) IPUP = 1
      TN = TN + H
      DO 215 JB = NQ,1,-1
         DO 215 JC = JB,NQ
           DO 210 I = 1,N
 210          YH(I,JC) = YH(I,JC) + YH(I,JC+1)
 215  CONTINUE
C-----------------------------------------------------------------------
C UP TO MAXCOR CORRECTOR ITERATIONS ARE TAKEN.  A CONVERGENCE TEST IS
C MADE ON THE R.M.S. NORM OF EACH CORRECTION, WEIGHTED BY H AND THE
C ERROR WEIGHT VECTOR EWT.  THE SUM OF THE CORRECTIONS IS ACCUMULATED
C IN ACOR(I).  THE YH ARRAY IS NOT ALTERED IN THE CORRECTOR LOOP.
C-----------------------------------------------------------------------
 220  DO 230 I = 1,N
        YDOT(I) = YH(I,2) / H
        Y(I)    = YH(I,1)
 230  CONTINUE
      M = 0
C     BND = 0.5D0/(14 * N)
      BND = 0.5D0/((NQ+2)*N)
C
C       TUNING NOTE .... THE PARAMETER 14 ABOVE AND IN DEFN OF DCON
C       COULD PROB BE IMPROVED UPON.......
      ALJH = AL(2)/H
      BLJH = BL(2)/H
      IF (IPUP .LE. 0) GOTO 232
C
C        EXIT TO FORM A NEW JACOBIAN WITHOUT PERFORMING ITERATIONS
C
      IPUP = 0
      ELSAVE = H * EL0
      MAXCOR = MAXIT
      MAXIT  = 0
      CRATE = 0.7D0
      RC = 1.0D0
      NSLP = NST
      INLN = 1
      JCUR = 1
      ISTEP = 0
      RETURN
232   INLN = 3
      MAXIT = MAXCOR
C----------------------------------------------------------------------
C        EXIT FOR A FUNCTION CALL, ITERATION LOOPING POINT.
C----------------------------------------------------------------------
      ISTEP = 0
      RETURN
240   INLN = 4
      ISTEP = 0
C
C       EXIT FOR A BACKSUBSTITUTION ONLY.
C
      RETURN
245   CONTINUE
      DO 246 I = 1,N
246      ACOR(I) = -SAVR(I)/ EL0
C
C       EXIT FOR A FUNCTION CALL AND A BACKSUBSTITUTION
C
      INLN = 5
      ISTEP = 0
      IFLAG = 1
      RETURN
C
C     START ITERATING WITH JACOBIAN MATRIX
C
 247  CONTINUE
      IF(M .EQ. 0)THEN
         DO 260 I = 1,N
            YH(I,IOMEGA) = 0.0D0
 260        YH(I,IOMEGB) = 0.0D0
      END IF
      NT = N
      DO 248 I = 1,N
         SAVR(I) = SAVR(I) / (EL0 * H)
         ACOR(I) = GAMMA(NQ)*H * (ACOR(I) + SAVR(I))/ELSAVE
         YH(I,IOMEGA) = YH(I,IOMEGA) - SAVR(I)
         YH(I,IOMEGB) = YH(I,IOMEGB) + ACOR(I)
         YDOT(I)  = YH(I,2)/H + ALJH *YH(I,IOMEGA) + BLJH *YH(I,IOMEGB)
         Y(I)     = YH(I,1)   + AL(1)*YH(I,IOMEGA) + BL(1)*YH(I,IOMEGB)
         SAVR(I)  = ACOR(I) - SAVR(I)
C        ORIGINAL GEAR-TYPE CONVERGENCE TEST
         IF(DABS(SAVR(I)*EWT(I)) .LT. BND)NT = NT -1
 248  CONTINUE
      IF(ITRACE .GE. 2)WRITE(IDEV,2481)(SAVR(I), I = 1,N)
2481  FORMAT(' SAVR=', 5D11.3)
      IF(NT .EQ. 0) GOTO 450
      DEL = VNORM (N, SAVR, EWT) * (NQ+2) / 0.5D0
C-----------------------------------------------------------------------
C TEST FOR CONVERGENCE.  IF M.GT.0, AN ESTIMATE OF THE CONVERGENCE
C RATE CONSTANT IS STORED IN CRATE, AND THIS IS USED IN THE TEST.
C-----------------------------------------------------------------------
      IF (M .GE. 1)THEN
          CRATE = DEL/DELP
          DCON = DEL*DMIN1(1.0D0,1.5D0*CRATE)
          IF(ITRACE.GE.2)WRITE(IDEV,390) DCON, CRATE, EL1H, DEL
 390  FORMAT(' DCON=',D11.3,' CRTE=',D11.3,' EL0H=',D11.3,' DEL=',D11.3)
          IF (DCON .LE. 1.0D0) GO TO 450
          IF (M .GE. MAXCOR .OR. DEL .GT. 0.9D0*DELP) GO TO 410
      END IF
      M = M + 1
      DELP = DEL
C     REVERSE COMMUNICATION RETURN FOR ANOTHER ITERATION
      GOTO 232
C-----------------------------------------------------------------------
C THE CORRECTOR ITERATION FAILED TO CONVERGE IN MAXCOR TRIES, OR ELSE
C RES HAS RETURNED ABNORMALLY. IF INLN = -3 OR -4, RETRACT THE YH ARRAY
C TO ITS VALUES BEFORE PREDICTION AND RETURN. OTHERWISE--
C IF THE JACOBIAN IS OUT OF DATE, PJAC IS CALLED FOR
C THE NEXT TRY.  OTHERWISE THE YH ARRAY IS RETRACTED TO ITS VALUES
C BEFORE PREDICTION, AND H IS REDUCED, IF POSSIBLE.  IF H CANNOT BE
C REDUCED OR MXNCF FAILURES HAVE OCCURRED, EXIT WITH KFLAG = -2 OR -4.
C-----------------------------------------------------------------------
 410  ICF = 1
      IF (JCUR .EQ. 1) GO TO 430
      IPUP = 1
      GO TO 220
 430  ICF = 2
      MAXIT = MAXCOR
      NCF = NCF + 1
      RMAX = 2.0D0
 435  TN = TOLD
      DO 445 JB = NQ,1,-1
        DO 445 JC = JB,NQ
           DO 440 I = 1,N
 440         YH(I,JC) = YH(I,JC) - YH(I,JC+1)
 445  CONTINUE
      IF (IZ .EQ. 1 .OR. IZ .EQ. 3) THEN
          KFLAG = -3
          GOTO 720
      ELSE IF(IZ .EQ. 0)THEN
          KFLAG = -7
C         WORKSPACE ERROR IN LINEAR ALGEBRA
          GOTO 720
      END IF
      IF (DABS(H) .LE. HMIN*1.00001D0) GO TO 675
      IF (NCF .EQ. MXNCF)THEN
          IF (IZ .EQ. 2)THEN
              KFLAG = -4
              GOTO 680
          ELSE IF(IZ .EQ. 4)THEN
              GOTO 685
          ELSE
              GOTO 670
          END IF
      END IF
      RH = 0.25D0
      IPUP = 1
      IREDO = 1
      GO TO 170
C-----------------------------------------------------------------------
C THE CORRECTOR HAS CONVERGED.  JCUR IS SET TO 0
C TO SIGNAL THAT THE JACOBIAN INVOLVED MAY NEED UPDATING LATER.
C THE LOCAL ERROR TEST IS MADE AND CONTROL PASSES TO STATEMENT 500
C IF IT FAILS.
C-----------------------------------------------------------------------
 450  JCUR = 0
      DO 4502 I = 1,N
4502     ACOR(I) = YH(I,IOMEGA) + YH(I,IOMEGB)
      DSN =     VNORM (N, ACOR, EWT)/TESCO(2,NQ)
      IF(ITRACE .GE.1)WRITE(IDEV,9502)DSN
 9502 FORMAT(' SCALED LOCAL ERROR ESTIMATE =',D11.3)
      IF(DSN .GT. 1.0D0) GO TO 500
C
C-----------------------------------------------------------------------
C AFTER A SUCCESSFUL STEP, UPDATE THE YH ARRAY.
C CONSIDER CHANGING H IF IALTH = 1.  OTHERWISE DECREASE IALTH BY 1.
C IF IALTH IS THEN 1 AND NQ .LT. MAXORD, THEN ACOR IS SAVED FOR
C USE IN A POSSIBLE ORDER INCREASE ON THE NEXT STEP.
C IF A CHANGE IN H IS CONSIDERED, AN INCREASE OR DECREASE IN ORDER
C BY ONE IS CONSIDERED ALSO.  A CHANGE IN H IS MADE ONLY IF IT IS BY A
C FACTOR OF AT LEAST 1.1.  IF NOT, IALTH IS SET TO 3 TO PREVENT
C TESTING FOR THAT MANY STEPS.
C-----------------------------------------------------------------------
      IF(RMAX .EQ. 1.0D+4)THEN
C        FIRST STEP - SEE IF INIT STEP WAS TOO SMALL
         EXSM = 1.0D0/DFLOAT(L)
         RHSM = 1.0D0/(1.2D0*DSN**EXSM + 0.0000012D0)
         IF(RHSM .GE. RMAX .AND. (H*HMXI) .LT. (1.D0-8.D0*U) )THEN
C           RETAKE THE STEP AS THE INITIAL STEP WAS FAR TOO SMALL
            RH = RMAX
            TN = TOLD
            DO 460 JB = NQ , 1 ,-1
               DO 460 JC = JB,NQ
                   DO 455 I = 1,N
 455                  YH(I,JC) = YH(I,JC) - YH(I,JC+1)
 460        CONTINUE
            IREDO = 3
            GOTO 170
         END IF
      END IF
      KFLAG = 0
      IREDO = 0
      NQU = NQ
      NCF = 0
      DO 470 J = 1,L
        ALJH = AL(J)
        BLJH = BL(J)
        DO 470 I = 1,N
 470      YH(I,J) = YH(I,J) + ALJH*YH(I,IOMEGA) + BLJH*YH(I,IOMEGB)
      IALTH = IALTH - 1
      IF (IALTH .EQ. 0) GO TO 520
      IF (IALTH .GT. 1) GO TO 700
      IF (L .EQ. LMAX) GO TO 700
      DO 490 I = 1,N
 490    YH(I,LMAX) = ACOR(I)
      GO TO 700
C-----------------------------------------------------------------------
C THE ERROR TEST FAILED.  KFLAG KEEPS TRACK OF MULTIPLE FAILURES.
C RESTORE TN AND THE YH ARRAY TO THEIR PREVIOUS VALUES, AND PREPARE
C TO TRY THE STEP AGAIN.  COMPUTE THE OPTIMUM STEP SIZE FOR THIS OR
C ONE LOWER ORDER.  AFTER 2 OR MORE FAILURES, H IS FORCED TO DECREASE
C BY A FACTOR OF 0.2 OR LESS.
C-----------------------------------------------------------------------
 500  KFLAG = KFLAG - 1
      TN = TOLD
      DO 515 JB = NQ , 1 ,-1
        DO 515 JC = JB,NQ
           DO 510 I = 1,N
 510          YH(I,JC) = YH(I,JC) - YH(I,JC+1)
 515    CONTINUE
      RMAX = 2.0D0
      IF (DABS(H) .LE. HMIN*1.00001D0) GO TO 650
      IF (KFLAG .LE. -4) GO TO 660
      IREDO = 2
      RHUP = 0.0D0
      GO TO 540
C-----------------------------------------------------------------------
C REGARDLESS OF THE SUCCESS OR FAILURE OF THE STEP, FACTORS
C RHDN, RHSM, AND RHUP ARE COMPUTED, BY WHICH H COULD BE MULTIPLIED
C AT ORDER NQ - 1, ORDER NQ, OR ORDER NQ + 1, RESPECTIVELY.
C IN THE CASE OF FAILURE, RHUP = 0.0 TO AVOID AN ORDER INCREASE.
C THE LARGEST OF THESE IS DETERMINED AND THE NEW ORDER CHOSEN
C ACCORDINGLY.  IF THE ORDER IS TO BE INCREASED, WE COMPUTE ONE
C ADDITIONAL SCALED DERIVATIVE.
C-----------------------------------------------------------------------
 520  RHUP = 0.0D0
      IF (L .EQ. LMAX) GO TO 540
      DO 530 I = 1,N
 530    YDOT(I) = ACOR(I) - YH(I,LMAX)
      DUP =     VNORM (N, YDOT, EWT)/TESCO(3,NQ)
      EXUP = 1.0D0/DFLOAT(L+1)
      RHUP = 1.0D0/(1.4D0*DUP**EXUP + 0.0000014D0)
 540  EXSM = 1.0D0/DFLOAT(L)
      RHSM = 1.0D0/(1.2D0*DSN**EXSM + 0.0000012D0)
      RHDN = 0.0D0
      IF (NQ .EQ. 1) GO TO 560
      DDN = VNORM (N, YH(1,L), EWT)/TESCO(1,NQ)
      EXDN = 1.0D0/DFLOAT(NQ)
      RHDN = 1.0D0/(1.3D0*DDN**EXDN + 0.0000013D0)
C560  IF(ITRACE .GE. 1)THEN
C        WRITE(IDEV,561)RHDN,RHSM,RHUP
C561     FORMAT(' DOWN',D11.3,' SAME',D11.3,' UP',D11.3)
C     END IF
 560  IF (RHSM .GE. RHUP) GO TO 570
      IF (RHUP .GT. RHDN) GO TO 590
      GO TO 580
 570  IF (RHSM .LT. RHDN) GO TO 580
      NEWQ = NQ
      RH = RHSM
      GO TO 620
 580  NEWQ = NQ - 1
      RH = RHDN
      IF (KFLAG .LT. 0 .AND. RH .GT. 1.0D0) RH = 1.0D0
      GO TO 620
 590  NEWQ = L
      RH = RHUP
      IF (RH .LT. 1.1D0) GO TO 610
      R =     AL(L)/DFLOAT(L)
      DO 600 I = 1,N
 600    YH(I,NEWQ+1) = ACOR(I)*R
      GO TO 630
 610  IALTH = 3
      GO TO 700
 620  IF ((KFLAG .EQ. 0) .AND. (RH .LT. 1.1D0)) GO TO 610
      IF (KFLAG .LE. -2) RH = DMIN1(RH,0.2D0)
C-----------------------------------------------------------------------
C IF THERE IS A CHANGE OF ORDER, RESET NQ, L, AND THE COEFFICIENTS.
C IN ANY CASE H IS RESET ACCORDING TO RH AND THE YH ARRAY IS RESCALED.
C THEN EXIT FROM 690 IF THE STEP WAS OK, OR REDO THE STEP OTHERWISE.
C-----------------------------------------------------------------------
      IF (NEWQ .EQ. NQ) GO TO 170
 630  NQ = NEWQ
      L = NQ + 1
      IRET = 2
      GO TO 150
C-----------------------------------------------------------------------
C ALL RETURNS ARE MADE THROUGH THIS SECTION.  H IS SAVED IN HOLD
C TO ALLOW THE CALLER TO CHANGE H ON THE NEXT STEP.
C-----------------------------------------------------------------------
C
C  KFLAG = -1 ; ERROR TEST FAILED REPEATEDLY WITH ABS(H) = HMIN
C
 650  KFLAG = -1
      CALL SERROR(' SBLEND- AT T=(R1) AND STEP SIZE H =(R2) THE ERROR
     1 TEST FAILED REPEATEDLY WITH ABS(H) = HMIN', 1, 0, 0, 0, 2, TN ,H)
      GO TO 720
C
C  KFLAG = -1 ; ERROR TEST FAILED REPEATEDLY
C
 660  KFLAG = -1
      CALL SERROR(' SBLEND- AT T=(R1) AND STEP SIZE H =(R2) THE ERROR
     1 TEST FAILED REPEATEDLY ', 1, 0, 0, 0, 2, TN, H)
      GO TO 720
C
C  KFLAG = -2 CONVERGANCE FAILURE
C
 670  KFLAG = -2
      CALL SERROR(' SBLEND- AT T=(R1) AND STEP SIZE H=(R2) THE CORRECTOR
     1 CONVERGENCE FAILED REPEATEDLY ', 1, 0, 0, 0, 2, TN, H)
      GO TO 720
C
C  KFLAG = -2 CONVERGANCE FAILURE  WITH MINIMUM STEPSIZE.
C
 675  KFLAG = -2
      CALL SERROR(' SBLEND- AT T=(R1) AND STEP SIZE H=(R2) THE CORRECTOR
     1 CONVERGENCE FAILED REPEATEDLY WITH H = HMIN', 1, 0, 0, 0, 2,TN,H)
      GO TO 720
C
C     KFLAG = -4  RES ORDERED RETURN
C
 680    CALL SERROR(' SBLEND- AT T=(R1) RESIDUAL ROUTINE RETURNED IRES
     1  =3 REPEATEDLY', 1, 0, 0, 0, 1, TN, 0.0D0)
      GO TO 720
 685  KFLAG = -5
      CALL SERROR(' SBLEND - AT T= (R1) A SINGULAR JACOBIAN MATRIX WAS
     1     REPEATEDLY ENCOUNTERED', 1, 0, 0, 0, 1, TN, 0.0D0)
      GO TO 720
 690  RMAX = 10.0D0
 720  CONTINUE
 700  R =  1.0D0/TESCO(2,NQU)
      IF(KFLAG .LT. 0)THEN
C        THE STEP HAS FAILED AND BEEN RETRACTED - INDICATE THAT LOCAL
C        ERROR HAS BEEN LOST
         CALL SERROR(' SPGEAR-CURRENT STEP FAILURE MEANS THAT OLD LOCAL
     1   ERROR ESTIMATE CANNOT BE RETRIEVED AND SO ACOR IS SET TO ZERO',
     2   1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         DO 702 I = 1,N
 702        ACOR(I) = 0.0D0
      END IF
      DO 710 I = 1,N
C       RESET YDOT AND SCALE ACOR SO THAT IT IS LOCAL ERROR ESTIMATE.
        Y   (I) = YH(I,1)
        YDOT(I) = YH(I,2)/H
 710    ACOR(I) = ACOR(I)*R
      HOLD = H
      ISTEP = KFLAG
      IF(ISTEP .EQ. 0)ISTEP = 1
      RETURN
C----------------------- END OF SUBROUTINE SBLEND ----------------------
      END
      SUBROUTINE CFODE (METH, A, B, TESCO, GAMMA, C)
      INTEGER METH, I, J
      DOUBLE PRECISION A, TESCO, B, GAMMA, C
      DIMENSION A(13,12), TESCO(3,12), B(12,11), GAMMA(11), C(11)
C-----------------------------------------------------------------------
C CFODE IS CALLED BY THE INTEGRATOR ROUTINE TO SET COEFFICIENTS
C NEEDED THERE.  THE COEFFICIENTS FOR THE CURRENT METHOD, AS
C GIVEN BY THE VALUE OF METH, ARE SET FOR ALL ORDERS AND SAVED.
C THE MAXIMUM ORDER ASSUMED HERE IS 12 IF METH = 1 AND 11 IF METH = 2.
C (A SMALLER VALUE OF THE MAXIMUM ORDER IS ALSO ALLOWED.)
C CFODE IS CALLED ONCE AT THE BEGINNING OF THE PROBLEM,
C AND IS NOT CALLED AGAIN UNLESS AND UNTIL METH IS CHANGED.
C
C THE A  ARRAY CONTAINS THE BASIC METHOD COEFFICIENTS.
C THE COEFFICIENTS AL(I), 1 .LE. I .LE. NQ+1, FOR THE METHOD OF
C ORDER NQ ARE STORED IN A(I,NQ). THE SAME APPLIES TO THE ARRAY B.
C
C THE TESCO ARRAY CONTAINS TEST CONSTANTS USED FOR THE
C LOCAL ERROR TEST AND THE SELECTION OF STEP SIZE AND/OR ORDER.
C AT ORDER NQ, TESCO(K,NQ) IS USED FOR THE SELECTION OF STEP
C SIZE AT ORDER NQ - 1 IF K = 1, AT ORDER NQ IF K = 2, AND AT ORDER
C NQ + 1 IF K = 3.
C-----------------------------------------------------------------------
      DO 10 I = 1,12
         A(2,I) = -1.0D0
         A(1,I) =  0.0D0
         DO 10 J = 3,13
            A(J,I) = 0.0D0
 10   CONTINUE
      A(1,1) = - .5D0
C
      A(1,2) = - .4166666666666667D0
      A(3,2) = - .5D0
C
      A(1,3) = - .375D0
      A(3,3) = - .75D0
      A(4,3) = - .1666666666666667D0
C
      A(1,4) = - .3486111111111111D0
      A(3,4) = - .9166666666666667D0
      A(4,4) = - .3333333333333333D0
      A(5,4) = - .04166666666666667D0
C
      A(1,5) = - .3298611111111111D0
      A(3,5) = - 1.041666666666667D0
      A(4,5) = - .4861111111111111D0
      A(5,5) = - .1041666666666667D0
      A(6,5) = - .008333333333333333D0
C
      A(1,6) = - .3155919312169312D0
      A(3,6) = - 1.141666666666667D0
      A(4,6) = - .625D0
      A(5,6) = - .1770833333333333D0
      A(6,6) = - .025D0
      A(7,6) = - .001388888888888889D0
C
      A(1,7) = - .3042245370370370D0
      A(3,7) = - 1.225D0
      A(4,7) = - .7518518518518519D0
      A(5,7) = - .2552083333333333D0
      A(6,7) = - .04861111111111111D0
      A(7,7) = - .004861111111111111D0
      A(8,7) = - .0001984126984126984D0
C
      A(1,8) = - .2948680004409171D0
      A(3,8) = - 1.296428571428485D0
      A(4,8) = - .8685185185185185D0
      A(5,8) = - .3357638888888888D0
      A(6,8) = - .07777777777777777D0
      A(7,8) = - .01064814814814815D0
      A(8,8) = - .0007936507936507937D0
      A(9,8) = - .0000248015873015873D0
C
      A(1,9) = - .2869754464285714D0
      A(3,9) = - 1.35892857142857D0
      A(4,9) = - .976554232804233D0
      A(5,9) = - .4171875D0
      A(6,9) = - .1113541666666667D0
      A(7,9) = - .01875D0
      A(8,9) = - .001934523809523809D0
      A(9,9) = - .000111607142857143D0
      A(10,9) = - .00000275573192239859D0
C
      A(1,10) = - .280189564439367D0
      A(3,10) = - 1.41448412698413D0
      A(4,10) = - 1.07721560846561D0
      A(5,10) = - .498567019400353D0
      A(6,10) = - .1484375D0
      A(7,10) = - .0290605709876543D0
      A(8,10) = - .0037202380952381D0
      A(9,10) = - .000299685846560847D0
      A(10,10) = - .0000137786596119929D0
      A(11,10) = - .00000027557319223986D0
C
      A(1,11) = - .274265540031599D0
      A(3,11) = - 1.46448412698413D0
      A(4,11) = - 1.17151455026455D0
      A(5,11) = - .579358190035273D0
      A(6,11) = - .188322861552028D0
      A(7,11) = - .0414303626543210D0
      A(8,11) = - .0062111447989418D0
      A(9,11) = - .000625206679894180D0
      A(10,11) = - .0000404174015285126D0
      A(11,11) = - .00000151565255731922D0
      A(12,11) = - .0000000250521083854417D0
C
      A(1,12) = - .2690288467736492D0
      A(3,12) = - 1.50993867243867D0
      A(4,12) = - 1.26027116402116D0
      A(5,12) = - .659234182098765D0
      A(6,12) = - .230458002645503D0
      A(7,12) = - .0556972461052322D0
      A(8,12) = - .00943948412698413D0
      A(9,12) = - .00111927496693122D0
      A(10,12) = - .0000909391534391534D0
      A(11,12) = - .00000482253086419753D0
      A(12,12) = - .000000150312650312650D0
      A(13,12) = - .00000000208767569878681D0
        DO 200 I = 1,11
           B(1,I) = -1.0D0
           DO 200 J = 2,12
 200          B(J,I) = 0.0D0
      B(2,1) = - 1.D0
C
      B(2,2) = - 1.5D0
      B(3,2) = - .5D0
C
      B(2,3) = - 1.833333333333333D0
      B(3,3) = - 1.D0
      B(4,3) = - .1666666666666667D0
C
      B(2,4) = - 2.083333333333333D0
      B(3,4) = - 1.458333333333333D0
      B(4,4) = - .4166666666666667D0
      B(5,4) = - .04166666666666667D0
C
      B(2,5) = - 2.283333333333333D0
      B(3,5) = - 1.875D0
      B(4,5) = - .7083333333333333D0
      B(5,5) = - .125D0
      B(6,5) = - .008333333333333333D0
C
      B(2,6) = - 2.45D0
      B(3,6) = - 2.255555555555556D0
      B(4,6) = - 1.020833333333333D0
      B(5,6) = - .2430555555555556D0
      B(6,6) = - .02916666666666667D0
      B(7,6) = - .001388888888888889D0
C
      B(2,7) = - 2.59285714285714D0
      B(3,7) = - 2.605555555555556D0
      B(4,7) = - 1.343055555555556D0
      B(5,7) = - .3888888888888889D0
      B(6,7) = - .06388888888888889D0
      B(7,7) = - .005555555555555556D0
      B(8,7) = - .000198412698412698D0
C
      B(2,8) = - 2.71785714285714D0
      B(3,8) = - 2.92966269841270D0
      B(4,8) = - 1.66875D0
      B(5,8) = - .5567708333333333D0
      B(6,8) = - .1125D0
      B(7,8) = - .01354166666666667D0
      B(8,8) = - .000892857142857143D0
      B(9,8) = - .0000248015873015873D0
C
      B(2,9) = - 2.82896825396825D0
      B(3,9) = - 3.23164682539683D0
      B(4,9) = - 1.99426807760141D0
      B(5,9) = - .7421875D0
      B(6,9) = - .174363425925926D0
      B(7,9) = - .02604166666666667D0
      B(8,9) = - .00239748677248677D0
      B(9,9) = - .000124007936507937D0
      B(10,9) = - .00000275573192239859D0
C
      B(2,10) = - 2.92896825396825D0
      B(3,10) = - 3.51454365079365D0
      B(4,10) = - 2.31743276014109D0
      B(5,10) = - .941614307760141D0
      B(6,10) = - .248582175925926D0
      B(7,10) = - .0434780092592593D0
      B(8,10) = - .00500165343915344D0
      B(9,10) = - .000363756613756614D0
      B(10,10) = - .0000151565255731922D0
      B(11,10) = - .000000275573192239859D0
C
      B(2,11) = - 3.01987734487735D0
      B(3,11) = - 3.78081349206349D0
      B(4,11) = - 2.63693672839506D0
      B(5,11) = - 1.15229001322751D0
      B(6,11) = - .334183476631393D0
      B(7,11) = - .06607638888888889D0
      B(8,11) = - .00895419973544974D0
      B(9,11) = - .000818452380952381D0
      B(10,11) = - .0000482253086419753D0
      B(11,11) = - .00000165343915343915D0
      B(12,11) = - .0000000250521083854417D0
C
C      DEFINE THE C AND GAMMA ARRAYS
C
      GAMMA(1) = 0.08578644D0
      GAMMA(2) = 0.1250D0
      GAMMA(3) = 0.121890D0
      GAMMA(4) = 0.12849970D0
      GAMMA(5) = 0.10872640D0
      GAMMA(6) = 0.096259610D0
      GAMMA(7) = 0.08754865D0
      GAMMA(8) = 0.08105623D0
      GAMMA(9) = 0.07599874D0
      GAMMA(10) = 0.07192936D0
      GAMMA(11) = 0.06857227D0
      C(1) = 0.29289320D0
      C(2) = 0.3374973D0
      C(3) = 0.3335427D0
      C(4) = 0.34273290D0
      C(5) = 0.31690580D0
      C(6) = 0.29929710D0
      C(7) = 0.28623920D0
      C(8) = 0.2760327D0
      C(9) = 0.2677630D0
      C(10) =0.26088340D0
      C(11) =0.25504260D0
      DO 300 J = 1,12
         TESCO(2,J) = J+1.0D0
         TESCO(3,J) = J+2.0D0
 300  CONTINUE
      TESCO(1,1) = 1.0D0
      TESCO(1,2) = 1.0D0
      TESCO(1,3) = 0.5D0
      TESCO(1,4) = 0.1667D0
      TESCO(1,5) = 0.4167D-1
      TESCO(1,6) = 0.8333D-2
      TESCO(1,7) = 0.1389D-2
      TESCO(1,8) = 0.1984D-3
      TESCO(1,9) = 0.248D-4
      TESCO(1,10) = 0.2756D-5
      TESCO(1,11) = 0.2756D-6
      TESCO(1,12) = 0.2505D-7
        RETURN
        END
