C MOD. MAY 1987
C PDCODE COMMENTED OUT SO CAN USE SPRBOX WITH REMESHING
C
C----------------------------------------------------------------------
C
C   HOW TO USE THIS MODULE
C   **********************
C (1) SET NPDE = NUMBER OF P.D.E.S TO BE SOLVED
C     SET NPTS = NUMBER OF MESH POINTS TO BE USED
C     DEFINE A MESH OF INCREASING MESH POINTS IN THE ARRAY X(NPTS).
C       (USING THE REMESH OPTION A START MESH IS CONSTRUCTED BY INITFD)
C     SET M FOR SPACE CO-ORDINATE TYPE
C         = 0 FOR CARTESIAN,  = 1 FOR CYLINDRICAL, = 2 FOR SPHERICAL
C     SET NV = NUMBER OF O.D.E.S COUPLED TO THE P.D.E.S
C     SET REMESH = TRUE IF REMESHING IS REQUIRED (ELSE FALSE) - SEE (3)
C     SET MAXNPT = MAX. NO. OF MESH POINTS TO BE USED
C         (MAXNPT=NPTS IF REMESH IS FALSE)
C     SPECIFY RESWK OF SIZE AT LEAST  MAXNPT
C             WKMON OF SIZE AT LEAST 4*MAXNPT (ONLY IF REMESHING USED)
C     FOR USE BY THE ROUTINE RES WHICH DEFINES THE O.D.E. SYSTEM BEING
C     SOLVED BY THE INTEGRATOR.
C
C     CALL THE INITIALISATION ROUTINE INITFD, SEE THE DOCUMENTATION AT
C     THE HEAD OF THIS ROUTINE FOR THE PRECISE DETAILS OF THE CALL.
C     SET TS AND TOUT FOR START AND END INTEGRATION TIMES
C     SET INFORM, WORK ARRAYS AS REQUIRED FOR TIME INTEGRATION
C                - SEE SPRINT DOCUMENTATION
C
C (2) PROVIDE 2 ROUTINES WHICH DESCRIBES THE INITIAL CONDITION
C
C       UINIT  : SUPPLIES THE INITIAL VALUES OF THE P.D.E. PART.
C       SVINIT : SUPPLIES THE INITIAL VALUES OF THE O.D.E. PART
C
C (3) REMESHING PARAMETERS:
C -------------------------
C     THE REMESHING ALGORITHM USED IS DESCRIBED BY
C     FURZELAND R.M. ???????
C
C     NOTE : REMESHING SHOULD NOT BE USED IF ANY OF THE FUNCTIONS
C            C OR F ARE ONLY PIECEWISE CONTINUOUS OR IF A COUPLED
C            O.D.E./P.D.E. PROBLEM IS SOLVED.
C     DEFINE A MESH OF INCREASING MESH POINTS IN THE ARRAY X(NPTS).
C      (THIS MESH IS USED BY INITFD AT TIME TS TO CONSTRUCT AN ADAPTIVE
C       MESH ACCORDING TO THE FORM OF THE REMESH MONITOR FUNCTION EVNFDB
C      SET UP THE FOLLOWING COMMON BLOCKS
C     COMMON /MESHIN/ CONST, STPRAT, IPMINF, NOP
C CONST - INPUT BOUND ON INTEGRAL OF MONITOR OVER EACH STEP. SIZE
C         DEPENDS ON NPTS AND MONITOR (TYPICAL VALUE 5.0D-2).
C         CONTROLS THE LOCAL TRUNCATION ERROR IF THE MONITOR IS CHOSEN
C         ACCORDINGLY
C STPRAT - INPUT BOUND ON ADJACENT MESH RATIO (>1 TYPICALLY 1.5-3)
C          THIS ALSO CONTROLS LOCAL TRUNCATION ERRORS.
C
C IPMINF - CONTROLS PRINT OUT OF MESH INFORMATION. IPMINF=0 NO PRINT,
C          =1 SUMMARY PRINT OF MESH CHARACTERISITICS E.G. MESH RATIOS &
C          MAX. INTEGRAL OF MONITOR PER INTERVAL (APPROX.=INPUT CONST)
C          =2 PRINT OF MONITORS, OLD & NEW MESHES AND MESH SIZES
C SET NOP=0 IF VARIABLE NO. OUTPUT PTS REQUIRED ELSE SET NOP=NPTS
C
C COMMON MESHTI USED FOR REMESHING IN TIME INFO
C     COMMON /MESHTI/ TOUT, REMESH, NRMESH, ICOUNT
C
C    SET TOUT - SEE BELOW
C    SET REMESH TRUE OR FALSE DEPENDING ON WHETHER REMESHING IS REQUIRED
C      (NOTE REMESH MAY CHANGE FROM TRUE TO FALSE DURING COURSE OF TIME)
C    SET NRMESH TO DICTATE HOW OFTEN TO REMESH
C     =1 THEN REMESHES EVERY TIME STEP THE SPRINT MONITR CALLED
C     =2        "        "   2ND TIME STEP, ETC....
C        (THE SIZE OF WORK(6)=HMAX SHOULD BE SET APPROPRIATELY IN
C         THE CALL TO SPRINT)
C     =0 THEN REMESHES AT CURRENT TIME TOUT BY THE CALL TO MONITR WITH
C         INFORM(2)=4 AND WORK(1) SET TO CURRENT TOUT. IF THE SOLUTION
C         PROFILE CHANGES RAPIDLY FOR CERTAIN T THEN SET TOUT ACCORDINGL
C
C ICOUNT USED TO COUNT UP TO NRMESH SO JUST INITIALISE TO ZERO
C     ICOUNT=0
C
C THE USER MUST ALSO SET THE FORM OF THE REMESH MONITOR FUNCTION.
C     WITHIN THE CONSTRAINTS DICTATED BY CONST AND STPRAT THIS MONITOR
C     IS USED TO CHOOSE A MESH WHICH EQUIDISTRIBUTES THE INTEGRAL OF
C     THE MONITOR AND SHOULD THUS RESEMBLE THE SPACE DISCR.ERROR TERM.
C     THE MONITOR SHOULD BE NON-NEGATIVE AND NOT EVERYWHERE ZERO.
C
C  RENAMED FROM EVNRMF TO EVNFDB (24.1.86)
C     SUBROUTINE EVNFDB( TIME, NPTS, NPDE, NV, X, U, UDOT, MAXNPT, FMON)
C
C  EXAMPLE REMESH MONITOR FUNCTION
C  E.G. THE 2ND DERIVATIVE OF U FOR THE HEAT EQN.
C  THE MONITOR FUNCTION IS RETURNED IN THE ARRAY FMON
C  THE RESULTING FMON SHOULD BE NOT BE NEGATIVE AT ANY POINT
C                                   OR EVERYWHERE ZERO
C
C IN GENERAL, SINCE X(1) POINTS TO RESWK(1), THE USER CAN ACCESS
C    OTHER SOLUTION PROFILE INFORMATION HELD BY THE USER IN RESWK
C
C     INTEGER NPTS, NPDE, MAXNPT, I, J, K, L, NV , NPTS1, IM
C     DOUBLE PRECISION U(NPDE,1), UDOT(NPDE,1)
C ALTERNATIVELY
C     DOUBLE PRECISION U(1), UDOT(1)
C     DOUBLE PRECISION X(NPTS), FMON(NPTS), TIME, DU2, DXPI, DXPIM
C     IF(TIME.LT.1.0D-3)GO TO 100
C MONITOR FOR TIME>0
C     NPTS1=NPTS-1
C     DO 10 I=2,NPTS1
C SET FMON = L2NORM OVER J=1,..,NPDE OF 2ND DERIVATIVE
C THE USER CAN CALL A SYSTEM F.D. BUILDING BLOCK TO FORM DERIVATIVES
C OR CODE HIS/HER OWN IN
C     IM=I-1
C     DXPI=X(I+1)-X(I)
C     DXPIM=X(I)-X(IM)
C     FMON(I)=0.0D0
C     D0 20 J=1,NPDE
C FORMULAE FOR 2ND DERIVATIVE ON NON-UNIFORM MESH
C        DU2=((U(J,I+1)-U(J,I))/DXPI-(U(J,I)-U(J,IM))/DXPIM)/
C    1   (0.5*(X(I+1)-X(IM)))
C        FMON(I)=FMON(I)+DU2**2
C  20 CONTINUE
C     FMON(I)=DSQRT(FMON(I))
C  10 CONTINUE
C END-PTS, - ASSUMES CONSTANT 2ND DERIV OVER END 3 PTS
C     FMON(1)=FMON(2)
C     FMON(NPTS)=FMON(NPTS1)
C     RETURN
C
C TIME NEAR ZERO PROFILE  (THIS MAY HAVE TO BE SCALED RELATIVE TO X=Z)
C IF TIME NEAR ZERO THEN NEED SAY EXP MONITOR NEAR X=0
C 100 DO 120 I=1,NPTS
C 120    FMON(I) = DEXP(-X(I))
C     RETURN
C     END
C
C
C
C EXAMPLE PROBLEM ONE
C *******************
C     THE EASIEST WAY TO DESCRIBE HOW THE PROBLEM DESCRIPTION ROUTINES
C     SHOULD BE WRITTEN IS BY A SIMPLE EXAMPLE. CONSIDER THE FLAME
C     PROPAGATION PROBLEM DEFINED BY
C       Y  = ( Y  )  - Y * KT
C        T      X  X
C  AND                            (NPDE IS 2 )
C       W  = ( W  )  + Y * KT
C        T      X  X
C                      WHERE KT = 14.6* EXP( 4.0*(1.0 - 1./(V + 0.01)))
C  AND   WHERE 0 =< X =< 10.0. AND  0=< T =< 2.8.
C
C     THE LEFT BOUNDARY CONDITION AT X = 0 (LEFT = .TRUE. ) IS GIVEN BY
C         W  = -T / 0.05 , 0 < T =< 0.05
C          X
C            = -COS( PI*0.5 *(T- 0.05) / 0.95)  0.05 =< T < 1.0
C            =  0   FOR T >= 1.0
C      AND
C         Y  = 0
C          X
C     THE RIGHT BOUNDARY CONDITION IS  (LEFT = .FALSE.)
C         W  = Y = 0 FOR ALL TIME T
C          X    X
C      THE ROUTINES SPDEF1 AND BNDR ARE LISTED IN TURN BELOW.
C      THE COMPONENT W OF THE P.D.E. AT THE ITH MESHPOINT IS HELD AS
C      U(1,I) IN THE PACKAGE AND THE COMPONENT Y IS HELD AS U(2,I)
C      THE INITIAL CONDITIONS ARE
C        W(I) = U(1,I) = 1.0D0  I = 1,.., NPTS
C        Y(I) = U(2,I) = 0.0D0  I = 1,...,NPTS
C
C**********************************************************************
C      SUBROUTINE UINIT( NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. INITIAL VALUES.
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS)
C      DO 10 I = 1,NPTS
C         U(1,I) = 1.0D0
C10       U(2,I) = 0.0D0
C      RETURN
C      END
C      SUBROUTINE SVINIT (V,NV)
C      ROUTINE FOR O.D.E. INITIAL VALUES.
C      INTEGER NV
C      DOUBLE PRECISION V(NV)
C      ....
C      RETURN
C      END
C
C
C**********************************************************************
C EXAMPLE  PROBLEM 2
C SOLUTION OF MOVING BOUNDARY  PROBLEM BY CO-ORDINATE TRANSFORMATION.
C********************************************************************
C  THIS PROBLEM IS THE ONE PHASE STEFAN PROBLEM (HOFFMAN (1977) ) SEE
C  FURZELAND R.M. A COMPARATIVE STUDY OF NUMERICAL METHODS FOR MOVING
C                 BOUNDARY PROBLEMS. J.I.M.A. (1977) ,26, PP 411 - 429.
C  THE PROBLEM HAS  MELTING DUE TO HEAT INPUT AT THE FIXED
C  BOUNDARY . THE P.D.E. IS DEFINED BY THE EQUATIONS
C         U  =  U        0 < Y < S(T) , 0.1 < T < 1
C          T     YY
C            U  = - EXP(T) , Y = 0
C             Y            .
C            U  =  0  AND  S(T) = - U   ON THE MOVING BOUNDARY Y = S(T).
C                                    Y
C  AND THE INITIAL SOLUTION VALUES AT T = 0.1 ARE GIVEN BY THE ANALYTIC
C  SOLUTION
C            U = EXP(T-Y) - 1 , S(T) = T.
C  THE PROBLEM IS REWRITTEN BY USING THE CO-ORDINATE TRANSFORMATION
C  GIVEN BY  X(T)  =  Y / S(T)  . THE EQUATIONS THEN READ
C                      .
C     S * S * U  - S * S  * X * U   =  U     , X IN (0,1).
C              T                 X      XX
C  WITH THE NEUMANN TYPE BOUNDARY CONDITIONS
C                                               .
C     U  = - EXP(T)  AT X=0  AND  U  = - S(T) * S(T) AT X = 1
C      X                           X
C  AND THE O.D.E. COUPLING POINT EQUATION AT X = 1 WHICH IMPLICITLY
C  DEFINES S(T) IS GIVEN  BY
C     U(1,T) = 0
C  THE EXACT SOLUTION IS NOW DEFINED BY
C     U(X,T) = EXP((T - X*S(T))  , S(T) = T
C
C WE SHALL NOW DETAIL THE ROUTINES NEEDED TO DESCRIBE THIS PROBLEM.
C        PROBLEM DESCRIPTION ROUTINES
C       ******************************
C EXACT SOLUTION
C      SUBROUTINE EXACT( TIME, NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. EXACT VALUES  (IF KNOWN)
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), TIME, P
C      P=DSQRT(2.0D0)*0.5D0
C      DO 10 I = 1,NPTS
C10       U(1,I) = DEXP( TIME * (1 - X(I))) - 1.0D0
C      RETURN
C      END
C      SUBROUTINE UINIT( NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. INITIAL VALUES.
C      INTEGER NPDE, NPTS
C      DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), TIME
C      TIME=0.1D0
C      CALL EXACT(TIME,NPDE,NPTS,X,U)
C      RETURN
C      END
C      SUBROUTINE SVINIT (V,NV)
C      ROUTINE FOR O.D.E. INITIAL VALUES.
C      INTEGER NV
C      DOUBLE PRECISION V(NV)
C      .....
C      RETURN
C      END
C
C
C***********************************************************************
C
        SUBROUTINE INITFD( TS, NEQN, NPDE, NPTS, MAXNPT, NV, REMESH,
     1                     U, RESWK, NRESWK, WKMON, NWKMON, IRES)
C
C***********************************************************************
C       PARAMETER LIST
C       --------------
C
C       TS-THE TIME AT WHICH INTEGRATION COMMENCES
C
C       NEQN : ON ENTRY CONTAINS THE SIZE OF THE O.D.E.
C              SYSTEM GENERATED WHEN THE P.D.E. IS DISCRETISED.
C              THIS VALUE IS NPDE * NPTS + NV
C
C       NPDE : THE NUMBER OF P.D.E. S
C
C       NPTS : THE NUMBER OF SPATIAL MESH POINTS IF THE REMESH OPTION IS
C              USED THIS NUMBER MAY HAVE BEEN CHANGED ON EXIT. AN UPPER
C              BOUND MAY BE SET BY USING THE PARAMETER NPTMAX.
C
C       MAXNPT: INPUT PARAMETER THAT SPECIFIES THE MAXIMUM NUMBER OF
C               SPATIAL MESH POINTS. THE NUMBER OF POINTS GENERATED IN
C               REMESHING WILL BE BOUNDED BY THIS VALUE.
C
C       NV   : THE NUMBER OF AUXILIARY O.D.E.S THAT ARE COUPLED TO THE
C              P.D.E. THIS VALUE IS USED IN DETERMINING THE SIZE OF NEQN
C
C       REMESH: LOGICAL PARAMETER THAT SHOULD BE SET TO .TRUE. ON ENTRY
C               IF REMESHING IS REQUIRED. OTHERWISE IT SHOULD BE SET TO
C               .FALSE.
C
C       U(NEQN) ARRAY THAT ON EXIT HAS THE INITIAL P.D.E. VALUES. THIS
C               ARRAY MUST BE PASSED ACROSS AS A ONE DIMENSIONAL ARRAY O
C               LENGTH NEQN TO THE SPRINT PACKAGE. THIS ARRAY IS ORDERED
C               BY THIS ROUTINE AS
C               U(1)           - U(NPDE*NPTS)  ; PDE COMPONENTS
C               U(NPDE*NPTS+1) - U(NEQN)       ; ODE COMPONENTS
C
C       RESWK:  RESIDUAL WORK ARRAY FROM DRIVER PROGRAM; FIRST NPTS
C               CONTAINS THE USER SUPPLIED MESH POINTS . ON OUTPUT, IF
C               THE REMESH OPTION IS USED, IT WILL CONTAIN THE MESH PTS
C               THAT HAVE BEEN CHOSEN BY THE REMESH ROUTINE.
C               THE REST OF RESWK IS AVAILABLE FOR COMMUNICATION BETWEEN
C               THE USER DRIVER, THE USER RESIDUAL ROUTINE, AND MONFDR
C               AND FURTHER INTO EVNFDB FOR REMESHING.
C
C       NRESWK: SIZE OF RESWK
C
C       WKMON(NWKMON) MONITOR WORKSPACE USED FOR REMESHING.
C
C       NWKMON MUST BE  >= 4*MAXNPT IF THE REMESH OPTION IS USED
C              ELSE  NWKMON=1
C
C       IRES:  RETURN CODE SET TO -1 IF FAILURE OCCURS IN CALL
C---------------------------------------------------------------------
C   FIVE IMPORTANT PARAMETERS ARE PASSED ACROSS FROM HERE IN
C   COMMON /SPFDBL/  NNPDE, NNPTS, NNV, NVST, NPMAX
C
C  NNPDE = NPDE     THE NUMBER OF P.D.E.S
C
C  NNPTS = NPTS     THE TOTAL NUMBER OF MESHPOINTS = NEL*(NPTL-1) + 1 ,
C                   IF REMESH THEN THE NO. MAY CHANGE ON OUTPUT.
C                           CO-ORDINATES IN USE.
C
C  NNV = NV         NUMBER OF AUXILLARY ODES
C
C  NVST             THE STARTING POINT OF THE ODE COMPONENTS IN THE
C                   SOLUTION VECTOR.
C  NPMAX            MAXNPT FROM THE PARAMETER LIST OF THIS ROUTINE.
C
C----------------------------------------------------------------------
C
C
C     WKMON STRUCTURE
C     SPLIT INTO I=1, I11, I12 ,I13 FOR  REMESHING
C       PROCESS TO GENERATE THE NEW MESH  (SEE INMSFD AND MONFDR)
C
C**********************************************************************
        INTEGER I, NPTS, NPDE, IDEV, ITRACE, NNPDE, NNPTS, MAXNPT
        INTEGER I11, I12, I13 , NWKMON, NRESWK , IRES
        INTEGER NV, NNV, NXI, NVST, NEQN, NPMAX, IDUM
        DOUBLE PRECISION WKMON(NWKMON), RESWK(NRESWK), TS, U(1), TDUM
        CHARACTER*6 PDCODE
        LOGICAL REMESH , REMSH1
        COMMON /DISCHK/ PDCODE
        COMMON /SDEV2/  ITRACE,IDEV
        COMMON /SPFDWK/ I11, I12, I13
        COMMON /SPFDBL/ NNPDE, NNPTS,  NNV, NVST, NPMAX
        COMMON /MESHTI/ TDUM, REMSH1, IDUM(2)
        SAVE   /SPFDBK/, /SPFDWK/, /DISCHK/
        IRES = 0
        NNPDE = NPDE
C IF ITIME>1, NPTS AND NEQN UPDATED IN MONFDR REMESHING MONITOR SUBR.
        NNPTS = NPTS
        NEQN = NPDE*NPTS + NV
        NPMAX = MAXNPT
        NXI=0
        IF(NV .LT. 0 .OR. NXI .LT. 0)THEN
           CALL SERROR(' INITFD - ERROR ILLEGAL VALUES OF NXI(=I1) ON
     1                   OR OF NV (=I2) ON ENTRY ', 1 , 2, NXI, NV, 0,
     2                   0.0D0, 0.0D0)
           IRES = -1
           RETURN
        END IF
        IF(REMESH)THEN
C---------------------------------------------------------------------
C   WORKSPACE ORGANISATION - THE PARTS OF THE WORKSPACE TO DO WITH
C   **********************   REMESHING ARE ORGANISED AS FOLLOWS:
C   RESWK(I)   ;  THE START OF THE INPUT (CURRENT) MESH
C   WKMON( 1 ) ;  THE START OF THE NEW MESH FOUND BY PADRMS. ARRAY XNP.
C   WKMON(I11) ;  THE START OF THE MESH SPACINGS , ARRAY DXP(MAXNPT)
C   WKMON(I12) ;  THE START OF THE MONITOR FUNCTION ARRAY FMON(MAXNPT)
C   WKMON(I13) ;  THE START OF THE ARRAY CHISTR(MAXNPT)
C                 FROM WKMON(I11) ONWARDS THE WORKSPACE IS ALSO USED IN
C                 THE CALL TO THE CUBIC SPLINE ROUTINE INTCUB.
C---------------------------------------------------------------------
           I11 = 1 + MAXNPT
           I12 = I11 + MAXNPT
           I13 = I12 + MAXNPT
           I = 4 * MAXNPT
        ELSE
           I11 = 1
           I12 = I11
           I13 = I12
           I   = 1
           REMSH1 = .FALSE.
        END IF
        IF(I .GT. NWKMON)THEN
           CALL SERROR(' INITFD - ERROR REAL WORKSPACE OF SIZE (=I1) IS
     1                   SMALLER THAN REQUIRED (=I2)', 1, 2,NWKMON,I, 0,
     2                   0.0D0, 0.0D0)
           IRES = -1
        END IF
        NNV = NV
        NVST = NPDE * NPTS + 1
        IF(NV .EQ. 0)NVST = NVST -1
        IF(NPTS .LE. 2 .OR. NPDE .LT. 1)THEN
          CALL SERROR(' INITFD - ILLEGAL VALUES OF NPTS(=I1) OR NPDE (=I
     1    2)',1,2,NPTS,NPDE, 0, 0.0D0, 0.0D0)
          IRES = -1
        END IF
        IF(  MAXNPT .LT. NPTS) THEN
           CALL SERROR(' INITFD - ILLEGAL VALUE OF MAXNPT (=I1) .LT.
     1                   NPTS (=I2) ', 1, 2, MAXNPT, NPTS, 0, 0.0D0,
     2                   0.0D0 )
           IRES = -1
        END IF
C
C        CHECK THE  MESH
C
        DO 9 I = 2,NPTS
           IF( RESWK(I) .LE. RESWK(I-1))THEN
             CALL SERROR(' INITFD-ERROR MESHPOINT NO (=I1) VALUE (=R1)
     1 HAS A SMALLER VALUE THAN THE NEXT MESHPOINT TO THE LEFT
     2 VALUE (=R2)', 1, 1, I, 2, RESWK(I), RESWK(I-1))
             IRES = -1
           END IF
 9     CONTINUE
        IF(IRES.EQ.-1)RETURN
        PDCODE = 'SFDBLK'
C
C         INITIALISE O.D.E. AND P.D.E. VARIABLES BY APPROPRIATE CALLS.
C
        IF(NV .GT. 0) CALL SVINIT( U(NVST), NV)
        CALL UINIT( NPDE, NPTS, RESWK, U )
C
C    CALL TO THE ROUTINE TO ADJUST THE INITIAL MESH.
C
        IF(REMESH)THEN
           CALL INMSFD( TS, U, NPDE,NPTS, RESWK, NRESWK, WKMON, NWKMON,
     -               NV,MAXNPT)
C RESETS NNPTS AND NEQN AFTER CALL TO INMSFD SINCE THIS MAY CHANGE NPTS
           NEQN = NPDE*NPTS + NV
           NNPTS = NPTS
C NVST WILL CHANGE IF NPTS HAS CHANGED
           NVST  = NPDE*NPTS
C UINIT ALREADY CALLED ON NEW MESH INSIDE ROUTINE INMSFD
           IF(NV .GT. 0) THEN
              NVST  = NPDE*NPTS + 1
              CALL SVINIT( U(NVST), NV)
           END IF
        END IF
        RETURN
        END
C
        SUBROUTINE INMSFD( T, U, NPDE,NPTS, RESWK, NRESWK,WKMON, NWKMON,
     1                     NV, MAXNPT)
C**********************************************************************
C       ROUTINE TO PROVIDE INITIAL MESH BY THREE CALLS TO PADRMS.
C**********************************************************************
        INTEGER  ITRACE, IDEV, NPTS, NPDE, MAXNPT, NV
        INTEGER I, J, K, I11, I12, I13
        INTEGER NOP, IPMINF, IFAIL, NOPUSR
        DOUBLE PRECISION  RESWK(NRESWK), U(1), UDOT, T,
     1          CONST, STPRAT,  WKMON(NWKMON)
        COMMON /SPFDWK/ I11, I12, I13
        COMMON /SDEV2/ ITRACE, IDEV
        COMMON /MESHIN/ CONST, STPRAT, IPMINF, NOPUSR
        SAVE /SPFDWK/
C
C NOP=0 THEN NOP CALCULATED BY PADRMS ELSE NOP SET = NIP BY PADRMS
C AND SO CONST CRITERION MAY BE OVER OR UNDER SATISFIED
C
      NIP=NPTS
      DO 700 K = 1,3
       NOP=0
       IF(NOPUSR.NE.0)NOP=NIP
       IFAIL=0
       ISTATE = 0
       IF(K.EQ.1.AND.ITRACE.GE.1)WRITE(IDEV,900)(RESWK(I),I=1,NIP)
  900  FORMAT('0 INITIAL MESH POINTS ',/20(5D12.4/))
C----------------------------------------------------------------------
C       FORMATION OF MONITOR FUNCTION BY CALLING EVNFDB
C       N.B. EVNFDB FORMS ITS OWN FLUXES .
C----------------------------------------------------------------------
      CALL EVNFDB( T, NIP, NPDE, NV, RESWK, U, UDOT, MAXNPT, WKMON(I12))
C
C INITIAL MESH IS IN RESWK(1,...,NIP)
C OUTPUT  MESH IS IN WKMON(1,....,NOP)  (SEE MONFDR FOR MORE DETAILS)
      CALL PADRMS( T, NPDE, NIP, RESWK, MAXNPT, CONST, STPRAT,
     1             IPMINF, WKMON(I12), NOP, WKMON ,WKMON(I11),
     2             WKMON(I13), IFAIL)
C
      IF(ITRACE .GT. 1)WRITE(IDEV,901)NOP,(WKMON(I),I=1,NOP)
  901                  FORMAT('0 ',I4,' NEW MESH POINTS ',/20(5D12.4/))
C---------------------------------------------------------------------
C       FIND THE CORRECT INITIAL VALUES AT NEW MESH POINTS.
C---------------------------------------------------------------------
      DO 666 I = 1, NOP
 666     RESWK(I) = WKMON(I)
      CALL UINIT( NPDE, NOP, RESWK, U)
      NIP = NOP
 700  NPTS = NOP
      IF(ITRACE.GE.1)WRITE(IDEV,902)(RESWK(I),I=1,NOP)
 902  FORMAT('0 FINAL MESH POINTS ',/20(5D12.4/))
      RETURN
      END
C
      SUBROUTINE MONFDR( NEQN, T,HLAST,H,Y, YDOT, YSAVE, NYH, R, ACOR,
     1       RESWK, NRESWK, WKMON, NWKMON, IMON, INLN, HMIN, HMAX)
C**********************************************************************
C  MONITOR ROUTINE FOR REMESHING WITH VARIABLE NO. OF PTS
C  FOR USE WITH USER DISCRETISATIONS OR F.D. BLOCKS.
C**********************************************************************
      INTEGER NEQN, NYH, NRESWK, NWKMON, IMON, INLN
      INTEGER ITRACE, IDEV, IOVFLO
      INTEGER NDUM, NQP1, NQ, NINTER, I, N, K, NOLD
      INTEGER NOP, IPMINF, IFAIL, NIP, NRMESH, ICOUNT, NOPUSR
      INTEGER NPTS, NPDE , M, NV, NXI, NVST, MAXNPT
      INTEGER I11, I12, I13
      DOUBLE PRECISION T, H, Y(1), YDOT(1), YSAVE(NYH,1), R(1), HLAST,
     1        ACOR(1), RESWK(NRESWK), WKMON(NWKMON), HMIN, HMAX,
     2        CONST, STPRAT, TOUT, DUNFLO, UROUND
      LOGICAL REMESH
      CHARACTER*6 PDCODE
      COMMON /LSTATS/ NQ, NDUM(5), NINTER
      COMMON /SDEV2/  ITRACE, IDEV
      COMMON /SPFDWK/ I11, I12, I13
      COMMON /SPFDBL/ NPDE, NPTS,  NV, NVST, MAXNPT
      COMMON /MESHIN/ CONST, STPRAT, IPMINF, NOPUSR
      COMMON /MESHTI/ TOUT, REMESH, NRMESH, ICOUNT
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
      COMMON /DISCHK/ PDCODE
      SAVE   /LSTATS/, /SPFDBL/, /SCONS1/, /DISCHK/
C-----------------------------------------------------------------------
C CHECK FOR STEP FAILURE , IF SO PASSES CONTROL BACK TO SPRINT
C-----------------------------------------------------------------------
C
      IF(IMON.LT.0)RETURN
C TEMP. COMMENTED OUT SO CAN USE SPRBOX WITH REMESHING
C     IF(PDCODE.NE.'SFDBLK')THEN
C         CALL SERROR(' REMESHING MONITOR - MONFDR - ERROR -THE SETUP
C    1                  ROUTINE INITFD HAS NOT BEEN CALLED '
C    2                  , 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
C         IMON = -2
C         RETURN
C       END IF
      IF( I13 .LE. 1 )THEN
         IF(REMESH)THEN
C         REMESHING IS BEING TRIED BUT THE WORKSPACE POINTERS HAVE NOT
C         BEEN SET UP
          CALL SERROR(' REMESHING MONITOR - MONFDR - ERROR -THE SETUP
     1                  ROUTINE INITFD WAS NOT CALLED WITH REMESH .TRUE.
     2                  BUT  REMESH WAS SET .TRUE. FOR  MONFDR IN COMMON
     3                  BLOCK /MESHTI/ ', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
          IMON = -2
          RETURN
        END IF
      END IF
      IF(.NOT.REMESH)GO TO 10
      IF(NRMESH.NE.0)THEN
         ICOUNT=ICOUNT+1
         IF(ICOUNT.EQ.NRMESH)GO TO 17
      ELSE
         IF(DABS((T-TOUT)/TOUT).LT.UROUND)GO TO 17
      ENDIF
C NORMAL EXIT TO CONTINUE INTEGRATION
   10 IMON=1
      RETURN
C--------------------------------------------------------------------
C REMESHING FOLLOWED BY INTERPOLATION
C   WORKSPACE ORGANISATION - THE PARTS OF THE WORKSPACE TO DO WITH
C   **********************   REMESHING ARE ORGANISED AS FOLLOWS:
C   RESWK(I)   ;  THE START OF THE INPUT (CURRENT) MESH
C   WKMON( 1 ) ;  THE START OF THE NEW MESH FOUND BY PADRMS. ARRAY XNP.
C   WKMON(I11) ;  THE START OF THE MESH SPACINGS , ARRAY DXP.
C   WKMON(I12) ;  THE START OF THE MONITOR FUNCTION ARRAY FMON.
C   WKMON(I13) ;  THE START OF THE ARRAY CHISTR.
C                 FROM WKMON(I11) ONWARDS THE WORKSPACE IS ALSO USED IN
C                 THE CALL TO THE CUBIC SPLINE ROUTINE INTCUB.
C---------------------------------------------------------------------
   17 ICOUNT=0
      IF(ITRACE .GT. 0 .OR. IPMINF .GT. 0)WRITE(IDEV,900)T
 900  FORMAT(' REMESHING PROCESS COMMENCED AT TIME T = ',D12.4)
      NOLD = NPTS*NPDE
      NIP=NPTS
C IF NOP=0 THEN NOP CALCULATED BY PADRMS ELSE NOP SET = NIP BY PADRMS
C IN WHICH CASE CONST CRITERION MAY BE OVER OR UNDER SATISFIED
      NOP=0
      IF(NOPUSR.NE.0)NOP=NIP
      IFAIL=0
      CALL EVNFDB( T, NIP, NPDE, NV, RESWK, Y, YDOT, MAXNPT, WKMON(I12))
      CALL PADRMS( T, NPDE, NIP, RESWK, MAXNPT, CONST, STPRAT,
     1             IPMINF, WKMON(I12), NOP, WKMON ,WKMON(I11),
     2             WKMON(I13), IFAIL)
      IF(IFAIL .NE. 0)THEN
         IMON = -2
         RETURN
      ENDIF
C NEW NO. OF PTS (NEEDED FOR COMMON SPFDBL) AND NEW VALUE OF NEQN
      NPTS = NOP
      N    = NPTS*NPDE
      NVST = NPTS*NPDE + 1
      NEQN = N + NV
      IF(ITRACE.GT.1) WRITE(IDEV,901) NOP, (WKMON(I),I=1,NOP)
  901 FORMAT(1H ,I3, ' NEW MESH POINTS ',/20(5D12.4/))
C----------------------------------------------------------------------
C INTERPOLATE Y ONTO NEW MESH AND PUT NEW Y FROM R INTO Y USING SPSWOP.
C-----------------------------------------------------------------------
      CALL INTCUB( WKMON, R, NOP, RESWK, Y, NIP, NPDE, IFAIL,
     1             WKMON(I11))
      CALL SPSWOP( Y, R, N, NOLD, NV)
C-----------------------------------------------------------------------
C INTERPOLATE YDOT (YDOT IS OVERWRITTEN WITH NEW YDOT FROM R)
C-----------------------------------------------------------------------
      CALL INTCUB( WKMON, R, NOP, RESWK, YDOT, NIP, NPDE,
     1             IFAIL, WKMON(I11))
      CALL SPSWOP( YDOT, R, N, NOLD, NV)
      NQP1=NQ + 1
      DO 18 I = 1,NEQN
         YSAVE(I,1) = Y(I)
  18     YSAVE(I,2) = YDOT(I) * H
      DO 20 K = 3,NINTER
C ALLOWS FOR THETA METHOD USE (AS WELL AS ADAMS/GEAR)
C MAY NEED MODS FOR OTHER INTEGRATORS????
         IF(NINTER.EQ.4)GO TO 19
         IF(K.LT.NINTER .AND. K.GT.NQP1)GOTO 20
  19     CONTINUE
C        INTERPOLATE YSAVE (AND OVERWRITE WITH NEW YSAVE)
         CALL INTCUB( WKMON, R, NOP, RESWK, YSAVE(1,K), NIP,
     1                NPDE, IFAIL, WKMON(I11))
         CALL SPSWOP( YSAVE(1,K), R, N, NOLD, NV)
   20 CONTINUE
C      UPDATE RESWK WITH XNP(NOP)
       DO 25 I=1,NOP
   25     RESWK(I) = WKMON(I)
       IMON = 3
C      IMON=3 TELLS INTEGRATOR TO RESTART WITH GIVEN YSAVE VALUES
      RETURN
       END
