C  THIS FILE CONTAINS THE SPRINT ROUTINES FOR SPARSE MATRIX FORMULATION
C  AND DECOMPOSITION OF THE JACOBIAN MATRIX.
C
C  28/7/87  LINE 1959  .LE. CHANGED TO .LT. AS RECOMMENDED BY HINDMARSH.
C  27/4/88  LINE 743  R.H.S. MULTIPLIED BY -1 FOR CORRECT ANALYTIC JAC.
C
C  IMPORTANT
C  ---------    BEFORE CALLING THE SPRINT PACKAGE WITH THE SPARSE
C               JACOBIAN FORMING ROUTINE PREPJS AND ITS ASSOCIATED
C               BACK SUBSTITUTION ROUTINE THE SETUP ROUTINE SMTSET
C               MUST BE CALLED .
C
       SUBROUTINE SMTSET( NEQ, NMOSS, SENS, NWKJAC, NIWJAC,
     1                    NELEMT, ISET, JSINFO)
      INTEGER NMOSS, NWKJAC, NIWJAC, NELEMT, ISET, NEQ,JSINFO(4)
      DOUBLE PRECISION SENS
C**********************************************************************
C  ROUTINE FOR THE SPARSE MATRIX SETUP FOR THE SPRINT PACKAGE. THIS
C  ROUTINE SETS UP THE POINTERS USED BY THE SPRINT INTERFACE TO THE
C  YALE SPARSE MATRIX PACKAGE.
C
C  INPUT PARAMETERS
C  NEQ   :THE NUMBER OF DIFFERENTIAL ALGEBRAIC EQUATIONS BEING SOLVED.
C  NMOSS : THIS METHOD INDICATES THE METHOD TO BE USED TO OBTAIN THE
C          SPARSITY STRUCTURE OF THE JACOBIAN MATRIX.
C          = 0 MEANS THAT THE USER HAS SUPPLIED THE SPARSITY PATTERN
C              IN THE INTEGER WORKSPACE JACPVT,(SEE BELOW). THE
C              JACOBIAN MATRIX WILL BE COMPUTED USING DIFFERENCES.
C              N.B. IF THE SPARSITY PATTERN CHANGES IN MID-INTEGRATION
C                   AND THIS OPTION IS IN USE , THE USER MUST FORCE AN
C                   EXIT FROM SPRINT (VIA THE RESID ROUTINE AND IRES =2)
C                   RESET THE SPARSITY PATTERN AND RE-ENTER SPRINT.
C          = 1 USER SUPPLIED JACOBIAN FORMING ROUTINE HAS BEEN GIVEN
C              AND WILL BE USED TO COMPUTE THE SPARSITY PATTERN .
C          = 2 SPARSITY PATTERN OF THE JACOBIAN IS ESTIMATED BY NEQ
C              FUNCTION CALLS TO THE RESIDUAL ROUTINE .
C
C SENS     THRESHOLD PARAMETER USED IN DETERMINING WHETHER OR NOT A
C          MATRIX ELEMENT EXISTS. IN THE CASE WHEN THIS IS SET TO ZERO
C          THE ROUTINE USES SENS = 100*UROUND WHERE UROUND IS THE UNIT
C          ROUNDOFF ERROR FOR THE MACHINE.
C
C NWKJAC   THE SIZE OF THE WORKSPACE THE USER HAS SUPPLIED FOR THE
C          SPARSE MATRIX ROUTINES.
C          THE SIZE OF THIS WORKSPACE CANNOT BE PRECISELY DETERMINED IN
C          ADVANCE AS IT DEPENDS ON BOTH THE NUMBER OF NON-ZERO ELEMENTS
C          AND ON THEIR SPARSITY PATTERN. THE STARTING ESTIMATE USED
C          HERE IS 4 * NEQ + 11 * NEQ /I WHERE I = 2 IF DOUBLE PRECISION
C          IS BEING USED AND I = 1 IF SINGLE PRECSION IS BEING USED.
C          SHOULD NWKJAC PROVE TO BE TOO SMALL AND AN ERROR IS RETURNED
C          FROM SPRINT , RECALL THIS ROUTINE WITH ISET = 2 AND ON ITS
C          RETURN LOOK AT THE ARRAY JSINFO (SEE BELOW).
C ****  WARNING ********************************************************
C *        MEMORY OVERWRITES WILL OCCUR IF YOU SET NWKJAC LARGER THAN  *
C *        THE ACTUAL SIZE OF THE ARRAY WKJAC IN THE CALLING PROGRAM.  *
C **********************************************************************
C
C NIWJAC   THE SIZE OF THE INTEGER WORKSPACE WHICH THE USER HAS SUPPLIED
C          FOR THE LINEAR ALGEBRA ROUTINES, JACPVT IN THE SPRINT CALL.
C          THIS ARRAY IS ONLY USED IF IMOSS = 0 IN WHICH CASE IT HOLDS
C          THE SPARSITY PATTERN. THE DIMENSION OF THIS ARRAY IN THAT
C          CASE MUST BE NEQ + NELEMT WHERE THE PARAMETER NELEMT IS
C          DESCRIBED BELOW. THE ARRAY ITSELF IS SPLIT INTO TWO
C          JACPVT(J) = IA(J) , J = 1,NEQ
C          JACPVT(NEQ+K) = JA(K) , K = 1, NELEMT.
C          THESE TWO ARRAYS DESCRIBE THE SPARSITY PATTERN TO BE USED FOR
C          THE JACOBIAN MATRIX. JA CONTAINS THE ROW INDICES WHERE NON-
C          ZERO ELEMENTS OCCUR, READING IN COLUMNWISE ORDER, AND IA
C          CONTAINS THE STARTING LOCATIONS IN JA OF THE DESCRIPTIONS OF
C          COLUMNS 1,..,NEQ IN THAT ORDER ,WITH IA(1) = 1. THUS FOR EACH
C          COLUMN INDEX J = 1,..,NEQ , THE VALUES OF THE ROW INDEX I IN
C          COLUMN J WHERE A NON-ZERO ELEMENT MAY OCCUR ARE GIVEN BY
C             I = JA(K) WHERE IA(J) .LE. K .LT. IA(J+1).
C          IF NELEMT IS THE TOTAL NUMBER OF NON-ZERO LOCATIONS ASSUMED
C          THEN THE LENGTH OF THE JA ARRAY IS NELEMT AND IA(NEQ+1) MUST
C          BE NELEMT + 1. DUPLICATE ENTRIES ARE NOT ALLOWED.
C
C NELEMT   AN OPTIONAL INPUT WHICH IS SUPPLIED ONLY BY THE USER WHEN
C          IMOSS = 0, I.E. THE USER KNOWS THE SPARSITY PATTERN . IN THIS
C          CASE NELEMT IS THE NUMBER OF NON-ZERO ELEMENTS IN THE
C          JACOBIAN MATRIX.
C
C ISET     ON ENTRY THIS PARAMETER INDICATES THE MODE OF OPERATION OF
C          THE ROUTINE :
C           = 0, 1 IMPLIES NORMAL ENTRY BEFORE A CALL TO SPRINT
C             >  1 IMPLIES CALL TO ACCESS STATISTICS ABOUT THE SPARSE
C                  MATRIX ROUTINES ON A RETURN FROM SPRINT. THESE STATS
C                  ARE IN THE ARRAY JSINFO DESCRIBED BELOW.
C
C OUTPUT PARAMETERS
C
C ISET     ON EXIT FROM THIS ROUTINE IF THE VALUE OF ISET IS:
C              >  0 THEN WORKSPACE SIZES ARE O.K.
C              = -1 THEN THE WORKSPACES AREW TOO SMALL AND AN ERROR
C                      MESSAGE IS OUTPUT.
C
C JSINFO : INTEGER ARRAY OF LENGTH 4 USED TO RETURN INFORMATION ABOUT
C          THE WORK DONE BY THE SPARSE MATRIX ROUTINES.
C          JSINFO(1) = LREQ ; THE REQUIRED LENGTH OF THE REAL WORKSPACE
C                      NEEDED BY THE SPARSE MATRIX ROUTINES.
C          JSINFO(2) = NLU  ; THE NUMBER OF LU DECOMPOSITIONS DONE.
C          JSINFO(3) = NGP  ; THE NUMBER OF FUNCTION CALLS NEEDED TO
C                             FORM THE JACOBIAN MATRIX.
C          JSINFO(4) = NNZ  ; THE NUMBER OF NON-ZEROES IN THE JACOBIAN.
C
C**********************************************************************
C
      INTEGER IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     1   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     2   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ, LJA, LENRAT, NEL,
     3   NSLJ, NGP, NLU, NNZ, NSP, IJCNT, N, LIW, LIA, IUPPEZ
      INTEGER ITRACE,IDEV,IOVFLO
      DOUBLE PRECISION DUNFLO, UROUND
      DOUBLE PRECISION CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH
      CHARACTER*6 ALCODE, SINGLR
      COMMON /ALGCHK/ ALCODE, SINGLR
      COMMON /SPJCNT/ IJCNT, N
      COMMON /SCONS1/ DUNFLO,UROUND,IOVFLO
      COMMON /SDEV2/  ITRACE, IDEV
      COMMON /LSS001/ CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH,
     1   IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     2   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     3   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     4   NSLJ, NGP, NLU, NNZ, NSP, IUPPEZ
      COMMON /SPRECN/ LENRAT
      SAVE   /ALGCHK/, /SPJCNT/, /SCONS1/, /LSS001/, /SPRECN/
C-----------------------------------------------------------------------
      IF(ISET .GT. 1) GOTO 100
C  FIRST CALL
      N = NEQ
      MOSS = NMOSS
      ISET = 0
      IF( NEQ .LT. 0 .OR. NELEMT .LT. 0)THEN
          CALL SERROR(' SPARSE SETUP ROUTINE SMTSET- ONE OR BOTH OF THE
     1              PARAMETERS NEQ (=I1) OR NELEMT (=I2) HAS A VALUE
     2                  LESS THAN 1 ', 1, 2, NEQ, NELEMT,0,0.0D0, 0.0D0)
          ISET = -1
      END IF
      ALCODE = 'SPARSE'
      SINGLR = 'SINGLR'
      LRAT = LENRAT
      IJCNT = 0
      LWMIN = 4*N + 11*N/LRAT
      IF (MOSS .LE. 1 )THEN
         LIW = N + 1 + NELEMT
      ELSE
         LIW = 1
      END IF
      IF(LIW .GT. NIWJAC) THEN
         CALL SERROR(' SPARSE SMTSET- INTEGER WORKSPACE JACPVT OF SIZE
     1                (=I1) SMALLER THAN REQUIRED SIZE (=I2)', 1, 2,
     2                 NIWJAC, LIW, 0, 0.0D0, 0.0D0)
         ISET = -1
      END IF
      LIA = 1
      LJA = LIA + N + 1
      LIA = MIN0(LIA,LIW)
      LJA = MIN0(LJA,LIW)
      LENWK = NWKJAC
      IF(LWMIN .GT. NWKJAC)THEN
         CALL SERROR(' SPARSE SMTSET- WARNING LINEAR ALGEBRA WORK SPACE
     1                OF SIZE (=I1) SMALLER THAN ESTIMATED WORKSPACE OF
     2                SIZE (=I2)', 1, 2, NWKJAC, LWMIN, 0, 0.0D0,0.0D0)
      END IF
C
C INITIALIZE ALL REMAINING PARAMETERS. ---------------------------------
C
      MSBJ = 50
      NLU = 0
      NSLJ = 0
      CCMXJ = 0.2D0
      PSMALL = 1000.0D0*UROUND
      IF(SENS .NE. 0.0D0)THEN
        SETH = DABS(SENS)
        PSMALL = SENS
      ELSE
         SETH = PSMALL
      END IF
      RBIG = 0.01D0/PSMALL
      RETURN
C
C   SUSEQUENT CALL OF THE ROUTINE
C
100   CONTINUE
      JSINFO(1) = LREQ
      JSINFO(2) = NLU
      JSINFO(3) = NGP
      JSINFO(4) = NNZ
C----------------- END OF ROUTINE SMTSET -------------------------------
      RETURN
      END
      SUBROUTINE PREP (NEQ, Y, YDOT, YH, NYH, SAVF, EWT, FTEM, IA, JA,
     1                     WK, IWK, IPPER, H, EL0, TN, JAC)
C********************************************************************
      INTEGER NEQ, IA, JA, IWK, IPPER, NYH
      INTEGER IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     1   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     2   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     3   NSLJ, NGP, NLU, NNZ, NSP, IUPPEZ
      INTEGER I, IBR, IER, IPTT1, IPTT2, J, K, KNEW,
     1   KMAX, KMIN, LDIF, LENIGP, LIWK, MAXG, MOSSP1, NP1, N
      INTEGER ITRACE, IDEV
      DOUBLE PRECISION Y, YH, SAVF, EWT, FTEM, WK, EL0, H, TN, YDOT
      DOUBLE PRECISION CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH
      DOUBLE PRECISION DQ, DYJ, ERWT, FAC, YJ, ELH, YDOTJ
      CHARACTER*6 ALCODE, SINGLR
      DIMENSION NEQ(1), Y(1), YH(NYH,1), SAVF(1), EWT(1), FTEM(1),
     1   IA(1), JA(1), WK(1), IWK(1), YDOT(1)
      EXTERNAL JAC
      COMMON /LSS001/ CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH,
     1   IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     2   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     3   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     4   NSLJ, NGP, NLU, NNZ, NSP, IUPPEZ
       COMMON /ALGCHK/ ALCODE, SINGLR
       COMMON /SDEV2/ ITRACE, IDEV
C
C     COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /PRPSAV/ DQ, DYJ, ELH, ERWT, FAC, YJ, YDOTJ,
     1   I, IBR, IPTT2, J, K, KMAX, KMIN, KNEW, LIWK, LDIF, LENIGP,
     2   MAXG, MOSSP1, N, NP1
      SAVE   /ALGCHK/, /PRPSAV/, /LSS001/
C-----------------------------------------------------------------------
C THIS ROUTINE PERFORMS PREPROCESSING RELATED TO THE SPARSE LINEAR
C SYSTEMS THAT MUST BE SOLVED IF MITER = 1 OR 2.
C THE OPERATIONS THAT ARE PERFORMED HERE ARE..
C  * COMPUTE SPARSENESS STRUCTURE OF JACOBIAN ACCORDING TO MOSS,
C  * COMPUTE GROUPING OF COLUMN INDICES (MITER = 2),
C  * COMPUTE A NEW ORDERING OF ROWS AND COLUMNS OF THE MATRIX,
C  * REORDER JA CORRESPONDING TO THE NEW ORDERING,
C  * PERFORM A SYMBOLIC LU FACTORIZATION OF THE MATRIX, AND
C  * SET POINTERS FOR SEGMENTS OF THE IWK/WK ARRAY.
C IN ADDITION TO VARIABLES DESCRIBED PREVIOUSLY, PREP USES THE
C FOLLOWING FOR COMMUNICATION..
C Y      = CURRENT SOLUTION VECTOR AT TIME TN
C YDOT   = CURRENT VALUES OF THE TIME DERIV AT TIME TN
C H,TN, EL0  CURRENT STEPSIZE ,TIME AND CONST IN YDOT.
C YH     = THE HISTORY ARRAY.  ONLY THE FIRST COLUMN, CONTAINING THE
C          CURRENT Y VECTOR, IS USED AND THE SECOND COLUMN CONTAINING
C          ITS CURRENT TIME DERIVATIVE MULTIPLIED BY H ARE USED.
C          OF DIMENSION (NYH, 1)
C SAVF   = A WORK ARRAY OF LENGTH NEQ, USED ONLY IF MOSS .NE. 0.
C          ON ENTRY IS ASSUMED TO CONTAIN THE CURRENT RESIDUAL.
C EWT    = ARRAY OF LENGTH NEQ CONTAINING (INVERTED) ERROR WEIGHTS.
C          USED ONLY IF MOSS = 2 OR IF ISTATE = MOSS = 1.
C FTEM   = A WORK ARRAY OF LENGTH NEQ, IDENTICAL TO ACOR IN THE DRIVER,
C          USED ONLY IF MOSS = 2.
C WK     = A REAL WORK ARRAY OF LENGTH LENWK, IDENTICAL TO WM IN
C          THE DRIVER.
C IWK    = INTEGER WORK ARRAY, ASSUMED TO OCCUPY THE SAME SPACE AS WK.
C LENWK  = THE LENGTH OF THE WORK ARRAYS WK AND IWK.
C ISTATC = A COPY OF THE DRIVER INPUT ARGUMENT ISTATE (= 1 ON THE
C          FIRST CALL, = 3 ON A CONTINUATION CALL).
C IYS    = FLAG VALUE FROM ODRV OR CDRV.
C IPPER  = OUTPUT ERROR FLAG WITH THE FOLLOWING VALUES AND MEANINGS..
C          0  NO ERROR.
C         -1  INSUFFICIENT STORAGE FOR INTERNAL STRUCTURE POINTERS.
C         -2  INSUFFICIENT STORAGE FOR JGROUP.
C         -3  INSUFFICIENT STORAGE FOR ODRV.
C         -4  OTHER ERROR FLAG FROM ODRV (SHOULD NEVER OCCUR).
C         -5  INSUFFICIENT STORAGE FOR CDRV.
C         -6  OTHER ERROR FLAG FROM CDRV.
C-----------------------------------------------------------------------
      IF(IPPER .GT. 0) GOTO 108
      IF(ALCODE .NE. 'SPARSE')THEN
         CALL SERROR(' SPARSE PREP ERROR-SETUP ROUTINE SMTSET WAS NOT
     1   CALLED PRIOR TO THE SPRINT ENTRY', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         IPPER = -2
         RETURN
      END IF
      IBIAN = LRAT*2
      IPIAN = IBIAN + 1
      N = NEQ(1)
      ELH = H * EL0
      NP1 = N + 1
      IPJAN = IPIAN + NP1
      IBJAN = IPJAN - 1
      LIWK = LENWK*LRAT
      IF (IPJAN+N-1 .GT. LIWK) GO TO 260
      MOSSP1 = MOSS + 1
      GOTO (30, 70, 100),MOSSP1
C
C MOSS = 0.  PROCESS USER-S IA,JA.
C
 30   KNEW = IPJAN
      KMIN = IA(1)
      IUPPEZ = 0
      IWK(IPIAN) = 1
      DO 60 J = 1,N
        KMAX = IA(J+1) - 1
        IF (KMIN .GT. KMAX) GO TO 45
        DO 40 K = KMIN,KMAX
          I = JA(K)
C         SEE IF THE WORKSPACE IS LARGE ENOUGH
          IF (KNEW .GT. LIWK) GO TO 260
          IF(I .NE. J)IUPPEZ = IUPPEZ + 1
          IWK(KNEW) = I
          KNEW = KNEW + 1
 40       CONTINUE
 45     IF (KNEW .GT. LIWK) GO TO 260
 50     IWK(IPIAN+J) = KNEW + 1 - IPJAN
        KMIN = KMAX + 1
 60     CONTINUE
      GO TO 140
C
C MOSS = 1.  COMPUTE STRUCTURE FROM USER-SUPPLIED JACOBIAN ROUTINE JAC.
C
 70    K = IPJAN
       IUPPEZ     = 0
       IWK(IPIAN) = 1
       DO 90 J = 1,N
          IF(K .GT. LIWK) GOTO 260
          DO 75 I = 1,N
             Y(I)    = YH(I,1)
             YDOT(I) = YH(I,2)/H
 75          SAVF(I) = 0.0D0
          CALL JAC( NEQ, TN, Y,YDOT,H,EL0,J,IWK(IPIAN),IWK(IPJAN),SAVF)
          DO 80 I = 1,N
             IF( DABS(SAVF(I)) .LE. SETH) GOTO 80
             IF(I.NE.J)IUPPEZ = IUPPEZ + 1
             IF(ITRACE .EQ. 3)WRITE(IDEV,751)I,J
 751         FORMAT(' ROW',I8,'  COL',I8,' IS FULL')
             IF(K .GT. LIWK) GOTO 260
             IWK(K) = I
             K = K + 1
 80       CONTINUE
          IWK(IPIAN+J) = K+1 - IPJAN
 90     CONTINUE
        GOTO 140
C
C MOSS = 2.  COMPUTE STRUCTURE FROM RESULTS OF N + 1 CALLS TO F. -------
C
 100  K = IPJAN
      IWK(IPIAN) = 1
      IUPPEZ= 0
      IPPER = 0
 105  IPPER = IPPER + 1
      J = IPPER
        IF (K .GT. LIWK) GO TO 260
        YJ     = YH(J,1)
        YDOTJ  = YH(J,2)/H
        ERWT   = 1.0D0/EWT(J)
        DYJ    = DSIGN(ERWT,YJ)
        Y(J)   = YJ + DYJ
        YDOT(J)= YDOT(J) + DYJ/ELH
        RETURN
C
C   FUNCTION CALL - REVERSE COMMUNICATIONS EXIT
C
108     J = IPPER
        Y(J)    = YJ
        YDOT(J) = YDOTJ
        DO 110 I = 1,N
          DQ = (FTEM(I) - SAVF(I))/DYJ
          IF (DABS(DQ) .LE. SETH) GO TO 110
          IF(ITRACE.GE.3)WRITE(IDEV,109)I,J
 109      FORMAT(' ROW',I8,'  COL',I8,' IS FULL')
C         CHECK IF THE WORKSPACE IS LARGE ENOUGH
          IF (K .GT. LIWK) GO TO 260
          IWK(K) = I
          IF(I.NE.J)IUPPEZ = IUPPEZ + 1
          K = K + 1
 110      CONTINUE
        IWK(IPIAN+J) = K + 1 - IPJAN
        IF(IPPER .LT. N)GOTO 105
        IPPER = 0
C
 140  CONTINUE
      NNZ = IWK(IPIAN + N) -1
      IF(ITRACE.GE.3)WRITE(IDEV,141)NNZ, IUPPEZ
 141  FORMAT(' NUMBER OF ELEMENTS IN JACOBIAN IS',I8/
     1       ' NUMBER OF OFF-DIAG ELEMENTS    IS',I8)
      NGP = 0
      LENIGP = 0
      IPIGP = IPJAN + NNZ
C
C COMPUTE GROUPING OF COLUMN INDICES (MITER = 2). ----------------------
C
      MAXG = NP1
      IPJGP = IPJAN + NNZ
      IBJGP = IPJGP - 1
      IPIGP = IPJGP + N
      IPTT1 = IPIGP + NP1
      IPTT2 = IPTT1 + N
      LREQ = IPTT2 + N - 1
      IF (LREQ .GT. LIWK) GO TO 210
      CALL JGROUP (N, IWK(IPIAN), IWK(IPJAN), MAXG, NGP, IWK(IPIGP),
     1   IWK(IPJGP), IWK(IPTT1), IWK(IPTT2), IER)
      IF (IER .NE. 0) GO TO 210
      LENIGP = NGP + 1
C
C COMPUTE NEW ORDERING OF ROWS/COLUMNS OF JACOBIAN. --------------------
C
 160  IPR = IPIGP + LENIGP
      IPC = IPR
      IPIC = IPC + N
      IPISP = IPIC + N
      IPRSP = (IPISP - 2)/LRAT + 2
      IESP = LENWK + 1 - IPRSP
      IF (IESP .LT. 0) GO TO 220
      IBR = IPR - 1
      DO 170 I = 1,N
 170    IWK(IBR+I) = I
      NSP = LIWK + 1 - IPISP
      CALL ODRV (N, IWK(IPIAN), IWK(IPJAN), WK, IWK(IPR), IWK(IPIC),
     1   NSP, IWK(IPISP), 1, IYS)
      IF (IYS .EQ. 11*N+1) THEN
          GO TO 230
      ELSE IF(IYS .EQ. 10*N+1)THEN
          LREQ = 3 * N + IUPPEZ * 2
          GOTO 221
      ELSE IF( IYS .NE. 0)THEN
          LREQ = LENWK + N
          GOTO 221
      END IF
C
C REORDER JAN AND DO SYMBOLIC LU FACTORIZATION OF MATRIX. --------------
C
      IPA = LENWK + 1 - NNZ
      NSP = IPA - IPRSP
      LREQ = MAX0(12*N/LRAT, 6*N/LRAT+2*N+NNZ) + 3
      LREQ = LREQ + IPRSP - 1 + NNZ
      IF (LREQ .GT. LENWK) GO TO 240
      IBA = IPA - 1
      DO 180 I = 1,NNZ
 180    WK(IBA+I) = 0.0D0
      IPISP = LRAT*(IPRSP - 1) + 1
      CALL CDRV (N,IWK(IPR),IWK(IPC),IWK(IPIC),IWK(IPIAN),IWK(IPJAN),
     1   WK(IPA),WK(IPA),WK(IPA),NSP,IWK(IPISP),WK(IPRSP),IESP,5,IYS)
      LREQ = LENWK - IESP
      IF (IYS .EQ. (10*N+1)) GO TO 240
      IF (IYS .NE. 0) GO TO 250
      CALL ADJLR (N, IWK(IPISP), LDIF)
      IF (LDIF .GT. 0) LREQ = LREQ + (LDIF - 1)/LRAT + 1
      IF (LREQ .GT. LENWK)GOTO 240
      NSP = NSP + LREQ - LENWK
      IPA = LREQ + 1 - NNZ
      IBA = IPA - 1
      IPPER = 0
      RETURN
C
 210  IPPER = -2
      LREQ = (LREQ - 1)/LRAT + 1
      CALL SERROR(' SPARSE-PREP LARGER WORKSPACE NEEDED FOR JGROUP USER
     1 PROVIDED (=I1) AMOUNT NEEDED IS (=I2)', 1, 2, LENWK, LREQ, 0,
     2 0.0D0, 0.0D0)
      RETURN
C
 220  IF(IESP .LE. 0)LREQ = LENWK - IESP + (2*NNZ + N)/LRAT
 221  IPPER = -2
      CALL SERROR(' SPARSE-PREP LARGER WORKSPACE REQUIRED FOR THE MIN
     1   DEGREE ORDERING OF THE SPARSE MATRIX.THE AMOUNT PROVIDED (=I1),
     2   WHILE AN ESTIMATE OF THE AMOUNT NEEDED IS (=I2)'
     3   , 1, 2, LENWK, LREQ, 0, 0.0D0, 0.0D0)
      RETURN
C
 230  IPPER = -2
      LREQ = LENWK + N
      CALL SERROR(' SPARSE-PREP IMPOSSIBLE ERROR OCCURED IN PERFORMING
     1 THE MINIMUM DEGREE ORDERING OF THE SPARSE MATRIX - PROBABLY NOT
     2 ENOUGH  WORKSPACE -TRY INCREASING THE SIZE OF ARRAY WKJAC FROM
     4 (=I1) TO (=I2) ',
     3 1, 1 ,LENWK, LREQ, 0, 0.0D0, 0.0D0)
      RETURN
C
 240  IPPER = -2
      CALL SERROR(' SPARSE PREP -NOT ENOUGH STORE FOR CDRV TO PERFORM
     1 SYMBOLIC L-U DECOMPOSITION OF SPARSE JACOBIAN MATRIX- WORKSPACE
     2 PROVIDED IS OF SIZE (=I1) WORKSPACE NEEDED IS  OF SIZE (=I2)',
     3  1, 2, LENWK, LREQ, 0, 0.0D0, 0.0D0)
      RETURN
C
 250  IPPER = -2
      CALL CDRERR( IYS,N, IPPER)
C          ROUTINE TO HANDLE ERROR MESSAGE NUMBER FROM CDRV
      LREQ = LENWK
      RETURN
C
 260  IPPER = -2
      LREQ = 2 + (2*N + 1)/LRAT
      LREQ = MAX0(LENWK+1,LREQ)
      CALL SERROR(' SPARSE PREP- NOT ENOUGH STORE PROVIDED FOR INTERNAL
     1  STORAGE POINTERS, (=I1) UNITS OF STORE NEEDED , AMOUNT PROVIDED
     2  IS (=I2)', 1, 2, LREQ, LENWK, 0, 0.0D0, 0.0D0)
      RETURN
C----------------------- END OF SUBROUTINE PREP ------------------------
      END
      SUBROUTINE ADJLR (N, ISP, LDIF)
      INTEGER N, ISP, LDIF
      DIMENSION ISP(1)
C-----------------------------------------------------------------------
C THIS ROUTINE COMPUTES AN ADJUSTMENT, LDIF, TO THE REQUIRED
C INTEGER STORAGE SPACE IN IWK (SPARSE MATRIX WORK SPACE).
C THIS IS TO ACCOUNT FOR THE POSSIBILITY THAT THE COMPRESSED
C POINTER STORAGE SCHEME MAY REQUIRE MORE SPACE THAN THE STANDARD ONE.
C-----------------------------------------------------------------------
      INTEGER IP, NPL, NPU, NTEST, NZLU
C
      IP = 2*N + 1
C GET NPL = IJL(N) AND NPU = IJU(N). -----------------------------------
      NPL = ISP(IP)
      NPU = ISP(IP+IP)
      NTEST = 3*N + 1 + MAX0(NPL,NPU) - MIN0(NPL,NPU)
C NZLU = (SIZE OF L) + (SIZE OF U) = (IL(N+1)-IL(1)) + (IU(N+1)-IU(1)).
      NZLU = ISP(N+1) - ISP(1) + ISP(IP+N+1) - ISP(IP+1)
      LDIF = MAX0(0, NTEST - NZLU)
      RETURN
C----------------------- END OF SUBROUTINE ADJLR -----------------------
      END
      SUBROUTINE JGROUP (N,IA,JA,MAXG,NGRP,IGP,JGP,INCL,JDONE,IER)
C
      INTEGER N, IA, JA, MAXG, NGRP, IGP, JGP, INCL, JDONE, IER
      DIMENSION IA(1), JA(1), IGP(1), JGP(N), INCL(N), JDONE(N)
C-----------------------------------------------------------------------
C THIS SUBROUTINE CONSTRUCTS GROUPINGS OF THE COLUMN INDICES OF
C THE JACOBIAN MATRIX, USED IN THE NUMERICAL EVALUATION OF THE
C JACOBIAN BY FINITE DIFFERENCES.
C
C INPUT..
C N      = THE ORDER OF THE MATRIX.
C IA,JA  = SPARSE STRUCTURE DESCRIPTORS OF THE MATRIX BY ROWS.
C MAXG   = LENGTH OF AVAILABLE STORATE IN THE IGP ARRAY.
C
C OUTPUT..
C NGRP   = NUMBER OF GROUPS.
C JGP    = ARRAY OF LENGTH N CONTAINING THE COLUMN INDICES BY GROUPS.
C IGP    = POINTER ARRAY OF LENGTH NGRP + 1 TO THE LOCATIONS IN JGP
C          OF THE BEGINNING OF EACH GROUP.
C IER    = ERROR INDICATOR.  IER = 0 IF NO ERROR OCCURRED, OR 1 IF
C          MAXG WAS INSUFFICIENT.
C
C INCL AND JDONE ARE WORKING ARRAYS OF LENGTH N.
C-----------------------------------------------------------------------
      INTEGER I, J, K, KMIN, KMAX, NCOL, NG
C
      IER = 0
      DO 10 J = 1,N
 10     JDONE(J) = 0
      NCOL = 1
      DO 60 NG = 1,MAXG
        IGP(NG) = NCOL
        DO 20 I = 1,N
 20       INCL(I) = 0
        DO 50 J = 1,N
C REJECT COLUMN J IF IT IS ALREADY IN A GROUP.--------------------------
          IF (JDONE(J) .EQ. 1) GO TO 50
          KMIN = IA(J)
          KMAX = IA(J+1) - 1
          DO 30 K = KMIN,KMAX
C REJECT COLUMN J IF IT OVERLAPS ANY COLUMN ALREADY IN THIS GROUP.------
            I = JA(K)
            IF (INCL(I) .EQ. 1) GO TO 50
 30         CONTINUE
C ACCEPT COLUMN J INTO GROUP NG.----------------------------------------
          JGP(NCOL) = J
          NCOL = NCOL + 1
          JDONE(J) = 1
          DO 40 K = KMIN,KMAX
            I = JA(K)
 40         INCL(I) = 1
 50       CONTINUE
C STOP IF THIS GROUP IS EMPTY (GROUPING IS COMPLETE).-------------------
        IF (NCOL .EQ. IGP(NG)) GO TO 70
 60     CONTINUE
C ERROR RETURN IF NOT ALL COLUMNS WERE CHOSEN (MAXG TOO SMALL).---------
      IF (NCOL .LE. N) GO TO 80
      NG = MAXG
 70   NGRP = NG - 1
      RETURN
 80   IER = 1
      RETURN
C----------------------- END OF SUBROUTINE JGROUP ----------------------
      END
      SUBROUTINE PREPJS ( NEQ, Y, YH, NYH, EWT, FTEM, SAVF, YDOT, WK,
     1                 JACPVT, IFJ, H, EL0, TN, IFUNC, JAC, IDAE)
C***********************************************************************
      INTEGER NEQ, NYH, IFJ, IFUNC, JACPVT(1), IDAE(1)
      INTEGER ICURR, IFSAVE, NP1, IJCNT, N, IPREP
      DOUBLE PRECISION Y, YH, EWT, FTEM, SAVF, WK, YDOT, H, EL0, TN
      DIMENSION NEQ(1), Y(1), YH(NYH,1), EWT(1), FTEM(1), SAVF(1),
     1   WK(1), YDOT(1)
      EXTERNAL JAC
C----------------------------------------------------------------------
C       DRIVING ROUTINE TO CALL EITHER THE ROUTINE PREP TO DETERMINE
C       THE SPARSITRY PATTERN OF THE JACOBIAN (PREP) AN /OR TO CALL
C       THE ROUTINE TO ACTUALLY FORM THE JACOBIAN.
C----------------------------------------------------------------------
      COMMON /SPJCNT/ IJCNT, N
      COMMON /SPJSVE/ IFSAVE, ICURR, NP1, IPREP
      SAVE   /SPJCNT/, /SPJSVE/
C
      IF(N .NE. NEQ(1))THEN
C        O.D.E. PROBLEM HAS CHANGED DIMENSION RECOMPUTE SPARSITY
         N = NEQ(1)
         IJCNT = 0
      END IF
      NP1 = NEQ(1) + 2
      IF(IFUNC .NE. 0)IFSAVE = IFUNC
      IF(IFUNC .EQ. 0)THEN
         IF(ICURR .EQ. 1)GOTO 20
      ELSE
         IFJ = 0
         IF(IJCNT .GT. 0)GOTO 20
      END IF
      CALL PREP( NEQ, Y, YDOT, YH, NYH, SAVF, EWT, FTEM, JACPVT(1),
     1           JACPVT(NP1), WK, WK, IFJ, H, EL0, TN, JAC)
      IPREP = 0
C           IS SET TO 1 BELOW WHEN SYMBOLIC FACTOR FAILS.
      IF(IFJ .GT. 0)THEN
         ICURR = 0
         IFUNC  = 0
         RETURN
      ELSE
         IFUNC = IFSAVE
C      IJCNT COULD BE USED TO FORCE A PERIODIC RE-EVALUATION OF SPARSITY
         IJCNT = 1
         IF(IFJ .LT. 0)THEN
            ICURR = 1
            IPREP = 1
            RETURN
         END IF
      END IF
20    IF(IPREP .EQ. 1)THEN
C        SYMBOLIC FACTOR FAILED
         IF(IFJ .LT. -2)IFJ = -2
         RETURN
      END IF
C
C       CALL ROUTINE TO FORM THE JACOBIAN
C
      CALL PRJS( NEQ, Y, YH, NYH, EWT, FTEM, SAVF, YDOT, WK, WK, IFJ,
     1           H, EL0, TN, IFUNC, JAC, IDAE)
      IF(IFJ .GT. 0)THEN
         ICURR = 1
         IFUNC = 0
      ELSE
         IFUNC = IFSAVE
      END IF
      RETURN
C
C----------------END OF PREPJS-----------------------------------------
      END
      SUBROUTINE PRJS ( NEQ, Y, YH, NYH, EWT, FTEM, SAVF, YDOT, WK,
     1                  IWK, IFJ, H, EL0, TN, IFUNC, JAC, IDAE)
C***********************************************************************
      INTEGER NEQ, NYH, IWK, IFJ, IFUNC, IDAE(1)
      INTEGER IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     1   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     2   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     3   NSLJ, NGP, NLU, NNZ, NSP, JPFLAG, NST, IUPPEZ
      INTEGER I, IMUL, J, JJ, JOK, JMAX, JMIN, K, KMAX, KMIN, NG, N
      INTEGER ITRACE, IDEV, IOVFLO
      DOUBLE PRECISION Y, YH, EWT, FTEM, SAVF, WK, YDOT, H, EL0, TN
      DOUBLE PRECISION CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH
      DOUBLE PRECISION CON, FAC, HL0, PIJ, R, R0, RCON, RCONT,
     1   SRUR, VNORM, DUNFLO, UROUND, HL0OLD, FAC1
      CHARACTER*6 ALCODE, SINGLR
      DIMENSION NEQ(1), Y(1), YH(NYH,1), EWT(1), FTEM(1), SAVF(1),
     1   WK(1), IWK(1), YDOT(1)
      EXTERNAL JAC
      COMMON /ALGCHK/ ALCODE, SINGLR
      COMMON /SDEV2/  ITRACE, IDEV
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
      COMMON /LSS001/ CON0, CONMIN, CCMXJ, PSMALL, RBIG, SETH,
     1   IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     2   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     3   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     4   NSLJ, NGP, NLU, NNZ, NSP, IUPPEZ
C
C    COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /PRJSAV/ CON, FAC, HL0, HL0OLD, PIJ, R, R0, RCONT, SRUR,
     1   RCON, I, J, JJ, JMAX, JMIN, JOK, JPFLAG, K, KMAX, KMIN, N, NG
      SAVE   /PRJSAV/, /LSS001/, /SCONS1/, /ALGCHK/
C
C-----------------------------------------------------------------------
C PRJS IS CALLED TO COMPUTE AND PROCESS THE MATRIX DF/DYDOT - H*EL0*J
C WHERE J = DF/DY AND BOTH J AND DF/DYDOT ARE
C COMPUTED BY COLUMNS, EITHER BY THE USER-SUPPLIED ROUTINE JAC
C IF MITER = 1, OR BY FINITE DIFFERENCING IF MITER = 2.
C IF MITER = 1 OR 2, AND IF THE EXISTING VALUE OF THE JACOBIAN
C (AS CONTAINED IN P) IS CONSIDERED ACCEPTABLE, THEN A NEW VALUE OF
C P IS RECONSTRUCTED FROM THE OLD VALUE.  IN ANY CASE, WHEN MITER
C IS 1 OR 2, THE P MATRIX IS SUBJECTED TO LU DECOMPOSITION IN CDRV.
C P AND ITS LU DECOMPOSITION ARE STORED (SEPARATELY) IN WK.
C
C IN ADDITION TO VARIABLES DESCRIBED PREVIOUSLY, COMMUNICATION
C WITH PRJS USES THE FOLLOWING..
C Y     = ARRAY CONTAINING PREDICTED VALUES ON ENTRY.
C YDOT  = ARRAY CONTAINING PREDICTED VALUES OF TIME DERIVATIVE ON ENTRY
C FTEM  = WORK ARRAY OF LENGTH N (ACOR IN STODE).
C SAVF  = ARRAY CONTAINING F EVALUATED AT PREDICTED Y.
C WK    = REAL WORK SPACE FOR MATRICES.  ON OUTPUT IT CONTAINS THE
C         THE SPARSE LU DECOMPOSITION IF MITER IS 1 OR 2.
C         STORAGE OF MATRIX ELEMENTS STARTS AT WK(1).
C IWK   = INTEGER WORK SPACE FOR MATRIX-RELATED DATA, ASSUMED TO
C         BE EQUIVALENCED TO WK.  IN ADDITION, WK(IPRSP) AND IWK(IPISP)
C         ARE ASSUMED TO HAVE IDENTICAL LOCATIONS.
C EL0   = EL(1) (INPUT).
C IERPJ = OUTPUT ERROR FLAG (IN COMMON).
C       = 0 IF NO ERROR.
C       = 1  IF ZERO PIVOT FOUND IN CDRV.
C       = 2  IF A SINGULAR MATRIX AROSE WITH MITER = 3.
C       = -1 IF INSUFFICIENT STORAGE FOR CDRV (SHOULD NOT OCCUR HERE).
C       = -2 IF OTHER ERROR FOUND IN CDRV (SHOULD NOT OCCUR HERE).
C H     =  CURRENT STEPSIZE IN USE
C TN    =  CURRENT VALUE OF THE TIME STEP
C IFJ   =  OUTPUT ERROR FLAG
C          < 0 IF THE JACOBIAN MATRIX IS SINGULAR
C          = 0 IF THE DECOMPOSTION WENT 0.K.
C          > 0 IF THIS IS A REVERSE COMMUNICATION EXIT.
C IFUNC = INDICATOR TO DETERMINE THE MODE OF OPERATION OF THE ROUTINE
C          = 0 MEANS REVERSE COMMUNICATION ENTRY.
C          = 1 MEANS CALCULATE THE NORMAL JACOBIAN MATRIX
C THIS ROUTINE ALSO USES UROUND IN COMMON/SCONS1/
C-----------------------------------------------------------------------
      IF(IFJ .GT. 0)GOTO 215
      N = NEQ(1)
      HL0 = H*EL0
      CON = -HL0
C
C  THE JACOBIAN IS TO BE REEVALUATED. --------------------------------
C
 20   NSLJ = 0
      IPLOST = 0
      HL0OLD = HL0
      CONMIN = DABS(CON)
      IF(MOSS .NE. 1)GOTO 200
C
C  MOSS = 1 - THE JACOBIAN IS TO BE FORMED FROM THE USERS ROUTINE.
C
      KMIN = IWK(IPIAN)
      IF( ITRACE .GE. 3)WRITE(IDEV,119)
 119  FORMAT(' DUMP OF ANALYTIC JACOBIAN (SCALED BY SYSTEM)')
      DO 130 J = 1,N
         KMAX = IWK(IPIAN+J) -1
         DO 110 I = 1,N
            Y(I)    = YH(I,1)
            YDOT(I) = YH(I,2)/H
 110        FTEM(I) = 0.0D0
         CALL JAC( NEQ, TN, Y, YDOT, H, EL0, J, IWK(IPIAN), IWK(IPJAN),
     1             FTEM)
         DO 120 K = KMIN,KMAX
            I = IWK(IBJAN+K)
 120        WK(IBA + K ) = -FTEM(I)*(IDAE(I) + (1-IDAE(I))/(EL0*H))
         IF(ITRACE.GE. 3)THEN
            DO 125 K = KMIN,KMAX
               I = IWK(IBJAN+K)
               WRITE(IDEV,126)I,J,WK(IBA+K)
 126           FORMAT(' ROW',I7,' COL',I7,' OF JAC IS',D11.3)
 125        CONTINUE
         END IF
         KMIN = KMAX + 1
 130   CONTINUE
       GOTO 290
C
C IF MITER = 2, MAKE NGP CALLS TO F TO APPROXIMATE J AND P. ------------
C
 200  CONTINUE
      FAC = VNORM(N, SAVF, EWT)
      R0 = 1000.0D0 * DABS(H) * UROUND * N * FAC
      IF (R0 .EQ. 0.0D0) R0 = 1.0D0
      SRUR = DSQRT( UROUND)
      JMIN = IWK(IPIGP)
      IFJ = 0
 205  IFJ = IFJ + 1
        NG = IFJ
        JMAX = IWK(IPIGP+NG) - 1
        DO 210 J = JMIN,JMAX
          JJ = IWK(IBJGP+J)
          R = DMAX1(SRUR*DABS(Y(JJ)),R0/EWT(JJ))
C   THE FOLLOWING LINE BY M.BERZINS 3/12/83. REMOVE FOR HINDMARSH CODE
          R = DMAX1(R, UROUND)
          YDOT(JJ) = YH(JJ,2)/H + R/HL0
          Y(JJ)    = YH(JJ,1)   + R
 210    CONTINUE
        RETURN
C
C    REVERSE COMMUNICATION EXIT FOR A FUNCTION CALL
C
 215    CONTINUE
        DO 230 J = JMIN,JMAX
          JJ = IWK(IBJGP+J)
          Y(JJ)    = YH(JJ,1)
          YDOT(JJ) = YH(JJ,2)/H
          R = DMAX1(SRUR*DABS(Y(JJ)),R0/EWT(JJ))
C    THE FOLLOWING LINE BY M.BERZINS 3/12/83. REMOVE FOR HINDMARSH CODE
          R = DMAX1(R, UROUND)
          FAC = -HL0/R
          FAC1= -1.0D0/R
          KMIN =IWK(IBIAN+JJ)
          KMAX =IWK(IBIAN+JJ+1) - 1
          DO 220 K = KMIN,KMAX
            I = IWK(IBJAN+K)
            WK(IBA+K) = (FTEM(I)-SAVF(I))*(FAC*IDAE(I)+(1-IDAE(I))*FAC1)
 220        CONTINUE
          IF(ITRACE.GE. 3)THEN
          DO 235 K = KMIN,KMAX
            I = IWK(IBJAN+K)
            WRITE(IDEV,219)I,JJ,WK(IBA+K)
 219        FORMAT(' ROW',I7,' COL',I7,' OF JAC IS',D11.3)
 235        CONTINUE
          END IF
 230      CONTINUE
        JMIN = JMAX + 1
      IF(NG .EQ. NGP)THEN
         IFJ = 0
      ELSE
         GOTO 205
      END IF
C
C DO NUMERICAL FACTORIZATION OF P MATRIX. ------------------------------
C
 290  NLU = NLU + 1
      CON0 = CON
      IFJ = 0
      DO 295 I = 1,N
 295    FTEM(I) = 0.0D0
      SINGLR = 'NSING3'
      CALL CDRV (N,IWK(IPR),IWK(IPC),IWK(IPIC),IWK(IPIAN),IWK(IPJAN),
     1   WK(IPA),FTEM,FTEM,NSP,IWK(IPISP),WK(IPRSP),IESP,2,IYS)
      IF (IYS .EQ. 0) RETURN
      IMUL = (IYS - 1)/N
      SINGLR = 'SINGLR'
C     IFJ = -1  OR -2
      CALL CDRERR( IYS, N, IFJ)
C                 TO HANDLE ERROR MESSAGE NUMBER FROM CDRV
      RETURN
C----------------------- END OF SUBROUTINE PRJS ----------------------
      END
      SUBROUTINE SOLSS (WK, IWK, X, N, IER)
C**********************************************************************
C  DUMMY ROUTINE TO INTERFACE FROM THE SPRINT BACK SUBSTITUTION INTER
C  FACE TO THE ON EDERIVED FROM LSODES. THE ONLY IMPORTANT DIFFERENCE
C  IS THAT IN LSODES THE WK ARRAY IS USED AS BOTH REAL AND INTEGER
C  ARRAYS WHEN PASSED TO SLSS AND IWK IS JUST A DUMMY ARRAY HERE.
C*********************************************************************
      INTEGER IWK, N, IER
      DOUBLE PRECISION WK, X
      CHARACTER*6 ALCODE, SINGLR
      DIMENSION WK(1), IWK(1), X(1)
      COMMON /ALGCHK/ ALCODE, SINGLR
      SAVE   /ALGCHK/
      IF(SINGLR .NE. 'NSING3')THEN
         CALL SERROR(' SOLSS - ERROR BACKSUBSTITUTION TRIED WITH EITHER
     1                 SINGULAR JACOBIAN MATRIX OR NO JACOBIAN MATRIX AT
     2                 ALL OR WITH SPRINT HAVING BEEN CALLED WITH A
     3                 JACOBIAN FORMING ROUTINE OTHER THAN PREPJS', 1,
     4                 0, 0, 0, 0, 0.0D0, 0.0D0)
         IER = -2
         RETURN
      END IF
      CALL SLSS( WK, WK, X, N, IER)
      RETURN
      END
      SUBROUTINE SLSS (WK, IWK, X, N, IER)
C**********************************************************************
      INTEGER IWK, IER, N
      INTEGER IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     1   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     2   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     3   NSLJ, NGP, NLU, NNZ, NSP, IUPPEZ
      DOUBLE PRECISION WK, X, RLSS
      DIMENSION WK(1), IWK(1), X(1)
      COMMON /LSS001/ RLSS(6),
     1   IPLOST, IESP, ISTATC, IYS, IBA, IBIAN, IBJAN, IBJGP,
     2   IPIAN, IPJAN, IPJGP, IPIGP, IPR, IPC, IPIC, IPISP, IPRSP, IPA,
     3   LENWK, LREQ, LRAT, LREST, LWMIN, MOSS, MSBJ,
     4   NSLJ, NGP, NLU, NNZ, NSP, IUPPEZ
      SAVE   /LSS001/
C-----------------------------------------------------------------------
C THIS ROUTINE MANAGES THE SOLUTION OF THE LINEAR SYSTEM ARISING FROM
C A CHORD ITERATION.
C IT CALLS CDRV TO ACCOMPLISH THIS.
C COMMUNICATION WITH SLSS USES THE FOLLOWING VARIABLES..
C WK    = REAL WORK SPACE CONTAINING THE
C         LU DECOMPOSITION OF THE MATRIX OTHERWISE.
C         STORAGE OF MATRIX ELEMENTS STARTS AT WK(1).
C IWK   = INTEGER WORK SPACE FOR MATRIX-RELATED DATA, ASSUMED TO
C         BE EQUIVALENCED TO WK.  IN ADDITION, WK(IPRSP) AND IWK(IPISP)
C         ARE ASSUMED TO HAVE IDENTICAL LOCATIONS.
C X     = THE RIGHT-HAND SIDE VECTOR ON INPUT, AND THE SOLUTION VECTOR
C         ON OUTPUT, OF LENGTH N.
C IER   = OUTPUT FLAG
C         IER   = 0  IF NO TROUBLE OCCURRED.
C         IER   = -1 IF CDRV RETURNED AN ERROR FLAG (MITER = 1 OR 2).
C                    THIS SHOULD NEVER OCCUR AND IS CONSIDERED FATAL.
C N     = THE DIMENSION OF THE O.D.E. SYSTEM
C THIS ROUTINE ALSO USES OTHER VARIABLES IN COMMON.
C-----------------------------------------------------------------------
      IER = 0
 100  CALL CDRV (N,IWK(IPR),IWK(IPC),IWK(IPIC),IWK(IPIAN),IWK(IPJAN),
     1   WK(IPA),X,X,NSP,IWK(IPISP),WK(IPRSP),IESP,4,IER)
      IF (IER .NE. 0) THEN
          I   =  IER
          CALL CDRERR( I, N, IER)
C         IER = -1 OR IER = -2
      END IF
      RETURN
C
C----------------------- END OF SUBROUTINE SOLSS -----------------------
      END
      SUBROUTINE CDRERR( IERR, NEQ, IFJ)
      INTEGER IERR, NEQ, IFJ
C***********************************************************************
C     ROUTINE TO PRINT THE ERROR MESSAGE THAT CORRESSPONDS TO THE ERROR
C     NUMBER RETURNED BY THE SPARSE MATRIX DRIVER CDRV.
C     INPUTS :
C        IERR - THE ERROR MESSAGE NUMBER GENERATED BY CDRV
C        NEQ  - THE NUMBER OF SPARSE LINEAR EQUATIONS BEING SOLVED.
C        IFJ  - SPRINT ERROR INDICATOR
C         IERR  - ERROR FLAG.  VALUES AND THEIR MEANINGS ARE --
C                     0     NO ERRORS DETECTED
C                     N+K   NULL ROW IN A  --  ROW = K
C                    2N+K   DUPLICATE ENTRY IN A  --  ROW = K
C                    3N+K   INSUFFICIENT STORAGE IN NSFC  --  ROW = K
C                    4N+1   INSUFFICIENT STORAGE IN NNFC
C                    5N+K   NULL PIVOT  --  ROW = K
C                    6N+K   INSUFFICIENT STORAGE IN NSFC  --  ROW = K
C                    7N+1   INSUFFICIENT STORAGE IN NNFC
C                    8N+K   ZERO PIVOT  --  ROW = K
C                   10N+1   INSUFFICIENT STORAGE IN CDRV
C                   11N+1   ILLEGAL PATH SPECIFICATION
C
C   OUTPUTS
C   ------      IFJ  = -2 IF THE ERROR IS A WORKSPACE ERROR
C                    = -1 IF THE MATRIX IS SINGULAR .
C
C
C**********************************************************************
      INTEGER IMODE, K
      IMODE = IERR / NEQ
      K     = IERR - IMODE * NEQ
      IFJ   = -2
      IF(IERR .EQ. 0)RETURN
      GOTO ( 10, 20, 30, 40, 50, 30, 40, 80, 5, 100, 110), IMODE
 5    CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     BUT THE ERROR MESSAGE PARAMETER HAD AN ILLEGAL VALUE - THE
     2     FORM OF THE ERROR IS NOT KNOWN',
     3             1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -2
      GOTO 1000
 10   CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     THE (=I1) TH ROW OF THE JACOBIAN MATRIX WAS FOUND TO BE ZERO'
     2            ,1, 1, K, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -1
      GOTO 1000
 20   CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     THE (=I1) TH ROW OF THE JACOBIAN MATRIX WAS FOUND TO HAVE A
     2     DUPLICATE ENTRY.', 1, 1, K, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -2
      GOTO 1000
 30   CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     THE ROUTINE NSFC DID NOT HAVE ENOUGH STORAGE WHEN FACTORING
     2     ROW NO (=I1) - TRY INCREASING THE WORKSPACE ',
     3             1, 1, K, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -2
      GOTO 1000
 40   CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     THE ROUTINE NSFC DID NOT HAVE ENOUGH STORAGE - TRY INCREASING
     2     THE SIZE OF THE WORKSPACE FOR THE JACOBIAN',
     3             1, 0,  0, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -2
      GOTO 1000
 50   CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     A NULL PIVOT WAS FOUND IN THE ROW NUMBER (=I1) ',
     2             1, 1, K, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -1
      IF( K .LE. 0 .OR. K .GT. NEQ)IFJ  = -2
      GOTO 1000
 80   CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     A ZERO PIVOT WAS FOUND IN THE ROW NUMBER (=I1) ',
     2             1, 1, K, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -1
      IF( K .LE. 0 .OR. K .GT. NEQ)IFJ  = -2
      GOTO 1000
 100  CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     INSUFFICIENT WORKSPACE WAS PROVIDED FOR THE ROUTINE CDRV -
     2     TRY INCREASING THE WORKSPACE ',
     3             1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -2
      GOTO 1000
 110  CALL SERROR(' SPARSE JACOBIAN - ERROR DETECTED  BY ROUTINE CDRV -
     1     ERROR IN SPARSE JACOBIAN PART OF SPRINT THE ROUTINE CDRV WAS
     2     ENTERED WITH AN ILLEGAL TASK SPECIFICATION ',
     3             1, 0, 0, 0, 0, 0.0D0, 0.0D0)
      IFJ  = -2
1000  CONTINUE
      RETURN
      END
      SUBROUTINE ODRV
     *     (N, IA,JA,A, P,IP, NSP,ISP, PATH, FLAG)
C
C                                                                3/12/82
C***********************************************************************
C  ODRV -- DRIVER FOR SPARSE MATRIX REORDERING ROUTINES
C***********************************************************************
C
C  DESCRIPTION
C
C    ODRV FINDS A MINIMUM DEGREE ORDERING OF THE ROWS AND COLUMNS
C    OF A MATRIX M STORED IN (IA,JA,A) FORMAT (SEE BELOW).  FOR THE
C    REORDERED MATRIX, THE WORK AND STORAGE REQUIRED TO PERFORM
C    GAUSSIAN ELIMINATION IS (USUALLY) SIGNIFICANTLY LESS.
C
C    NOTE.. ODRV AND ITS SUBORDINATE ROUTINES HAVE BEEN MODIFIED TO
C    COMPUTE ORDERINGS FOR GENERAL MATRICES, NOT NECESSARILY HAVING ANY
C    SYMMETRY.  THE MIMINUM DEGREE ORDERING IS COMPUTED FOR THE
C    STRUCTURE OF THE SYMMETRIC MATRIX  M + M-TRANSPOSE.
C    MODIFICATIONS TO THE ORIGINAL ODRV MODULE HAVE BEEN MADE IN
C    THE CODING IN SUBROUTINE MDI, AND IN THE INITIAL COMMENTS IN
C    SUBROUTINES ODRV AND MD.
C
C    IF ONLY THE NONZERO ENTRIES IN THE UPPER TRIANGLE OF M ARE BEING
C    STORED, THEN ODRV SYMMETRICALLY REORDERS (IA,JA,A), (OPTIONALLY)
C    WITH THE DIAGONAL ENTRIES PLACED FIRST IN EACH ROW.  THIS IS TO
C    ENSURE THAT IF M(I,J) WILL BE IN THE UPPER TRIANGLE OF M WITH
C    RESPECT TO THE NEW ORDERING, THEN M(I,J) IS STORED IN ROW I (AND
C    THUS M(J,I) IS NOT STORED),  WHEREAS IF M(I,J) WILL BE IN THE
C    STRICT LOWER TRIANGLE OF M, THEN M(J,I) IS STORED IN ROW J (AND
C    THUS M(I,J) IS NOT STORED).
C
C
C  STORAGE OF SPARSE MATRICES
C
C    THE NONZERO ENTRIES OF THE MATRIX M ARE STORED ROW-BY-ROW IN THE
C    ARRAY A.  TO IDENTIFY THE INDIVIDUAL NONZERO ENTRIES IN EACH ROW,
C    WE NEED TO KNOW IN WHICH COLUMN EACH ENTRY LIES.  THESE COLUMN
C    INDICES ARE STORED IN THE ARRAY JA.  I.E., IF  A(K) = M(I,J),  THEN
C    JA(K) = J.  TO IDENTIFY THE INDIVIDUAL ROWS, WE NEED TO KNOW WHERE
C    EACH ROW STARTS.  THESE ROW POINTERS ARE STORED IN THE ARRAY IA.
C    I.E., IF M(I,J) IS THE FIRST NONZERO ENTRY (STORED) IN THE I-TH ROW
C    AND  A(K) = M(I,J),  THEN  IA(I) = K.  MOREOVER, IA(N+1) POINTS TO
C    THE FIRST LOCATION FOLLOWING THE LAST ELEMENT IN THE LAST ROW.
C    THUS, THE NUMBER OF ENTRIES IN THE I-TH ROW IS  IA(I+1) - IA(I),
C    THE NONZERO ENTRIES IN THE I-TH ROW ARE STORED CONSECUTIVELY IN
C
C            A(IA(I)),  A(IA(I)+1),  ..., A(IA(I+1)-1),
C
C    AND THE CORRESPONDING COLUMN INDICES ARE STORED CONSECUTIVELY IN
C
C            JA(IA(I)), JA(IA(I)+1), ..., JA(IA(I+1)-1).
C
C    SINCE THE COEFFICIENT MATRIX IS SYMMETRIC, ONLY THE NONZERO ENTRIES
C    IN THE UPPER TRIANGLE NEED BE STORED.  FOR EXAMPLE, THE MATRIX
C
C             ( 1  0  2  3  0 )
C             ( 0  4  0  0  0 )
C         M = ( 2  0  5  6  0 )
C             ( 3  0  6  7  8 )
C             ( 0  0  0  8  9 )
C
C    COULD BE STORED AS
C
C            - 1  2  3  4  5  6  7  8  9 10 11 12 13
C         ---+--------------------------------------
C         IA - 1  4  5  8 12 14
C         JA - 1  3  4  2  1  3  4  1  3  4  5  4  5
C          A - 1  2  3  4  2  5  6  3  6  7  8  8  9
C
C    OR (SYMMETRICALLY) AS
C
C            - 1  2  3  4  5  6  7  8  9
C         ---+--------------------------
C         IA - 1  4  5  7  9 10
C         JA - 1  3  4  2  3  4  4  5  5
C          A - 1  2  3  4  5  6  7  8  9          .
C
C
C  PARAMETERS
C
C    N    - ORDER OF THE MATRIX
C
C    IA   - INTEGER ONE-DIMENSIONAL ARRAY CONTAINING POINTERS TO DELIMIT
C           ROWS IN JA AND A.  DIMENSION = N+1
C
C    JA   - INTEGER ONE-DIMENSIONAL ARRAY CONTAINING THE COLUMN INDICES
C           CORRESPONDING TO THE ELEMENTS OF A.  DIMENSION = NUMBER OF
C           NONZERO ENTRIES IN (THE UPPER TRIANGLE OF) M
C
C    A    - REAL ONE-DIMENSIONAL ARRAY CONTAINING THE NONZERO ENTRIES IN
C           (THE UPPER TRIANGLE OF) M, STORED BY ROWS.  DIMENSION =
C           NUMBER OF NONZERO ENTRIES IN (THE UPPER TRIANGLE OF) M
C
C    P    - INTEGER ONE-DIMENSIONAL ARRAY USED TO RETURN THE PERMUTATION
C           OF THE ROWS AND COLUMNS OF M CORRESPONDING TO THE MINIMUM
C           DEGREE ORDERING.  DIMENSION = N
C
C    IP   - INTEGER ONE-DIMENSIONAL ARRAY USED TO RETURN THE INVERSE OF
C           THE PERMUTATION RETURNED IN P.  DIMENSION = N
C
C    NSP  - DECLARED DIMENSION OF THE ONE-DIMENSIONAL ARRAY ISP.  NSP
C           MUST BE AT LEAST  3N+4K,  WHERE K IS THE NUMBER OF NONZEROES
C           IN THE STRICT UPPER TRIANGLE OF M
C
C    ISP  - INTEGER ONE-DIMENSIONAL ARRAY USED FOR WORKING STORAGE.
C           DIMENSION = NSP
C
C    PATH - INTEGER PATH SPECIFICATION.  VALUES AND THEIR MEANINGS ARE -
C             1  FIND MINIMUM DEGREE ORDERING ONLY
C             2  FIND MINIMUM DEGREE ORDERING AND REORDER SYMMETRICALLY
C                  STORED MATRIX (USED WHEN ONLY THE NONZERO ENTRIES IN
C                  THE UPPER TRIANGLE OF M ARE BEING STORED)
C             3  REORDER SYMMETRICALLY STORED MATRIX AS SPECIFIED BY
C                  INPUT PERMUTATION (USED WHEN AN ORDERING HAS ALREADY
C                  BEEN DETERMINED AND ONLY THE NONZERO ENTRIES IN THE
C                  UPPER TRIANGLE OF M ARE BEING STORED)
C             4  SAME AS 2 BUT PUT DIAGONAL ENTRIES AT START OF EACH ROW
C             5  SAME AS 3 BUT PUT DIAGONAL ENTRIES AT START OF EACH ROW
C
C    FLAG - INTEGER ERROR FLAG.  VALUES AND THEIR MEANINGS ARE -
C               0    NO ERRORS DETECTED
C              9N+K  INSUFFICIENT STORAGE IN MD
C             10N+1  INSUFFICIENT STORAGE IN ODRV
C             11N+1  ILLEGAL PATH SPECIFICATION
C
C
C  CONVERSION FROM REAL TO DOUBLE PRECISION
C
C    CHANGE THE REAL DECLARATIONS IN ODRV AND SRO TO DOUBLE PRECISION
C    DECLARATIONS.
C
C-----------------------------------------------------------------------
C
      INTEGER  IA(1), JA(1),  P(1), IP(1),  ISP(1),  PATH,  FLAG,
     *   V, L, HEAD,  TMP, Q, N, NSP, MAX, NEXT
      DOUBLE PRECISION  A(1)
      LOGICAL  DFLAG
C
C----INITIALIZE ERROR FLAG AND VALIDATE PATH SPECIFICATION
      FLAG = 0
      IF (PATH.LT.1 .OR. 5.LT.PATH)  GO TO 111
C
C----ALLOCATE STORAGE AND FIND MINIMUM DEGREE ORDERING
      IF ((PATH-1) * (PATH-2) * (PATH-4) .NE. 0)  GO TO 1
        MAX = (NSP-N)/2
        V    = 1
        L    = V     +  MAX
        HEAD = L     +  MAX
        NEXT = HEAD  +  N
        IF (MAX.LT.N)  GO TO 110
C
        CALL  MD
     *     (N, IA,JA, MAX,ISP(V),ISP(L), ISP(HEAD),P,IP, ISP(V), FLAG)
        IF (FLAG.NE.0)  GO TO 100
C
C----ALLOCATE STORAGE AND SYMMETRICALLY REORDER MATRIX
   1  IF ((PATH-2) * (PATH-3) * (PATH-4) * (PATH-5) .NE. 0)  GO TO 2
        TMP = (NSP+1) -      N
        Q   = TMP     - (IA(N+1)-1)
        IF (Q.LT.1)  GO TO 110
C
        DFLAG = PATH.EQ.4 .OR. PATH.EQ.5
        CALL SRO
     *     (N,  IP,  IA, JA, A,  ISP(TMP),  ISP(Q),  DFLAG)
C
   2  RETURN
C
C ** ERROR -- ERROR DETECTED IN MD
C             FLAG = 9 * N + VI FROM ROUTINE MDI.
C
 100  RETURN
C ** ERROR -- INSUFFICIENT STORAGE
 110  FLAG = 10*N + 1
      RETURN
C ** ERROR -- ILLEGAL PATH SPECIFIED
 111  FLAG = 11*N + 1
      RETURN
      END
      SUBROUTINE MD
     *     (N, IA,JA, MAX, V,L, HEAD,LAST,NEXT, MARK, FLAG)
C
C***********************************************************************
C  MD -- MINIMUM DEGREE ALGORITHM (BASED ON ELEMENT MODEL)
C***********************************************************************
C
C  DESCRIPTION
C
C    MD FINDS A MINIMUM DEGREE ORDERING OF THE ROWS AND COLUMNS OF A
C    GENERAL SPARSE MATRIX M STORED IN (IA,JA,A) FORMAT.
C    WHEN THE STRUCTURE OF M IS NONSYMMETRIC, THE ORDERING IS THAT
C    OBTAINED FOR THE SYMMETRIC MATRIX  M + M-TRANSPOSE.
C
C
C  ADDITIONAL PARAMETERS
C
C    MAX  - DECLARED DIMENSION OF THE ONE-DIMENSIONAL ARRAYS V AND L.
C           MAX MUST BE AT LEAST  N+2K,  WHERE K IS THE NUMBER OF
C           NONZEROES IN THE STRICT UPPER TRIANGLE OF M
C
C    V    - INTEGER ONE-DIMENSIONAL WORK ARRAY.  DIMENSION = MAX
C
C    L    - INTEGER ONE-DIMENSIONAL WORK ARRAY.  DIMENSION = MAX
C
C    HEAD - INTEGER ONE-DIMENSIONAL WORK ARRAY.  DIMENSION = N
C
C    LAST - INTEGER ONE-DIMENSIONAL ARRAY USED TO RETURN THE PERMUTATION
C           OF THE ROWS AND COLUMNS OF M CORRESPONDING TO THE MINIMUM
C           DEGREE ORDERING.  DIMENSION = N
C
C    NEXT - INTEGER ONE-DIMENSIONAL ARRAY USED TO RETURN THE INVERSE OF
C           THE PERMUTATION RETURNED IN LAST.  DIMENSION = N
C
C    MARK - INTEGER ONE-DIMENSIONAL WORK ARRAY (MAY BE THE SAME AS V).
C           DIMENSION = N
C
C    FLAG - INTEGER ERROR FLAG.  VALUES AND THEIR MEANINGS ARE -
C             0      NO ERRORS DETECTED
C             11N+1  INSUFFICIENT STORAGE IN MD
C
C
C  DEFINITIONS OF INTERNAL PARAMETERS
C
C    ---------+---------------------------------------------------------
C    V(S)     - VALUE FIELD OF LIST ENTRY
C    ---------+---------------------------------------------------------
C    L(S)     - LINK FIELD OF LIST ENTRY  (0 =) END OF LIST)
C    ---------+---------------------------------------------------------
C    L(VI)    - POINTER TO ELEMENT LIST OF UNELIMINATED VERTEX VI
C    ---------+---------------------------------------------------------
C    L(EJ)    - POINTER TO BOUNDARY LIST OF ACTIVE ELEMENT EJ
C    ---------+---------------------------------------------------------
C    HEAD(D)  - VJ =) VJ HEAD OF D-LIST D
C             -  0 =) NO VERTEX IN D-LIST D
C
C
C             -                  VI UNELIMINATED VERTEX
C             -          VI IN EK           -       VI NOT IN EK
C    ---------+-----------------------------+---------------------------
C    NEXT(VI) - UNDEFINED BUT NONNEGATIVE   - VJ =) VJ NEXT IN D-LIST
C             -                             -  0 =) VI TAIL OF D-LIST
C    ---------+-----------------------------+---------------------------
C    LAST(VI) - (NOT SET UNTIL MDP)         - -D =) VI HEAD OF D-LIST D
C             --VK =) COMPUTE DEGREE        - VJ =) VJ LAST IN D-LIST
C             - EJ =) VI PROTOTYPE OF EJ    -  0 =) VI NOT IN ANY D-LIST
C             -  0 =) DO NOT COMPUTE DEGREE -
C    ---------+-----------------------------+---------------------------
C    MARK(VI) - MARK(VK)                    - NONNEG. TAG .LT. MARK(VK)
C
C
C             -                   VI ELIMINATED VERTEX
C             -      EI ACTIVE ELEMENT      -           OTHERWISE
C    ---------+-----------------------------+---------------------------
C    NEXT(VI) - -J =) VI WAS J-TH VERTEX    - -J =) VI WAS J-TH VERTEX
C             -       TO BE ELIMINATED      -       TO BE ELIMINATED
C    ---------+-----------------------------+---------------------------
C    LAST(VI) -  M =) SIZE OF EI = M        - UNDEFINED
C    ---------+-----------------------------+---------------------------
C    MARK(VI) - -M =) OVERLAP COUNT OF EI   - UNDEFINED
C             -       WITH EK = M           -
C             - OTHERWISE NONNEGATIVE TAG   -
C             -       .LT. MARK(VK)         -
C
C-----------------------------------------------------------------------
C
      INTEGER  IA(1), JA(1),  V(1), L(1),  HEAD(1), LAST(1), NEXT(1),
     *   MARK(1),  FLAG,  TAG, DMIN, VK,EK, TAIL, N, MAX, K
      EQUIVALENCE  (VK,EK)
C
C----INITIALIZATION
      TAG = 0
      CALL  MDI
     *   (N, IA,JA, MAX,V,L, HEAD,LAST,NEXT, MARK,TAG, FLAG)
      IF (FLAG.NE.0)  RETURN
C
      K = 0
      DMIN = 1
C
C----WHILE  K .LT. N  DO
   1  IF (K.GE.N)  GO TO 4
C
C------SEARCH FOR VERTEX OF MINIMUM DEGREE
   2    IF (HEAD(DMIN).GT.0)  GO TO 3
          DMIN = DMIN + 1
          GO TO 2
C
C------REMOVE VERTEX VK OF MINIMUM DEGREE FROM DEGREE LIST
   3    VK = HEAD(DMIN)
        HEAD(DMIN) = NEXT(VK)
        IF (HEAD(DMIN).GT.0)  LAST(HEAD(DMIN)) = -DMIN
C
C------NUMBER VERTEX VK, ADJUST TAG, AND TAG VK
        K = K+1
        NEXT(VK) = -K
        LAST(EK) = DMIN - 1
        TAG = TAG + LAST(EK)
        MARK(VK) = TAG
C
C------FORM ELEMENT EK FROM UNELIMINATED NEIGHBORS OF VK
        CALL  MDM
     *     (VK,TAIL, V,L, LAST,NEXT, MARK)
C
C------PURGE INACTIVE ELEMENTS AND DO MASS ELIMINATION
        CALL  MDP
     *     (K,EK,TAIL, V,L, HEAD,LAST,NEXT, MARK)
C
C------UPDATE DEGREES OF UNELIMINATED VERTICES IN EK
        CALL  MDU
     *     (EK,DMIN, V,L, HEAD,LAST,NEXT, MARK)
C
        GO TO 1
C
C----GENERATE INVERSE PERMUTATION FROM PERMUTATION
   4  DO 5 K=1,N
        NEXT(K) = -NEXT(K)
   5    LAST(NEXT(K)) = K
C
      RETURN
      END
      SUBROUTINE MDI
     *     (N, IA,JA, MAX,V,L, HEAD,LAST,NEXT, MARK,TAG, FLAG)
C
C***********************************************************************
C  MDI -- INITIALIZATION
C***********************************************************************
      INTEGER  IA(1), JA(1),  V(1), L(1),  HEAD(1), LAST(1), NEXT(1),
     *   MARK(1), TAG,  FLAG,  SFS, VI,DVI, VJ, N, MAX, JMIN, JMAX, J,
     *   LVK, KMAX, K, NEXTVI
C
C----INITIALIZE DEGREES, ELEMENT LISTS, AND DEGREE LISTS
      DO 1 VI=1,N
        MARK(VI) = 1
        L(VI) = 0
   1    HEAD(VI) = 0
      SFS = N+1
C
C----CREATE NONZERO STRUCTURE
C----FOR EACH NONZERO ENTRY A(VI,VJ)
      DO 6 VI=1,N
        JMIN = IA(VI)
        JMAX = IA(VI+1) - 1
        IF (JMIN.GT.JMAX)  GO TO 6
        DO 5 J=JMIN,JMAX
          VJ = JA(J)
          IF (VJ-VI) 2, 5, 4
C
C------IF A(VI,VJ) IS IN STRICT LOWER TRIANGLE
C------CHECK FOR PREVIOUS OCCURRENCE OF A(VJ,VI)
   2      LVK = VI
          KMAX = MARK(VI) - 1
          IF (KMAX .EQ. 0) GO TO 4
          DO 3 K=1,KMAX
            LVK = L(LVK)
            IF (V(LVK).EQ.VJ) GO TO 5
   3        CONTINUE
C----FOR UNENTERED ENTRIES A(VI,VJ)
   4        IF (SFS.GE.MAX)  GO TO 101
C
C------ENTER VJ IN ELEMENT LIST FOR VI
            MARK(VI) = MARK(VI) + 1
            V(SFS) = VJ
            L(SFS) = L(VI)
            L(VI) = SFS
            SFS = SFS+1
C
C------ENTER VI IN ELEMENT LIST FOR VJ
            MARK(VJ) = MARK(VJ) + 1
            V(SFS) = VI
            L(SFS) = L(VJ)
            L(VJ) = SFS
            SFS = SFS+1
   5      CONTINUE
   6    CONTINUE
C
C----CREATE DEGREE LISTS AND INITIALIZE MARK VECTOR
      DO 7 VI=1,N
        DVI = MARK(VI)
        NEXT(VI) = HEAD(DVI)
        HEAD(DVI) = VI
        LAST(VI) = -DVI
        NEXTVI = NEXT(VI)
        IF (NEXTVI.GT.0)  LAST(NEXTVI) = VI
   7    MARK(VI) = TAG
C
      RETURN
C
C ** ERROR-  INSUFFICIENT STORAGE
 101  FLAG = 9*N + VI
      RETURN
      END
      SUBROUTINE MDM
     *     (VK,TAIL, V,L, LAST,NEXT, MARK)
C
C***********************************************************************
C  MDM -- FORM ELEMENT FROM UNELIMINATED NEIGHBORS OF VK
C***********************************************************************
      INTEGER  VK, TAIL,  V(1), L(1),   LAST(1), NEXT(1),   MARK(1),
     *   TAG, S,LS,VS,ES, B,LB,VB, BLP,BLPMAX
      EQUIVALENCE  (VS, ES)
C
C----INITIALIZE TAG AND LIST OF UNELIMINATED NEIGHBORS
      TAG = MARK(VK)
      TAIL = VK
C
C----FOR EACH VERTEX/ELEMENT VS/ES IN ELEMENT LIST OF VK
      LS = L(VK)
   1  S = LS
      IF (S.EQ.0)  GO TO 5
        LS = L(S)
        VS = V(S)
        IF (NEXT(VS).LT.0)  GO TO 2
C
C------IF VS IS UNELIMINATED VERTEX, THEN TAG AND APPEND TO LIST OF
C------UNELIMINATED NEIGHBORS
          MARK(VS) = TAG
          L(TAIL) = S
          TAIL = S
          GO TO 4
C
C------IF ES IS ACTIVE ELEMENT, THEN ...
C--------FOR EACH VERTEX VB IN BOUNDARY LIST OF ELEMENT ES
   2      LB = L(ES)
          BLPMAX = LAST(ES)
          DO 3 BLP=1,BLPMAX
            B = LB
            LB = L(B)
            VB = V(B)
C
C----------IF VB IS UNTAGGED VERTEX, THEN TAG AND APPEND TO LIST OF
C----------UNELIMINATED NEIGHBORS
            IF (MARK(VB).GE.TAG)  GO TO 3
              MARK(VB) = TAG
              L(TAIL) = B
              TAIL = B
   3        CONTINUE
C
C--------MARK ES INACTIVE
          MARK(ES) = TAG
C
   4    GO TO 1
C
C----TERMINATE LIST OF UNELIMINATED NEIGHBORS
   5  L(TAIL) = 0
C
      RETURN
      END
      SUBROUTINE MDP
     *     (K,EK,TAIL, V,L, HEAD,LAST,NEXT, MARK)
C
C***********************************************************************
C  MDP -- PURGE INACTIVE ELEMENTS AND DO MASS ELIMINATION
C***********************************************************************
      INTEGER  EK, TAIL,  V(1), L(1),  HEAD(1), LAST(1), NEXT(1),
     *   MARK(1),  TAG, FREE, LI,VI,LVI,EVI, S,LS,ES, ILP, ILPMAX, K, I
C
C----INITIALIZE TAG
      TAG = MARK(EK)
C
C----FOR EACH VERTEX VI IN EK
      LI = EK
      ILPMAX = LAST(EK)
      IF (ILPMAX.LE.0)  GO TO 12
      DO 11 ILP=1,ILPMAX
        I = LI
        LI = L(I)
        VI = V(LI)
C
C------REMOVE VI FROM DEGREE LIST
        IF (LAST(VI).EQ.0)  GO TO 3
          IF (LAST(VI).GT.0)  GO TO 1
            HEAD(-LAST(VI)) = NEXT(VI)
            GO TO 2
   1        NEXT(LAST(VI)) = NEXT(VI)
   2      IF (NEXT(VI).GT.0)  LAST(NEXT(VI)) = LAST(VI)
C
C------REMOVE INACTIVE ITEMS FROM ELEMENT LIST OF VI
   3    LS = VI
   4    S = LS
        LS = L(S)
        IF (LS.EQ.0)  GO TO 6
          ES = V(LS)
          IF (MARK(ES).LT.TAG)  GO TO 5
            FREE = LS
            L(S) = L(LS)
            LS = S
   5      GO TO 4
C
C------IF VI IS INTERIOR VERTEX, THEN REMOVE FROM LIST AND ELIMINATE
   6    LVI = L(VI)
        IF (LVI.NE.0)  GO TO 7
          L(I) = L(LI)
          LI = I
C
          K = K+1
          NEXT(VI) = -K
          LAST(EK) = LAST(EK) - 1
          GO TO 11
C
C------ELSE ...
C--------CLASSIFY VERTEX VI
   7      IF (L(LVI).NE.0)  GO TO 9
            EVI = V(LVI)
            IF (NEXT(EVI).GE.0)  GO TO 9
              IF (MARK(EVI).LT.0)  GO TO 8
C
C----------IF VI IS PROTOTYPE VERTEX, THEN MARK AS SUCH, INITIALIZE
C----------OVERLAP COUNT FOR CORRESPONDING ELEMENT, AND MOVE VI TO END
C----------OF BOUNDARY LIST
                LAST(VI) = EVI
                MARK(EVI) = -1
                L(TAIL) = LI
                TAIL = LI
                L(I) = L(LI)
                LI = I
                GO TO 10
C
C----------ELSE IF VI IS DUPLICATE VERTEX, THEN MARK AS SUCH AND ADJUST
C----------OVERLAP COUNT FOR CORRESPONDING ELEMENT
   8            LAST(VI) = 0
                MARK(EVI) = MARK(EVI) - 1
                GO TO 10
C
C----------ELSE MARK VI TO COMPUTE DEGREE
   9            LAST(VI) = -EK
C
C--------INSERT EK IN ELEMENT LIST OF VI
  10      V(FREE) = EK
          L(FREE) = L(VI)
          L(VI) = FREE
  11    CONTINUE
C
C----TERMINATE BOUNDARY LIST
  12  L(TAIL) = 0
C
      RETURN
      END
      SUBROUTINE MDU
     *     (EK,DMIN, V,L, HEAD,LAST,NEXT, MARK)
C
C***********************************************************************
C  MDU -- UPDATE DEGREES OF UNELIMINATED VERTICES IN EK
C***********************************************************************
      INTEGER  EK, DMIN,  V(1), L(1),  HEAD(1), LAST(1), NEXT(1),
     *   MARK(1),  TAG, VI,EVI,DVI, S,VS,ES, B,VB, ILP,ILPMAX,
     *   BLP,BLPMAX, I
      EQUIVALENCE  (VS, ES)
C
C----INITIALIZE TAG
      TAG = MARK(EK) - LAST(EK)
C
C----FOR EACH VERTEX VI IN EK
      I = EK
      ILPMAX = LAST(EK)
      IF (ILPMAX.LE.0)  GO TO 11
      DO 10 ILP=1,ILPMAX
        I = L(I)
        VI = V(I)
        IF (LAST(VI))  1, 10, 8
C
C------IF VI NEITHER PROTOTYPE NOR DUPLICATE VERTEX, THEN MERGE ELEMENTS
C------TO COMPUTE DEGREE
   1      TAG = TAG + 1
          DVI = LAST(EK)
C
C--------FOR EACH VERTEX/ELEMENT VS/ES IN ELEMENT LIST OF VI
          S = L(VI)
   2      S = L(S)
          IF (S.EQ.0)  GO TO 9
            VS = V(S)
            IF (NEXT(VS).LT.0)  GO TO 3
C
C----------IF VS IS UNELIMINATED VERTEX, THEN TAG AND ADJUST DEGREE
              MARK(VS) = TAG
              DVI = DVI + 1
              GO TO 5
C
C----------IF ES IS ACTIVE ELEMENT, THEN EXPAND
C------------CHECK FOR OUTMATCHED VERTEX
   3          IF (MARK(ES).LT.0)  GO TO 6
C
C------------FOR EACH VERTEX VB IN ES
              B = ES
              BLPMAX = LAST(ES)
              DO 4 BLP=1,BLPMAX
                B = L(B)
                VB = V(B)
C
C--------------IF VB IS UNTAGGED, THEN TAG AND ADJUST DEGREE
                IF (MARK(VB).GE.TAG)  GO TO 4
                  MARK(VB) = TAG
                  DVI = DVI + 1
   4            CONTINUE
C
   5        GO TO 2
C
C------ELSE IF VI IS OUTMATCHED VERTEX, THEN ADJUST OVERLAPS BUT DO NOT
C------COMPUTE DEGREE
   6      LAST(VI) = 0
          MARK(ES) = MARK(ES) - 1
   7      S = L(S)
          IF (S.EQ.0)  GO TO 10
            ES = V(S)
            IF (MARK(ES).LT.0)  MARK(ES) = MARK(ES) - 1
            GO TO 7
C
C------ELSE IF VI IS PROTOTYPE VERTEX, THEN CALCULATE DEGREE BY
C------INCLUSION/EXCLUSION AND RESET OVERLAP COUNT
   8      EVI = LAST(VI)
          DVI = LAST(EK) + LAST(EVI) + MARK(EVI)
          MARK(EVI) = 0
C
C------INSERT VI IN APPROPRIATE DEGREE LIST
   9    NEXT(VI) = HEAD(DVI)
        HEAD(DVI) = VI
        LAST(VI) = -DVI
        IF (NEXT(VI).GT.0)  LAST(NEXT(VI)) = VI
        IF (DVI.LT.DMIN)  DMIN = DVI
C
  10    CONTINUE
C
  11  RETURN
      END
      SUBROUTINE SRO
     *     (N, IP, IA,JA,A, Q, R, DFLAG)
C
C***********************************************************************
C  SRO -- SYMMETRIC REORDERING OF SPARSE SYMMETRIC MATRIX
C***********************************************************************
C
C  DESCRIPTION
C
C    THE NONZERO ENTRIES OF THE MATRIX M ARE ASSUMED TO BE STORED
C    SYMMETRICALLY IN (IA,JA,A) FORMAT (I.E., NOT BOTH M(I,J) AND M(J,I)
C    ARE STORED IF I NE J).
C
C    SRO DOES NOT REARRANGE THE ORDER OF THE ROWS, BUT DOES MOVE
C    NONZEROES FROM ONE ROW TO ANOTHER TO ENSURE THAT IF M(I,J) WILL BE
C    IN THE UPPER TRIANGLE OF M WITH RESPECT TO THE NEW ORDERING, THEN
C    M(I,J) IS STORED IN ROW I (AND THUS M(J,I) IS NOT STORED),  WHEREAS
C    IF M(I,J) WILL BE IN THE STRICT LOWER TRIANGLE OF M, THEN M(J,I) IS
C    STORED IN ROW J (AND THUS M(I,J) IS NOT STORED).
C
C
C  ADDITIONAL PARAMETERS
C
C    Q     - INTEGER ONE-DIMENSIONAL WORK ARRAY.  DIMENSION = N
C
C    R     - INTEGER ONE-DIMENSIONAL WORK ARRAY.  DIMENSION = NUMBER OF
C            NONZERO ENTRIES IN THE UPPER TRIANGLE OF M
C
C    DFLAG - LOGICAL VARIABLE.  IF DFLAG = .TRUE., THEN STORE NONZERO
C            DIAGONAL ELEMENTS AT THE BEGINNING OF THE ROW
C
C-----------------------------------------------------------------------
C
      INTEGER  IP(1),  IA(1), JA(1),  Q(1), R(1), N, I, JMIN, JMAX, J, K
     *        ,ILAST, JDUMMY, JAK
      DOUBLE PRECISION  A(1),  AK
      LOGICAL  DFLAG
C
C
C--PHASE 1 -- FIND ROW IN WHICH TO STORE EACH NONZERO
C----INITIALIZE COUNT OF NONZEROES TO BE STORED IN EACH ROW
      DO 1 I=1,N
  1     Q(I) = 0
C
C----FOR EACH NONZERO ELEMENT A(J)
      DO 3 I=1,N
        JMIN = IA(I)
        JMAX = IA(I+1) - 1
        IF (JMIN.GT.JMAX)  GO TO 3
        DO 2 J=JMIN,JMAX
C
C--------FIND ROW (=R(J)) AND COLUMN (=JA(J)) IN WHICH TO STORE A(J) ...
          K = JA(J)
          IF (IP(K).LT.IP(I))  JA(J) = I
          IF (IP(K).GE.IP(I))  K = I
          R(J) = K
C
C--------... AND INCREMENT COUNT OF NONZEROES (=Q(R(J)) IN THAT ROW
  2       Q(K) = Q(K) + 1
  3     CONTINUE
C
C
C--PHASE 2 -- FIND NEW IA AND PERMUTATION TO APPLY TO (JA,A)
C----DETERMINE POINTERS TO DELIMIT ROWS IN PERMUTED (JA,A)
      DO 4 I=1,N
        IA(I+1) = IA(I) + Q(I)
  4     Q(I) = IA(I+1)
C
C----DETERMINE WHERE EACH (JA(J),A(J)) IS STORED IN PERMUTED (JA,A)
C----FOR EACH NONZERO ELEMENT (IN REVERSE ORDER)
      ILAST = 0
      JMIN = IA(1)
      JMAX = IA(N+1) - 1
      J = JMAX
      DO 6 JDUMMY=JMIN,JMAX
        I = R(J)
        IF (.NOT.DFLAG .OR. JA(J).NE.I .OR. I.EQ.ILAST)  GO TO 5
C
C------IF DFLAG, THEN PUT DIAGONAL NONZERO AT BEGINNING OF ROW
          R(J) = IA(I)
          ILAST = I
          GO TO 6
C
C------PUT (OFF-DIAGONAL) NONZERO IN LAST UNUSED LOCATION IN ROW
  5       Q(I) = Q(I) - 1
          R(J) = Q(I)
C
  6     J = J-1
C
C
C--PHASE 3 -- PERMUTE (JA,A) TO UPPER TRIANGULAR FORM (WRT NEW ORDERING)
      DO 8 J=JMIN,JMAX
  7     IF (R(J).EQ.J)  GO TO 8
          K = R(J)
          R(J) = R(K)
          R(K) = K
          JAK = JA(K)
          JA(K) = JA(J)
          JA(J) = JAK
          AK = A(K)
          A(K) = A(J)
          A(J) = AK
          GO TO 7
  8     CONTINUE
C
      RETURN
      END
      SUBROUTINE CDRV
     *     (N, R,C,IC, IA,JA,A, B, Z, NSP,ISP,RSP,ESP, PATH, FLAG)
C
C*** SUBROUTINE CDRV
C*** DRIVER FOR SUBROUTINES FOR SOLVING SPARSE NONSYMMETRIC SYSTEMS OF
C       LINEAR EQUATIONS (COMPRESSED POINTER STORAGE)
C
C
C    PARAMETERS
C    CLASS ABBREVIATIONS ARE--
C       N - INTEGER VARIABLE
C       F - REAL VARIABLE
C       V - SUPPLIES A VALUE TO THE DRIVER
C       R - RETURNS A RESULT FROM THE DRIVER
C       I - USED INTERNALLY BY THE DRIVER
C       A - ARRAY
C
C CLASS - PARAMETER
C ------+----------
C       -
C         THE NONZERO ENTRIES OF THE COEFFICIENT MATRIX M ARE STORED
C    ROW-BY-ROW IN THE ARRAY A.  TO IDENTIFY THE INDIVIDUAL NONZERO
C    ENTRIES IN EACH ROW, WE NEED TO KNOW IN WHICH COLUMN EACH ENTRY
C    LIES.  THE COLUMN INDICES WHICH CORRESPOND TO THE NONZERO ENTRIES
C    OF M ARE STORED IN THE ARRAY JA.  I.E., IF  A(K) = M(I,J),  THEN
C    JA(K) = J.  IN ADDITION, WE NEED TO KNOW WHERE EACH ROW STARTS AND
C    HOW LONG IT IS.  THE INDEX POSITIONS IN JA AND A WHERE THE ROWS OF
C    M BEGIN ARE STORED IN THE ARRAY IA.  I.E., IF M(I,J) IS THE FIRST
C    NONZERO ENTRY (STORED) IN THE I-TH ROW AND A(K) = M(I,J),  THEN
C    IA(I) = K.  MOREOVER, THE INDEX IN JA AND A OF THE FIRST LOCATION
C    FOLLOWING THE LAST ELEMENT IN THE LAST ROW IS STORED IN IA(N+1).
C    THUS, THE NUMBER OF ENTRIES IN THE I-TH ROW IS GIVEN BY
C    IA(I+1) - IA(I),  THE NONZERO ENTRIES OF THE I-TH ROW ARE STORED
C    CONSECUTIVELY IN
C            A(IA(I)),  A(IA(I)+1),  ..., A(IA(I+1)-1),
C    AND THE CORRESPONDING COLUMN INDICES ARE STORED CONSECUTIVELY IN
C            JA(IA(I)), JA(IA(I)+1), ..., JA(IA(I+1)-1).
C    FOR EXAMPLE, THE 5 BY 5 MATRIX
C                ( 1. 0. 2. 0. 0.)
C                ( 0. 3. 0. 0. 0.)
C            M = ( 0. 4. 5. 6. 0.)
C                ( 0. 0. 0. 7. 0.)
C                ( 0. 0. 0. 8. 9.)
C    WOULD BE STORED AS
C               - 1  2  3  4  5  6  7  8  9
C            ---+--------------------------
C            IA - 1  3  4  7  8 10
C            JA - 1  3  2  2  3  4  4  4  5
C             A - 1. 2. 3. 4. 5. 6. 7. 8. 9.         .
C
C NV    - N     - NUMBER OF VARIABLES/EQUATIONS.
C FVA   - A     - NONZERO ENTRIES OF THE COEFFICIENT MATRIX M, STORED
C       -           BY ROWS.
C       -           SIZE = NUMBER OF NONZERO ENTRIES IN M.
C NVA   - IA    - POINTERS TO DELIMIT THE ROWS IN A.
C       -           SIZE = N+1.
C NVA   - JA    - COLUMN NUMBERS CORRESPONDING TO THE ELEMENTS OF A.
C       -           SIZE = SIZE OF A.
C FVA   - B     - RIGHT-HAND SIDE B.  B AND Z CAN THE SAME ARRAY.
C       -           SIZE = N.
C FRA   - Z     - SOLUTION X.  B AND Z CAN BE THE SAME ARRAY.
C       -           SIZE = N.
C
C         THE ROWS AND COLUMNS OF THE ORIGINAL MATRIX M CAN BE
C    REORDERED (E.G., TO REDUCE FILLIN OR ENSURE NUMERICAL STABILITY)
C    BEFORE CALLING THE DRIVER.  IF NO REORDERING IS DONE, THEN SET
C    R(I) = C(I) = IC(I) = I  FOR I=1,...,N.  THE SOLUTION Z IS RETURNED
C    IN THE ORIGINAL ORDER.
C         IF THE COLUMNS HAVE BEEN REORDERED (I.E.,  C(I).NE.I  FOR SOME
C    I), THEN THE DRIVER WILL CALL A SUBROUTINE (NROC) WHICH REARRANGES
C    EACH ROW OF JA AND A, LEAVING THE ROWS IN THE ORIGINAL ORDER, BUT
C    PLACING THE ELEMENTS OF EACH ROW IN INCREASING ORDER WITH RESPECT
C    TO THE NEW ORDERING.  IF  PATH.NE.1,  THEN NROC IS ASSUMED TO HAVE
C    BEEN CALLED ALREADY.
C
C NVA   - R     - ORDERING OF THE ROWS OF M.
C       -           SIZE = N.
C NVA   - C     - ORDERING OF THE COLUMNS OF M.
C       -           SIZE = N.
C NVA   - IC    - INVERSE OF THE ORDERING OF THE COLUMNS OF M.  I.E.,
C       -           IC(C(I)) = I  FOR I=1,...,N.
C       -           SIZE = N.
C
C         THE SOLUTION OF THE SYSTEM OF LINEAR EQUATIONS IS DIVIDED INTO
C    THREE STAGES --
C      NSFC -- THE MATRIX M IS PROCESSED SYMBOLICALLY TO DETERMINE WHERE
C               FILLIN WILL OCCUR DURING THE NUMERIC FACTORIZATION.
C      NNFC -- THE MATRIX M IS FACTORED NUMERICALLY INTO THE PRODUCT LDU
C               OF A UNIT LOWER TRIANGULAR MATRIX L, A DIAGONAL MATRIX
C               D, AND A UNIT UPPER TRIANGULAR MATRIX U, AND THE SYSTEM
C               MX = B  IS SOLVED.
C      NNSC -- THE LINEAR SYSTEM  MX = B  IS SOLVED USING THE LDU
C  OR           FACTORIZATION FROM NNFC.
C      NNTC -- THE TRANSPOSED LINEAR SYSTEM  MT X = B  IS SOLVED USING
C               THE LDU FACTORIZATION FROM NNF.
C    FOR SEVERAL SYSTEMS WHOSE COEFFICIENT MATRICES HAVE THE SAME
C    NONZERO STRUCTURE, NSFC NEED BE DONE ONLY ONCE (FOR THE FIRST
C    SYSTEM).  THEN NNFC IS DONE ONCE FOR EACH ADDITIONAL SYSTEM.  FOR
C    SEVERAL SYSTEMS WITH THE SAME COEFFICIENT MATRIX, NSFC AND NNFC
C    NEED BE DONE ONLY ONCE (FOR THE FIRST SYSTEM).  THEN NNSC OR NNTC
C    IS DONE ONCE FOR EACH ADDITIONAL RIGHT-HAND SIDE.
C
C NV    - PATH  - PATH SPECIFICATION.  VALUES AND THEIR MEANINGS ARE --
C       -           1  PERFORM NROC, NSFC, AND NNFC.
C       -           2  PERFORM NNFC ONLY  (NSFC IS ASSUMED TO HAVE BEEN
C       -               DONE IN A MANNER COMPATIBLE WITH THE STORAGE
C       -               ALLOCATION USED IN THE DRIVER).
C       -           3  PERFORM NNSC ONLY  (NSFC AND NNFC ARE ASSUMED TO
C       -               HAVE BEEN DONE IN A MANNER COMPATIBLE WITH THE
C       -               STORAGE ALLOCATION USED IN THE DRIVER).
C       -           4  PERFORM NNTC ONLY  (NSFC AND NNFC ARE ASSUMED TO
C       -               HAVE BEEN DONE IN A MANNER COMPATIBLE WITH THE
C       -               STORAGE ALLOCATION USED IN THE DRIVER).
C       -           5  PERFORM NROC AND NSFC.
C
C         VARIOUS ERRORS ARE DETECTED BY THE DRIVER AND THE INDIVIDUAL
C    SUBROUTINES.
C
C NR    - FLAG  - ERROR FLAG.  VALUES AND THEIR MEANINGS ARE --
C       -             0     NO ERRORS DETECTED
C       -             N+K   NULL ROW IN A  --  ROW = K
C       -            2N+K   DUPLICATE ENTRY IN A  --  ROW = K
C       -            3N+K   INSUFFICIENT STORAGE IN NSFC  --  ROW = K
C       -            4N+1   INSUFFICIENT STORAGE IN NNFC
C       -            5N+K   NULL PIVOT  --  ROW = K
C       -            6N+K   INSUFFICIENT STORAGE IN NSFC  --  ROW = K
C       -            7N+1   INSUFFICIENT STORAGE IN NNFC
C       -            8N+K   ZERO PIVOT  --  ROW = K
C       -           10N+1   INSUFFICIENT STORAGE IN CDRV
C       -           11N+1   ILLEGAL PATH SPECIFICATION
C
C         WORKING STORAGE IS NEEDED FOR THE FACTORED FORM OF THE MATRIX
C    M PLUS VARIOUS TEMPORARY VECTORS.  THE ARRAYS ISP AND RSP SHOULD BE
C    EQUIVALENCED.  INTEGER STORAGE IS ALLOCATED FROM THE BEGINNING OF
C    ISP AND REAL STORAGE FROM THE END OF RSP.
C
C NV    - NSP   - DECLARED DIMENSION OF RSP.  NSP GENERALLY MUST
C       -           BE LARGER THAN  8N+2 + 2K  (WHERE  K = (NUMBER OF
C       -           NONZERO ENTRIES IN M)).
C NVIRA - ISP   - INTEGER WORKING STORAGE DIVIDED UP INTO VARIOUS ARRAYS
C       -           NEEDED BY THE SUBROUTINES.  ISP AND RSP SHOULD BE
C       -           EQUIVALENCED.
C       -           SIZE = LRATIO*NSP.
C FVIRA - RSP   - REAL WORKING STORAGE DIVIDED UP INTO VARIOUS ARRAYS
C       -           NEEDED BY THE SUBROUTINES.  ISP AND RSP SHOULD BE
C       -           EQUIVALENCED.
C       -           SIZE = NSP.
C NR    - ESP   - IF SUFFICIENT STORAGE WAS AVAILABLE TO PERFORM THE
C       -           SYMBOLIC FACTORIZATION (NSFC), THEN ESP IS SET TO
C       -           THE AMOUNT OF EXCESS STORAGE PROVIDED (NEGATIVE IF
C       -           INSUFFICIENT STORAGE WAS AVAILABLE TO PERFORM THE
C       -           NUMERIC FACTORIZATION (NNFC)).
C
C
C  CONVERSION TO DOUBLE PRECISION
C
C    TO CONVERT THESE ROUTINES FOR DOUBLE PRECISION ARRAYS..
C    (1) USE THE DOUBLE PRECISION DECLARATIONS IN PLACE OF THE REAL
C    DECLARATIONS IN EACH SUBPROGRAM, AS GIVEN IN COMMENT CARDS.
C    (2) CHANGE THE DATA-LOADED VALUE OF THE INTEGER  LRATIO
C    IN SUBROUTINE CDRV, AS INDICATED BELOW.
C    (3) CHANGE E0 TO D0 IN THE CONSTANTS IN STATEMENT NUMBER 10
C    IN SUBROUTINE NNFC AND THE LINE FOLLOWING THAT.
C
      INTEGER  R(1), C(1), IC(1),  IA(1), JA(1),  ISP(1), ESP,  PATH,
     *   FLAG,  D, U, Q, ROW, TMP, AR, UMAX, N, NSP, IL, IJL, IU, IJU,
     *   IRL, JRL, JL, MAX, JLMAX, IRA, JRA, IRAC, IRU, JRU,
     *   JUTMP, JUMAX, I, JU, L, LMAX, J, LRATIO
      DOUBLE PRECISION  A(1), B(1), Z(1), RSP(1)
C
C  SET LRATIO EQUAL TO THE RATIO BETWEEN THE LENGTH OF FLOATING POINT
C  AND INTEGER ARRAY DATA.  E. G., LRATIO = 1 FOR (REAL, INTEGER),
C  LRATIO = 2 FOR (DOUBLE PRECISION, INTEGER)
C
      COMMON /SPRECN/ LRATIO
      SAVE /SPRECN/
C
      IF (PATH.LT.1 .OR. 5.LT.PATH)  GO TO 111
C******INITIALIZE AND DIVIDE UP TEMPORARY STORAGE  *******************
      IL   = 1
      IJL  = IL  + (N+1)
      IU   = IJL +   N
      IJU  = IU  + (N+1)
      IRL  = IJU +   N
      JRL  = IRL +   N
      JL   = JRL +   N
C
C  ******  REORDER A IF NECESSARY, CALL NSFC IF FLAG IS SET  ***********
      IF ((PATH-1) * (PATH-5) .NE. 0)  GO TO 5
        MAX = (LRATIO*NSP + 1 - JL) - (N+1) - 5*N
        JLMAX = MAX/2
        Q     = JL   + JLMAX
        IRA   = Q    + (N+1)
        JRA   = IRA  +   N
        IRAC  = JRA  +   N
        IRU   = IRAC +   N
        JRU   = IRU  +   N
        JUTMP = JRU  +   N
        JUMAX = LRATIO*NSP  + 1 - JUTMP
        ESP = MAX/LRATIO
        IF (JLMAX.LE.0 .OR. JUMAX.LE.0)  GO TO 110
C
        DO 1 I=1,N
          IF (C(I).NE.I)  GO TO 2
   1      CONTINUE
        GO TO 3
   2    AR = NSP + 1 - N
        CALL  NROC
     *     (N, IC, IA,JA,A, ISP(IL), RSP(AR), ISP(IU), FLAG)
        IF (FLAG.NE.0)  GO TO 100
C
   3    CALL  NSFC
     *     (N, R, IC, IA,JA,
     *      JLMAX, ISP(IL), ISP(JL), ISP(IJL),
     *      JUMAX, ISP(IU), ISP(JUTMP), ISP(IJU),
     *      ISP(Q), ISP(IRA), ISP(JRA), ISP(IRAC),
     *      ISP(IRL), ISP(JRL), ISP(IRU), ISP(JRU),  FLAG)
        IF(FLAG .NE. 0)  GO TO 100
C  ******  MOVE JU NEXT TO JL  *****************************************
        JLMAX = ISP(IJL+N-1)
        JU    = JL + JLMAX
        JUMAX = ISP(IJU+N-1)
        IF (JUMAX.LE.0)  GO TO 5
        DO 4 J=1,JUMAX
   4      ISP(JU+J-1) = ISP(JUTMP+J-1)
C
C  ******  CALL REMAINING SUBROUTINES  *********************************
   5  JLMAX = ISP(IJL+N-1)
      JU    = JL  + JLMAX
      JUMAX = ISP(IJU+N-1)
      L     = (JU + JUMAX - 2 + LRATIO)  /  LRATIO    +    1
      LMAX  = ISP(IL+N) - 1
      D     = L   + LMAX
      U     = D   + N
      ROW   = NSP + 1 - N
      TMP   = ROW - N
      UMAX  = TMP - U
      ESP   = UMAX - (ISP(IU+N) - 1)
      IF(ESP .LT. 0)GOTO 110
C
      IF ((PATH-1) * (PATH-2) .NE. 0)  GO TO 6
        IF (UMAX.LT.0)  GO TO 110
        CALL NNFC
     *     (N,  R, C, IC,  IA, JA, A, Z, B,
     *      LMAX, ISP(IL), ISP(JL), ISP(IJL), RSP(L),  RSP(D),
     *      UMAX, ISP(IU), ISP(JU), ISP(IJU), RSP(U),
     *      RSP(ROW), RSP(TMP),  ISP(IRL), ISP(JRL),  FLAG)
        IF(FLAG .NE. 0)  GO TO 100
C
   6  IF ((PATH-3) .NE. 0)  GO TO 7
        CALL NNSC
     *     (N,  R, C,  ISP(IL), ISP(JL), ISP(IJL), RSP(L),
     *      RSP(D),    ISP(IU), ISP(JU), ISP(IJU), RSP(U),
     *      Z, B,  RSP(TMP))
C
   7  IF ((PATH-4) .NE. 0)  GO TO 8
        CALL NNTC
     *     (N,  R, C,  ISP(IL), ISP(JL), ISP(IJL), RSP(L),
     *      RSP(D),    ISP(IU), ISP(JU), ISP(IJU), RSP(U),
     *      Z, B,  RSP(TMP))
   8  RETURN
C
C ** ERROR.. ERROR DETECTED IN NROC, NSFC, NNFC, OR NNSC
 100  RETURN
C ** ERROR.. INSUFFICIENT STORAGE
 110  FLAG = 10*N + 1
      RETURN
C ** ERROR.. ILLEGAL PATH SPECIFICATION
 111  FLAG = 11*N + 1
      RETURN
      END
      SUBROUTINE NROC (N, IC, IA, JA, A, JAR, AR, P, FLAG)
C
C
C       ----------------------------------------------------------------
C
C               YALE SPARSE MATRIX PACKAGE - NONSYMMETRIC CODES
C                    SOLVING THE SYSTEM OF EQUATIONS MX = B
C
C    I.   CALLING SEQUENCES
C         THE COEFFICIENT MATRIX CAN BE PROCESSED BY AN ORDERING ROUTINE
C    (E.G., TO REDUCE FILLIN OR ENSURE NUMERICAL STABILITY) BEFORE USING
C    THE REMAINING SUBROUTINES.  IF NO REORDERING IS DONE, THEN SET
C    R(I) = C(I) = IC(I) = I  FOR I=1,...,N.  IF AN ORDERING SUBROUTINE
C    IS USED, THEN NROC SHOULD BE USED TO REORDER THE COEFFICIENT MATRIX
C    THE CALLING SEQUENCE IS --
C        (       (MATRIX ORDERING))
C        (NROC   (MATRIX REORDERING))
C         NSFC   (SYMBOLIC FACTORIZATION TO DETERMINE WHERE FILLIN WILL
C                  OCCUR DURING NUMERIC FACTORIZATION)
C         NNFC   (NUMERIC FACTORIZATION INTO PRODUCT LDU OF UNIT LOWER
C                  TRIANGULAR MATRIX L, DIAGONAL MATRIX D, AND UNIT
C                  UPPER TRIANGULAR MATRIX U, AND SOLUTION OF LINEAR
C                  SYSTEM)
C         NNSC   (SOLUTION OF LINEAR SYSTEM FOR ADDITIONAL RIGHT-HAND
C                  SIDE USING LDU FACTORIZATION FROM NNFC)
C    (IF ONLY ONE SYSTEM OF EQUATIONS IS TO BE SOLVED, THEN THE
C    SUBROUTINE TRK SHOULD BE USED.)
C
C    II.  STORAGE OF SPARSE MATRICES
C         THE NONZERO ENTRIES OF THE COEFFICIENT MATRIX M ARE STORED
C    ROW-BY-ROW IN THE ARRAY A.  TO IDENTIFY THE INDIVIDUAL NONZERO
C    ENTRIES IN EACH ROW, WE NEED TO KNOW IN WHICH COLUMN EACH ENTRY
C    LIES.  THE COLUMN INDICES WHICH CORRESPOND TO THE NONZERO ENTRIES
C    OF M ARE STORED IN THE ARRAY JA.  I.E., IF  A(K) = M(I,J),  THEN
C    JA(K) = J.  IN ADDITION, WE NEED TO KNOW WHERE EACH ROW STARTS AND
C    HOW LONG IT IS.  THE INDEX POSITIONS IN JA AND A WHERE THE ROWS OF
C    M BEGIN ARE STORED IN THE ARRAY IA.  I.E., IF M(I,J) IS THE FIRST
C    (LEFTMOST) ENTRY IN THE I-TH ROW AND  A(K) = M(I,J),  THEN
C    IA(I) = K.  MOREOVER, THE INDEX IN JA AND A OF THE FIRST LOCATION
C    FOLLOWING THE LAST ELEMENT IN THE LAST ROW IS STORED IN IA(N+1).
C    THUS, THE NUMBER OF ENTRIES IN THE I-TH ROW IS GIVEN BY
C    IA(I+1) - IA(I),  THE NONZERO ENTRIES OF THE I-TH ROW ARE STORED
C    CONSECUTIVELY IN
C            A(IA(I)),  A(IA(I)+1),  ..., A(IA(I+1)-1),
C    AND THE CORRESPONDING COLUMN INDICES ARE STORED CONSECUTIVELY IN
C            JA(IA(I)), JA(IA(I)+1), ..., JA(IA(I+1)-1).
C    FOR EXAMPLE, THE 5 BY 5 MATRIX
C                ( 1. 0. 2. 0. 0.)
C                ( 0. 3. 0. 0. 0.)
C            M = ( 0. 4. 5. 6. 0.)
C                ( 0. 0. 0. 7. 0.)
C                ( 0. 0. 0. 8. 9.)
C    WOULD BE STORED AS
C               - 1  2  3  4  5  6  7  8  9
C            ---+--------------------------
C            IA - 1  3  4  7  8 10
C            JA - 1  3  2  2  3  4  4  4  5
C             A - 1. 2. 3. 4. 5. 6. 7. 8. 9.         .
C
C         THE STRICT UPPER (LOWER) TRIANGULAR PORTION OF THE MATRIX
C    U (L) IS STORED IN A SIMILAR FASHION USING THE ARRAYS  IU, JU, U
C    (IL, JL, L)  EXCEPT THAT AN ADDITIONAL ARRAY IJU (IJL) IS USED TO
C    COMPRESS STORAGE OF JU (JL) BY ALLOWING SOME SEQUENCES OF COLUMN
C    (ROW) INDICES TO USED FOR MORE THAN ONE ROW (COLUMN)  (N.B., L IS
C    STORED BY COLUMNS).  IJU(K) (IJL(K)) POINTS TO THE STARTING
C    LOCATION IN JU (JL) OF ENTRIES FOR THE KTH ROW (COLUMN).
C    COMPRESSION IN JU (JL) OCCURS IN TWO WAYS.  FIRST, IF A ROW
C    (COLUMN) I WAS MERGED INTO THE CURRENT ROW (COLUMN) K, AND THE
C    NUMBER OF ELEMENTS MERGED IN FROM (THE TAIL PORTION OF) ROW
C    (COLUMN) I IS THE SAME AS THE FINAL LENGTH OF ROW (COLUMN) K, THEN
C    THE KTH ROW (COLUMN) AND THE TAIL OF ROW (COLUMN) I ARE IDENTICAL
C    AND IJU(K) (IJL(K)) POINTS TO THE START OF THE TAIL.  SECOND, IF
C    SOME TAIL PORTION OF THE (K-1)ST ROW (COLUMN) IS IDENTICAL TO THE
C    HEAD OF THE KTH ROW (COLUMN), THEN IJU(K) (IJL(K)) POINTS TO THE
C    START OF THAT TAIL PORTION.  FOR EXAMPLE, THE NONZERO STRUCTURE OF
C    THE STRICT UPPER TRIANGULAR PART OF THE MATRIX
C            D 0 X X X
C            0 D 0 X X
C            0 0 D X 0
C            0 0 0 D X
C            0 0 0 0 D
C    WOULD BE REPRESENTED AS
C                - 1 2 3 4 5 6
C            ----+------------
C             IU - 1 4 6 7 8 8
C             JU - 3 4 5 4
C            IJU - 1 2 4 3           .
C    THE DIAGONAL ENTRIES OF L AND U ARE ASSUMED TO BE EQUAL TO ONE AND
C    ARE NOT STORED.  THE ARRAY D CONTAINS THE RECIPROCALS OF THE
C    DIAGONAL ENTRIES OF THE MATRIX D.
C
C    III. ADDITIONAL STORAGE SAVINGS
C         IN NSFC, R AND IC CAN BE THE SAME ARRAY IN THE CALLING
C    SEQUENCE IF NO REORDERING OF THE COEFFICIENT MATRIX HAS BEEN DONE.
C         IN NNFC, R, C, AND IC CAN ALL BE THE SAME ARRAY IF NO
C    REORDERING HAS BEEN DONE.  IF ONLY THE ROWS HAVE BEEN REORDERED,
C    THEN C AND IC CAN BE THE SAME ARRAY.  IF THE ROW AND COLUMN
C    ORDERINGS ARE THE SAME, THEN R AND C CAN BE THE SAME ARRAY.  Z AND
C    ROW CAN BE THE SAME ARRAY.
C         IN NNSC OR NNTC, R AND C CAN BE THE SAME ARRAY IF NO
C    REORDERING HAS BEEN DONE OR IF THE ROW AND COLUMN ORDERINGS ARE THE
C    SAME.  Z AND B CAN BE THE SAME ARRAY.  HOWEVER, THEN B WILL BE
C    DESTROYED.
C
C    IV.  PARAMETERS
C         FOLLOWING IS A LIST OF PARAMETERS TO THE PROGRAMS.  NAMES ARE
C    UNIFORM AMONG THE VARIOUS SUBROUTINES.  CLASS ABBREVIATIONS ARE --
C       N - INTEGER VARIABLE
C       F - REAL VARIABLE
C       V - SUPPLIES A VALUE TO A SUBROUTINE
C       R - RETURNS A RESULT FROM A SUBROUTINE
C       I - USED INTERNALLY BY A SUBROUTINE
C       A - ARRAY
C
C CLASS - PARAMETER
C ------+----------
C FVA   - A     - NONZERO ENTRIES OF THE COEFFICIENT MATRIX M, STORED
C       -           BY ROWS.
C       -           SIZE = NUMBER OF NONZERO ENTRIES IN M.
C FVA   - B     - RIGHT-HAND SIDE B.
C       -           SIZE = N.
C NVA   - C     - ORDERING OF THE COLUMNS OF M.
C       -           SIZE = N.
C FVRA  - D     - RECIPROCALS OF THE DIAGONAL ENTRIES OF THE MATRIX D.
C       -           SIZE = N.
C NR    - FLAG  - ERROR FLAG.  VALUES AND THEIR MEANINGS ARE --
C       -            0     NO ERRORS DETECTED
C       -            N+K   NULL ROW IN A  --  ROW = K
C       -           2N+K   DUPLICATE ENTRY IN A  --  ROW = K
C       -           3N+K   INSUFFICIENT STORAGE FOR JL  --  ROW = K
C       -           4N+1   INSUFFICIENT STORAGE FOR L
C       -           5N+K   NULL PIVOT  --  ROW = K
C       -           6N+K   INSUFFICIENT STORAGE FOR JU  --  ROW = K
C       -           7N+1   INSUFFICIENT STORAGE FOR U
C       -           8N+K   ZERO PIVOT  --  ROW = K
C NVA   - IA    - POINTERS TO DELIMIT THE ROWS OF A.
C       -           SIZE = N+1.
C NVRA  - IJL   - POINTERS TO THE FIRST ELEMENT IN EACH COLUMN IN JL,
C       -           USED TO COMPRESS STORAGE IN JL.
C       -           SIZE = N.
C NVRA  - IJU   - POINTERS TO THE FIRST ELEMENT IN EACH ROW IN JU, USED
C       -           TO COMPRESS STORAGE IN JU.
C       -           SIZE = N.
C NVRA  - IL    - POINTERS TO DELIMIT THE COLUMNS OF L.
C       -           SIZE = N+1.
C NVRA  - IU    - POINTERS TO DELIMIT THE ROWS OF U.
C       -           SIZE = N+1.
C NVA   - JA    - COLUMN NUMBERS CORRESPONDING TO THE ELEMENTS OF A.
C       -           SIZE = SIZE OF A.
C NVRA  - JL    - ROW NUMBERS CORRESPONDING TO THE ELEMENTS OF L.
C       -           SIZE = JLMAX.
C NV    - JLMAX - DECLARED DIMENSION OF JL.  JLMAX MUST BE LARGER THAN
C       -           THE NUMBER OF NONZEROS IN THE STRICT LOWER TRIANGLE
C       -           OF M PLUS FILLIN MINUS COMPRESSION.
C NVRA  - JU    - COLUMN NUMBERS CORRESPONDING TO THE ELEMENTS OF U.
C       -           SIZE = JUMAX.
C NV    - JUMAX - DECLARED DIMENSION OF JU.  JUMAX MUST BE LARGER THAN
C       -           THE NUMBER OF NONZEROS IN THE STRICT UPPER TRIANGLE
C       -           OF M PLUS FILLIN MINUS COMPRESSION.
C FVRA  - L     - NONZERO ENTRIES IN THE STRICT LOWER TRIANGULAR PORTION
C       -           OF THE MATRIX L, STORED BY COLUMNS.
C       -           SIZE = LMAX.
C NV    - LMAX  - DECLARED DIMENSION OF L.  LMAX MUST BE LARGER THAN
C       -           THE NUMBER OF NONZEROS IN THE STRICT LOWER TRIANGLE
C       -           OF M PLUS FILLIN  (IL(N+1)-1 AFTER NSFC).
C NV    - N     - NUMBER OF VARIABLES/EQUATIONS.
C NVA   - R     - ORDERING OF THE ROWS OF M.
C       -           SIZE = N.
C FVRA  - U     - NONZERO ENTRIES IN THE STRICT UPPER TRIANGULAR PORTION
C       -           OF THE MATRIX U, STORED BY ROWS.
C       -           SIZE = UMAX.
C NV    - UMAX  - DECLARED DIMENSION OF U.  UMAX MUST BE LARGER THAN
C       -           THE NUMBER OF NONZEROS IN THE STRICT UPPER TRIANGLE
C       -           OF M PLUS FILLIN  (IU(N+1)-1 AFTER NSFC).
C FRA   - Z     - SOLUTION X.
C       -           SIZE = N.
C
C       ----------------------------------------------------------------
C
C*** SUBROUTINE NROC
C*** REORDERS ROWS OF A, LEAVING ROW ORDER UNCHANGED
C
C
C       INPUT PARAMETERS.. N, IC, IA, JA, A
C       OUTPUT PARAMETERS.. JA, A, FLAG
C
C       PARAMETERS USED INTERNALLY..
C NIA   - P     - AT THE KTH STEP, P IS A LINKED LIST OF THE REORDERED
C       -           COLUMN INDICES OF THE KTH ROW OF A.  P(N+1) POINTS
C       -           TO THE FIRST ENTRY IN THE LIST.
C       -           SIZE = N+1.
C NIA   - JAR   - AT THE KTH STEP,JAR CONTAINS THE ELEMENTS OF THE
C       -           REORDERED COLUMN INDICES OF A.
C       -           SIZE = N.
C FIA   - AR    - AT THE KTH STEP, AR CONTAINS THE ELEMENTS OF THE
C       -           REORDERED ROW OF A.
C       -           SIZE = N.
C
      INTEGER  IC(1), IA(1), JA(1), JAR(1), P(1), FLAG, K, JMIN, JMAX,
     *         J, NEWJ, I, N
      DOUBLE PRECISION  A(1), AR(1)
C
C  ******  FOR EACH NONEMPTY ROW  *******************************
      DO 5 K=1,N
        JMIN = IA(K)
        JMAX = IA(K+1) - 1
        IF(JMIN .GT. JMAX) GO TO 5
        P(N+1) = N + 1
C  ******  INSERT EACH ELEMENT IN THE LIST  *********************
        DO 3 J=JMIN,JMAX
          NEWJ = IC(JA(J))
          I = N + 1
   1      IF(P(I) .GE. NEWJ) GO TO 2
            I = P(I)
            GO TO 1
   2      IF(P(I) .EQ. NEWJ) GO TO 102
          P(NEWJ) = P(I)
          P(I) = NEWJ
          JAR(NEWJ) = JA(J)
          AR(NEWJ) = A(J)
   3      CONTINUE
C  ******  REPLACE OLD ROW IN JA AND A  *************************
        I = N + 1
        DO 4 J=JMIN,JMAX
          I = P(I)
          JA(J) = JAR(I)
   4      A(J) = AR(I)
   5    CONTINUE
      FLAG = 0
      RETURN
C
C ** ERROR.. DUPLICATE ENTRY IN A
 102  FLAG = N + K
      RETURN
      END
      SUBROUTINE NSFC
     *      (N, R, IC, IA,JA, JLMAX,IL,JL,IJL, JUMAX,IU,JU,IJU,
     *       Q, IRA,JRA, IRAC, IRL,JRL, IRU,JRU, FLAG)
C
C*** SUBROUTINE NSFC
C*** SYMBOLIC LDU-FACTORIZATION OF NONSYMMETRIC SPARSE MATRIX
C      (COMPRESSED POINTER STORAGE)
C
C
C       INPUT VARIABLES.. N, R, IC, IA, JA, JLMAX, JUMAX.
C       OUTPUT VARIABLES.. IL, JL, IJL, IU, JU, IJU, FLAG.
C
C       PARAMETERS USED INTERNALLY..
C NIA   - Q     - SUPPOSE  M*  IS THE RESULT OF REORDERING  M.  IF
C       -           PROCESSING OF THE ITH ROW OF  M*  (HENCE THE ITH
C       -           ROW OF  U) IS BEING DONE,  Q(J)  IS INITIALLY
C       -           NONZERO IF  M*(I,J) IS NONZERO (J.GE.I).  SINCE
C       -           VALUES NEED NOT BE STORED, EACH ENTRY POINTS TO THE
C       -           NEXT NONZERO AND  Q(N+1)  POINTS TO THE FIRST.  N+1
C       -           INDICATES THE END OF THE LIST.  FOR EXAMPLE, IF N=9
C       -           AND THE 5TH ROW OF  M*  IS
C       -              0 X X 0 X 0 0 X 0
C       -           THEN  Q  WILL INITIALLY BE
C       -              A A A A 8 A A 10 5           (A - ARBITRARY).
C       -           AS THE ALGORITHM PROCEEDS, OTHER ELEMENTS OF  Q
C       -           ARE INSERTED IN THE LIST BECAUSE OF FILLIN.
C       -           Q  IS USED IN AN ANALOGOUS MANNER TO COMPUTE THE
C       -           ITH COLUMN OF  L.
C       -           SIZE = N+1.
C NIA   - IRA,  - VECTORS USED TO FIND THE COLUMNS OF  M.  AT THE KTH
C NIA   - JRA,      STEP OF THE FACTORIZATION,  IRAC(K)  POINTS TO THE
C NIA   - IRAC      HEAD OF A LINKED LIST IN  JRA  OF ROW INDICES I
C       -           SUCH THAT I .GE. K AND  M(I,K)  IS NONZERO.  ZERO
C       -           INDICATES THE END OF THE LIST.  IRA(I)  (I.GE.K)
C       -           POINTS TO THE SMALLEST J SUCH THAT J .GE. K AND
C       -           M(I,J)  IS NONZERO.
C       -           SIZE OF EACH = N.
C NIA   - IRL,  - VECTORS USED TO FIND THE ROWS OF  L.  AT THE KTH STEP
C NIA   - JRL       OF THE FACTORIZATION,  JRL(K)  POINTS TO THE HEAD
C       -           OF A LINKED LIST IN  JRL  OF COLUMN INDICES J
C       -           SUCH J .LT. K AND  L(K,J)  IS NONZERO.  ZERO
C       -           INDICATES THE END OF THE LIST.  IRL(J)  (J.LT.K)
C       -           POINTS TO THE SMALLEST I SUCH THAT I .GE. K AND
C       -           L(I,J)  IS NONZERO.
C       -           SIZE OF EACH = N.
C NIA   - IRU,  - VECTORS USED IN A MANNER ANALOGOUS TO  IRL AND JRL
C NIA   - JRU       TO FIND THE COLUMNS OF  U.
C       -           SIZE OF EACH = N.
C
C  INTERNAL VARIABLES..
C    JLPTR - POINTS TO THE LAST POSITION USED IN  JL.
C    JUPTR - POINTS TO THE LAST POSITION USED IN  JU.
C    JMIN,JMAX - ARE THE INDICES IN  A OR U  OF THE FIRST AND LAST
C                ELEMENTS TO BE EXAMINED IN A GIVEN ROW.
C                FOR EXAMPLE,  JMIN=IA(K), JMAX=IA(K+1)-1.
C
      INTEGER CEND, QM, REND, RK, VJ
      INTEGER IA(1), JA(1), IRA(1), JRA(1), IL(1), JL(1), IJL(1)
      INTEGER IU(1), JU(1), IJU(1), IRL(1), JRL(1), IRU(1), JRU(1)
      INTEGER R(1), IC(1), Q(1), IRAC(1), FLAG, N, JLMAX, JUMAX,
     *        JLMIN, JUPTR, K, IAK, JAIAK, LUK, M, LASTID,
     *        JMIN, JMAX, LONG, JTMP, J, IRLL, IRUL, IRAI,
     *        JAIRAI, NP1, LASTI, I, I1, JLPTR, JUMIN
C
C  ******  INITIALIZE POINTERS  ****************************************
      NP1 = N + 1
      JLMIN = 1
      JLPTR = 0
      IL(1) = 1
      JUMIN = 1
      JUPTR = 0
      IU(1) = 1
      DO 1 K=1,N
        IRAC(K) = 0
        JRA(K) = 0
        JRL(K) = 0
   1    JRU(K) = 0
C  ******  INITIALIZE COLUMN POINTERS FOR A  ***************************
      DO 2 K=1,N
        RK = R(K)
        IAK = IA(RK)
        IF (IAK .GE. IA(RK+1))  GO TO 101
        JAIAK = IC(JA(IAK))
        IF (JAIAK .GT. K)  GO TO 105
        JRA(K) = IRAC(JAIAK)
        IRAC(JAIAK) = K
   2    IRA(K) = IAK
C
C  ******  FOR EACH COLUMN OF L AND ROW OF U  **************************
      DO 41 K=1,N
C
C  ******  INITIALIZE Q FOR COMPUTING KTH COLUMN OF L  *****************
        Q(NP1) = NP1
        LUK = -1
C  ******  BY FILLING IN KTH COLUMN OF A  ******************************
        VJ = IRAC(K)
        IF (VJ .EQ. 0)  GO TO 5
   3      QM = NP1
   4      M = QM
          QM =  Q(M)
          IF (QM .LT. VJ)  GO TO 4
          IF (QM .EQ. VJ)  GO TO 102
            LUK = LUK + 1
            Q(M) = VJ
            Q(VJ) = QM
            VJ = JRA(VJ)
            IF (VJ .NE. 0)  GO TO 3
C  ******  LINK THROUGH JRU  *******************************************
   5    LASTID = 0
        LASTI = 0
        IJL(K) = JLPTR
        I = K
   6      I = JRU(I)
          IF (I .EQ. 0)  GO TO 10
          QM = NP1
          JMIN = IRL(I)
          JMAX = IJL(I) + IL(I+1) - IL(I) - 1
          LONG = JMAX - JMIN
          IF (LONG .LT. 0)  GO TO 6
          JTMP = JL(JMIN)
          IF (JTMP .NE. K)  LONG = LONG + 1
          IF (JTMP .EQ. K)  R(I) = -R(I)
          IF (LASTID .GE. LONG)  GO TO 7
            LASTI = I
            LASTID = LONG
C  ******  AND MERGE THE CORRESPONDING COLUMNS INTO THE KTH COLUMN  ****
   7      DO 9 J=JMIN,JMAX
            VJ = JL(J)
   8        M = QM
            QM = Q(M)
            IF (QM .LT. VJ)  GO TO 8
            IF (QM .EQ. VJ)  GO TO 9
              LUK = LUK + 1
              Q(M) = VJ
              Q(VJ) = QM
              QM = VJ
   9        CONTINUE
            GO TO 6
C  ******  LASTI IS THE LONGEST COLUMN MERGED INTO THE KTH  ************
C  ******  SEE IF IT EQUALS THE ENTIRE KTH COLUMN  *********************
  10    QM = Q(NP1)
        IF (QM .NE. K)  GO TO 105
        IF (LUK .EQ. 0)  GO TO 17
        IF (LASTID .NE. LUK)  GO TO 11
C  ******  IF SO, JL CAN BE COMPRESSED  ********************************
        IRLL = IRL(LASTI)
        IJL(K) = IRLL + 1
        IF (JL(IRLL) .NE. K)  IJL(K) = IJL(K) - 1
        GO TO 17
C  ******  IF NOT, SEE IF KTH COLUMN CAN OVERLAP THE PREVIOUS ONE  *****
  11    IF (JLMIN .GT. JLPTR)  GO TO 15
        QM = Q(QM)
        DO 12 J=JLMIN,JLPTR
          IF (JL(J) - QM)  12, 13, 15
  12      CONTINUE
        GO TO 15
  13    IJL(K) = J
        DO 14 I=J,JLPTR
          IF (JL(I) .NE. QM)  GO TO 15
          QM = Q(QM)
          IF (QM .GT. N)  GO TO 17
  14      CONTINUE
        JLPTR = J - 1
C  ******  MOVE COLUMN INDICES FROM Q TO JL, UPDATE VECTORS  ***********
  15    JLMIN = JLPTR + 1
        IJL(K) = JLMIN
        IF (LUK .EQ. 0)  GO TO 17
        JLPTR = JLPTR + LUK
        IF (JLPTR .GT. JLMAX)  GO TO 103
          QM = Q(NP1)
          DO 16 J=JLMIN,JLPTR
            QM = Q(QM)
  16        JL(J) = QM
  17    IRL(K) = IJL(K)
        IL(K+1) = IL(K) + LUK
C
C  ******  INITIALIZE Q FOR COMPUTING KTH ROW OF U  ********************
        Q(NP1) = NP1
        LUK = -1
C  ******  BY FILLING IN KTH ROW OF REORDERED A  ***********************
        RK = R(K)
        JMIN = IRA(K)
        JMAX = IA(RK+1) - 1
        IF (JMIN .GT. JMAX)  GO TO 20
        DO 19 J=JMIN,JMAX
          VJ = IC(JA(J))
          QM = NP1
  18      M = QM
          QM = Q(M)
          IF (QM .LT. VJ)  GO TO 18
          IF (QM .EQ. VJ)  GO TO 102
            LUK = LUK + 1
            Q(M) = VJ
            Q(VJ) = QM
  19      CONTINUE
C  ******  LINK THROUGH JRL,  ******************************************
  20    LASTID = 0
        LASTI = 0
        IJU(K) = JUPTR
        I = K
        I1 = JRL(K)
  21      I = I1
          IF (I .EQ. 0)  GO TO 26
          I1 = JRL(I)
          QM = NP1
          JMIN = IRU(I)
          JMAX = IJU(I) + IU(I+1) - IU(I) - 1
          LONG = JMAX - JMIN
          IF (LONG .LT. 0)  GO TO 21
          JTMP = JU(JMIN)
          IF (JTMP .EQ. K)  GO TO 22
C  ******  UPDATE IRL AND JRL, *****************************************
            LONG = LONG + 1
            CEND = IJL(I) + IL(I+1) - IL(I)
            IRL(I) = IRL(I) + 1
            IF (IRL(I) .GE. CEND)  GO TO 22
              J = JL(IRL(I))
              JRL(I) = JRL(J)
              JRL(J) = I
  22      IF (LASTID .GE. LONG)  GO TO 23
            LASTI = I
            LASTID = LONG
C  ******  AND MERGE THE CORRESPONDING ROWS INTO THE KTH ROW  **********
  23      DO 25 J=JMIN,JMAX
            VJ = JU(J)
  24        M = QM
            QM = Q(M)
            IF (QM .LT. VJ)  GO TO 24
            IF (QM .EQ. VJ)  GO TO 25
              LUK = LUK + 1
              Q(M) = VJ
              Q(VJ) = QM
              QM = VJ
  25        CONTINUE
          GO TO 21
C  ******  UPDATE JRL(K) AND IRL(K)  ***********************************
  26    IF (IL(K+1) .LE. IL(K))  GO TO 27
          J = JL(IRL(K))
          JRL(K) = JRL(J)
          JRL(J) = K
C  ******  LASTI IS THE LONGEST ROW MERGED INTO THE KTH  ***************
C  ******  SEE IF IT EQUALS THE ENTIRE KTH ROW  ************************
  27    QM = Q(NP1)
        IF (QM .NE. K)  GO TO 105
        IF (LUK .EQ. 0)  GO TO 34
        IF (LASTID .NE. LUK)  GO TO 28
C  ******  IF SO, JU CAN BE COMPRESSED  ********************************
        IRUL = IRU(LASTI)
        IJU(K) = IRUL + 1
        IF (JU(IRUL) .NE. K)  IJU(K) = IJU(K) - 1
        GO TO 34
C  ******  IF NOT, SEE IF KTH ROW CAN OVERLAP THE PREVIOUS ONE  ********
  28    IF (JUMIN .GT. JUPTR)  GO TO 32
        QM = Q(QM)
        DO 29 J=JUMIN,JUPTR
          IF (JU(J) - QM)  29, 30, 32
  29      CONTINUE
        GO TO 32
  30    IJU(K) = J
        DO 31 I=J,JUPTR
          IF (JU(I) .NE. QM)  GO TO 32
          QM = Q(QM)
          IF (QM .GT. N)  GO TO 34
  31      CONTINUE
        JUPTR = J - 1
C  ******  MOVE ROW INDICES FROM Q TO JU, UPDATE VECTORS  **************
  32    JUMIN = JUPTR + 1
        IJU(K) = JUMIN
        IF (LUK .EQ. 0)  GO TO 34
        JUPTR = JUPTR + LUK
        IF (JUPTR .GT. JUMAX)  GO TO 106
          QM = Q(NP1)
          DO 33 J=JUMIN,JUPTR
            QM = Q(QM)
  33        JU(J) = QM
  34    IRU(K) = IJU(K)
        IU(K+1) = IU(K) + LUK
C
C  ******  UPDATE IRU, JRU  ********************************************
        I = K
  35      I1 = JRU(I)
          IF (R(I) .LT. 0)  GO TO 36
          REND = IJU(I) + IU(I+1) - IU(I)
          IF (IRU(I) .GE. REND)  GO TO 37
            J = JU(IRU(I))
            JRU(I) = JRU(J)
            JRU(J) = I
            GO TO 37
  36      R(I) = -R(I)
  37      I = I1
          IF (I .EQ. 0)  GO TO 38
          IRU(I) = IRU(I) + 1
          GO TO 35
C
C  ******  UPDATE IRA, JRA, IRAC  **************************************
  38    I = IRAC(K)
        IF (I .EQ. 0)  GO TO 41
  39      I1 = JRA(I)
          IRA(I) = IRA(I) + 1
          IF (IRA(I) .GE. IA(R(I)+1))  GO TO 40
          IRAI = IRA(I)
          JAIRAI = IC(JA(IRAI))
          IF (JAIRAI .GT. I)  GO TO 40
          JRA(I) = IRAC(JAIRAI)
          IRAC(JAIRAI) = I
  40      I = I1
          IF (I .NE. 0)  GO TO 39
  41    CONTINUE
C
      IJL(N) = JLPTR
      IJU(N) = JUPTR
      FLAG = 0
      RETURN
C
C ** ERROR.. NULL ROW IN A
 101  FLAG = N + RK
      RETURN
C ** ERROR.. DUPLICATE ENTRY IN A
 102  FLAG = 2*N + RK
      RETURN
C ** ERROR.. INSUFFICIENT STORAGE FOR JL
 103  FLAG = 3*N + K
      RETURN
C ** ERROR.. NULL PIVOT
 105  FLAG = 5*N + K
      RETURN
C ** ERROR.. INSUFFICIENT STORAGE FOR JU
 106  FLAG = 6*N + K
      RETURN
      END
      SUBROUTINE NNFC
     *     (N, R,C,IC, IA,JA,A, Z, B,
     *      LMAX,IL,JL,IJL,L, D, UMAX,IU,JU,IJU,U,
     *      ROW, TMP, IRL,JRL, FLAG)
C
C*** SUBROUTINE NNFC
C*** NUMERICAL LDU-FACTORIZATION OF SPARSE NONSYMMETRIC MATRIX AND
C      SOLUTION OF SYSTEM OF LINEAR EQUATIONS (COMPRESSED POINTER
C      STORAGE)
C
C
C       INPUT VARIABLES..  N, R, C, IC, IA, JA, A, B,
C                          IL, JL, IJL, LMAX, IU, JU, IJU, UMAX
C       OUTPUT VARIABLES.. Z, L, D, U, FLAG
C
C       PARAMETERS USED INTERNALLY..
C NIA   - IRL,  - VECTORS USED TO FIND THE ROWS OF  L.  AT THE KTH STEP
C NIA   - JRL       OF THE FACTORIZATION,  JRL(K)  POINTS TO THE HEAD
C       -           OF A LINKED LIST IN  JRL  OF COLUMN INDICES J
C       -           SUCH J .LT. K AND  L(K,J)  IS NONZERO.  ZERO
C       -           INDICATES THE END OF THE LIST.  IRL(J)  (J.LT.K)
C       -           POINTS TO THE SMALLEST I SUCH THAT I .GE. K AND
C       -           L(I,J)  IS NONZERO.
C       -           SIZE OF EACH = N.
C FIA   - ROW   - HOLDS INTERMEDIATE VALUES IN CALCULATION OF  U AND L.
C       -           SIZE = N.
C FIA   - TMP   - HOLDS NEW RIGHT-HAND SIDE  B*  FOR SOLUTION OF THE
C       -           EQUATION UX = B*.
C       -           SIZE = N.
C
C  INTERNAL VARIABLES..
C    JMIN, JMAX - INDICES OF THE FIRST AND LAST POSITIONS IN A ROW TO
C      BE EXAMINED.
C    SUM - USED IN CALCULATING  TMP.
C
      INTEGER RK,UMAX,N,LMAX,K,I2,JMIN,JMAX,J,MU,IJLB,I1,I
      INTEGER  R(1), C(1), IC(1), IA(1), JA(1), IL(1), JL(1), IJL(1)
      INTEGER  IU(1), JU(1), IJU(1), IRL(1), JRL(1), FLAG
      DOUBLE PRECISION  A(1), L(1), D(1), U(1), Z(1), B(1), ROW(1)
      DOUBLE PRECISION  TMP(1), LKI, SUM, DK
C
C  ******  INITIALIZE POINTERS AND TEST STORAGE  ***********************
      IF(IL(N+1)-1 .GT. LMAX) GO TO 104
      IF(IU(N+1)-1 .GT. UMAX) GO TO 107
      DO 1 K=1,N
        IRL(K) = IL(K)
        JRL(K) = 0
   1    CONTINUE
C
C  ******  FOR EACH ROW  ***********************************************
      DO 19 K=1,N
C  ******  REVERSE JRL AND ZERO ROW WHERE KTH ROW OF L WILL FILL IN  ***
        ROW(K) = 0
        I1 = 0
        IF (JRL(K) .EQ. 0) GO TO 3
        I = JRL(K)
   2    I2 = JRL(I)
        JRL(I) = I1
        I1 = I
        ROW(I) = 0
        I = I2
        IF (I .NE. 0) GO TO 2
C  ******  SET ROW TO ZERO WHERE U WILL FILL IN  ***********************
   3    JMIN = IJU(K)
        JMAX = JMIN + IU(K+1) - IU(K) - 1
        IF (JMIN .GT. JMAX) GO TO 5
        DO 4 J=JMIN,JMAX
   4      ROW(JU(J)) = 0
C  ******  PLACE KTH ROW OF A IN ROW  **********************************
   5    RK = R(K)
        JMIN = IA(RK)
        JMAX = IA(RK+1) - 1
        DO 6 J=JMIN,JMAX
          ROW(IC(JA(J))) = A(J)
   6      CONTINUE
C  ******  INITIALIZE SUM, AND LINK THROUGH JRL  ***********************
        SUM = B(RK)
        I = I1
        IF (I .EQ. 0) GO TO 10
C  ******  ASSIGN THE KTH ROW OF L AND ADJUST ROW, SUM  ****************
   7      LKI = -ROW(I)
C  ******  IF L IS NOT REQUIRED, THEN COMMENT OUT THE FOLLOWING LINE  **
          L(IRL(I)) = -LKI
          SUM = SUM + LKI * TMP(I)
          JMIN = IU(I)
          JMAX = IU(I+1) - 1
          IF (JMIN .GT. JMAX) GO TO 9
          MU = IJU(I) - JMIN
          DO 8 J=JMIN,JMAX
   8        ROW(JU(MU+J)) = ROW(JU(MU+J)) + LKI * U(J)
   9      I = JRL(I)
          IF (I .NE. 0) GO TO 7
C
C  ******  ASSIGN KTH ROW OF U AND DIAGONAL D, SET TMP(K)  *************
  10    IF (ROW(K) .EQ. 0.0D0) GO TO 108
        DK = 1.0D0 / ROW(K)
        D(K) = DK
        TMP(K) = SUM * DK
        IF (K .EQ. N) GO TO 19
        JMIN = IU(K)
        JMAX = IU(K+1) - 1
        IF (JMIN .GT. JMAX)  GO TO 12
        MU = IJU(K) - JMIN
        DO 11 J=JMIN,JMAX
  11      U(J) = ROW(JU(MU+J)) * DK
  12    CONTINUE
C
C  ******  UPDATE IRL AND JRL, KEEPING JRL IN DECREASING ORDER  ********
        I = I1
        IF (I .EQ. 0) GO TO 18
  14    IRL(I) = IRL(I) + 1
        I1 = JRL(I)
        IF (IRL(I) .GE. IL(I+1)) GO TO 17
        IJLB = IRL(I) - IL(I) + IJL(I)
        J = JL(IJLB)
  15    IF (I .GT. JRL(J)) GO TO 16
          J = JRL(J)
          GO TO 15
  16    JRL(I) = JRL(J)
        JRL(J) = I
  17    I = I1
        IF (I .NE. 0) GO TO 14
  18    IF (IRL(K) .GE. IL(K+1)) GO TO 19
        J = JL(IJL(K))
        JRL(K) = JRL(J)
        JRL(J) = K
  19    CONTINUE
C
C  ******  SOLVE  UX = TMP  BY BACK SUBSTITUTION  **********************
      K = N
      DO 22 I=1,N
        SUM =  TMP(K)
        JMIN = IU(K)
        JMAX = IU(K+1) - 1
        IF (JMIN .GT. JMAX)  GO TO 21
        MU = IJU(K) - JMIN
        DO 20 J=JMIN,JMAX
  20      SUM = SUM - U(J) * TMP(JU(MU+J))
  21    TMP(K) =  SUM
        Z(C(K)) =  SUM
  22    K = K-1
      FLAG = 0
      RETURN
C
C ** ERROR.. INSUFFICIENT STORAGE FOR L
 104  FLAG = 4*N + 1
      RETURN
C ** ERROR.. INSUFFICIENT STORAGE FOR U
 107  FLAG = 7*N + 1
      RETURN
C ** ERROR.. ZERO PIVOT
 108  FLAG = 8*N + K
      RETURN
      END
      SUBROUTINE NNSC
     *     (N, R, C, IL, JL, IJL, L, D, IU, JU, IJU, U, Z, B, TMP)
C
C*** SUBROUTINE NNSC
C*** NUMERICAL SOLUTION OF SPARSE NONSYMMETRIC SYSTEM OF LINEAR
C      EQUATIONS GIVEN LDU-FACTORIZATION (COMPRESSED POINTER STORAGE)
C
C
C       INPUT VARIABLES..  N, R, C, IL, JL, IJL, L, D, IU, JU, IJU, U, B
C       OUTPUT VARIABLES.. Z
C
C       PARAMETERS USED INTERNALLY..
C FIA   - TMP   - TEMPORARY VECTOR WHICH GETS RESULT OF SOLVING  LY = B.
C       -           SIZE = N.
C
C  INTERNAL VARIABLES..
C    JMIN, JMAX - INDICES OF THE FIRST AND LAST POSITIONS IN A ROW OF
C      U OR L  TO BE USED.
C
      INTEGER R(1), C(1), IL(1), JL(1), IJL(1), IU(1), JU(1), IJU(1)
      INTEGER N, K, JMIN, JMAX, I, MU, ML, J
      DOUBLE PRECISION  L(1), D(1), U(1), B(1), Z(1), TMP(1), TMPK,SUM
C
C  ******  SET TMP TO REORDERED B  *************************************
      DO 1 K=1,N
   1    TMP(K) = B(R(K))
C  ******  SOLVE  LY = B  BY FORWARD SUBSTITUTION  *********************
      DO 3 K=1,N
        JMIN = IL(K)
        JMAX = IL(K+1) - 1
        TMPK = -D(K) * TMP(K)
        TMP(K) = -TMPK
        IF (JMIN .GT. JMAX) GO TO 3
        ML = IJL(K) - JMIN
        DO 2 J=JMIN,JMAX
   2      TMP(JL(ML+J)) = TMP(JL(ML+J)) + TMPK * L(J)
   3    CONTINUE
C  ******  SOLVE  UX = Y  BY BACK SUBSTITUTION  ************************
      K = N
      DO 6 I=1,N
        SUM = -TMP(K)
        JMIN = IU(K)
        JMAX = IU(K+1) - 1
        IF (JMIN .GT. JMAX) GO TO 5
        MU = IJU(K) - JMIN
        DO 4 J=JMIN,JMAX
   4      SUM = SUM + U(J) * TMP(JU(MU+J))
   5    TMP(K) = -SUM
        Z(C(K)) = -SUM
        K = K - 1
   6    CONTINUE
      RETURN
      END
      SUBROUTINE NNTC
     *     (N, R, C, IL, JL, IJL, L, D, IU, JU, IJU, U, Z, B, TMP)
C
C*** SUBROUTINE NNTC
C*** NUMERIC SOLUTION OF THE TRANSPOSE OF A SPARSE NONSYMMETRIC SYSTEM
C      OF LINEAR EQUATIONS GIVEN LU-FACTORIZATION (COMPRESSED POINTER
C      STORAGE)
C
C
C       INPUT VARIABLES..  N, R, C, IL, JL, IJL, L, D, IU, JU, IJU, U, B
C       OUTPUT VARIABLES.. Z
C
C       PARAMETERS USED INTERNALLY..
C FIA   - TMP   - TEMPORARY VECTOR WHICH GETS RESULT OF SOLVING UT Y = B
C       -           SIZE = N.
C
C  INTERNAL VARIABLES..
C    JMIN, JMAX - INDICES OF THE FIRST AND LAST POSITIONS IN A ROW OF
C      U OR L  TO BE USED.
C
      INTEGER R(1), C(1), IL(1), JL(1), IJL(1), IU(1), JU(1), IJU(1)
      INTEGER N, K, JMIN, JMAX, MU, ML, I, J
      DOUBLE PRECISION L(1), D(1), U(1), B(1), Z(1), TMP(1), TMPK,SUM
C
C  ******  SET TMP TO REORDERED B  *************************************
      DO 1 K=1,N
   1    TMP(K) = B(C(K))
C  ******  SOLVE  UT Y = B  BY FORWARD SUBSTITUTION  *******************
      DO 3 K=1,N
        JMIN = IU(K)
        JMAX = IU(K+1) - 1
        TMPK = -TMP(K)
        IF (JMIN .GT. JMAX) GO TO 3
        MU = IJU(K) - JMIN
        DO 2 J=JMIN,JMAX
   2      TMP(JU(MU+J)) = TMP(JU(MU+J)) + TMPK * U(J)
   3    CONTINUE
C  ******  SOLVE  LT X = Y  BY BACK SUBSTITUTION  **********************
      K = N
      DO 6 I=1,N
        SUM = -TMP(K)
        JMIN = IL(K)
        JMAX = IL(K+1) - 1
        IF (JMIN .GT. JMAX) GO TO 5
        ML = IJL(K) - JMIN
        DO 4 J=JMIN,JMAX
   4      SUM = SUM + L(J) * TMP(JL(ML+J))
   5    TMP(K) = -SUM * D(K)
        Z(R(K)) = TMP(K)
        K = K - 1
   6    CONTINUE
      RETURN
      END
