C     THIS FILE CONTAINS THE BANDED AND FULL MATRIX ROUTINES USED BY THE
C     SPRINT PACKAGE.
C     THE ROUTINES CALLED TO FORM THE JACOBIAN MATRIX ARE
C     FULL CASE  PREPJF ; BANDED CASE PREPJB.
C     THE ROUTINES CALLED TO BACKSUBSTITUTE USING ITS FACTORED FORM ARE
C     FULL CASE  SOLSF  ; BANDED CASE SOLSB
C
C  IMPORTANT BEFORE THE MAIN SPRINT PACKAGE IS USED WITH THESE
C  ROUTINES  THE SETUP ROUTINE  MATSET (........) MUST BE CALLED.
C
       SUBROUTINE MATSET( LINALG, N, MU, ML, NWKJAC, NIWJAC, ISET)
       INTEGER LINALG, N, MU, ML, NWKJAC, NIWJAC, ISET
C**********************************************************************
C  ROUTINE TO INITIALISE THE FULL AND BANDED MATRIX ROUTINES USED
C  BY THE SPRINT PACKAGE.
C  PARAMETER LIST
C ----------------
C LINALG = 2 ; FULL MATRIX ROUTINES ARE BEING USED
C        = 1 ; BANDED MATRIX ROUTINES ARE IN USE. IN THIS CASE
C              ML IS THE LOWER HALF BANDWIDTH OF THE D.A.E. SYSTEM AND
C              MU IS THE UPPER HALF BANDWIDTH OF THE SYSTEM . BOTH
C              THESE PARAMETERS MUST BE SUPPLIED BY THE USER.
C
C N            THE NUMBER OF ORDINARY DIFFERENTIAL AND /OR ALGEBRAIC
C              EQUATIONS BEING SOLVED.
C       NOTE     IN THE CASE WHEN THIS NUMBER MAY CHANGE IN
C       ----   MID-INTEGRATION THIS SHOULD BE THE MAX NUMBER OF EQNS.
C              I.E. N = NMAX = MAX ( NEQ ).
C  NWKJAC    ; THE SIZE OF THE REAL WORKSPACE SUPPLIED BY THE USER FOR
C              THE LINEAR ALGEBRA ROUTINES.
C              IN THE CASE WHEN THE BANDED OPTION IS USED ,LINALG = 1,
C                     NWKJAC = (2*ML + MU + 1) * NMAX + 2
C              IN THE CASE WHEN THE FULL MATRIX OPTION IS USED.
C                     NWKJAC = NMAX * NMAX + 2
C              WHERE NMAX IS THE LARGEST SIZE OF THE O.D.E. PROBLEM
C              DIMENSION (IF THIS MAY CHANGE IN MID-INTEGRATION) OTHER-
C              WISE   NMAX = NEQ WHERE NEQ IS THE NUMBER OF ORDINARY
C              DIFFERENTIAL AND /OR ALGEBRAIC EQUATIONS.
C        NOTE  IF THE O.D.E. PROBLEM DOES CHANGE IN MID-INTEGRATION AND
C        ----  THE BANDED OPTION IS BEING USED AND EITHER OR BOTH OF
C              THE UPPER AND LOWER BANDWIDTHS CHANGE THEN THIS SETUP
C              ROUTINE MUST BE RECALLED BEFORE INTEGRATION IS
C              RECOMMENCED. (THIS CALL CAN BE MADE FROM INSIDE A
C              MONITOR ROUTINE IF NEED BE.)
C  NIWJAC    ; THE SIZE OF THE INTEGER WORKSPACE SUPPLIED BY THE USER
C              FOR THE LINEAR ALGEBRA ROUTINES. MUST BE OF SIZE >= NMAX
C              SEE ABOVE.
C
C  ISET  : ON ENTRY; ISET = 0 IMPLIES THAT THE JACOBIAN MATRIX IS TO BE
C              CALCULATED BY FINITE DIFFERENCING.
C              ISET = 1 IMPLIES THAT THE USER SUPPLIED FORM OF THE
C              JACOBIAN IS TO BE USED.
C
C  OUTPUT PARAMETER;
C  ISET    ON  EXIT . ON EXIT IF ISET < 0 THEN EITHER ISET WAS 1 OR ONE
C              OF NIWJAC , NWKJAC IS TOO SMALL - AN ERROR MESSAGE IS
C              OUTPUT VIA THE ERROR HANDLING ROUTINE SERROR
C**********************************************************************
       INTEGER MU1, ML1, I, ICALL
       CHARACTER*6 ALCODE, SINGLR
       COMMON /ALGCHK/ ALCODE,SINGLR
       COMMON /PASJB1/ MU1, ML1, ICALL
       SAVE   /ALGCHK/, /PASJB1/
C
       ICALL = ISET
       SINGLR = 'SINGLR'
       IF(LINALG .EQ. 1) THEN
         MU1 = MU
         ALCODE = 'LPBAND'
         ML1 = ML
         I = (2*ML + MU + 1)*N + 2
         IF(ML.LT.0.OR.ML.GT.N) ISET= -1
         IF(MU.LT.0.OR.MU.GT.N) ISET= -1
         IF(ISET .EQ.-1)THEN
        CALL SERROR(' MATSET- BANDWIDTHS MU (= I1) AND ML (=I2) ARE NOT
     1       IN THE RANGE 0 TO N', 1, 2, MU, ML, 0, 0.0D0, 0.0D0)
        RETURN
      END IF
        ELSE IF (LINALG .EQ. 2) THEN
                 I = N*N + 2
                 ALCODE = 'LPFULL'
             ELSE
                 CALL SERROR(' SETUP ROUTINES FOR LINPACK FULL OR BAND
     1           MATRICES- ILLEGAL VALUE OF LINALG (=I1)', 1, 1, LINALG,
     2           0, 0, 0.0D0, 0.0D0)
                 ISET = -1
        END IF
        IF(I .GT. NWKJAC)THEN
          CALL SERROR(' SETUP ROUTINES FOR FULL OR BANDED MATRICES THE
     1    REAL USER WORKSPACE OF LENGTH (=I1) IS SMALLER THAN NEEDED=I2'
     2    , 1, 2, NWKJAC, I, 0, 0.0D0, 0.0D0)
          ISET = -1
        END IF
        IF( NIWJAC .LT. N)THEN
          CALL SERROR(' SETUP ROUTINES FOR FULL OR BANDED MATRICES THE
     1    INTEGER USER WORKSPACE OF LENGTH (=I1) IS SMALLER THAN NEEDED
     2    (=I2)', 1, 2, NIWJAC, N, 0, 0.0D0, 0.0D0)
          ISET = -1
        END IF
        RETURN
        END
      SUBROUTINE PREPJB (NEQ, Y, YH, NYH, EWT, RTEM, SAVR, YDOT, WM,
     1   IWM, IFJ, H, EL0, TN, IFUNC, JAC, IDAE)
C**********************************************************************
      INTEGER NEQ, NYH, IWM, IFJ, IFUNC, IDAE(1)
      INTEGER I, I1, I2, IER, II, J, J1, JJ, LENP, ICALL,
     1   MBA, MBAND, MEB1, MEBAND, ML, ML3, MU, N, JZ
      INTEGER IOVFLO, ITRACE, IDEV
      DOUBLE PRECISION Y, YH, EWT, RTEM, SAVR, YDOT, WM, H, EL0, TN
      DOUBLE PRECISION DUNFLO, UROUND
      DOUBLE PRECISION CON, DI, FAC, HL0, R, R0, SRUR, YI, YJ, YJJ,
     1   VNORM, FAC1
      CHARACTER*6 ALCODE,SINGLR
      DIMENSION NEQ(1), Y(1), YH(NYH,1), EWT(1), RTEM(1),
     1   YDOT(1), SAVR(1), WM(1), IWM(1)
      EXTERNAL  JAC
      COMMON /SDEV2/ ITRACE,IDEV
      COMMON /ALGCHK/ ALCODE, SINGLR
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
      COMMON /PASJB1/ MU,ML, ICALL
C
C     EXTRA COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /JBSAVE/ CON, DI, FAC, HL0, R, R0, SRUR, YI, YJ, YJJ,
     1   I, I1, I2, IER, II, J, J1, JJ, LENP, MBA, MBAND, MEB1,
     2   MEBAND, ML3, N
      SAVE   /JBSAVE/, /ALGCHK/, /SCONS1/, /PASJB1/
C-----------------------------------------------------------------------
C PREPJI IS CALLED BY SPRINT TO COMPUTE AND PROCESS THE JACOBIAN MATRIX
C P = A - H*EL(1)*J.
C HERE P IS COMPUTED BY DIFFERENCING OR BY A USER SUPPLIED ROUTINE.
C P IS STORED IN WM, AND RESCALED.
C P IS THEN SUBJECTED TO LU DECOMPOSITION IN PREPARATION
C FOR LATER SOLUTION OF LINEAR SYSTEMS WITH P AS COEFFICIENT
C MATRIX.  THIS IS DONE BY THE LINPACK ROUTINE DGBFA
C
C IN ADDITION TO VARIABLES DESCRIBED PREVIOUSLY, COMMUNICATION
C WITH PREPJI USES THE FOLLOWING..
C Y     = ARRAY CONTAINING PREDICTED VALUES ON ENTRY.
C RTEM  = WORK ARRAY OF LENGTH N (ACOR IN SPRINT).
C SAVR  = ARRAY USED FOR OUTPUT ONLY.  ON OUTPUT IT CONTAINS THE
C         RESIDUAL EVALUATED AT CURRENT VALUES OF T AND Y.
C YDOT  = ARRAY CONTAINING PREDICTED VALUES OF DY/DT .
C WM    = REAL WORK SPACE FOR MATRICES.  ON OUTPUT IT CONTAINS THE
C         LU DECOMPOSITION OF P.
C IWM   = INTEGER WORK SPACE CONTAINING PIVOT INFORMATION.
C  H    = CURRENT TIMESTEP .
C  TN   = CURRENT TIME LEVEL.
C EL0   = EL(1) (INPUT).
C IFJ   = OUTPUT ERROR FLAG
C         < 0  IF THE P MATRIX IS SINGULAR
C         = 0  IF THIS ROUTINE HAS SUCCESSFULLY FORMED THE JACOBIAN
C         > 0  IF THE EXIT FROM THIS ROUTINE IS A REVERSE COMMUNICATION
C              EXIT FOR A FUNCTION CALL TO RESID.
C IFUNC   INDICATOR TO DETERMINE MODE OF OPERATION OF THE ROUTINE.
C         = 1 EVALUATE THE JACOBIAN MATRIX NORMALLY
C         = 0 REVERSE COMMUNICATION ENTRY
C
C THIS ROUTINE ALSO USES THE COMMON VARIABLE UROUND.
C-----------------------------------------------------------------------
      N = NEQ(1)
C
      IF(IFJ.GT.0)GOTO 10
C
C  CHECK FOR ERRORS IN THE BANDWIDTHS
C
      IF(ALCODE .NE. 'LPBAND')THEN
         CALL SERROR(' PREPJB-ERROR SETUP MODULE MATSET WAS NOT CALLED
     1   PRIOR TO SPRINT BEING ENTERED', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         IFJ = -2
         RETURN
      END IF
      IF(ML.LT.0.OR.ML.GT.N) IFJ = -1
      IF(MU.LT.0.OR.MU.GT.N) IFJ = -1
      IF(IFJ .EQ.-1)THEN
        CALL SERROR(' PREPJB- BANDWIDTHS MU (= I1) AND ML (=I2) ARE NOT
     1  IN THE RANGE 0 TO NEQ', 1, 2, MU, ML, 0, 0.0D0, 0.0D0)
        RETURN
      END IF
C
C  INITIALISATION OF COMMON VARIABLES FOR THIS ROUTINE
C
      ML3 = ML + 1
      MBAND = ML + MU + 1
      MEBAND = MBAND + ML
      HL0 = H*EL0
      IF(ICALL .EQ. 1) THEN
C        ZERO MATRIX AND CALL ANALYTIC JAC ROUTINE.
         DO 8 I = 1,N
            Y(I)    = YH(I,1)
 8          YDOT(I) = YH(I,2)/H
         J = MEBAND * N
         DO 9 I = 1,J
 9          WM(I) = 0.0D0
         CALL JAC( NEQ, TN, Y, YDOT, H, EL0, ML, MU, WM(ML3), MEBAND)
         CALL BANDSC( WM(ML3), MEBAND, N, ML, MU, IDAE, H, EL0)
         IF(ITRACE .GE. 3)THEN
            WRITE(IDEV,91)
 91         FORMAT(' DUMP OF ANAYTIC JACOBIAN (SCALED BY THE SYSTEM) ')
            MBA = MIN0( MBAND, N)
            DO 95 J = 1,MBA
            DO 95 JJ = J,N,MBAND
               I1 = MAX0(JJ-MU,1)
               I2 = MIN0(JJ+ML,N)
               II = JJ*MEB1 - ML
               DO 95 I = I1,I2
                  JZ = II + I
                  IF(ABS(WM(JZ)) .GE. SRUR)WRITE(IDEV,93)I,JJ,WM(JZ)
 93               FORMAT(' ROW',I3,'  COL',I3,' OF JAC. IS',D11.3)
 95         CONTINUE
          END IF
          GOTO 560
      END IF
      SRUR = DSQRT(UROUND)
C MAKE ML + MU + 2 CALLS TO RES TO APPROXIMATE J. --------
10    MBA = MIN0(MBAND,N)
C         REVERSE COMMUNICATION JUMPS
      IF(IFJ.GT.0)GOTO 535
      MEB1 = MEBAND - 1
      FAC = VNORM (N, SAVR, EWT)
      R0 = 1000.0D0*DABS(H)*UROUND*N*FAC
      IF (R0 .EQ. 0.0D0) R0 = 1.0D0
      IF(ITRACE .GE. 3)WRITE(IDEV,592)
 592  FORMAT(' DUMP OF BANDED JACOBIAN MATRIX ')
      IFJ = 1
 520    J = IFJ
C
        DO 530 I = J,N,MBAND
          YI = YH(I,1)
          R = DMAX1(SRUR*DABS(YI),R0/EWT(I))
C    THE FOLLOWING LINE BY M.BERZINS 3/12/83.REMOVE FOR HINDMARSH CODE.
          R = DMAX1(R,UROUND)
          Y(I)    = YH(I,1)   + R
 530      YDOT(I) = YH(I,2)/H + R/HL0
        RETURN
C
C           REVERSE COMMUNICATION EXIT TO TIME MANAGEMENT SCHEME
C
 535    CONTINUE
        DO 550 JJ = J,N,MBAND
          YDOT(JJ) = YH(JJ,2)/H
          Y(JJ)    = YH(JJ,1)
          YJJ      = Y(JJ)
          R = DMAX1(SRUR*DABS(YJJ),R0/EWT(JJ))
C  THE FOLLOWING LINE BY M. BERZINS 3/12/83. REMOVE FOR HINDMARSH CODE
          R = DMAX1(R, UROUND)
          FAC = -HL0/R
          FAC1 =-1.0D0/R
          I1 = MAX0(JJ-MU,1)
          I2 = MIN0(JJ+ML,N)
          II = JJ*MEB1 - ML
          DO 540 I = I1,I2
            WM(II+I) = (RTEM(I)-SAVR(I))*(FAC*IDAE(I)+FAC1*(1-IDAE(I)))
 540      CONTINUE
 550      CONTINUE
        IF(ITRACE .GE. 3)THEN
        DO 555 JJ = J,N,MBAND
          I1 = MAX0(JJ-MU,1)
          I2 = MIN0(JJ+ML,N)
          II = JJ*MEB1 - ML
          DO 555 I = I1,I2
            JZ = II + I
            IF( ABS(WM(JZ)) .GE. SRUR)WRITE(IDEV,539)I,JJ,WM(JZ)
 539        FORMAT(' ROW',I3,'  COL',I3,' OF JAC.  IS',D11.3)
 555    CONTINUE
        END IF
        IFJ = IFJ + 1
        IF(IFJ.LE.MBA)GOTO 520
 560  IFJ = 0
C
C DO LU DECOMPOSITION OF P. --------------------------------------------
C
      SINGLR = 'NSING1'
      CALL DGBFA (WM, MEBAND, N, ML, MU, IWM, IER)
      IF (IER .NE. 0) THEN
          IFJ = -1
          SINGLR = 'SINGLR'
      END IF
      RETURN
C----------------------- END OF SUBROUTINE PREPJB ----------------------
      END
      SUBROUTINE SOLSB (WM, IWM, X, N, IER)
      INTEGER IWM(1), N, IER
      INTEGER MEBAND, ML, MU, ICALL
      DOUBLE PRECISION WM(1), X(1)
      CHARACTER*6 ALCODE, SINGLR
      COMMON /ALGCHK/ ALCODE, SINGLR
      COMMON /PASJB1/ MU, ML, ICALL
      SAVE   /PASJB1/, /ALGCHK/
C-----------------------------------------------------------------------
C THIS ROUTINE MANAGES THE SOLUTION OF THE BANDED LINEAR SYSTEM ARISING
C FROM A CHORD ITERATION . IT CALLS TH LINPACK ROUTINE DGBSL TO PERFORM
C THE BACK SUBSTITUTION .
C COMMUNICATION WITH SOLSY USES THE FOLLOWING VARIABLES..
C WM    = REAL WORK SPACE CONTAINING LU DECOMPOSITION OF THE MATRIX
C         STORAGE OF MATRIX ELEMENTS STARTS AT WM(3).
C         WM ALSO CONTAINS THE FOLLOWING MATRIX-RELATED DATA..
C         WM(1) = SQRT(UROUND) (NOT USED HERE),
C         WM(2) = HL0, THE PREVIOUS VALUE OF H*EL0, USED IF MITER = 3.
C IWM   = INTEGER WORK SPACE CONTAINING PIVOT INFORMATION, STARTING AT
C         IWM(21), IWM ALSO CONTAINS BAND PARAMETERS
C         ML = IWM(1) AND MU = IWM(2) .
C X     = THE RIGHT-HAND SIDE VECTOR ON INPUT, AND THE SOLUTION VECTOR
C         ON OUTPUT, OF LENGTH N.
C IER   = OUTPUT FLAG .  IER = 0 IF NO TROUBLE OCCURRED.
C                        IER = -1 OTHERWISE.
C-----------------------------------------------------------------------
      IER = 0
      IF(ALCODE .NE. 'LPBAND')THEN
        CALL SERROR(' SOLSB - ERROR ROUTINE MATSET WAS NOT CALLED PRIOR
     1  TO SPRINT ENTRY', 1, 0, 0,0, 0, 0.0D0, 0.0D0)
        IER = -2
        RETURN
      ELSE IF(SINGLR .NE. 'NSING1')THEN
        CALL SERROR(' SOLSB - ERROR- ROUTINE SOLSB CALLED TO BACK SUB
     1                WITH SINGULAR JACOBIAN OR BEFORE JACOBIAN HAS BEEN
     2                FORMED OR WHEN THE BANDED JACOBIAN ROUTINE PREPJB
     3                WAS NOT SUPPLIED TO SPRINT', 1, 0, 0, 0, 0, 0.0D0,
     4                0.0D0 )
        IER = -2
        RETURN
      END IF
      MEBAND = 2*ML + MU + 1
      CALL DGBSL (WM, MEBAND, N, ML, MU, IWM, X, 0)
      RETURN
C----------------------- END OF SUBROUTINE SOLSB -----------------------
      END
      SUBROUTINE BANDSC( ABD, LDA, N, ML, MU, IDAE, H, EL0)
C***********************************************************************
C     ROUTINE TO SCALE THE ROWS OF THE MATRIX IN ABD BY 1/(H*EL0)
C     WHERE H* EL0 IS A FACTOR
C***********************************************************************
      INTEGER LDA, N, ML, MU, IDAE(1), I1, I2, I, J, K, M
      DOUBLE PRECISION ABD(LDA,1), H, EL0, EL0H
      EL0H = 1.0D0 / (EL0*H)
      M = ML + MU + 1
      DO 20 J = 1, N
         I1 = MAX0(1, J-MU)
         I2 = MIN0(N, J+ML)
         DO 10 I = I1,I2
            K = I-J+M
 10         ABD(K,J) = -ABD(K,J) * (IDAE(I) + (1-IDAE(I))*EL0H )
 20   CONTINUE
      RETURN
      END
      SUBROUTINE PREPJF (NEQ, Y, YH, NYH, EWT, RTEM, SAVR, YDOT, WM,
     1   IWM, IFJ, H, EL0, TN, IFUNC, JAC, IDAE)
C***********************************************************************
      INTEGER NEQ, NYH, IWM, IFJ, IFUNC, IDAE(1)
      INTEGER I, IER, II, J, J1, JJ, N, MU, ML, ICALL
      INTEGER IOVFLO, ITRACE, IDEV
      DOUBLE PRECISION Y, YH, EWT, RTEM, SAVR, YDOT, WM, H, EL0, TN
      DOUBLE PRECISION DUNFLO, UROUND
      DOUBLE PRECISION CON, DI, FAC, HL0, R, R0, SRUR, YI, YJ, YJJ,
     1   VNORM, FAC1
      CHARACTER*6 ALCODE, SINGLR
      DIMENSION NEQ(1), Y(1), YH(NYH,1), EWT(1), RTEM(1),
     1   YDOT(1), SAVR(1), WM(1), IWM(1)
      EXTERNAL JAC
      COMMON /ALGCHK/ ALCODE, SINGLR
      COMMON /PASJB1/ MU, ML, ICALL
      COMMON /SDEV2/  ITRACE,IDEV
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
C
C     EXTRA COMMON BLOCK FOR REVERSE COMMUNICATION
C
      COMMON /JFSAVE/ CON, DI, FAC, HL0, R, R0, SRUR, YI, YJ, YJJ,
     1   I, IER, II, J, J1, JJ, N
      SAVE   /JFSAVE/, /SCONS1/, /ALGCHK/, /PASJB1/
C-----------------------------------------------------------------------
C PREPJI IS CALLED BY SPRINT TO COMPUTE AND PROCESS THE MATRIX
C P = A - H*EL(1)*J , WHERE J IS AN APPROXIMATION TO THE JACOBIAN DR/DY,
C WHERE R = G(T,Y) - A(T,Y)*S. HERE J IS COMPUTED BY DIFFERENCING.
C J IS STORED IN WM, AND RESCALED.
C    P IS THEN SUBJECTED TO LU DECOMPOSITION IN PREPARATION
C FOR LATER SOLUTION OF LINEAR SYSTEMS WITH P AS COEFFICIENT
C MATRIX.  THIS IS DONE BY THE LINPACK ROUTINE DGEFA
C
C IN ADDITION TO VARIABLES DESCRIBED PREVIOUSLY, COMMUNICATION
C WITH PREPJI USES THE FOLLOWING..
C Y     = ARRAY CONTAINING PREDICTED VALUES ON ENTRY.
C RTEM  = WORK ARRAY OF LENGTH N (ACOR IN SPRINT).
C SAVR  = ARRAY USED FOR OUTPUT ONLY.  ON OUTPUT IT CONTAINS THE
C         RESIDUAL EVALUATED AT CURRENT VALUES OF T AND Y.
C YDOT  = ARRAY CONTAINING PREDICTED VALUES OF DY/DT .
C WM    = REAL WORK SPACE FOR MATRICES.  ON OUTPUT IT CONTAINS THE
C         LU DECOMPOSITION OF P.
C IWM   = INTEGER WORK SPACE CONTAINING PIVOT INFORMATION.
C  H    = CURRENT TIMESTEP .
C  TN   = CURRENT TIME LEVEL.
C EL0   = EL(1) (INPUT).
C IFJ   = OUTPUT ERROR FLAG
C         < 0  IF THE P MATRIX IS SINGULAR
C         = 0  IF THIS ROUTINE HAS SUCCESSFULLY FORMED THE JACOBIAN
C         > 0  IF THE EXIT FROM THIS ROUTINE IS A REVERSE COMMUNICATION
C              EXIT FOR A FUNCTION CALL TO RESID.
C IFUNC   INDICATOR TO DETERMINE MODE OF OPERATION OF THE ROUTINE.
C         = 1 EVALUATE THE JACOBIAN MATRIX NORMALLY
C         = 0 REVERSE COMMUNICATION EXIT AND ENTRY.
C THIS ROUTINE ALSO USES THE COMMON VARIABLE UROUND.
C-----------------------------------------------------------------------
      N = NEQ(1)
      IF(IFJ.GT.0)GOTO 535
      IF(ALCODE .NE. 'LPFULL')THEN
        CALL SERROR(' PREPJF ERROR- SETUP MODULE MATSET WAS NOT CALLED
     1  PRIOR TO THE SPRINT ENTRY', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
        IFJ = -2
        RETURN
      END IF
C
C  INITIALISATION OF COMMON VARIABLES FOR THIS ROUTINE
C
      HL0 = H*EL0
      SRUR = DSQRT(UROUND)
      IF(ICALL .EQ. 1) THEN
C        ZERO THE MATRIX AND CALL THE ANALYTIC JAC ROUTINE
         DO 8 I = 1,N
           Y(I)    = YH(I,1)
 8         YDOT(I) = YH(I,2)/H
         J = N * N
         DO 9 I = 1,J
 9          WM(I) = 0.0D0
         CALL JAC( NEQ, TN, Y, YDOT, H, EL0, ML, MU, WM, N)
         CALL FULLSC( WM, N, N, IDAE, H, EL0)
C
         IF(ITRACE .GE. 3)THEN
            WRITE(IDEV,92)
 92         FORMAT(' DUMP OF ANALYTIC JACOBIAN (SCALED BY THE SYSTEM)')
            DO 94 J = 1,N
            DO 94 I = 1,N
               J1 = (J-1) * N + I
               R = DABS(WM(J1))
               IF(R .GT. UROUND)WRITE(IDEV,93)I,J,WM(J1)
 93            FORMAT(' ROW ',I4,' COL',I4,' OF JAC = ',D11.3)
 94         CONTINUE
         END IF
         GOTO 550
      END IF
C MAKE  N  CALLS TO RES TO APPROXIMATE J. --------
      FAC = VNORM (N, SAVR, EWT)
      R0 = 1000.0D0*DABS(H)*UROUND*N*FAC
      IF (R0 .EQ. 0.0D0) R0 = 1.0D0
      II = 0
      IF(ITRACE .GE. 3)WRITE(IDEV,592)
 592  FORMAT(' DUMP OF FULL JACOBIAN MATRIX (SCALED BY THE SYSTEM)')
      IFJ = 1
 520    I = IFJ
        YI = Y(I)
        R = DMAX1(SRUR*DABS(YI),R0/EWT(I))
C   THE FOLLOWING LINE BY M.BERZINS 2/12/83. REMOVE FOR HINDMARSH CODE.
        R = DMAX1(R,UROUND)
        Y(I)    = YH(I,1)   + R
        YDOT(I) = YH(I,2)/H + R/HL0
        RETURN
C
C           REVERSE COMMUNICATION EXIT TO TIME MANAGEMENT SCHEME
C
 535    JJ = IFJ
        YDOT(JJ) = YH(JJ,2)/H
        Y(JJ)    = YH(JJ,1)
        YJJ      = Y(JJ)
        R = DMAX1(SRUR*DABS(YJJ),R0/EWT(JJ))
C  THE FOLLOWING LINE BY M.BERZINS 3/12/83. REMOVE FOR HINDMARSH CODE.
        R = DMAX1( R, UROUND)
        FAC = -HL0/R
        FAC1= -1.0D0/R
        DO 540 I = 1,N
           WM(II+I) = (RTEM(I)-SAVR(I))*(FAC*IDAE(I)+FAC1*(1-IDAE(I)))
 540    CONTINUE
        IF(ITRACE .GE. 3)THEN
        DO 545 I = 1,N
           J1 = II + I
           R = DABS(WM(J1))
           IF(R .GT. UROUND)WRITE(IDEV,539)I,IFJ,WM(J1)
 539       FORMAT(' ROW ',I4,' COL',I4,' OF JAC = ',D11.3)
 545    CONTINUE
        END IF
        II = II + N
        IFJ = IFJ + 1
        IF(IFJ .LE. N)GOTO 520
 550  IFJ = 0
C
C DO LU DECOMPOSITION OF P. --------------------------------------------
C
      SINGLR = 'NSING2'
      CALL DGEFA (WM, N, N, IWM, IER)
      IF (IER .NE. 0) THEN
          IFJ = -1
          IF(ITRACE .GE. 1)WRITE(IDEV,999)IER
 999      FORMAT(' PIVOT ON ROW AND COL ',I5,' IS ZERO ')
          SINGLR = 'SINGLR'
      END IF
      RETURN
C----------------------- END OF SUBROUTINE PREPJB ----------------------
      END
      SUBROUTINE SOLSF (WM, IWM, X, N, IER)
      INTEGER IWM(1), N, IER
      DOUBLE PRECISION WM(1), X(1)
      CHARACTER*6 ALCODE, SINGLR
      COMMON /ALGCHK/ ALCODE, SINGLR
      SAVE   /ALGCHK/
C-----------------------------------------------------------------------
C THIS ROUTINE MANAGES THE SOLUTION OF THE BANDED LINEAR SYSTEM ARISING
C FROM A CHORD ITERATION . IT CALLS THE LINPACK ROUTINE DGESL TO PERFORM
C THE BACK SUBSTITUTION .
C COMMUNICATION WITH SOLSY USES THE FOLLOWING VARIABLES..
C WM    = REAL WORK SPACE CONTAINING LU DECOMPOSITION OF THE MATRIX
C         STORAGE OF MATRIX ELEMENTS STARTS AT WM(3).
C IWM   = INTEGER WORK SPACE CONTAINING PIVOT INFORMATION, STARTING AT
C         IWM(1).
C X     = THE RIGHT-HAND SIDE VECTOR ON INPUT, AND THE SOLUTION VECTOR
C         ON OUTPUT, OF LENGTH N.
C IER   = OUTPUT FLAG .  IER = 0 IF NO TROUBLE OCCURRED.
C                        IER = -1 OTHERWISE.
C-----------------------------------------------------------------------
      IF(ALCODE .NE. 'LPFULL')THEN
         CALL SERROR(' SOLSF - ERROR MATSET SETUP ROUTINE WAS NOT CALLED
     1   PRIOR TO THE SPRINT ENTRY', 1, 0, 0, 0, 0, 0.0D0, 0.0D0)
         IER = -2
         RETURN
      ELSE IF( SINGLR .NE. 'NSING2')THEN
         CALL SERROR(' SOLSF - ERROR-BACK SUBSTITUTION TRIED WITH EITHER
     1                 SINGULAR MATRIX OR WHEN THE JACOBIAN HAS NOT BEEN
     2                 FORMED OR WHEN SPRINT WAS NOT CALLED WITH THE
     3                 JACOBIAN FORMING ROUTINE PREPJF', 1, 0, 0, 0, 0,
     4                 0.0D0, 0.0D0)
         IER = -2
         RETURN
      END IF
      IER = 0
      CALL DGESL (WM, N, N, IWM, X, 0)
      RETURN
C----------------------- END OF SUBROUTINE SOLSB -----------------------
      END
      SUBROUTINE FULLSC( A, LDA, N, IDAE, H, EL0)
C***********************************************************************
C     ROUTINE TO SCALE THE FULL MATRIX A(LDA,N) BY THE FACTOR
C     (H*EL0)**(-1) WHERE POSSIBLE
C***********************************************************************
      INTEGER LDA, N, IDAE(1), I, J
      DOUBLE PRECISION A(LDA,1), H, EL0, EL0H
      EL0H = 1.0D0 / (EL0* H)
      DO 100 J = 1,N
        DO 100 I = 1,N
 100       A(I,J) = -A(I,J) * (IDAE(I) + (1-IDAE(I)) * EL0H )
      RETURN
      END
      SUBROUTINE DGEFA(A,LDA,N,IPVT,INFO)
      INTEGER LDA,N,IPVT(1),INFO
      DOUBLE PRECISION A(LDA,1)
C
C     DGEFA FACTORS A DOUBLE PRECISION MATRIX BY GAUSSIAN ELIMINATION.
C
C     DGEFA IS USUALLY CALLED BY DGECO, BUT IT CAN BE CALLED
C     DIRECTLY WITH A SAVING IN TIME IF  RCOND  IS NOT NEEDED.
C     (TIME FOR DGECO) = (1 + 9/N)*(TIME FOR DGEFA) .
C
C     ON ENTRY
C
C        A       DOUBLE PRECISION(LDA, N)
C                THE MATRIX TO BE FACTORED.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  A .
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C     ON RETURN
C
C        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS
C                WHICH WERE USED TO OBTAIN IT.
C                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
C                L  IS A PRODUCT OF PERMUTATION AND UNIT LOWER
C                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N)
C                AN INTEGER VECTOR OF PIVOT INDICES.
C
C        INFO    INTEGER
C                = 0  NORMAL VALUE.
C                = K  IF  U(K,K) .EQ. 0.0 .  THIS IS NOT AN ERROR
C                     CONDITION FOR THIS SUBROUTINE, BUT IT DOES
C                     INDICATE THAT DGESL OR DGEDI WILL DIVIDE BY ZERO
C                     IF CALLED.  USE  RCOND  IN DGECO FOR A RELIABLE
C                     INDICATION OF SINGULARITY.
C
C     LINPACK. THIS VERSION DATED 08/14/78 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
C
C     SUBROUTINES AND FUNCTIONS
C
C     BLAS DAXPY,DSCAL,IDAMAX
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION T
      INTEGER IDAMAX,J,K,KP1,L,NM1
C
C
C     GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING
C
      INFO = 0
      NM1 = N - 1
      IF (NM1 .LT. 1) GO TO 70
      DO 60 K = 1, NM1
         KP1 = K + 1
C
C        FIND L = PIVOT INDEX
C
         L = IDAMAX(N-K+1,A(K,K),1) + K - 1
         IPVT(K) = L
C
C        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED
C
         IF (A(L,K) .EQ. 0.0D0) GO TO 40
C
C           INTERCHANGE IF NECESSARY
C
            IF (L .EQ. K) GO TO 10
               T = A(L,K)
               A(L,K) = A(K,K)
               A(K,K) = T
   10       CONTINUE
C
C           COMPUTE MULTIPLIERS
C
            T = -1.0D0/A(K,K)
            CALL DSCAL(N-K,T,A(K+1,K),1)
C
C           ROW ELIMINATION WITH COLUMN INDEXING
C
            DO 30 J = KP1, N
               T = A(L,J)
               IF (L .EQ. K) GO TO 20
                  A(L,J) = A(K,J)
                  A(K,J) = T
   20          CONTINUE
               CALL DAXPY(N-K,T,A(K+1,K),1,A(K+1,J),1)
   30       CONTINUE
         GO TO 50
   40    CONTINUE
            INFO = K
   50    CONTINUE
   60 CONTINUE
   70 CONTINUE
      IPVT(N) = N
      IF (A(N,N) .EQ. 0.0D0) INFO = N
      RETURN
      END
      SUBROUTINE DGESL(A,LDA,N,IPVT,B,JOB)
      INTEGER LDA,N,IPVT(1),JOB
      DOUBLE PRECISION A(LDA,1),B(1)
C
C     DGESL SOLVES THE DOUBLE PRECISION SYSTEM
C     A * X = B  OR  TRANS(A) * X = B
C     USING THE FACTORS COMPUTED BY DGECO OR DGEFA.
C
C     ON ENTRY
C
C        A       DOUBLE PRECISION(LDA, N)
C                THE OUTPUT FROM DGECO OR DGEFA.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  A .
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C        IPVT    INTEGER(N)
C                THE PIVOT VECTOR FROM DGECO OR DGEFA.
C
C        B       DOUBLE PRECISION(N)
C                THE RIGHT HAND SIDE VECTOR.
C
C        JOB     INTEGER
C                = 0         TO SOLVE  A*X = B ,
C                = NONZERO   TO SOLVE  TRANS(A)*X = B  WHERE
C                            TRANS(A)  IS THE TRANSPOSE.
C
C     ON RETURN
C
C        B       THE SOLUTION VECTOR  X .
C
C     ERROR CONDITION
C
C        A DIVISION BY ZERO WILL OCCUR IF THE INPUT FACTOR CONTAINS A
C        ZERO ON THE DIAGONAL.  TECHNICALLY THIS INDICATES SINGULARITY
C        BUT IT IS OFTEN CAUSED BY IMPROPER ARGUMENTS OR IMPROPER
C        SETTING OF LDA .  IT WILL NOT OCCUR IF THE SUBROUTINES ARE
C        CALLED CORRECTLY AND IF DGECO HAS SET RCOND .GT. 0.0
C        OR DGEFA HAS SET INFO .EQ. 0 .
C
C     TO COMPUTE  INVERSE(A) * C  WHERE  C  IS A MATRIX
C     WITH  P  COLUMNS
C           CALL DGECO(A,LDA,N,IPVT,RCOND,Z)
C           IF (RCOND IS TOO SMALL) GO TO ...
C           DO 10 J = 1, P
C              CALL DGESL(A,LDA,N,IPVT,C(1,J),0)
C        10 CONTINUE
C
C     LINPACK. THIS VERSION DATED 08/14/78 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
C
C     SUBROUTINES AND FUNCTIONS
C
C     BLAS DAXPY,DDOT
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION DDOT,T
      INTEGER K,KB,L,NM1
C
      NM1 = N - 1
      IF (JOB .NE. 0) GO TO 50
C
C        JOB = 0 , SOLVE  A * X = B
C        FIRST SOLVE  L*Y = B
C
         IF (NM1 .LT. 1) GO TO 30
         DO 20 K = 1, NM1
            L = IPVT(K)
            T = B(L)
            IF (L .EQ. K) GO TO 10
               B(L) = B(K)
               B(K) = T
   10       CONTINUE
            CALL DAXPY(N-K,T,A(K+1,K),1,B(K+1),1)
   20    CONTINUE
   30    CONTINUE
C
C        NOW SOLVE  U*X = Y
C
         DO 40 KB = 1, N
            K = N + 1 - KB
            B(K) = B(K)/A(K,K)
            T = -B(K)
            CALL DAXPY(K-1,T,A(1,K),1,B(1),1)
   40    CONTINUE
      GO TO 100
   50 CONTINUE
C
C        JOB = NONZERO, SOLVE  TRANS(A) * X = B
C        FIRST SOLVE  TRANS(U)*Y = B
C
         DO 60 K = 1, N
            T = DDOT(K-1,A(1,K),1,B(1),1)
            B(K) = (B(K) - T)/A(K,K)
   60    CONTINUE
C
C        NOW SOLVE TRANS(L)*X = Y
C
         IF (NM1 .LT. 1) GO TO 90
         DO 80 KB = 1, NM1
            K = N - KB
            B(K) = B(K) + DDOT(N-K,A(K+1,K),1,B(K+1),1)
            L = IPVT(K)
            IF (L .EQ. K) GO TO 70
               T = B(L)
               B(L) = B(K)
               B(K) = T
   70       CONTINUE
   80    CONTINUE
   90    CONTINUE
  100 CONTINUE
      RETURN
      END
      SUBROUTINE DGBFA(ABD,LDA,N,ML,MU,IPVT,INFO)
      INTEGER LDA,N,ML,MU,IPVT(1),INFO
      DOUBLE PRECISION ABD(LDA,1)
C
C     DGBFA FACTORS A DOUBLE PRECISION BAND MATRIX BY ELIMINATION.
C
C     DGBFA IS USUALLY CALLED BY DGBCO, BUT IT CAN BE CALLED
C     DIRECTLY WITH A SAVING IN TIME IF  RCOND  IS NOT NEEDED.
C
C     ON ENTRY
C
C        ABD     DOUBLE PRECISION(LDA, N)
C                CONTAINS THE MATRIX IN BAND STORAGE.  THE COLUMNS
C                OF THE MATRIX ARE STORED IN THE COLUMNS OF  ABD  AND
C                THE DIAGONALS OF THE MATRIX ARE STORED IN ROWS
C                ML+1 THROUGH 2*ML+MU+1 OF  ABD .
C                SEE THE COMMENTS BELOW FOR DETAILS.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  ABD .
C                LDA MUST BE .GE. 2*ML + MU + 1 .
C
C        N       INTEGER
C                THE ORDER OF THE ORIGINAL MATRIX.
C
C        ML      INTEGER
C                NUMBER OF DIAGONALS BELOW THE MAIN DIAGONAL.
C                0 .LE. ML .LT. N .
C
C        MU      INTEGER
C                NUMBER OF DIAGONALS ABOVE THE MAIN DIAGONAL.
C                0 .LE. MU .LT. N .
C                MORE EFFICIENT IF  ML .LE. MU .
C     ON RETURN
C
C        ABD     AN UPPER TRIANGULAR MATRIX IN BAND STORAGE AND
C                THE MULTIPLIERS WHICH WERE USED TO OBTAIN IT.
C                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
C                L  IS A PRODUCT OF PERMUTATION AND UNIT LOWER
C                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N)
C                AN INTEGER VECTOR OF PIVOT INDICES.
C
C        INFO    INTEGER
C                = 0  NORMAL VALUE.
C                = K  IF  U(K,K) .EQ. 0.0 .  THIS IS NOT AN ERROR
C                     CONDITION FOR THIS SUBROUTINE, BUT IT DOES
C                     INDICATE THAT DGBSL WILL DIVIDE BY ZERO IF
C                     CALLED.  USE  RCOND  IN DGBCO FOR A RELIABLE
C                     INDICATION OF SINGULARITY.
C
C     BAND STORAGE
C
C           IF  A  IS A BAND MATRIX, THE FOLLOWING PROGRAM SEGMENT
C           WILL SET UP THE INPUT.
C
C                   ML = (BAND WIDTH BELOW THE DIAGONAL)
C                   MU = (BAND WIDTH ABOVE THE DIAGONAL)
C                   M = ML + MU + 1
C                   DO 20 J = 1, N
C                      I1 = MAX0(1, J-MU)
C                      I2 = MIN0(N, J+ML)
C                      DO 10 I = I1, I2
C                         K = I - J + M
C                         ABD(K,J) = A(I,J)
C                10    CONTINUE
C                20 CONTINUE
C
C           THIS USES ROWS  ML+1  THROUGH  2*ML+MU+1  OF  ABD .
C           IN ADDITION, THE FIRST  ML  ROWS IN  ABD  ARE USED FOR
C           ELEMENTS GENERATED DURING THE TRIANGULARIZATION.
C           THE TOTAL NUMBER OF ROWS NEEDED IN  ABD  IS  2*ML+MU+1 .
C           THE  ML+MU BY ML+MU  UPPER LEFT TRIANGLE AND THE
C           ML BY ML  LOWER RIGHT TRIANGLE ARE NOT REFERENCED.
C
C     LINPACK. THIS VERSION DATED 08/14/78 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
C
C     SUBROUTINES AND FUNCTIONS
C
C     BLAS DAXPY,DSCAL,IDAMAX
C     FORTRAN MAX0,MIN0
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION T
      INTEGER I,IDAMAX,I0,J,JU,JZ,J0,J1,K,KP1,L,LM,M,MM,NM1
C
C
      M = ML + MU + 1
      INFO = 0
C
C     ZERO INITIAL FILL-IN COLUMNS
C
      J0 = MU + 2
      J1 = MIN0(N,M) - 1
      IF (J1 .LT. J0) GO TO 30
      DO 20 JZ = J0, J1
         I0 = M + 1 - JZ
         DO 10 I = I0, ML
            ABD(I,JZ) = 0.0D0
   10    CONTINUE
   20 CONTINUE
   30 CONTINUE
      JZ = J1
      JU = 0
C
C     GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING
C
      NM1 = N - 1
      IF (NM1 .LT. 1) GO TO 130
      DO 120 K = 1, NM1
         KP1 = K + 1
C
C        ZERO NEXT FILL-IN COLUMN
C
         JZ = JZ + 1
         IF (JZ .GT. N) GO TO 50
         IF (ML .LT. 1) GO TO 50
            DO 40 I = 1, ML
               ABD(I,JZ) = 0.0D0
   40       CONTINUE
   50    CONTINUE
C
C        FIND L = PIVOT INDEX
C
         LM = MIN0(ML,N-K)
         L = IDAMAX(LM+1,ABD(M,K),1) + M - 1
         IPVT(K) = L + K - M
C
C        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED
C
         IF (ABD(L,K) .EQ. 0.0D0) GO TO 100
C
C           INTERCHANGE IF NECESSARY
C
            IF (L .EQ. M) GO TO 60
               T = ABD(L,K)
               ABD(L,K) = ABD(M,K)
               ABD(M,K) = T
   60       CONTINUE
C
C           COMPUTE MULTIPLIERS
C
            T = -1.0D0/ABD(M,K)
            CALL DSCAL(LM,T,ABD(M+1,K),1)
C
C           ROW ELIMINATION WITH COLUMN INDEXING
C
            JU = MIN0(MAX0(JU,MU+IPVT(K)),N)
            MM = M
            IF (JU .LT. KP1) GO TO 90
            DO 80 J = KP1, JU
               L = L - 1
               MM = MM - 1
               T = ABD(L,J)
               IF (L .EQ. MM) GO TO 70
                  ABD(L,J) = ABD(MM,J)
                  ABD(MM,J) = T
   70          CONTINUE
               CALL DAXPY(LM,T,ABD(M+1,K),1,ABD(MM+1,J),1)
   80       CONTINUE
   90       CONTINUE
         GO TO 110
  100    CONTINUE
            INFO = K
  110    CONTINUE
  120 CONTINUE
  130 CONTINUE
      IPVT(N) = N
      IF (ABD(M,N) .EQ. 0.0D0) INFO = N
      RETURN
      END
      SUBROUTINE DGBSL(ABD,LDA,N,ML,MU,IPVT,B,JOB)
      INTEGER LDA,N,ML,MU,IPVT(1),JOB
      DOUBLE PRECISION ABD(LDA,1),B(1)
C
C     DGBSL SOLVES THE DOUBLE PRECISION BAND SYSTEM
C     A * X = B  OR  TRANS(A) * X = B
C     USING THE FACTORS COMPUTED BY DGBCO OR DGBFA.
C
C     ON ENTRY
C
C        ABD     DOUBLE PRECISION(LDA, N)
C                THE OUTPUT FROM DGBCO OR DGBFA.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  ABD .
C
C        N       INTEGER
C                THE ORDER OF THE ORIGINAL MATRIX.
C
C        ML      INTEGER
C                NUMBER OF DIAGONALS BELOW THE MAIN DIAGONAL.
C
C        MU      INTEGER
C                NUMBER OF DIAGONALS ABOVE THE MAIN DIAGONAL.
C
C        IPVT    INTEGER(N)
C                THE PIVOT VECTOR FROM DGBCO OR DGBFA.
C
C        B       DOUBLE PRECISION(N)
C                THE RIGHT HAND SIDE VECTOR.
C
C        JOB     INTEGER
C                = 0         TO SOLVE  A*X = B ,
C                = NONZERO   TO SOLVE  TRANS(A)*X = B , WHERE
C                            TRANS(A)  IS THE TRANSPOSE.
C
C     ON RETURN
C
C        B       THE SOLUTION VECTOR  X .
C
C     ERROR CONDITION
C
C        A DIVISION BY ZERO WILL OCCUR IF THE INPUT FACTOR CONTAINS A
C        ZERO ON THE DIAGONAL.  TECHNICALLY THIS INDICATES SINGULARITY
C        BUT IT IS OFTEN CAUSED BY IMPROPER ARGUMENTS OR IMPROPER
C        SETTING OF LDA .  IT WILL NOT OCCUR IF THE SUBROUTINES ARE
C        CALLED CORRECTLY AND IF DGBCO HAS SET RCOND .GT. 0.0
C        OR DGBFA HAS SET INFO .EQ. 0 .
C
C     TO COMPUTE  INVERSE(A) * C  WHERE  C  IS A MATRIX
C     WITH  P  COLUMNS
C           CALL DGBCO(ABD,LDA,N,ML,MU,IPVT,RCOND,Z)
C           IF (RCOND IS TOO SMALL) GO TO ...
C           DO 10 J = 1, P
C              CALL DGBSL(ABD,LDA,N,ML,MU,IPVT,C(1,J),0)
C        10 CONTINUE
C
C     LINPACK. THIS VERSION DATED 08/14/78 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
C
C     SUBROUTINES AND FUNCTIONS
C
C     BLAS DAXPY,DDOT
C     FORTRAN MIN0
C
C     INTERNAL VARIABLES
C
      DOUBLE PRECISION DDOT,T
      INTEGER K,KB,L,LA,LB,LM,M,NM1
C
      M = MU + ML + 1
      NM1 = N - 1
      IF (JOB .NE. 0) GO TO 50
C
C        JOB = 0 , SOLVE  A * X = B
C        FIRST SOLVE L*Y = B
C
         IF (ML .EQ. 0) GO TO 30
         IF (NM1 .LT. 1) GO TO 30
            DO 20 K = 1, NM1
               LM = MIN0(ML,N-K)
               L = IPVT(K)
               T = B(L)
               IF (L .EQ. K) GO TO 10
                  B(L) = B(K)
                  B(K) = T
   10          CONTINUE
               CALL DAXPY(LM,T,ABD(M+1,K),1,B(K+1),1)
   20       CONTINUE
   30    CONTINUE
C
C        NOW SOLVE  U*X = Y
C
         DO 40 KB = 1, N
            K = N + 1 - KB
            B(K) = B(K)/ABD(M,K)
            LM = MIN0(K,M) - 1
            LA = M - LM
            LB = K - LM
            T = -B(K)
            CALL DAXPY(LM,T,ABD(LA,K),1,B(LB),1)
   40    CONTINUE
      GO TO 100
   50 CONTINUE
C
C        JOB = NONZERO, SOLVE  TRANS(A) * X = B
C        FIRST SOLVE  TRANS(U)*Y = B
C
         DO 60 K = 1, N
            LM = MIN0(K,M) - 1
            LA = M - LM
            LB = K - LM
            T = DDOT(LM,ABD(LA,K),1,B(LB),1)
            B(K) = (B(K) - T)/ABD(M,K)
   60    CONTINUE
C
C        NOW SOLVE TRANS(L)*X = Y
C
         IF (ML .EQ. 0) GO TO 90
         IF (NM1 .LT. 1) GO TO 90
            DO 80 KB = 1, NM1
               K = N - KB
               LM = MIN0(ML,N-K)
               B(K) = B(K) + DDOT(LM,ABD(M+1,K),1,B(K+1),1)
               L = IPVT(K)
               IF (L .EQ. K) GO TO 70
                  T = B(L)
                  B(L) = B(K)
                  B(K) = T
   70          CONTINUE
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
      RETURN
      END
      SUBROUTINE DAXPY(N,DA,DX,INCX,DY,INCY)
C
C     CONSTANT TIMES A VECTOR PLUS A VECTOR.
C     USES UNROLLED LOOPS FOR INCREMENTS EQUAL TO ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DY(1),DA
      INTEGER I,INCX,INCY,IX,IY,M,MP1,N
C
      IF(N.LE.0)RETURN
      IF (DA .EQ. 0.0D0) RETURN
      IF(INCX.EQ.1.AND.INCY.EQ.1)GO TO 20
C
C        CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS
C          NOT EQUAL TO 1
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DY(IY) = DY(IY) + DA*DX(IX)
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      RETURN
C
C        CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   20 M = MOD(N,4)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DY(I) = DY(I) + DA*DX(I)
   30 CONTINUE
      IF( N .LT. 4 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,4
        DY(I) = DY(I) + DA*DX(I)
        DY(I + 1) = DY(I + 1) + DA*DX(I + 1)
        DY(I + 2) = DY(I + 2) + DA*DX(I + 2)
        DY(I + 3) = DY(I + 3) + DA*DX(I + 3)
   50 CONTINUE
      RETURN
      END
      SUBROUTINE DSCAL(N,DA,DX,INCX)
C
C     SCALES A VECTOR BY A CONSTANT.
C     USES UNROLLED LOOPS FOR INCREMENT EQUAL TO ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DA,DX(1)
      INTEGER I,INCX,M,MP1,N,NINCX
C
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1)GO TO 20
C
C        CODE FOR INCREMENT NOT EQUAL TO 1
C
      NINCX = N*INCX
      DO 10 I = 1,NINCX,INCX
        DX(I) = DA*DX(I)
   10 CONTINUE
      RETURN
C
C        CODE FOR INCREMENT EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   20 M = MOD(N,5)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DX(I) = DA*DX(I)
   30 CONTINUE
      IF( N .LT. 5 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,5
        DX(I) = DA*DX(I)
        DX(I + 1) = DA*DX(I + 1)
        DX(I + 2) = DA*DX(I + 2)
        DX(I + 3) = DA*DX(I + 3)
        DX(I + 4) = DA*DX(I + 4)
   50 CONTINUE
      RETURN
      END
      DOUBLE PRECISION FUNCTION DDOT(N,DX,INCX,DY,INCY)
C
C     FORMS THE DOT PRODUCT OF TWO VECTORS.
C     USES UNROLLED LOOPS FOR INCREMENTS EQUAL TO ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DY(1),DTEMP
      INTEGER I,INCX,INCY,IX,IY,M,MP1,N
C
      DDOT = 0.0D0
      DTEMP = 0.0D0
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1.AND.INCY.EQ.1)GO TO 20
C
C        CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS
C          NOT EQUAL TO 1
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DTEMP = DTEMP + DX(IX)*DY(IY)
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      DDOT = DTEMP
      RETURN
C
C        CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   20 M = MOD(N,5)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DTEMP = DTEMP + DX(I)*DY(I)
   30 CONTINUE
      IF( N .LT. 5 ) GO TO 60
   40 MP1 = M + 1
      DO 50 I = MP1,N,5
        DTEMP = DTEMP + DX(I)*DY(I) + DX(I + 1)*DY(I + 1) +
     *   DX(I + 2)*DY(I + 2) + DX(I + 3)*DY(I + 3) + DX(I + 4)*DY(I + 4)
   50 CONTINUE
   60 DDOT = DTEMP
      RETURN
      END
      INTEGER FUNCTION IDAMAX(N,DX,INCX)
C
C     FINDS THE INDEX OF ELEMENT HAVING MAX. ABSOLUTE VALUE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DMAX
      INTEGER I,INCX,IX,N
C
      IDAMAX = 0
      IF( N .LT. 1 ) RETURN
      IDAMAX = 1
      IF(N.EQ.1)RETURN
      IF(INCX.EQ.1)GO TO 20
C
C        CODE FOR INCREMENT NOT EQUAL TO 1
C
      IX = 1
      DMAX = DABS(DX(1))
      IX = IX + INCX
      DO 10 I = 2,N
         IF(DABS(DX(IX)).LE.DMAX) GO TO 5
         IDAMAX = I
         DMAX = DABS(DX(IX))
    5    IX = IX + INCX
   10 CONTINUE
      RETURN
C
C        CODE FOR INCREMENT EQUAL TO 1
C
   20 DMAX = DABS(DX(1))
      DO 30 I = 2,N
         IF(DABS(DX(I)).LE.DMAX) GO TO 30
         IDAMAX = I
         DMAX = DABS(DX(I))
   30 CONTINUE
      RETURN
      END
