C       PARAMETER DESCRIPTION
C      ***********************
C
C  NPDE : THE NUMBER OF PARABOLIC P.D.E.S
C  NPTS : THE NUMBER OF GRID POINTS IN THE SPACE DIRECTION
C  X(NPTS) : THE ARRAY OF GRID POINTS
C  M     : INTEGER PARAMETER SPECIFYING WHETHER THE P.D.E IS IN
C          CARTESIAN (M=0) OR POLAR (M > 0) CO-ORDINATES.
C  U(NPDE,NPTS) THE SOLUTION VALUES AT TIME TS .
C  TS : THE TIME AT WHICH INTEGRATION IS TO BE STARTED OR CONTINUED.
C  TOUT : THE NEXT OUTPUT TIME LEVEL
C  ABS : THE ABSOLUTE LOCAL ERROR TOLERANCE.
C  REL : THE RELATIVE LOCAL ERROR TOLERANCE.
C  LINALG ; INTEGER PARAMETER TO DEFINE THE TYPE OF LINEAR ALGEBRA
C           USED IUN THE SOLUTION OF THE O.D.E. SYSTEM. ALLOWED VALUES
C           ARE :  = 0 YALE SPARSE MATRIX ROUTINES.
C                  = 1 LINPACK BANDED MATRIX ROUTINES.
C                  = 2 LINPACK FULL MATRIX ROUTINES.
C                  = 3 NAG SPARSE MATRIX ROUTINES
C  IODE  =  0       STHETA METHOD
C        =  1       STHETB METHOD
C        =  2       SPGEAR METHOD
C
C**********************************************************************
C     THIS PROBLEM IS DEFINED BY
C               -1
C       U  = ( C   U  )  - V * EXP(-2U) + EXP(-U)  ,  X IN (-1,0)
C        T      1   X  X    1
C  AND
C               -1
C       U  = ( C   U  )  - V * EXP(-2U) + EXP(-U)  ,  X IN (0,1)
C        T      2   X  X    2
C  AND
C               -1
C       U  = ( C   U  )  - V * EXP(-2U) + EXP(-U)  ,  X IN (1,2)
C        T      3   X  X    3
C  WHERE
C       C  = 0.1  ,   C   = 1.0  AND  C   = 2.0
C        1             2               3
C  AND
C       V   =  C    FOR I = 1 , 2, 3
C        I      I
C
C     THE LEFT BOUNDARY CONDITION AT X =-1 (LEFT = .TRUE. ) IS GIVEN BY
C     U(-1,T)  = LOG ( - C  + T + P)
C                         1
C     THE RIGHT BOUNDARY CONDITION IS  (LEFT = .FALSE.)
C     U( 2,T) + C  / (C + C + T + P ) U  = LOG ( C +C  + T + P) + 1.0D0
C                3     3   2           X          2  3
C
C      THE INITIAL CONDITION IS GIVEN BY THE EXACT SOLUTION ;
C        U( X, T )  = LOG ( C X + T + P )  , X IN ( -1, 0)
C                            1
C        U( X, T )  = LOG ( C X + T + P )  , X IN (  0, 1)
C                            2
C        U( X, T )  = LOG(  C   +  C  *(X-1) + T + P) X IN ( 1, 2)
C                            2       3
C***********************************************************************
C    ERROR MESSAGES ARE OUTPUT ON DEVICE IDEV WHERE IDEV IS SET UP
C    BY THE COMMON  STATEMENT  COMMON/SDEV2/ITRACE,IDEV
C    THE INITIALISATION FOR THE DISCRETISATION CODE IS DONE BY CALLING
C    THE SUBROUTINE INITSK - SEE THE SPDIFF MODULE FOR DETAILS.
C
      INTEGER NPTS, NPDE, LIW, IMESH,
     -        IBAND, IWORK(3220),IDEV, ITRACE, I, J, NWKMON,
     -        ITOL, NSTEPS, NEQN, IFUN, IJAC, M, IOVFLO
      INTEGER INFORM(213), NWKDRV, NWKJAC, IRET, N, NOP, IPMINF, MAXNPT,
     1        JSINFO(4), NITER, NEL, ISET, IRES, NRWK, NV, NXI, NFIX,
     2        IA(1), NIA, JA(1), NJA, NJCPVT, ISPLIT, ICALL, IGROW,
     3        LIWREQ, LIWUSD, LRWREQ, LRWUSD, NLU, NBLOCK, NNZ, NGP
      DOUBLE PRECISION  X(81), U(2,81), WORK(900), UROUND, XI,
     -        TS, TOUT, ABS, REL, DUNFLO, WKMON(1), ENORM, GERR, CONST,
     -        WKJAC(10000), YSAVE(161,15), YDOT(161), TTIME, XFIX(2),
     -        DXMESH,STPRAT, DELX, DX, SFNOFC, SENS,UPIV,ETA,RESWK(500)
      LOGICAL REMESH, LBLOCK
      CHARACTER*6 SNORM
      CHARACTER*12 JCEVAL
      EXTERNAL SBLEND,PREPJB, SOLSB, SOLSS, PREPJS, STHETA, STHETB,
     1         RES, JAC, PREPJF, SOLSF, MVMESH, DERIV, SPGEAR, SKLRES,
     2         SPANMM, SPANNV
      COMMON /SDEV2/ ITRACE,IDEV
      COMMON /ST002A/ SFNOFC
      COMMON /SCONS1/ DUNFLO, UROUND, IOVFLO
        NPDE = 1
        WRITE(6,1011)
C       CALL ERRSET ( 208, 256, -1, -1, 0) IBM ONLY TO DISABLE UNDERFLOW
        CALL ERRSET ( 208, 256, -1, -1, 0)
1011    FORMAT(' PLEASE INPUT IODE , LINALG AND IMESH')
        READ(5,*)IODE, LINALG, IMESH
        WRITE(6,1013)IODE, LINALG, IMESH
1013    FORMAT(' IODE = ',I3,' LINALG = ',I3,' IMESH =',I3)
        ITRACE = 1
        CALL TIMER(TTIME,1)
        IDEV   = 6
        WRITE(IDEV,1013)IODE, LINALG, IMESH
        NPTS = 31
C       NPTS = 15
        NEQN = NPDE*NPTS
        M = 0
        NWKMON = 1
        ITOL = 1
C       LINALG = 2
        ABS = 1.0D-4
        REL = 1.0D-4
       IF( IODE .EQ. 1)THEN
           WRITE(6,777)
777        FORMAT(' INPUT THE VALUE OF FN OVER FC, IN RANGE 1 TO 9 SAY')
           READ(5,*)SFNOFC
       END IF
C
C  INITIALISE THE DISCRETISATION WORKSPACE
C
C REMESH INFO (VARIABLE NO. POINTS OPTION THEN SET NOP=0 ELSE NOP=NPTS)
      REMESH = .FALSE.
      NOP= 0
      STPRAT=1.5D0
      IPMINF=0
C NRMESH DICTATES HOW OFTEN REMESH :
C =1 THEN REMESHES EVERY TIME STEP WHEN MVMESH ROUTINE CALLED
C =2                     2ND TIME STEP, ETC....
C =0 THEN REMESHES AT CURRENT TIME TOUT BY THE CALL TO MVMESH WITH
C    INFORM(2)=4 AND WORK(1) SET TO CURRENT TOUT. IF THE SOLUTION
C    PROFILE CHANGES RAPIDLY FOR CERTAIN T THEN SET TOUT ACCORDINGLY.
C
C IF HMAX=WORK(6) IS LARGE THEN NRMESH*HMAX MAY BE TOO LARGE
C FOR REMESHING PURPOSES SO SET ACCORDINGLY
      WORK(6)=0.2D0
      WORK(5)=0.0D0
      WORK(7)=0.0D0
C
        NV = 3
        NXI = 1
        XI = 0.0D0
        IRES = 1
        NRWK = 500
        X(1)    = -1.0D0
        X(NPTS) =  2.0D0
        NFIX = 0
        MAXNPT  = NPTS
        XFIX(1) = 0.0D0
        XFIX(2) = 1.0D0
        DO 10 I = 2, NPTS-1
 10        X(I) = X(1) + (I-1)*(X(NPTS) - X(1)) / (NPTS-1)
        IF(IMESH.EQ.1)THEN
           REMESH = .TRUE.
           CONST  = 1.5D0 / (NPTS-1)
           NFIX  = 2
           WRITE(6,1014)
 1014      FORMAT(' INPUT NRMESH AND DXMESH  ')
           READ(5,*)NRMESH,DXMESH
           WRITE(IDEV,1016)NRMESH, DXMESH
 1016      FORMAT(' NRMESH=',I4,' DXMESH= ',D12.5)
           CALL SRMESH( NRMESH, DXMESH, TOUT, IPMINF, NOP, STPRAT,
     1                  CONST, NPTS, ISET)
        END IF
        CALL SETSKL( NEQN, NPDE, NPTS, X, U, RESWK, NRWK, M, TS, IBAND,
     1               IRES, REMESH, MAXNPT, NV, NXI, XI, XFIX, NFIX)
        IF(IRES .LT. 0)GOTO 106
C
       IF(LINALG .EQ. 1)THEN
C        SET UP THE BAND INFORMATION NEEDED BY BANDED MATRIX ROUTINES
         IWORK(1) = IBAND
         IWORK(2) = IBAND
       END IF
C
C  PASS THE INTEGRATION PARAMETERS INTO THE ARRAY INFORM
C
      N = NEQN
      INFORM(1) = 0
      INFORM(2) = 1
      INFORM(3) = 1
      DO 50 I = 4,14
50       INFORM(I) = 0
C
C   SET UP THE TIME INTEGRATION  PARAMETERS
C
      IF(IODE .EQ. 0)THEN
         CALL THESET(8,IRET)
      ELSE IF(IODE .EQ. 1)THEN
         CALL THBSET(8, IRET, .FALSE.,.FALSE.,0.55D0)
C        CALL THBSET(8, IRET, .TRUE.,.TRUE.,0.55D0)
      ELSE
         CALL BDFSET(8, IRET, 3, 2 , .TRUE.)
      END IF
C
      IF(IRET .LT. 0)GOTO 1000
      NWKDRV = 900
      NYSAVE = 161
      NWKJAC = 10000
      LIW = 3220
C
C   SET UP THE PARAMETERS FOR THE LINEAR ALGEBRA INITIALISATION CALLS
C
      NEL = 3*N
      ISET = 0
      SENS = 0.0D0
      IF(LINALG .EQ. 0)THEN
          CALL SMTSET( NEQN, 2, SENS, NWKJAC, LIW, NEL, ISET, JSINFO)
      ELSE IF(LINALG .LE. 2)THEN
          CALL MATSET( LINALG, NEQN, IWORK, IWORK(2), NWKJAC, LIW, ISET)
      ELSE
          NIA = 1
          NJA = 1
          NJCPVT = 166
          ISPLIT = 73
          LBLOCK = .FALSE.
          ETA    = 10.0D0
          JCEVAL = 'NUMERICAL  '
          UPIV   = 0.1
          CALL SPANUF( NEQN, NEQN, JCEVAL, NWKJAC, IA, NIA, JA, NJA,
     1                 IWORK, LIW, SENS, UPIV, ETA, LBLOCK, ISPLIT,ISET)
      ENDIF
      IF(ISET .LT. 0)GOTO 1000
C**********************************************************************
C    CALL THE SPRINT PACKAGE
C
      CALL TIMER(TTIME,2)
      ENORM = 0.0D0
      GERR  = 0.0D0
      WRITE(IDEV,2000)IFUN, IJAC, NITER, NSTEPS, TTIME, ENORM, GERR
      TS    = 0.0D0
      TOUT  = 0.30D0
100   SNORM = 'AVERL2'
C
      IF(IODE .EQ. 0)THEN
C
C          STHETA METHOD WITH FULL MATRIX ROUTINES
C
        IF(LINALG .EQ. 0)THEN
C                                  SPARSE MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETA,YSAVE, NYSAVE
     2                ,PREPJS, WKJAC, NWKJAC, IWORK, JAC, SOLSS,
     3                 MVMESH, WKMON, NWKMON)
        ELSE IF(LINALG .EQ. 1)THEN
C                                  BANDED MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETA,YSAVE, NYSAVE
     2                ,PREPJB, WKJAC, NWKJAC, IWORK, JAC, SOLSB,
     3     MVMESH, WKMON, NWKMON)
        ELSE IF(LINALG .EQ. 2)THEN
C                                  FULL MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETA,YSAVE, NYSAVE
     2                ,PREPJF, WKJAC, NWKJAC, IWORK, JAC, SOLSF,
     3     MVMESH, WKMON, NWKMON)
             ELSE
C                                  NAG SPARSE  MATRIX ROUTINES
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETA,YSAVE, NYSAVE
     2                ,SPANMM, WKJAC, NWKJAC, IWORK, JAC, SPANNV,
     3                 MVMESH, WKMON, NWKMON)
             END IF
       ELSE IF(IODE .EQ. 1)THEN
C
C      SPRINT USED WITH STHETB INTEGRATOR
C      **********************************
        IF(LINALG .EQ. 0)THEN
C                                  SPARSE MATRIX ROUTINES USED.
           CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETB,YSAVE, NYSAVE
     2                 ,PREPJS, WKJAC, NWKJAC, IWORK, JAC, SOLSS,
     3                  MVMESH, WKMON, NWKMON)
        ELSE IF(LINALG .EQ. 1)THEN
C                                  BANDED MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETB,YSAVE, NYSAVE
     2                ,PREPJB, WKJAC, NWKJAC, IWORK, JAC, SOLSB,
     3                 MVMESH, WKMON, NWKMON)
        ELSE IF(LINALG .EQ. 2)THEN
C                                  FULL MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETB,YSAVE, NYSAVE
     2                ,PREPJF, WKJAC, NWKJAC, IWORK, JAC, SOLSF,
     3                 MVMESH, WKMON, NWKMON)
             ELSE
C                                  NAG SPARSE MATRIX ROUTINES
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, STHETB,YSAVE, NYSAVE
     2                ,SPANMM, WKJAC, NWKJAC, IWORK, JAC, SPANNV,
     3                 MVMESH, WKMON, NWKMON)
             END IF
C
      ELSE IF(IODE .EQ. 2)THEN
C
C       SPRINT USED WITH ADAMS/B.D.F METHODS
C       ************************************
        IF(LINALG .EQ. 0)THEN
C                                  SPARSE MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, SPGEAR,YSAVE, NYSAVE
     2                ,PREPJS, WKJAC, NWKJAC, IWORK, JAC, SOLSS,
     3                 MVMESH, WKMON, NWKMON)
        ELSE IF(LINALG .EQ. 1)THEN
C                                  BANDED MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, SPGEAR,YSAVE, NYSAVE
     2                ,PREPJB, WKJAC, NWKJAC, IWORK, JAC, SOLSB,
     3                 MVMESH, WKMON, NWKMON)
        ELSE IF(LINALG .EQ. 2)THEN
C                                  FULL MATRIX ROUTINES USED.
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, SPGEAR,YSAVE, NYSAVE
     2                ,PREPJF, WKJAC, NWKJAC, IWORK, JAC, SOLSF,
     3                 MVMESH, WKMON, NWKMON)
             ELSE
C                                  NAG SPARSE MATRIX ROUTINES
          CALL SPRINT( NEQN, TS, TOUT, U, YDOT, WORK, NWKDRV, REL, ABS,
     1    ITOL, INFORM, SNORM, SKLRES, RESWK, NRWK, SPGEAR,YSAVE, NYSAVE
     2                ,SPANMM, WKJAC, NWKJAC, IWORK, JAC, SPANNV,
     3                MVMESH, WKMON, NWKMON)
             END IF
       END IF
C   PASS STATS BACK ACROSS.
C
       IF(LINALG .EQ. 3)THEN
          ICALL = 0
          IF(INFORM(1) .NE. 2)ICALL = 1
          CALL SPANXF(NEQN, ICALL, LIWREQ, LIWUSD, LRWREQ, LRWUSD, NLU,
     1                NNZ, NGP, ISPLIT, IGROW, LBLOCK, NBLOCK)
       END IF
C
C   ACCESS THE CURRENT MESH.
C
79     IRES = 2
       CALL SETSKL( NEQN,NPDE, NPTS, X, U, RESWK, NRWK, M, TS, IBAND,
     1              IRES, .TRUE. ,NPTS, NV, NXI, XI, XFIX, NFIX)
        IFUN = INFORM(7)
        IJAC = INFORM(8)
        NSTEPS = INFORM(6)
        NITER = INFORM(11)
C
        DELX = 1000.0D0
        DO 677 I = 2, NPTS
          DX  = X(I) -X(I-1)
          IF(DX .LE. DELX)DELX = DX
677     CONTINUE
        WRITE(IDEV,81)TOUT,INFORM(1),DELX
        WRITE(6,81)TOUT,INFORM(1),DELX
81      FORMAT('  SOL AT',D12.4/' INFORM(1) =',I3,' MESH WIDTH =',D11.3)
        CALL TIMER(TTIME,2)
C
        CALL ERROR( U, NPDE, NPTS, X, M, ENORM, GERR, TS, REL, ABS)
C
        WRITE(IDEV,2000)IFUN, IJAC, NITER, NSTEPS, TTIME, ENORM, GERR
106     CONTINUE
        IF(TOUT .LT. 0.35D0)THEN
           CALL SPATEV
C          ITRACE = 3
        END IF
        IF(INFORM(1) .EQ. 2)THEN
           IF(TOUT .GE. 2.7D0)GOTO 1000
           TOUT = TOUT + 0.10D0
C          IF(TOUT .GE. 0.51)ITRACE = 3
           GOTO 100
        END IF
 1000 CONTINUE
      CALL TIMER(TTIME,2)
      WRITE(IDEV,2000)IFUN,IJAC,NITER, NSTEPS, TTIME, ENORM, GERR
 2000 FORMAT(' FUN=',I5,' IJAC=',I5,' NITER=',I5,' NSTEPS=',I5,' CPU=',
     1       D11.3,' ENORM=',D11.3,' GRID ERR=',D11.3)
      STOP
      END
C**********************************************************************
C        PROBLEM DESCRIPTION ROUTINES
C**********************************************************************
       SUBROUTINE UINIT( NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. INITIAL VALUES.
       INTEGER NPDE, NPTS
       DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), T
         T = 0.0D0
         CALL EXACT( T, NPDE, NPTS, X, U )
       RETURN
       END
       SUBROUTINE SPDEF1(T, X, NPDE, U, DUDX, C, F, R, NV, V,VD,L)
C      ROUTINE TO DESCRIBE THE BODY OF THE P.D.E.
       INTEGER NPDE, J, NV, L
       DOUBLE PRECISION T, KT, X, U(NPDE), DUDX(NPDE),
     1            C(NPDE,NPDE), F(NPDE), R(NPDE), V(3), VD, CC, VTEMP
       C(1,1) = 1.0D0
       IF(X .LE. 0.0D0 )THEN
C        ELEMENT TO LEFT OF THE INTERFACE AT 0.0
         CC =  0.1D0
         VTEMP = V(1)
       ELSE IF(X .GT. 0.0D0 .AND. X .LE. 1.0D0)THEN
C        ELEMENT BETWEEN 0.0 AND 1.0
         CC =  1.0D0
         VTEMP = V(2)
       ELSE
         CC =  2.0D0
         VTEMP = V(3)
       END IF
       R(1) = DUDX(1) /CC
       F(1) = -DEXP(-U(1))- DEXP(-2.0D0*U(1))* VTEMP
       RETURN
       END
       SUBROUTINE BNDR( T, BETA, GAMMA, U, UX, NPDE, LEFT, NV, V, VD,L)
C      BOUNDARY CONDITIONS ROUTINE
       INTEGER NPDE, I,NV
       DOUBLE PRECISION T, BETA(NPDE), GAMMA(NPDE), U(NPDE), HPI, VD
     1                    ,UX(NPDE), C2, C3, V(3)
       LOGICAL LEFT
       IF(LEFT) THEN
          BETA (1) = 1.0D0
          GAMMA(1) = V(1) / ( -0.1 + T + 1.0D0)
       ELSE
          C2 = 1.0D0
          C3 = 2.0D0
C         BETA (1) = C3 * ( C2 + C3 + T + 1.0D0)
          BETA (1) = 0.00D0
          GAMMA(1) = U(1) - DLOG( C2 + C3 + T + 1.0D0)
       END IF
       RETURN
       END
C      DUMMY ROUTINES FOR NON-EXISTANT O.D.E. PART
       SUBROUTINE SVINIT( V , NV)
       INTEGER NV
       DOUBLE PRECISION V(3)
       V(1) = 0.1D0
       V(2) = 1.0D0
       V(3) = 2.0D0
       RETURN
       END
       SUBROUTINE SODEFN(T, NV, V, VDOT, NPDE, NXI, XI, UI, UXI, RI,
     1                    UTI, UTXI, VRES, IRES)
C      ROUTINE TO PROVIDE RESIDUAL OF COUPLED ODE SYSTEM FOR THE
C      MOVING BOUNDARY PROBLEM.
       INTEGER NPDE, NXI, NV, IRES
       DOUBLE PRECISION T, XI(NXI), UI(NPDE,NXI), UXI(NPDE,NXI),
     1         RI(NPDE,NXI), UTI(NPDE,NXI), UTXI(NPDE,NXI), VRES(NV),
     2         V(NV), VDOT(NV), TEM
       DO 10 I = 1,3
 10       VRES(I) = VDOT(I)
       RETURN
       END
       SUBROUTINE SET( A, N, B, M)
       INTEGER N, M, I
       DOUBLE PRECISION A(N),B(M)
       IF(N .LE. M)THEN
          DO 10 I = 1,N
             B(I) = A(I)
 10          A(I) = 0.0D0
          DO 20 I = N+1,M
 20          B(I) = 0.0D0
       ELSE
          WRITE(6,30)
 30       FORMAT(' INVALID CALL TO SET ROUTINE - NOTHING DONE')
       END IF
       RETURN
       END
C**********************************************************************
      SUBROUTINE EVNRMF( TIME, NIP, NPDE, X, U, R, MAXNPT, FMON)
C**********************************************************************
C     EXAMPLE MONITOR  FUNCTION ROUTINE FOR REMESHING.
C     R(J,I) CONTAINS THE FLUX FOR JTH P.D.E. AT POINT Y(I)
C        WHERE Y(1) = X(1) , Y(NIP+1) = X(NIP)
C              Y(I) =(X(I+1) - X(I)) * 0.5   I = 2,..., NIP
C     THE MONITOR FUNCTION IS RETURNED IN THE ARRAY FMON
C
      INTEGER NIP, NPDE, MAXNPT, I, J, K, L
      DOUBLE PRECISION U(NPDE,1), X(NIP), SUM,FMON(NIP), TIME,
     1                 R(NPDE,1),  DRDX, H
      DO 10 I=1,NIP
        K = MAX0(1,I-1)
        L = MIN0(NIP,I+1)
        H = (X(L) - X(K)) * 0.5D0
        SUM=0.0D0
        J = 1
C       DO 5 J=1,NPDE
C 1ST DERIV
           DRDX = R(J,I)
C 2ND DERIV
C          DRDX = (R(J,I+1) - R(J,I)) / H
    5      SUM = SUM + DRDX**2
        FMON(I)=DSQRT(SUM)
   10 CONTINUE
      RETURN
      END
C EXACT SOLUTION
       SUBROUTINE EXACT( TIME, NPDE, NPTS, X, U)
C      ROUTINE FOR P.D.E. EXACT VALUES  (IF KNOWN)
       INTEGER NPDE, NPTS, I
       DOUBLE PRECISION X(NPTS), U(NPDE,NPTS), TIME, C, C2
       DO 10 I = 1,NPTS
          IF(X(I) .LE. 0.0D0 )THEN
C            ELEMENT TO LEFT OF THE INTERFACE AT 0.0
             C =  0.1D0
             U(1,I) = DLOG( TIME + 1.0D0 + C * X(I))
          ELSE IF(X(I) .GT. 0.0D0 .AND. X(I) .LE. 1.0D0)THEN
C            ELEMENT BETWEEN 0.0 AND 1.0
             C =  1.0D0
             U(1,I) = DLOG( TIME + 1.0D0 + C * X(I))
          ELSE
             C  = 1.0D0
             C2 = 2.0D0
             U(1,I) = DLOG( TIME + 1.0D0 + C2* (X(I) - 1.0D0) + C )
          END IF
10     CONTINUE
       RETURN
       END
        SUBROUTINE ERROR( U, NPDE, NPTS, X, M, ENORM, GERR, T,
     -                    RELERR,  ABSERR)
C
C***********************************************************************
        INTEGER I, IN, NP, NPTS, NPDE, JI, M, ITRACE, IDEV, IONE, IWK
     -         , IFLAG
        DOUBLE PRECISION UN(5), US(404), WX(5), X(NPTS), XP(201), ERR(5)
     -       ,EPS, RELERR, ABSERR, EREL, EABS, WS, HH, ENORM, GERR, ER
     -       ,U(NPDE,NPTS), T
C
C**********************************************************************
C       THE FOLLOWING ROUTINE COMPUTES THE ERROR ENORM IN THE NUMERICAL
C       SOLUTION BY USING A COMBINATION OF THE L2 FUNCTION AND VECTOR
C       NORMS. GERR IS THE MAXIMUM ERROR AT THE GRID POINTS
C       THE EXACT SOLUTION IS ASSUMED TO BE GIVEN BY:
C                SUBROUTINE EXACT(T,NPDE, NP, XP, US)
C                 DIMENSION US(NPDE, NP)
C***********************************************************************
        COMMON/SDEV2/ ITRACE, IDEV
        IONE = 1
C
C   SET UP AVEREDGED L2 NORM WEIGHTS AND ESTIMATE NORM USING NP POINTS
C
        EPS=DMAX1(RELERR,ABSERR)
        IF(EPS.LE.0.0)RETURN
        EREL=RELERR/EPS
        EABS=ABSERR/EPS
        DO 10 IN=1,NPDE
          WX(IN)=0.0D0
          DO 9 I=1,NPTS
            EPS=DABS(U(IN,I))
            IF(WX(IN).LT.EPS)WX(IN)=EPS
 9        CONTINUE
          WX(IN)=WX(IN)*EREL+EABS
10      CONTINUE
        NP = 201
        HH = (X(NPTS) - X(1))/(NP-1)
        DO 30 IN = 1,NPDE
30        ERR(IN) = 0.0D0
        WS = 1.0D0
        DO 40 I=1,NP
40        XP(I)=X(1)+(I-1)*HH
        CALL INTERX(XP,US,NP,X,U,NPTS,NPDE,IFLAG,IONE)
        JI=1
        DO 60 I=1,NP
          CALL EXACT( T, NPDE, IONE, XP(I), UN)
          IF(M.NE.0) WS = XP(I)**M
          DO 50 IN = 1,NPDE
             ER = DABS(US(JI)-UN(IN))
C            IF(ITRACE.GE.1)WRITE(IDEV,49)IN,US(JI),UN(IN),ER
C49          FORMAT(2X,I3,' INTER=',D11.3,'  ACT=',D11.3,' ER=',D11.3)
             ERR(IN) = ERR(IN) + WS * ER**2
50           JI=JI+1
60      CONTINUE
        ENORM  = 0.0D0
        DO 70 IN = 1,NPDE
70         ENORM = ENORM + ERR(IN)/WX(IN)**2
        ENORM   = DSQRT(ENORM*HH)
C
C       COMPUTE THE MAXIMUM ERROR AT THE GRID POINTS
C
        IF(ITRACE .GE. 1)WRITE(IDEV,666)
666     FORMAT(6X,' MESH',6X,'NUM SOL',10X,'SOL',7X,'ERROR')
        GERR = 0.0D0
        DO 90 I = 1,NPTS
           CALL EXACT(T,NPDE, IONE, X(I), UN)
           DO 90 IN = 1,NPDE
              ER = DABS(U(IN,I) - UN(IN))
              IF(ITRACE.GE.2)WRITE(IDEV,900)X(I),U(IN,I),UN(IN),ER
900           FORMAT(4(2X,D11.3))
              IF(GERR.LT.ER) GERR = ER
90      CONTINUE
        IF(ITRACE .GT. 0)WRITE(IDEV,880)ENORM,GERR
880     FORMAT('  ENORM=',D11.3,'       GERR=',D11.3)
        RETURN
        END
