C  
C  Example simple driver routine for o.d.e. problems with DISCONTINUITIE
C  This example is the 3 o.d.e. problem of Sijbrand (1984)
C
       INTEGER NEQ, NRW, ITOL, INFORM(17), NWKRES, JACPVT(3),
     1         NWKMON, IRET, NEQMAX
       INTEGER  ITRACE, IDEV, I, NY2DIM, METH, MAXORD, LINALG, MU, ML
       INTEGER  NFCN, NOINV, NOSOLV, IDISC
       DOUBLE PRECISION T, TOUT, Y(20), RWORK(30), RTOL, ATOL, WKRES(1),
     1         YSAVE(3,14), WKJAC(11), WKMON(6), YDOT(3),
     2         GOLD,H0,HMIN,HMAX,HMINMN
       DOUBLE PRECISION T0, Y0(3), TF, TTIME
       CHARACTER*6 SNORM
       EXTERNAL RESID, PREPJF, SOLSF, MONITD, SPRJAC, SPGEAR
C Output channel number IDEV (DEFAULT=6)
       COMMON /SDEV2/ ITRACE, IDEV
C COMMON blocks for discty handling:
C        TOUT is needed for MONITD
C        HMINM is the minimum step-size for integration near the discty
C        IDISC is used by MONITD to indicate proximity to the discty
C              the user just needs to set this to zero initially.
      COMMON/MAINMD/TOUT,HMINMN,IDISC
C        GOLD has to be initialised to value of SWITCH at start time.
      COMMON/DISRES/GOLD
C problem data:  start and end times and initial values
       DATA T0 / 0.D0/
       DATA TF / 1.D0/
       DATA Y0 / 1.0D-4, 1.0D-10, 0.0D0/
C
C setup the problem inputs for SPRINT
C
       NEQ = 3
       NEQMAX = 3
       NWKRES = 1
C MONITD requires workspace WKMON(NWKMON) of size 2*NEQ
       NWKMON = 2 * NEQ
       NWKJAC = NEQ*NEQ + 2
       NRW    = 21 + 3*NEQ
       RTOL    = 1.0D-3
C Y2 component small near discty so ATOL needs to be small
C (could use ATOL/RTOL array option ITOL=4)
       ATOL    = 1.0D-11
       ITOL   = 1
       MAXORD = 5
C Use averaged L2 norm rather than 'L2NORM' for compatibility with MONIT
       SNORM = 'AVERL2'
C
C Initialisations (T = T0)
       T = T0
       TOUT = TF
       CALL TIMER(TTIME,1)
       DO 10 I = 1,NEQ
   10    Y(I) = Y0(I)
C For discty handling - initialise IDISC and SWITCH value GOLD;
C should be done for each (T,TOUT) interval
      IDISC=0
      CALL SWITCH(NEQ,T,Y,GOLD)
C
C Set up the optional inputs:
C For discontinuity handling it is more efficient to set the following
C min/max stepsizes to realisitic values -
C HMIN is the minimum step-size for normal integration
C efficiency is aided by setting HMIN small but not too small
      HMIN=1.0D-6
C HMINM is the minimum step-size for integration near the discontinuity
      HMINMN=1.0D-16
C HMAX is the maximum step-size used and should be set realistic to avoi
C over-stepping the discty by too much (inefficient)
      HMAX=0.1D0
      RWORK(6) = HMAX
      RWORK(7) = HMIN
      INFORM(1) = 0
      INFORM(2) = 1
      INFORM(3) = 1
      INFORM(4) = 1000
C SPRINT trace option level 1 output to channel no. 4
C ITRACE non-zero also sets on diagnostic messages from MONITD
      ITRACE=1
      IDEV = 4
C
C call the setup routine MATSET for the linear algebra
C LINALG = 2 for full matrix
C
      LINALG = 2
C bandwidths MU and ML not used
      CALL MATSET( LINALG, NEQ, MU, ML, NWKJAC, NEQMAX, IRET)
      IF(IRET .LT. 0)THEN
         WRITE(IDEV,20)
 20      FORMAT(' SETUP FAILED IN MATSET')
         STOP
      END IF
C
C call the setup routine BDFSET for the integration method
C METH=2 for Gear (b.d.f) method
C
      NY2DIM = 6
      MAXORD = 5
      METH = 2
      CALL BDFSET( NY2DIM, IRET, MAXORD, METH, .FALSE.)
      IF(IRET .LT. 0)THEN
         WRITE(IDEV,21)
 21      FORMAT(' SETUP FAILED IN INTEGRATOR SETUP')
         STOP
      END IF
C
C Call SPRINT with SPGEAR integrator and MONITD discty routine
C
      CALL SPRINT( NEQ, T, TOUT, Y, YDOT, RWORK, NRW, RTOL, ATOL, ITOL,
     1       INFORM, SNORM, RESID, WKRES, NWKRES, SPGEAR, YSAVE, NEQMAX,
     2       PREPJF, WKJAC, NWKJAC, JACPVT, SPRJAC, SOLSF,
     3       MONITD, WKMON, NWKMON)
C
C  Retrieve and print the integration statistics
C
       NFCN   = INFORM(7)
       NOSTEP = INFORM(6)
       NOINV  = INFORM(8)
       NOSOLV = INFORM(11)
       CALL TIMER(TTIME,2)
       WRITE(IDEV,997) RTOL,NOSTEP,NFCN,NOINV,NOSOLV
  997  FORMAT('0',3X,D9.3,2X,I5,2X,I5,3X,I4,5X,I4)
       WRITE(IDEV,998)
  998  FORMAT('0      TTIME   INFORM(1)  T ')
       WRITE(IDEV,999) TTIME,INFORM(1),T
  999  FORMAT('0',D11.3,4X,I2,4X,D10.4)
       STOP
C
      END
C
      SUBROUTINE RESID( N, T, Y, YDOT, R, IRES, RWORK, NR)
C
C On input, IRES is used as normal to indicate the form of the residual
C required by SPRINT (the values are IRES = -1 or 1 as explained in
C Chapter 3).
C For discontinuity handling we use the SWITCH routine to see if thedis-
C continuity has been crossed and if so IRES must be set to 4 on output.
C-------------------------------------
C This tells SPRINT to pass control to the MONITD routine.
C
C GOLD is the last timestep value of SWITCH (held in COMMON DISRES)
C GVAL is the current value of SWITCH evaluated in this RESID call.
C GOLD*GVAL<0 indicates discty crossed so set IRES=4 and return.
C
      INTEGER I,N,IRES,NR,ITRACE,IDEV
      DOUBLE PRECISION T, Y(1), YDOT(1), R(1), RWORK(1), GOLD, GVAL, RF
      COMMON/DISRES/GOLD
      COMMON /SDEV2/ ITRACE, IDEV
C CHECK EVERY TIME TO SEE IF DISCTY HAS BEEN CROSSED
      CALL SWITCH(N,T,Y,GVAL)
      IF(GOLD*GVAL.LT.0.0D0)THEN
         IRES=4
C IRES = 4 returns control to SPRINT which passes control to MONITD
C optional print message to indicate discontinuity reached
         WRITE(IDEV,900)T,GOLD,GVAL,IRES
  900    FORMAT(//' RESID ROUTINE DETECTED DISCTY AT T = ',D14.6,
     -   /' GOLD AND GVAL = ',2D14.6,/' IRES = ',I3)
      ELSE
C Normal computation of residuals (IRES=-1 or 1 cases)
         IF(IRES .EQ. -1)THEN
            DO 29 I = 1,N
 29            R(I) = -YDOT(I)
            RETURN
         END IF
C full residual (IRES=1)
         RF=3080.25D0*(Y(1)*Y(2)-1.0D-14)
         R(1) = (1.0D-3-Y(1))-RF
         R(2) = (Y(3)-Y(2))-RF
C discontinuous right-hand side depending on GVAL=current value of SWITC
         IF(GVAL.LT.0.0D0)THEN
            R(3) = -20.0D0*Y(3)
         ELSE
            R(3) = 20.0D0*(1.0D-2-Y(3))
         ENDIF
         DO 30 I = 1,N
 30         R(I) = R(I) - YDOT(I)
      ENDIF
      RETURN
      END
C
C Switch routine to describe the, in general, non-linear equation
C        G(N,T,Y) = 0             at the discontinuity.
C This function is used to describe the position of the discontinuity
C and also whether the discontinuity has been crossed.
C Input parameters: N - no. of differential/algebraic equations
C                   T - current time
C                   Y - array of size N of current solution values
C Output parameter: G - current value of the switch function G(N,T,Y)
C
      SUBROUTINE SWITCH(N,T,Y,G)
      INTEGER N
      DOUBLE PRECISION Y(N),T,G
C Switch in first component y(1) at the value 5.0D-4
      G=Y(1)-5.0D-4
      RETURN
      END
