      SUBROUTINE CONLAB
C---------------------------------------------------------------C
C***BEGIN PROLOGUE CONLAB                                       C
C                                                               C
C   AUTHOR :                                                    C
C     David Di Ruscio                ITK/NTH                    C
C     Work started november 1988                                C
C                                                               C
C  KEYWORDS: - Multivariable frequency analysis.                C
C            - Singualer Value decomposision.                   C
C            - Feedback (Controller) syntesis.                  C
C               - Solution of Riccati eq. by Laubs Schur method.C
C               - Prescribed degree of stability.               C
C               - Eigenvalues of closed loop matrix (A+B*G).    C
C               - Performance criterion value.                  C
C               - Frobenius norm of closed loop matrix (A+B*G). C
C                 (robustnes measure)                           C
C            - Optimal Estimator Syntesis. (Kalman filter).     C
C            - Stability analysis.                              C
C            - Graphic plotting of time series.                 C
C            - Matrix Calculations  MATLAB                      C
C            - Linear simulation of total system LQ,LQG etc.    C
C            - Solution of stiff ODE by SPRINT based on LSODI   C
C              on low lewel.                                    C
C                                                               C
C   PARAMETER DEFINITIONS :                                     C
C     NA - INTEGER maximum number of states.                    C
C     NB - INTEGER maximum number of control variables.         C
C     NC - INTEGER maximum number of disturbances.              C
C     ND - INTEGER maximum number of measurements.              C
C     IP - INTEGER maximum number of samples to be stored.      C
C                                                               C
C***END PROLOGUE CONLAB                                         C
C---------------------------------------------------------------C
C1... Initialize maximum system dimensions   ....................
      INTEGER NX,NU,NV,NY,NA,NB,NC,ND,NJOB,ISVAR,IERR,NXX,IP,IWTE
      PARAMETER (NA = 50, NB = 50, NC = 50, ND = 50, IP = 200 )
      PARAMETER (NXX = NA + NA, NJOB = 19, IWTE = 6)
      INTEGER IJOB(NJOB),IND(NXX),INFO,ISYSDEF,OPTIONS,IDEF,ISVD
      INTEGER IF
      DOUBLE PRECISION A(NA,NA),B(NB,NB),C(NC,NC),D(ND,ND)
      DOUBLE PRECISION Q(NA,NA),P(NB,NB),R(NA,NA),G(NB,NA)
      DOUBLE PRECISION WY(ND,ND),VV(NC,NC),KA(NA,ND),XA(NA,NA)
      DOUBLE PRECISION PW(NB,NB),H(NA,NA),WP(NA,NB),AA(NA,NA)
      DOUBLE PRECISION Z(NXX,NXX),WZ(NXX,NXX),E(ND,ND)
      DOUBLE PRECISION W3(NB),ER(NXX),EI(NXX),WV(NXX)
      DOUBLE PRECISION FNORM,DJ,DJ2,ALFA,DET
      DOUBLE PRECISION W1(NA,NA),W2(NA),W,DT
      REAL PL(IP,NA)   
      INTEGER IPVT(NA),IND1(NA),IND2(NA),IFIL,ISTAT,COL1,COL2
      INTEGER*4 ITER,FIRST,NELEM
      COMPLEX*16 WI,HS(ND,NB),WH(NA,NA),ZC(NA)
      COMPLEX*16 S(NA),ES(NA),U(NA,NA),V(NA,NA)
      CHARACTER CFORM*6,NAME*12,CFILE*12,CSYS(10)*2

C... Initialize                                 ..................
      CFORM = '(5X,A)'
      NAME  = 'CONLAB'
      CFILE = 'CONLAB.DAT'
      ITER  = 100
      ISVD  = 1
      NX = 1
      NU = 1
      NV = 1
      NY = 1

CUT. Initialize CYPROS macro, graphics etc.     .................
CUT   CALL CYPROS('CONLAB')

C2... Print program description on terminal     .................
      CALL COHEAD(IWTE)

C3... List menu of CONLAB utilities             .................
      IDEF = 2
100   CALL MENU(IJOB,NJOB,IDEF)
      IF (IJOB( 8).EQ. 8) GO TO 80
      IF (IJOB(10).EQ.10) GO TO 600
      IF (IJOB(11).EQ.11) GO TO 110
      IF (IJOB(12).EQ.12) GO TO 120
      IF (IJOB(13).EQ.13) GO TO 120
      IF (IJOB(14).EQ.14) GO TO 120
      IF (IJOB(15).EQ.15) GO TO 150
      IF (IJOB(16).EQ.16) GO TO 160
      IF (IJOB(17).EQ.17) GO TO 170
C... if IJOB(NJOB) equal 0 then terminate program
      IF (IJOB(19).EQ. 0) GO TO 1000

C4... READ (or store) SYSTEM FROM DATA FILE     .................
      IF (IJOB(1).EQ.1.OR.IJOB(6).EQ.6) THEN
       IF (IJOB(6).EQ.6 .AND. ISYSDEF.NE.2) THEN
        CALL CONERR(1,IWTE)
        GO TO 100
       ENDIF

       IF (IJOB(1).EQ.1) THEN
        IFLAG = 1
        ISYSDEF = 2
       ELSEIF (IJOB(6).EQ.6) THEN
        IFLAG = 2
       ENDIF

       CALL RCSTR('Specify file name  ',CFILE,1,1,IIII)
       IF (CFILE.EQ.'0') THEN
        CFILE = 'CONLAB.DAT'
        GO TO 100
       ENDIF
       CALL SYSDAT(A,B,C,P,Q,G,KA,D,WY,VV,NX,NU,NV,NY,NA,
     +             IFLAG,CFILE,IERR)
       IF (IERR.EQ.0) THEN
        IDEF = 3
        GO TO 100
       ELSEIF (IERR.NE.0) THEN
        CFILE='CONLAB.DAT'
        IDEF = 2
        GOTO 100
       ENDIF
      ENDIF

C5... Define system and system dimensions       .................
      IF (IJOB(2).EQ.2) THEN
       CALL SYSDEF(NX,NU,NV,NY,CSYS)
       IDEF = 3
       ISYSDEF = 2
       GO TO 100
      ENDIF

C... If system dimensions not initialized print message   .......
      IF (ISYSDEF.NE.2) THEN
       CALL CONERR(1,IWTE)
       GO TO 100
      ENDIF

C6... Define system matrixes                .....................
      IF (IJOB(3).EQ.3) THEN
       CALL ERRCHK(1,IERR,NX,NU,NV,NY,CSYS)
       IF (IERR.NE.0) GO TO 100
       CALL SYSMAT(A,B,C,D,G,KA,W1,W2,NX,NU,NV,NY,NA,NB,NC,ND,CSYS,5,
     +             IERR)
       IDEF = 4
       IF (IERR.NE.0) GO TO 100
       GO TO 100
      ENDIF

C7... COMPUTE OPTIMAL FEEDBACK  ( solve Riccati equation ) ......
      IF (IJOB(4).EQ.4) THEN

C...  Specify weight matrixes and degree of stability      ......
400    CALL PQINP(Q,P,A,AA,ALFA,W3,W1,W2,NX,NU,NA,NB,IFLAG)

       ISVAR = IYESANS('Compute optimal feedback ?',2)
       IF (ISVAR.EQ.1) GO TO 100
       WRITE(IWTE,CFORM)'*   COMPUTING OPTIMAL FEEDBACK   *'

C...  Compute B*inv(P)*tr(B) = H                           ......
       CALL OPTFEED(B,P,H,W3,PW,WP,NX,NU,NA,NB,IERR)
       CALL MATCOP(Q,WP,NX,NX,NA,NA)

C...  Solve Riccati equation by Laubs Schur method         ......
       CALL RICSOL(AA,H,WP,Z,WZ,NX,ER,EI,WV,IND,INFO,NA,NA,NA)
       IF (INFO.NE.0) THEN
        CALL RICERR(INFO)
       ENDIF
       IF (ALFA.NE.0.0D0) THEN
        CALL EIGCOR(ER,NX,ALFA)
       ENDIF
       CALL MATCOP(WP,R,NX,NX,NA,NA)

C...  Compute Feedback G from inv(P),B and R               ......
       CALL GMAT(G,PW,B,R,WP,NX,NU,NA,NB)

C...  Compute Frobenius norm of A+B*G = WP                 ......
       CALL CLABG(WP,A,B,G,NX,NU,NA,NB)
       FNORM = DFNORM(WP,NA,NX)

C...  Compute kost kriterium value, trace(R) = DJ          ......
       CALL TRACE(R,DJ,NX,NA)

       IF (INFO.EQ.0) WRITE(IWTE,CFORM)'* Done'
C...  List computed results                                ......
       CALL OPTLIS(G,R,ER,EI,FNORM,DJ,NX,NU,NA,NB)

       ISVAR = IYESANS('Leave OPtimal modus ?',2)
       IF (ISVAR.EQ.1) GO TO 400
       IDEF = 5
       GO TO 100
      ENDIF

C8... Singular Value Analysis, compute HS(jw) = D*(jw*I-A)inv*B+E
      IF (IJOB(5).EQ.5) THEN

C...Determine what to be done   .................................
       CALL SVDUTL(ISVD,IWTE)

C...Define the matrixes PW,H,WP and E in HS = WP(jw-H)inv*PW+E ..
C...as a function of the system matrixes A,B,C,D,G and E.
       IF (ISVD.NE.0) THEN
       CALL SVDMAT(A,B,C,D,G,E,H,PW,WP,NA,NB,NC,ND,NX,NU,NV,NY,ISVD)
       ELSEIF (ISVD.EQ.0) THEN
        IDEF = 2
        GO TO 100
       ENDIF

C... Define simulation parameters                       .........
       WRITE(*,CFORM)'*  SINGULAR VALUE ANALYSIS'
       WRITE(*,CFORM)'*  Compute SVD( H(jw) ) = s(jw)  '

       CALL DREAD('Start at frequency w ?',W,1,1,IIII)
       CALL NREAD('Number of iterations ?',ITER,1,1,IIII)
       CALL DREAD('Sample intervall DT  ?',DT,1,1,IIII)

C... Start computing HS(jw) and svd( HS(jw) ) --> P(IP,ITER) ....
       WRITE(IWTE,CFORM)'*  Computing svd( HS(jw) )           *'
       CALL SVDJOB(H,PW,WP,E,HS,W,WH,W3,ZC,IPVT,IND1,IND2,
     +              S,ES,U,V,NX,NU,NY,NA,NB,ND,PL,IP,ITER,DT)

C... Store results P (singular values)          .................
       ISVAR=IYESANS('Store and display singular values ?',2)
       IF (ISVAR.EQ.1) GO TO 100
       IF (ISVAR.EQ.2) THEN
        CALL RCSTR('Specify ts - file name !',NAME,1,1,IIII)
        WRITE(*,'(5X,A)')'No graphics implemented'
CUT     CALL STORE(NAME,PL,NU,IP,NA,ITER,DT,IFIL,ISTAT)
        IJOB(7) = 7
       ENDIF
      ENDIF

C9... Display a TS - file IFIL on terminal        ...............
      IF (IJOB(7).EQ.7) THEN
       WRITE(*,'(5X,A)')'No graphics implemented'
CUT    CALL TSPLOT(IFIL,COL1,COL2,FIRST,NELEM,OPTIONS)
       IDEF = 2
       GO TO 100
      ENDIF

C10.. Matrix Calculation modus                   ................
80    IF (IJOB(8).EQ.8) THEN
       CALL MATCAL(A,B,C,D,G,R,P,Q,WY,VV,XA,KA,NX,NU,NV,NY,
     +             NA,NB,NC,ND,IERR)
       IDEF = 2
       ISYSDEF = 2
       GO TO 100
      ENDIF

C11.. Optimal Estimator modus                    ................
      IF (IJOB(9).EQ.9) THEN

C...  Specify weight matrixes WY and VV          ................
900    CALL WVINP(WY,VV,W3,W1,W2,NY,NV,ND,NC,IFLAG)

       ISVAR = IYESANS('Compute Kalman filter gain ?',2)
       IF (ISVAR.EQ.1) GO TO 100
       WRITE(IWTE,CFORM)'* COMPUTING OPTIMAL KALMAN FILTER GAIN *'

C...  Compute tr(D)*inv(WY)*D = H , inv(WY) = PW ................
       CALL INVERS(WY,PW,W3,DET,NY,ND)
       CALL MATBA1(D,PW,H,NY,NX,ND,NB,W3)

C...  Compute C*VV*tr(C) = WP                    ................
       CALL MABAT1(C,VV,WP,NX,NV,NC,NB,W3)
       CALL TRANSP(A,AA,NX,NX,NA,NA)

C...  Solve Riccati equation for cov. error matrix X = WP   .....
       CALL RICSOL(AA,H,WP,Z,WZ,NX,ER,EI,WV,IND,INFO,NA,NA,NA)
       IF (INFO.NE.0) CALL RICERR(INFO)
       CALL MATCOP(WP,XA,NX,NX,NA,NA)

C...  Compute K = X*tr(D)*inv(W)  ( KA = XA*tr(D)*PW)
       CALL TRANSMULT(XA,D,WP,NX,NX,NY,NA,1)
       CALL MATMULT(WP,PW,KA,NX,NY,NY,NA)

C...  Compute trace of cov. error matrix XA      ................
       CALL TRACE(XA,DJ2,NX,NA)

       IF (INFO.EQ.0) WRITE(IWTE,CFORM)'* Done'
C...  List results                               ................
       CALL ESTLIS(XA,KA,ER,EI,DJ2,NX,NY,NA,ND)

       ISVAR = IYESANS('Leave estimator modus ?',2)
       IF (ISVAR.EQ.1) GO TO 900
       IDEF = 2
       GO TO 100
      ENDIF

C12.. Time Domain Simulation modus - TDSIM       ................
      IF (IJOB(10).EQ.10) THEN

600    CALL TDINIT(IWTE)
       CALL DIMCOP(IND,NX,NU,NV,NY,2)
CUT    WRITE(*,'(5X,A)')'Not implemented on this unix version'
       CALL TDLSIM(A,B,C,D,G,KA,IND,0)
       CALL DIMCOP(IND,NX,NU,NV,NY,1)

       IDEF = 2
       GO TO 100
      ENDIF

C13.. CALL USER MODUS                            ................
      IF (IJOB(11).EQ.11) THEN
110    CALL USERM(A,B,C,D,NX,NU,NV,NY,NA,NB,NC,ND)
       IDEF = 3
       GOTO 100
      ENDIF

C14.. SAVE, LOAD and LIST modus                  ................
      IF (IJOB(12).EQ.12.OR.IJOB(13).EQ.13.OR.IJOB(14).EQ.14) THEN
120    IF (IJOB(12).EQ.12) IF = 1
       IF (IJOB(13).EQ.13) IF = 2
       IF (IJOB(14).EQ.14) IF = 3
       CALL DATCOM(A,B,C,D,G,KA,NX,NU,NV,NY,NA,NB,NC,ND,IF)
       IDEF = 2
       GO TO 100
      ENDIF

C15.. HELP modus                                 ................
      IF (IJOB(15).EQ.15) THEN
150    CALL CONHLP(IWTE)
       IDEF = 2
       GOTO 100
      ENDIF

160   IF (IJOB(16).EQ.16) THEN
C16.. ZEROES computation                         ................
       CALL ZERCMP(A,B,NX,NU,NA,NB)
       IDEF = 2
       GO TO 100
      ENDIF

170   IF (IJOB(17).EQ.17) THEN
C17.. Modal optimal design                       ................
       CALL MODOPT(A,B,P,Q,R,G,NX,NU,NA,NB)
       IDEF = 8
       GO TO 100
      ENDIF

180   IF (IJOB(18).EQ.18) THEN
C18.. Solve Liapunov matrix equation             ................
       CALL DLYPSOL(A,Q,R,NA,NX,WZ,WV,NXX,IND,INFO)
       IDEF = 8
       GO TO 100
      ENDIF


1000  ISVAR = IYESANS('End CONLAB ?',2)
      IF (ISVAR.EQ.1) THEN
       IDEF = 2
       GO TO 100
      ENDIF
C...Last line of SUBROUTINE CONLAB          .....................
      END

      SUBROUTINE SYSDEF(NX,NU,NV,NY,CSYS)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE SYSDEF                                       C
C                                                               C
C   PURPOSE                                                     C
C   SYSDEF defines system and system dimensions.                C
C                                                               C
C***END PROLOGUE SYSDEF                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NV,NY,IPRO,IFLAG,INFO(3)
      PARAMETER (IPRO = 5)
      CHARACTER CJOB*10,CFORM*6,CSYS(*)*(*),CPRO(IPRO)*1
      DATA CPRO(1)/'A'/, CPRO(2)/'B'/, CPRO(3)/'C'/, CPRO(4)/'D'/,
     +     CPRO(5)/'G'/

C1... Initialize format and system parameters   .................
      CFORM = '(8X,A)'
      CJOB  = 'A,B,C,D,G'

C2... Output text                       .........................
100   WRITE(*,CFORM)'             < S D E F  >              '
      WRITE(*,CFORM)'       SYSTEM DEFINITION MODUS         '
      WRITE(*,CFORM)'---------------------------------------'
      WRITE(*,CFORM)'MODEL TYPE :    .                      '
      WRITE(*,CFORM)'                x = A x + B u + C v    '
      WRITE(*,CFORM)'                y = D x                '
      WRITE(*,CFORM)'                u = G y                '
      WRITE(*,CFORM)'---------------------------------------'
      WRITE(*,CFORM)' '

C... Specify system
      CALL RCSTR('Specify system matrixes',CJOB,1,1,IIII)
      IF (CJOB.EQ.'0') GO TO 1000
      INFO(1)=1
      CALL COMSTR(CJOB,CPRO,IPRO,CSYS,IFLAG,INFO)
      IF (IFLAG.NE.0) THEN
       CJOB = 'A,B,C,D,G'
       GO TO 100
      ENDIF
C3... Read system dimensions on A, B, C, and D  .................

C4..  Specify system dimension                  .................
      IF (CSYS(1)(1:1).EQ.CPRO(1)) THEN
200    CALL NREAD('Number of states        nx = ?',NX,1,1,IIII)
       IF (NX.LE.0) THEN
        CALL DIMERR('CONLAB','NX',NX,IERR)
        IF (IERR.EQ.1) GO TO 200
       ENDIF
      ENDIF

C5.. Specify input dimension                    .................
      IF (CSYS(2)(1:1).EQ.CPRO(2)) THEN
300    CALL NREAD('Number of inputs        nu = ?',NU,1,1,IIII)
       IF (NU.LE.0) THEN
        CALL DIMERR('CONLAB','NU',NU,IERR)
        IF (IERR.EQ.1) GO TO 300
       ENDIF
      ENDIF

C6.. Specify disturbance dimension              .................
      IF (CSYS(3)(1:1).EQ.CPRO(3)) THEN
400    CALL NREAD('Number of disturbances  nv = ?',NV,1,1,IIII)
       IF (NV.LE.0) THEN
        CALL DIMERR('CONLAB','NV',NV,IERR)
        IF (IERR.EQ.1) GO TO 400
       ENDIF
      ENDIF

C7.. Specify output (measurement) dimension     ................
      IF (CSYS(4)(1:1).EQ.CPRO(4)) THEN
500    CALL NREAD('Number of outputs       ny = ?',NY,1,1,IIII)
       IF (NY.LE.0) THEN
        CALL DIMERR('CONLAB','NY',NY,IERR)
        IF (IERR.EQ.1) GO TO 500
       ENDIF
      ENDIF

      ISVAR = IYESANS('Leave system definition modus ?',2)
      IF (ISVAR.EQ.1) GO TO 100

1000  RETURN
      END

      SUBROUTINE COHEAD(IWTE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE COHEAD                                       C
C   AUTHOR : David Di Ruscio         NTH/ITK   26 NOVEMBER 1988 C
C                                                               C
C   PURPOSE                                                     C
C   COHEAD is used for program description when the program     C
C   is started. COHEAD should print a nize text on terminal.    C
C                                                               C
C***END PROLOGUE COHEAD                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER IWTE
      CHARACTER CF*6

      CF = '(7X,A)'

      WRITE(IWTE,CF)
     +'---------------------------------------------------------------'
      WRITE(IWTE,CF)
     +'|                                                             |'
      WRITE(IWTE,CF)            
     +'|                   <   C  O  N  L  A  B   >                  |'
      WRITE(IWTE,CF)                                                   
     +'|                                                             |'
      WRITE(IWTE,CF)
     +'|            An interactive C O N trol L A B oratory          |'  
      WRITE(IWTE,CF)
     +'|                        program system                       |'  
      WRITE(IWTE,CF)                                                   
     +'|                                                             |'
      WRITE(IWTE,CF)
     +'|                   Version of october  1989                  |'
      WRITE(IWTE,CF)
     +'|                  by David Di Ruscio ITK/NTH                 |'
      WRITE(IWTE,CF)
     +'---------------------------------------------------------------'


      RETURN
      END

      SUBROUTINE DIMERR(NAME,CNAME,IDIM,IERR)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DIMERR                                      C
C                                                              C
C   PURPOSE                                                    C
C   DIMERR is used for dimension error cheqing. Error is       C
C   detected if the dimension IDIM .LE. 0.                     C
C                                                              C
C   ON INPUT                                                   C
C   NAME   Program or subprogram name whit 6 letters           C
C   CNAME  CHARACTER name of integer IDIM.                     C
C   IDIM   INTEGER specifying parameter                        C
C                                                              C
C   ON OUTPUT                                                  C
C   IERR   INTEGER Would be internally specifyed. IERR equal 2 C
C          would stop the program. FORTRAN STOP is used.       C
C          IERR = 1 an error message is signaled.              C
C                                                              C
C   ROUTINES CALLED : IYESANS   ()                             C
C                                                              C
C***END PROLOGUE DIMERR                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER IDIM,IERR
      CHARACTER CNAME*(*),CFORM*10,NAME*(*),TEXT*30


C1...Initialize format and question trext ......................
      CFORM = '(8X,A,A,A)'
      TEXT(1 :21) = 'Do you want to leave '
      TEXT(22:28) = NAME
      TEXT(28:30) = ' ?'

C2...Error handler                      ........................
      IF (IDIM.LE.0) THEN
       IERR = IYESANS(TEXT,1)
       IF (IERR.EQ.2) THEN
        STOP
       ELSEIF (IERR.EQ.1) THEN
        WRITE(*,CFORM)'* Specify ',CNAME,' > 0'
        GO TO 100
       ENDIF
      ENDIF

100   RETURN
      END

      SUBROUTINE MENU(IJOB,NJOB,IDEF)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE MENU                                         C
C                                                               C
C   PURPOSE                                                     C
C   MENU is used for listing of the utilities of                C
C   PROGRAM CONLAB on terminal.                                 C
C                                                               C
C   ON INPUT                                                    C
C   NJOB    INTEGER number of avaiable utilities declared in    C
C           the calling program                                 C
C   IJOB    INTEGER array of dimension (NJOB).                  C
C   IDEF    INTEGER number corresponding to default question.   C
C           Eks. IDEF = 3 on input. The default job to do is    C
C                IJOB(3)=3 ---> system matrix definitions       C
C   ON OUTPUT                                                   C
C   IJOB    Is set to the user specified job to be done.        C
C           IJOB(1)=1 --> read system from file                 C
C           IJOB(2)=2 --> system definitions.                   C
C             .             .                                   C
C                                                               C
C   ROUTINES CALLED : RCSTR  (read a character string. )        C
C***END PROLOGUE MENU                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER IJOB(NJOB),NJOB,IDEF,ICOM,IPOS
      PARAMETER ( ICOM = 19 )
      CHARACTER CFORM*6,TEXT*26,CJOB*8,COM(ICOM)*(6)

      DATA COM( 1)/'SREAD'/,  COM( 2)/'SDEF'/,   COM( 3)/'SMAT'/,
     +     COM( 4)/'OPTIM'/,  COM( 5)/'SVD'/,    COM( 6)/'STORE'/,
     +     COM( 7)/'TSDIS'/,  COM( 8)/'MATLAB'/, COM( 9)/'ESTIM'/,
     +     COM(10)/'TDSIM'/,  COM(11)/'USER'/,   COM(12)/'SAVE'/,
     +     COM(13)/'LOAD'/,   COM(14)/'LIST'/,   COM(15)/'HELP'/,
     +     COM(16)/'ZEROES'/, COM(17)/'MODOPT'/, COM(18)/'LYP'/
     +     COM(19)/'EXIT'/

C1...Initialisation                     .........................
      CFORM = '(8X,A)'
      DO 10 I = 1,NJOB
         IJOB(I) = 101
10    CONTINUE

C ...Initialize default                         .................
      IF (    IDEF.EQ.1) THEN
       CJOB = 'SREAD '
      ELSEIF (IDEF.EQ.2) THEN
       CJOB = 'SDEF  '
      ELSEIF (IDEF.EQ.3) THEN
       CJOB = 'SMAT  '
      ELSEIF (IDEF.EQ.4) THEN
       CJOB = 'OPTIM '
      ELSEIF (IDEF.EQ.5) THEN
       CJOB = 'SVD   '
      ELSEIF (IDEF.EQ.6) THEN
       CJOB = 'STORE '
      ELSEIF (IDEF.EQ.7) THEN
       CJOB = 'TSDIS '
      ELSEIF (IDEF.EQ.0) THEN
       CJOB = 'EXIT  '
      ELSE
       CJOB = 'SDEF  '
      ENDIF

C3... Read CONLAB command operation from terminal         .......
      WRITE(*,CFORM)'                                           '
      TEXT = 'Main CONLAB operation :'
100   CALL RCSTR(TEXT,CJOB,1,1,IIII)
      WRITE(*,CFORM)'                                           '

C4... Extract a IJOB number from a number command CJOB    .......
      NUM = ICNUM(CJOB)
      IF (NUM.EQ.0) THEN
       IJOB(NJOB) = 0
       GO TO 1000
      ELSEIF (NUM.GE.1.AND.NUM.LE.NJOB-1) THEN
       IJOB(NUM) = NUM
       GO TO 1000
      ENDIF

C5... Extraxt IJOB number from a character command CJOB   .......
      CALL COMCHK(CJOB,COM,ICOM,IFLAG,IPOS)
      IF (IFLAG.EQ.0) THEN
       IF (IPOS.EQ.NJOB) THEN
        IJOB(IPOS) = 0
       ELSE
        IJOB(IPOS) = IPOS
       ENDIF
       GO TO 1000
      ELSEIF (IFLAG.EQ.1) THEN
       WRITE(*,'(5X,A,1X,A)')'ERROR SYMBOL OR COMMAND :',CJOB
       CALL COMLIS('Avaiable is :',COM,ICOM)
       CJOB = 'SDEF'
       GO TO 100
      ENDIF

1000  RETURN
      END

      SUBROUTINE ERRCHK(ICHK,IERR,NX,NU,NV,NY,CSYS)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE ERRCHK                                       C
C                                                               C
C                                                               C
C                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER ICHK,IERR,NX,NU,NV,NY
      CHARACTER CFORM*6,CSYS(*)*(*)

C1... Initialize
      IERR = 0
      CFORM = '(5X,A)'

C1...Test dimensions consistency whit CJOB   ...................
      IF (ICHK.EQ.1.AND.CSYS(1).EQ.'A') THEN
       IF (NX.LE.0) THEN
        IERR = 1
        WRITE(*,CFORM)'* Dimensions out of bounds. NX < 0       '
        GO TO 100
       ENDIF
      ENDIF

      IF (ICHK.EQ.1.AND.CSYS(2).EQ.'B') THEN
       IF (NU.LE.0) THEN
        IERR = 2
        WRITE(*,CFORM)'* Dimension out of bound.  NU < 0'
        GO TO 100
       ENDIF
      ENDIF

100   RETURN
      END

      SUBROUTINE OPTFEED(B,P,H,W3,PW,BP,NX,NU,NA,NB,IERR)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE OPTFEED                                      C
C                                                               C
C   PURPOSE                             -1 T                    C
C   OPTFEED computes the product H = B P  B                     C
C   ON INPUT                                                    C
C   B      DOUBLE PRECISION (NX,NU)                             C
C   P      DOUBLE PRECISION (NU,NU)                             C
C   PW     DOUBLE PRECISION (NU,NU) used as work array.         C
C   BP     DOUBLE PRECISION (NU,NU) used as work array.         C
C   W3     DOUBLE PRECISION (NB) used as work array.            C
C   ON OUTPUT                                                   C
C   H      DOUBLE PRECISION (NX,NX) contains the result.        C
C                                                               C
C   ROUTINES CALLED : INVERS,MATMULT,TRANSMULT                  C
C***END PROLOGUE OPTFEED                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NA,NB                 
      DOUBLE PRECISION B(NB,NU),P(NB,NU)
      DOUBLE PRECISION H(NA,NX),PW(NB,NU),W3(NB),DET
      DOUBLE PRECISION BP(NA,NU)

C1... Compute PW = inv(P)                               .........
      CALL INVERS(P,PW,W3,DET,NU,NB)

C2... Compute BP = B*inv(P)     dim B = (NX x NU)       .........
      CALL MATMULT(B,PW,BP,NX,NU,NU,NA)

C3... Compute H = B*inv(P)*transp(B)                    .........
      CALL TRANSMULT(BP,B,H,NX,NU,NX,NA,1)

      RETURN
      END

      SUBROUTINE PQINP(Q,P,A,AA,ALFA,WV,W1,W2,NX,NU,NA,NB,IFLAG)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE PQINP                                        C
C                                                               C
C   PURPOSE                                                     C
C   PQINP is used to read the weight matrixes P and Q from      C
C   terminal. Symmetri and positive definitness is cheked.      C
C                                                               C
C                                                               C
C   ROUTINES CALLED : PQMAT,IYESANS,RREAD                       C
C                                                               C
C***END PROLOGUE PQINP                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NA,NB,IFLAG,ISVAR
      DOUBLE PRECISION Q(NA,NX),P(NB,NU),A(NA,NX),AA(NA,NX),ALFA
      DOUBLE PRECISION WV(NA)
      DOUBLE PRECISION W1(NA,NA),W2(NA),RALF
      CHARACTER CFORM*6

C1..  Initialize                        .........................
      CFORM = '(5X,A)'                                        
      WRITE(*,CFORM)'                                           '
      WRITE(*,CFORM)'-------------------------------------------'
      WRITE(*,CFORM)'             < O P T I M >                 '
      WRITE(*,CFORM)'        OPTIMAL FEEDBACK MODUS             '
      WRITE(*,CFORM)'-------------------------------------------'

C2..  Specify weight matrixes Q and P   .........................
      ISVAR = IYESANS('Specify weight matrixes ?',2)
      IF (ISVAR.EQ.2) THEN
       CALL PQMAT(Q,P,WV,W1,W2,NX,NU,NA,NB,'QP')
      ENDIF

C3... Specified degree of stability             .................
      ISVAR = IYESANS('Specified degree of stability ?',1)
      IF (ISVAR.EQ.2) THEN
       RALF = ALFA
300    CALL DREAD('Degree of stability (alfa) ?',RALF,1,1,IIII)
       ALFA = RALF
      ELSE
       ALFA = 0.0D0
      ENDIF

C3... Correction for specified degree of stability     ..........
      DO 10 I=1,NX
         DO 20 J=1,NX
            IF (I.EQ.J) THEN
             AA(I,J) = A(I,J) + ALFA
            ELSE
             AA(I,J) = A(I,J)
            ENDIF
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE GMAT(G,PI,B,R,W,NX,NU,NA,NB)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE GMAT                                         C
C   PURPOSE                                                     C
C   GMAT computes the optimal feedback G = -inv(P)*tr(B)*R      C
C                                                               C
C***END PROLOGUE GMAT                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NA,NB,I,J
      DOUBLE PRECISION G(NB,NX),PI(NB,NU),B(NB,NU),R(NA,NX),W(NB,NX)

C1... Compute feedback G = -inv(P)*tr(B)*R      .................
      CALL TRANSMULT(PI,B,W,NU,NU,NX,NA,1)
      CALL MATMULT(W,R,G,NU,NX,NX,NA)
      DO 10 I=1,NU
         DO 20 J=1,NX
            G(I,J) = - G(I,J)
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE OPTLIS(G,R,ER,EI,FNORM,DJ,NX,NU,NA,NB)
C---------------------------------------------------------------C
C***BEGIN PROLOGUE OPTLIS                                       C
C   AUTHOR : David Di Ruscio            ITK/NTH DECEMBER 1988   C
C                                                               C
C   PURPOSE                                                     C
C   OPTLIS list results from the OPtimal feedback modus.        C
C                                                               C
C   ON INPUT                                                    C
C   G,R,ER,EI  DOUBLE PRECISION (NB,NX),(NA,NX),(*),(*)         C
C              respectively. Unchanged on output.               C
C   FNORM      DOUBLE PRECISION scalar containing the Frobenius C
C              norm of A+B*G                                    C
C   DJ         DOUBLE PRECISION scalar containing the           C
C              performance criterion value, trace( R ) = DJ     C
C   ROUTINES CALLED : MATPRT,RCSTR                              C
C***END PROLOGUE OPTLIS                                         C
C---------------------------------------------------------------C
      INTEGER NX,NU,NA,NB
      DOUBLE PRECISION G(NB,NX),R(NA,NX),ER(*),EI(*),FNORM,DJ
      CHARACTER CMAT*5,CFORM*6

C1...Initialization                     .........................
      CFORM = '(5X,A)'
      CMAT  = 'G  '

C2... Specify what to be listed on terminal     .................
100   CALL RCSTR('List results ',CMAT,1,1,IIII)

C3... Exequte specified jobs on terminal        .................
      IF (CMAT.EQ.'G') THEN
       CALL MATPRT('*  Optimal feedback matrix G =',G,NB,NA,NU,NX)
       CMAT = 'EIg'
       GO TO 100

      ELSEIF (CMAT.EQ.'R') THEN
       CALL MATPRT('*  Solution of Riccati eq. R =',R,NA,NA,NX,NX)
       CMAT = 'No '
       GO TO 100

      ELSEIF (CMAT(1:1).EQ.'E'.AND.CMAT(2:2).EQ.'I') THEN
       WRITE(*,CFORM)'*  Closed loop eigenvalues of (A + B*G) :'
       WRITE(*,1) (I,ER(I),EI(I),I=1,NX)
1      FORMAT(5X,'s(',I3,') = ',1X,D12.5,1X,'+ i',1X,D12.5)
       CMAT = 'FN '
       GO TO 100

      ELSEIF (CMAT(1:1).EQ.'F'.AND.CMAT(2:2).EQ.'N') THEN
       WRITE(*,CFORM)'*  Frobenius norm of closed loop matrix : '
       WRITE(*,2) FNORM
2      FORMAT(8X,'Fnorm( A+B*G ) = ',G16.9)
       CMAT = 'J  '
       GO TO 100

      ELSEIF (CMAT.EQ.'J') THEN
       WRITE(*,CFORM)'*  Performance criterion value          : '
       WRITE(*,3) DJ
3      FORMAT(8X,'J = trace( R ) = ',G16.9)
       CMAT = 'R  '
       GO TO 100

      ELSEIF (CMAT.EQ.'0') THEN
       GO TO 1000
      ELSEIF(CMAT(1:1).EQ.'N') THEN
       GO TO 1000
      ELSE
       WRITE(*,CFORM)'*       Avaiable are :                    '
       WRITE(*,CFORM)'*  G       - Feedback matrix              '
       WRITE(*,CFORM)'*  EIg     - Eigenvalues of     (A + B*G) '
       WRITE(*,CFORM)'*  FNo     - Frobenius norm of  (A + B*G) '
       WRITE(*,CFORM)'*  J       - Performance criterion value  '
       WRITE(*,CFORM)'*  R       - Solution of Riccati equation '
       WRITE(*,CFORM)'*  0 or No - Exit                         '
       CMAT = 'No '
       GO TO 100
      ENDIF

1000  RETURN
      END

      SUBROUTINE CONERR(IERR,IWTE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE CONERR                                       C
C                                                               C
C   PURPOSE                                                     C
C   ERROR give error information on terminal.                   C
C   ON INPUT                                                    C
C   IERR     INTEGER error number.                              C
C   IWTE     INTEGER terminal logical number.                   C
C***END PROLOGUE CONERR                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER IERR,IWTE
      CHARACTER CFORM*6

C1.. Initialize format statements               .................
      CFORM = '(5X,A)'

C2...Error handler                              .................
      IF (IERR.EQ.1) THEN
       WRITE(IWTE,CFORM)'* System dimensions not defined '
       GO TO 100
      ENDIF

100   RETURN
      END

      SUBROUTINE EIGCOR(ER,NX,ALFA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE EIGCOR                                        C
C                                                                C
C   PURPOSE                                                      C
C   Performs vector subtraction  ER = ER - alfa                  C
C                                                                C
C***END PROLOGUE EIGCOR                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX
      DOUBLE PRECISION ER(*),ALFA

      DO 10 I=1,NX
         ER(I) = ER(I) - ALFA
10    CONTINUE

      RETURN
      END

      SUBROUTINE SVDUTL(ISVD,IWTE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE SVDUTL                                       C
C                                     ITK/NTH 19 december 1988  C
C   PURPOSE                                                     C
C   List menu of Multivariable Frequensy Response analysis      C
C   utilities and choose SVD-job.                               C
C                                                               C
C   ROUTINES CALLED   :                                         C
C***END PROLOGUE SVDUTL                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER ISVD,IWTE
      CHARACTER CF*6

      CF   = '(5X,A)'

      WRITE(IWTE,CF)'-------------------------------------------'
      WRITE(IWTE,CF)'  Singular Value Analysis utilities        '
      WRITE(IWTE,CF)'                                           '
      WRITE(IWTE,CF)'                      -1                   '
      WRITE(IWTE,CF)' 1.  Hu(jw) = D(jw - A)  B                 '
      WRITE(IWTE,CF)'                           -1              '
      WRITE(IWTE,CF)' 2.  Hyv(jw) = D(jw - (A+BG))  C           '
      WRITE(IWTE,CF)'                                           '
      WRITE(IWTE,CF)' 3.  HS(jw) = I + H0(jw), H0 = Hu Hc       '
      WRITE(IWTE,CF)'                                           '
      WRITE(IWTE,CF)'-------------------------------------------'

C... Choose job                                         .........
100   CALL NREAD('Chose job ',ISVD,1,1,IIII)

      IF (ISVD.LT.0) THEN
       WRITE(IWTE,CF)'---> Chose number > 0'
       ISVD=1
       GO TO 100
      ENDIF
      IF (ISVD.EQ.0) GO TO 1000

1000  RETURN
      END

      SUBROUTINE SVDMAT(A,B,C,D,G,E,H,PW,WP,NA,NB,NC,ND,NX,NU,NV,
     +                    NY,ISVD)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE SVDMAT                                       C
C   PURPOSE                                                     C
C   SVDMAT is used to define the matrixes H, PW, WP and E       C
C   used in the computation of HS = WP*((jw-H)inv)*PW+E as      C
C   a function of A, B, C, D, E, and G.                         C
C                                                               C
C   ON INPUT                                                    C
C   A, B, C, D, E, and G   system matrixes                      C
C   ISVD  INTEGER whit the following options,                   C
C    ISVD=1    HS = D(jw-A)inv*B                                C
C    ISVD=2    HS = D(jw-(A+BG))inv*C                           C
C                                                               C
C   ROUTINES CALLED   : MATCOP,CLABG                            C
C***END PROLOGUE SVDMAT                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NA,NB,NC,ND,NX,NU,NV,NY,ISVD
      DOUBLE PRECISION A(NA,NA),B(NB,NB),C(NC,NC),D(ND,ND)
      DOUBLE PRECISION G(NB,NA),E(ND,ND),H(NA,NA),PW(NB,NB)
      DOUBLE PRECISION WP(NA,NB)

C1.. Job is H(jw) = D*(jw-A)inv*B                       ..........
      IF (ISVD.EQ.1) THEN
       CALL MATCOP(A,H,NX,NX,NA,NA)
       CALL MATCOP(B,PW,NX,NU,NA,NA)
       CALL MATCOP(D,WP,NY,NX,NA,NA)

C... Job is H(jw) = D*(jw-(A+B*G))inv*C                ..........
      ELSEIF (ISVD.EQ.2) THEN
       CALL CLABG(H,A,B,G,NX,NU,NA,NB)
       CALL MATCOP(C,PW,NX,NV,NA,NA)
       CALL MATCOP(D,WP,NY,NX,NA,NA)

C... Job is H(jw) = I + H0(jw)                         ..........
      ELSEIF (ISVD.EQ.3) THEN
       CALL MATCOP(A,H,NX,NX,NA,NA)
       CALL MATMULT(B,G,PW,NX,NU,NX,NA)
       CALL IDMATR(WP,NX,NA)
       CALL IDMATR(E,NY,NA)
      ENDIF

      RETURN
      END

      SUBROUTINE MATCAL(A,B,C,D,G,R,P,Q,WY,VV,XA,KA,NX,NU,NV,NY,        
     +                  NA,NB,NC,ND,IERR)
C----------------------------------------------------------------C
C***PURPOSE                                                      C
C   This is a interface routine to the matrix mathematic         C
C   modus MATLAB.                                                C
C                                                                C
C   ROUTINES CALLED : MATLAB,MATZ                                C
C----------------------------------------------------------------C
      INTEGER NX,NU,NV,NY,NA,NB,NC,ND,IERR
      DOUBLE PRECISION A(NA,NA),B(NB,NB),C(NC,NC),D(ND,ND),G(NB,NA)
      DOUBLE PRECISION R(NA,NA),P(NB,NB),Q(NA,NA),KA(NA,ND)
      DOUBLE PRECISION WY(ND,ND),VV(NC,NC),XA(NA,NA)
      INTEGER NDUMX,NDUMU,NDUMV,NDUMY
      CHARACTER CFORM*6

      INTEGER IDA(4),IDB(4),IDC(4),IDD(4),IDG(4),IDR(4),IDK(4)
      INTEGER IDP(4),IDQ(4),IDW(4),IDV(4),IDX(4),JOB
      DATA IDA/'A',' ',' ',' '/, IDB/'B',' ',' ',' '/
      DATA IDC/'C',' ',' ',' '/, IDD/'D',' ',' ',' '/
      DATA IDG/'G',' ',' ',' '/, IDK/'K',' ',' ',' '/
      DATA IDP/'P',' ',' ',' '/, IDQ/'Q',' ',' ',' '/
      DATA IDR/'R',' ',' ',' '/, IDW/'W',' ',' ',' '/          
      DATA IDV/'V',' ',' ',' '/, IDX/'X',' ',' ',' '/

C1... Initialization                            .................
      CFORM = '(5X,A)'

C2... Produce MATLAB gretings and wait for input ................
      CALL MATLAB(0)

C3... Put matrixes into MATLAB   JOB = 1         ................
      JOB = 1
      CALL MATZ(A,NA,NX,NX,IDA,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(B,NB,NX,NU,IDB,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(C,NC,NX,NV,IDC,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(D,ND,NY,NX,IDD,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(G,NB,NU,NX,IDG,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(KA,NA,NX,NY,IDK,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(P,NB,NU,NU,IDP,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(Q,NA,NX,NX,IDQ,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(R,NA,NX,NX,IDR,JOB,IERR)
      IF (IERR.NE.0) GO TO 90

      CALL MATZ(WY,ND,NY,NY,IDW,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(VV,NC,NV,NV,IDV,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(XA,NA,NX,NX,IDX,JOB,IERR)
      IF (IERR.NE.0) GO TO 90

      CALL MATLAB(1)

C4... Get matrixes from MATLAB JOB = 0          ................
      JOB = 0
      CALL MATZ(A,NA,NX,NX,IDA,JOB,IERR)
      NDUMX = NX
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(B,NB,NX,NU,IDB,JOB,IERR)
      NDUMU = NU
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(C,NC,NX,NV,IDC,JOB,IERR)
      NDUMV = NV
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(D,ND,NY,NX,IDD,JOB,IERR)
      NDUMY = NY
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(G,NB,NU,NX,IDG,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(KA,NA,NX,NY,IDK,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(P,NB,NU,NU,IDP,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(Q,NA,NX,NX,IDQ,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(R,NA,NX,NX,IDR,JOB,IERR)
      IF (IERR.NE.0) GO TO 90

      CALL MATZ(WY,ND,NY,NY,IDW,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(VV,NC,NV,NV,IDV,JOB,IERR)
      IF (IERR.NE.0) GO TO 90
      CALL MATZ(XA,NA,NX,NX,IDX,JOB,IERR)
      IF (IERR.NE.0) GO TO 90

      NX = NDUMX
      NU = NDUMU
      NV = NDUMV
      NY = NDUMY

      GO TO 100
90    WRITE(*,CFORM)'*---> ERROR IN CALL TO MATZ'
      WRITE(*,CFORM)'*---> Error in comunication with MATLAB'

100   RETURN
      END

      SUBROUTINE IDMATR(A,N,NA)
C---------------------------------------------------------------C
C***PURPOSE                                                     C
C   Make the identity matrix A = I                              C
C                                                               C
C---------------------------------------------------------------C
      INTEGER N,NA,I,J
      DOUBLE PRECISION A(NA,N)

      DO 10 I=1,N
         DO 20 J=1,N
            IF (I.EQ.J) THEN
             A(I,J)=1.0
            ELSE
             A(I,J)=0.0
            ENDIF
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE WVINP(WY,VV,WV,W1,W2,NY,NV,ND,NC,IFLAG)
C---------------------------------------------------------------C
C***BEGIN PROLOGUE WVINP                                        C
C   PURPOSE                                                     C
C   WVINP is used to read the weight matrixes W and V from      C
C   terminal.                                                   C
C   ROUTINES CALLED : WVMAT,IYESANS                             C
C***END PROLOGUE WVINP                                          C
C---------------------------------------------------------------C
      INTEGER NY,NV,ND,NC,ISVAR
      DOUBLE PRECISION WY(ND,NY),VV(NC,NV),WV(NC)
      DOUBLE PRECISION W1(ND,ND),W2(ND)
      CHARACTER CFORM*6

C1... Initialize                                .................
      CFORM = '(5X,A)'
      WRITE(*,CFORM)'                                           '
      WRITE(*,CFORM)'-------------------------------------------'
      WRITE(*,CFORM)'             < E S T I M >                 '
      WRITE(*,CFORM)'        OPTIMAL ESTIMATOR MODUS            '
      WRITE(*,CFORM)'       COMPUTE KALMAN FILTER GAIN          '
      WRITE(*,CFORM)'-------------------------------------------'

C2... Specify weight matrixes WY = W and VV = V         .........
      ISVAR = IYESANS('Specify weight matrixes ?',2)
      IF (ISVAR.EQ.2) THEN
       CALL WVMAT(VV,WY,WV,W1,W2,NV,NY,NC,ND,'VW')
      ENDIF

      RETURN
      END

      SUBROUTINE ESTLIS(XA,KA,ER,EI,DJ2,NX,NY,NA,ND)
C----------------------------------------------------------------C
C***BEGIN PROLOGUE ESTLIS                                        C
C                                                                C
C   PURPOSE                                                      C
C   ESTLIS is used to list results from the Optimal Estimator    C
C   Modus - ESTIM.                                               C
C                                                                C
C   ROUTINES CALLED : MATPRT,RCSTR                               C
C***END PROLOGUE ESTLIS                                          C
C----------------------------------------------------------------C
      INTEGER NX,NA,NY,ND
      DOUBLE PRECISION XA(NA,NX),KA(NA,NY),ER(*),EI(*),DJ2
      CHARACTER CMAT*4,CFORM*6

C1... Initialization                                    ..........
      CFORM = '(5X,A)'
      CMAT  = 'K  '

C2... Specify what to be listed                         ..........
100   CALL RCSTR('List results ',CMAT,1,1,IIII)

C3... Execute specified jobs on terminal                ..........
      IF (CMAT.EQ.'X') THEN
       CALL MATPRT('* Cov. error matrix  X =',XA,NA,NA,NX,NX)
       CMAT = 'No '
       GO TO 100
      ELSEIF (CMAT.EQ.'K') THEN
       CALL MATPRT('* Kalman filter gain K =',KA,NA,ND,NX,NY)
       CMAT = 'EIg'
       GO TO 100
      ELSEIF (CMAT(1:1).EQ.'E'.AND.CMAT(2:2).EQ.'I') THEN
       WRITE(*,CFORM)'*  Eigenvalues of (A - KD)     :'
       WRITE(*,1) (I,ER(I),EI(I),I=1,NX)
1      FORMAT(5X,'s(',I3,') = ',1X,D12.5,1X,'+ i',1X,D12.5)
       CMAT = 'J  '
       GO TO 100
      ELSEIF (CMAT.EQ.'J') THEN
       WRITE(*,CFORM)'*  Performance kriterion value :'
       WRITE(*,2) DJ2
2      FORMAT(8X,'J = trace( X ) = ',G16.9)
       CMAT = 'X  '
       GO TO 100
      ELSEIF (CMAT.EQ.'0') THEN
       GO TO 1000
      ELSEIF (CMAT(1:1).EQ.'N') THEN
       GO TO 1000
      ELSE
       WRITE(*,CFORM)'*     Avaiable are :                      '
       WRITE(*,CFORM)'*  K       -  Kalman filter gain          '
       WRITE(*,CFORM)'*  X       -  Covariance error matrix     '
       WRITE(*,CFORM)'*  Eig     -  Eigenvalues of (A - K*D)    '
       WRITE(*,CFORM)'*  J       -  trace( X )                  '
       WRITE(*,CFORM)'*  0 or No -  Exit                        '
       CMAT = 'No '
       GO TO 100
      ENDIF

1000  RETURN
      END

      SUBROUTINE TRANSP(A,B,N,M,NA,NB)
C----------------------------------------------------------------C
C***BEGIN PROLOGUE TRANSP                                        C
C   PURPOSE                               T                      C
C   TRANSP performs matrix transpose B = A . A is unchanged on   C
C   output.                                                      C
C   ROUTINES CALLED : (none)                                     C
C***END PROLOGUE TRANSP                                          C
C----------------------------------------------------------------C
      INTEGER N,M,NA,NB
      DOUBLE PRECISION A(NA,M),B(NB,N)

      DO 10 I=1,M
         DO 20 J=1,N
            B(I,J) = A(J,I)
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE TDINIT(IWTE)
C---------------------------------------------------------------C
C***BEGIN PROLOGUE TDINIT                                       C
C   PURPOSE                                                     C
C   TDINIT is used for initialization for the TDSIM modus.      C
C                                                               C
C***END PROLOGUE TDINIT                                         C
C---------------------------------------------------------------C
      INTEGER IWTE
      CHARACTER CFORM*6
      CFORM = '(5X,A)'

      WRITE(IWTE,CFORM)'----------------------------------------'
      WRITE(IWTE,CFORM)'           < T D S I M >                '
      WRITE(IWTE,CFORM)'       Time Domain Simulation           '
      WRITE(IWTE,CFORM)'----------------------------------------'

      RETURN
      END

      SUBROUTINE COMNUM(CJOB,IJOB,NJOB)
      INTEGER IJOB(NJOB)
      CHARACTER CJOB*(*)

      IF (CJOB.EQ. '1') IJOB( 1) = 1
      IF (CJOB.EQ. '2') IJOB( 2) = 2
      IF (CJOB.EQ. '3') IJOB( 3) = 3
      IF (CJOB.EQ. '4') IJOB( 4) = 4
      IF (CJOB.EQ. '5') IJOB( 5) = 5
      IF (CJOB.EQ. '6') IJOB( 6) = 6
      IF (CJOB.EQ. '7') IJOB( 7) = 7
      IF (CJOB.EQ. '8') IJOB( 8) = 8
      IF (CJOB.EQ. '9') IJOB( 9) = 9
      IF (CJOB.EQ.'10') IJOB(10) =10
      IF (CJOB.EQ.'11') IJOB(11) =11
      IF (CJOB.EQ.'12') IJOB(12) =12
      IF (CJOB.EQ.'13') IJOB(13) =13
      IF (CJOB.EQ.'14') IJOB(14) =14
      IF (CJOB.EQ.'15') IJOB(15) =15
      IF (CJOB.EQ. '0') IJOB(16) = 0

      RETURN
      END

      SUBROUTINE CONHLP(IWTE)
C---------------------------------------------------------------C
C***BEGIN PROLOGUE HELP                                         C
C   PURPOSE                                                     C
C   HELP is used to give program information on terminal.       C
C***END PROLOGUE HELP                                           C
C---------------------------------------------------------------C
      INTEGER IWTE
      CHARACTER CFORM*6

      CFORM = '(8X,A)'

C... CONLAB help                        .........................
      WRITE(IWTE,CFORM)'                                             '
      WRITE(IWTE,CFORM)'         < CONLAB UTILITIES >                '
      WRITE(IWTE,CFORM)'----------------------------------------------'
      WRITE(IWTE,CFORM)' 1.  Read system from file              SREAD '
      WRITE(IWTE,CFORM)' 2.  System definitions                 SDEF  '
      WRITE(IWTE,CFORM)' 3.  System matrix definitions          SMAT  '
      WRITE(IWTE,CFORM)' 4.  Optimal feedback syntesis          OPTIM '
      WRITE(IWTE,CFORM)' 5.  Singular Value Analysis            SVD   '
      WRITE(IWTE,CFORM)' 6.  Store system on file               STORE '
      WRITE(IWTE,CFORM)' 7.  Display a TS - file                TSDIS '
      WRITE(IWTE,CFORM)' 8.  Matrix Calculations                MATLAB'
      WRITE(IWTE,CFORM)' 9.  Optimal Estimator Syntesis         ESTIM '
      WRITE(IWTE,CFORM)'10.  Time Domain Simulation             TDSIM '
      WRITE(IWTE,CFORM)'11.  CALL USER                          USER  '
      WRITE(IWTE,CFORM)'12.  Save matrix on file                SAVE  '
      WRITE(IWTE,CFORM)'13.  Load matrix from file              LOAD  '
      WRITE(IWTE,CFORM)'14.  List matrix on terminal            LIST  '
      WRITE(IWTE,CFORM)'15.  Help                               HELP  '
      WRITE(IWTE,CFORM)'16.  Zeroes computation                 ZEROES'
      WRITE(IWTE,CFORM)'17.  Modal optimal controller design    MODOPT'
      WRITE(IWTE,CFORM)'18.  Solve Liapunov matrix equation     LYP   '
      WRITE(IWTE,CFORM)' 0.  Leave CONLAB                       EXIT  '
      WRITE(IWTE,CFORM)'----------------------------------------------'
CUT   WRITE(IWTE,CFORM)' - Choose either a number or a name from the  '
CUT   WRITE(IWTE,CFORM)'   above menu as a CONLAB command.            '
CUT   WRITE(IWTE,CFORM)'                                              '

      RETURN
      END

      SUBROUTINE ZERCMP(A,B,NX,NU,NA,NB)
      INTEGER NA,NB,NX,NU
      DOUBLE PRECISION A(NA,NA),B(NA,NA)

      WRITE(*,'(5X,A)')'Solve the generalized eigenvalue problem  '
      WRITE(*,'(5X,A)')'                                          '
      WRITE(*,'(5X,A)')'( A B )(x)          ( I 0 )               '
      WRITE(*,'(5X,A)')'( D E )(u) = lambda ( 0 0 )               '
      WRITE(*,'(5X,A)')'                                          '
      WRITE(*,'(5X,A)')'ZEROES not yet implemented                '
      WRITE(*,'(5X,A)')'Use matlab function GEIG(MAT1,MAT2)       '

      END

      SUBROUTINE DLYPSOL(A,Q,R,NA,NX,WZ,WV,NXX,IND,INFO)
      INTEGER NA,NX,NXX,INFO
      INTEGER IND(NXX)
      DOUBLE PRECISION A(NA,NA),Q(NA,NA),R(NA,NA)
      DOUBLE PRECISION WZ(NXX,NXX),WV(NXX)

      WRITE(*,'(5X,A)')'        T             '
      WRITE(*,'(5X,A)')'Solve  A R + R A = - Q'

      CALL DLYP(A,Q,R,NA,NX,WZ,WV,NXX,IND,INFO)

      IF (INFO.EQ.0) THEN
        CALL MATPRT('* Solution R =',R,NA,NA,NX,NX)
      ELSE
        WRITE(*,'(5X,A)')'System singular'
      ENDIF

      RETURN
      END
