      SUBROUTINE SYSMAT2(A,B,C,D,G,KA,P,Q,R,XA,WORK,NX,NU,NV,NY,
     +                  NA,NB,NC,ND,CSYS,ISYS,IERR)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DSYSMAT                                      C
C   AUTHOR  David Di Ruscio           NTH/ITK 25 november 1988  C
C                                                               C
C   PURPOSE                                                     C
C   DSYSMAT could be used to read the total system matrixes     C
C   A, B, C, and D or only A and B etc.                         C
C   ON INPUT                                                    C
C       CSYS is an CHARACTER(*)*(*) array of legal matrix names.C
C       ISYS Size of CSYS array.                                C
C        NX  integer                                            C
C            the number of the states.                          C
C        NU  integer                                            C
C            the number of inputs.                              C
C        NV  integer                                            C
C            the number of disturbances.                        C
C        NY  integer                                            C
C            the number of outputs (measurements).              C
C        NA  integer                                            C
C            the leading or row dimension of the real array A   C
C            as declared in the main calling program.           C
C        NB  integer                                            C
C            the leading or row dimension of the real array B   C
C            as declared in the main calling program.           C
C        NC  integer                                            C
C            the leading or row dimension of the real array C   C
C            as declared in the main calling program.           C
C        ND  integer                                            C
C            the leading or row dimension of the real array D   C
C            as declared in the main calling program.           C
C                                                               C
C        A   double precision(na,na)                            C
C            a real nx x nx matrix (the system matrix).         C
C        B   double precision(nb,nb)                            C
C            a real nx x nu matrix (the input matrix).          C
C        C   double precision(nc,nc)                            C
C            a real nx x nv matrix (the disturbance matrix).    C
C        D   double precision(nd,nd)                            C
C            a real ny x nx matrix (the output measurement mat.)C
C        WORK double precision(na)                              C
C            used as work space.                                C
C                                                               C
C   ON OUTPUT                                                   C
C        A, B, C, G, D and KA                                   C
C        IERR = 0 for successfully operations                   C
C        IERR = 1 dimensions out of bounds  (.LE.0)             C
C                                                               C
C   ROUTINES CALLED    : DREMA                                  C
C***END PROLOGUE DSYSMAT                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NV,NY,NA,NB,NC,ND,IERR,ISVAR,NSI,I,ISYS
      DOUBLE PRECISION A(NA,NA),B(NB,NB),C(NC,NC),D(ND,ND),G(NB,NA)
      DOUBLE PRECISION KA(NA,ND),P(NB,NB),Q(NA,NA),WORK(NA)
      DOUBLE PRECISION R(NA,NA),XA(NA,NA)
      CHARACTER CSYS(*)*(*),CFORM*6,CMAT*10,CANEXT*4

C1..  Format initialitation                                 .....
      CFORM = '(5X,A)'
      IERR  = 0

C2..  First, find the most probability matrix to start whit .....
      CMAT = CANEXT(CSYS,1,ISYS)
      IF (CMAT.EQ.' ') CMAT = 'No'

C3.. Second - ask user for matrix name and execute the job  .....
90    CALL RCSTR('Edit matrix ',CMAT,1,1,IIII)

C4.. Find next matrix to set as default on return to 90     .....
      IF     (CMAT.EQ.CSYS(1)) THEN
       IF (ISYS.GE.2) THEN
        CMAT = CANEXT(CSYS,2,ISYS)
        IF (CMAT.NE.' ') GO TO 100
       ENDIF
       CMAT = 'No'
       GO TO 100
      ELSEIF (CMAT.EQ.CSYS(2)) THEN
       IF (ISYS.GE.3) THEN
        CMAT = CANEXT(CSYS,3,ISYS)
        IF (CMAT.NE.' ') GO TO 200
       ENDIF
       CMAT='No'
       GO TO 200
      ELSEIF (CMAT.EQ.CSYS(3)) THEN
       IF (ISYS.GE.4) THEN
        CMAT = CANEXT(CSYS,4,ISYS)
        IF (CMAT.NE.' ') GO TO 300
       ENDIF
       CMAT = 'No'
       GO TO 300
      ELSEIF (CMAT.EQ.CSYS(4)) THEN
       IF (ISYS.GE.5) THEN
        CMAT = CANEXT(CSYS,5,ISYS)
        IF (CMAT.NE.' ') GO TO 400
       ENDIF
       CMAT='No'
       GO TO 400
      ELSEIF (CMAT.EQ.CSYS(5)) THEN
       IF (ISYS.GE.6) THEN
        CMAT = CANEXT(CSYS,6,ISYS)
        IF (CMAT.NE.' ') GO TO 500
       ENDIF
       CMAT='No'
       GO TO 500
      ELSEIF (CMAT.EQ.CSYS(6)) THEN
       IF (ISYS.GE.7) THEN
        CMAT = CANEXT(CSYS,7,ISYS)
        IF (CMAT.NE.' ') GO TO 600
       ENDIF
       CMAT='No'
       GO TO 600
      ELSEIF (CMAT.EQ.CSYS(7)) THEN
       IF (ISYS.GE.8) THEN
        CMAT = CANEXT(CSYS,8,ISYS)
        IF (CMAT.NE.' ') GO TO 700
       ENDIF
       CMAT='No'
       GO TO 700
      ELSEIF (CMAT.EQ.CSYS(8)) THEN
       IF (ISYS.GE.9) THEN
        CMAT = CANEXT(CSYS,9,ISYS)
        IF (CMAT.NE.' ') GO TO 800
       ENDIF
       CMAT='No'
       GO TO 800
      ELSEIF (CMAT.EQ.CSYS(9)) THEN
       IF (ISYS.GE.10) THEN
        CMAT = CANEXT(CSYS,10,ISYS)
        IF (CMAT.NE.' ') GO TO 900
       ENDIF
       CMAT='No'
       GO TO 900
      ELSEIF (CMAT.EQ.CSYS(10)) THEN
       CMAT='No'
       GO TO 1000
      ELSEIF (CMAT.EQ.'N'.OR.CMAT.EQ.'No'.OR.CMAT.EQ.'0') THEN
       GO TO 2000
      ELSEIF (CMAT.EQ.'NO'.OR.CMAT.EQ.'EXIT') THEN
       GO TO 2000
      ELSE
       CALL COMLIS('* Avaiable is :',CSYS,ISYS)
       WRITE(*,CFORM)'* Type No or 0 to exit'
       CMAT='No'
       GO TO 90
      ENDIF
       
CA... Read system matrix A from terminal                .........
100   IF (NX.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(1)
      CALL DREMA(A,WORK,NX,NA,NX,NA,CSYS(1))
      GO TO 90
C
CB... Read input matrix B from terminal                 .........
200   IF (NX.LE.0.OR.NU.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(2)
      CALL DREMA(B,WORK,NX,NA,NU,NB,CSYS(2))
      GO TO 90
C
CC... Read disturbance matrix C from terminal           .........
300   IF (NX.LE.0.OR.NV.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(3)
      CALL DREMA(C,WORK,NX,NA,NV,NC,CSYS(3))
      GO TO 90
C
CD... Read mesurement matrix D from terminal            .........
400   IF (NY.LE.0.OR.NX.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(4)
      CALL DREMA(D,WORK,NY,ND,NX,NA,CSYS(4))
      GO TO 90

CG... Read controller matrix G  (u = Gy -> Output feedback ......
500   IF (NU.LE.0.OR.NY.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(5)
      CALL DREMA(G,WORK,NU,NB,NY,ND,CSYS(5))
      GO TO 90

CKA.. Read Kalman filter gain matrix                    .........
600   IF (NX.LE.0.OR.NY.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(6)
      CALL DREMA(KA,WORK,NX,NA,NY,ND,CSYS(6))
      GO TO 90

CP... Read control input weight matrix P                .........
700   IF (NU.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(7)
      CALL DREMA(P,WORK,NU,NB,NU,NB,CSYS(7))
      GO TO 90

CQ... Read state weight matrix Q ( CSYS(8 )             .........
800   IF (NX.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(8)
      CALL DREMA(Q,WORK,NX,NA,NX,NA,CSYS(8))
      GO TO 90

CR... Read state weight matrix R                        .........
900   IF (NX.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(9)
      CALL DREMA(R,WORK,NX,NA,NX,NA,CSYS(9))
      GO TO 90

CXA.. Read state weight matrix XA                       .........
1000  IF (NX.LE.0) GO TO 2000
      WRITE(*,CFORM)'*         Matrix '//CSYS(10)
      CALL DREMA(XA,WORK,NX,NA,NX,NA,CSYS(10))
      GO TO 90

C
C... Last lines of DSYSMAT                     ..................
2000  IF (NX.LE.0.OR.NU.LE.0.OR.NV.LE.0.OR.NY.LE.0) THEN
       WRITE(*,CFORM)'*  Dimension out of bounds'
       IERR = 1
      ENDIF
C
      RETURN
      END

      CHARACTER*(*) FUNCTION CANEXT(CSYS,N,ISYS)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                      C
C   This function scan the array CSYS from position N to ISYS    C
C   and return the first caracter name different from space.     C
C   ROUTINES CALLED : (None)                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,ISYS
      CHARACTER CSYS(*)*(*)

      DO 10 I=N,ISYS
         IF(CSYS(I).NE.' ') THEN
          CANEXT = CSYS(I)
          GO TO 1000
         ENDIF
10    CONTINUE
      CANEXT = ' '

1000  RETURN
      END


