
      SUBROUTINE DEQSOL(SD1,NEQ,YS,INFO,T1,T2,NAME)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DEQSOL                                        C
C   AUTHOR : David Di Ruscio              ITK/NTH february 1989  C
C   PURPOSE                                                      C
C   DEQSOL integrate a set of stiff or non stiff diferential     C
C   equations defined in SUBROUTINE SD1. SD1 must be declared    C
C   as external in calling program. The user can choose between  C
C   5 methods. (BLEND, GEAR, THETA, THETB and ADAMS). These      C
C   methods are described in SPRINT users manual.                C
C   ON INPUT                                                     C
C   SD1  : Name of a external declared subroutine whitch the     C
C          ODE set to be integrated is defined.                  C
C   NEQ  : Actual number of ODE to be integrated.                C
C   YS   : DOUBLE PRECISION (NEQMAX)                             C
C          An optionaly array of ODE initial values.             C
C   INFO : Dummy integer ( for future use)                       C
C   T1,T2: DOUBLE PRECISION scalars, defining simulation time    C
C          interval.                                             C
C   NAME : CHARACTER, optional default TS/MATLAB-file name.      C
C   INFORMATION                                                  C
C   NEQMAX is defined in the include file DEQSOL.INC.            C
C   The user can increase NEQMAX up to MAXSPR. MAXSPR is defined C
C   in SPRITE.INC and can be changed by the user.                C
C   NB. The deqlaration of Y in calling program shuld be equal   C
C   to double precision Y(NEQMAX).                               C
C                                                                C
C   LIBRARIES USED : MVD$DUA1:[UTIL.SPRINT]         SPRINT/LIB   C
C                  : SYS$SYSDEVICE:[CYPROS.LIBRARY] CYPROS/LIB   C
C                  : SYS$SYSDEVICE:[DAVID.CYPLIB]   CYPLIB/LIB   C
C   SPRINT ROUTINES : SPRITE                                     C
C   CYPLIB ROUTINES : RVEC  (read double precision vector)       C
C   CYPROS ROUTINES : IREAD, CREAD, RREAD,   ICDREAD             C
C***END PROLOGUE DEQSOL                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C... Initialize max dimensions and common block               ....
C... /STORING/ SOLTAB,STCOUNT,NUMCOL,LSTSTORE in INC-file     ....
      INCLUDE 'DEQSOL.INC'
      INTEGER NEQ, INFO, NCOL
      DOUBLE PRECISION YS(NEQMAX),T1,T2

C... Parameter declarations for SPRITE                        ....
      INTEGER NT,IW(MAXIW),NIW(3),NRW2,ITRACE
      DOUBLE PRECISION T,DTOUT(MAXNT),Y(NEQMAX)
      DOUBLE PRECISION RW2(MAXRW),RTOL(NEQMAX),ATOL(NEQMAX)
      CHARACTER MATZ*6, STEP*6, OUT*6, SNORM*6
C... End par. dec. for SPRITE
      DOUBLE PRECISION TFINAL,TS,DT
      INTEGER ICOM,NCOM
      CHARACTER DIFCOM(6)*9, NAME*12

C... Declare names of SPRITE routines used. SPRDAT initialize
C    machine dependent constants.
      EXTERNAL SD1, MONITR, SPRDAT

      DATA DIFCOM(1)/'auto     '/, DIFCOM(2)/'manual   '/,
     +     DIFCOM(3)/'info     '/, DIFCOM(4)/'output   '/,
     +     DIFCOM(5)/'return   '/, DIFCOM(6)/'0'/
      DATA NCOM/6/

C1... Specify start values defined in calling program     .......
      DO 5 I=1,NEQ
         Y(I) = YS(I)
5     CONTINUE
      ITRACE = 0
      T      = T1
      TFINAL = T2

C2... Choose command interactively                        .......
      ICOM = 1
90    ICOM = ICREAD('Deqsol command :',DIFCOM,NCOM,ICOM)

C3... Integerate SD1 whit manual or automatic set up      .......
      IF (ICOM.EQ.1.OR.ICOM.EQ.2) THEN
       CALL PAINIT(NEQ,NIW,NRW2,MAXIW,MAXRW)

C.... Automatic setup of parameters                       .......
       IF (ICOM.EQ.1) THEN
        CALL AUTSET(SNORM,MATZ,STEP,OUT,IW,RW2,RTOL,ATOL,NEQ,ICOM)
        ICOM  = 3

C.... Manual setup of parameters                          .......
       ELSEIF (ICOM.EQ.2) THEN
        CALL MANSET(SNORM,MATZ,STEP,OUT,IW,RW2,RTOL,ATOL,NEQ,ITRACE)
       ENDIF

C4... Chose integration intervall (T1,T2) and initialize DTOUT ..
       CALL DTINIT(T,DTOUT,NT,MAXNT,TFINAL)

C...  Integrate whit SPRITE                               .......
       WRITE(*,'(5X,A,A)')'Integrating whit method ',STEP
       IW(1) = 0
       TS    = T
       CALL STINIT(NEQ,T,TFINAL,Y)
       
       CALL SPRITE(NEQ,T,DTOUT,NT,OUT,Y,RTOL,ATOL,ITRACE,
     2             SNORM,MATZ,STEP,SD1,MONITR,
     3             RW2,NRW2,IW,NIW)

       IF (IW(1).NE.2) THEN
        WRITE(*,'(5X,A,1X,I3)')'Error IW =',IW(1)
        WRITE(*,'(5X,A)'      )'Check output file for info.'
       ELSEIF (IW(1).EQ.2) THEN
        WRITE(*,'(5X,A,G12.4,A,G12.4)')'Successfull integration from
     +  T1 = ',TS,' to T2 = ',DTOUT(NT)
       ENDIF
       ICOM = 3
       GO TO 90

      ELSEIF (ICOM.EQ.3) THEN
       CALL INTRES(IW,STEP,OUT,MATZ,SNORM,RTOL,ATOL)
       ICOM = 4
       GO TO 90
      ELSEIF (ICOM.EQ.4) THEN
       DT = 1.0E0
C... Do not store step size in position SOLTAB(I,NUMCOL)
       NCOL = NUMCOL - 2
C...
CUT    NAME = ' '
       CALL PLTOUT(SOLTAB,STCOUNT,NCOL,MAXSAMP,MAXCOL,DT,IFIL,NAME)
       ICOM = 5
       GO TO 90
      ELSEIF (ICOM.EQ.5) THEN
       GO TO 1000
      ELSEIF (ICOM.EQ.6) THEN
       GO TO 1000
      ENDIF

1000  T1 = TS
      T2 = TFINAL
      DO 10 I=1,NEQ
         YS(I) = Y(I)
10    CONTINUE

      RETURN
      END

      SUBROUTINE STINIT(NEQ,T,TFINAL,Y)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                     C
C   To initialize solution array SOLTAB(STCOUNT,NEQ+2) at time  C
C   T = 0 and counter STCOUNT.                                  C
C                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NEQ
      DOUBLE PRECISION T,TFINAL,Y(*)
C..Initialize common block in DEQSOL.INC. The common block is ...
C..COMMON /STORING/ SOLTAB,STCOUNT,NUMCOL,LSTSTORE         ......
      INCLUDE 'DEQSOL.INC'

      NUMCOL      = NEQ+2
      LSTSTORE    = 0.0
      STCOUNT     = 1
      SOLTAB(1,1) = T  

      DO 10 I=2,NEQ+2
         SOLTAB(1,I) = Y(I-1)  
10    CONTINUE

      RETURN
      END

      SUBROUTINE CAINIT(NORM,MATZ,STEP,OUT,ICOM)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE CAINIT                                       C
C   PURPOSE                                                     C
C   To specify norm type, matrix type, integration method       C
C   and type of output (output to data file)                    C
C   ROUTINES CALLED : CREAD                                     C
C***END PROLOGUE CAINIT                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER ICOM
      CHARACTER NORM*6,MATZ*6,STEP*6,OUT*6,CF*6

      CF = '(2X,A)'
C1... Specify matrix norm                               .........
      IF (ICOM.EQ.4) THEN
100    CALL RCSTR('Specify norm type ',NORM,1,1,IIII)
       IF(NORM.NE.'L2NORM'.AND.NORM.NE.'MAXMUM'
     +                                .AND.NORM.NE.'MODMAX') THEN
        WRITE(*,CF)'Avaiable is : L2NORM   MAXMUM   MODMAX'
        NORM = 'L2NORM'
        GO TO 100
       ENDIF
       ICOM = 5
       GO TO 1000

C2... Specify matrix type                                  .......
      ELSEIF (ICOM.EQ.5) THEN
200    CALL RCSTR('Specify matrix type ',MATZ,1,1,IIII)
       IF (MATZ.NE.'FULL'.AND.MATZ.NE.'BANDED'
     +                                .AND.MATZ.NE.'SPARSE') THEN
        WRITE(*,CF)'Avaiable is : FULL   BANDED   SPARSE'
        MATZ = 'FULL'
        GO TO 200
       ENDIF
       ICOM = 6
       GO TO 1000

C3.. Specify integration method                           .......
      ELSEIF (ICOM.EQ.6) THEN
300    CALL RCSTR('Specify integration method :',STEP,1,1,IIII)
       IF (STEP.NE.'BLEND'.AND.STEP.NE.'GEAR'.AND.STEP.NE.'THETA'
     +             .AND.STEP.NE.'THETB'.AND.STEP.NE.'ADAMS') THEN
        WRITE(*,CF)'Avaiable is : ADAMS   BLEND   GEAR   THETA   THETB'
        STEP = 'THETB'
        GO TO 300
       ENDIF
       ICOM = 7
       GO TO 1000

C4... Specify type of output to channal 4 (FOR004.DAT    ........
      ELSEIF (ICOM.EQ.7) THEN
400    CALL RCSTR('Specify output to FOR018.DAT :',OUT,1,1,IIII)
       IF (OUT.NE.'BRIEF'.AND.OUT.NE.'LONG') THEN
        WRITE(*,CF)'Avaiable is : BRIEF   LONG'
        OUT = 'LONG'
        GO TO 400
       ENDIF
       ICOM = 8
       GO TO 1000

      ENDIF

1000  RETURN
      END

      SUBROUTINE DTINIT(T,DTOUT,NT,MAXNT,TFINAL)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DTINIT                                       C
C   PURPOSE                                                     C
C   To read time intervall (T1,T2) and number of "samples" NT   C
C   to reach T2.                                                C
C   ROUTINES CALLED : RREAD                                     C
C***END PROLOGUE DTINIT                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NT,II
      DOUBLE PRECISION T,TFINAL,DTOUT(MAXNT),DTI,DT,DNT,TI(2)

100   TI(1)  = T
      TI(2)  = TFINAL
CUT   TI(2)  = TFINAL + T
      CALL DREAD('Time intervall (T1-T2) :',TI,2,2,IIII)
      T      = TI(1)
      TFINAL = TI(2)
      IF (TFINAL.LE.T) THEN
       WRITE(*,'(5X,A)')'Error -->   Specify T2 > T1'
       GO TO 100
      ENDIF

200   NT = 1
      CALL NREAD('Specify number of iterations :',NT,1,1,IIII)
      IF (NT.LE.0.OR.NT.GT.MAXNT) THEN
       WRITE(*,'(5X,A,I3)')'Error -->  Specify  0 < NT < ',MAXNT
       GO TO 200
      ENDIF
      DNT = DFLOAT(NT)
      DTI = (TFINAL-T)/DNT
       
      DT = T
      DO 11 II = 1,NT
         DT = DT + DTI
         DTOUT(II) = DT
11    CONTINUE
      TFINAL = DTOUT(NT)

      RETURN
      END

 
      SUBROUTINE PAINIT(NEQ,NIW,NRW2,MAXIW,MAXRW)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE PAINIT                                       C
C   PURPOSE                                                     C
C   To initialize array sizes used inside SPRITE /SPRINT.       C
C   NWKMON is the size of array WKMON(NWKMON) used to pass      C
C   information from each inp/output from SPRINT to routine     C
C   MONITR.                                                     C
C                                                               C
C***END PROLOGUE PAINIT                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NEQ,NIW(*),NRW2,MAXIW,MAXRW
      INTEGER NWKRES, NWKMON

      NWKRES = 1
      NWKMON = NEQ + 1

C... Paramerets to SPRITE                               ........
      NIW(1) = MAXIW
      NIW(2) = NWKRES
      NIW(3) = NWKMON
      NRW2   = MAXRW

      RETURN
      END

      SUBROUTINE INTRES(IW,STEP,OUT,MATZ,NORM,RTOL,ATOL)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE INTRES                                         C
C   PURPOSE                                                       C
C   To write info from the integration time.                      C
C   ROUTINES CALLED : (none)                                      C
C***EBD PROLOGUE INTRES                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER IW(*)
      DOUBLE PRECISION RTOL(*),ATOL(*)
      CHARACTER STEP*(*), OUT*(*), MATZ*(*), NORM*(*) ,CF*6
      CF = '(5X,A)'

      WRITE(*,'(5X,A,A)')'SPRITE integration method      : ',STEP
      WRITE(*,'(5X,A,A)')'File output from sprint as     : ',OUT
      WRITE(*,'(5X,A,A)')'Jacobian matrix specified as   : ',MATZ
      WRITE(*,'(5X,A,A)')'SPRITE using matrix norm       : ',NORM
      WRITE(*,'(5X,A,E12.4)')'Relative tolerance RTOL(1) : ',RTOL(1)
      WRITE(*,'(5X,A,E12.4,/)')'Absolute tolerance ATOL(1) : ',ATOL(1)

      WRITE(*,1) IW(6),IW(7),IW(8),IW(11),IW(9)
1     FORMAT(5X,'#STEPS = ',I3,2X,'#FCN = ',I5,2X,'#JACOB = ',
     +I5,2X,'#SOLVE = ',I5,2X,'#ORDER = ',I2,/)
      WRITE(*,CF)'#STEPS : Number of steps taken so far.  '
      WRITE(*,CF)'#FCN   : Number of function evaluations.'
      WRITE(*,CF)'#JACOB : Number of Jacobian evaluations.'
      WRITE(*,CF)'#SOLVE : Number of iterations used by the non
     + linear solver'
      WRITE(*,CF)'#ORDER : The method order last used (successfully).'
      IW(6)  =0
      IW(7)  =0
      IW(8)  =0
      IW(9)  =0
      IW(11) =0

      RETURN
      END

      SUBROUTINE AUTSET(SNORM,MATZ,STEP,OUT,IW,RW2,RTOL,ATOL,NEQ,IC)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE AUTSET                                        C
C   PURPOSE                                                      C
C   To specify default parameters to the integration package     C
C   SPRITE.                                                      C
C   ROUTINES CALLED : (none)                                     C
C***END PROLOGUE AUTSET                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NEQ, IW(*), I, IC
      DOUBLE PRECISION RW2(*), RTOL(*), ATOL(*)
      CHARACTER SNORM*6,MATZ*6, STEP*6, OUT*6

      SNORM      = 'L2NORM'
      MATZ       = 'FULL  '
      STEP       = 'THETB '
      OUT        = 'LONG  '
      IW(3)      = 0
      RW2(1)     = 0.0D0

      IF (IC.EQ.1) THEN
       DO 10 I=1,NEQ
          RTOL(I) = 1.0D-3
          ATOL(I) = 1.0D-3
10     CONTINUE
      ENDIF

      RETURN
      END

      SUBROUTINE MANSET(SNORM,MATZ,STEP,OUT,IW,RW2,RTOL,ATOL,NEQ,
     2                  ITRACE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE MANSET                                        C
C   PURPOSE                                                      C
C   To manually setup of SPRITE user optionally parameters.      C
C                                                                C
C***END PROLOGUE MANSET                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NEQ, IW(*), ICOM, NCOM
      DOUBLE PRECISION RW2(*), RTOL(NEQ), ATOL(NEQ), HWORK
      CHARACTER SNORM*6, MATZ*6, STEP*6, OUT*6, COM(11)*6

      DATA COM( 1)/'RTOL  '/, COM(2)/'ATOL  '/, COM(3)/'H0    '/,
     2     COM( 4)/'NORM  '/, COM(5)/'MATZ  '/, COM(6)/'METHOD'/,
     3     COM( 7)/'OUTPUT'/, COM(8)/'TRACE '/, COM(9)/'HELP  '/,
     4     COM(10)/'RETURN'/, COM(11)/'0'/
      DATA NCOM/11/

C1...Initialize default parameters whit AUTSET          .........
      CALL AUTSET(SNORM,MATZ,STEP,OUT,IW,RW2,RTOL,ATOL,NEQ,1)

C2...Read command from terminal                         .........
      ICOM = 1
90    ICOM = ICREAD('Specify parameter :',COM,NCOM,ICOM)

C3...Execute commands                                   .........
      IF (ICOM.EQ.10.OR.ICOM.EQ.11) THEN
       GO TO 1000

C4... Specify rel. and abs. tolerances                  .........
      ELSEIF (ICOM.EQ.1.OR.ICOM.EQ.2) THEN
       CALL ACCURACY(RTOL,ATOL,ITOL,NEQ,ICOM)
       IF (ICOM.EQ.1) THEN
        ICOM = 2
       ELSE
        ICOM = 3
       ENDIF
       GO TO 90

C5... Specify initial stepsize                          .........
      ELSEIF (ICOM.EQ.3) THEN
       HWORK = RW2(1)
       CALL DREAD('Initial stepsize H0 : ',HWORK,1,1,IIII)
       RW2(1) = HWORK
       IF (RW2(1).EQ.0.0) THEN
        IW(3) = 0
       ELSE
        IW(3) = 1
       ENDIF
       ICOM = 4
       GO TO 90

C6...Specify integration method etc                     .........
      ELSEIF (ICOM.EQ.4.OR.ICOM.EQ.5.OR.ICOM.EQ.6.OR.ICOM.EQ.7) THEN
       CALL CAINIT(SNORM,MATZ,STEP,OUT,ICOM)
       GO TO 90

C ...Specify whitch channel the output is written       .........
      ELSEIF (ICOM.EQ.8) THEN
200    CALL NREAD('Specify output trace (0,1,2 or 3)',ITRACE,1,1,IIII)
       IF (ITRECE.LT.0.OR.ITRACE.GE.5) THEN
        WRITE(*,'(5X,A)')'Trace 0, 1, 2, 3 or 4 '
        WRITE(*,'(5X,A)')'This means output type to FORT.4'
        ITRACE = 0
        GO TO 200
       ENDIF
       ICOM = 9
       GO TO 90

      ELSEIF (ICOM.EQ.9) THEN
       WRITE(*,'(4X,A)')'Sorry no help. Check SPRINT manual.'
       ICOM = 10
       GO TO 90
      ENDIF

1000  RETURN
      END

      SUBROUTINE ACCURACY(RTOL,ATOL,ITOL,NEQ,ICOM)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE ACCURACY                                     C
C   PURPOSE                                                     C
C   To specify relative and absolute error tolerances.          C
C   The error bounds are calculating as                         C
C   EWT(I) = RTOL(I)*ABS(Y(I)) + ATOL(I) for I = 1,...,NEQ      C
C   ROUTINES CALLED : RREAD, RVEC ( defined in CYPLIB)          C
C   NB : ITOL = 4 is used by SPRITE                             C
C***END PROLOGUE ACCURACY                                       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER ITOL,NEQ,RSVAR,ASVAR,ICOM
      DOUBLE PRECISION RTOL(NEQ), ATOL(NEQ),RS, AS

C1... Relative error tolerance RTOL                     .........
      IF (ICOM.EQ.1) THEN
       DO 10 I=1,NEQ
          IF (RTOL(I).EQ.0.0 ) RTOL(I) = 1.0E-3
10     CONTINUE

       RSVAR = IYESANS('Relative error tolerance a vector',1)   
       IF (RSVAR .EQ. 1) THEN
100     RS = RTOL(1)
        CALL DREAD('Rel. error tolerance RTOL',RS,1,1,IIII)
        IF (RS.LE.0.0) THEN
         WRITE(*,'(5X,A)')'Error ---> specify RTOL > 0'
         GO TO 100
        ENDIF
        DO 20 I=1,NEQ
           RTOL(I) = RS
20      CONTINUE
       ELSE
        CALL DRVEC('Rel. error tolerance RTOL',RTOL,NEQ,NEQ)
       ENDIF

C2... Absolute error tolerance ATOL                      .........
      ELSEIF (ICOM.EQ.2) THEN
       DO 30 I=1,NEQ
          IF (ATOL(I).EQ.0.0) ATOL(I) = 1.0E-3
30     CONTINUE

       ASVAR = IYESANS('Absolute error tolerance a vector',1)
       IF (ASVAR .EQ. 1) THEN
200     AS = ATOL(1)
        CALL DREAD('Abs. error tolerance ATOL',AS,1,1,IIII)
        IF (AS.LE.0) THEN
         WRITE(*,'(5X,A)')'Error ---> specify ATOL > 0'
         GO TO 200
        ENDIF
        DO 40 I=1,NEQ
           ATOL(I) = AS
40      CONTINUE
       ELSE
        CALL DRVEC('Abs. error tolerance ATOL',ATOL,NEQ,NEQ)
       ENDIF

      ENDIF

C3... Parameter ITOL (used only by SPRINT. SPRITE uses only ITOL=4)
CUT   if( rsvar.eq.1 .and. asvar.eq.1 )then
CUT       itol = 1
CUT   elseif( rsvar.eq.1 .and. asvar.eq.2 )then
CUT       itol = 2
CUT   elseif( rsvar.eq.2 .and. asvar.eq.1 )then
CUT       itol = 3 
CUT   else
CUT       itol = 4 
CUT   endif

      RETURN
      END

      SUBROUTINE MONITR(NEQ,T,HLAST,HNEXT,Y,YDOT,YSAVE,MAXNEQ,R,ACOR,
     >                  WKRES,NWKRES,WKMON,NWKMON,IMON,INLN,HMIN,HMAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE MONITR                                            C
C   PURPOSE                                                          C
C   MONITR is called by SPRINT after each succesfull integration     C
C   step at time T. MONITR is used to store the solution Y(NEQ)      C
C   at time T in the double subscripted array SOLTAB(MAXSAMP,MAXCOL) C
C   defined DEQSOL.INC. Se DEQSOL.INC for common block.              C
C***END PROLOGUE MONITR                                              C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NEQ,MAXNEQ,NWKRES,NWKMON,IMON,INLN
      DOUBLE PRECISION T,HLAST,HNEXT,Y(NEQ),YDOT(NEQ),YSAVE(MAXNEQ,1)
      DOUBLE PRECISION R(NEQ),ACOR(MAXNEQ,2),WKRES(NWKRES)
      DOUBLE PRECISION WKMON(NWKMON),HMIN,HMAX
      INCLUDE 'DEQSOL.INC'
C... YSAVE(*,1) solutions at (T-1) previous timestep            ......
C... Y(*) solution at time T                                    ......

CUT   if( t.ge.(lststore+dtstore) )then
      IF (STCOUNT.LT.MAXSAMP.AND.T.GT.LSTSTORE) THEN
         STCOUNT = STCOUNT + 1
         SOLTAB(STCOUNT,1 ) = T
         DO 10 I=2,NEQ+1
CUT         soltab( stcount,i ) = real(ysave( i-1,1 ))
            SOLTAB( STCOUNT,I ) = Y(I-1)
10       CONTINUE
         SOLTAB(STCOUNT,NEQ+2 ) = HLAST
         LSTSTORE = T
      ENDIF

      RETURN
      END

