      subroutine ricsol (f,g,h,z,w,n,er,ei,work,ind,info,nf,ng,nh)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     *****parameters:
      integer n,ind(*),info,nf,ng,nh
      double precision f(nf,*),g(ng,*),h(nh,*),z(*),w(*),
     +                 er(*),ei(*),work(*)
c
c     *****local variables:
      integer nn,j,low,igh,ifail
      double precision eps
      logical fail
c
c     *****subroutines called: (total routines dependnes)
c     hqr4,eigord,eigval,exchng,split            (Eispack extended)
c     balanc,balbak,orthes,ortran                (Eispack)
c     dgefa,dgesl,daxpy,ddot,dscal,idamax        (Linpack)
c     ham,trp
c     ------------------------------------------------------------------
c
c     *****purpose:
c     this subroutine solves the continuous-time algebraic
c     matrix riccati equation
c                      t
c                     f *x + x*f - x*g*x + h = 0
c
c     by laub's variant of the hamiltonian-eigenvector approach.
c     ref.: laub, a.j., a schur method for solving algebraic riccati
c           equations, ieee trans. aut. contr., ac-24(1979), 913-921.
c
c     *****parameter description:
c     on input:
c        f                n x n (real) matrix;
c
c        g,h              n x n symmetric, nonnegative definite
c                         (real) matrices.
c
c        n                order of the matrices f,g,h;
c
c        nf,ng,nh         row dimensions of the arrays containing
c                         f,g,h, respectively, as declared in the 
c                         calling program dimension statement;
c
c     on output:
c
c        h                n x n array containing the unique positive
c                         (or nonnegative) definite solution of the
c                         riccati equation;
c
c        er,ei            real scratch vectors of length 2*n; on output
c                         (er(i),ei(i)), i=1,n contain the real and
c                         imaginary parts, respectively, of the n
c                         closed loop eigenvalues (i.e., the
c                         spectrum of f - g*x);
c
c        z,w              4*n*n real scratch vectors used for
c                         computations involving the hamiltonian
c                         matrix associated with the riccati equation;
c
c        work             real scratch vectors of lengths 2*n.
c
c        ind              integer scratch vector of lengths 2*n.
c
c        info             integer which contains an error signal whose 
c                         value is
c                            0  if the solution was found,
c                            1  if the real schur decompostion 
c                               was not successful,
c                            2  if the eigenspace reordering failed.
c                            3  if the linear equations are singular
c
c     ***note:  all scratch arrays must be declared and included
c               in the call.***
c
c     *****algorithm notes:
c     it is assumed that g and h are nonnegative definite and that (f,b)
c                                                      t
c     is stabilizable and (c,f) is detectable where b*b  = g
c                                       t
c     (b of full rank = rank(g)) and c*c = h (c of full
c     rank = rank(h)) in which case the solution (returned in the
c     array h) is unique and nonnegative definite.
c
c     *****history:
c     written by alan j. laub (dep't. of elec. engrg.-systems, univ.
c     of southern calif., los angeles, ca 90007; ph.: (213) 743-5535),
c     sep. 1977.
c     most recent version:  apr. 15, 1981.
c
c     ------------------------------------------------------------------
c

      nn = n + n
      eps = 1.0d0
   10 eps = eps/2.0d0
      if (eps+1.0d0 .gt. 1.0d0) go to 10
      eps = 2.0d0*eps
c
c     set up hamiltonian matrix
c
      call ham (f,g,h,w,n,nf,ng,nh,nn)
c
c     reduce w to real schur form with eigenvalues in the left half
c     plane in the upper left n x n upper quasi-triangular block
c
      call balanc (nn,nn,w,low,igh,work)
      call orthes (nn,nn,low,igh,w,er)


      call ortran (nn,nn,low,igh,w,er,z)
      call hqr4 (w,z,nn,low,igh,eps,ifail,nn,nn)

      if (ifail .ne. 0) then
         info = 1
C This go to  is removed by D. Di Ruscio 3/12-1988.
C The solution  exist if IFAIL > 0 in HQR4.           ..........
CUT      go to 100
      endif
c
c     order eigenspace with stable part in upper left corner
c
      call eigord (w,z,nn,eps,-1,ind,fail,nn,nn)
      if (fail) then
         info = 2
         go to 100
      endif
      call eigval (w,z,nn,1,nn,er,ei,nn,nn)
c
c     compute solution of the riccati equation from the orthogonal
c     matrix now in the array z.  store the result in the array h.
c
      call balbak (nn,nn,low,igh,work,n,z)
      call dgefa (z,nn,n,ind,info)
      if (info .ne. 0) then
         info = 3
         goto 100
      endif
      call trp (z(n+1),h,n,nn,nh)
c.    solve linear system using the factors computed by degefa
      do 20 j=1,n
         call dgesl (z,nn,n,ind,h(1,j),1)
   20 continue
  100 continue
      return
c
c     last line of ricsol
c
      end
      subroutine ham (f,g,h,w,n,nf,ng,nh,nw)
c     Purpose :
c     ham setts up the Hamiltonian matrix
c
      integer n,nf,ng,nh,nw,i,j,npi,npj
      real*8 f(nf,*),g(ng,*),h(nh,*),w(nw,*)
      do 20 j=1,n
         npj = n+j
         do 10 i=1,n
            npi = n+i
            w(i,j) = f(i,j)
            w(npi,j) = -h(i,j)
            w(i,npj) = -g(i,j)
            w(npi,npj) = -f(j,i)
   10    continue
   20 continue
      return
      end
      subroutine trp (z,h,n,nz,nh)
c
c     Purpose :
c     Take the transpose of a matrix  h = z**T
c
      integer n,nz,nh,i,j
      real*8 z(nz,*),h(nh,*)
      do 10 j=1,n
      do 10 i=1,n
      h(i,j) = z(j,i)
   10 continue
      return
      end
