      SUBROUTINE AXBSOL2(A,N,NA,B,DET,II,III,IWORK,INFO)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                               C
C     David Di Ruscio                   NTH/ITK 13 oktober 83   C
C                                                               C
C     BESKRIVELSE                                               C
C     AXBSOL2 loeser likningssystemet Ax = b ved foerst aa      C
C     dekomponere A = L*U ved partiell pivotsoekning. Deretter  C
C     loeses L*U*x = b ved forovereliminasjon etterfulgt av     C
C     bakoversubstitusjon.                                      C
C                                                               C
C     INN                                                       C
C     A     : DOUBLE PRECISION array med dimensjon (NA,N)       C
C     N     : INTEGER dimensjon paa problemet                   C
C     NA    : Ledende dimensjon paa A i det kallende program    C
C     B     : DOUBLE PRECISION vektor av dimensjon (N)          C
C             inneholdende hoeyresiden                          C
C     II,III: INTEGER flag for spesifisering om A skal          C
C             dekomponeres ved hvert kall. A dekomponeres       C
C             dersom II=1 og III=1                              C
C     IWORK : INTEGER array med im. minst (N). Benyttes som     C
C             arbeidslager for pivotindekser.                   C
C     UT                                                        C
C     B     : Loesningsvektoren x med dimensjon N               C
C     DET   : Determinanten til A-matrisen                      C
C     INFO  : INFO = 0 --> OK operation                         C
C             INFO = 1 --> System singular or close to          C
C                          singularity.                         C
C                                                               C
C     BENYTTEDE RUTINER : LUDEC2,LUSOL2                         C
C                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,II,III,IWORK(N)
      DOUBLE PRECISION A(NA,N),B(N),DET
C
C.... Dekomponerer A = L*U  (L-lower,U-upper)   .................
      IF (II.EQ.1.AND.III.EQ.1) THEN
       CALL LUDEC2(A,N,NA,IWORK,DET)
      ENDIF

C.... Check for singularity
      INFO = 0
      IF (DET.EQ.0.0D0) THEN
       INFO = 1
       GO TO 1000
      ENDIF

C.... Loeser L*U*x = b                          .................
      CALL LUSOL2(A,N,NA,IWORK,B)

1000  RETURN
      END
