      SUBROUTINE DGEIGV(NM,N,A,B,ALFR,ALFI,BETA,MATZ,Z,IERR,LARGE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DGEIGV                                       C
C   PURPOSE                                                     C
C   To solve the generalized eigenvalue problem  A*x = labda*B  C
C   Infinite eigenvalues are set to LARGE.                      C
C   ON INPUT                                                    C
C   N    INTEGER dimension of A and B (quadratic matrixes)      C
C   NM   INTEGER lading dimension on A and B in calling program.C
C   MATZ INTEGER flag. MATZ = 0 only eig.val. MATZ=1 add eigvec.C
C   A    DOUBLE PRECISION (NM,N) rhd matrix.                    C
C   B    DOUBLE PRECISION (NM,N) lhd matrix.                    C
C   LARGEDOUBLE PRECISION value for infinity (ex. 1.0e30)       C
C   ON OUTPUT                                                   C
C   ALFR DOUBLE PRECISION (N) real part of gen. eigenvalues.    C
C   ALFI DOUBLE PRECISION (N) imaginary part of gen. eig.       C
C   BETA DOUBLE PRECISION (N) contains the denominator of eigv. C
C   Z    DOUBLE PRECISION (NM,N) contains the eigenvector if    C
C        MATZ is set to 1.                                      C
C   IERR INTEGER IERR=0 for succsesfull operation. Se subroutineC
C        QZIT (EISPACK) for other values.                       C
C   ROUTINES CALLED : RGG (EISPACK), EVAL                       C
C***END PROLOGUE DGEIGV                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NM,N,MATZ,IERR
      DOUBLE PRECISION A(NM,N),B(NM,N),ALFR(N),ALFI(N),BETA(N)
      DOUBLE PRECISION Z(NM,N)

C... Solve generalized eigenvalue problem                      ....
      CALL RGG(NM,N,A,B,ALFR,ALFI,BETA,MATZ,Z,IERR)

CUT   WRITE(*,'(2X,A)')'NO.       ALFR         ALFI         BETA'
CUT   WRITE(*,1) (I,ALFR(I),ALFI(I),BETA(I),I=1,N)
CUT1  FORMAT(2X,'(',I3,' ) =',E12.4,1X,E12.4,1X,E12.4)

C... Evaluate generalized eigenvalues from ALFR, ALFI and BETA ....
      CALL EVAL(N,ALFR,ALFI,BETA,LARGE)

      RETURN
      END

      SUBROUTINE EVAL(N,ALFR,ALFI,BETA,LARGE)
C... Routine called from DGEIGV used to reorder gen. eigenvalues
C... Infinite gen. eig. values are set to LARGE
      INTEGER N,I
      DOUBLE PRECISION ALFR(*),ALFI(*),BETA(*),LARGE,MAG
      DO 10 I=1,N
         MAG = ABS( DCMPLX( ALFR(I),ALFI(I) ) )
C... Infinite or NO-NO values are set to LARGE
         IF (BETA(I)*LARGE.LE.MAG) THEN
          IF (ALFR(I).EQ.0.0 .OR. ALFI(I).EQ.0.0) THEN
           ALFR(I) = LARGE
           ALFI(I) = LARGE
          ELSE
           ALFR(I) = ALFR(I)/MAG*LARGE
           ALFI(I) = ALFI(I)/MAG*LARGE
          ENDIF
C... Finite values
         ELSE
          ALFR(I) = ALFR(I)/BETA(I)
          ALFI(I) = ALFI(I)/BETA(I)
         ENDIF
10    CONTINUE
      RETURN
      END

