      SUBROUTINE LUDEC2(A,N,NA,IPIV,DET)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                               C
C  David Di Ruscio              NTH/ITK 13 oktober 1988         C
C                                                               C
C  BESKRIVELSE                                                  C
C  LUDEC2 dekomponerer  en generell reell matrise A = L*U       C
C  ved partiell pivotsoekning.                                  C
C  Med partiell pivotsoekning menes at stoerste element i       C
C  kolonnen A(I,K) for I=1,N-1 benyttes som pivotelement.       C
C                                                               C
C  INN                                                          C
C  A    : DOUBLE PRECISION matrise med dimensjon (NA,N)         C
C  N    : INTEGER dimensjon paa problemet                       C
C  NA   : INTEGER ledende dimensjon p} A i det kallende program C
C  UT   :                                                       C
C  A    : Faktoriseringen L*U. L lower triangulaer matrise med  C
C         diagonalelementer lik 1. Lagret i nedre del av A. U   C
C         upper triangulaer matrise lagret i oevre del av A.    C
C  IPIV : INTEGER array med dimensjon (N). Vektor av pivot-     C
C         elementer.                                            C
C  DET  : Determinanten til A                                   C
C                                                               C
C  BENYTTEDE SUBRUTINER : Ingen                                 C
C                                                               C
C  REFERANSE : "Matrix Computation",Golub,Van Loan (Oxford 83)  C
C               kapittel 4                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,NA,IPIV(N)
      DOUBLE PRECISION A(NA,N),DET
C..  Interne variable                           .................
      INTEGER NM1,I,J,K,KP1,M
      DOUBLE PRECISION P,T
C..  Foerste utfoerende setninger               .................
      DET=1.0
      IPIV(N)=1
      IF (N.EQ.1) GO TO 70
      NM1=N-1
C
      DO 60 K=1,NM1
         KP1=K+1
C
C.... Finner pivot ( partiell pivot )           ..................
         M=K
         DO 10 I=KP1,N
            IF ( DABS(A(I,K)) .GT. DABS(A(M,K)) ) M=I
10       CONTINUE
         IPIV(K)=M
         IF (M.NE.K) IPIV(N)=-IPIV(N)
         P=A(M,K)
         A(M,K)=A(K,K)
         A(K,K)=P
         DET=DET*P
         IF (P.EQ.0.0) GO TO 60
C
C..... Beregner multipliakatorer                .................
         DO 30 I=KP1,N
            A(I,K)=-A(I,K)/P
30       CONTINUE
C
C..... Ombytting og eliminasjon av kolonner     .................
         DO 50 J=KP1,N
            T=A(M,J)
            A(M,J)=A(K,J)
            A(K,J)=T
            IF (T.EQ.0.0) GO TO 50
            DO 40 I=KP1,N
               A(I,J)=A(I,J)+A(I,K)*T
40          CONTINUE
50       CONTINUE
60    CONTINUE
C
70    DET =DET*A(N,N)*FLOAT(IPIV(N))
C
      RETURN
      END
