      SUBROUTINE LUSOL2(A,N,NA,IPIV,B)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                               C
C  David Di Ruscio                        NTH/ITK 13 oktober 88 C
C                                                               C
C  BESKRIVELSE                                                  C
C  LUSOL2 loeser L*U*x = b ved forrovereliminasjon og deretter  C
C  bakovereliminasjon.                                          C
C                                                               C
C  INN                                                          C
C  A    : DOUBLE PRECISION matrise med dimensjon (NA,N)         C
C         A inneholder LU faktorisert av LUDEC2                 C
C  N    : INTEGER dimensjonen paa problemet                     C
C  IPIV : INTEGER array med dimensjon (N). Vektor som inneholderC
C         pivoteinndekser gitt fra LUDEC2.                      C
C  B    : DOUBLE PRECISION vektor med dimensjon (N). Inneholder C
C         hoeyre siden i problemet. Inneholder loesningen x     C
C         ved utgangen.                                         C
C  UT                                                           C
C  B    : Loesningen paa problemet.                             C
C                                                               C
C  BENYTTEDE SUBRUTINER : Ingen                                 C
C                                                               C
C  REFERANSE                                                    C
C  "MATRIX Computations",Golub,Van Loan (OXFORD 83)             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NA,N,IPIV(N)
      DOUBLE PRECISION A(NA,N),B(N)
C.... Interne parametre                         .................
      INTEGER NM1,K,KB,KP1,KM1,M,I
      DOUBLE PRECISION T
C
C.... Forrover eliminasjon  ( loeser Ly = b )   .................
      IF (N.EQ.1) GO TO 15
      NM1=N-1
      DO 10 K=1,NM1
         KP1=K+1
         M=IPIV(K)
         T=B(M)
         B(M)=B(K)
         B(K)=T
         DO 20 I=KP1,N
            B(I)=B(I)+A(I,K)*T
20       CONTINUE
10    CONTINUE
C.... Bakover substitusjon ( loeser Ux = y )    .................
      DO 30 KB=1,NM1
         KM1=N-KB
         K=KM1+1
         B(K)=B(K)/A(K,K)
         T=-B(K)
         DO 40 I=1,KM1
            B(I)=B(I)+A(I,K)*T
40       CONTINUE
30    CONTINUE
C
C.... Loesning dersom N=1                       .................
15    B(1)=B(1)/A(1,1)
C
      RETURN
      END
