        subroutine dfromin(nm,n,a,d,niter)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c***Start prologue fromin                                             c
c   Purpose                                                           c
c   FROMIN minimizes Frobenius norm of matrix a by applying           c
c   similarity 'scaling' with real diagonal matrices.                 c
c               d*a*dinv or d*da*dinv                                 c
c   where a and da are complex and real matrix respectively.          c
c   the scaling matrix is returned in d.                              c
c   On input                                                          c
c   a      complex double subscripted array of dimension (n,n)        c
c   nm     maximum leading dimension on a and da in calling prog.     c
c   job    have the following options                                 c
c   On output                                                         c
c   d      a array of dimension n containing the scaling              c
c   a      contains the product d*a*dinv (complex matrix)             c
c   da     contains the product d*da*dinv if job=1                    c
c   niter  the number of iterations used                              c
c   History                                                           c
c   Writen by S. Skogestad IKT/NTH , 20 mai 1985 for complex          c
c   matrixes only.                                                    c
c   Extended by D. Di Ruscio ITK/NTH 17 november 1988                 c
c   Rererences : On Pre-Conditioning of Matrices,E.E.Osborne          c
c                I.Assoc.Comut.Mach.,7,p.338-345 (1960)               c
c   routines called : (none)                                          c
c***end prologue fromin                                               c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        integer nm,n,niter
cut     complex a(nm,nm)
        double precision d(nm),a(nm,nm)
c***    local variables
        integer iosmax,ioserr,it
        real minval,epsosb,diter,rsum,csum
        logical change
c
        parameter ( iosmax = 500 )
        parameter ( epsosb = 1.0e-10,minval = 1.0e-30 )
        ioserr = 0
c
        do 50 ii=1,n
        d(ii)=1.0
50      continue
c
        do 500 it=1,iosmax
        change=.false.
c-------------------------------------------------------
        do 400 ii=1,n
                rsum=0.
                csum=0.
           do 200 l=1,n
                if(ii.ne.l) then
c               rsum = cabs(cmplx(cabs(a(ii,l)),rsum))
                rsum = cabs(cmplx(abs(a(ii,l)),rsum))
c               csum = cabs(cmplx(cabs(a(l,ii)),csum))
                csum = cabs(cmplx(abs(a(l,ii)),csum))
                endif
200        continue
          rsum = sqrt(rsum)
          csum = sqrt(csum)
c
          if(rsum.ne.csum) then
             if (csum.le.(rsum*minval)) then
                 diter = minval 
             elseif (rsum.le.(csum*minval)) then
                 diter = 1/minval       
             else
                 diter = csum/rsum
             endif
             do 300 l=1,n
                 if(ii.ne.l) then
                 a(l,ii)=a(l,ii)/diter
                 a(ii,l)=a(ii,l)*diter
                 endif
300          continue
             d(ii)=d(ii)*diter
             if(abs(diter-1.0).gt.epsosb) change = .true.
           endif
400     continue
c------------------------------------------------------
        if(.not.change) goto 600
500     continue
c not converged....
        ioserr=1  
c converged........
600     continue
        niter = it
c
c
        return
        end



