      SUBROUTINE DLYP(A,Q,X,NX,N,WA,WB,NP,IW,INFO)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DLYP                                         C
C   AUTHOR : David Di Ruscio                                    C
C                                                               C
C    PURPOSE                                  T                 C
C    DLYP solve the matrix Liapunov equation A X + X A = -Q     C
C    directly with first to transform the problem to a linear   C
C    set of equatios  AMOD*p = b where dim(AMOD)=NP x NP        C
C    and dim(p)=dim(b)=NP where NP = N(N+1)/2. The solution X   C
C    is then found from back transforming p.                    C
C                                                               C
C    ON INPUT                                                   C
C    N integer actual problem dimension                         C
C    A  double precision(nx,nx)                                 C
C    Q  double precision(nx,nx)                                 C
C    NP integer dimension on work arrays. NP = NX(NX+1)/2       C
C    WA double precision(np,np) work array                      C
C    WB double precision(np)    work array                      C
C    IW integer(np)             work array                      C
C    ON OUTPUT                                                  C
C    X  double precision(nx,nx) solution matrix.                C
C                                                               C
C    INFO   INFO = 0 ---> Succesful operation                   C
C           INFO = 1 ---> System singular, no solution exists.  C
C           INFO = 2 ---> Work arrays to small.                 C
C                                                               C
C    ROUTINES CALLED : REDMOD, AXBSOL2, MVTRAN                  C
C***END PROLOGUE DLYP                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,NI,IW(NP),INFO
      DOUBLE PRECISION A(NX,NX),Q(NX,NX),X(NX,NX)
      DOUBLE PRECISION WA(NP,NP),WB(NP),DET
      INTEGER IW1

      INFO = 0
C... Check if work arrays big enoufgh
      IW1 = N*(N+1)/2
      IF (NP.LT.IW1) THEN
       INFO = 2
       GO TO 1000
      ENDIF

C... Make modified system matrix WA and right hand side vector WB
      CALL REDMOD(A,Q,NX,N,WA,WB,NP,NI)

C... Solve linear set of equations WA x = WB
C     CALL DGEFA(WA,NP,NI,IW,INFO)
C     IF (INFO.NE.0) THEN
C      INFO = 1
C      GO TO 1000
C     ENDIF
C     CALL DGESL(WA,NP,NI,IW,WB,1)

      CALL AXBSOL2(WA,NI,NP,WB,DET,1,1,IW,INFO)
      IF (INFO.NE.0) THEN
       INFO = 1
       GO TO 1000
      ENDIF

C... Transform vector WB to solution matrix X
      CALL MVTRAN(X,WB,N,NX,NP,3)

1000  RETURN
      END

      SUBROUTINE REDMOD(A,Q,NX,N,AUT,B,NP,NI)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                              C
C   Rutinen setter opp den modifiserte A matrisen for direkte  C
C   l|sning av Liapunovs ligning X*A + AT*X =-Q ved } l|se     C
C   likningssystemet AUT*x = b. Rutinen tar hensyn til at      C
C   X er symmetrisk.                                           C
C                                                              C
C   Input  : AIN(N,N)  , N er dimensjon p} AIN                 C
C            Q(N,N)                                            C
C   Output : AUT(NI,NI), NI = N*(N+1)/2  er dimensjon p} AUT   C
C            B(NI)                                             C
C                                                              C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,N,M,NI,K,L,LM
      DOUBLE PRECISION A(NX,NX),Q(NX,NX),AUT(NP,NP),B(NP)    

      M  = 0
      NI = N*(N+1)/2
C....Resetter arrayene .........................................

      DO 1 I=1,NI
         B(I)=0.0
         DO 2 J=1,NI
            AUT(I,J)=0.0
2        CONTINUE
1     CONTINUE

C....Setter opp modifisert matrise AUT=(AUT,B), dim (NI x NI+1).
      DO 10 I=1,N
         DO 20 J=I,N

C...M teller fra 1 til NI ...............
            M=M+1      

            IF (I.EQ.J) THEN
             B(M)=-Q(I,J)/2.0
            ELSEIF (I.NE.J) THEN
             B(M)=-Q(I,J)    
            ENDIF
             
            LM=0
            DO 30 K=1,N
               DO 40 L=K,N

C...LM teller fra 1 til NI .....................................
           LM=LM+1

           IF (K.EQ.I.AND.L.NE.J    ) THEN
            AUT(M,LM)=A(L,J)
           ELSEIF (K.NE.I.AND.L.EQ.J) THEN             
            AUT(M,LM)=A(K,I)
           ELSEIF (K.NE.I.AND.L.NE.J.AND.K.EQ.J.AND.L.NE.I) THEN
            AUT(M,LM)=A(L,I)
           ELSEIF (K.NE.I.AND.L.NE.J.AND.K.NE.J.AND.L.EQ.I) THEN
            AUT(M,LM)=A(K,J)
           ELSEIF (K.NE.I.AND.L.NE.J.AND.K.NE.J.AND.L.NE.I) THEN
            AUT(M,LM)=0.0
           ELSEIF (K.EQ.I.AND.L.EQ.J.AND.K.EQ.J.AND.L.EQ.I) THEN
            AUT(LM,LM)=A(K,I)
           ELSEIF (K.EQ.I.AND.L.EQ.J.AND.K.NE.J.AND.L.NE.I) THEN
            AUT(M,LM)=A(K,I)+A(L,J)
           ENDIF          

40             CONTINUE
30          CONTINUE
                     
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE MVTRAN(X,P,N,NX,NP,IFLAG)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   IFLAG=1                                                     C
C   Rutinen transformerer en n x n matrise X til en vektor      C
C   p med dimensjon n*n.                                        C
C   pt =(X(1,1),X(1,2),..,X(N1,N),X(2,1),X(2,2),..,X(2,N),..)   C
C                                                               C
C   X  matrise med dim N x N                                    C
C   P  vektor med dim NN = N*N                                  C
C                                                               C
C   IFLAG=2                                                     C
C   Rutinen transformerer en vektor P med dim.=N*N=NN til en    C
C   N x N matrise X.                                            C
C   IFLAG = 3                                                   C
C   Transformerer en n(n+1)/2 vector P til en symmeterisk       C
C   n x n matrise X.                                            C
C                                                               C
C   KALLER ingen andre rutiner                                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,NX,NP,IFLAG
      INTEGER I,J,IN,MM
      DOUBLE PRECISION X(NX,NX),P(NP)

C1...Transformerer en matrise X til en vektor P ................
      IF (IFLAG.EQ.1) THEN
       DO 10 I=1,N
          DO 20 J=1,N
             P(J+(I-1)*N)=X(I,J)
CUT          X(I,J)=0.0
20        CONTINUE
10     CONTINUE

C2...Transformerer en vektor P til en matrise X ................
      ELSEIF (IFLAG.EQ.2) THEN
       DO 30 I=1,N
          DO 40 J=1,N
             IN=J+(I-1)*N
             X(I,J)=P(IN)
CUT          P(IN)=0.0
40        CONTINUE
30     CONTINUE

C3...Transformerer en N(N+1)/2 vektor P til en N x N symmetrisk
C....matrise X        
      ELSEIF (IFLAG.EQ.3) THEN
       MM=0
       DO 50 I=1,N
          DO 60 J=I,N
             MM=MM+1
             X(I,J)=P(MM)
             X(J,I)=X(I,J)
             P(MM)=0.0
60        CONTINUE
50     CONTINUE

      ENDIF

      RETURN
      END

