      SUBROUTINE NLPINT(N,X,F,XL,XU,WA,LWA,KWA,LKWA,IPRINT,IFAIL)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE NLPINT                                        C
C   PURPOSE                                                      C
C   This is a interface routine to the optimization routine      C
C   NLPQ0.                                                       C
C                                                                C
C***END PROLOGUE NLPINT                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,LWA,LKWA,KWA(LKWA),IPRINT,IFAIL
      DOUBLE PRECISION X(N),F,XL(N),XU(N),WA(LWA)
C..Local parameters                                     ..........
      INTEGER M,ME,MAXIT,MAXFUN,NC,IGRAD
      COMMON /IDB/ NC,IGRAD
      NC = 0
      IGRAD = 0

C... Initialize parameters before call to NLPQ0         ..........
      CALL BNDSPE(M,ME,N,XL,XU,MAXFUN,MAXIT)

C... Call NLPQ0 to solve the problem                    ..........
      CALL NLPQL0(M,ME,N,X,F,XL,XU,IPRINT,IFAIL,WA,LWA,KWA,LKWA,
     +                  MAXIT,MAXFUN)

C... Print info                                         ..........
      IF (IPRINT.GE.1 .AND. IPRINT.LE.4) THEN
       WRITE(*,'(5X,A,1X,I5)')
     + 'Actual no. of function call  is : ',NC
      ENDIF
       
      RETURN
      END

      SUBROUTINE BNDSPE(M,ME,N,XL,XU,MAXFUN,MAXIT)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                     C
C   To initialize simple bounds on parameter vector X and       C
C   prepare other parameters before call to NLPQ                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER M,ME,N,MAXFUN,MAXIT
      DOUBLE PRECISION XL(N),XU(N)
      DOUBLE PRECISION EPS        
      COMMON /CMACHE/ EPS

C... Prepare parameters before call to NLPQ
C...Machine epsilon shold be computed
      EPS = 0.13E-16
      M   = 0
      ME  = 0
      DO 10 I=1,N
         XL(I) = -10000.0
         XU(I) =  10000.0
10    CONTINUE
      MAXFUN = 10
      MAXIT  = 100

      RETURN
      END

      SUBROUTINE NLGRAD(M,ME,MMAX,N,F,G,DF,DG,X,ACTIVE,WA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                         C
C   To evaluate the gradient vector DF = df/dx numericaly.          C
C   This routine is called from NLPQL0 or NLPQL1.                   C
C   If analytical gradient is avaiable it should be specified here. C
C   ROUTINES CALLED : DFJAC1                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER M,ME,MMAX,N
      DOUBLE PRECISION F,G(MMAX),DF(N),DG(MMAX,N),X(N),WA(MMAX)
      LOGICAL ACTIVE(MMAX)

C...Compute gradient vector DF =df/dx forward difference     ........
C...F(X) at X is known input --> DF = (F(X+Hi) - F(X))/Hi
      CALL DGRAD1(X,DF,F,N,N)

      RETURN
      END

      SUBROUTINE DGRAD1(X,GRAD,F1,N,MAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                     C
C   To numerically compute the gradient vector dF/dx at X.      C
C   F1 must contain the functional value at X.                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,MAX,I
      DOUBLE PRECISION X(MAX),GRAD(MAX),F1
      DOUBLE PRECISION DEPS,H,F2       
      PARAMETER (H = 1.0D-5)           
      INTEGER NC,IGRAD
      COMMON /IDB/ NC,IGRAD
      IGRAD = IGRAD + 1

      DO 10 I=1,N
         DEPS    = DABS( H*X(I) )
         IF (DEPS.LT.H) DEPS = H
         X(I)    = X(I) + DEPS
         CALL FX(F2,X,N)
         GRAD(I) = (F2 - F1)/DEPS
         X(I)    = X(I) - DEPS
10    CONTINUE

      RETURN
      END


      SUBROUTINE NLFUNC(M,ME,MMAX,N,F,G,X,ACTIVE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    PURPOSE                                                       C
C    To evaluate functional value. This routine is called from     C
C    NLPQ0 or NLPQ1.                                               C
C    Routines called FX.                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER M,ME,MMAX,N
      DOUBLE PRECISION F,G(MMAX),X(N)
      LOGICAL ACTIVE(MMAX)

C... Compute functional value F(X)                        .........
      CALL FX(F,X,N)

      RETURN
      END

