      SUBROUTINE DFJAC(FX,X,T,DJAC,W,N,M,NI,NJ)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DFJAC1                                       C
C   AUTHOR : David Di Ruscio            ITK/NTH 27/3-1989       C
C                                                               C
C   PURPOSE : To compute the Jacobian approximation to dF/dX    C
C             at (X,T) placed in DJAC for N nonlinear functions C
C             F with M dependent arguments X.                   C
C             DJAC = ( (F(X+EPSi,T)-F(X,T))/EPSi) is used.      C
C             EPSi= MAX( 1E-7,ABS((1E-7)*X(i)) ) for i = 1,...M C
C             Actual number of functional evaluation is N+1.    C
C   EXTERNAL FX , SUBROUTINE FX(F,X,T,N,M) is used to compute   C
C             F=F(X,T) for given arguments X,T. User supplied.  C
C   ON INPUT                                                    C
C   F   : DOUBLE PRECISION(NI) F= F(X,T) with actual dim. N.    C
C   X   : DOUBLE PRECISION(NJ) argument with actual dim. M.     C
C   T   : DOUBLE PRECISION argument.                            C
C   W   : DOUBLE PRECISION(1) work array of size 2*NI.          C
C   N   : INTEGER number of equations F.                        C
C   M   : INTEGER number of arguments in X.                     C
C   ON OUTPUT                                                   C
C   DJAC: DOUBLE PRECISION(NI,NJ) Jacobian matrix at (X,T).     C
C         with actual dimension (N,M).                          C
C   ROUTINES CALLED : FX(F,X,T,N,M)                             C
C***END PROLOGUE DFJAC1                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,M,NI,NJ,N1,N2,IDEB
      DOUBLE PRECISION X(NJ),T,DJAC(NI,M),W(1),DEPS,EPS
      PARAMETER (EPS = 1.0D-5)
      COMMON IDEB
      EXTERNAL FX

C1... Initialize ponters                                .........
      N1 = 1
      N2 = N1+N

C2... Compute function value F(X,T) = W(N) at (X,T)     .........
      CALL FX(W(N1),X,T,N,M)

C3... Compute Jacobian matrix                           .........
      DO 10 I=1,M

C... Compute parturbation value DEPS for row I          .........
         DEPS = DABS( EPS*X(I) )
         IF ( DEPS .LT. EPS ) DEPS = EPS
         IF (IDEB.EQ.1) THEN
          WRITE(*,'(2X,A,I4,A,E16.9)')'EPS(',I,') = ',DEPS
         ENDIF

         X(I) = X(I) + DEPS
         CALL FX(W(N2),X,T,N,M)
         DO 20 J=1,N
            DJAC(J,I) = ( W(N2+J-1) - W(N1+J-1) )/DEPS
20       CONTINUE
         X(I) = X(I) - DEPS
10    CONTINUE

      RETURN
      END
