      SUBROUTINE NLPINT(N,X,F,XL,XU,WA,LWA,KWA,LKWA,IPRINT,IFAIL)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE NLPINT                                        C
C   PURPOSE                                                      C
C   This is a interface routine to the optimization routine      C
C   NLPQ0.                                                       C
C                                                                C
C***END PROLOGUE NLPINT                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,LWA,LKWA,KWA(LKWA),IPRINT,IFAIL,IJOB
      DOUBLE PRECISION X(N),F,XL(N),XU(N),WA(LWA)
C..Local parameters                                     ..........
      INTEGER M,ME,MAXIT,MAXFUN,NC,IGRAD
      COMMON /IDB/ NC,IGRAD
      EXTERNAL FX
      NC    = 0
      IGRAD = 0

C... Initialize parameters before call to NLPQ0         ..........
      CALL BNDSPE(M,ME,N,XL,XU,MAXFUN,MAXIT,IJOB)

C.. Estimate optimal perturbation if desired            ..........
C   to be used for numericaly estimating the gradient vector
      IF (IFAIL.EQ.1 .OR. IJOB.EQ.1) THEN
       CALL INACCH(F,X,N,FX)
      ENDIF

C... Call NLPQ0 to solve the problem                    ..........
      CALL NLPQL0(M,ME,N,X,F,XL,XU,IPRINT,IFAIL,WA,LWA,KWA,LKWA,
     +                  MAXIT,MAXFUN)

C... Print info                                         ..........
      IF (IPRINT.GE.1 .AND. IPRINT.LE.4) THEN
       WRITE(*,'(5X,A,1X,I5)')
     + 'Actual no. of function call  is : ',NC
      ENDIF
       
      RETURN
      END

      SUBROUTINE BNDSPE(M,ME,N,XL,XU,MAXFUN,MAXIT,IJOB)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                     C
C   To initialize simple bounds on parameter vector X and       C
C   prepare other parameters before call to NLPQ                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER M,ME,N,MAXFUN,MAXIT,IJOB
      DOUBLE PRECISION XL(N),XU(N)
C.. Common with NLPQ0
      DOUBLE PRECISION EPS        
      COMMON /CMACHE/ EPS
C.. Common with numerical gradient computation routine DGRAD1
      DOUBLE PRECISION H,DH
      COMMON /DSTEP/ H    

C.. Local parameters for COMCHK
      INTEGER IDAT,IFLAG,IPOS
      PARAMETER (IDAT = 12)
      CHARACTER CDAT(IDAT)*6,CJOB*10
      DATA CDAT( 1)/'xl'/,    CDAT( 2)/'xu'/,   CDAT( 3)/'maxfun'/,
     +     CDAT( 4)/'maxit'/, CDAT( 5)/'estim'/, CDAT( 6)/'step'/,
     +     CDAT( 7)/'dum3'/,  CDAT( 8)/'dum4'/, CDAT( 9)/'dum5'/,
     +     CDAT(10)/'help'/,  CDAT(11)/'quit'/, CDAT(12)/'0'/


C... Prepare default parameters before call to NLPQ     ..........
C...Machine epsilon shold be computed
      EPS = 0.13E-16
      M   = 0
      ME  = 0
      DO 10 I=1,N
         XL(I) = -100000.0
         XU(I) =  100000.0
10    CONTINUE
      MAXFUN = 10
      MAXIT  = 100
      IJOB   = 0
      H      = 1.0D-5

C... Interactice specifications
      ISVAR = IYESANS('Specify optimization parameters ?',1)           
      IF (ISVAR.EQ.2) THEN

C.. Read job
       CJOB = 'xl '
100    CALL RCSTR('Specify parameters :',CJOB,1,1,IIII)

C.. Decode job
       CALL COMCHK(CJOB,CDAT,IDAT,IFLAG,IPOS)
       IF (IFLAG.NE.0) THEN
        CALL COMLIS('Available : ',CDAT,IDAT)
        CJOB = 'quit'
        GO TO 100
       ELSE

        IF (IPOS.EQ.1) THEN
         CALL DRVEC('Lower parameter bound XL',XL,N,N)
         CJOB = 'xu'
         GO TO 100
        ELSEIF (IPOS.EQ.2) THEN
         CALL DRVEC('Upper parameter bound XU',XU,N,N)
         CJOB = 'maxfun'
         GO TO 100
        ELSEIF (IPOS.EQ.3) THEN
300      CALL NREAD('Max function call during line search ?',
     +               MAXFUN,1,1,IIII)
         IF (MAXFUN.LT.0) GO TO 300
         CJOB = 'maxit'
         GO TO 100

        ELSEIF (IPOS.EQ.4) THEN
400      CALL NREAD('Max number of iterations ?',MAXIT,1,1,IIII)
         IF (MAXIT.LT.0) GO TO 400
         CJOB = 'estim'
         GO TO 100

        ELSEIF (IPOS.EQ.5) THEN
500      ISVAR = IYESANS('Estimate optimal perturbations ?',1)   
         IF (ISVAR.EQ.1) THEN
          IJOB = 0
         ELSEIF (ISVAR.EQ.2) THEN
          IJOB = 1
         ENDIF
         IF (IJOB.LT.0 .OR. IJOB.GT.1) GO TO 500
         CJOB = 'step'
         GO TO 100

        ELSEIF (IPOS.EQ.6) THEN
600      DH = H
         CALL DREAD('Specify initial stepsize H',DH,1,1,IIII)
         IF (DH.LT.0.0) GO TO 600
         H = DH
         CJOB = 'quit'
         GO TO 100
        ELSEIF (IPOS.EQ.7) THEN
         WRITE(*,'(5X,A)')'Dummy parameter'
         CJOB = 'quit'
         GO TO 100
        ELSEIF (IPOS.EQ.9) THEN
         WRITE(*,'(5X,A)')'Dummy parameter'
         CJOB = 'quit'
         GO TO 100
        ELSEIF (IPOS.EQ.10) THEN
         WRITE(*,'(5X,A)')'           <  H E L P > '                 
         WRITE(*,'(5X,A)')
     +   'Step size for numerical gradient computation     :  step'
         CJOB = 'quit'
         GO TO 100
        ELSEIF (IPOS.EQ.11) THEN
         GO TO 1000
        ELSEIF (IPOS.EQ.12) THEN
         GO TO 1000
        ENDIF
       ENDIF

      ENDIF

1000  RETURN
      END

      SUBROUTINE NLGRAD(M,ME,MMAX,N,F,G,DF,DG,X,ACTIVE,WA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                         C
C   To evaluate the gradient vector DF = df/dx numericaly.          C
C   This routine is called from NLPQL0 or NLPQL1.                   C
C   If analytical gradient is avaiable it should be specified here. C
C   ROUTINES CALLED : DFJAC1                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER M,ME,MMAX,N
      DOUBLE PRECISION F,G(MMAX),DF(N),DG(MMAX,N),X(N),WA(MMAX)
      LOGICAL ACTIVE(MMAX)

C...Compute gradient vector DF =df/dx forward difference     ........
C...F(X) at X is known input --> DF = (F(X+Hi) - F(X))/Hi
      CALL DGRAD1(X,DF,F,N,N)

      RETURN
      END

      SUBROUTINE DGRAD1(X,GRAD,F1,N,MAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                     C
C   To numerically compute the gradient vector dF/dx at X.      C
C   F1 must contain the functional value at X.                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,MAX,I
      DOUBLE PRECISION X(MAX),GRAD(MAX),F1
      DOUBLE PRECISION DEPS,F2         
CUT   PARAMETER (H = 1.0D-5)           
C...Common initial stepsize
      DOUBLE PRECISION H
      COMMON /DSTEP/ H
C...Common for statistic
      INTEGER NC,IGRAD
      COMMON /IDB/ NC,IGRAD
C...Common from program calling nlpint
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

      IGRAD = IGRAD + 1

      DO 10 I=1,N
         DEPS    = DABS( H*X(I) )
         IF (DEPS.LT.H) DEPS = H
         X(I)    = X(I) + DEPS

CUT      WRITE(*,'(5X,A,I3,A,D30.23)')'Perturbation H(',I,' ) = ',DEPS

         CALL FX(F2,X,N)
         GRAD(I) = (F2 - F1)/DEPS
         X(I)    = X(I) - DEPS
10    CONTINUE

      RETURN
      END


      SUBROUTINE NLFUNC(M,ME,MMAX,N,F,G,X,ACTIVE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    PURPOSE                                                       C
C    To evaluate functional value. This routine is called from     C
C    NLPQ0 or NLPQ1.                                               C
C    Routines called FX.                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER M,ME,MMAX,N
      DOUBLE PRECISION F,G(MMAX),X(N)
      LOGICAL ACTIVE(MMAX)

C... Compute functional value F(X)                        .........
      CALL FX(F,X,N)

      RETURN
      END


