      subroutine dhqr5(a,u,na,n,itype,ipos,nb1,ieig)    
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***begin prologue dhqr5                                         c
C   author : david di ruscio                    itk/nth 30/9-89  c
C   purpose                                                      c
C   dhqr5 reorder a matrix in real upper schur form such         c
C   that block in position (ipos) is moved to position           c
C   in lower right corner. The order of the other blocks is      c
C   unchanged on output.                                         c
C                                                                c
C   on input                                                     c
C   a    double precision(na,n) matrix in real schur form.       c
C   u    double precision(na,n) orthogonal transformation.       c
C   na   integer leading dimension of a and u in calling program c
C   n    integer actual dimension of a.                          c
C   ipos integer position of block to be moved.                  c
C   type integer(n) vector                                       c
C        an integer vector of length n whose                     c
C        i-th entry is                                           c
C        0  if the i-th eigenvalue is real,                      c
C        1  if the i-th eigenvalue is complex with               c
C           positive imaginary part,                             c
C        2  if the i-th eigenvalue is complex with               c
C           negative imaginary part,                             c
C   on output                                                    c
C   a    double precision(na,n) reordered schur form.            c
C   u    double precision(na,n) orthogonal transformation.       c
C        Postmultiplied into u from input.                       c
C   nb1  integer size of block now in position lower right       c
C        corner.                                                 c
C                                                                c
C   routines called : exchng (from Stewarts hqr3 software)       c
C***end dhq5                                                     c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      integer na,n,itype(n),ipos,nb1,ieig(n)
      double precision a(na,n),u(na,n)        
      double precision eps        
      integer ncomp,nblock,ichng,istep,i,iii,nb2
      logical fail

c...  compute macine epsilon
      call maceps(eps)

c...  count number of blocks from position ipos to t22 (nblock)
c     and neccesary no. of excahnges (ichng).
      ncomp = 0
      do 5 i=ipos,n
         if (itype(i).eq.1.or.itype(i).eq.2) then
          ncomp = ncomp + 1
         endif
5     continue
      if (ncomp.ne.0) then
       ncomp=ncomp/2
      endif
      nblock = n - ipos + 1 - ncomp
      ichng = nblock-1    

c... determine size of block to be moved
      if (itype(ipos).eq.0) then
       nb1   = 1
      endif
      if (itype(ipos).eq.1 .or. itype(ipos).eq.2) then
       nb1   = 2
      endif

c...  move block in position ipos to position T22
      istep = 1
      do 10 iii = 1,ichng

c...     specify size of next block
         if (nb1.eq.1) then
          if (itype(ipos+1).eq.0) then
           istep = 1
           nb2   = 1
          endif
          if (itype(ipos+1).eq.1.or.itype(ipos+1).eq.2) then
           istep = 2
           nb2   = 2
          endif
         elseif (nb1.eq.2) then
          if (itype(ipos+2).eq.0) then
           istep = 1
           nb2   = 1
          endif
          if (itype(ipos+2).eq.1.or.itype(ipos+2).eq.2) then
           istep = 2
           nb2   = 2
          endif
         endif

c...     exching block in position (ipos) and (ipos + istep)
         call exchng(a,u,n,ipos,nb1,nb2,eps,fail,na,na)
         if (fail) then   
          ieig(ipos)   = -1
          ieig(ipos+1) = -1
         endif
         call dzerhes(a,na,n)

c...     update position (ipos) of block to be moved
         ipos = ipos + istep

10    continue

1000  return
      end
