      subroutine dricsol(a,h,q,n,na,f,u,nf,er,ei,dscl,itype,deps,info)
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***begin prologue dricsl                                               c
C   author: David Di Ruscio                     ITK/NTH 9/11-1989       c
C   purpose                                                             c
C   dricsl solves the Algebraic Riccati Equation (ARE)                  c
C                   T                                                   c
C                  A R + R A -R H R + Q = 0                             c
C                                                                       c
C   by Laubs Schur method. The corresponding Hamiltonian matrix F to    c
C   the ARE are reduced to Upper Real Schur form. ie the Hamiltonian    c
C   matrix and the relation to the ARE solution R is given by,          c
C                                                                       c
C                           (  A  -H  )         -1                      c
C                       F = (       T ) ,  (I 0)            (I 0)       c
C                           ( -Q  -A  )    (R I)  F = (eig) (R I)       c
C                                                                       c
C   Both unique positive and negative solution to the ARE corresponding c
C   to stable and unstable eigenvalues, respectively, can be computed.  c
C   Se input parameter info.                                            c
C                                                                       c
C   on input                                                            c
C   a,h,q  double precision(na,n) matrixes in ARE. h overwritten with   c
C          ARE solution R on output, a and q unchanged.                 c
C   n      integer actual dimension of a,h and q.                       c
C   na     integer leading dimension of a,h and q in calling program.   c
C   f,u    double precision(nf,*) work arrays used for computation      c
C          involving the Hamiltonian matrix associated with the ARE.    c
C          f and u must be declared at least (2n,2n) = (n+n,n+n)        c
C   nf     integer leading dimension on f and u in calling program.     c
C          Must be declared at least nf = na + na.                      c
C   info   integer.   info = 0 --> Unique positive solution to the ARE  c
C                                  corresponding to stable eigenvalues  c
C                                  are computed.                        c
C                     info = 1 --> Unique max. negativ definit solution c
C                                  to the ARE corresponding to unstable c
C                                  eigenvalues are computed.            c
C   dscl   double precision(*) work vector dimensioned at least n+n.    c
C          On output dscl(1) contains an estimate of the condition      c
C          number of the set of linear equations solved.                c
C   itype  integer(*) work vector dimensioned at least n+n.             c
C   deps   double precision machine precision. Shuld be set to the      c
C          smallest machine precision such that deps+1 .ge. 1.          c
C          (typically deps = 1.0e-16)                                   c
C                                                                       c
C   on output                                                           c
C   h      double precision (n x n) array containing the unique         c
C          positive solution of the ARE.                                c
C   er,ei  double precision(n+n) vectors containig the eigenspectrum    c
C          of the Hamiltonian. The n first values contains the real     c
C          and imaginary parts of the closed loop spectrum, ie the      c
C          eigenvalues of (A-H*R).                                      c
C   info   integer which contains an error singnal whose value is,      c
C          info = 0    ---> succesfull operation.                       c
C          info = 1    ---> eigenvalue(s) not proper isolated in HQR3.  c
C          info = 2    ---> reordering eigenspectrum failed.            c
C          info = 3    ---> linear system of equation singular          c
C                           ( system  tr(U11) R = tr(U21) singular )    c
C                                                                       c
C   routines called: hqr3, dschur, eigord, eigval, mlineq               c
C***end prologue dricsl                                                 c
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      integer n,na,nf,itype(*),info
      double precision a(na,n),h(na,n),q(na,n),f(nf,n+n),u(nf,n+n)
      double precision er(*),ei(*),dscl(*)
c... Local parameters
      double precision cond
      integer isol
      logical fail
c... Common block for debug purposes
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

      if (info.eq.0) then
       isol = 1
      elseif (info.eq.1) then
       isol = 2
       info = 0
      endif
      nn   = n + n
c1... Set up Hamiltonian matrix F
      IF (IPRT.EQ.1) THEN
       call dmaprt('DRICSOL A =',a,na,na,n,n,iout)
       call dmaprt('DRICSOL H =',h,na,na,n,n,iout)
       call dmaprt('DRICSOL Q =',q,na,na,n,n,iout)
      ENDIF

      call dhamc(a,h,q,f,n,na,na,na,nf)

      IF (IPRT.EQ.1) THEN
       call dmaprt('DRICSOL F =',F,nf,nf,nn,nn,iout)
      ENDIF

c2... Reduce F to upper real Schur form, and eigenvalues ordered
c     according to parameter isol.
      call dschur(f,u,er,ei,nf,nn,deps,dscl,itype,isol)
      do 10 i=1,n
         if (itype(i).eq.-1) then
          info = i
         endif
10    continue

cut   IF (IPRT.EQ.1) THEN
cut    call dmaprt('out dschur u =',u,nf,nf,nn,nn,iout)
cut   ENDIF

c3... Order eigenspectrum according to parameter isol
cut   if (isol.eq.1) then
c.... Order eigenspectrum with stable part in upper left corner
cut    call eigord(f,u,nn,deps,-1,itype,fail,nf,nf)
cut   elseif (isol.eq.2) then
c.... Order eigenspectrum with unstable part in upper left corner
cut    call eigord(f,u,nn,deps,1,itype,fail,nf,nf)
cut   endif

cut   if (fail) then
cut    info = 2
cut   endif
cut   call eigval(f,u,nn,1,nn,er,ei,nf,nf)

      IF (IPRT.EQ.1) THEN
       call dmaprt('ordered u =',u,nf,nf,nn,nn,iout)
       call deigprt('EIG(F)',er,ei,nn,iout)
      ENDIF

c4... Solve tr(U11) R = tr(U21) for ARE solution R stored in h
      do 20 i=1,n
         do 30 j=1,n
            f(i,j) = 0.0d0
            f(i,j) = u(j,i)
            h(i,j) = 0.0d0
            h(i,j) = u(j+n,i)
30       continue
20    continue

      IF (IPRT.EQ.1) THEN
       call dmaprt('tr(u11) =',f,nf,nf,n,n,iout)
       call dmaprt('tr(u21) =',h,na,na,n,n,iout)
      ENDIF

      call mlineq(nf,na,n,n,f,h,cond,itype,dscl)
      dscl(1) = cond
      if (cond.eq.0.0d0) then
       call dmzero(h,na,n,n)
       info = 3
      endif

      return
      end

      subroutine dricerr(info,iout)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C   purpose                                                        c
C   Error handler for dricerr.                                     c
C   on input                                                       c
C   info integer error signal                                      c
C   iout integer device to print messages to                       c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      integer info,iout
      if (info.eq.1) then
       write(iout,'(1x,a)')'Error -> Eigenvalue(s) not isolated'
       write(iout,'(1x,a)')'         in HQR3' 
      elseif (info.eq.2) then
       write(iout,'(1x,a)')'Error -> Ordering eigenspectrum failed'
      elseif (info.eq.3) then
       write(iout,'(1x,a)')'Error -> Linear system singular'
      endif
      return
      end

      subroutine dmzero(a,na,n,m)
      integer na,n,m
      double precision a(na,*)
      do 10 i=1,n
         do 20 j=1,m
            a(i,j) = 0.0d0
20       continue
10    continue
      return
      end
