      subroutine dschur(a,u,er,ei,na,n,deps,dscl,itype,iord)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C  author: David Di Ruscio                 26/9-1989             c
C  purpose                                                       c
C  calculate Upper Real Schur decomposition and order diagonal   c
C  blocks such that eigenspectrum is in descending order         c
C  iord.ne.1, or with stable eigenvalues appering first.         c
C                                                                c
C                          T         T                           c
C                     T = U A U , U U  = I                       c
C  on input                                                      c
C  a    double precision(na,n) general matrix to be reduced.     c
C  na   integer leading dimension of a and u in calling program. c
C  n    integer actual dimension of a.                           c
C  deps double precision scalar machine epsilon deps such that   c
C       deps+1 .ge. 1 (typically deps = 1.0e-16)                 c
C  dscl double precision(n) work vector used when balancing      c
C  iord integer parameter.                                       c
C       iord = 1  --> Eigenvalues and basis for eigenvector u    c
C                     is ordered in increasing order of the      c
C                     real eigenvalue parts, ie. stable eigen-   c
C                     values appears first.                      c
C       iord = 2  --> Eigenvalues and basis for eigenvector u    c
C                     is ordered in descending order of the      c
C                     real eigenvalue parts, ie. unstable eigen- c
C                     values appears first.                      c
C       iord.ne.1 --> Eigenvalues and basis for eigenvectors u   c
C                     is ordered in descending order of          c
C                     magintude, ie sorted after                 c
C                     abs( sqrt( er**2+ei**2) ).                 c
C  on output                                                     c
C  a    double precision(na,n) in upper real schur form.         c
C  u    double precision(na,n) orthogonal transformation         c
C       matrix.                                                  c
C er,ei double precision(n) real and imaginary part of eigen-    c
C       values respectively.                                     c
C itype integer(n) vector                                        c
C       an integer vector of length n whose                      c
C       i-th entry is                                            c
C       0  if the i-th eigenvalue is real,                       c
C       1  if the i-th eigenvalue is complex with                c
C          positive imaginary part,                              c
C       2  if the i-th eigenvalue is complex with                c
C          negative imaginary part,                              c
C       -1 if the i-th eigenvalue was not calculated             c
C          successfully.                                         c
C                                                                c
C   routines called: dbalanc, dorthes, dortran, dbalbak (eispack)c
C                    hqr3   (from Stewart's 1976 software)       c
C***end prologue dschur                                          c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      integer na,n,itype(n),iord
      double precision a(na,n),u(na,n),er(n),ei(n),dscl(n)
      double precision deps
      integer low,igh,low2,igh2

c1... scale (balance) a with diagonal scaling
      call dbalanc(na,n,a,low,igh,dscl)

c2... reduce a to upper Hessenberg form and accumulate 
c     transformation matrix in u

      low2 = 1
      igh2 = n
c     store information about transformation temporary in er(1:n)
      call dorthes(na,n,low2,igh2,a,er)
      call dortran(na,n,low2,igh2,a,er,u)

c3... compute upper real Schur decomposition and ordering
c     eigenspectrum in descending order.
      call dhqr3(a,u,n,low2,igh2,deps,er,ei,itype,na,na,iord)

c4... insert zeroes in lower part of T (a). not necessary, but
c     usful when T is to be presented.
      call dzerhes(a,na,n)

c5... baktransform u with scaling dscl from dbalanc.
      call dbalbak(na,n,low,igh,dscl,n,u)

      return
      end
