      SUBROUTINE DCCBUF(Z,ZD,IZ,NX,NU,NV,NY,IBUF,DEPS,JOB)
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***BEGIN PROLOGUE DCCBUF                                                 c
C   AUTHOR : DAVID DI RUSCIO                  ITK/NTH 19. november 1989   c
C                                                                         c
C   PURPOSE                                                               c
C   DCCBUF (Double precision Continnuous Controller BUFfer) solves        c
C   the LQG-continnuous controller problem.                               c
C   This is a buffer routine used only because SYNPACK handles real       c
C   aritmetic. This routine shuld be replaced by DDRSOL and DCLQEST.      c
C   See subroutine DDRSOL and DCLQEST for a complete description, a short c
C   documentation is given below.                                         c
C   The system matrixes is stored in the real array Z. All computations   c
C   is done with double precision aritmetic, thereby the double precision c
C   work array ZD.                                                        c
C                                                                         c
C   ON INPUT                                                              c
C   JOB   INTEGER parameter.   JOB = 1 ---> Solve LQ-controller problem.  c
C                              JOB = 2 ---> Solve LQ-estimator problem.   c
C   NX    INTEGER state dimension.               (JOB = 1 and JOB = 2)    c
C   NU    INTEGER control input dimension.       (JOB = 1)                c
C   NV    INTEGER disturbance input dimension.   (JOB = 2)                c
C   NY    INTEGER measurement output dimension.  (JOB = 2)                c
C   Z     REAL(*) array of sypack matrixes, must hold A, B, P and Q in    c
C         JOB = 1 --> Z must hold A, B, P and Q in                        c
C                     positions defined by IZ(*) and SYNPACK.INC.         c
C         JOB = 2 --> Z must hold A, D, W, V and C in                     c
C                     positions defined by IZ(*) and SYNPACK.INC.         c
C                                                                         c
C   ZD    DOUBLE PRECISION(*) work array. Must be declared at least,      c
C         JOB = 1 --> NZD = 11*NX*NX + 2*NX*NU + 6*NX + NU*NU + 1         c
C         JOB = 2 --> NZD = 12*NX*NX + NY*NX + 2*NV*NX + 6*NX + NY*NY     c
C                            + NV*NV + 1                                  c
C         REMARK :                                                        c
C         Use subroutine DBUFCHK to check if ZD is big enufgh before      c
C         call to this routine.                                           c
C   IBUF  INTEGER(*) work array. Must be declared at least 2*NX.          c
C   DEPS  DOUBLE PRECISION machine precision.                             c
C                                                                         c
C   ON OUTPUT                                                             c
C   Z     REAL array holding solutions R and G or X and K                 c
C         positions according to IZ(*) and SYNPACK.INC.                   c
C                                                                         c
C   ROUTINES CALLED: DSCOPZ  --> Transform between real and double prec.  c
C                    DDRSOL  --> Solve LQ-contrller problem.              c
C                    DCLQEST --> Solve LQ-estimator problem.              c
C                    DCCERR  --> Print error messages.                    c
C   INCLUDE FILES  : SYNPACK.INC                                          c
C***END PROLOGUE DCCBUF                                                   c
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER IZ(*),NX,NU,NV,NY,INFO,IBUF(*),JOB
      REAL Z(*)              
      DOUBLE PRECISION ZD(*),DEPS
C...Local par.
      INTEGER NDIM,IXX,IUX,IUU,IDA,IDB,IDP,IDQ,IDR,IDG
      INTEGER IYX,IXV,IYY,IVV,IDD,IDW,IDV,IDC,IDX,IDK
      INCLUDE 'SYNPACK.INC'

C...Set pointers to matrices in REAL stack Z(*)
      NA  = IZ(IA)
      NB  = IZ(IB)
      NC  = IZ(IC)
      ND  = IZ(ID)
      NR  = IZ(IR)
      NG  = IZ(IG)
      NXX = IZ(IX)
      NKA = IZ(IKA)
      NP  = IZ(IP)
      NQ  = IZ(IQ)
      NVV = IZ(IV)
      NW  = IZ(IW)

C1..  Copy REAL data in array Z to double precision vector ZD
C...  JOB=1 --> LQ-controller problem, data is (A,B,P and Q).
      IF (JOB.EQ.1) THEN
       IXX = NX*NX
       IUX = NU*NX
       IUU = NU*NU
       IDA = 1
       IDB = IDA + IXX
       IDP = IDB + IUX
       IDQ = IDP + IUU
       CALL SDCOPZ(Z(NA),ZD(IDA),IXX,1)
       CALL SDCOPZ(Z(NB),ZD(IDB),IUX,1)
       CALL SDCOPZ(Z(NP),ZD(IDP),IUU,1)
       CALL SDCOPZ(Z(NQ),ZD(IDQ),IXX,1)

C... JOB=2 --> LQ-estimator problem, data is (A,D,W,V,C)
      ELSEIF (JOB.EQ.2) THEN
       IXX = NX*NX
       IYX = NY*NX
       IYY = NY*NY
       IVV = NV*NV
       IXV = NX*NV
       IDA = 1
       IDD = IDA + IXX
       IDW = IDD + IYX
       IDV = IDW + IYY
       IDC = IDV + IVV
       CALL SDCOPZ(Z(NA),ZD(IDA),IXX,1)
       CALL SDCOPZ(Z(ND),ZD(IDD),IYX,1)
       CALL SDCOPZ(Z(NW),ZD(IDW),IYY,1)
       CALL SDCOPZ(Z(NVV),ZD(IDV),IVV,1)
       CALL SDCOPZ(Z(NC),ZD(IDC),IXV,1)
      ENDIF

C2.. Solve specified LQ-problem
      IF (JOB.EQ.1) THEN
C... Solve LQ- controller problem
       CALL DDRSOL(ZD,IBUF,NX,NU,DEPS,INFO)

      ELSEIF (JOB.EQ.2) THEN
C... Solve LQ- estimator problem
       CALL DCLQEST(ZD,IBUF,NX,NV,NY,DEPS,INFO)
      ENDIF

C... Give error messages, see description of INFO in routine DCLQEST
C    and/or DDRSOL.
      IF (INFO.NE.0) THEN
       CALL DCCERR(INFO,6)
      ENDIF

C3..  Copy double precision solutions in ZD to real stack Z
C...  JOB = 1 --> LQ-controller problem
      IF (JOB.EQ.1) THEN
       IDR = IDQ + IXX
       IDG = IDR + IXX
       IER = IDG + IUX
       IEI = IER + NX
       CALL SDCOPZ(Z(NR),ZD(IDR),IXX,2)
       CALL SDCOPZ(Z(NG),ZD(IDG),IUX,2)

C... LQ-estimator problem
      ELSEIF (JOB.EQ.2) THEN
       IDX = IDC + IXV
       IDK = IDX + IXX
       IER = IDK + IYX
       IEI = IER + NX 
       CALL SDCOPZ(Z(NXX),ZD(IDX),IXX,2)
       CALL SDCOPZ(Z(NKA),ZD(IDK),IYX,2)
      ENDIF

      RETURN
      END

      SUBROUTINE DCCERR(INFO,IOUT)
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C   PURPOSE                                                             c
C   DCCERR give error messages according to INTEGER parameter INFO.     c
C   Se subroutine DCLQEST and/or DDRSOL for explamation of param. INFO. c
C   ROUTINES CALLED FROM DCCBUF.                                        c
C   ROUTINES CALLED: WRAPMSG                                            c
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER INFO,IOUT

      IF (INFO.EQ.0) GO TO 1000
      IF     (INFO.EQ.1) THEN
       CALL WRAPMSG('Eigenvalues not proper isolated',0)

      ELSEIF (INFO.EQ.2) THEN
       CALL WRAPMSG('Reordering eigenspectrum failed',0)

      ELSEIF (INFO.EQ.3) THEN
       CALL WRAPMSG('Linear system of equation is singular',0)   
       CALL WRAPMSG('Canonical system matrix is singular',0)

      ELSE                     
       WRITE(IOUT,'(1X,A,I4)')'Error ---> INFO =',INFO
      ENDIF

1000  RETURN
      END

      SUBROUTINE DBUFCHK(NX,NU,NV,NY,NZD,NBUF,OK,JOB)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***BEGIN PROLOGUE DBUFCHK                                           c
C   PURPOSE                                                          c
C   DBUFCHK is used to check if the double prec. work array ZD(NZD)  c
C   and integer work array IBUF(NBUF) is big enugh for the routine   c
C   DCCBUF. This routine shuld be called before call to DCCBUF.      c
C                                                                    c
C   ON INPUT                                                         c
C   NX    INTEGER state dimension.                                   c
C   NU    INTEGER control input dimension.                           c
C   NV    INTEGER disturbance input dimension.                       c
C   NY    INTEGER measurement output dimension.                      c
C   NZD   INTEGER max dimension on double precision array ZD(NZD).   c
C   NBUF  INTEGER max dimension on integer array IBUF(NBUF).         c
C   JOB   INTEGER parameter. JOB=1 --> LQ-controller problem.        c
C                            JOB=2 --> LQ-estimator problem.         c
C                                                                    c
C   ON OUTPUT                                                        c
C   OK    LOGICAL messages param.  OK = .TRUE.  --> Array ok.        c
C                                  OK = .FALSE. --> Array too small  c
C   ROUTINES CALLED : WRAPMSG                                        c
C***END PROLOGUE DBUFCHK                                             c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER NX,NU,NV,NY,NZD,NBUF,JOB
      LOGICAL OK
      INTEGER NZ

      OK = .TRUE.

      IF (JOB.EQ.1) THEN
C...Check double precision work array for LQ-controller problem.
       NZ = 11*NX*NX + 2*NX*NU + 6*NX + NU*NU + 1
       IF (NZ.GT.NZD) THEN
        CALL WRAPMSG('Double precision work array too small',0)
        OK = .FALSE.
        RETURN
       ENDIF

      ELSEIF (JOB.EQ.2) THEN
C...Check double precision work array for LQ-estimator problem.
       NZ = 12*NX*NX + NY*NX + 2*NV*NX + 6*NX + NY*NY + NV*NV + 1
       IF (NZ.GT.NZD) THEN
        CALL WRAPMSG('Double precision work array too small',0)
        OK = .FALSE.
        RETURN
       ENDIF
      ENDIF

C...Check integer work space
      IF (2*NX.GT.NBUF) THEN
       CALL WRAPMSG('Integer work array too small',0)
       OK = .FALSE.
       RETURN
      ENDIF

      RETURN
      END
