      SUBROUTINE WVMAT(V,W,WV,W1,W2,NX,NU,NA,NB,CJOB)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE WVMAT                                        C
C                                                               C
C   PURPOSE                                                     C
C   WVMAT is used to read the covariance matrix V for the       C
C   disturbances v and the cov. matrix W for the measurements   C
C   w. It is possible to read only the diagonal matrix elements.C
C   This routine is analogue to the routine PQMAT.              C
C                                                               C
C   ON INPUT                                                    C
C   V      DOUBLE PRECISION (NA,NX) cov. matrix.                C
C   W      DOUBLE PRECISION (NB,NU) cov. matrix.                C
C   NX     INTEGER actual dimension on V.                       C
C   NU     INTEGER actual dimension on W.                       C
C   NA     INTEGER leading dimension on V in calling program.   C
C   NB     INTEGER leading dimension on W in calling program.   C
C   WV     DOUBLE PRECISION (NA) work vector.                   C
C   W1     DOUBLE PRECISION (NA,NA) Dummy array                 C
C   W2     DOUBLE PRECISION (NA) work vector.                   C
C   CJOB   CHARACTER*(*) - Character expansion.                 C
C            CJOB = 'WV'  W and V are read.                     C
C            CJOB = '0V'  only V are read etc.                  C
C   ROUTINES CALLED : DREMA,DRVEC (use CYPROS DREAD) ,RCSTR     C
C                     DIAGM                                     C
C***END PROLOGUE WVMAT                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NA,NB,ISVAR,is
      DOUBLE PRECISION V(NA,NX),W(NB,NU),WV(NA)
      DOUBLE PRECISION W1(NA,NA),W2(NA),DK
      CHARACTER CFORM*6,CJOB*(*),CMAT*4,CF*16

C1...Initialize                         .........................
      CFORM = '(5X,A)'
      CMAT  = CJOB(1:1)
      CF    = '(5X,A,I3,A,I3,A)'

C2... Specify which matrix to edit              .................
90    CALL RCSTR('Edit matrix ',CMAT,1,1,IIII)
      IF (CMAT.EQ.'V'.AND.CJOB(1:1).EQ.'V') THEN
       IF (CJOB(2:2).EQ.'W') THEN
        CMAT = 'W'
       ELSE
        CMAT = 'No'
       ENDIF
       GO TO 100
      ELSEIF (CMAT.EQ.'W'.AND.CJOB(2:2).EQ.'W') THEN
       CMAT = 'No'
       GO TO 200
      ELSEIF (CMAT.EQ.'N'.OR.CMAT.EQ.'No'.OR.CMAT.EQ.'0') THEN
       GO TO 1000
      ELSE
       DO 10 I=1,LEN( CJOB )
          IF (CJOB(I:I).NE.'0') THEN
           WRITE(*,'(5X,A,1X,A)')'* Avaiable ',CJOB(I:I)
          ENDIF
10     CONTINUE
       WRITE(*,CFORM)'* Type No or 0 to exit'
       CMAT = 'No'
       GO TO 90
      ENDIF

C3...Read V                             .........................
100   IF (CJOB(1:1).EQ.'V') THEN
       ISVAR = IYESANS('Diagonal matrix V ?',2)
       WRITE(*,CF)'* Covariance matrix V(',NX,',',NX,')'
       IF (ISVAR.EQ.1) THEN
        CALL DREMA(V,W2,NX,NA,NX,NA,'V')
       ELSEIF (ISVAR.EQ.2) THEN
        IS = IYESANS('Equal diagonal elements',2)
        IF (IS.EQ.1) THEN
         DO 20 I=1,NX
            WV(I) = V(I,I)
20       CONTINUE
         CALL DRVEC('Diagonal element V',WV,NX,NA)
        ELSEIF (IS.EQ.2) THEN
         DK = V(1,1)
         CALL DREAD('Value on diagonal ?',DK,1,1,IIII)
         DO 21 I=1,NX
            WV(I) = DK
21       CONTINUE
        ENDIF
        CALL DIAGM(V,WV,NX,NX,NA,NA)
       ENDIF
       GO TO 90
      ENDIF

C4...Read W                             .........................
200   IF (CJOB(2:2).EQ.'W') THEN
       ISVAR = IYESANS('Diagonal matrix W ?',2)
       WRITE(*,CF)'* Covariance matrix W(',NU,',',NU,' )'
       IF (ISVAR.EQ.1) THEN
        CALL DREMA(W,W2,NU,NB,NU,NB,'W')
       ELSEIF (ISVAR.EQ.2) THEN
        IS = IYESANS('Equal diagonal elements ?',2)
        IF (IS.EQ.1) THEN
         DO 30 I=1,NU
            WV(I) = W(I,I)
30       CONTINUE
         CALL DRVEC('Diagonal element W',WV,NU,NA)
        ELSEIF (IS.EQ.2) THEN
         DK = W(1,1)
         CALL DREAD('Value on diagonal ?',DK,1,1,IIII)
         DO 31 I=1,NU
            WV(I) = DK
31       CONTINUE
        ENDIF
        CALL DIAGM(W,WV,NU,NU,NB,NA)
       ENDIF
       GO TO 90
      ENDIF

C... Reset work vector WV                      ..................
      DO 40 I=1,NX
         WV(I)= 0.0D0
40    CONTINUE
1000  RETURN
      END
