      SUBROUTINE SYSMAT(A,B,C,D,G,KA,W1,W2,NX,NU,NV,NY,NA,NB,NC,ND,
     +                  CSYS,ISYS,IERR)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE SYSMAT                                       C
C   AUTHOR  David Di Ruscio           NTH/ITK 25 november 1988  C
C                                                               C
C   PURPOSE                                                     C
C   SYSMAT could be used to read the total system matrixes      C
C   A, B, C, and D or only A and B etc.                         C
C   ON INPUT                                                    C
C       CSYS is an CHARACTER(*)*(*) array of legal matrix names.C
C       ISYS Size of CSYS array.                                C
C        NX  integer                                            C
C            the number of the states.                          C
C        NU  integer                                            C
C            the number of inputs.                              C
C        NV  integer                                            C
C            the number of disturbances.                        C
C        NY  integer                                            C
C            the number of outputs (measurements).              C
C        NA  integer                                            C
C            the leading or row dimension of the real array A   C
C            as declared in the main calling program.           C
C        NB  integer                                            C
C            the leading or row dimension of the real array B   C
C            as declared in the main calling program.           C
C        NC  integer                                            C
C            the leading or row dimension of the real array C   C
C            as declared in the main calling program.           C
C        ND  integer                                            C
C            the leading or row dimension of the real array D   C
C            as declared in the main calling program.           C
C                                                               C
C        A   double precision(na,na)                            C
C            a real nx x nx matrix (the system matrix).         C
C        B   double precision(nb,nb)                            C
C            a real nx x nu matrix (the input matrix).          C
C        C   double precision(nc,nc)                            C
C            a real nx x nv matrix (the disturbance matrix).    C
C        D   double precision(nd,nd)                            C
C            a real ny x nx matrix (the output measurement mat.)C
C        W1,W2  DOUBLE PRECISION(na,na) and DOUBLE PRECISION(na)C
C             respectively                                      C
C            used as work space for IO between real and double  C
C            precision transformations.                         C
C        W1 - DUMMY ARRAY                                       C
C   ON OUTPUT                                                   C
C        A, B, C, G, D and KA                                   C
C        IERR = 0 for successfully operations                   C
C        IERR = 1 dimensions out of bounds  (.LE.0)             C
C                                                               C
C   ROUTINES CALLED    : DREMA                                  C
C***END PROLOGUE SYSMAT                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NV,NY,NA,NB,NC,ND,IERR,ISYS
      DOUBLE PRECISION A(NA,NA),B(NB,NB),C(NC,NC),D(ND,ND),G(NB,NA)
      DOUBLE PRECISION KA(NA,ND)
      DOUBLE PRECISION W1(NA,NA),W2(NA)
      CHARACTER CSYS(*)*(2),CFORM*6,CMAT*10,CANEXT*4

C1..  Format initialitation                                 .....
      CFORM = '(5X,A)'
      IERR  = 0

C2..  First, find the most probability matrix to start whit .....
      CMAT = CANEXT(CSYS,1,ISYS)
      IF (CMAT.EQ.' ') CMAT = 'No'

C3.. Second - ask user for matrix name and execute the job  .....
90    CALL RCSTR('Edit matrix ',CMAT,1,1,IIII)

C4.. Find next matrix to set as default on return to 90     .....
      IF     (CMAT.EQ.CSYS(1)) THEN
       IF (ISYS.GE.2) THEN
        CMAT = CANEXT(CSYS,2,ISYS)
        IF (CMAT.NE.' ') GO TO 100
       ENDIF
       CMAT = 'No'
       GO TO 100
      ELSEIF (CMAT.EQ.CSYS(2)) THEN
       IF (ISYS.GE.3) THEN
        CMAT = CANEXT(CSYS,3,ISYS)
        IF (CMAT.NE.' ') GO TO 200
       ENDIF
       CMAT='No'
       GO TO 200
      ELSEIF (CMAT.EQ.CSYS(3)) THEN
       IF (ISYS.GE.4) THEN
        CMAT = CANEXT(CSYS,4,ISYS)
        IF (CMAT.NE.' ') GO TO 300
       ENDIF
       CMAT = 'No'
       GO TO 300
      ELSEIF (CMAT.EQ.CSYS(4)) THEN
       IF (ISYS.GE.5) THEN
        CMAT = CANEXT(CSYS,5,ISYS)
        IF (CMAT.NE.' ') GO TO 400
       ENDIF
       CMAT='No'
       GO TO 400
      ELSEIF (CMAT.EQ.CSYS(5)) THEN
       IF (ISYS.GE.6) THEN
        CMAT = CANEXT(CSYS,6,ISYS)
        IF (CMAT.NE.' ') GO TO 500
       ENDIF
       CMAT='No'
       GO TO 500
      ELSEIF (CMAT.EQ.CSYS(6)) THEN
       CMAT='No'
       GO TO 600
      ELSEIF (CMAT.EQ.'N'.OR.CMAT.EQ.'No'.OR.CMAT.EQ.'0') THEN
       GO TO 1000
      ELSEIF (CMAT.EQ.'NO'.OR.CMAT.EQ.'EXIT') THEN
       GO TO 1000
      ELSE
       CALL COMLIS('* Avaiable is :',CSYS,ISYS)
       WRITE(*,CFORM)'* Type No or 0 to exit'
       CMAT='No'
       GO TO 90
      ENDIF
       
CA... Read system matrix A from terminal                .........
100   IF (NX.LE.0) GO TO 1000
      WRITE(*,CFORM)'*         Matrix '//CSYS(1)  
      CALL DREMA(A,W2,NX,NA,NX,NA,CSYS(1))
      GO TO 90
C
CB... Read input matrix B from terminal                 .........
200   IF (NX.LE.0.OR.NU.LE.0) GO TO 1000
      WRITE(*,CFORM)'*         Matrix '//CSYS(2)  
      CALL DREMA(B,W2,NX,NA,NU,NB,CSYS(2))
      GO TO 90
C
CC... Read disturbance matrix C from terminal           .........
300   IF (NX.LE.0.OR.NV.LE.0) GO TO 1000
      WRITE(*,CFORM)'*         Matrix '//CSYS(3)  
      CALL DREMA(C,W2,NX,NA,NV,NC,CSYS(3))
      GO TO 90
C
CD... Read mesurement matrix D from terminal            .........
400   IF (NY.LE.0.OR.NX.LE.0) GO TO 1000
      WRITE(*,CFORM)'*         Matrix '//CSYS(4)   
      CALL DREMA(D,W2,NY,ND,NX,NA,CSYS(4))
      GO TO 90

CG... Read controller matrix G                          .........
500   IF (NU.LE.0.OR.NX.LE.0) GO TO 1000
      WRITE(*,CFORM)'*         Matrix '//CSYS(5)  
      CALL DREMA(G,W2,NU,NB,NX,NA,CSYS(5))
      GO TO 90

CKA.. Read Kalman filter gain matrix                    .........
600   IF (NX.LE.0.OR.NY.LE.0) GO TO 1000
      WRITE(*,CFORM)'*         Matrix '//CSYS(6)  
      CALL DREMA(KA,W2,NX,NA,NY,ND,CSYS(6))
      GO TO 90

C
C... Last lines of SYSMAT                      ..................
1000  IF (NX.LE.0.OR.NU.LE.0.OR.NV.LE.0.OR.NY.LE.0) THEN
       WRITE(*,CFORM)'*  Dimension out of bounds'
       IERR = 1
      ENDIF
C
      RETURN
      END
