      PROGRAM DESREG
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***START PROLOGUE PROGRAM DESREG                                    C
C                                                                    C
C   AUTHOR : David Di Ruscio           ITK/NTH AUTOMN 88             C
C            Pertially revised 25/4-1990                             C
C   PURPOSE                                                          C
C   DESREG is a program for computing a optimal decentralized        C
C   controller (feedback) with prescribed structure. The             C
C   algorithm ensures that the closed loop eigenspectrum of          C
C   (A + BGD) is stable.                                             C
C   A initial stabilizing controller is not required, but            C
C   the algorithm does not guaranty to find a stabilizing            C
C   controller in this case.                                         C
C   If a initial stabilizing controller exists then a optimal        C
C   tuned controller may always be found.                            C
C                                                                    C
C   REMARKS :                                                        C
C   The program may be further optimized.                            C
C   - The main program shoud be made smaller.                        C
C   - The Bartels Stewart algorithm for solving the Lyapunov         C
C     equation shoud be implemented as an option to the existing     C
C     routine.                                                       C
C   - The problem of finding an initial stabilizig controller        C
C     should be further studied.  - Gersgorin sircles (block)        C
C                                 - Hurwitz determinant conditioons  C
C                                                                    C
C     DESREG should be linked with :                                 C
C***END PROLOGUE PROGRAM DESREG                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,M,NY
      INTEGER I,J,ITER,ISVAR,IFLAG,III,MET,IWTE,MAX
      INTEGER NC,IJAC  
      INTEGER LWA,LKWA
      PARAMETER ( IWTE = 6 ,MAX = 20)
      PARAMETER ( LWA = MAX*MAX + 25*MAX , LKWA = 50)
      DOUBLE PRECISION A(MAX,MAX),B(MAX,MAX),P(MAX,MAX),Q(MAX,MAX)
      DOUBLE PRECISION G(MAX,MAX),R(MAX,MAX),K(MAX,MAX),DG(MAX,MAX)
      DOUBLE PRECISION L0,L1,D(MAX,MAX),DA,ALFA,GDUM(MAX,MAX)
      DOUBLE PRECISION DMAX,AA(MAX,MAX),F(MAX,MAX),EPS,T,Z(MAX)
      INTEGER NM,IKONT,ITEST
      DOUBLE PRECISION ER(MAX),EI(MAX),WORK(MAX)
      DOUBLE PRECISION DJX(MAX),DX(MAX),DET,J1,J2,X1(MAX),ALF
      DOUBLE PRECISION DMUR,GD(MAX,MAX)
      DOUBLE PRECISION X(MAX),HESS(MAX,MAX)
C...Parameters in NLPINT call
      INTEGER KWA(LKWA),IPRINT,IFAIL
      DOUBLE PRECISION XL(MAX),XU(MAX),WA(LWA),CC(MAX)
      LOGICAL ACTIVE(MAX)
C...
      CHARACTER TAST*2,CRIT*3,CNAME*12
      COMMON /DESMAT/ AA,B,D,P,Q,R,F
      COMMON /INT/ N,M,NY
      COMMON /PAR/ DMUR,CRIT
C... Common block for statistic parameters used during optimization
      INTEGER NC,ND,IGRAD
      COMMON /IDB/ NC,ND,IGRAD

C... Debug utilities
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

C... System parameters in DATA block 1
      INTEGER ISYS
      PARAMETER (ISYS = 10)
      CHARACTER CSYS(ISYS)*2
C... Main commmand parameters in DATA block 2
      INTEGER ICOM,IMAIN
      PARAMETER (ICOM = 14)
      CHARACTER COM(ICOM)*6

      EXTERNAL FXEIG

C.DATA block 1  System matrix names  A,B,D,G,P,Q and R
      DATA  CSYS(1)/'A'/, CSYS(2)/'B'/, CSYS(3)/' '/, CSYS(4)/'D'/,
     +      CSYS(5)/'G'/, CSYS(6)/' '/, CSYS(7)/'P'/, CSYS(8)/'Q'/,
     +      CSYS(9)/'R'/, CSYS(10)/' '/

C.DATA block 2 Main commmands
      DATA  COM(1)/'DIM'/,  COM(2)/'SYS'/,  COM(3)/'SOLVE'/,
     +      COM(4)/'LOAD'/, COM(5)/'SAVE'/, COM(6)/'HELP'/,
     +      COM(7)/'EXIT'/, COM(8)/'0'/,    COM(9)/'STRUC'/,
     +      COM(10)/'EIG'/, COM(11)/'INIT'/,COM(12)/'CNST'/,
     +      COM(13)/'MUL'/, COM(14)/'SENS'/

C ... Print program description on terminal     .................
      CALL DESHEAD

C... Initialize statistics counters
      NC = 0
      ND = 0
      IGRAD = 0

C1....Initialize default parameters             .................
      IOUT = 6
      IPRT = 0
      ITER  = 100
      IJAC  = 1
      ITEST = 3
      MET   = 2
      EPS   = 1.0D-4
      CRIT  = 'TR'
      CNAME = 'DESREG.DAT'
      ALFA  = 0.0

C.. MAIN INPUT COMMANDS
      IMAIN = 1
100   WRITE(*,*)'  '
      IMAIN = ICREAD('DESREG main command :',COM,ICOM,IMAIN)

      IF (IMAIN.EQ.0) THEN
       GO TO 1000
CUT    STOP '<   E X I T  D E S R E G   >'
      ELSEIF (IMAIN.EQ.1) THEN

C... Specify dimensions
101    CALL NREAD('Dimension on x  NX = ',N,1,1,IIII)
       CALL NREAD('Dimension on u, NU = ',M,1,1,IIII)
       CALL NREAD('Dimension on y, NY = ',NY,1,1,IIII)

       CALL DIMCHK(N,M,NY,MAX,IERR)
       IF (IERR.NE.0) THEN
        WRITE(*,'(1X,A,I4)')'Maximum dimensions on (x u y) is :',MAX
        GO TO 101
       ENDIF

       IMAIN = 2
       GO TO 100
C.SYS
      ELSEIF (IMAIN.EQ.2) THEN

C3....  Interactiv reading    ....................................
       CALL SYSMAT2(A,B,DG,D,G,K,P,Q,R,AA,WORK,N,M,1,NY,
     +               MAX,MAX,MAX,MAX,CSYS,ISYS,IERR)

       IMAIN = 9
       GO TO 100

C.STRUC
      ELSEIF (IMAIN.EQ.9) THEN
C4..  Specify controller structure matrix F
CUT    CALL STRUCT(G,F,N,M,MAX)
       CALL STRUCT(G,F,NY,M,MAX)

       IMAIN = 3
       GO TO 100

C.LOAD
      ELSEIF (IMAIN.EQ.4) THEN

C2....  Read matrices from file CNAME         ....................
       CALL RCSTR('Specify file name',CNAME,1,1,IIII)
       IF (CNAME.EQ.'0') THEN
        CNAME = 'DESREG.DAT'
        GO TO 100
       ENDIF
       CALL SYSDAT(A,B,R,P,Q,G,K,D,DG,GD,N,M,N,NY,MAX,1,CNAME,
     +              IERR)
       IF (IERR.NE.0) THEN
        CNAME = 'DESREG.DAT'
       ENDIF
       IMAIN = 2
       GO TO 100

C.SAVE
      ELSEIF (IMAIN.EQ.5) THEN

C2....  Save matrices on file CNAME         ....................
       CALL RCSTR('Specify file name',CNAME,1,1,IIII)
       IF (CNAME.EQ.'0') THEN
        CNAME = 'DESREG.DAT'
        GO TO 100
       ENDIF
       CALL SYSDAT(A,B,R,P,Q,G,K,D,DG,GD,N,M,N,NY,MAX,2,CNAME,
     +             IERR)

       IF (IERR.NE.0) THEN
        CNAME = 'DESREG.DAT'
       ENDIF
       IMAIN = 7
       GO TO 100

C. HELP
      ELSEIF (IMAIN.EQ.6) THEN
       CALL DESHLP
       IMAIN = 1
       GO TO 100
C. EXIT
      ELSEIF (IMAIN.EQ.7) THEN
CUT    STOP '<   E X I T  D E S R E G   >'
       GO TO 1000
C. 0
      ELSEIF (IMAIN.EQ.8) THEN
CUT    STOP '<   E X I T  D E S R E G   >'
       GO TO 1000
C.EIG
      ELSEIF (IMAIN.EQ.10) THEN
C...Test stability of (A+BG+alfa*I) and A . Computing the ......
C   Frobeniusnorm                                         ......
C... AA = A + alfa*I
c       CALL MATCOP(A,AA,N,N,MAX,MAX)
       WRITE(*,'(1X,A,G16.6)')'Stability margin ALFA = ',ALFA
       CALL AALFA(A,ALFA,AA,N,MAX)
       CALL STABIL(AA,B,G,D,GD,ER,EI,N,M,NY,MAX)
       IMAIN = 3
       GO TO 100

C.PAR
      ELSEIF (IMAIN.EQ.11) THEN
       CALL DESPAR(ALFA,IPRT,IOUT,CRIT,DMUR)
       IMAIN = 3
       GO TO 100
C.Constraints
      ELSEIF (IMAIN.EQ.12) THEN
       CALL XG(G,F,X,NY,M,NM,1,MAX)
       CALL VECPRT('* Startvalue X =','X',X,MAX,NM)
       DO 12 I=1,NM
          ACTIVE(I) = .TRUE.
12     CONTINUE
       CALL FXCNST(CC,X,NM,ACTIVE,MC,ME,MAX)
       CALL MATPRT('R =',R,MAX,MAX,N,N)
       CALL VECPRT('Sub determinants of R','CC',CC,MAX,N)
       IMAIN = 3
       GO TO 100

      ELSEIF (IMAIN.EQ.13) THEN

       CALL DPTOD(J2,Q,P,R,G,D,N,M,NY,MAX,ER,GDUM,HESS,Z,EI,IERR)
       WRITE(*,'(1X,A,G16.6)')'Robust criterion due to Patel and Toda'
       WRITE(*,'(1X,A,G16.6)')'mu < ',J2
       IMAIN = 3
       GO TO 100

C... Compute sensitivity matrix
      ELSEIF (IMAIN.EQ.14) THEN
       CALL MATCOP(A,AA,N,N,MAX,MAX)
       CALL XG(G,F,X,NY,M,NM,1,MAX)
       CALL VECPRT('* Startvalue X =','X',X,MAX,NM)
       CALL FXEIG(ER,X,DEPS,NN,NM)
       CALL VECPRT('* Eigenvalues =','ER',ER,MAX,N)
       CALL DFJAC(FXEIG,X,DEPS,HESS,DJX,N,NM,MAX,MAX)
       CALL MATPRT('Sensitivity matrix =',HESS,MAX,MAX,N,NM)
       IMAIN = 3
       GO TO 100
      ENDIF
C.SOLVE
      IF (IMAIN.EQ.3) THEN

C5... Choose solution-algorithm 1 or 2 or 3     .................
120    CALL NREAD('Solution method 1,2 or 3 ? ',ITEST,1,1,IIII)
       IF (ITEST.EQ.0) THEN
        GO TO 100
       ELSEIF (ITEST.EQ.1) THEN
        IKONT=IYESANS('Carriage controll on each iteration ?',1)
        CALL NREAD('Compute grad. with IJAC (1 or 2)  ',IJAC,1,1,IIII)
CUT     CALL DREAD('Skrittlengde faktor            PI ',PI,1,1,IIII)
        CALL DREAD('Init. val. on line search par. DA ',DA,1,1,IIII)
CUT     CALL NREAD('Lyp. method 1 or 2             MET',MET,1,1,IIII)
        CALL DREAD('Max error in gradient          EPS',EPS,1,1,IIII)
        CALL NREAD('Max number of iterations ?        ',ITER,1,1,IIII)
       ELSEIF (ITEST.EQ.2) THEN
        IKONT=IYESANS('Carriage controll at each iteration ?',1)
119     CALL NREAD('Compute grad. with IJAC (1 or 2)  ',IJAC,1,1,IIII)
        IF (IJAC.LT.1 .OR. IJAC.GT.2) THEN
         WRITE(*,'(5X,A)')'Available is : (1-2)'
         WRITE(*,'(5X,A)')'1 compute gradient with backward difference'
         WRITE(*,'(5X,A)')' method.'                          
         WRITE(*,'(5X,A)')'2 compute gradient with midtpoint'
         WRITE(*,'(5X,A)')'  difference method.'
         GO TO 119
        ENDIF
CUT     CALL DREAD('Skrittlengde faktor            PI ',PI,1,1,IIII)
CUT     CALL DREAD('Init. VAL. on line search par. DA ',DA,1,1,IIII)
CUT     CALL NREAD('Lyp. method 1 or 2        MET=?   ',MET,1,1,IIII)
        CALL DREAD('Max error in gradient          EPS',EPS,1,1,IIII)
        CALL NREAD('Max number of iterations ?        ',ITER,1,1,IIII)
       ELSEIF (ITEST.EQ.3) THEN
121     WRITE(*,'(5X,A)')'Specify IPRINT (0-1-2-3-4)'
        CALL NREAD('Output type IPRINT =',IPRINT,1,1,IIII)
        IF (IPRINT.LT.0 .OR. IPRINT.GT.4) THEN
         GO TO 121
        ENDIF
       ELSE
        WRITE(*,'(5X,A)')'Available is : (0-1-2-3)'
        WRITE(*,'(5X,A)')'0 return main modus.'
        WRITE(*,'(5X,A)')'1 solution method by Geromel.'
        WRITE(*,'(5X,A)')'2 solution method by Di Ruscio'
        WRITE(*,'(5X,A)')'  with quasi Newton optimization.'
        WRITE(*,'(5X,A)')'3 solution method by Di Ruscio'
        WRITE(*,'(5X,A)')'  with NLPQL (Schittkowski) optimization.'
        GO TO 120
       ENDIF


C... ERROR check
       CALL DESERR(N,M,NY,F,MAX,IERR,IMAIN)
       IF (IERR.NE.0) THEN
        IF (IERR.EQ.1) IMAIN = 1
        IF (IERR.EQ.9) IMAIN = 9
        GO TO 100
       ENDIF

C*** START MAIN COMPUTING WORK ******************************************
C*** START SOLUTION

C....Prescribed degree of stability, uppdate  AA = A + alfa*I ...
       CALL AALFA(A,ALFA,AA,N,MAX)


C***  Solution algorithm 1 ( Geromel)
C...  Search in negativ gradient direction 
       IF (ITEST.EQ.1) THEN
        IF (N.NE.NY) THEN
         WRITE(*,'(3X,A)')'NX not equal NY use method 2 or 3'
         GO TO 100
        ENDIF

       DO 30 III=1,ITER

C.... Computing R and K for given G ,(E(G)=0)  ..................
          CALL DLDK(AA,B,G,P,Q,R,K,N,M,IFLAG,MET)
          IF (IFLAG.NE.0) THEN
           WRITE(*,'(3X,A,I3)')'IFLAG (DLDK) =',IFLAG
          ENDIF

C....Beregner dL/dG for gitt R,K og G ...........................
          CALL DLDG(B,G,P,R,K,DG,D,F,N,M)

C....Tester konvergensen av dL/dG - D , mulige retninger ........
          CALL MATSORT(D,DMAX,M,N,MAX,MAX)
          IF (ABS(DMAX).LE.EPS) THEN       
           CALL SCLPRT('* Konvergens max(D) =',DMAX,IWTE)
           GO TO 150
          ENDIF

C....Beregner funksjonsverdien L(Gk) = L0........................
CUT       CALL FUNCVAL(AA,B,G,R,K,P,Q,L0,N,M)
          CALL TRACE(R,L0,N,MAX)
          IF (L0.LT.0.0.AND.III.EQ.1) THEN
           WRITE(*,'(A)')'** L(G0) = tr(R) < 0.0) **'
           GO TO 150
          ENDIF

C.....Tester om man kan g} helt til min av L(Gk+1) ..............
          CALL LINE(DA,DG,D,K,P,R,N,M)

C....Oppdaterer G dva. Gk+1 = Gk - DAk*Gk .......................
200       DO 40 I=1,M
             DO 50 J=1,N
                GDUM(I,J)=G(I,J)
                G(I,J)=G(I,J)-DA*D(I,J)
50           CONTINUE
40        CONTINUE

          CALL FUNCVAL(AA,B,G,R,K,P,Q,L1,N,M)

C         IF (L1.GE.L0.OR.L1.LT.0.0) THEN
C         WRITE(*,*)'ALARM L1,L0=',L1,L0
C         ENDIF

          IF (L1.LT.0.0) THEN             
           WRITE(*,*)'ALRAM L1.LT.0.0'
          ENDIF

          WRITE(*,*)'ITERASJON NR.',III
          WRITE(*,*)'L0,L1,DA',L0,L1,DA

30     CONTINUE


C********************************************************************
C...  LOESNINGSALGORITME 2 ( modifisert Newtons met. ) .........
       ELSEIF (ITEST.EQ.2) THEN
         NC = 0
         CALL XG(G,F,X,NY,M,NM,1,MAX)
         CALL VECPRT('* Startvalue X =','X',X,MAX,NM)
         IF (IPRT.EQ.2) THEN
          CALL DESOUT(AA,Q,R,N,M,NY,MAX)
         ENDIF

         CALL JVALUE(AA,B,D,P,Q,R,X,F,L0,N,M,NY,IFLAG,MAX)
         CALL SCLPRT('* J(X0) = trace(R) =',L0,IWTE)
         IF (IPRT.EQ.2) THEN
          CALL DESOUT(AA,Q,R,N,M,NY,MAX)
         ENDIF
         IF (L0.LT.0.0) THEN
          WRITE(*,'(5X,A)')'ERROR ---> J(X0) < 0.0 must be > 0'
          WRITE(*,'(5X,A)')'Initial system is probably unstable'           
          GO TO 100
         ENDIF

C... NEW ALGHORITM OF SOLUTION                         ..........
         ILINE = 0
         WRITE(*,'(5X,A)')'Start main iteration procedure'
         DO 19 III=1,ITER

C... COMPUTE GRADIENT VECTOR                           ..........
            CALL GRAD2(NM,X,DJX,MAX,IJAC)

            IF (IPRT.EQ.1) THEN
             CALL VECPRT('*  Gradient DJX =','DJX',DJX,MAX,NM)
            ENDIF

C... TEST FOR CONVERGENCE compute infinity norm of DJX ..........
            CALL DINORM(DJX,DMAX,NM,MAX)
            IF (DMAX.LE.EPS) THEN
             CALL XG(G,F,X,NY,M,NM,2,MAX)
             GO TO 150
            ENDIF

C...COMPUTE THE HESSIAN MATRIX                         ..........
            CALL HESS2(NM,X,DJX,HESS,MAX,IJAC)

            IF (IPRT.EQ.1) THEN
             CALL MATPRT('*  Hessian matrix',HESS,MAX,MAX,NM,NM)
            ENDIF

C....UPDATE X WITH NEWTON ITERATION                    ..........
            CALL FCN(NM,1,T,X,Z(1))
            J1=Z(1)

C.... COMPUTE SEARCH DIRECTION  P = DX                 ..........
            CALL NEWT2(NM,HESS,DJX,DX,DET,MAX,IFLAG)
            IF (IPRT.EQ.1) THEN
            WRITE(*,'(3X,A,E12.5,2X,A,I2)')'DET =',DET,'IFLAG =',IFLAG
            ENDIF

C.... S|k i negativ P retn. dersom HESS er neg. definit .........
            IF (IFLAG.EQ.4) THEN
161          ISVAR=IYESANS2('Search in -P direction (0 to quit) ?',2)
             IF (ISVAR.EQ.0) GO TO 100
             IF (ISVAR.EQ.2) THEN
              DO 18 I=1,NM
                 DX(I)=-DX(I)
18            CONTINUE
             ENDIF
            ENDIF

            ALF=1.0
            DO 15 I=1,NM
               X1(I)=X(I)
15          CONTINUE

C..... Update X --> X1                                ...........
160         DO 16 I=1,NM
               X(I) = X1(I) + ALF*DX(I)
16          CONTINUE

            CALL FCN(NM,1,T,X,Z(1))
            J2=Z(1)

C... Check constraints CC(I) >= 0, I=1,...,NM
C           CALL FCNCST(CC,X,NM)
C           DO 163 I=1,NM
C              IF (CC(I).LT.0.0) THEN
C               WRITE(*,'(3X,A)')'Constraints not satisfied'
C               CALL VECCOP(X1,X,NM)
C               CALL XG(G,F,X,NY,M,NM,2,MAX)
C               J2 = J1
C               GO TO 150
C              ENDIF
C163        CONTINUE

C... Constraint j > 0 not satisfied
            IF (J2.LT.0.0) THEN
             CALL SCLPRT('WARNING * J2 < 0 *',J2,IWTE)
             WRITE(*,'(3X,A)')'Use method 3'
CUT          CALL VECCOP(X1,X,NM)
CUT          CALL XG(G,F,X,NY,M,NM,2,MAX)
             GO TO 150
            ENDIF

C..... LINE SEARCH IF  J(Xk+1) > J(Xk) and J(Xk+1) > 0  .........
            IF (J2.GT.J1) THEN
             ILINE = ILINE + 1
             IF (ILINE.GT.20) THEN
              CALL SCLPRT('* J2 > J1 line search J2 = ',J2,IWTE)
             ENDIF
             ALF=ALF*0.5
             GO TO 160
            ENDIF
            ILINE = 0

C..... Must find new search direction
            IF (J2.GE.J1) THEN     
             WRITE(*,'(5X,A)')'J2 >= J1 '
CUT          GO TO 161
            ENDIF
            IF (IPRT.EQ.1) THEN
             IF (J2.LT.J1) THEN
C..... Accsepting solution vector X
              WRITE(*,'(5X,A)')'Solution vector found'
             ENDIF
            ENDIF
         
C...UTSKRIFT ....................................................
            IF (IPRT.EQ.1) THEN
             WRITE(*,'(3X,A)')'*   X=X+DX, DX, DJX    *'
             WRITE(*,17) (I,X(I),I,DX(I),I,DJX(I),I=1,NM)
17           FORMAT(1X,'X(',I3,')=',1X,E12.4,2X,'DX(',I3,')=',1X,
     +       E12.4,1X,'DJX(',I3,')=',1X,E12.4)
             WRITE(*,*)'* J(X(k+1)) , J(X(k)) ',J2,J1
            ENDIF

            WRITE(*,'(3X,A,I3,A,G16.6)')
     +      'Iteration no. :',III,' function value : ',J2
        
            IF (IKONT.EQ.2) THEN
             CALL RCSTR('Tast return for more !',TAST,1,1,IIII)
            ENDIF

19       CONTINUE

C... Transform vector X to matrix G     .........................
         CALL XG(G,F,X,NY,M,NM,2,MAX)

C******************************************************************
C3...METHOD 3 (USING NLPQ0 OPTIMIZATION ALGHORITM)      .........
        ELSEIF (ITEST.EQ.3) THEN
         CALL XG(G,F,X,NY,M,NM,1,MAX)

C... NLPINT calls JVALUE for function value
CUT      CALL NLPINT(NM,X,J2,XL,XU,WA,LWA,KWA,LKWA,IPRINT,IFAIL)
         MC = N
         ME = 0
         CALL NLPINT2(MC,ME,NM,X,J2,XL,XU,WA,LWA,KWA,LKWA,IPRINT,
     +                IFAIL)
         IF (IFAIL.NE.0) THEN
          WRITE(*,'(5X,A,1X,I3)')'NLPQ0 ERROR ---> IFAIL =',IFAIL
         ENDIF

CUT      CALL XG(G,F,X,N,M,NM,2,MAX)
         CALL XG(G,F,X,NY,M,NM,2,MAX)
        ENDIF

C... End solution methods
150     WRITE(*,'(3X,A,I4)')'Actual no. of FCN calls is  NC =',NC
        WRITE(*,'(3X,A,I4)')'Actual no. of CNST calls is ND =',ND
        WRITE(*,'(3X,A,G16.6)')'Optimal function value      J2 =',J2
        IF (ITEST.EQ.1 .OR. ITEST.EQ.2) THEN
         CALL SCLPRT('* Infinity gradient norm = ',DMAX,IWTE)
         WRITE(*,'(3X,A,I4)')'Number of iterations used      =',III
         WRITE(*,2) III-1,L0,III,L1
2        FORMAT(2X,'L(G',I3,')=',E12.4,1X,'L(G',I3,')=',E12.4)
        ENDIF
        L0=0.0
        L1=0.0

        III = 0
        NC  = 0
        ND  = 0
        IGRAD = 0              

C5.... Print results          ...................................
        ISVAR=IYESANS('Print results on terminal ?',2)
        IF (ISVAR.EQ.2) THEN
         CALL GRKUT(G,R,K,DG,N,M,NY,MAX,MAX,ITEST)
        ENDIF
       
        IMAIN = 5
        GO TO 100
C.. End main comand loop
       ENDIF

1000   ISVAR=IYESANS('Terminate program ?',2)
       IF (ISVAR.EQ.1) GO TO 100
       STOP '<   E X I T  D E S R E G   >'

       END

       SUBROUTINE DESHEAD
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DESHEAD                                      C
C   PURPOSE                                                     C
C   DESHEAD print program description on terminal when the      C
C   program is exequted. Else no function.                      C
C***END PROLOGUE DESHEAD                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      CHARACTER CFORM*6

C... Initialize format statement                .................
      CFORM = '(8X,A)'

      WRITE(*,CFORM)'-------------------------------------------'
      WRITE(*,CFORM)'.                                         .'
      WRITE(*,CFORM)'.          <  D E S R E G  >              .'
      WRITE(*,CFORM)'.                                         .'
      WRITE(*,CFORM)'.  Program for syntesises of LQ - optimal .'
      WRITE(*,CFORM)'.  controller feedback with a prescribed  .'
      WRITE(*,CFORM)'.  contoller sructure or output feedback  .'
      WRITE(*,CFORM)'.  syntesis.                              .'
      WRITE(*,CFORM)'.                           Version 1.1   .'
      WRITE(*,CFORM)'-------------------------------------------'
      WRITE(*,CFORM)'  '

      RETURN
      END

       SUBROUTINE GRKUT(G,R,K,DG,N,M,NY,NN,NM,ITEST)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE GRKUT                                      C
C   PURPOSE                                                   C
C   GRKUT prints matrix results G,R,K and DG on terminal      C
C                                                             C
C   ROUTINES CALLED : RCSTR             (CYPROS)              C
C***END PROLOGUE GRKUT                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       INTEGER N,M,NN,NM,ITEST
       DOUBLE PRECISION G(NN,NM),R(NN,NN),K(NN,NN),DG(NN,NM)
       CHARACTER TAST*8
       TAST = 'RETURN'
C......Print matrix results on terminal ......................

       CALL MATPRT('*  G =',G,20,20,M,NY)
       IF (NY.GT.6) THEN
        CALL RCSTR('TAST FOR MORE !',TAST,1,1,IIII)
       ENDIF
       CALL MATPRT('*  R =',R,20,20,N,N)
       IF (N.GT.6) THEN
        CALL RCSTR('TAST FOR MORE !',TAST,1,1,IIII)
       ENDIF

       IF (ITEST.EQ.1) THEN
        CALL MATPRT('*  K =',K,20,20,N,N)
        CALL RCSTR('TAST FOR MORE !',TAST,1,1,IIII)

        CALL MATPRT('*  DG =',DG,20,20,M,N)
        CALL RCSTR('TAST FOR MORE !',TAST,1,1,IIII)
       ENDIF

       RETURN
       END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                     C
C  Rutinen legger st|rste matriseelement i A(I,J) i   C
C  DMAX.                                              C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       SUBROUTINE MATSORT(A,DMAX,M,N,MA,NA)

       INTEGER I,N,M,MA,NA
       DOUBLE PRECISION A(NA,MA),X(20),S(20),DMAX       

C....Sorterer A kolonnevis, dvs. legger st|rste element i ver .. 
C    kolonne i A i vektoren S(I) I=1,N ......................... 
       DO 10 II=1,N
C... Lagrer spalter i A i vektoren X
          DO 20 I=1,M
             X(I)=ABS( A(I,II) )
20        CONTINUE
C... Bestemmer stoerste element i X vektor og lagrer denne i S ..
          CALL VECSORT(X,M,20)
          S(II)=X(1)

10     CONTINUE
       
C..SORTERER ARRAY S(II) AV STOERSTE ELEMENTER I SPALTEVEKTORENE .
       CALL VECSORT(S,N,20)
       DMAX=S(1)

       RETURN
       END
        
       SUBROUTINE VECSORT(A,N,NA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE VECSORT                                     C
C   AUTHOR : David Di Ruscio            ITK/NTH sept 1988      C
C                                                              C
C   PURPOSE                                                    C
C   VECSORT sort the vector A(I). The largest vector element   C
C   stored in A(1) on output. The smallest in X(N) etc.        C
C   ON INPUT                                                   C
C   A    DOUBLE PRECISION (N) vector. A is destroyed on output.C
C   N    INTEGER dimension on A                                C
C   NA   INTEGER leading dimension on A in calling program.    C
C   ON OUTPUT                                                  C
C   A(1) The first element in the vector A(I) contains the     C
C        largest element in the input vector A.                C
C   A(I+1) The elements in A in descending order.              C
C   ROUTINES CALLED : (NONE)                                   C
C***END PROLOGUE VECSORT                                       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       INTEGER I,J,JJ,N,NN,NA
       DOUBLE PRECISION A(NA),C

       NN=N-1
       DO 10 I=1,N-1
          JJ=I+1
          DO 20 J=JJ,N
             IF (A(I).GE.A(J)) GO TO 20
             C = A(I)
             A(I) = A(J)
             A(J) = C
20        CONTINUE
10     CONTINUE

       RETURN
       END

C****************************************************************

      SUBROUTINE DLDK(A,B,G,P,Q,R,K,N,M,IFLAG,MET)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                               C
C     Rutinen l|ser Liapunovligningen                           C
C            T                    T                             C
C      (A+BG) *R + R*(A+BG)+ Q + G *P*G = 0     (1)             C
C                                                               C
C     med hensyn p} R for gitt G samt                           C
C            T                                                  C
C      (A+BG) *K + K*(A+BG) + I = 0             (2)             C
C                                                               C
C     med hensyn p} K                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      INTEGER I,J,N,M,IFLAG,MET
      DOUBLE PRECISION A(20,20),B(20,20),G(20,20),P(20,20)
      DOUBLE PRECISION Q(20,20),R(20,20)

      PARAMETER (N1 = 20, NP = N1*(N1+1)/2 )
      INTEGER IND(NP),INFO
      DOUBLE PRECISION WM(NP,NP),WV(NP)

      DOUBLE PRECISION ABG(20,20),GTP(20,20),GTPG(20,20)
      DOUBLE PRECISION QQ(20,20),K(20,20),A2(20,20),Q2(20,20)

C...  L\SER LIGNING (1) .........................................
C1... Setter opp A+BG ...........................................
      
      CALL CLABG(ABG,A,B,G,N,M,20,20)

C2... Setter opp QQ = Q + GT*P*G ................................
      CALL TRANSMULT(G,P,GTP,N,M,M,20,2)
      CALL MATMULT(GTP,G,GTPG,N,M,N,20)
      DO 30 I=1,N
         DO 40 J=1,N
            QQ(I,J)=Q(I,J)+GTPG(I,J)
40       CONTINUE
30    CONTINUE

C3... L|ser Liapunovlikningen mht. R ............................

      CALL DLYP(ABG,QQ,R,20,N,WM,WV,NP,IND,INFO)
      IF (INFO.NE.0) WRITE(*,'(1X,A,I5)')'DLYP INFO =',INFO

C...  L\SER LIGNING (2) .........................................
      
      DO 50 I=1,N
         DO 60 J=1,N
            A2(I,J)=ABG(J,I)
            IF (I.EQ.J) THEN
             Q2(I,J)=1
            ELSE
             Q2(I,J)=0.0
            ENDIF
60       CONTINUE
50    CONTINUE

CUT   CALL LEQSOLV(A2,Q2,K,N,JFLAG,MET)
      CALL DLYP(A2,Q2,K,20,N,WM,WV,NP,IND,INFO)
      IF (INFO.NE.0) WRITE(*,'(1X,A,I5)')'DLYP INFO =',INFO

C... Korrigerer for eventuell usymmetri i R og K ........
      DO 70 I=1,N
         DO 80 J=1,N
            R(I,J)=0.5*(R(I,J)+R(J,I))
            K(I,J)=0.5*(K(I,J)+K(J,I))
80       CONTINUE
70    CONTINUE

      RETURN
      END
C********************************************************
     
      SUBROUTINE DLDG(B,G,P,R,K,DG,D,F,N,M)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                       C
C   Rutinen beregner gradienten                         C
C               T                                       C
C   dL/dG =2*( B *R + P*G)*K (=DG)                      C
C                                                       C
C                                                       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,N,M
      DOUBLE PRECISION B(20,20),G(20,20),P(20,20),R(20,20)
      DOUBLE PRECISION K(20,20),DG(20,20),D(20,20),F(20,20)

      DOUBLE PRECISION BTR(20,20),PG(20,20),SUM(20,20)

C1... Beregner (BT*R+P*G) ...............................
      CALL TRANSMULT(B,R,BTR,M,N,N,20,2)
      CALL MATMULT(P,G,PG,M,M,N,20)
      DO 10 I=1,M
         DO 20 J=1,N
            SUM(I,J)=BTR(I,J)+PG(I,J)
20       CONTINUE
10    CONTINUE

C2... Beregner dL/dG ....................................
      CALL MATMULT(SUM,K,DG,M,N,N,20)

      DO 30 I=1,M
         DO 40 J=1,N
            DG(I,J)=2*DG(I,J)
40       CONTINUE
30    CONTINUE

C3... Beregner mulige retninger fra DG =dL/dG ...........
      CALL ELMULT(DG,F,D,N,M,20,20)

      RETURN
      END
C********************************************************
      
      SUBROUTINE FUNCVAL(A,B,G,R,K,P,Q,L,N,M)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                T      C
C  Rutinen beregner funksjonsverdien L=tr(R)+tr(K *E)   C
C  der         T               T                        C
C      E=(A+BG) *R+R*(A+BG)+Q+G *P*G                    C
C                                                       C
C                                                       C
C                                                       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,N,M
      DOUBLE PRECISION A(20,20),B(20,20),G(20,20),R(20,20)
      DOUBLE PRECISION K(20,20),P(20,20),Q(20,20),L

      DOUBLE PRECISION GTP(20,20),GTPG(20,20)
      DOUBLE PRECISION ABGTR(20,20),RABG(20,20),E(20,20)
      DOUBLE PRECISION ARG(20,20),KTE(20,20),ABG(20,20)

C1... Beregner A+BG .....................................
      CALL CLABG(ABG,A,B,G,N,M,20,20)


C2... Beregner GT*P*G =GTPG ............................
      CALL TRANSMULT(G,P,GTP,N,M,M,20,2)
      CALL MATMULT(GTP,G,GTPG,N,M,N,20)

C4... Beregner (A+BG)T*R =ABGTR  samt R*(A+BG) =RABG ...
      CALL TRANSMULT(ABG,R,ABGTR,N,N,N,20,2)
      CALL MATMULT(R,ABG,RABG,N,N,N,20)

C4 .. Beregner E .......................................
      DO 30 I=1,N
         DO 40 J=1,N
            E(I,J)=ABGTR(I,J)+RABG(I,J)+Q(I,J)+GTPG(I,J)
40       CONTINUE
30    CONTINUE
      
C5... Beregner KT*E =KTE ................................
      CALL TRANSMULT(K,E,KTE,N,N,N,20,2)

C6... Beregner funkval. L = tr(R+KT*E) = tr(ARG) ........
      DO 50 I=1,N
         DO 60 J=1,N
            ARG(I,J)=R(I,J)+KTE(I,J)
60       CONTINUE
50    CONTINUE

C6... Beregner tr(ARG)=L ................................

      CALL TRACE(ARG,L,N,20)

      RETURN
      END
C********************************************************
      SUBROUTINE TRACE(A,L,N,NA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE TRACE                                         C
C   AUTHOR : David Di Ruscio    ITK/NTH  7 DECEMBER 1988         C
C   PURPOSE                                                      C
C   TRACE computes the trace of a squere matrix. trace(A) = L    C
C                                                                C
C   ON INPUT                                                     C
C   A        DOUBLE PRECISION (N,N) matrix.                      C
C   N        INTEGER actual dimension on A.                      C
C   NA       INTEGER leading dimension on A in calling program.  C
C   ON OUTPUT                                                    C
C   L        DOUBLE PRECISION scalar containing the trace(A).    C
C                                                                C
C   ROUTINES CALLED : (NONE)                                     C
C***END PROLOGUE TRACE                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,N,NA
      DOUBLE PRECISION A(NA,N),L

      L=0.0D0
      DO 10 I=1,N
         L=L+A(I,I)
10    CONTINUE

      RETURN
      END
C********************************************************
      SUBROUTINE LINE(DA,DG,D,K,P,R,N,M)

      INTEGER I,J,N,M
      DOUBLE PRECISION DA,DG(20,20),D(20,20),K(20,20)
      DOUBLE PRECISION P(20,20),R(20,20),L1DA,L2DA
      DOUBLE PRECISION PD(20,20),DTPD(20,20),KDTPD(20,20)
      DOUBLE PRECISION SK,L1,L2,DGTD(20,20),LR,LMIN,LSK

      DOUBLE PRECISION C(20,20),CD(20,20),DTCD(20,20),LMOD
      DOUBLE PRECISION CMOD

C...  Initialiserer ......................................
      CMOD=0.0
100   DO 10 I=1,M
         DO 20 J=1,M
            IF (I.EQ.J) THEN
             C(I,J)=CMOD
            ELSE 
             C(I,J)=0.0
            ENDIF
20       CONTINUE
10    CONTINUE

C1...Bestemmer tr((dL/dG)Transp*D)=L.....................
      CALL TRANSMULT(DG,D,DGTD,N,M,N,20,2)
      
      CALL TRACE(DGTD,L1,N,20)

C2....Bestemmer d2L/da2 = L2DA ,(den andrederiverte )....
      CALL MATMULT(P,D,PD,M,M,N,20)
      CALL TRANSMULT(D,PD,DTPD,N,M,N,20,2)
      CALL MATMULT(K,DTPD,KDTPD,N,N,N,20)

      CALL TRACE(KDTPD,L2,N,20)

C....Modifiserer kriteriet ..............................
      CALL MATMULT(C,D,CD,M,M,N,20)
      CALL TRANSMULT(D,CD,DTCD,N,M,N,20,2)
      CALL TRACE(DTCD,LMOD,N,20)

      L2DA=2*(L2+LMOD)

C3....Bestemmer dL/da = L1DA ............................
      L1DA=L2DA*DA-L1

C4....Bestemmer funksjonsverdien i min L(Gk+1) ..........
      CALL TRACE(R,LR,N,20)

      LMIN=LR-L1*DA+L2*DA*DA

C... Oppdaterer skrittlengden DA ........................
      SK=DA-L1DA/L2DA
      LSK=LR-L1*SK+L2*SK*SK

      IF (LSK.LT.0.0) THEN
       CMOD=CMOD+1.0
       WRITE(*,*)'CMOD=',CMOD
       IF (CMOD.GE.200) GO TO 200
       GO TO 100
      ENDIF

200   IF (LSK.GT.0.0.AND.SK.GT.0.0) THEN
       DA=SK
       WRITE(*,*)'OPPDAT NEWTON'
      ENDIF

      IF (L2DA.LT.0.0) THEN
       WRITE(*,*)'* DEN ANDRE DERIVERTE ER NEGATIV *'
      ENDIF

      WRITE(*,*)'LMOD,L2DA',LMOD,L2DA
      WRITE(*,*)'LMIN,LSK,AM',LMIN,LSK,SK
      WRITE(*,*)'LR,L1,L2',LR,L1,L2

      RETURN
      END

      SUBROUTINE STABIL(A,B,G,D,GD,ER,EI,N,M,NY,MAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                               C
C   Rutinen beregner egenverdiene av A+BG samt A                C
C                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,M,ISVAR,ISVAR2
      DOUBLE PRECISION A(MAX,MAX),B(MAX,MAX),G(MAX,MAX),D(MAX,MAX)
      DOUBLE PRECISION ER(*),EI(*),GD(MAX,MAX)
      INTEGER NA
      PARAMETER (NA = 20)
      INTEGER ITYPE(NA)
      DOUBLE PRECISION ABG(NA,NA),FNORM,DSCL(NA),DEPS

      IOUT = 6
      CALL MACEPS(DEPS)

C1....Beregner egenverdiene til A ...............................
      ISVAR=IYESANS('Compute eigenvalues of A    ?',1)
      IF (ISVAR.EQ.2) THEN
       CALL MATCOP(A,ABG,N,N,MAX,MAX)
       CALL DSCHUR(ABG,GD,ER,EI,MAX,N,DEPS,DSCL,ITYPE,0)
       CALL DEIGPRT('EIG(A) =',ER,EI,N,IOUT)
      ENDIF

C2....Beregner Frobenius-normen til A+BG ........................
      ISVAR2=IYESANS('Compute Frobeniusnorm of (A+BGD) ?',1)

C3....Beregner egenverdiene til A+BG ............................
      ISVAR=IYESANS('Compute eigenvalues of (A+BGD) ?',1)

      IF (ISVAR.EQ.2.OR.ISVAR2.EQ.2) THEN
       CALL MATMULT(G,D,GD,M,NY,N,MAX)
       CALL CLABG(ABG,A,B,GD,N,M,NA,NA)
       CALL DSCHUR(ABG,GD,ER,EI,MAX,N,DEPS,DSCL,ITYPE,0)
       CALL DEIGPRT('EIG(A + BGD) =',ER,EI,N,IOUT)
       WRITE(*,2) (I,ER(I),EI(I),I=1,N)
2      FORMAT(1X,'EIG(',I3,') = ',E24.16,' +i ',E24.16)
      ENDIF

C...Beregner Frobeniusnormen
      IF (ISVAR2.EQ.2) THEN
       FNORM=DFNORM(ABG,20,N)
       WRITE(*,1) FNORM
1      FORMAT(2X,'FROBENIUSNORMEN TIL (A+BG) = ',E12.4)
      ENDIF

      RETURN
      END
C***************************************************************
      SUBROUTINE ELMULT(D,F,DXF,N,M,NN,NM)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                              C
C  PURPOSE                                                     C
C  ELMULT computes element multiplication of two matrixes      C
C  D(N,M) and F(N,M). The result are stored in DXF(N,M).       C
C  DXF(N,M) = D(N,M) x F(N,M)                                  C
C                                                              C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,N,M,NN,NM
      DOUBLE PRECISION D(NN,NM),F(NN,NM),DXF(NN,NM)

      DO 10 I=1,M
         DO 20 J=1,N
            DXF(I,J)=D(I,J)*F(I,J)
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE STRUCT(G,F,N,M,NA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE STRUCT                                       C
C   PURPOSE                                                     C
C   Specify controller structure.                               C
C***END PROLOGUE STRUCT                                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,N,M,IVALG,NA
      DOUBLE PRECISION F(NA,NA),G(NA,NA)
      INTEGER MAX
      PARAMETER (MAX = 20)
      DOUBLE PRECISION GS(MAX,MAX)
      CHARACTER CFORM*6

      IVALG = 2
100   CFORM = '(5X,A)'
      WRITE(*,CFORM)'     <  STRUCTURE SPECIFICATIONS  >  '
      WRITE(*,CFORM)'-----------------------------------------'
      WRITE(*,CFORM)' 1. Sentraliced controller   G.          '
      WRITE(*,CFORM)' 2. Decentralized controller G.          '
      WRITE(*,CFORM)'-----------------------------------------'
      WRITE(*,CFORM)'  '
      
      CALL NREAD('Choose 1 or 2  !     ',IVALG,1,1,IIII)

C1...Sentralized computation ie. F = (1)  .......................
      IF (IVALG.EQ.1) THEN
       WRITE(*,'(1X,A)')'Warning -> It is faster to use CONLAB'
       WRITE(*,'(1X,A)')'           or MATLAB/control.'
       DO 10 I=1,M
          DO 20 J=1,N
             F(I,J)=1.0D0
20        CONTINUE
10     CONTINUE

C2...Decentralized computing, (specifying structure on F(N,M))
      ELSEIF (IVALG.EQ.2) THEN
       WRITE(*,*)' Specify controller structure matrix F.     '
       WRITE(*,*)' Type F(I,J)=1 for legal element and        '
       WRITE(*,*)'      F(I,J)=0 for illegal element in G.    '
       WRITE(*,*)'                                            '
       CALL DREMA(F,GS,M,20,N,20,'F')

C3..Correct the startvalue on G with the structure matrix F .....
       CALL ELMULT(G,F,GS,N,M,20,20)
       DO 30 I=1,M
          DO 40 J=1,N
             G(I,J)=GS(I,J)
40        CONTINUE
30     CONTINUE
      ELSE
       WRITE(*,'(5X,A)')'Available : 1 or 2'
       IVALG = 2
       GO TO 100
      ENDIF

      RETURN
      END

      SUBROUTINE JVALUE(A,B,D,P,Q,R,X,F,L,N,M,NY,IFLAG,MAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                              C
C  Rutinen beregner funksjonsverdien J(X)=L(G)=tr(R) der       C
C  R er gitt av,                                               C
C           T                     T                            C
C     (A+BG) *R + R*(A+BG) = -(Q+G *P*G)                       C
C                                                              C
C  X(I),I=1,...,N*M vektor fra spalter i G(I,J)                C
C  G(I,J) en M x N matrise.                                    C
C                                                              C
C  ROUTINES CALLED : MVTRAN,XG,REQ1,TRACE                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,M,NY,IFLAG,MET,NM,MAX
      DOUBLE PRECISION A(MAX,MAX),B(MAX,MAX),P(MAX,MAX),Q(MAX,MAX)
      DOUBLE PRECISION X(MAX),F(MAX,MAX),D(MAX,MAX),R(MAX,MAX)
      INTEGER NW
C... NOTE NW must be equal MAX
      PARAMETER (NW = 20)
      INTEGER ITYPE(NW)
      DOUBLE PRECISION G(NW,NW),L,DEPS,DMUR,DMU
      DOUBLE PRECISION ER(NW),EI(NW),U(NW,NW),WQ(NW,NW),WV(NW)
      DOUBLE PRECISION E1,E2,E3
      PARAMETER (E1 = 6.0, E2 = 4.0, E3 = 2.0)
      CHARACTER CRIT*3
      COMMON /PAR/ DMUR,CRIT
C.. Debug parameters
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

C1... Transformerer vektoren X(I) til matrisen G(I,J) ..........
CUT   CALL XG(G,F,X,N,M,NM,2,MAX)
      CALL XG(G,F,X,NY,M,NM,2,MAX)
      
C2... L|ser R fra likning (2) se rutine DLDK ...................
      MET=2
      CALL REQ1(A,B,D,G,P,Q,R,N,M,NY,IFLAG,MET,MAX)

C*..  Compute criterion value
C1... Beregner L(X) = L(G) = tr(R) .............................
      IF (CRIT.EQ.'TR') THEN
       CALL TRACE(R,L,N,MAX)

C2.. Robust criterion (Patel and Toda)
      ELSEIF (CRIT.EQ.'MU') THEN
       CALL MACEPS(DEPS)
       CALL DRELYP(DMU,Q,P,R,G,D,N,M,NY,NW,NW,DEPS,ER,EI,U,WQ,WV,
     +             ITYPE,1)

       L = (DMUR - DMU)*(DMUR - DMU)
CUT    WRITE(*,'(1X,A,G16.6,A,G16.6)')'DMU =',DMU,' L =',L

C3.. To find stabilizing feedback matrix
      ELSEIF (CRIT.EQ.'ST') THEN
       CALL MACEPS(DEPS)
       CALL DRELYP(DMU,Q,P,R,G,D,N,M,NY,NW,NW,DEPS,ER,EI,U,WQ,WV,
     +             ITYPE,2)

       IF (DMU.LT.0.0) THEN
        L = (DMUR - DMU)*(DMUR - DMU)*(DMUR - DMU)*(DMUR - DMU)
CUT     L = (DMUR - DABS(DMU))*(DMUR - DABS(DMU))
        IF (IPRT.EQ.1) THEN
         WRITE(*,'(1X,A,G16.6,A,G16.6)')' - DMU =',DMU,' L(R)=',L
        ENDIF
 
       ELSEIF (DMU.GE.0.0) THEN
        L = (DMUR - DMU)*(DMUR - DMU)
CUT     L = (DMUR - DABS(DMU))*(DMUR - DABS(DMU))
CUT     WRITE(*,'(1X,A,G16.6,A,G16.6)')' + DMU =',DMU,' L(R)=',L
CUT     WRITE(*,'(1X,A,G16.6)')'Stable system found :'
       ENDIF

      ENDIF

      RETURN
      END

      SUBROUTINE REQ1(A,B,D,G,P,Q,R,N,M,NY,IFLAG,MET,MAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                               C
C     Rutinen l|ser Liapunovligningen                           C
C            T                    T                             C
C      (A+BG) *R + R*(A+BG)+ Q + G *P*G = 0     (1)             C
C                                                               C
C     med hensyn p} R for gitt G.                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      INTEGER I,J,N,M,NY,IFLAG,MET,MAX
      DOUBLE PRECISION A(MAX,MAX),B(MAX,MAX),G(MAX,MAX),P(MAX,MAX)
      DOUBLE PRECISION Q(MAX,MAX),R(MAX,MAX),D(MAX,MAX)

      PARAMETER (N1 = 20, NP = N1*(N1+1)/2 )
      INTEGER IND(NP)
      DOUBLE PRECISION WM(NP,NP),WV(NP)
      DOUBLE PRECISION ABG(20,20),GTP(20,20),GTPG(20,20),QQ(20,20)
      DOUBLE PRECISION GD(20,20)
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

      IF (IPRT.EQ.2) THEN
       CALL DMAPRT('Enter REQ1  A =',A,20,20,N,N,IOUT)
       CALL DMAPRT('Enter REQ1  B =',B,20,20,N,M,IOUT)
       CALL DMAPRT('Enter REQ1  G =',G,20,20,M,NY,IOUT)
       CALL DMAPRT('Enter REQ1  D =',D,20,20,NY,N,IOUT)
      ENDIF

C... If output feedback, compute GD = G*D as new feedback matrix
      CALL MATMULT(G,D,GD,M,NY,N,MAX)
      IF (IPRT.EQ.2) THEN
       CALL DMAPRT('Exit G*D = GD =',GD,20,20,M,N,IOUT)
      ENDIF

C...  L\SER LIGNING (1) .........................................
C1... Setter opp A+BG ...........................................
      
CUT   CALL CLABG(ABG,A,B,G,N,M,MAX,MAX)
      CALL CLABG(ABG,A,B,GD,N,M,MAX,MAX)
      IF (IPRT.EQ.2) THEN
       CALL DMAPRT('Exit CLABG ABGD =',ABG,20,20,N,N,IOUT)
      ENDIF

C2... Setter opp QQ = Q + GT*P*G ................................
CUT   CALL TRANSMULT(G,P,GTP,N,M,M,MAX,2)
      CALL TRANSMULT(GD,P,GTP,N,M,M,MAX,2)
CUT   CALL MATMULT(GTP,G,GTPG,N,M,N,MAX)
      CALL MATMULT(GTP,GD,GTPG,N,M,N,MAX)
      DO 30 I=1,N
         DO 40 J=1,N
            QQ(I,J)=Q(I,J)+GTPG(I,J)
40       CONTINUE
30    CONTINUE

C3... L|ser Liapunovlikningen mht. R ............................

      IF (IPRT.EQ.2) THEN
       CALL DMAPRT('Enter DLYP  ABGD =',ABG,20,20,N,N,IOUT)
       CALL DMAPRT('Enter DLYP  R    =',R,20,20,N,N,IOUT)
       CALL DMAPRT('Enter DLYP  QQ   =',QQ,20,20,N,N,IOUT)
      ENDIF

      CALL DLYP(ABG,QQ,R,20,N,WM,WV,NP,IND,INFO)
      IF (INFO.NE.0) WRITE(*,'(1X,A,I5)')'DLYP INFO =',INFO

      IF (IPRT.EQ.2) THEN
       CALL DMAPRT('Exit DLYP  ABGD =',ABG,20,20,N,N,IOUT)
       CALL DMAPRT('Exit DLYP  R    =',R,20,20,N,N,IOUT)
       CALL DMAPRT('Exit DLYP  QQ   =',QQ,20,20,N,N,IOUT)
      ENDIF


      RETURN
      END

C****************************************************************

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                              C
C    Rutine for definering av Z(X)                             C
C                                                              C
C    N - aktuell dimensjon p} param. vektor X(I),I=1,...,N     C
C    M - dimensjon p} funksjonal. Settes til 1 , M=1           C
C    T - Forel|pig en dummy- parameter                         C
C    X - Parameter-vektoren med dim N                          C
C    Z - Objektfunksjonen det skal finnes min av med dim M=1   C
C    IERR- flag som setter IFLAG=2 i kallende rutine OPTIM     C
C          og avslutter iterasjonene dersom IERR=2             C
C                                                              C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       SUBROUTINE FCN(NM,M1,T,X,Z)

       INTEGER NM,M1
       DOUBLE PRECISION T,X(NM),Z(NM)

       INTEGER N,M,NY,IFLAG,MAX
       PARAMETER (MAX = 20)
       DOUBLE PRECISION AA(MAX,MAX),B(MAX,MAX),P(MAX,MAX),Q(MAX,MAX)
       DOUBLE PRECISION XX(MAX),F(MAX,MAX),L,D(MAX,MAX),R(MAX,MAX)
       INTEGER NC,ND,IGRAD

       COMMON /DESMAT/ AA,B,D,P,Q,R,F
       COMMON /INT/ N,M,NY
       COMMON /IDB/ NC,ND,IGRAD
       NC = NC + 1

       DO 10 I=1,NM
          XX(I)=X(I)
10     CONTINUE

       CALL JVALUE(AA,B,D,P,Q,R,XX,F,L,N,M,NY,IFLAG,MAX)
       Z(1)=L

       RETURN
       END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                             C
C      Rutine for loesning av Ax = b ved LU - dekomponering.  C
C                                                             C
C      A  DOUBLE PRECISION used as work space.                C
C                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       SUBROUTINE AXB(AIN,B,DET,N,II,III,IFLAG,MAX,A)

       INTEGER NM1,N,I,KP1,J,IM1,IB,IP1,II,III,IFLAG
       DOUBLE PRECISION AIN(MAX,MAX),B(MAX),T,DET
       DOUBLE PRECISION A(MAX,MAX)

       DET=1.0
       IFLAG=0
       DO 110 I=1,N
          DO 120 J=1,N
             A(I,J)=AIN(I,J)
120       CONTINUE
110    CONTINUE

C......LU - dekomponerer A....................................
       IF (II.EQ.1 .AND. III.EQ.1) THEN
       NM1=N-1
       DO 10 K=1,NM1

          KP1=k+1
          DO 20 I=KP1,N

             IF (A(K,K).EQ.0) THEN
              IFLAG=3
              DET=0.0
              GO TO 100
             ENDIF

             T=A(I,K)/A(K,K)
             A(I,K)=T

             DO 30 J=KP1,N
                A(I,J)=A(I,J)-T*A(K,J)
30           CONTINUE

20        CONTINUE
10     CONTINUE
       ENDIF

C.....Beregner detA ........................................
       DO 80 I=1,N
          DET=DET*A(I,I)
          IF (A(I,I).LT.0.0) IFLAG=4
80     CONTINUE
       IF (DET.EQ.0) THEN
        IFLAG=3
        GO TO 100
       ENDIF

C......Loser Ly=b...........................................
       DO 40 I=2,N
          IM1=I-1
          DO 50 K=1,IM1
             IF(A(I,K).NE.0) THEN
             B(I)=B(I)-A(I,K)*B(K)
             ENDIF
50        CONTINUE
40     CONTINUE

C......Loser Lx = y .......................................
       B(N)=B(N)/A(N,N)
       DO 60 IB=2,N
          I=N-IB+1
          IP1=I+1
          DO 70 K=IP1,N
             IF (A(I,K).NE.0) THEN
             B(I)=B(I)-A(I,K)*B(K)
             ENDIF
70        CONTINUE
          B(I)=B(I)/A(I,I)
60     CONTINUE

100    RETURN
       END

       SUBROUTINE GRAD2(N,X,GRAD,MM,IJAC)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE GRAD2                                         C
C   PURPOSE                                                      C
C   GRAD2 is used to numerically computation of the gradient     C
C   vector dJ/dx.                                                C
C                                                                C
C***END PROLOGUE GRAD2                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       INTEGER I,N,IJAC
       DOUBLE PRECISION X(MM),GRAD(MM)
       INTEGER MAX
       PARAMETER (MAX = 20)
       DOUBLE PRECISION W1(MAX),W2(MAX),Z(MAX)
       DOUBLE PRECISION DEPS,H,T,J1,J2
       PARAMETER (H = 1.0D-05)

C...Forward difference 
       IF (IJAC.EQ.1) THEN

        CALL FCN( N,1,T,X,Z(1) )
        J1 = Z(1)
        DO 10 I=1,N
           DEPS = DABS( H*X(I) )
           IF ( DEPS.LT.H ) DEPS = H
           X(I) = X(I) + H
           CALL FCN(N,1,T,X,Z(1))
           J2 = Z(1)
           GRAD(I) = (J2-J1)/DEPS
           X(I) = X(I) - H
10      CONTINUE

C...Midtpont difference
       ELSEIF (IJAC.EQ.2) THEN

        DO 20 I=1,N
           W1(I) = X(I)
           W2(I) = X(I)
20      CONTINUE

        DO 30 I=1,N

           W1(I)=W1(I)+H  
           CALL FCN(N,1,T,W1,Z(1))
           J1=Z(1)

           W2(I)=W2(I)-H
           CALL FCN(N,1,T,W2,Z(1))
           J2=Z(1)
           GRAD(I)=(J1-J2)/(2.0*H)

           W1(I)=W1(I)-H
           W2(I)=W2(I)+H
30      CONTINUE

       ENDIF
        
       RETURN
       END

       SUBROUTINE HESS2(N,X,G,HESS,MM,IJAC)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE HESS2                                        C
C   PURPOSE                                                     C
C   HESS2 is used to numericaly computation of the Hessian      C
C   matrix.                                                     C
C***END PROLOGUE HESS2                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       INTEGER I,J,N,MM,IJAC
       DOUBLE PRECISION X(MM),G(MM),HESS(MM,MM),H
       INTEGER MAX
       PARAMETER (MAX = 20)
       DOUBLE PRECISION XK(MAX),GK(MAX),HE(MAX,MAX)

       PARAMETER (H = 1.0D-05)

       DO 10 J=1,N
          DO 20 I=1,N
             IF (I.EQ.J) THEN
              XK(I)=X(I)+H
             ELSE
              XK(I)=X(I)
             ENDIF
20        CONTINUE
          
          CALL GRAD2(N,XK,GK,MM,IJAC)

          DO 30 I=1,N
             HE(I,J)=(GK(I)-G(I))/H
30        CONTINUE
10     CONTINUE

       DO 40 I=1,N
          DO 50 J=1,N
             HESS(I,J)=(HE(I,J)+HE(J,I))/2.0
50        CONTINUE
40     CONTINUE

       RETURN
       END

      SUBROUTINE XG(G,F,X,N,M,NM,II,MAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE XG                                           C
C    Rutinen transformerer,                                     C
C    1. elementer forskjellig fra null i en matrise G (M x N)   C
C       til en vektor X.                                        C
C    2. en vektor X til en matrise G med struktur gitt av F     C
C***END PROLOGUE XG                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,N,M,II,NM,MAX
      DOUBLE PRECISION G(MAX,MAX),F(MAX,MAX),X(MAX)

C1...Transformerer elementer forskjellig fra null i G til .......
C... en vektor X ................................................
      MM=0
      
      IF (II.EQ.1) THEN
       DO 10 I=1,M
          DO 20 J=1,N
             IF (F(I,J).EQ.1.0) THEN
              MM=MM+1
              X(MM)=G(I,J)
             ENDIF
20        CONTINUE
10     CONTINUE
       NM=MM
      ELSEIF (II.EQ.2) THEN

C2... Transformerer vektor X til matrise G ......................
      DO 30 I=1,M
         DO 40 J=1,N
            IF (F(I,J).EQ.1) THEN
             MM=MM+1
             G(I,J)=X(MM)
            ELSE
             G(I,J)=0.0
            ENDIF
40       CONTINUE
30    CONTINUE

      ENDIF

      RETURN
      END

      SUBROUTINE NEWT2(NM,HESS,GRAD,P,DET,MM,IFLAG)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE NEWT2                                         C
C   PURPOSE                                                      C
C   Solve  HESS*( X(k+1) - X(k) ) = -GRAD = P for X(k+1)         C
C                                                                C
C   ROUTINES CALLED : AXB                                        C
C***END PROLOGUE NEWT2                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,NM,MM,IFLAG
      DOUBLE PRECISION HESS(MM,MM),GRAD(MM),DET
      DOUBLE PRECISION P(MM)
      INTEGER MAX
      PARAMETER (MAX = 20)
      DOUBLE PRECISION WM(MAX,MAX)

C1... Lager hoeyresiden i HESS*(X(k+1)-X(k)) = -GRAD .............
      DO 10 I=1,NM
         P(I)=-GRAD(I)
10    CONTINUE

C2... Loeser lineart likningssystem ved LU dekomposisjon .........
      CALL AXB(HESS,P,DET,NM,1,1,IFLAG,MM,WM)

      RETURN
      END
      SUBROUTINE XXMULT(X,H,N,NA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  ITK/NTH 31/10-88                                              C
C                                                                C
C  XXMULT utfoerer vektormultiplikasjonen H = X*Xtransp          C
C  INPUT                                                         C
C  X1   Vektor med dimensjon N                                   C
C  N    Dimensjonen paa X                                        C
C  NA   Dimensjonen paa X og H i det kallende program            C
C  H    En vilkaarlig matrise med dimensjon N x N                C
C  OUTPUT                                                        C
C  H    Inneholder produktet X*Xtransponert                      C
C                                                                C
C  VERSJON : DOUBLE PRECISION                                    C
C  KALLER  : (ingen)                                             C
C                                                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,N,NA
      DOUBLE PRECISION X(NA),H(NA,NA),C

      DO 10 I=1,N
         DO 20 J=1,N
            C=0.0
            C = C + X(I)*X(J)
            H(I,J)=C
20       CONTINUE
10    CONTINUE

      RETURN
      END
        
      function dfnorm(mat,nm,n)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                  C
C    FNORM computes the Frobenius norm of a matrix                 C
C                                                                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      integer nm,n
      double precision mat(nm,nm),fnorm1,fabs
c
      fnorm1=0.0
c
      do 100 i=1,n
         do 100 j=1,n
         fabs = abs(mat(i,j))
         if (fabs.gt.1.e15) fabs=1.e15
         fnorm1 = fnorm1 + fabs*fabs
100   continue
      dfnorm = sqrt(fnorm1)
c
      return
      end
      SUBROUTINE MATMULT(A,B,C,N,M,P,NA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  AUTHOR                                                       C
C  David Di Ruscio october 1986                                 C
C  VERSJON 2:   17 october 1988                                 C
C                                                               C
C    PURPOSE                                                    C
C      Routine for matrix multiplication                        C
C      C=A B , A=(N xM), B=(M x P), C=(N x P)                   C
C                                                               C
C    ON INPUT                                                   C
C    A, B, C                                                    C
C    NA  Leding dimensjon of A, B, ond C in calling             C
C        program                                                C
C    ON OUTPUT                                                  C
C    C   The product C = A*B                                    C
C                                                               C
C    ROUTINES CALLED  : (none)                                  C
C    VERSION          : DOUBLE PRECISION                        C
C                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,K,N,M,P,NA
      DOUBLE PRECISION A(NA,N),B(NA,N),C(NA,N),X

      DO 10 I=1,N
         DO 20 J=1,P
            X=0.0
            DO 30 K=1,M
               X=X+A(I,K)*B(K,J)
30          CONTINUE
            C(I,J)=X
20       CONTINUE
10    CONTINUE
      RETURN
      END
C****************************************************************

      SUBROUTINE TRANSMULT(A,B,C,N,M,P,NA,JOB)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      AUTHOR
C      David Di Ruscio       NTH october 1986                   C
C      Versjon 2:             17 october 1988                   C
C                                                               C
C      PURPOUSE                                                 C
C      TRANSMULT performs multiplication of a matrix with the   C
C      transpose of another matrix. Bohuf pre and post-         C
C      multipilication could be done.                           C
C                                                               C
C      ON INPUT                                                 C
C      A=(N x M),B=(M x P), C=(N x P)                           C
C      NA             Leading dimensjon of A,B and C in calling C
C                     program.                                  C
C      JOB = 1        C = A*Btrans                              C
C      JOB = 2        C = Atrans*B                              C
C                                                               C
C      C              The matrix product                        C
C      A,B            Are unchanged on output                   C
C                                                               C
C      ROUTINES CALLED : (none)                                 C
C      VERSION         : DOUBLE PRECISION                       C
C                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER I,J,K,N,M,P,NA,JOB
      DOUBLE PRECISION A(NA,N),B(NA,N),C(NA,N),X

C....JOB=1 then postmult C = A*BT ...............................
      IF (JOB.EQ.1) THEN
       DO 10 I=1,N
          DO 20 J=1,P
             X=0.0
             DO 30 K=1,M
                X=X+A(I,K)*B(J,K)
30           CONTINUE
             C(I,J)=X
20        CONTINUE
10     CONTINUE

C...  JOB=2 then premult C = AT*B ...............................
      ELSEIF (JOB.EQ.2) THEN
       DO 11 I=1,N
          DO 21 J=1,P
             X=0.0
             DO 31 K=1,M
                X=X+A(K,I)*B(K,J)
31           CONTINUE
             C(I,J)=X
21        CONTINUE
11     CONTINUE
      ENDIF

      RETURN
      END

      SUBROUTINE CLABG(ABG,A,B,G,NX,NU,NA,NB)
C---------------------------------------------------------------C
C***BEGIN PROLOGUE CLABG                                        C
C                                                               C
C   PURPOSE                                                     C
C   CLABG computes the closed loop matrix A + B*G. A, B, and G  C
C   are unchanged on output.                                    C
C   ON INPUT                                                    C
C   A        DOUBLE PRECISION (NA,NX) system matrix.            C
C   B        DOUBLE PRECISION (NB,NU) control matrix.           C
C   G        DOUBLE PRECISION (NB,NX) feedback matrix.          C
C   NX       INTEGER number of states.                          C
C   NU       INTEGER number of control variable.                C
C   NA,NB    INTEGER leading dimensions on A and B respectively.C
C   ON OUTPUT                                                   C
C   ABG      DOUBLE PRECISION (NA,NX) the result A+B*G.         C
C                                                               C
C   ROUTINES CALLED : MATMULT                                   C
C***END PROLOGUE CLABG                                          C
C---------------------------------------------------------------C
      INTEGER NX,NU,NA,NB
      DOUBLE PRECISION ABG(NA,NX),A(NA,NX),B(NB,NU),G(NB,NX)

C1... Compute the product B*G           .........................
      CALL MATMULT(B,G,ABG,NX,NU,NX,NA)

C2... Compute A + B*G                   .........................
      DO 10 I=1,NX
         DO 20 J=1,NX
            ABG(I,J) = A(I,J) + ABG(I,J)
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE AALFA(A,ALFA,AA,N,NA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE AALFA                                         C
C   PURPOSE                                                      C
C   Compute the matrix  AA = A + alfa*I                          C
C***END PROLOGUE AALFA                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER N,NA,I,J
      DOUBLE PRECISION A(NA,N),AA(NA,N),ALFA

      DO 10 I=1,N
         DO 20 J=1,N
            IF (I.EQ.J) THEN
             AA(I,J) = A(I,J) + ALFA
            ELSE
             AA(I,J) = A(I,J)
            ENDIF
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE DIMCHK(NX,NU,NY,MAX,IERR)
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C    Check input dimension for consistensy and maximum dimensions       c
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER NX,NU,NY,MAX,IERR

      IERR = 0
      IF (NX.GT.MAX) IERR = 1
      IF (NU.GT.MAX) IERR = 2
      IF (NY.GT.MAX) IERR = 3

      RETURN
      END

      SUBROUTINE DESHLP
C--------------------------------------------------------------------c
C     Print help info on terminal                                    c
C--------------------------------------------------------------------c
      CHARACTER CF*6
      CF = '(1X,A)'

      WRITE(*,CF)'                < DESREG HELP >'
      WRITE(*,CF)'----------------------------------------------------'
      WRITE(*,CF)'1.  Specify system dimensions ( NX NU NY )     DIM  '
      WRITE(*,CF)'    NX = # of states                                '
      WRITE(*,CF)'    NU = # of inputs                                '
      WRITE(*,CF)'    NY = # of outputs                               '
      WRITE(*,CF)'2.  Read system matrices from terminal         SYS  '
      WRITE(*,CF)'3.  Solve problem                              SOL  '
      WRITE(*,CF)'4.  Load system matrices from data file        LOAD '
      WRITE(*,CF)'5.  Save system matrices on data file          SAVE '
      WRITE(*,CF)'6.  Help                                       HELP '
      WRITE(*,CF)'7.  Terminate program with                0 or EXIT '
      WRITE(*,CF)'9.  Controller structure specification         STRUC'
      WRITE(*,CF)'10. Compute eigenvalues etc.                   EIG  '
      WRITE(*,CF)'11. Initialize optional parameters             INIT '
      WRITE(*,CF)'----------------------------------------------------'
      WRITE(*,CF)'    '

      RETURN
      END

      SUBROUTINE DESPAR(ALFA,IPRT,IOUT,CRIT,DMUR)
      INTEGER IPRT,IOUT
      DOUBLE PRECISION ALFA,DMUR
      CHARACTER CRIT*(*)
      INTEGER IUT(2)

C6.. Specify other parameters
100   CALL DREAD('Prescribed degre of stability ALFA=?',ALFA,1,1,IIII)
      IF (ALFA.LT.0.0) THEN
       ALFA = 0.0
       WRITE(*,'(1X,A)')'---> Alfa must be ge. zero'
       GO TO 100
      ENDIF

C... Specify output chanel
      IUT(1) = IPRT
      IUT(2) = IOUT
      CALL NREAD('IPRT - IOUT ?',IUT,2,2,IIII)
      IPRT = IUT(1)
      IOUT = IUT(2)

C... Specify criterion
200   CALL RCSTR('Choose criterion ( TR MU or ST):',CRIT,1,1,IIII)
C... MU -> robust criterion
      IF (CRIT.EQ.'MU') THEN
210   CALL DREAD('Nominal stability DMUR =',DMUR,1,1,IIII)
      IF (DMUR.LT.0.0) GO TO 210

C... TR -> normal LQ-criterion
      ELSEIF (CRIT.EQ.'TR') THEN

C... ST -> criterion to find a initializing stabilizing feedback G
      ELSEIF (CRIT.EQ.'ST') THEN
       CALL DREAD('Specified min EIG(R) =',DMUR,1,1,IIII)

      ELSE
       WRITE(*,'(1X,A)')'Available : TR or MU'
       CRIT = 'TR'
       GO TO 200
      ENDIF

      RETURN
      END

      SUBROUTINE DESERR(NX,NU,NY,F,MAX,IERR,IMAIN)
      INTEGER NX,NU,NY,IERR,IMAIN
      DOUBLE PRECISION F(MAX,MAX)

      IERR = 0
C... Check dimensions
      IF (NX.EQ.0 .OR. NU.EQ.0 .OR. NY.EQ.0) THEN
       WRITE(*,'(1X,A)')'Error --> dimensions out of bounds'
       WRITE(*,'(1X,A,I5)')'(NX NU NY ) = ',NX,NU,NY
       IERR  = 1
       IMAIN = 1
       GO TO 1000
      ENDIF

C.. Check that F is not zero
      IERR = 9
      DO 11 I=1,NU
         DO 12 J=1,NY
            IF (F(I,J).NE.0) THEN
             IERR = 0
             GO TO 1000
            ENDIF
12       CONTINUE
11    CONTINUE
      IF (IERR.EQ.9) THEN
       WRITE(*,'(1X,A)')'ERROR --> Structure matrix F is zero.'
       GO TO 1000
      ENDIF

1000  RETURN
      END

      SUBROUTINE FX(FVAL,X,NM)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                       C
C   To compute the functional value F(X) to find suboptimal       C
C   controllers.                                                  C
C   Called from NLPL0 used in DESREG.                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NM
      DOUBLE PRECISION FVAL,X(NM)
C.... Parameters common with main program               ...........
      INTEGER N,M,NY,IFLAG,MAX,NC,ND,IGRAD
      PARAMETER (MAX = 20)
      DOUBLE PRECISION AA(MAX,MAX),B(MAX,MAX),P(MAX,MAX),Q(MAX,MAX)
      DOUBLE PRECISION XX(MAX),F(MAX,MAX),L,D(MAX,MAX),R(MAX,MAX)
C... DEBUG parameters
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT
C... System data
      COMMON /DESMAT/ AA,B,D,P,Q,R,F
      COMMON /INT/ N,M,NY
C... Counter
      COMMON /IDB/ NC,ND,IGRAD
      NC = NC + 1

C...  Compute functional value                         ...........
      DO 10 I=1,NM
         XX(I) = X(I)
10    CONTINUE
      CALL JVALUE(AA,B,D,P,Q,R,XX,F,L,N,M,NY,IFLAG,MAX)

      IF (IPRT.EQ.2) THEN
       CALL DESOUT(AA,Q,R,N,M,NY,MAX)
       WRITE(*,'(1X,A,G16.6)')'j(x0) = ',L
      ENDIF

C...  Store functional value
      FVAL = L

      RETURN
      END

      SUBROUTINE DRELYP(DMU,Q,P,R,G,D,N,M,NY,NQ,NP,DEPS,ER,EI,U,WQ,WV,
     +                   ITYPE,ICRIT)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***BEGIN PROLOGUE DRELYP                                              c
C                                                                      c
C***END PROLOGUE DRELYP                                                c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER N,M,NY,NQ,NP,ITYPE(N)
      DOUBLE PRECISION Q(NQ,N),P(NP,M),R(NQ,N),G(NP,N),DMU,U(NQ,N)
      DOUBLE PRECISION DEPS,LQ,LR,ER(N),EI(N),WV(N),WQ(NQ,N),D(NQ,N)
      PARAMETER (MAX = 20)
      DOUBLE PRECISION GD(MAX,MAX)
      INTEGER IOUT,IPRT
      common /iutil/ iout,iprt

C.. Robust criterion Patel and Toda
      IF (ICRIT.EQ.1) THEN
C... Set up Q = Q + tr(G)PG
       CALL MATMULT(G,D,GD,M,NY,N,MAX)
       CALL MATBA1(GD,P,WQ,M,N,NP,NP,WV)
       IF (IPRT.EQ.1) THEN
        CALL MATPRT('GD =',GD,NQ,NQ,M,N)
        CALL MATPRT('tr(G)inv(P)G =',WQ,NQ,NQ,N,N)
       ENDIF
       DO 10 I=1,N
          DO 20 J=1,N
             WQ(I,J) = Q(I,J) + WQ(I,J)
20        CONTINUE
10     CONTINUE
       IF (IPRT.EQ.1) THEN
        CALL MATPRT('QQ =',WQ,NQ,NQ,N,N)
       ENDIF

C... Compute eigenvalues of WQ
       CALL DSCHUR(WQ,U,ER,EI,NQ,N,DEPS,WV,ITYPE,0)
       IF (IPRT.EQ.1) THEN
        CALL VECPRT('ER(Q) =','ER',ER,N,N)
       ENDIF
       LQ = ER(N)

      ENDIF

C... Compute eigenvalues of R
      CALL MATCOP(R,WQ,N,N,NQ,NQ)
C     CALL DSCHUR(WQ,U,ER,EI,NQ,N,DEPS,WV,ITYPE)
      CALL DSCHUR(WQ,U,ER,EI,NQ,N,DEPS,WV,ITYPE,0)
      LR = ER(1)
      IF (IPRT.EQ.1) THEN
       CALL VECPRT('ER(R) =','ER',ER,N,N)
      ENDIF

C... Compute robustness kriterion value
      IF (ICRIT.EQ.1) THEN
       DMU = 0.5*LQ/LR

C... To find initializing control
      ELSEIF (ICRIT.EQ.2) THEN
       CALL DVSORT(ER,N,N)
       DMU = ER(N)
       IF (DMU.GE.0.0) THEN
C... System stable, stabilizing controller is found.
C       CALL VECPRT('ER(R) =','ER',ER,N,N)
C       CALL MATPRT('G = ',G,NP,N,M,NY)
C       CALL MATPRT('R = ',R,NQ,N,N,N)
C... Force optimization algoritm to stop
CUT     STOP
cut     DMU = 1E10
       ENDIF

      ENDIF

      RETURN
      END

      SUBROUTINE DGMAT(G,PI,B,R,W,NX,NU,NA,NB)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C***BEGIN PROLOGUE DGMAT                                        C
C   PURPOSE                                                     C
C   DGMAT computes the optimal feedback G = -inv(P)*tr(B)*R     C
C                                                               C
C***END PROLOGUE DGMAT                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NX,NU,NA,NB,I,J
      DOUBLE PRECISION G(NB,NX),PI(NB,NU),B(NB,NU),R(NA,NX),W(NB,NX)

C1... Compute feedback G = -inv(P)*tr(B)*R      .................
      CALL TRANSMULT(PI,B,W,NU,NU,NX,NA,1)
      CALL MATMULT(W,R,G,NU,NX,NX,NA)
      DO 10 I=1,NU
         DO 20 J=1,NX
            G(I,J) = - G(I,J)
20       CONTINUE
10    CONTINUE

      RETURN
      END

      SUBROUTINE DESOUT(A,Q,R,NX,NU,NY,MAX)
      DOUBLE PRECISION A(MAX,MAX),Q(MAX,MAX),R(MAX,MAX)

      WRITE(*,*)'NX NU NY ',NX,NU,NY
      IOUT = 6
      CALL DMAPRT('A =',A,MAX,MAX,NX,NX,IOUT)
      CALL DMAPRT('Q =',Q,MAX,MAX,NX,NX,IOUT)
      CALL DMAPRT('R =',R,MAX,MAX,NX,NX,IOUT)

      RETURN
      END

      SUBROUTINE VECCOP(XIN,XUT,N)
      INTEGER N
      DOUBLE PRECISION XIN(N),XUT(N)
      DO 10 I=1,N
         XUT(I) = XIN(I)
10    CONTINUE
      
      RETURN
      END

      SUBROUTINE FXCNST(CC,X,NM,ACTIVE,MC,ME,MMAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                       C
C   To compute the functional value F(X) to find suboptimal       C
C   controllers.                                                  C
C   Called from NLPL0 used in DESREG.                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER NM,MC,ME
      DOUBLE PRECISION CC(NM),X(NM)
      LOGICAL ACTIVE(MMAX)
C.... Parameters common with main program               ...........
      INTEGER N,M,NY,IFLAG,MAX,NC
      PARAMETER (MAX = 20)
      DOUBLE PRECISION AA(MAX,MAX),B(MAX,MAX),P(MAX,MAX),Q(MAX,MAX)
      DOUBLE PRECISION XX(MAX),F(MAX,MAX),L,D(MAX,MAX),R(MAX,MAX)
      INTEGER NP,NI,NW
      PARAMETER (NP = MAX*(MAX+1)/2, NI=NP+3, NW=NP+MAX)
      INTEGER IW(NI)
      DOUBLE PRECISION DW(NW),DET

C... DEBUG parameters
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT
C... System data
      COMMON /DESMAT/ AA,B,D,P,Q,R,F
      COMMON /INT/ N,M,NY
C... Counter
      COMMON /IDB/ NC,ND,IGRAD
      ND = ND + 1

C...  Compute functional value                         ...........
      DO 10 I=1,NM
         XX(I) = X(I)
10    CONTINUE
      CALL JVALUE(AA,B,D,P,Q,R,XX,F,L,N,M,NY,IFLAG,MAX)

      IF (IPRT.EQ.2) THEN
       CALL DESOUT(AA,Q,R,N,M,NY,MAX)
       WRITE(*,'(1X,A,G16.6)')'j(x0) = ',L
      ENDIF

C...  Compute sub determinants of R
      IF (ACTIVE(1)) CC(1) = R(1,1)
      DO 20 II=2,N
         IF (ACTIVE(II)) THEN
          CALL DSDETI(R,II,MAX,IW,NI,DW,NW,DET,010)
          CC(II) = DET
         ENDIF
20    CONTINUE

      IF (IPRT.EQ.1) THEN
       CALL MATPRT('R =',R,MAX,MAX,N,N)
       CALL VECPRT('Sub determinants of R','CC',CC,MAX,N)
      ENDIF

      RETURN
      END

      SUBROUTINE DPSDCHK(A,N,NA,IW,NI,DW,ND,DETA)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C   purpose                                                    c
C   To compute the determinants of the principal submatrices   c
C   of a symmetric matrix A.                                   c
C                                                              c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER N,IW(NI),NA                         
      DOUBLE PRECISION A(NA,N),DET,DW(ND),DETA(N)      

      DETA(1) = A(1,1)
      DO 10 II=2,N
         CALL DSDETI(A,II,NA,IW,NI,DW,ND,DET,010)
         DETA(II) = DET
10    CONTINUE

      RETURN
      END


      SUBROUTINE FCNCST(CC,X,NM)
C    Compute value of constraints, (sub determinants of R)

      INTEGER NM
      DOUBLE PRECISION CC(NM),X(NM)

      PARAMETER (MAX = 20)
      DOUBLE PRECISION AA(MAX,MAX),B(MAX,MAX),D(MAX,MAX),P(MAX,MAX)
      DOUBLE PRECISION Q(MAX,MAX),R(MAX,MAX),F(MAX,MAX)

      INTEGER NP,NI,NW
      PARAMETER (NP = MAX*(MAX+1)/2, NI=NP+3, NW=NP+MAX)
      INTEGER IW(NI)
      DOUBLE PRECISION DW(NW),DET
      
      COMMON /DESMAT/ AA,B,D,P,Q,R,F
      COMMON /INT/ N,M,NY
      INTEGER NC,ND,IGRAD
      COMMON /IDB/ NC,ND,IGRAD
      ND = ND + 1

      CC(1) = R(1,1)
      DO 10 II=2,N
         CALL DSDETI(R,II,MAX,IW,NI,DW,NW,DET,010)
         CC(II) = DET
10    CONTINUE

      RETURN
      END

      SUBROUTINE DPTOD(DMU,Q,P,R,G,D,N,M,NY,MAX,ER,GD,WQ,W1,W2,IERR)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C***BEGIN PROLOGUE DPTOD                                               c
C   ON INPUT                                                           c
C                                                                      c
C***END PROLOGUE DPTOD                                                 c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      INTEGER N,M,NY,MAX
      DOUBLE PRECISION Q(MAX,MAX),P(MAX,MAX),R(MAX,MAX),G(MAX,MAX)
      DOUBLE PRECISION D(MAX,MAX),ER(N),WQ(MAX,MAX),W1(N),W2(N),DMU
      DOUBLE PRECISION LQ,LR

      INTEGER IOUT,IPRT
      common /iutil/ iout,iprt

       IERR = 0
C.. Robust criterion Patel and Toda
       
C... Set up Q = Q + tr(G)PG
       CALL MATMULT(G,D,GD,M,NY,N,MAX)
       CALL MATBA1(GD,P,WQ,M,N,MAX,MAX,W1)
       IF (IPRT.EQ.1) THEN
        CALL MATPRT('GD =',GD,MAX,MAX,M,N)
        CALL MATPRT('tr(G)inv(P)G =',WQ,MAX,MAX,N,N)
       ENDIF
       DO 10 I=1,N
          DO 20 J=1,N
             WQ(I,J) = Q(I,J) + WQ(I,J)
20        CONTINUE
10     CONTINUE
       IF (IPRT.EQ.1) THEN
        CALL MATPRT('QQ =',WQ,MAX,MAX,N,N)
       ENDIF

C... Compute eigenvalues of WQ
       CALL RS(MAX,N,WQ,ER,0,GD,W1,W2,IERR)
       IF (IERR.NE.0) WRITE(*,*)'ERROR ---> EIG(QQ) routine RS'
       IF (IPRT.EQ.1) THEN
        CALL VECPRT('ER(Q) =','ER',ER,N,N)
       ENDIF
       LQ = ER(1)

C... Compute eigenvalues of R
      CALL MATCOP(R,WQ,N,N,MAX,MAX)

      CALL RS(MAX,N,WQ,ER,0,GD,W1,W2,IERR)
      IF (IERR.NE.0) WRITE(*,*)'ERROR ---> eig(R) routine RS'
      LR = ER(N)
      IF (IPRT.EQ.1) THEN
       CALL VECPRT('ER(R) =','ER',ER,N,N)
      ENDIF

C... Compute robustness kriterion value
      IF (LR.NE.0.0) THEN
       DMU = 0.5*LQ/LR
      ENDIF

      RETURN
      END


      SUBROUTINE FXEIG(L,X,T,NN,NM)
C
C     Compute the eigenvalues L(1:NN) = f(X(1:NM))

      INTEGER NN,NM
      DOUBLE PRECISION L(*),X(*),T
C...
      INTEGER N,M,NY,MAX
      PARAMETER ( MAX = 20)
      DOUBLE PRECISION AA(MAX,MAX),B(MAX,MAX),D(MAX,MAX),P(MAX,MAX)
      DOUBLE PRECISION Q(MAX,MAX),R(MAX,MAX),F(MAX,MAX)
      COMMON /DESMAT/ AA,B,D,P,Q,R,F
      COMMON /INT/ N,M,NY
      INTEGER ITYPE(MAX)
      DOUBLE PRECISION G(MAX,MAX),GD(MAX,MAX),ABG(MAX,MAX)
      DOUBLE PRECISION ER(MAX),EI(MAX),WV(MAX),DEPS
C... Debug utilities
      INTEGER IOUT,IPRT
      COMMON /IUTIL/ IOUT,IPRT

      NN = N
C... Transform vector X to feedback matrix G
      CALL XG(G,F,X,NY,M,NM,2,MAX)

      IF (IPRT.EQ.1) THEN
       CALL MATPRT('ENTER  A  =',AA,MAX,MAX,N,N)
       CALL MATPRT('ENTER  B  =',B,MAX,MAX,N,M)
       CALL MATPRT('ENTER  G  =',G,MAX,MAX,M,NY)
       CALL MATPRT('ENTER  D  =',D,MAX,MAX,NY,N)
      ENDIF

C... Compute A + BGD
      CALL MATMULT(G,D,GD,M,NY,N,MAX)
      CALL CLABG(ABG,AA,B,GD,N,M,MAX,MAX)

      IF (IPRT.EQ.1) THEN
       CALL MATPRT('CALL DSCHUR  GD      =',GD,MAX,MAX,M,N)
       CALL MATPRT('CALL DSCHUR  A + BGD =',ABG,MAX,MAX,N,N)
      ENDIF

C... Compute eigenvalues
      CALL MACEPS(DEPS)
      CALL DSCHUR(ABG,GD,ER,EI,MAX,N,DEPS,WV,ITYPE,0)

      IF (IPRT.EQ.1) THEN
       CALL DEIGPRT('Exit DSCHUR EIG(A + BGD) =',ER,EI,N,6)
      ENDIF
      DO 10 I=1,N
         L(I) = ER(I)
10    CONTINUE

      RETURN
      END
