      SUBROUTINE MODOPTNEW(A,B,P,Q,R,G,nx,nu,na,nb)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Flytting av egenverdier i LQ-optimale systemet vha.              c
c     "upper-real" Schur dekomposisjon (RSD).                          c
c                                                                      c
c                                                                      c
c                                                                      c
c     INPUT                                                            c
c     A(n,n)  - systemmatrise                                          c
c     B(n,m)  - padragsmatrise                                         c
c     P(m,m)  - vektmatrise for padragene                              c
c     Q(n,n)  - vektmatrise for tilstandene                            c
c                                                                      c
c     nx,nu   - aktuelle dimensjoner : n = nx,  m = nu                 c
c                                                                      c
c     OUTPUT                                                           c
c     A(n,n)  - evtentuelt et nytt system                              c
c     B(n,m)  -                                                        c
c     P(m,m)  -                                                        c
c     Q(n,n)  -                                                        c
c     R(n,n)  - losning av Riccati-ligningen                           c
c     G(m,n)  - tilbakekoplings-matrise : G = -INV(P)*TSP(B)*R         c
c                                                                      c
c     nx,nu   - aktuelle dimensjoner : n = nx,  m = nu                 c
c                                                                      c
c     RUTINER SOM KALLES : DINI       DCALAN      DMEIG                c
c                          MATCOP     DCSCHF      DUMRQ                c
c                          RICSOL     DMENU       DRABPQ               c
c                          DMZER      MATPRT      DSYSOP               c
c                          DCALG      DTRACE      DPHLP1               c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  nx,nu,na,nb
      DOUBLE PRECISION         A(na,na),  B(na,nb)
      DOUBLE PRECISION         P(nb,nb),  Q(na,na)
      DOUBLE PRECISION         R(na,na),  G(nb,na)

c...  Lokale variabler.                                       ..........

      INTEGER                  n,m,nm,mm,nd,md
      PARAMETER              ( nm = 50, mm = 50, nd = 100, md = 100 )

      DOUBLE PRECISION         AN(nm,mm), H(nm,mm)
      DOUBLE PRECISION         PI(nm,mm), HS(nm,mm)
      DOUBLE PRECISION         Z(nd,md),  W(nd,md)
      DOUBLE PRECISION         QR(nm,mm), QS(nm,mm)
      DOUBLE PRECISION         RR(nm,mm), RS(nm,mm)
c     DOUBLE PRECISION         er(nd),    ei(nd)
      DOUBLE PRECISION         er(nm),    ei(nm)
      DOUBLE PRECISION         T(nm,mm),  U(nm,mm),  UT(nm,mm)
      DOUBLE PRECISION         wv1(nm),   wv2(nd)
      DOUBLE PRECISION         wm1(nm,mm),wm2(nm,mm),wm3(nm,mm)
      DOUBLE PRECISION         trace

c     INTEGER                  ind(nd), ix(nm), info, mc
      INTEGER                  ind(nm), ix(nm), info, mc
      LOGICAL                  qzer, mov

c...  Benyttes av MODOPT*, DSYSOP*, DCBRIC og DCALQS          ..........
      COMMON                   / CQ12 / askq
      INTEGER                  askq    

c...  Benyttes av MODOPT*, DSYSOP*, DRNEV og DCMIVA           ..........
      COMMON                   / MAXI / miv
      LOGICAL                  miv

c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER..
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps

      COMMON                   / MINF / iout, iprt
      INTEGER                  iout, iprt

      n = nx
      m = nu
                
      iout = 6
      iprt = 0
      askq = 1      
      miv  = .TRUE.

50    CALL DINI(A,AN,B,P,PI,Q,QR,QS,RR,RS,H,G,U,n,m,nm,mm,qzer,wv1,wm1)

      IF     ( .NOT. qzer ) THEN
        CALL   MATCOP(Q,R,n,n,nm,nm)
        CALL   DMZSEL(R,n,n,nm,nm)
        CALL   RICSOL(AN,H,R,Z,W,n,er,ei,wv2,ind,info,nm,nm,nm)
      ELSE
        CALL   DMZER(R,n,n,nm,nm)
        WRITE(*,*) ' Matrix Q is zero.'
      END IF

      CALL     DCALAN(AN,A,H,R,n,nm,wm1)

      CALL     MATCOP(AN,T,n,n,nm,nm)
c     CALL     DCSCHF(T,U,n,nm,mm,er,ei,ind,wv1,wm1,wm2)
      CALL     DSCHU2(T,U,er,ei,nm,n,eps,wv1,ind,0,wm1,wm2)
      write(*,*)'eps ',eps

100   CALL     DCALG(G,PI,B,R,n,m,nm,mm,wm1,wm2,wm3)

300   CALL     DMENU(mc)

      GOTO   ( 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19 )
     $ mc
          WRITE(*,*) 'Ukjent valg.'
          GOTO 20
1         CALL MATPRT('Matrise A : ',A,nm,nm,n,n)
          GOTO 20      
2         CALL MATPRT('Matrise B : ',B,nm,mm,n,m)
          GOTO 20      
3         CALL MATPRT('Matrise P : ',P,mm,mm,m,m)
          GOTO 20      
4         CALL MATPRT('Matrise Q : ',Q,nm,nm,n,n)
          GOTO 20      
5         CALL MATPRT('Matrise G : ',G,mm,nm,m,n)
          GOTO 20      
6         CALL MATPRT('Matrise R : ',R,nm,nm,n,n)
          CALL DTRACE(R,n,nm,trace)
          WRITE(*,*) 
          WRITE(*,65) '  Trace(R) --> ',trace
65        FORMAT(X,A,G16.6)
          GOTO 20      
7         CALL MATPRT('Matrise A + BG : ',AN,nm,nm,n,n)
          GOTO 20      

8         CALL DMEIG(RS,QS,T,U,UT,H,HS,n,n,nm,mm,er,ei,ind,ix,mov,wm1)
          CALL MATPRT('RS-> DMEIG =',RS,NM,NM,N,N)
          CALL MATPRT('QS-> DMEIG =',QS,NM,NM,N,N)
          CALL MATPRT('T -> DMEIG =',T,NM,NM,N,N)
          CALL MATPRT('U -> DMEIG =',U,NM,NM,N,N)
          CALL MATPRT('H -> DMEIG =',H,NM,NM,N,N)
          IF   ( .NOT.mov ) GOTO 100

          CALL DUMRQ(Q,QR,QS,U,n,n,nm,nm,wm1)        
          CALL DUMRQ(R,RR,RS,U,n,n,nm,nm,wm1)

          CALL DCALAN(AN,A,H,R,n,nm,wm1)

          CALL MATPRT('AN =',AN,NM,NM,N,N)

          CALL MATCOP(AN,T,n,n,nm,nm)
c         CALL DCSCHF(T,U,n,nm,mm,er,ei,ind,wv1,wm1,wm2)
          CALL DSCHU2(T,U,er,ei,nm,n,eps,wv1,ind,0,wm1,wm2)
          GOTO 8  

9         CALL DRABPQ(A,B,P,Q,n,m,nm,mm,wv1)
          GOTO 50

10        qzer = false
          CALL DSYSOP(Q,n,nm,qzer)
          IF   ( qzer ) GOTO 50
          GOTO 20 

11        CALL DPHLP1
          GOTO 20

12        GOTO 999

13        CALL MATPRT('Matrise RR : ',RR,nm,nm,n,n)
          GOTO 20      
14        CALL MATPRT('Matrise QR : ',QR,nm,nm,n,n)
          GOTO 20      
15        CALL DMZER(Q,n,n,nm,nm)
          GOTO 50      
16        CALL MATPRT('Matrise B*inv(P)*tsp(B) : ',H,nm,nm,n,n)
          GOTO 20      
17        CALL MATPRT('Matrise T : ',T,nm,nm,n,n)
          GOTO 20      
18        CALL MATPRT('Matrise U : ',U,nm,nm,n,n)
          GOTO 20      
19        CALL DPRTLEV
          GOTO 20
20    GOTO 300 

999   nx = n
      nu = m

      END
                                                                            


      SUBROUTINE DRABPQ(A,B,P,Q,n,m,nm,mm,wv1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Rutine som som tillater brukeren a endre pa systemets            c
c     dimensjoner og matriser. Gammelt system beholdes ved a           c
c     trykke <ENTER> gjennom hele rutinen.                             c
c                                                                      c
c     INPUT                                                            c
c     A(n,n)  - systemmatrise                                          c
c     B(n,m)  - padragsmatrise                                         c
c     P(m,m)  - vektmatrise for padragene                              c
c     Q(n,n)  - vektmatrise for tilstandene                            c
c     wm1(nm) - arbeidsomrade                                          c
c     n,m     - antall tilstander og padrag                            c
c     nm,mm   - max dimensjoner                                        c
c                                                                      c
c     OUTPUT                                                           c
c     A(n,n)  - evt. oppdaterte matriser                               c
c     B(n,m)  -                                                        c
c     P(m,m)  -                                                        c
c     Q(n,n)  -                                                        c
c     n,m     - evt. nye dimensjoner                                   c
c                                                                      c
c     RUTINER SOM KALLES : NREAD                                       c
c                          DREMA                                       c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm

      DOUBLE PRECISION         A(nm,mm),  B(nm,mm)
      DOUBLE PRECISION         P(nm,mm),  Q(nm,mm)
      DOUBLE PRECISION         wv1(nm)

c...  Gamle verdier skrives ut som deaffult-verdier.          ..........
c...  Ved a trykke <ENTER>, beholdes gamle verdier.           ..........

c...  Skriv ut og registrer evt. endring i antall tilstander og padrag.
      CALL     NREAD('Number of states      nx = ? ',n,1,1,IIII)
               WRITE (*,*)
      CALL     NREAD('Number of inputs      nu = ? ',m,1,1,IIII)
               WRITE (*,*)

c...  Skriv ut og registrer evt. endring i matrisene A,B,P,Q ..........
      CALL     DREMA(A,wv1,n,nm,n,nm,'A')
               WRITE (*,*)
      CALL     DREMA(B,wv1,n,nm,m,mm,'B')
               WRITE (*,*)
      CALL     DREMA(P,wv1,m,mm,m,mm,'P')
               WRITE (*,*)
      CALL     DREMA(Q,wv1,n,nm,n,nm,'Q')

      RETURN
      END



      SUBROUTINE DSYSOP(Q,n,nm,qzer)

      INTEGER                  n,nm
      DOUBLE PRECISION         Q(nm,nm)
      LOGICAL                  qzer

c...  Rutine som tilordner COMMON-blokkene CQ12 og MAXI       ..........

c...  Benyttes av MODOPT*, DSYSOP*, DCBRIC og DCALQS          ..........
      COMMON                   / CQ12 / askq
      INTEGER                  askq    

c...  Benyttes av MODOPT*, DSYSOP*, DRNEV og DCMIVA           ..........
      COMMON                   / MAXI / miv
      LOGICAL                  miv

      COMMON                   / MINF / iout, iprt
      INTEGER                  iout, iprt

      INTEGER                  ijob,ians,IYESANS
      INTEGER                  icom,ipos,iflag
      PARAMETER              ( icom=6 )
      CHARACTER                cjob*6, com(1:icom)*(6)

      DATA COM(1)/'MAX'/, COM(2)/'q12'/, COM(3)/'ZERQ'/
     +     COM(4)/'DEBU'/, COM(5)/'HELP'/, COM(6)/'EXIT'/

50    WRITE(*,*)
      WRITE(*,*) '----------                                   '
      WRITE(*,*) 'ROP MENU :                                   '
      WRITE(*,*) '---------------------------------------'
      WRITE(*,*) 'Command :  MAX q12 ZERQ DEBU HELP EXIT '
      WRITE(*,*) 'Numeric :   1   2   3    4    5    0   '
      WRITE(*,*) '---------------------------------------'

c...  Valg ved bare a trykke <ENTER> er flytting av egenverdier 
c...  automatisk valg............................................

      cjob = 'EXIT'

c.... Leser inn menyvalg.........................................

100   WRITE(*,*)
      CALL RCSTR('ROP command :',cjob,1,1,1)
      WRITE(*,*)

c...  Undersoker om innlest valg er fra 0 til icom...............

      ipos = ICNUM(cjob)
      IF ((ipos.GE.0).AND.(ipos.LE.(icom-1))) THEN
       ijob = ipos
       GO TO 200
      ENDIF

c...  Undersoker om innlest valg er alfanummerisk................

      CALL COMCHK(cjob,com,icom,iflag,ipos)
      IF (iflag.EQ.0) THEN
        ijob = ipos

c...  Skriv ut hvilke muligheter som finnes hvis feil er registrert.

      ELSEIF (iflag.EQ.1) THEN
       CALL COMLIS('Available is :',com,icom)
       cjob = 'EXIT'
       GO TO 100
      ENDIF

200   IF ((ijob.EQ.0).OR.(ijob.EQ.icom)) THEN
         GOTO 1000

      ELSE IF ( ijob .EQ. 1 ) THEN
c...     "miv" settes lik .TRUE. hvis brukeren onsker informasjon.......
c...     om maximal imaginaerdel under flytting av egenverdiene.........

         ians = IYESANS('Information of max imaginary value ? ',2)
         miv = ( ians .EQ. 2 )

      ELSE IF ( ijob .EQ. 2 ) THEN
c...     askq = 1 ---> MODOPT velger q12 midt i lovlig intervall .......
c...            2 ---> Bruker velger q12 i lovlig intervall    .........
c...            3 ---> MODOPT velger q12 som gir min(Tr(R22))  .........

300      askq = 1
         CALL NREAD('Registrate option on q12 (1-3) : ',askq,1,1,IIII)

         IF (( askq .LT. 1 ) .OR. ( askq .GT. 3 )) THEN
            WRITE(*,*)
            WRITE(*,*) ' Possible options are 1-3.'
            WRITE(*,*) ' 1 - Let MODOPT choose q12.'
            WRITE(*,*) ' 2 - Let  user specify q12.'
            WRITE(*,*) ' 3 - Let MODOPT optimize'
            WRITE(*,*) '     to find J(q12)=min(Tr(R))'
            WRITE(*,*)
            GOTO 300
         END IF
      ELSE IF ( ijob .EQ. 3 ) THEN
         qzer = .TRUE.
         CALL DMZER(Q,n,n,nm,nm)
         WRITE(*,*)
         WRITE(*,*) ' Matrix Q is set to zero'
      ELSE IF ( ijob .EQ. 4 ) THEN
         WRITE(*,*)
         IF ( iprt .EQ. 0 ) THEN
            iprt = 1
            WRITE(*,*) ' DEBUG switch is ON'
         ELSE
            iprt = 0
            WRITE(*,*) ' DEBUG switch is OFF'
         END IF
      ELSE IF ( ijob .EQ. 5 ) THEN
         CALL DPHLP2
      END IF

      GOTO 50

1000  RETURN
      END


      SUBROUTINE DPRTLEV

c...  Rutine som tilordner COMMON-blokken PRLE.               ..........
c...  Skriver ut ekstra informasjon under optimaliseringen    ..........
c...  J(QS(1,2) = Trace(R).                                   ..........
c...  Mengden av tilleggsinformasjonen oker med okende        ..........
c...  verdi pa variabelen "prtlev".                           ..........

c...  Benyttes av DPRTLEV*, DCBRIC, DCNVEG og FX              ..........
      COMMON                   / PRLE / prtlev      
      INTEGER                  prtlev  

c...  Godtar bare verdiene 0-6                                ..........
10    CALL NREAD('Registrate output level (0-6) : ',prtlev,1,1,IIII)
      IF (( prtlev .LT. 0 ) .OR. ( prtlev .GT. 6 )) GOTO 10

      RETURN
      END



      SUBROUTINE DINI(A,AN,B,P,PI,Q,QR,QS,RR,RS,H,G,U,n,m,nm,mm,
     +           ze,v1,m1)

c...  Initialiseringsrutine                                ............

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         A(nm,mm),  AN(nm,mm), H(nm,mm)
      DOUBLE PRECISION         Q(nm,mm),  QS(nm,mm), QR(nm,mm)
      DOUBLE PRECISION         RR(nm,mm), RS(nm,mm)
      DOUBLE PRECISION         B(nm,mm),  G(nm,mm),  U(nm,mm)
      DOUBLE PRECISION         P(nm,mm),  PI(nm,mm)
      DOUBLE PRECISION         v1(nm),    m1(nm,mm)
      LOGICAL                  ze

      DOUBLE PRECISION         det
      LOGICAL                  sy

c...  Undersoker om matrisen Q er symmetrisk               ............
      CALL     DTSYM(Q,n,nm,sy)
      IF     ( .NOT. sy ) THEN
        WRITE(*,*) ' Makes matrix Q symmetric.'
        CALL   MATCOP(Q,QR,n,n,nm,nm)
        CALL   DMSYM(Q,QR,n,nm)
      END IF

c...  Undersoker om alle elementene i matrisen Q er lik null  .........
c...  Returnerer i sa fall det logiske flagget ze = .true.    ........
      CALL     DTZER(Q,n,n,nm,nm,ze)

c...  Nullstiller matrisene                                ............
      CALL     DMZER(QS,n,n,nm,nm)
      CALL     DMZER(RR,n,n,nm,nm)
      CALL     DMZER(RS,n,n,nm,nm)
      CALL     DMZER(G,m,n,mm,nm)

c...  Finner den inverse av matrisen P.                    ............
c...  Forutsetter at P er invertibel.                      ............
      CALL     MATCOP(P,m1,m,m,mm,mm)
      CALL     INVERS(m1,PI,v1,det,m,mm)

c...  Utforer matriseproduktet H = B * INV(P) * TSP(B)     ............
      CALL     DMTMP(H,B,PI,n,m,nm,mm,m1)

c...  Kopierer matrisene Q og A over i QR og AN            ............
      CALL     MATCOP(Q,QR,n,n,nm,nm)
      CALL     MATCOP(A,AN,n,n,nm,nm)

c...  Hvis systemet har mindre enn tre tilstander, er      ............
c...  system-matrisen A alltid pa Schur-form. Matrisen U   ............
c...  med de ortogonale similaritetstransformasjonene      ............
c...  kan da settes lik identitetsmatrisen.                ............
      IF ( n .LE. 2 ) THEN
         CALL  DMZER(U,n,n,nm,mm)
         DO 10 i = 1,n
            U(i,i) = 1.0
10       CONTINUE     
      END IF

c...  Finner konvergenskriteriet eps for denne datamaskinen ...........
      CALL     DCEPS 

      RETURN
      END


      SUBROUTINE DCALAN(AN,A,H,R,n,nm,wm1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              12. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Utforer matriseoperasjonen :                                     c
c                                                                      c
c             AN = A - H * R                                           c
c                                                                      c
c     INPUT                                                            c
c     A(n,n)   - kvadratiske matriser med lik dimensjon.               c
c     H(n,n)   -                                                       c
c     R(n,n)   -                                                       c
c     wm1(n,n) - arbeidsomrade                                         c
c     n        - aktuell dimensjon                                     c
c     nm       - max dimensjon                                         c
c                                                                      c
c     OUTPUT                                                           c
c     AN(n,n)  - resultat-matrise.                                     c
c                                                                      c
c     RUTINER SOM KALLES : MATMULT                                     c
c                          DMSUM                                       c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         AN(nm,nm), A(nm,nm)
      DOUBLE PRECISION         H(nm,nm),  R(nm,nm)
      DOUBLE PRECISION         wm1(nm,nm)

c...  Utforer : wm1 = H * R                                ............
      CALL MATMULT(H,R,wm1,n,n,n,nm)

c...  Utforer : AN = A - wm1                               ............
      CALL DMSUM(AN,A,wm1,n,n,nm,nm,'-')

      RETURN
      END



      subroutine dschu2(a,u,er,ei,na,n,deps,dscl,itype,iord,wm1,wm2)
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C  author: David Di Ruscio                 26/9-1989             c
C  purpose                                                       c
C  calculate Upper Real Schur decomposition and order diagonal   c
C  blocks such that eigenspectrum is in descending order         c
C  iord.ne.1, or with stable eigenvalues appering first.         c
C                                                                c
C                          T         T                           c
C                     T = U A U , U U  = I                       c
C  on input                                                      c
C  a    double precision(na,n) general matrix to be reduced.     c
C  na   integer leading dimension of a and u in calling program. c
C  n    integer actual dimension of a.                           c
C  deps double precision scalar machine epsilon deps such that   c
C       deps+1 .ge. 1 (typically deps = 1.0e-16)                 c
C  dscl double precision(n) work vector used when balancing      c
C  iord integer parameter.                                       c
C       iord = 1  --> Eigenvalues and basis for eigenvector u    c
C                     is ordered in increasing order of the      c
C                     real eigenvalue parts, ie. stable eigen-   c
C                     values appears first.                      c
C       iord = 2  --> Eigenvalues and basis for eigenvector u    c
C                     is ordered in descending order of the      c
C                     real eigenvalue parts, ie. unstable eigen- c
C                     values appears first.                      c
C       iord.ne.1 --> Eigenvalues and basis for eigenvectors u   c
C                     is ordered in descending order of          c
C                     magintude, ie sorted after                 c
C                     abs( sqrt( er**2+ei**2) ).                 c
C  on output                                                     c
C  a    double precision(na,n) in upper real schur form.         c
C  u    double precision(na,n) orthogonal transformation         c
C       matrix.                                                  c
C er,ei double precision(n) real and imaginary part of eigen-    c
C       values respectively.                                     c
C itype integer(n) vector                                        c
C       an integer vector of length n whose                      c
C       i-th entry is                                            c
C       0  if the i-th eigenvalue is real,                       c
C       1  if the i-th eigenvalue is complex with                c
C          positive imaginary part,                              c
C       2  if the i-th eigenvalue is complex with                c
C          negative imaginary part,                              c
C       -1 if the i-th eigenvalue was not calculated             c
C          successfully.                                         c
C                                                                c
C   routines called: dbalanc, dorthes, dortran, dbalbak (eispack)c
C                    hqr3   (from Stewart's 1976 software)       c
C***end prologue dschur                                          c
Cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      integer na,n,itype(n),iord
      double precision a(na,n),u(na,n),er(n),ei(n),dscl(n)
      double precision wm1(na,na), wm2(na,na)
      double precision deps
      integer low,igh,low2,igh2

      COMMON                   / MINF / iout, iprt
      INTEGER                  iout, iprt

      IF ( iprt .EQ. 1 ) THEN
         WRITE(*,*) 'For Schur-Transform :'
         WRITE(*,*)
         CALL MATPRT('Matrise A + BG : ',a,na,na,n,n)
      END IF

      IF ( n .GE. 3 ) THEN
c1...    scale (balance) a with diagonal scaling
c        call dbalanc(na,n,a,low,igh,dscl)

c2...    reduce a to upper Hessenberg form and accumulate
c        transformation matrix in u

         low2 = 1
         igh2 = n
c        store information about transformation temporary in er(1:n)
         call dorthes(na,n,low2,igh2,a,er)
         call dortran(na,n,low2,igh2,a,er,u)

c3...    compute upper real Schur decomposition and ordering
c        eigenspectrum in descending order.
         call dhqr3(a,u,n,low2,igh2,deps,er,ei,itype,na,na,iord)

c4...    insert zeroes in lower part of T (a). not necessary, but
c        usful when T is to be presented.
         call dzerhes(a,na,n)

c5...    baktransform u with scaling dscl from dbalanc.
c        call dbalbak(na,n,low,igh,dscl,n,u)
      ELSE
c...     Finner egenverdier og egenverditype for           ...........
c...     systemer med dimensjon 1 eller 2.                 ............
         CALL DCEIGV(a,u,n,na,er,ei,itype,wm1,wm2)
      END IF

      IF ( iprt .EQ. 1 ) THEN
         WRITE(*,*)
         WRITE(*,*) 'Etter Schur-Transform :'
         CALL DCHEX(u,a,n,na)
         WRITE(*,*)
         WRITE(*,'(1X,A,6i3)')'DSCHUR IND = ',(itype(I),I=1,n)
      END IF
 

      return
      end


      SUBROUTINE               DCSCHF(T,U,n,nm,mm,er,ei,ind,wv1,wm1,wm2)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              12. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Finner Real Schur-form (RSD) for en generell kvadratisk          c
c     matrise. Returnerer ogsa matrisens egenverdier og                c
c     egenverdityper.                                                  c
c                                                                      c
c     INPUT                                                            c
c     T(n,n)   - en matrise med dimensjon n og vilkarlig form          c
c     wm1(n,n) - arbeidsomrader                                        c
c     wm2(n,n)                                                         c
c     n        - aktuell dimensjon                                     c
c     nm,mm    - max dimensjoner                                       c
c                                                                      c
c     OUTPUT                                                           c
c     Schur-formen :                                                   c
c     T(n,n)   - Ovre semitriangulaer form (RSD) med egenverdier i     c
c                blokkform paa hoveddiagonalen.                        c
c     U(n,n)   - Ortogonale similaritets-transformasjoner.             c
c     er(n)    - vektor med egenverdienes realverdi.                   c
c     ei(n)    - vektor med egenverdienes imaginaerverdi.              c
c     ind(n)   - vektor med egenverdienes type.                        c
c                0 : reell egenverdi.                                  c
c                1 : imaginaer egenverdi m/positiv imaginaerdel.       c
c                2 : imaginaer egenverdi m/negativ imaginaerdel.       c
c               -1 : finner ikke aktuell egenverdi i HQR3              c
c                                                                      c
c     RUTINER SOM KALLES : ORTHES     DMZLT                            c
c                          ORTRAN     DCEIGV                           c
c                          HQR3                                        c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm,mm,ind(nm)
      DOUBLE PRECISION         U(nm,mm), T(nm,mm)
      DOUBLE PRECISION         wm1(nm,mm), wm2(nm,mm)
      DOUBLE PRECISION         wv1(nm), er(nm), ei(nm)
 
c...  Konvergenskriterium som brukes i HQR3                ............
c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER..
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps
 
      COMMON                   / MINF / iout, iprt
      INTEGER                  iout, iprt

      INTEGER                  low,igh,low2,igh2

c...  Matriser med dimensjon 1 eller 2 er alltid pa Schur-form ........
      IF ( n.GE.3 ) THEN

         IF ( iprt .EQ. 1 ) THEN
            WRITE(*,*) 'For Schur-Transform :'
            WRITE(*,*)
            CALL MATPRT('Matrise A +BG : ',T,nm,nm,n,n)
         END IF

         CALL DMZSEL(T,n,n,nm,nm)
c        CALL DBALANC(nm,n,T,low,igh,wv1)

         low2 = 1
         igh2 = n
c...     Reduserer en generell matrise med ortogonale similaritets- ...
c...     transformasjoner til ovre Hessenberg-form         ............
         CALL DORTHES(nm,n,low2,igh2,T,er)
         CALL DORTRAN(nm,n,low2,igh2,T,er,U)

c...     Reduserer videre til real Schur-form (RSD)        ............
         CALL HQR3(T,U,n,low2,igh2,eps,er,ei,ind,nm,nm)

c...     Nullstiller alle elementer under 1. subdiagonal i T...........
c        CALL DMZLT(T,n,nm)

c        CALL BALBAK(nm,n,low,igh,wv1,n,U)

         IF ( iprt .EQ. 1 ) THEN
            WRITE(*,*)
            WRITE(*,*) 'Etter Schur-Transform :'
            CALL DCHEX(U,T,n,nm)
         END IF
      ELSE
c...     Finner egenverdier og egenverditype for en matrise ...........
c...     som allerede var pa Schur-form.                   ............
         CALL DCEIGV(T,U,n,nm,er,ei,ind,wm1,wm2)
      END IF
 
      RETURN
      END



      SUBROUTINE               DCEIGV(T,U,n,nm,er,ei,ind,wm1,wm2)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              12. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Finner egenverdier og egenverditype til matrisen T som er        c
c     pa Real Schur-form (RSD) og har den ortogonale similaritets-     c
c     transformasjonen U.                                              c
c                                                                      c
c     INPUT                                                            c
c     Schur-formen :                                                   c
c     T(n,n)   - Ovre semitriangulaer form (RSD) med egenverdier i     c
c                blokkform paa hoveddiagonalen.                        c
c     U(n,n)   - Ortogonale similaritets-transformasjoner.             c
c     wm1(n,n) - arbeidsomrader                                        c
c     wm2(n,n)                                                         c
c     n        - aktuell dimensjon                                     c
c     nm,mm    - max dimensjon                                         c
c                                                                      c
c     OUTPUT                                                           c
c     er(n)    - vektor med egenverdienes realverdi.                   c
c     ei(n)    - vektor med egenverdienes imaginaerverdi.              c
c     ind(n)   - vektor med egenverdienes type.                        c
c                0 : reell egenverdi.                                  c
c                1 : imaginaer egenverdi m/positiv imaginaerdel.       c
c                2 : imaginaer egenverdi m/negativ imaginaerdel.       c
c                                                                      c
c     RUTINER SOM KALLES : MATCOP                                      c
c                          EIGVAL                                      c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm           
      DOUBLE PRECISION         T(nm,nm),  U(nm,nm)
      DOUBLE PRECISION         wm1(nm,nm),wm2(nm,nm)
      DOUBLE PRECISION         er(nm),    ei(nm)
      INTEGER                  ind(nm)

      INTEGER                  i

c...  Kopierer matrisene T og U over i wm1 og wm2          ............
c...  for a unnga at de endres pa av rutinen EIGVAL.       ............
      CALL     MATCOP(T,wm1,n,n,nm,nm)
      CALL     MATCOP(U,wm2,n,n,nm,nm)

c...  Tilordner vektorene er(n) og ei(n).                  ............
      CALL     EIGVAL(wm1,wm2,n,1,n,er,ei,nm,nm)

c...  Tilorner vektoren ind(n) ut fra er(n) og ei(n)       ............
      DO 10 i = 1,n
         IF ( ei(i) .EQ. 0.0 ) THEN
            ind(i) = 0
         ELSE IF ( ei(i) .GT. 0.0 ) THEN
            ind(i) = 1
         ELSE
            ind(i) = 2
         END IF
10    CONTINUE

      RETURN
      END

      SUBROUTINE DMEIG(RS,QS,T,U,UT,H,HS,n,m,nm,mm,er,ei,ind,ix,mov,wm1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Lenker sammen to rutiner som i hovedtrekk :                      c
c                                                                      c
c        - utforer flytting av egenverdier og finner alle mulige       c
c          blokker QS22 i matrisen QS.                                 c
c                                                                      c
c        - finner blokken QS22 av de mulige som minimaliserer          c
c          Trace(RS22) under forutsetning av at matrisen RS22 blir     c
c          positiv definit.                                            c
c                                                                      c
c     INPUT                                                            c
c     RS(n,n) - matriseproduktet : RS = TSP(U)*R*U                     c
c               der matrisen R er losningen av Riccati-ligningen.      c
c     QS(n,n) - matriseproduktet : QS = TSP(U)*Q*U,                    c
c               der matrisen Q er tilstandenes vektmatrise.            c
c     T(n,n)  - Ovre semitriangulaer form med egenverdier i            c
c               blokkform paa hoveddiagonalen.                         c
c     U(n,n)  - Ortogonale similaritets-transformasjoner.              c
c     H(n,n)  - matriseproduktene : H  = B*INV(P)*TSP(B)               c
c                                                                      c
c     n,m     - aktuelle dimensjoner                                   c
c     nm,mm   - max dimensjoner                                        c
c                                                                      c
c     er(n)   - vektor med egenverdienes realdel                       c
c     ei(n)   - vektor med egenverdienes imaginaerdel                  c
c     ind(n)  - vektor som viser egenverdiens type                     c
c               0 : reell egenverdi.                                   c
c               1 : imaginaer egenverdi m/positiv imaginaerdel.        c
c               2 : imaginaer egenverdi m/negativ imaginaerdel.        c
c              -1 : har ikke funnet egenverdi                          c
c                                                                      c
c     ix(n)    - arbeidsomrader                                        c
c     wm1(n,n) -                                                       c
c                                                                      c
c     OUTPUT                                                           c
c     RS(n,n) - med oppdatert blokk RS22.                              c
c     QS(n,n) - med oppdatert blokk QS22.                              c
c     T(n,n)  - omformet                                               c
c     U(n,n)  - omformet                                               c
c     UT(n,n) - = TSP(U).                                              c
c     HS(n,n) - matriseproduktet  : HS = TSP(U)*H*U                    c
c     er(n)   - oppdatert                                              c
c     ei(n)   -                                                        c
c     ind(n)  -                                                        c
c     mov     - flagg som viser om noen egenverdier er flyttet         c
c               .TRUE.  - egenverdi(er) flyttet                        c
c               .FALSE. - ingen flytting                               c
c                                                                      c
c     RUTINER SOM KALLES : DRMOV                                       c
c                          DCBRIC                                      c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm

      DOUBLE PRECISION         RS(nm,mm), HS(nm,mm), QS(nm,mm), H(nm,mm)
      DOUBLE PRECISION         T(nm,mm),  U(nm,mm),  UT(nm,mm)
      DOUBLE PRECISION         er(nm),    ei(nm),    wm1(nm,mm)

      INTEGER                  ind(nm), ix(nm)
      LOGICAL                  mov

c...  Kaller rutine som utforer flytting av egenverdier    ............
c...  og finner alle mulige matriser QS.                   ............
      CALL DRMOV(QS,T,U,UT,H,HS,n,n,nm,mm,er,ei,ind,ix,mov,wm1)

      CALL MATPRT('QS ->DRMOV',QS,NM,NM,N,N)
      CALL MATPRT('HS ->DRMOV',HS,NM,NM,N,N)
      CALL MATPRT('T  ->DRMOV',T ,NM,NM,N,N)
      CALL MATPRT('U  ->DRMOV',U ,NM,NM,N,N)
      CALL MATPRT('UT ->DRMOV',UT,NM,NM,N,N)

c...  Returnerer hvis ikke flytting er utfort.             ............
      IF     ( .NOT.mov ) GOTO 999

c...  Finner matrisen QS av de mulige som forer til minste ............
c...  Trace(RS) under forutsetning av at matrisen RS er    ............
c...  positiv definit.                                     ............
      CALL MATPRT('RS ->CALL DCBRIC',RS,NM,NM,N,N)
      CALL DCBRIC(T,HS,RS,QS,n,m,nm,mm)
      CALL MATPRT('T  ->DCBRIC',T ,NM,NM,N,N)
      CALL MATPRT('HS ->DCBRIC',HS,NM,NM,N,N)
      CALL MATPRT('RS ->DCBRIC',RS,NM,NM,N,N)
      CALL MATPRT('QS ->DCBRIC',QS,NM,NM,N,N)


999   RETURN
      END



      SUBROUTINE DCBRIC(T,HS,RS,QS,n,m,nm,mm)
c...                                                       ............

      INTEGER                  n,m,nm,mm

      DOUBLE PRECISION         T(nm,mm),  HS(nm,mm)
      DOUBLE PRECISION         RS(nm,mm), QS(nm,mm)

c...  Benyttes av DCBRIC og DRNEV*                            ..........
      COMMON                   / MOVE / mole
      LOGICAL                  mole

c...  Benyttes av MODOPT*, DSYSOP*, DCBRIC og DCALQS          ..........
      COMMON                   / CQ12 / askq
      INTEGER                  askq    

c...  Benyttes av DCBRIC* og DCNQ2M                           ..........
      COMMON                   / SSET / sgn
      CHARACTER                sgn

c...  Benyttes av DPRTLEV*, DCBRIC, DCNVEG og FX              ..........
      COMMON                   / PRLE / prtlev
      INTEGER                  prtlev  

c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER..
      COMMON                   / KCRIT / epsilon
      DOUBLE PRECISION         epsilon

c...  Benyttes av DCBRIC* og NLPQ0
      COMMON                   /CMACHE/ eps
      DOUBLE PRECISION         eps

c...  Benyttes av DCBRIC*, DGGRAD og DFGRAD                   ..........
      COMMON                   / DSTEP / HH, stp
      DOUBLE PRECISION         HH,stp

c...  Benyttes av DCBRIC, DCNQ2M, DRMOV*, DCALQS*             ..........
      COMMON                   / QVEK / QV
      DOUBLE PRECISION         QV(1:11)

c...  Benyttes av DCBRIC*,DCONST* og FX*                      ..........
      COMMON                   / BLK2 / T2, H2, Q2, R2
      DOUBLE PRECISION         T2(2,2), H2(2,2), Q2(2,2), R2(2,2)

c...  Benyttes av DCBRIC og DRMOV*                            ..........
      LOGICAL                  onereal
      COMMON                   / SCRI / onereal
                             
      DOUBLE PRECISION         Z(4,4) , W(4,4)     
      DOUBLE PRECISION         er2(4), ei2(4),   wv2(4)
      INTEGER                  ind2(4), info   

      INTEGER                  mc,mec,kwa(100),iprint,ifail,maxit,maxfun
      DOUBLE PRECISION         wa(1000)
      DOUBLE PRECISION         ll, ul, q12, tra, ntra
      DOUBLE PRECISION         qval,qmin,newtra,oldtra
      DOUBLE PRECISION         con(2)
      LOGICAL                  act(2)
                                      
      INTEGER                  dim

      IF ( n .GE. 2 ) THEN
         CALL DGMBFD(T,T2,n,nm,2,n-1)
         CALL DGMBFD(HS,H2,n,nm,2,n-1)
      END IF

      IF ((askq.EQ.3).AND.(mole).AND.((QV(1).EQ.1.0).OR.(QV(1).EQ.2.0))
     +    .AND.(ABS(QV(3)-QV(2)).GT.1.0)) THEN
 
         WRITE(*,*)'LOOP 1'

         stp    = 0.1
 
         mc     = 2
         mec    = 0
         dim    = 1
         ll     = QV(2)+stp
         ul     = QV(3)-stp
         q12    = (ll + ul)/2.0D0
         iprint = 0
         ifail  = 0
         maxit  = 100
         maxfun = 10
 
         eps = epsilon
         HH  = 1.0D-5

         IF ( prtlev.GE.2 ) iprint = prtlev - 2

         sgn = '+'
         CALL FX(tra,(ll+stp),dim)
         IF ( prtlev .GE. 1 ) THEN
            WRITE(*,*)
            WRITE(*,*) 'q12 min : ',(ll+stp)
            CALL MATPRT('R2 min :',R2,nm,nm,2,2)
         END IF

         CALL FX(ntra,(ll+9.0*stp),dim)
         IF ( ntra .GT. tra ) sgn = '-'

         IF ( prtlev .GE. 1 ) THEN
            CALL FX(tra,q12,dim)
            WRITE(*,*)
            WRITE(*,*) 'q12 mid : ',q12
            CALL MATPRT('R2 min :',R2,nm,nm,2,2)

            CALL FX(tra,ul,dim)
            WRITE(*,*)
            WRITE(*,*) 'q12 max : ',ul
            CALL MATPRT('R2 min :',R2,nm,nm,2,2)

            WRITE(*,*)
            WRITE(*,39) 
39          FORMAT('Registrer startverdi for q12 : ',$)
            READ(*,*) q12
         END IF

         tra = 0.0

c        IF ( prtlev .GE. 1 ) THEN
c           WRITE(*,*)
c           WRITE(*,*) 'For NLPQL0'
c           WRITE(*,*)
c           WRITE(*,*) 'M       : ',mc
cc          WRITE(*,*) 'ME      : ',mec
cc          WRITE(*,*) 'N       : ',dim
c           WRITE(*,*) 'X(1)    : ',q12 
c           WRITE(*,*) 'F       : ',tra
c           WRITE(*,*) 'XL(1)   : ',ll   
c           WRITE(*,*) 'XU(1)   : ',ul   
c           WRITE(*,*) 'IPRINT  : ',iprint
c           WRITE(*,*) 'IFAIL   : ',ifail
c           WRITE(*,*) 'MAXIT   : ',maxit
c           WRITE(*,*) 'MAXFUN  : ',maxfun
c           WRITE(*,*)
c           WRITE(*,*) 'eps     : ',eps
c           WRITE(*,*) 'H       : ',HH
c           WRITE(*,*)
c        END IF
c
c... Call NLPQ0 to solve the problem                    ..........
 
CUT      CALL NLPQL0(mc,mec,dim,q12,tra,ll,ul,iprint,ifail,
CUT  +               wa,1000,kwa,100,maxit,maxfun)

c        IF ( prtlev .GE. 1 ) THEN
c           WRITE(*,*)
c           WRITE(*,*)
c           WRITE(*,*) 'Etter NLPQL0'
c           WRITE(*,*)
c           WRITE(*,*) 'M       : ',mc
c           WRITE(*,*) 'ME      : ',mec
c           WRITE(*,*) 'N       : ',dim
c           WRITE(*,*) 'X(1)    : ',q12 
c           WRITE(*,*) 'F       : ',tra
c           WRITE(*,*) 'XL(1)   : ',ll
c           WRITE(*,*) 'XU(1)   : ',ul
c           WRITE(*,*) 'IPRINT  : ',iprint
c           WRITE(*,*) 'IFAIL   : ',ifail
c           WRITE(*,*) 'MAXIT   : ',maxit
cc          WRITE(*,*) 'MAXFUN  : ',maxfun
c        END IF


         CALL FX(tra,q12,dim)

         IF ( prtlev .GE. 1 ) THEN
            WRITE(*,*)
            WRITE(*,*) 'ifail : ',ifail
            WRITE(*,*)
            WRITE(*,*) 'Result of NLPQL0 :'
            WRITE(*,*) '------------------'
            WRITE(*,*) 'q12 and trace(R) : ',q12,tra
            CALL MATPRT('R2 min :',R2,nm,nm,2,2)
            CALL MATPRT('Q2 min :',Q2,nm,nm,2,2)

            stp = ( ul - ll )/100
            oldtra = 1.0e+15
            qval = ll
            act(1) = .TRUE.
            act(2) = .TRUE.
            DO 77 i = 1,98
               qval = qval + stp
               CALL FX(newtra,qval,1)
               IF (( newtra .LT. oldtra ).AND.(newtra.GE.0.0)) THEN
                  CALL DCNVEG(R2,2,nm,con,act,2,2,2)
c                 CALL DCRPD(R2,2,nm,sema,semb)
                  IF ((con(1).GE.0.0).AND.(con(2).GE.0.0)) THEN
                     oldtra = newtra
                     qmin = qval
                  END IF
               END IF
77          CONTINUE

            WRITE(*,*)
            WRITE(*,*)
            IF ( oldtra .LT. 1.0e+14 ) THEN
               WRITE(*,*) 'Result of the interval search :'
               WRITE(*,*) '-------------------------------'
               WRITE(*,*) 'q12 and trace(R) : ',qmin,oldtra
               CALL FX(tra,qmin,dim)
               CALL MATPRT('R2 min :',R2,nm,nm,2,2)
               CALL MATPRT('Q2 min :',Q2,nm,nm,2,2)
            ELSE
               WRITE(*,*) 'No result of the interval search '
            END IF

            CALL FX(tra,q12,dim)
         END IF

         IF ( ifail .NE. 0 ) THEN
            act(1) = .TRUE.
            act(2) = .TRUE.
            CALL DCNVEG(R2,2,nm,con,act,2,2,2)
c           CALL DCRPD(R2,2,nm,sema,semb)
            IF ((con(1).LT.0.0).OR.(con(2).LT.0.0)) THEN
               WRITE(*,*)
               WRITE(*,*)' ---> Warning : Can''t find ',
     +                         'the matrix : R22 > 0'
               q12 = (ll + ul)/2.0
               CALL FX(tra,q12,dim)
            END IF
         END IF

         CALL DSMBTD(QS,Q2,n,nm,2,n-1)
 
      ELSE

      WRITE(*,*)'LOOP 2'
      WRITE(*,*)'n =',n
      WRITE(*,*)'onereal =',onereal


c...     Bestemt q12

         IF ( n .GE. 2 ) THEN
          WRITE(*,*)'ENTER SUBLOOP'

cut         CALL MATPRT('QS -> DCBRIC - call DGMBFD',QS,NM,MM,n,n)
            CALL DGMBFD(QS,Q2,n,nm,2,n-1)
cut         CALL MATPRT('Q2 -> DCBRIC - DGMBFD',Q2,2,2,2,2)
cut         WRITE(*,*)'Q2 ',Q2
cut         CALL MATPRT('QS -> DCBRIC - DGMBFD',QS,NM,MM,n,n)

            IF ( onereal ) THEN
             CALL DCSRIC(T2,H2,Q2,R2,2,nm)
cut          CALL MATPRT('R2 -> DCBRIC - DCSRIC',R2,2,2,2,2)
            ELSE
             CALL MATCOP(Q2,R2,2,2,nm,nm)
             CALL DMZSEL(R2,2,2,nm,nm)
             CALL RICSOL(T2,H2,R2,Z,W,2,er2,ei2,wv2,ind2,info,nm,nm,nm)

             CALL MATPRT('R2 -> DCBRIC - RICSOL',R2,2,2,2,2)

c            WRITE(*,*)
c            CALL MATPRT('Matrise T2: ',T2,nm,nm,2,2)
c            WRITE(*,*)
c            CALL MATPRT('Matrise H2: ',H2,nm,nm,2,2)
c            WRITE(*,*)
c            CALL MATPRT('Matrise Q2: ',Q2,nm,nm,2,2)
c            WRITE(*,*)
c            CALL MATPRT('Matrise R2: ',R2,nm,nm,2,2)
c            WRITE(*,*)
c            WRITE(*,*) 'info : ',info
            END IF
         ELSE
            CALL DCSRIC(T,HS,QS,RS,1,nm)
         END IF

      END IF

      IF ( n .GE. 2 ) CALL DSMBTD(RS,R2,n,nm,2,n-1)

      RETURN
      END




c     SUBROUTINE               DCBRIC(T,HS,RS,QS,n,m,nm,mm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner losningen RS22 av Riccati-ligningen som funksjon        c
c     av blokkene T22, HS22 og QS22 som trekkes ut av T,HS og QS.      c
c                                                                      c
c     Hvis det eksisterer et lovlig intervall for QS(n-1,n),           c
c     finner rutinen den blokken QS22 som minimaliserer                c
c     funksjonen :                                                     c
c                  J(QS(n-1,n)) = Trace(RS22)                          c
c                                                                      c
c                  med bibet.: RS22 >= 0                               c
c                                                                      c
c     Hvis ikke et minimum finnes, settes QS(n-1,n) til en verdi       c
c     midt i det lovlige intervallet uten hensyn til bibetingelsen.    c
c                                                                      c
c     Rutinen returnerer matrisene RS og QS med oppdaterte             c
c     blokker RS22 og QS22.                                            c
c                                                                      c
c     INPUT                                                            c
c     T(n,n)  - Ovre semitriangulaer form med egenverdier i            c
c               blokkform paa hoveddiagonalen.                         c
c     HS(n,n) - matriseproduktet : HS = TSP(U)*H*U                     c
c     RS(n,n) - matriseproduktet : RS = TSP(U)*R*U                     c
c               der matrisen R er losningen av Riccati-ligningen.      c
c     QS(n,n) - matriseproduktet : QS = TSP(U)*Q*U,                    c
c               der matrisen Q er tilstandenes vektmatrise.            c
c                                                                      c
c     n,m     - aktuelle dimensjoner                                   c
c     nm,mm   - max dimensjoner                                        c
c                                                                      c
c     OUTPUT                                                           c
c     RS(n,n) - med oppdatert blokk RS22.                              c
c     QS(n,n) - med oppdatert blokk QS22.                              c
c                                                                      c
c     RUTINER SOM KALLES : DGMBFD       DCNVEG        MATCOP           c
c                          FX           DSMBTD        RICSOL           c
c                          NLPQL0       DCSRIC                         c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     INTEGER                  n,m,nm,mm
c
c     DOUBLE PRECISION         T(nm,mm),  HS(nm,mm)
c     DOUBLE PRECISION         RS(nm,mm), QS(nm,mm)
c
c...  Benyttes av DCBRIC og DRNEV*                            ..........
c...  Logisk variabel som er satt hvis polene er flyttet mot venstre....
c     COMMON                   / MOVE / mole
c     LOGICAL                  mole
c
c...  Benyttes av DCBRIC* og DCNQ2M                           ..........
c...  Fortegn som bestemmer hvilken losning som skal brukes av to mulige
c     COMMON                   / SSET / sgn
c     CHARACTER                sgn
c
c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER..
c...  Konvergenskriterium i MODOPT.                           ..........
c     COMMON                   / KCRIT / epsilon
c     DOUBLE PRECISION         epsilon
c
c...  Benyttes av DCBRIC* og NLPQ0
c...  Konvergenskriterium i NLPQL0.                           ..........
c     COMMON                   /CMACHE/ eps
c     DOUBLE PRECISION         eps
c
c...  Benyttes av DCBRIC*, DGGRAD og DFGRAD                   ..........
c     COMMON                   / DSTEP / HH, stp
c     DOUBLE PRECISION         HH, stp
c
c...  Benyttes av DCBRIC, DCNQ2M, DRMOV*, DCALQS*             ..........
c...  Vektor som bestemmer QS(n-1,n-1) og QS(n,n) som         ..........
c...  funksjon av QS(n-1,n).                                  ..........
c     COMMON                   / QVEK / QV
c     DOUBLE PRECISION         QV(1:11)
c
c...  Benyttes av DCBRIC*,DCONST* og FX*                      ..........
c...  Lokale matriser av dimensjon 2.                         ..........
c     COMMON                   / BLK2 / T2, H2, Q2, R2
c     DOUBLE PRECISION         T2(2,2), H2(2,2), Q2(2,2), R2(2,2)
c
c...  Benyttes av DCBRIC og DRMOV*                            ..........
c...  Logisk variabel som er satt hvis en reell egenverdi     ..........
c...  skal flyttes og skalar losning av Riccati ma brukes.    ..........
c     COMMON                   / SCRI / onereal
c     LOGICAL                  onereal
c                            
c...  Lokale parametre                                        ..........
c     DOUBLE PRECISION         Z(4,4) , W(4,4)     
c     DOUBLE PRECISION         er2(4), ei2(4),   wv2(4)
c     INTEGER                  ind2(4), info, dim
c
c     INTEGER                  mc,mec,kwa(100),iprint,ifail,maxit,maxfun
c     DOUBLE PRECISION         wa(1000)
c     DOUBLE PRECISION         ll, ul, q12, tra, ntra, con(2)
c     LOGICAL                  act(2)
c
c...  Trekker ut blokkene T22 og HS22 hvis systemet har       ..........
c...  dimensjon to eller storre.                              ..........
c     IF ( n .GE. 2 ) THEN
c        CALL DGMBFD(T,T2,n,nm,2,n-1)
c        CALL DGMBFD(HS,H2,n,nm,2,n-1)
c     END IF
c
c...  Hvis egenverdiene er flyttet til venstre og QS(n-1,n)   ..........
c...  kan variere over et intervall som er storre enn 1       ..........
c...  i absoluttverdi.                                        ..........
c     IF ((mole).AND.((QV(1).EQ.1.0).OR.(QV(1).EQ.2.0)).AND.
c    +    (ABS(QV(3)-QV(2)).GT.1.0)) THEN
c
c        stp    = 0.1
c
c...     Parametre for rutinen NLPQL0.                        ..........
c        mc     = 2
c        mec    = 0
c        dim    = 1
c        ll     = QV(2)+stp
c        ul     = QV(3)-stp
c        q12    = (ll + ul)/2.0
c        iprint = 0
c        ifail  = 0
c        maxit  = 100
c        maxfun = 10
c
c        eps = epsilon
c        HH  = 1.0D-5
c
c...     Hvis Trace(RS22) har to losninger (ellipse).         ..........
c        IF ( QV(1) .EQ. 1.0 ) THEN
c           sgn = '+'
c           CALL FX(tra,(ll+stp),dim)
c           CALL FX(ntra,(ll+9.0*stp),dim)
c
c...        Bruker losning som gir minimum av Trace(RS22)     ..........
c           IF ( ntra .GT. tra ) sgn = '-'
c        END IF 
c
c        tra = 0.0
c
c...     Kaller rutine som finner minimum av den ulineaere    ..........
c...     funksjonen J(q12) = Trace(RS22) med de ulineaere     ..........
c...     bibetingelsene RS22 > 0.                             ..........
c
c        CALL NLPQL0(mc,mec,dim,q12,tra,ll,ul,iprint,ifail,
c    +               wa,1000,kwa,100,maxit,maxfun)
c
c...     Finner endelig RS22 og funksjonsverdien Trace(RS22)  ..........
c        CALL FX(tra,q12,dim)
c
c...     Hvis feil har inntruffet i NLPQL0                    ..........
c        IF ( ifail .NE. 0 ) THEN
c           act(1) = .TRUE.
c           act(2) = .TRUE.
c
c...        Beregner bibetingelsene for aktuell RS22          ..........
c           CALL DCNVEG(R2,2,nm,con,act,2,2,2)
c
c...        Skriver feilmelding hvis ikke begge bibetingelsene .........
c...        er oppfyllt.                                      ..........
c           IF ((con(1).LT.0.0).OR.(con(2).LT.0.0)) THEN
c              WRITE(*,*)
c              WRITE(*,*)' ---> Warning : Can''t find ',
c    +                         'the matrix : R22 > 0'
c...           Bruker q12 som ligger midt i intervallet.      ..........
c              q12 = (ll + ul)/2.0
c              CALL FX(tra,q12,dim)
c           END IF
c        END IF
c
c...     Setter losningen QS22 inn i matrisen QS.             ..........
c        CALL DSMBTD(QS,Q2,n,nm,2,n-1)
c
c     ELSE
c
c...     Hvis systemets dimensjon er storre enn 1             ..........
c        IF ( n .GE. 2 ) THEN
c...        Trekker ut blokken QS22 fra QS.                   ..........
c           CALL DGMBFD(QS,Q2,n,nm,2,n-1)
c
c           IF ( onereal ) THEN
c...           Beregner skalar losning av Riccati-ligningen.  ..........
c              CALL DCSRIC(T2,H2,Q2,R2,2,nm)
c           ELSE
c              CALL MATCOP(Q2,R2,2,2,nm,nm)
c              CALL DMZSEL(R2,2,2,nm,nm)
c...           Beregner losningen RS22 for 2. ordens system   ..........
c            CALL RICSOL(T2,H2,R2,Z,W,2,er2,ei2,wv2,ind2,info,nm,nm,nm)
c           END IF
c        ELSE
c...        Beregner skalar losning for skalart system.       ..........
c           CALL DCSRIC(T,HS,QS,RS,1,nm)
c        END IF
c
c     END IF
c
c...  Setter losningen RS22 inn i matrisen RS, hvis systemets ..........
c...  dimensjon er to eller storre.                           ..........
c     IF ( n .GE. 2 ) CALL DSMBTD(RS,R2,n,nm,2,n-1)
c
c     RETURN
c     END
c



c
c
c     SUBROUTINE               DCNVEG(R2,n,nm,G,ACTIVE,M,ME,MMAX)
c     SUBROUTINE               DCRPD(R2,n,nm,sema,semb)
c
c     INTEGER                  n,nm
c     DOUBLE PRECISION         R2(nm,nm),sema,semb
c
c     INTEGER                  x
c
c     x = n-1
c
c     sema = R2(x,x)
c     semb = ((R2(x,x)*R2(n,n))-(R2(x,n)*R2(x,n)))
c
c     RETURN
c     END
c
c

      SUBROUTINE NLFUNC(M,ME,MMAX,N,F,G,X,ACTIVE)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    PURPOSE                                                       C
C    To evaluate function and constraint values.                   C
C    This routine is called from NLPQ0 or NLPQ1.                   C
C    Routines called: FX and DCONST                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER                  M,ME,MMAX,N
      DOUBLE PRECISION         F,G(MMAX),X(N)
      LOGICAL                  ACTIVE(MMAX)

c... Compute functional values F(X)                       .........
      CALL FX(F,X,N)

c... Compute constraint values G(X) if constraints exists ........
      IF ( M.GT.0 ) CALL DCONST(G,X,N,ACTIVE,M,ME,MMAX)
 
      RETURN
      END


      SUBROUTINE               NLGRAD(M,ME,MMAX,N,F,G,DF,DG,X,ACTIVE,WA)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                            C
C   To evaluate the gradient vectors DF = df/dx and DG = dg/dx         C
C   numericaly. This routine is called from NLPQL0 or NLPQL1.          C
C   ROUTINES CALLED : DFGRAD and DGGRAD                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER                  M,ME,MMAX,N
      DOUBLE PRECISION         F,G(MMAX),DF(N),DG(MMAX,N),X(N),WA(MMAX)
      LOGICAL                  ACTIVE(MMAX)

c...Compute gradient vector DF =df/dx forward difference     ........
c...F(X) at X is known input --> DF = (F(X+Hi) - F(X))/Hi

      CALL DFGRAD(X,DF,F,N,N)

c...Compute gradient vector DG =dg/dx forward difference     ........
c...G(X) at X is known input --> DG = (G(X+Hi) - G(X))/Hi

      CALL DGGRAD(X,DG,G,WA,ACTIVE,N,M,ME,MMAX)

      RETURN
      END




      SUBROUTINE               DFGRAD(X,GRAD,F1,N,MAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                     C
C   To numerically compute the gradient vector dF/dx at X.      C
C   F1 must contain the functional value at X.                  C
C   ROUTINES CALLED : FX                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER                  N,MAX,I
      DOUBLE PRECISION         X(MAX),GRAD(MAX),F1
      DOUBLE PRECISION         DEPS,F2 

c... Common with DCBRIC*, DGGRAD og DFGRAD                    ..........
c... Initial and maximal stepsize                             ..........
      DOUBLE PRECISION H,stp
      COMMON /DSTEP/ H,stp

      DO 10 I=1,N
         DEPS    = DABS( H*X(I) )
         IF (DEPS.LT.H)   DEPS = H
         IF (DEPS.GT.stp) DEPS = stp
         X(I)    = X(I) + DEPS
 
         CALL FX(F2,X,N)
         GRAD(I) = (F2 - F1)/DEPS
         X(I)    = X(I) - DEPS
10    CONTINUE

      RETURN
      END



      SUBROUTINE               DGGRAD(X,DG,G,WA,ACTIVE,N,M,ME,MMAX)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   PURPOSE                                                     C
C   To numerically compute the gradient vector dG/dx at X.      C
C   G must contain the values for the constraints at X.         C
C   ROUTINES CALLED : DCONST                                    C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER                  M,ME,MMAX,N
      DOUBLE PRECISION         G(MMAX),DG(MMAX,N),X(N),WA(MMAX)
      LOGICAL                  ACTIVE(MMAX)

c... Common with DCBRIC*, DGGRAD og DFGRAD                    ..........
c... Initial and maximal stepsize                             ..........
      DOUBLE PRECISION H, stp
      COMMON /DSTEP/ H, stp

      DO 10 i = 1,N
         X(i) = X(i) + H
10    CONTINUE

      CALL DCONST(WA,X,N,ACTIVE,M,ME,MMAX)

      DO 20 i = 1,M
         IF ( ACTIVE(i) ) DG(i,1) = ((WA(i) - G(i))/ H)
20    CONTINUE

      DO 30 i = 1,N
         X(i) = X(i) - H
30    CONTINUE

      RETURN
      END




 
      SUBROUTINE               DCONST(G,q12,dim,ACTIVE,M,ME,MMAX)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Kaller en rutine som beregner bibetingelsene G(M) som            c
c     funksjon av matrisen R2(2,2).                                    c
c                                                                      c
c     INPUT                                                            c
c     q12       - argument til funksjonsverdien trace(R).              c
c                 Element (1,2) og (2,1) i matrisen Q2.                c
c     dim       - dimensjon av funksjonsverdien trace(R).              c
c                 I dette tilfellet er dim = 1.                        c
c     ACTIVE(M) - logisk vektor som forteller hvilke bibetingelser     c
c                 som er aktive.                                       c
c     M         - totalt antall bibetingelser                          c
c     ME        - antall ulikheter av disse                            c
c     MMAX      - max antall bibetingelser                             c
c                                                                      c
c     OUTPUT                                                           c
c     G(M)      - vektor med verdien av bibetingelsene                 c
c                                                                      c
c     COMMON-blokken                                                   c
c     BLK2 - Med oppdaterte matriser Q2(2,2) og R2(2,2)                c
c                                                                      c
c     RUTINER SOM KALLES : FX                                          c
c                          DCNEVEG                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c...                                                          ..........
      INTEGER                  dim,M,ME,MMAX
      DOUBLE PRECISION         G(MMAX),q12
      LOGICAL                  ACTIVE(MMAX)

c...  Benyttes av DCBRIC*,DCONST* og FX*                      ..........
      COMMON                   / BLK2 / T2, H2, Q2, R2
      DOUBLE PRECISION         T2(2,2), H2(2,2), Q2(2,2), R2(2,2)

c...  Lokale variable                                         ..........
      INTEGER                  nm   
      DOUBLE PRECISION         trace

 
c...  Max dimensjon pa R2                                     ..........
      nm = 50
        
      CALL FX(trace,q12,dim)
      CALL DCNVEG(R2,2,nm,G,ACTIVE,M,ME,MMAX)

      RETURN
      END
 



      SUBROUTINE               DCNVEG(R2,n,nm,G,ACTIVE,M,ME,MMAX)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner bibetingelsene G(M) som funksjon av matrisen R2(n,n)    c
c     for de bibetingelsene som er aktive.                             c
c                                                                      c
c        1. G(1) = R2(n-1,n-1)                                         c
c        2. G(2) = R2(n-1,n-1)*R2(n,n) - R2(n-1,n)*R2(n-1,n)           c
c                                                                      c
c     INPUT                                                            c
c     R2(n,n)   - matrise                                              c
c     n         - aktuell dimensjon                                    c
c     nm        - max dimensjon                                        c
c     ACTIVE(M) - logisk vektor som forteller hvilke bibetingelser     c
c                 som er aktive.                                       c
c     M         - totalt antall bibetingelser                          c
c     ME        - antall ulikheter av disse                            c
c     MMAX      - max antall bibetingelser                             c
c                                                                      c
c     OUTPUT                                                           c
c     G(M)      - vektor med verdien av bibetingelsene                 c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm,M,ME,MMAX
      DOUBLE PRECISION         R2(nm,nm), G(MMAX)  
      LOGICAL                  ACTIVE(MMAX)

c...  Lokal variabel                                          ..........
      INTEGER                  x

      x = n-1

c...  Beregn verdien av bibetingelsene hvis de er aktive      ..........
c...  Vektoren "ACTIVE(M)" tilordnes av rutinen NLPQ0         ..........

      IF ( ACTIVE(1) ) G(1) = R2(x,x)
      IF ( ACTIVE(2) ) G(2) = R2(x,x)*R2(n,n) - R2(x,n)*R2(x,n)

      RETURN
      END




c
c     SUBROUTINE               DCNVEG(R2,n,nm,G,ACTIVE,M,ME,MMAX)
c
c     INTEGER                  n,nm,M,ME,MMAX
c     DOUBLE PRECISION         R2(nm,nm), G(MMAX)
c     LOGICAL                  ACTIVE(MMAX)
c
c...  Benyttes av DPRTLEV*, DCBRIC, DCNVEG og FX              ..........
c     COMMON                   / PRLE / prtlev
c     INTEGER                  prtlev  
c
c     INTEGER                  x
c
c     x = n-1
c
c     IF ( ACTIVE(1) ) THEN
c        G(1) = R2(x,x)
c        IF ( prtlev .EQ. 2 ) WRITE(*,*) '   G(1) : ',G(1)
c     END IF
c     IF ( ACTIVE(2) ) THEN
c        G(2) = R2(x,x)*R2(n,n) - R2(x,n)*R2(x,n)
c        IF ( prtlev .EQ. 2 ) WRITE(*,*) '   G(2) : ',G(2)
c     END IF
c                                   
c     RETURN
c     END
c




      SUBROUTINE FX(trace,q12,dim)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner og returnerer funksjonsverdien "trace" = Trace(R2).     c
c                                                                      c
c     Kaller rutiner som beregner :                                    c
c          - matrisen Q2 som funksjon av parameteren "q12".            c
c          - losningen R2 av Riccati-ligningen som funksjon            c
c            av matrisene T2, H2 og Q2.                                c
c          - tracen til R som funksjon av matrisen R2.                 c
c                                                                      c
c     INPUT                                                            c
c     q12  - element (2,1) og (1,2) i matrisen Q2(2,2)                 c
c     dim  - dimensjonen til funksjonsverdien Trace(R).                c
c            I dette tilfelle er : dim = 1.                            c
c                                                                      c
c     COMMON-blokken                                                   c
c     BLK2 - Med matrisene T2(2,2) og H2(2,2)                          c
c                                                                      c
c     OUTPUT                                                           c
c     trace - tracen til matrisen R2                                   c
c                                                                      c
c     COMMON-blokken                                                   c
c     BLK2 - Med oppdaterte matriser Q2(2,2) og R2(2,2)                c
c                                                                      c
c     RUTINER SOM KALLES : DCNQ2M     RICSOL                           c
c                          MATCOP     DTRACE                           c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  dim
      DOUBLE PRECISION         trace,q12

c...  Benyttes av DPRTLEV*, DCBRIC, DCNVEG og FX              ..........
      COMMON                   / PRLE / prtlev
      INTEGER                  prtlev  

c...  Benyttes av DCBRIC*,DCONST* og FX*                      ..........
c...  De 2.dimensjonale matriser T2 og H2 brukes i beregningen av R2....
      COMMON                   / BLK2 / T2, H2, Q2, R2
      DOUBLE PRECISION         T2(2,2), H2(2,2), Q2(2,2), R2(2,2)

c...  Lokale variable                                         ..........
      DOUBLE PRECISION         Z(4,4) , W(4,4)
      DOUBLE PRECISION         er2(4), ei2(4),   wv2(4)
      INTEGER                  ind2(4), info, nm 

c...  Max dimensjon                                           ..........
      nm = 50

      IF ( prtlev .EQ. 2 ) WRITE(*,*) 'FX q12  : ',q12

c...  Beregner matrisen Q2 og kopierer denne til R2           ..........
      CALL DCNQ2M(Q2,2,nm,q12)
      CALL MATCOP(Q2,R2,2,2,nm,nm)    
      CALL DMZSEL(R2,2,2,nm,nm)

c...  Finner losningen R2 av Riccati-ligningen fra T2,H2 og Q2 .........
      CALL RICSOL(T2,H2,R2,Z,W,2,er2,ei2,wv2,ind2,info,nm,nm,nm)
      CALL MATPRT('R2 -> FX-RICSOL',R2,2,2,2,2)

c...  Finner funksjonsverdien trace = Trace(R2)               ..........
      CALL DTRACE(R2,2,nm,trace)

      RETURN
      END




      SUBROUTINE               DCNQ2M(Q2,n,nm,q12)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Tilordner matrisen Q2 som funksjon av parameteren "q12" og       c
c     COMMON-blokkene SSET og QVEK. Teorien bak denne rutinen          c
c     star i kap3 i Bergslid (1989)                                    c
c                                                                      c
c     INPUT                                                            c
c     q12  - element (n-1,n) og (n,n-1) i matrisen Q2(n,n)             c
c                                                                      c
c     COMMON-blokkene                                                  c
c     SSET - forteller hvilken av 2 alternative                        c
c            losninger som skal brukes.                                c
c     QV   - vektor som bestemmer Q2(n-1,n-1) og Q2(n,n)               c
c            som funksjon av q12.                                      c
c                                                                      c
c     OUTPUT                                                           c
c     Q2(n,n) - matrisen                                               c
c     n       - aktuell dimensjon                                      c
c     nm      - max dimensjon                                          c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         Q2(nm,nm), q12

c...  Benyttes av DCBRIC* og DCNQ2M                           ..........
      COMMON                   / SSET / sgn
      CHARACTER                sgn
c...  Benyttes av DCBRIC, DCNQ2M, DRMOV*, DCALQS*             ..........
      COMMON                   / QVEK / QV
      DOUBLE PRECISION         QV(1:11)

c...  Lokale variable                                         ..........
      INTEGER                  x
      DOUBLE PRECISION         wv1, wv3

      x = n-1

c...  Sett elementene i pos (x,n) og (n,x) direkte            ..........
      Q2(x,n) = q12
      Q2(n,x) = q12

c...  Hvis to alternative losninger finnes (elliptisk funksjon).........
      IF ( QV(1) .EQ. 1.0 ) THEN
         wv1 = QV(4)*Q2(x,n)*Q2(x,n)+QV(5)*Q2(x,n)+QV(6)
         wv3 = QV(7)*Q2(x,n) + QV(8)

         IF ((QV(10).EQ.0.0).OR.(QV(11).EQ.0.0)) THEN
            WRITE(*,*) 'FX: QV(10) eller QV(11) er lik null'
         END IF
         IF (wv1.LT.0.0) THEN
            WRITE(*,*) 'FX: wv(1) er mindre enn null'
         END IF

c...     Finn Q2(x,x) og Q2(n,n) som funksjon av "q12" og QV  ..........
c...     og avhengig av "sgn"                                 ..........
         IF ( sgn .EQ. '+' ) THEN
            Q2(x,x) = ((wv3+QV(9)+SQRT(wv1))/QV(10))
            Q2(n,n) = ((wv3-QV(9)-SQRT(wv1))/QV(11))
         ELSE
            Q2(x,x) = ((wv3+QV(9)-SQRT(wv1))/QV(10))
            Q2(n,n) = ((wv3-QV(9)+SQRT(wv1))/QV(11))
         END IF

c...  Finn Q2(x,x) og Q2(n,n) som funksjon av QV og "q12"     ..........
c...  Determinanten til H2 er lik null                        ..........
      ELSE IF ( QV(1) .EQ. 2.0 ) THEN
         Q2(x,x) = QV(4)*Q2(x,n) + QV(5)
         Q2(n,n) = QV(6)*Q2(x,n) + QV(7)
      END IF


      RETURN
      END


                                                       
      SUBROUTINE DCSRIC(T,H,Q,R,n,nm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Loser den skalare Riccati-ligningen ut fra elementene            c
c     i pos (n,n) i matrisene T,H og Q. Losningen legges i             c
c     pos(n,n) i matrisen R. De andre elementene i R nullsettes.       c
c                                                                      c
c     INPUT                                                            c
c     T(n,n) - matriser                                                c
c     H(n,n)                                                           c
c     Q(n,n)                                                           c
c     n      - aktuell dimensjon                                       c
c     nm     - max dimensjon                                           c
c                                                                      c
c     OUTPUT                                                           c
c     R(n,n) - matrise med losningen                                   c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         T(nm,nm), H(nm,nm)
      DOUBLE PRECISION         Q(nm,nm), R(nm,nm)

c...  Lokale variable                                         ..........
      DOUBLE PRECISION         a,b,c,d

c...  Skalar Riccati-ligning :                                ..........
c...  H(n,n)*R(n,n)*R(n,n) - 2.0*T(n,n)*R(n,n) - Q(n,n)       ..........

      a = H(n,n)
      b = -2.0D0*T(n,n)
      c = -Q(n,n)                  
      d = b*b - 4.0D0*a*c

c...  Nullstiller matrisen R                                  ..........
      CALL DMZER(R,n,n,nm,nm)

c...  Finn positiv losning av 2.gradsligningen hvis den eksisterer......
      IF ((a.NE.0.0 ).AND.(d.GE.0.0)) THEN
         R(n,n) = (-b + SQRT(d))/(2.0D0*a)
      ELSE
         WRITE(*,*)
         WRITE(*,*) ' Not possible with this location'
      END IF

      RETURN                
      END

      SUBROUTINE DRMOV(QS,T,U,UT,H,HS,n,m,nm,mm,er,ei,ind,ix,mov,wm1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              12. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Lenker sammen flere mindre rutiner som i hovedtrekk :            c
c                                                                      c
c       - skriver ut egenverdiene og registrerer hvilke som            c
c         skal flyttes.                                                c
c       - finner ny ortogonal similaritetstransf. U slik at            c
c         egenverdien(e) som skal flyttes havner i blokken T22.        c
c       - beregner ny UT = TSP(U) og HS = TSP(U)*H*U                   c
c       - registrerer nye egenverdier og kontrollerer at de            c
c         er lovlige.                                                  c
c       - finner alle mulige matriser QS som sorger for at             c
c         systemet far de spesifiserte egenverdiene nar QS(n-1,n)      c
c         kan variere. Med valgt QS(n-1,n) finnes den entydige         c
c         QS-mtrisen.                                                  c
c                                                                      c
c     INPUT                                                            c
c     QS(n,n) - matriseproduktet : QS = TSP(U)*Q*U,                    c
c               der matrisen Q er tilstandenes vektmatrise.            c
c     T(n,n)  - Ovre semitriangulaer form med egenverdier i            c
c               blokkform paa hoveddiagonalen.                         c
c     U(n,n)  - Ortogonale similaritets-transformasjoner.              c
c     H(n,n)  - matriseproduktene : H  = B*INV(P)*TSP(B)               c
c                                                                      c
c     n,m     - aktuelle dimensjoner                                   c
c     nm,mm   - max dimensjoner                                        c
c                                                                      c
c     er(n)   - vektor med egenverdienes realdel                       c
c     ei(n)   - vektor med egenverdienes imaginaerdel                  c
c     ind(n)  - vektor som viser egenverdiens type                     c
c               0 : reell egenverdi.                                   c
c               1 : imaginaer egenverdi m/positiv imaginaerdel.        c
c               2 : imaginaer egenverdi m/negativ imaginaerdel.        c
c              -1 : har ikke funnet egenverdi                          c
c                                                                      c
c     ix(n)    - arbeidsomrader                                        c
c     wm1(n,n) -                                                       c
c                                                                      c
c     OUTPUT                                                           c
c     QS(n,n) - med blokken QS22 oppdatert                             c
c     T(n,n)  - omformet                                               c
c     U(n,n)  -                                                        c
c     UT(n,n) - = TSP(U)                                               c
c     HS(n,n) - matriseproduktet  : HS = TSP(U)*H*U                    c
c                                                                      c
c     er(n)   - oppdatert                                              c
c     ei(n)   -                                                        c
c     ind(n)  -                                                        c
c                                                                      c
c     mov     - flagg som viser om noen egenverdier er flyttet         c
c               .TRUE.  - egenverdi(er) flyttet                        c
c               .FALSE. - ingen flytting                               c
c                                                                      c
c     COMMON-blokkene                                                  c
c     QVEK    - inneholder QS(x,x) og QS(n,n) som funksjon av QS(x,n)  c
c     SCRI    - inneholder flagget onereal                             c
c               .TRUE.  - en reell egenverdi flyttes skalart.          c
c               .FALSE. - flytting i 2.ordens system.                  c
c                                                                      c
c     RUTINER SOM KALLES : DWEIGV     MATCOP      DCMIVA               c
c                          DCNOTU     DMZER       DRNEV                c
c                          DMTSP      DREAD       DCALQS               c
c                          DMTMP      DCOEFS      DPRERR               c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm,ind(nm),ix(nm)
      DOUBLE PRECISION         QS(nm,mm)
      DOUBLE PRECISION         T(nm,mm), H(nm,mm), HS(nm,mm)
      DOUBLE PRECISION         U(nm,mm), UT(nm,mm)
      DOUBLE PRECISION         er(nm),    ei(nm)
      DOUBLE PRECISION         wm1(nm,mm)
      DOUBLE PRECISION         wv1(50), det
      LOGICAL                  mov  

c...  Benyttes av DCBRIC, DCNQ2M, DRMOV*, DCALQS*          ............
      COMMON                   / QVEK / QV
      DOUBLE PRECISION         QV(1:11)

c...  Benyttes av DCBRIC og DRMOV*                         ............
      COMMON                   / SCRI / onereal
      LOGICAL                  onereal

      COMMON                   / MINF / iout, iprt
      INTEGER                  iout, iprt

c...  Lokale variable                                      ............
      INTEGER                  nf,if,tep,aep(2)
      DOUBLE PRECISION         nrv,maksim


c...  Skriver ut egenverdiene og registrerer hvilke som    ............
c...  evt. skal flyttes pa.                                ............
100   CALL DWEIGV(er,ei,ind,n,nm,tep,aep,ix)

c...  Hvis minst en egenverdi onskes flyttet               ............
      IF ((aep(1).GE.1).AND.(tep.GE.1)) THEN

         mov = .TRUE.

         IF ( n .GE. 3 ) THEN

c...        Finner ny matrise U slik at egenverdiene       ............
c...        som skal flyttes star i blokken T22            ............
c...        Oppdaterer samtidig er,ei og ind.              ............
            CALL DCNOTU(T,U,n,m,nm,mm,er,ei,ind,aep,ix)

            CALL MATPRT('T = schur(T11 T22)',T,NM,NM,N,N)
            CALL MATPRT('U = schur(U11-U22)',U,NM,NM,N,N)

c...        Finner UT = TSP(U) og HS = TSP(U)*H*U          ............
            CALL DMTSP(UT,U,n,n,nm,nm)
c           CALL MATCOP(U,wm1,n,n,nm,nm)
c           CALL INVERS(wm1,UT,wv1,det,n,nm)
            CALL DMTMP(HS,UT,H,n,n,nm,nm,wm1)
         ELSE   
c...        Matrisen U er lik identitetsmatrisen           ............
c...        og HS blir da lik H.                           ............
            CALL MATCOP(H,HS,n,n,nm,nm)
         END IF

c...     Skalart system eller komplekse egenverdier        ............
c...     i pos (n-1) og (n-2) i T.                         ............
         IF (( n.EQ.1 ).OR.((ind(n).EQ.0).AND.(ind(n-1).NE.0))) THEN

c...        Flytter en reell egenverdi (spesialtilfelle)   ............
            onereal = .TRUE.
            QV(1) = 0.0

            CALL DMZER(QS,n,n,nm,nm)

c...        Gammel verdi er deaffault-verdi                ............
            nrv = T(n,n)
            WRITE (*,*)
            CALL DREAD('New real eigenvalue        : ',nrv,1,1,IIII)

c...        Bruk formel for skalar flytting.               ............
c...        Bare element (n,n) i QS far verdi ulik null    ............
            QS(n,n) = ((nrv**2-T(n,n)**2)/HS(n,n))

         ELSE

c...        Benytter 2. ordens system. Finner da alle lovlige QS.......
            onereal = .FALSE.

c...        Beregner koeff. som brukes i rutinen DCALQS    ............
            CALL DCOEFS(T,HS,n,m,nm,mm)
 
c...        Hvis bruker onsker a flytte pa to egenverdier samtidig,....
c...        beregn og skriv ut max imaginaerdel for komplekse egenv.... 
            IF (( ind(n) .NE. 0 ).OR.(( aep(1).NE.aep(2) ).AND.
     +          ( aep(2) .NE. 0 ))) 
     +          CALL DCMIVA(T,HS,n,m,nm,mm,if,maksim)

c...        Registrer nye egenverdier. Kontroller om de er lovlige.....
200         CALL DRNEV(er,ei,ind,n,nm,aep,maksim,nf,if)
c...        Returner hvis bruker angrer.                   ............
            IF ( nf .EQ. 1 ) GOTO 100

c...        Beregn alle mulige QS-matriser for optimalisering..........
            CALL DCALQS(QS,n,m,nm,mm,nf)

            IF ( iprt .EQ. 1 ) THEN
               WRITE(*,*)
               CALL MATPRT('Matrise T : ',T,nm,nm,n,n)
               WRITE(*,*)
               CALL MATPRT('Matrise HS: ',HS,nm,nm,n,n)
               WRITE(*,*)
               CALL MATPRT('Matrise QS: ',QS,nm,nm,n,n)
            END IF

c...        Kall rutine for utskrift av feilmelding hvis feil .........
c...        har inntruffet.                                ............
            IF ( nf .NE. 0 ) THEN
               CALL DPRERR(nf)
               GOTO 100
            END IF

         END IF
c...  Bruker onsker ikke a flytte egenverdier. Returner    ............
      ELSE IF (aep(1) .EQ. 0) THEN
         mov = .FALSE.
      ELSE
         GOTO 100
      END IF

      WRITE(*,*)'ONEREAL',ONEREAL


      RETURN
      END





      SUBROUTINE DPRERR(nf)

c...  Skriver ut feilmeldinger avhengig av heltallsvariabelen nf.......
c...  Kalles av rutinen DRMOV.                             ............

      INTEGER                  nf

      WRITE (*,*)
      IF ( nf .EQ. 1 ) THEN
         WRITE (*,*) ' Not possible to find q12 = q21 in Q22'
         GOTO 999
      ELSE IF ( nf .EQ. 2 ) THEN
         WRITE (*,*) ' Not possible to find q11 and q22 in Q22'
         GOTO 999
      ELSE IF ( nf .EQ. 3 ) THEN
         WRITE (*,*) ' Not possible to control the system',              
     +               ' with these values'
         GOTO 999
      ELSE IF ( nf .EQ. 4 ) THEN
         WRITE (*,*) ' The determinant of H is negative'
         GOTO 999
      ELSE IF ( nf .EQ. 5 ) THEN
         WRITE (*,*) ' a(0) is zero, but not c(1)'
         GOTO 999
      ELSE IF ( nf .EQ. 6 ) THEN
         WRITE (*,*) ' Have found that J(q12) is negativ'
         GOTO 999
      END IF

999   RETURN
      END




      SUBROUTINE DWEIGV(er,ei,ind,n,nm,j,k,ix)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH 1989                                c
c     Versjon 1 :              11. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Skriver ut og nummererer  egenverdiene som ligger i              c
c     vektorene er og ei. Registerer hvilke egenverdier som            c
c     brukeren onsker a flytte og kontrollerer at det pekes            c
c     pa hoyst to egenverdier.                                         c
c                                                                      c
c     INPUT                                                            c
c     er(n)  - vektor med egenverdienes realdel                        c
c     ei(n)  - vektor med egenverdienes imaginaerdel                   c
c     n      - aktuell dimensjon                                       c
c     nm     - max dimensjon                                           c
c     ind(n) - vektor som viser egenverdiens type                      c
c              0 : reell egenverdi.                                    c
c              1 : imaginaer egenverdi m/positiv imaginaerdel.         c
c              2 : imaginaer egenverdi m/negativ imaginaerdel.         c
c             -1 : har ikke funnet egenverdi                           c
c                                                                      c
c     OUTPUT                                                           c
c     j     - antall egenverdipar                                      c
c     ix(n) - indekstabell som viser hvilket egenverdinummer           c
c             et egenverdipar har. Bade to komplekskonjungerte         c
c             og en reell egenverdi representerer et egenverdipar.     c
c     k(2)  - peker til en eller to egenverdipar. Brukes som           c
c             indeks i ix(n). Inneholder pekere til max to             c
c             egenverdier.                                             c
c                                                                      c
c     RUTINER SOM KALLES : NREAD                                       c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n, nm
      INTEGER                  j, k(2), ix(nm), ind(nm)
      DOUBLE PRECISION         er(nm), ei(nm) 

c...  Lokale variable                                      ............
      INTEGER                  i,in,ij


100   WRITE(*,*)
      WRITE(*,*) ' Eigenvalues of (A + BG):'
      WRITE(*,*) ' ------------------------'

       i = 0
      ij = 0

c...  Skriver ut egenverdier og nummererer i egenverdipar  ............
10    CONTINUE            
         i = i + 1
         ij = ij + 1

c...     Reell egenverdi                                   ............
         IF (ind(i).EQ.0) THEN
            ix(ij) = i
            WRITE(*,1) ij,i,er(i)
1           FORMAT (I2,'.   s(',I2,') =',G16.6)

c...     Komplekskonjungert egenverdi                      ............
         ELSE IF ((ind(i).EQ.1).OR.(ind(i).EQ.2)) THEN
            IF ((ind(i).EQ.1).AND.(ind(i+1).EQ.-1)) THEN
               ind(i+1) = 2
            ELSE IF ((ind(i).EQ.2).AND.(ind(i+1).EQ.-1)) THEN
               ind(i+1) = 1
            END IF
            ix(ij) = i
            WRITE(*,2) ij,i, er(i), ei(i)
            WRITE(*,3) i+1, er(i+1),ei(i+1)
2           FORMAT (I2,'.   s(',I2,') =',G16.6,4X,G16.6,' i')
3           FORMAT (2X,'    s(',I2,') =',G16.6,4X,G16.6,' i')
            i = i + 1
         ELSE IF (ind(i).EQ.-1) THEN
            ix(ij) = i
            WRITE(*,4) ij,i,er(i)
4           FORMAT (I2,'.   s(',I2,') =',G16.6,'  *')
         END IF

      IF ( i .LT. n ) GOTO 10

      in = 0
      DO 20 i = 1,n
         IF (ind(i) .NE. -1) in = in + 1
20    CONTINUE

c     WRITE(*,*)
c     WRITE(*,'(1X,A,6i3)')'DWEIGV IND = ',(ind(i),i=1,n)

      IF ( in .LT. n ) THEN
         WRITE(*,*)
         WRITE(*,*) ' WARNING --> Not possible to',
     +              ' isolate marked eigenvalues'
      END IF

      j = ij

c...  Initier pekere                                       ............
200   k(1) = 0
      k(2) = 0
      WRITE(*,*)
      CALL NREAD('Eigenvalue numbers (0-Exit) : ',k,2,2,IIII)
      ij = j

c...  Hvis ulovlig egenverdinummer, registrer pa nytt      ............
      IF ((k(1).LT.0).OR.(k(1).GT.ij).OR.
     +    (k(2).LT.0).OR.(k(2).GT.ij)) THEN
         WRITE(*,*)
         WRITE(*,5) 'Possible numbers is 0 - ',ij,'.'
5        FORMAT(1X,A,I1,A)
         GOTO 200

      ELSE IF ((ind(ix(k(1))).EQ.-1).OR.(ind(ix(k(2))).EQ.-1)) THEN
         WRITE(*,*)
         WRITE(*,6) 'Not possible to move marked eigenvalues.'
6        FORMAT(1X,A)
         GOTO 200

c...  Registerer pa nytt hvis bruker onsker a              ............
c...  flytte pa flere enn to egenverdier samtidig          ............
      ELSE IF ((k(1).NE.0).AND.(k(2).NE.0).AND.
     +        ((ind(ix(k(1))).NE.0).OR.(ind(ix(k(2))).NE.0))) THEN
         WRITE(*,*)
         WRITE(*,*) ' Can only move two eigenvalues a time.'
         GOTO 200
      END IF

c     WRITE(*,*)
c     WRITE(*,'(1X,A,6i3)')'DWEIGV j   = ',j
c     WRITE(*,'(1X,A,6i3)')'DWEIGV k   = ',(k(i),i=1,2)
c     WRITE(*,'(1X,A,6i3)')'DWEIGV ix  = ',(ix(i),i=1,n)
 
      RETURN
      END



      SUBROUTINE DRNEV(er,ei,ind,n,nm,aep,maksim,nf,if)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH 1989                                c
c     Versjon 1 :              11. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Brukeren onsker a flytte egenverdier, enten en eller to reelle   c
c     eller et komplekskonjungert par. Rutinen skriver ut gamle        c
c     egenverdier som deffault-verdi og registrerer nye sammen         c
c     med egenverditype. Type viser programmet om innleste verdier     c
c     skal tolkes som imagiaere (1) eller reelle (2). Det kontrolleres c
c     om brukeren registrerer en imaginaerverdi som er lavere enn      c
c     ovre grense.                                                     c
c                                                                      c
c     Avhengig av type, beregnes koeffisientene p1 og p2 som           c
c     legges i COMMON-blokken NEIG.                                    c
c                                                                      c
c     EKS                                                              c
c     New values and type :( -3.0 , -5.2 , 2 ) ---> 2.0 1.5 1          c
c                                                                      c
c     Brukeren har valgt a flytte de reelle (2) egenverdiene -3.0      c
c     og -5.2 til et komplekskonjungert par (1) med realverdi -2.0 og  c
c     imaginaer verdi 1.5. Hvis brukeren velger type lik 0,            c
c     returnerer rutinen uten a registrere nye egenverdier.            c
c                                                                      c
c     INPUT                                                            c
c     er(n)  - vektor med egenverdienes realdel                        c
c     ei(n)  - vektor med egenverdienes imaginaerdel                   c
c     ind(n) - vektor med egenverdienes type.                          c
c              0 : reell egenverdi.                                    c
c              1 : imaginaer egenverdi m/positiv imaginaerdel.         c
c              2 : imaginaer egenverdi m/negativ imaginaerdel.         c
c             -1 : har ikke funnet egenverdi                           c
c     n      - aktuell dimensjon                                       c
c     nm     - max dimensjon                                           c
c     maksim - storste tillatte imaginaerdel                           c
c     if     - heltallsflagg for imaginaerdelen                        c
c              1 : ovre grense eksisterer                              c
c              0 : ingen ovre grense                                   c
c                                                                      c
c     OUTPUT                                                           c
c     aep(2) - peker til en eller to egenverdipar.                     c
c              Inneholder pekere til max to egenverdier.               c
c     nf     - heltallsflagg                                           c
c              1 : egenverdier er ikke flyttet                         c
c              0 : egenverdier er flyttet                              c
c                                                                      c
c     COMMON-blokkene                                                  c
c     NEIG  - der koeff. p1 og p2 far verdier avhengig av              c
c             innleste egenverdier. Disse benyttes i rutinen DCALQS.   c
c     MOVE  - Inneholder logisk variabel "mole".                       c
c             .TRUE.  - egenverdien(e) flyttes mot venstre             c
c             .FALSE. - ingen flytting mot venstre.                    c
c                                                                      c
c     RUTINER SOM KALLES : DREAD                                       c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         er(nm), ei(nm), maksim
      INTEGER                  ind(nm), aep(2), nf, if

c...  Benyttes av DRNEV*, DCALQS                           ............ 
c...  Koeffisienter som brukes i rutinen DCALQS            ............
      COMMON                   / NEIG / p1,p2
      DOUBLE PRECISION         p1,p2

c...  Benyttes av MODOPT*, DSYSOP*, DRNEV og DCMIVA          ...........
c...  Viser om bruker onsker informasjon om max imaginaerdel ...........
      COMMON                   / MAXI / miv 
      LOGICAL                  miv

c...  Benyttes av DCBRIC og DRNEV*                            .........
c...  Settes lik .TRUE. hvis egenverdien(e) flyttes mot venstre........
      COMMON                   / MOVE / mole
      LOGICAL                  mole
                                       
c...  Lokale variable                                      ............
      DOUBLE PRECISION         dw(3),ww(3),cv,rv,nv,LL,UL
      PARAMETER              ( nv = 0.0, cv = 1.0, rv = 2.0 )
      LOGICAL                  same

      mole = .FALSE.

c...  Bruker vil bare flytte en reell egenverdi            ............
      IF (((aep(2).EQ.0).OR.(aep(2).EQ.aep(1))).AND.
     +     (ind(n) .EQ. 0)) THEN

         dw(3) = rv

c...     La deffault-verdi vaere gammel verdi ved ny innlesing ........
         IF ((aep(1).EQ.1).AND.(n.EQ.2)) THEN
            dw(1) = er(n-1)
            dw(2) = er(n)
         ELSE
            dw(1) = er(n)
            dw(2) = er(n-1)
         END IF

         WRITE (*,*)
         ww(1) = dw(1)
         CALL DREAD('New real eigenvalue        : ',dw(1),1,1,IIII)
c...     Returner, hvis samme verdi er registrert.         ............
         IF ( ABS(ww(1)) .EQ. ABS(dw(1)) ) GOTO 888

      ELSE

c...     Skriv ut max imaginaerdel hvis den finnes og bruker onsker det.
50       IF (( miv ) .AND. ( if .EQ. 1 )) THEN
            WRITE(*,*)
            WRITE(*,10) 'Max imaginary value is : ',maksim
10          FORMAT(1X,A,G16.6)
         END IF
        
100      WRITE(*,*)
c...     Lar gamle egenverdier vaere deffault-verdi        ............
         IF (ei(n).EQ.0.0) THEN
            dw(3) = rv
            dw(2) = er(n)
         ELSE
            dw(3) = cv
            dw(2) = ei(n-1)
         END IF

         dw(1) = er(n-1)

         DO 60 i = 1,3
            ww(i) = dw(i)
60       CONTINUE

c...     Registrer nye verdier.........................................
         CALL DREAD('New values and type :',dw,3,3,IIII)

         same = .TRUE.
         DO 70 i = 1,3
            IF ( same ) same = (ABS(ww(i)).EQ.ABS(dw(i)))
70       CONTINUE
c...     Returner, hvis de samme verdiene er registrert.   ............
         IF ( same ) GOTO 888

      END IF

c...  Beregn p1 og p2 avhengig av om de nye verdiene skal  ............
c...  vaere komplekse eller reelle.                        ............
c...  Sett logisk variabel mole = .TRUE. hvis de nye polene............
c...  flyttes mot venstre.                                 ............
      IF ( dw(3) .EQ. cv ) THEN

c...     Bruker onsker a flytte til komplekskonjungert pol.............
c...     Registrer pa nytt hvis ulovlig stor imaginaerdel  ............
         IF ((if.EQ.1).AND.( ABS(dw(2)).GT.ABS(maksim))) GOTO 50

         p1 = (2 * ( dw(1)**2 - dw(2)**2 ))
         p2 = (( dw(1)**2 + dw(2)**2 )**2 )

c...                                                       ............
         IF (ei(n).NE.0.0) THEN
c...        De gamle polene var komplekskonjungerte.      .............
            mole = ABS(dw(1)).GT.ABS(er(n-1))
         ELSE
c...        De gamle polene var reelle.                   .............
            mole = ((ABS(dw(1)).GT.ABS(er(n-1))).AND.
     +              (ABS(dw(1)).GT.ABS(er(n))))
         END IF
      ELSE IF ( dw(3) .EQ. rv ) THEN

c...     Bruker onsker a flytte til reelle poler          .............
         p1 = ( dw(1)**2 + dw(2)**2 )
         p2 = ( dw(1)**2 * dw(2)**2 )

         IF (ei(n).NE.0.0) THEN
c...        De gamle polene var komplekskonjungerte.      .............
            mole = ((ABS(dw(1)).GT.ABS(er(n-1))).AND.
     +              (ABS(dw(2)).GT.ABS(er(n-1))))
         ELSE

c...        De gamle polene var reelle.                   .............
c...        Finn storste og minste realdel til nye egenverdier.........
            IF (ABS(dw(1)).LE.ABS(dw(2))) THEN
               LL = dw(1)
               UL = dw(2)
            ELSE
               LL = dw(2)
               UL = dw(1)
            END IF

            mole =(((ABS(LL).GE.ABS(er(n-1))).OR.
     +              (ABS(LL).GE.ABS(er(n)))).AND.
     +             ((ABS(UL).GE.ABS(er(n-1))).AND.
     +              (ABS(UL).GE.ABS(er(n)))))

         END IF

      ELSE IF ( dw(3) .EQ. nv ) THEN
c...     Returner hvis type = 0 eller ingen endring        ............
c...     mellom gamle og nye egenverdier.                  ............
888      nf = 1
         GOTO 999
      ELSE
         WRITE(*,*)
         WRITE(*,*) ' Possible type is 0 - 2. '
         WRITE(*,*) ' 0 - return from this level.'
         WRITE(*,*) ' 1 - new complex values.  '
         WRITE(*,*) ' 2 - new real values.     '
         GOTO 100
      END IF

c...  Nye verdier er registrert                            ............
      nf = 0

999   RETURN
      END


                              
      SUBROUTINE DMTMP(H,M1,M2,n,m,nm,mm,wm1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner matrise-produktet : H = M1 * M2 * TSP(M1)               c
c                                                                      c
c     INPUT                                                            c
c     M1(n,m) - matrise                                                c
c     M2(m,m) - matrise                                                c
c     n,m     - aktuelle dimensjoner                                   c
c     nm,mm   - max dimensjoner                                        c
c                                                                      c
c     OUTPUT                                                           c
c     H(n,n)  - matrise                                                c
c                                                                      c
c     RUTINER SOM KALLES : TRANSMULT                                   c
c                          MATMULT                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         H(nm,mm),  wm1(nm,mm)
      DOUBLE PRECISION         M1(nm,mm), M2(nm,mm)

      CALL     TRANSMULT(M2,M1,wm1,m,m,n,nm,1)
      CALL     MATMULT(M1,wm1,H,n,m,n,nm)

      RETURN
      END



      SUBROUTINE DUMRQ(M1,M1R,M1S,U,n,m,nm,mm,wm1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner uttrykket : M1 = M1 + ( U * M1 * TSP(U) )               c
c                                                                      c
c     INPUT                                                            c
c     U(n,n)                                                           c
c     M1(n,n)  - matriser                                              c
c     M1S(n,n)                                                         c
c     n,m      - aktuelle dimensjoner                                  c
c     nm,mm    - max dimensjoner                                       c
c                                                                      c
c     OUTPUT                                                           c
c     M1(n,n)  - oppdatert matrise                                     c
c     M1R(n,n) - matrise produktet : U * M1S * TSP(U)                  c
c                                                                      c
c     RUTINER SOM KALLES : TRANSMULT                                   c
c                          MATMULT                                     c
c                          DMSUM                                       c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         M1(nm,mm), M1R(nm,mm), M1S(nm,mm)
      DOUBLE PRECISION         U(nm,mm), wm1(nm,mm)

      CALL     TRANSMULT(M1S,U,wm1,n,n,n,nm,1)
      CALL     MATMULT(U,wm1,M1R,n,n,n,nm)
      CALL     DMSUM(M1,M1,M1R,n,n,nm,nm,'+')

      RETURN
      END




      SUBROUTINE DCALG(G,PI,B,R,n,m,nm,mm,wm1,wm2)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner matrise-produktet : G = - PI * TSP(B) * R               c
c                                                                      c
c     INPUT                                                            c
c     PI(m,m)    - matriser                                            c
c     B(n,m)     -                                                     c
c     R(n,n)     -                                                     c
c     wm1(nm,mm) - to midlertidige lagringsmatriser                    c
c     wm2(nm,mm)                                                       c
c     n,m        - aktuelle dimensjoner                                c
c     nm,mm      - max dimensjoner                                     c
c                                                                      c
c     OUTPUT                                                           c
c     G(m,n)     - resultat matrise                                    c
c                                                                      c
c     RUTINER SOM KALLES : TRANSMULT       DMZER                       c
c                          MATMULT         DMSUM                       c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         G(nm,mm),   PI(nm,mm)
      DOUBLE PRECISION         B(nm,mm),   R(nm,mm)
      DOUBLE PRECISION         wm1(nm,mm), wm2(nm,mm)

      CALL     TRANSMULT(B,R,wm1,m,n,n,nm,2)
      CALL     MATMULT(PI,wm1,wm2,m,m,n,nm)
      CALL     DMZER(wm1,m,n,mm,nm)
      CALL     DMSUM(G,wm1,wm2,m,n,mm,nm,'-')

      RETURN
      END




      SUBROUTINE DCNOTU(T,U,n,m,nm,mm,er,ei,ind,aep,ix)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              11. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Finner en ny ortogonal transformasjon U' og tilhorende           c
c     matrise T' pa Schur-form som forer til at egenverdiene           c
c     som pekes pa brukeren havner i blokken T22 i T'                  c
c                                                                      c
c     For             Etter                                            c
c     U * T* tsp(U) = U' * T'* tsp(U') = Konstant                      c      
c                                                                      c
c     INPUT                                                            c
c     Schur-formen :                                                   c
c     T(n,n) - Ovre semitriangulaer form med egenverdier i             c
c              blokkform paa hoveddiagonalen.                          c
c     U(n,n) - Ortogonale similaritets-transformasjoner.               c
c     n,m    - aktuelle dimensjoner                                    c
c     nm,mm  - max dimensjoner                                         c
c     er(n)  - vektor med egenverdienes realverdi.                     c
c     ei(n)  - vektor med egenverdienes imaginaerverdi.                c
c     ind(n) - vektor med egenverdienes type.                          c
c              0 : reell egenverdi.                                    c
c              1 : imaginaer egenverdi m/positiv imaginaerdel.         c
c              2 : imaginaer egenverdi m/negativ imaginaerdel.         c
c             -1 : har ikke funnet egenverdi.                          c
c     ix(n)  - indekstabell som viser hvilket egenverdinummer          c
c              et egenverdipar har. Bade to komplekskonjungerte        c
c              og en reell egenverdi representerer et egenverdipar.    c
c     aep(2) - peker til en eller to egenverdipar. Brukes som          c
c              indeks i ix(n). Inneholder pekere til max to            c
c              egenverdier.                                            c
c                                                                      c
c     OUTPUT                                                           c
c     T'(n,n) - omformet                                               c
c     U'(n,n) - omformet                                               c
c     ind(n) - oppdatert                                               c
c     er(n)  - oppdatert                                               c
c     ei(n)  - oppdatert                                               c
c                                                                      c
c     RUTINER SOM KALLES : DCEXCH                                      c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm,ind(nm),aep(2),ix(nm)
      DOUBLE PRECISION         T(nm,mm), U(nm,mm)
      DOUBLE PRECISION         er(nm),   ei(nm)

      COMMON                   / MINF / iout, iprt
      INTEGER                  iout, iprt

c...  Lokale parametre                                     ............ 
      INTEGER                  iah,ial

c...  Hvis noen av egenverdiene ligger utenfor T22         ............ 
      IF ((ix(aep(1)).LT.(n-1)).OR.
     +   ((aep(2).GE.1).AND.(ix(aep(2)).LT.(n-1)))) THEN


c...     Hvis aep(1) eller aep(2) peker pa komplekse egenverdier ...... 
         IF (ind(ix(aep(1))).NE.0) THEN

            CALL DCEXCH(T,U,n,m,nm,mm,er,ei,ind,ix(aep(1)))

         ELSE IF (ind(ix(aep(2))).NE.0) THEN

            CALL DCEXCH(T,U,n,m,nm,mm,er,ei,ind,ix(aep(2)))

c...     En eller to reelle egenverdier flyttes til T22    ............ 
         ELSE    

c...        Finner element i aep med lavest egenverdinummer  .......... 
            IF  (aep(1).GE.aep(2)) THEN
                iah = aep(1)
                ial = aep(2)
            ELSE
                iah = aep(2)
                ial = aep(1)
            END IF

c...        Flytter egenverdi med hoyest egenverdinummer hvis  ........ 
c...        den ikke ligger i posisjon n.                  ............ 
            IF (ix(iah).LT.n) THEN

               CALL DCEXCH(T,U,n,m,nm,mm,er,ei,ind,ix(iah))

            END IF

c...        Flytter egenverdi med lavest nummer hvis den ligger  ...... 
c...        utenfor T22 og de to pekerne ikke peker pa samme verdi .... 
            IF ((ix(ial).LT.(n-1)).AND.(ial.NE.iah).AND.
     +               (ial.NE.0)) THEN

               CALL DCEXCH(T,U,n,m,nm,mm,er,ei,ind,ix(ial))

            END IF

         END IF

      END IF

c...  Flytter reell egenverdi fra posisjon n-1 til n hvis bare ........
c...  en reell egenverdi skal flyttes                      ............ 
      IF (((aep(2).EQ.0).OR.(aep(2).EQ.aep(1))).AND.
     +     (ind(n) .EQ. 0).AND.(n.NE.2).AND.(ix(aep(1)).EQ.(n-1)))

     +      CALL DCEXCH(T,U,n,m,nm,mm,er,ei,ind,ix(aep(1)))

c...  Oppdater er og ei slik at egenverdiene kommer i riktig .......... 
c...  rekkefolge                                           ............ 
      CALL EIGVAL(T,U,n,1,n,er,ei,nm,nm)

      IF ( iprt .EQ. 1 ) THEN
         WRITE(*,*)
         WRITE(*,*) 'ETTER evt ombytting av egenv.'
         CALL DCHEX(U,T,n,nm)
      END IF

      RETURN
      END




      SUBROUTINE DCHEX(U,T,n,nm)
 
      INTEGER                  n,nm
      DOUBLE PRECISION         T(nm,mm),wm2(50,50)
      DOUBLE PRECISION         U(nm,mm),wm3(50,50)
      DOUBLE PRECISION         wm1(50,50), wm4(50,50)
      DOUBLE PRECISION         wv1(50), det
 
      WRITE(*,*)
      CALL MATMULT(U,T,wm1,n,n,n,nm)
      CALL DMTSP(wm3,U,n,n,nm,nm)
c     CALL MATCOP(U,wm4,n,n,nm,nm)
c     CALL INVERS(wm4,wm3,wv1,det,n,nm)
      CALL MATMULT(wm1,wm3,wm2,n,n,n,nm)
      CALL MATPRT('Prod: UTU'' ',wm2,nm,nm,n,n)
 
      RETURN
      END
 





      SUBROUTINE DCEXCH(T,U,n,m,nm,mm,er,ei,ind,i)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              4.  oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Omformer schur-formen slik at den egenverdien som pekes          c
c     paa av "i" blir liggende i nederste hoyre hjorne i T(n,n).       c
c                                                                      c
c     For             Etter                                            c
c     U * T* tsp(U) = U' * T'* tsp(U') = Konstant                      c      
c                                                                      c
c     INPUT                                                            c
c     Schur-formen :                                                   c
c     T(n,n) - Ovre semitriangulaer form med egenverdier i             c
c              blokkform paa hoveddiagonalen.                          c
c     U(n,n) - Ortogonale similaritets-transformasjoner.               c
c     n,m    - aktuelle dimensjoner                                    c
c     nm,mm  - max dimensjoner                                         c
c     er(n)  - vektor med egenverdienes realverdi.                     c
c     ei(n)  - vektor med egenverdienes imaginaerverdi.                c
c     ind(n) - vektor med egenverdienes type.                          c
c              0 : reell egenverdi.                                    c
c              1 : imaginaer egenverdi m/positiv imaginaerdel.         c
c              2 : imaginaer egenverdi m/negativ imaginaerdel.         c
c             -1 : har ikke funnet egenverdi                           c
c     i      - posisjon til egenverdi(er) som skal transformeres       c
c              til nedre hoyre hjorne i T(n,n).                        c
c                                                                      c
c     OUTPUT                                                           c
c     T'(n,n) - omformet                                               c
c     U'(n,n) - omformet                                               c
c     ind(n) - oppdatert                                               c
c                                                                      c
c     RUTINER SOM KALLES : EXCHNG (HQR3)                               c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n, m, nm, mm   
      INTEGER                  ind(nm),  i  
      DOUBLE PRECISION         T(nm,mm), U(nm,mm)
      DOUBLE PRECISION         er(nm),    ei(nm)

c...  Konvergenskriterium                                  ............ 
c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER..
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps
       
c...  Lokale variable                                      ............ 
      INTEGER                  T1,T2
      LOGICAL                  fail
       
20    CONTINUE
         IF (ind(i).EQ.0) THEN
            IF (ind(i+1).EQ.0) THEN
c....          Bytter om plasseringen av to reelle egenverdier i "T"...
               CALL EXCHNG(T,U,n,i,1,1,eps,fail,nm,nm)
               T1 = ind(i)
               ind(i)   = ind(i+1)
               ind(i+1) = T1
               i = i + 1
            ELSE
c....          Bytter om plasseringen av en reell egenverdi............
c....          og et komplekskonjungert egenverdipar i "T".............
               CALL EXCHNG(T,U,n,i,1,2,eps,fail,nm,nm)
               T1 = ind(i)
               ind(i)   = ind(i+1)
               ind(i+1) = ind(i+2)
               ind(i+2) = T1
               i = i + 2
            END IF
         ELSE
            IF (ind(i+2).EQ.0) THEN
c....          Bytter om plasseringen av et komplekskonjungert.........
c....          egenverdipar og en reell egenverdi i "T"................
               CALL EXCHNG(T,U,n,i,2,1,eps,fail,nm,nm)
               T1 = ind(i)
               ind(i)   = ind(i+2)
               ind(i+2) = ind(i+1)
               ind(i+1) = T1
               i = i + 1
            ELSE
c....          Bytter om plasseringen av to komplekskonjungerte........
c....          egenverdipar i "T"......................................
               CALL EXCHNG(T,U,n,i,2,2,eps,fail,nm,nm)
               T1 = ind(i)
               T2 = ind(i+1)
               ind(i)   = ind(i+2)
               ind(i+1) = ind(i+3)
               ind(i+2) = T1
               ind(i+3) = T2
               i = i + 2
            END IF
         END IF

c.... Bytt om til aktuelt par befinner seg i bunnen av "T"..............

      IF (((ind(i).EQ.0).AND.(i.LT.n)).OR.
     +    ((ind(i).NE.0).AND.(i.LT.(n-1)))) GOTO 20

      RETURN
      END



      SUBROUTINE DCOEFS(T,H,n,m,nm,mm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              11. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner koeffisient-vektorene a,b,c,d,f,g,k som funksjoner      c
c     av blokkene T22 og H22 i matrisene T og H.                       c
c                                                                      c
c     Uttrykkene for koeffisientene kan finnes appendix i              c
c     Ruscio (1989) eller i kap.3 i Bergslid (1989).                   c
c                                                                      c
c     INPUT                                                            c
c     T(n,n) - matriser                                                c
c     H(n,n) -                                                         c
c     n,m    - aktuelle dimensjoner                                    c
c     nm,mm  - max dimensjoner                                         c
c                                                                      c
c     OUTPUT                                                           c
c                                                                      c
c     Koeffisient-vektorene a,b,c,d,f,g,k som legges i                 c
c     COMMON-blokken COEF                                              c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         T(nm,mm), H(nm,mm)

c...  Benyttes av DCOEFS* og DCALQS                          .......... 
      COMMON                   / COEF / a,b,c,d,f,g,k
      DOUBLE PRECISION         a(0:2),b(1:3),c(1:3),d(1:2)
      DOUBLE PRECISION         f(1:4),g(1:2),k(1:2)

c...  Lokale variable                                        .......... 
      DOUBLE PRECISION         wv(1:5)
      INTEGER                  x

      x = n-1

      a(0)   = H(x,x)*H(n,n) - H(x,n)*H(n,x)
      a(1)   = H(x,x)*T(n,n)*T(n,n) - 2.0*H(x,n)*T(x,n)*T(n,n) +
     +         H(n,n)*T(x,n)*T(x,n)
      a(2)   = H(x,x)*T(n,x)*T(n,x) - 2.0*H(x,n)*T(x,x)*T(n,x) +
     +         H(n,n)*T(x,x)*T(x,x)
      b(1)   = H(x,x)
      b(2)   = H(n,n)
      d(1)   = 2.0*H(x,n)
      d(2)   = T(x,x)*T(x,x) + T(n,n)*T(n,n) + 2.0*T(x,n)*T(n,x)
      k(1)   = 2.0*T(x,n)*( H(x,n)*T(n,x) - H(n,n)*T(x,x)) +
     +         2.0*T(n,n)*(-H(x,x)*T(n,x) + H(x,n)*T(x,x))
      wv(1) = T(x,x)*T(n,n) - T(x,n)*T(n,x)
      k(2)   = wv(1) * wv(1)

      g(1)   = -2.0*a(0)*a(0)*d(1)
      wv(1) = a(0)*d(2) - a(1)*b(2) - a(2)*b(1)
      g(2)   = a(0)*(4.0*b(1)*b(2)*k(1) + 2.0*d(1)*wv(1))
      f(1)   = a(0)*a(0)
      f(2)   = -2.0*a(0)*wv(1)
      f(3)   = wv(1)*wv(1) + 4.0*b(1)*b(2)*(a(0)*k(2)-a(1)*a(2))
      f(4)   = -4.0*a(0)*b(1)*b(2)

      c(1)   = -4.0*a(0)*a(0)*a(0)

      RETURN
      END


      SUBROUTINE DCMIVA(T,H,n,m,nm,mm,if,maksim)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              11. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Finner storste imaginaerdel for nye egenverdier som funksjon     c
c     av blokkene T22 og H22 i de tilfeller at det(H22) > 0.           c
c     Formelen som brukes star i kap.3 i rapporten for diplomen        c
c     til Magne M. Bergslid.                                           c
c                                                                      c
c     Skriver ut melding om at det ikke er noen begrensning            c
c     pa imaginaerdelen hvis det(H22) = 0 og brukeren onsker           c
c     informasjon om maksimal imaginaerdel.                            c
c                                                                      c
c     INPUT                                                            c
c     T(n,n) - matriser                                                c
c     H(n,n) -                                                         c
c     n,m    - aktuelle dimensjoner                                    c
c     nm,mm  - max dimensjoner                                         c
c                                                                      c
c     OUTPUT                                                           c
c     maksim - beregnet storste imaginaerdel                           c
c     if = 1 : begrensning pa imaginaerdelen                           c
c          0 : ingen begrensning pa imaginaerdelen                     c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm,if
      DOUBLE PRECISION         T(nm,mm),H(nm,mm),maksim

c...  Globale variable.                                      .......... 
c...  Benyttes av MODOPT*, DSYSOP*, DRNEV og DCMIVA           ..........
      COMMON                   / MAXI / miv
      LOGICAL                  miv

c...  Konvergenskriterium                                   ...........
c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER. 
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps
                                        
c...  Lokale variable.                                       .......... 
      INTEGER                  x
      DOUBLE PRECISION         a0,c1,c2

      x = n-1

c...  Beregner determinanten til H22.                          .......
      a0 = H(x,x)*H(n,n) - H(x,n)*H(n,x)

      IF (a0.GT.SQRT(eps)) THEN
         c1 = H(x,n)*T(n,n) + H(x,x)*T(n,x)
         c2 = H(n,n)*T(x,n) + H(x,n)*T(x,x)

         maksim = (c1-c2)/(2.0*SQRT(a0))
         if = 1
      ELSE
c...     Hvis bruker onsker info om max imaginaerdel.        .......... 
         IF ( miv ) THEN
            WRITE(*,*)
            WRITE(*,*) ' No max limit of the imaginary value'
         END IF
         if = 0
      END IF

      RETURN
      END


      SUBROUTINE DCALQS(QS,n,m,nm,mm,nf)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              11. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Finner (2x2)-blokk i nedre hoyre hjorne av QS som funksjon       c
c     lovlig intervall for QS(x,n) (x=n-1). Dette intervallet          c
c     finnes av ulikheten                                              c
c                                                                      c
c     J(QS(x,n)) = c(1)*QS(x,n)*QS(x,n) + c(2)*QS(x,n) + c(3) >= 0     c
c                                                                      c
c     der c(1),c(2) og c(3) er funksjoner av tilsvarende blokker       c
c     i matrisene T og H pa Schur-form og de nye egenverdiene.         c
c                                                                      c
c     Rutinen finner ogsa blokken i QS som funksjon av QS(x,n)         c
c     nar denne ikke har noe ulovlig intervall.                        c
c                                                                      c
c     Beregningene i rutinen er naert knyttet til resultatene          c
c     i i artikkelen "A Schur method for designing LQ-optimal          c
c     systems with prescribed eigenvalues", utgitt i oktober -89       c
c     av David D. Ruscio ved Inst. for Teknisk Kybernetikk ved NTH.    c
c                                                                      c
c     Koeffisientene som benyttes kan finnes i                         c
c     appendix i dette notatet.                                        c
c                                                                      c
c     INPUT                                                            c
c     QS(n,n) - matrise                                                c
c     n,m     - aktuelle dimensjoner                                   c
c     nm,mm   - max dimensjoner                                        c
c                                                                      c
c     COMMON-blokkene COEF,NEIG og CQ12                                c
c                                                                      c
c     OUTPUT                                                           c
c     QS(n,n) - oppdatert (2x2)-blokk i nedre hoyre hjorne             c
c                                                                      c
c     nf =  0  : Lovlig tilstand. Ingen feil har inntruffet.           c
c          1-6 : Feiltilstand er oppdaget. Brukes av                   c
c                rutinen DPRERR som skriver ut feilmelding             c
c                avhengig av verdi.                                    c
c                                                                      c
c     COMMON-blokken QVEK som tilornes slik at QS(x,x) og QS(n,n)      c
c                         kan finnes som funksjoner av QS(x,n)         c
c                                                                      c
c     RUTINER SOM KALLES : DCSOIE                                      c
c                          DREAD                                       c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm,nf
      DOUBLE PRECISION         QS(nm,mm)

c...  Globale variable                                      ...........

c...  Benyttes av DCOEFS* og DCALQS                         ........... 
c...  Koeff. avhengig av matrisene T og H pa Schur-form     ...........
      COMMON                   / COEF / a,b,c,d,f,g,k
      DOUBLE PRECISION         a(0:2),b(1:3),c(1:3),d(1:2)
      DOUBLE PRECISION         f(1:4),g(1:2),k(1:2)

c...  Benyttes av DRNEV*, DCALQS                            ........... 
c...  Koeff. som er avhengig av nye egenverdier.            ...........
      COMMON                   / NEIG / p1,p2
      DOUBLE PRECISION         p1,p2

c...  Benyttes av MODOPT*, DSYSOP*, DCBRIC og DCALQS        ...........
      COMMON                   / CQ12 / aq
      INTEGER                  aq      

c...  Benyttes av DCBRIC, DCNQ2M, DRMOV*, DCALQS*           ........... 
c...  Far verdier som bestemmer QS(x,x) og QS(n,n)          ...........   
c...  som funksjon av QS(x,n)                               ...........
      COMMON                   / QVEK / QV
      DOUBLE PRECISION         QV(1:11)

c...  Konvergenskriterium                                   ...........
c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER. 
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps

c...  Lokale variable                                       ...........
      INTEGER                  x,styp
      DOUBLE PRECISION         wv(1:5)
      DOUBLE PRECISION         v(1:2),w(1:2)
      DOUBLE PRECISION         LL,UL,grd

      x = n-1

c...  Test om determinanten til (2x2)-blokken i             ...........
c...  Nedre venstre hjorne i H pa Schur-form er lik null    ...........

      IF ( a(0) .GT. SQRT(eps) ) THEN

c...     c(2) og c(3) er funksjoner av koeff. i COEF og NEIG...........
         c(2) = g(1)*p1 + g(2)
         c(3) = f(1)*p1*p1 + f(2)*p1 + f(3) + f(4)*p2

c...     Finn losningsomrade for ulikheten                  ...........
         CALL DCSOIE(c(1),c(2),c(3),LL,UL,grd,styp)

         IF ( styp .EQ. 0 ) THEN
            nf = 1
c...        Ingen verdier for Q(1,2) = Q(2,1) er lovlig ............
            GOTO 999
         ELSE IF ( styp .EQ. 1 ) THEN
            QS(x,n) = grd
            WRITE(*,*)
            WRITE(*,1) 'q12 = q21 in Q22 must be exact :',QS(x,n)
1           FORMAT(1X,A,G16.6)
         ELSE IF ( styp .EQ. 4 ) THEN
            wv(1) = grd
            IF ( aq .EQ. 2 ) THEN
c...           Brukeren vil velge QS(x,n)                   ...........
               WRITE (*,*)                              
               WRITE (*,*)' Valid interval for q12 in Q22 :'
               WRITE (*,2) LL,UL
2              FORMAT (1X,G16.6,'<  q12  <',G16.6)
               WRITE (*,*)
100            wv(2) = wv(1)
               CALL DREAD('Registrate value : ',wv(2),1,1,IIII)
               IF ((wv(2).LT.LL).OR.(wv(2).GT.UL)) GOTO 100
               QS(x,n) = wv(2)
            ELSE
               QS(x,n) = wv(1)
            END IF
         ELSE IF ( styp .EQ. 5 ) THEN
            nf = 4
c...        Determinanten til H er mindre enn null, returner..........
            GOTO 999
         ELSE
            nf = 5
c...        a(0) <> 0.0 , men c(1) = 0.0. Returner          ...........
            GOTO 999
         END IF

c...     Har funnet lovlig losningsintervall av ulikheten.  ...........
c...     Beregn lokale hjelpevariable.                      ...........

         b(3) = d(1)*QS(x,n) + d(2) - p1

         wv(1) = c(1)*QS(x,n)*QS(x,n) + c(2)*QS(x,n) + c(3)
         wv(2) = -a(0)*b(3)
         wv(3) = a(1)*b(2) - a(2)*b(1)
         wv(4) = 2.0*a(0)*b(1)
         wv(5) = 2.0*a(0)*b(2)

c...     Nemneren i losningen for QS(x,x) eller QS(n,n) er lik null....
         IF ((wv(4).EQ.0.0).OR.(wv(5).EQ.0.0)) THEN
            nf = 2
c...        Finner ikke Q(1,1) og Q(2,1). Returner.          ..........
            GOTO 999
         END IF

         IF ( wv(1) .LT. 0.0 ) THEN
            nf = 6
c...        Finner at J(q12) er negativ. Returner.           ...........
            GOTO 999
         END IF

         IF ((styp.EQ.4).AND.(aq.NE.2)) THEN

c...        Initier COMMON-blokken QVEK med losningsintervall..........
c...        for QS(x,n).                                    ...........
            QV(1)  = 1.0
            QV(2)  = LL
            QV(3)  = UL
            QV(4)  = c(1)
            QV(5)  = c(2)
            QV(6)  = c(3)
            QV(7)  = -a(0)*d(1)
            QV(8)  = -a(0)*(d(2)-p1)
            QV(9)  = wv(3)
            QV(10) = wv(4)
            QV(11) = wv(5)

         ELSE
c...        Bestemt losning for QS(x,n).                    ...........
            QV(1)  = 0.0
         END IF

c...     Tilordner QS verdier uansett, fordi det er en       ..........
c...     grense for hvor lite losningsintervall som godtas   ..........
c...     Vet heller ikke om noen verdier av QS(x,n) vil      ..........
c...     gi positiv definit losning av Riccati-ligningen, og ..........
c...     da brukes denne som losning.                        ..........

         QS(n,x) = QS(x,n)
         QS(x,x) = ((wv(2)+wv(3)+SQRT(wv(1)))/wv(4))
         QS(n,n) = ((wv(2)-wv(3)-SQRT(wv(1)))/wv(5))

      ELSE 
c...     Determinanten til (2x2)-blokken i nedre hoyre      ...........
c...     av H er lik null. Alle verdier for QS(x,n) er lovlige ........

         wv(1) = a(1)*b(2) - a(2)*b(1)
         write(*,*) 'wv(1) : ',wv(1)
         IF ( ABS(wv(1)) .LT. SQRT(eps) ) THEN
            nf = 3
c...        Systemet er ikke styrbart. Returner.             ...........
            GOTO 999
         ELSE
c...        Beregn lokale hjelpevariable                    ...........
            v(1) = ((a(2)*d(1) - b(2)*k(1))/wv(1))
            v(2) = ((b(2)*(p2-k(2)) - a(2)*(p1-d(2)))/wv(1))
            w(1) = ((b(1)*k(1) - a(1)*d(1))/wv(1))
            w(2) = ((-b(1)*(p2-k(2)) + a(1)*(p1-d(2)))/wv(1))

            QS(x,n) = 0.0
            IF ( aq .EQ. 2 ) THEN
               QV(1) = 0.0
               WRITE(*,*)
               WRITE(*,*) ' No invalid interval for q12 = q21 in Q22'
               WRITE(*,*)
               CALL DREAD('Registrate value : ',QS(x,n),1,1,IIII)
            ELSE
               QV(1) = 2.0
               QV(2) = -10000.0
               QV(3) =  10000.0
               QV(4) = v(1)
               QV(5) = v(2)
               QV(6) = w(1)
               QV(7) = w(2)
            END IF

c...        Tilordner QS verdi uansett. Beholdes som losning    .......   
c...        hvis ikke noen verdier av QS(x,n) gir positiv losning......   
c...        av Riccati-ligningen.                               .......  

            QS(n,x) = QS(x,n)
            QS(x,x) = v(1) * QS(x,n) + v(2)
            QS(n,n) = w(1) * QS(x,n) + w(2)

         END IF

      END IF

c...  Ingen feil har inntruffet.                             ..........
      nf = 0

      GOTO 1000

c...  Tilornes til feiltilstand                              ..........
999   QV(1) = -1

1000  RETURN
      END

      SUBROUTINE DCSOIE(a,b,c,LL,UL,grd,styp)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              11. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner losningsomradet til ulikheten                           c
c                                                                      c
c              J(x) = a*x*x + b*x + c >= 0                             c
c                                                                      c
c     INPUT                                                            c
c     a,b,c  - koeffisienter i ulikheten                               c
c                                                                      c
c     OUTPUT                                                           c
c     LL,UL  - nedre og ovre grense for losningsintervallet            c
c                                                                      c
c     styp = 0 : Ingen losning                                         c
c            1 : x = -b/(2*a) = d                                      c
c            2 : x >= LL                                               c
c            3 : x <= UL                                               c
c            4 : LL <= x <= UL                                         c
c            5 : x <= LL V x >= UL                                     c
c            6 : Alle verdier mulig, x = R                             c
c                                                                      c
c     grd  = 0  for styp = 0,5,6                                       c
c            d  for styp = 1,4                                         c
c            LL for styp = 2                                           c
c            UL for styp = 3                                           c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      DOUBLE PRECISION         a,b,c
      DOUBLE PRECISION         LL,UL,grd
      INTEGER                  styp
    
      LL  = 0.0
      UL  = 0.0
      grd = 0.0

      IF ( a .EQ. 0.0 ) THEN
         IF ( b .EQ. 0.0 ) THEN
            IF ( c .GE. 0.0 ) THEN
c...           Alle verdier lovlige, x = R .................
               styp = 6
            ELSE
c...           Ingen verdier lovlige .......................
               styp = 0
            END IF
         ELSE 
            grd = (-c/b)
            IF ( b .GT. 0.0 ) THEN
c...           Lovlig intervall: x >= (-c/b) ...............
               styp = 2
               LL   = (-c/b)
            ELSE 
c...           Lovlig intervall: x <= (-c/b) ...............
               styp = 3
               UL   = (-c/b)
            END IF
         END IF
      ELSE 
         hh = b*b - 4.0*a*c

         IF ( hh .LT. 0.0 ) THEN
c...        Ingen losning mulig ............................
            styp = 0
         ELSE IF ( hh .EQ. 0.0 ) THEN
c...        Bare en losning mulig : x = ((-b)/(2.0*a))......
            styp = 1
            grd  = ((-b)/(2.0*a))
         ELSE
            IF ( a .LT. 0.0 ) THEN
c...           Lovlig intervall for x er mellom LL og UL ...
               styp = 4
               grd = (-b/(2.0*a))
               UL = ((-b-SQRT(hh))/(2.0*a))
               LL = ((-b+SQRT(hh))/(2.0*a))
            ELSE
c...           Ulovlig intervall for x er mellom LL og UL ...
               styp = 5
               LL = ((-b-SQRT(hh))/(2.0*a))
               UL = ((-b+SQRT(hh))/(2.0*a))
            END IF
         END IF
      END IF

999   RETURN
      END

  

      SUBROUTINE DGMBFD(MS,ML,n,nm,ibst,ipos)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Trekker ut en matriseblokk fra en kvadratisk matrise.            c
c     Matriseblokken er mindre enn eller lik den kvadratiske matrisen, c
c     og trekkes ut fra diagonalen med angitt startposisjon.           c
c     Matriseblokken er ogsa kvadratisk.                               c
c                                                                      c
c     INPUT                                                            c
c     MS(n,n) - kvadratisk matrise                                     c
c     n       - aktuell dimensjon                                      c
c     nm      - max dimensjon                                          c
c     ibst    - storrelsen pa blokken som skal trekkes ut av MS        c
c     ipos    - startposisjon pa diagonalen.                           c
c                                                                      c
c     OUTPUT                                                           c
c     ML(ibst,ibst) - kvadratisk matriseblokk                          c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm,ibst,ipos
      DOUBLE PRECISION         MS(nm,nm), ML(nm,nm)

      INTEGER                  imax,i,j

      imax = ipos + ibst - 1

      IF (imax.LE.n) THEN 
         DO 10 i = ipos,(ipos+ibst-1)
            DO 20 j = ipos,(ipos+ibst-1)
               ML((i-ipos+1),(j-ipos+1)) = MS(i,j)
20          CONTINUE
10       CONTINUE
      END IF

      RETURN
      END



      SUBROUTINE DSMBTD(MS,ML,n,nm,ibst,ipos)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Setter inn en matriseblokk pa en kvadratisk matrise.             c
c     Matriseblokken er mindre enn eller lik den kvadratiske matrisen, c
c     og settes inn paa diagonalen med angitt startposisjon.           c
c     Matriseblokken er ogsa kvadratisk.                               c
c                                                                      c
c     INPUT                                                            c
c     ML(ibst,ibst) - kvadratisk matriseblokk                          c
c     n             - aktuell dimensjon                                c
c     nm            - max dimensjon                                    c
c     ibst          - storrelsen pa blokken som skal settes inn pa MS  c
c     ipos          - startposisjon paa diagonalen.                    c
c                                                                      c
c     OUTPUT                                                           c
c     MS(n,n) - kvadratisk matrise                                     c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm,ibst,ipos
      DOUBLE PRECISION         MS(nm,nm), ML(nm,nm)

      INTEGER                  imax,i,j

      imax = ipos + ibst - 1

      IF (imax.LE.n) THEN 
         DO 10 i = ipos,(ipos+ibst-1)
            DO 20 j = ipos,(ipos+ibst-1)
               MS(i,j) = ML((i-ipos+1),(j-ipos+1))
20          CONTINUE
10       CONTINUE
      END IF

      RETURN
      END



      SUBROUTINE DMENU(ijob)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Skriver ut en meny og returnerer valgt funksjon.                 c
c     Valget kan registreres bade nummerisk og alfanummerisk.          c
c                                                                      c
c     OUTPUT                                                           c
c     ijob  - heltall som viser menyvalget.                            c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  ijob

      INTEGER                  icom,ipos,iflag
      PARAMETER              ( icom=19 )
      CHARACTER                cjob*6, com(1:icom)*(6)

      DATA COM(1)/'A'/,   COM(2)/'B'/, COM(3)/'P'/,    COM(4)/'Q'/,
     +     COM(5)/'G'/,   COM(6)/'R'/, COM(7)/'A+BG'/, COM(8)/'MOV'/,
     +     COM(9)/'RNS'/, COM(10)/'ROP'/, COM(11)/'HELP'/,
     +     COM(12)/'EXIT'/

      WRITE(*,*)
      WRITE(*,*) '-------------                                      '
      WRITE(*,*) 'MODOPT MENU :                                      '
      WRITE(*,*) '-------------------------------------------------- '
      WRITE(*,*) 'Command : A B P Q G R A+BG MOV RNS  ROP HELP EXIT  '
      WRITE(*,*) 'Numeric : 1 2 3 4 5 6  7    8   9   10   11   0    '
      WRITE(*,*) '-------------------------------------------------- '

c...  Valg ved bare a trykke <ENTER> er flytting av egenverdier 
c...  automatisk valg............................................

      cjob = 'MOV'

c.... Leser inn menyvalg.........................................

100   WRITE(*,*)
      CALL RCSTR('Modopt command :',cjob,1,1,1)
      WRITE(*,*)

c...  Undersoker om innlest valg er fra 0 til icom...............
c...  Godtar ogsa verdier pa icom som er utenfor menyvalgene ....
c...  for a kunne skrive ut tilleggsinformasjon styrt av ........
c...  den kallende rutinen MODOPT.                       ........

      ipos = ICNUM(cjob)
      IF ((ipos.GE.0).AND.(ipos.LE.icom)) THEN
       ijob = ipos
       GO TO 1000
      ENDIF

c...  Undersoker om innlest valg er alfanummerisk................

      CALL COMCHK(cjob,com,icom,iflag,ipos)
      IF (iflag.EQ.0) THEN
        ijob = ipos

c...  Skriv ut hvilke muligheter som finnes hvis feil er registrert.

      ELSEIF (iflag.EQ.1) THEN
       CALL COMLIS('Available is :',com,icom)
       cjob = 'MOV'
       GO TO 100
      ENDIF

1000  IF ( ijob .EQ. 0 ) ijob = 12

      RETURN
      END



      SUBROUTINE DPHLP1

      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)'                         < MODOPT UTILITIES >       '   
      WRITE(*,*)'                   -------------------------------- '  
      WRITE(*,*)'                    1. Print matrix A -------> A    '
      WRITE(*,*)'                    2. Print matrix B -------> B    '
      WRITE(*,*)'                    3. Print matrix P -------> P    '
      WRITE(*,*)'                    4. Print matrix Q -------> Q    '
      WRITE(*,*)'                    5. Print matrix G -------> G    '
      WRITE(*,*)'                    6. Print matrix R -------> R    '
      WRITE(*,*)'                    7. Print matrix A+BG ----> A+BG '
      WRITE(*,*)'                    8. Move eigenvalues -----> MOV  '
      WRITE(*,*)'                    9. Read new system ------> RNS  '
      WRITE(*,*)'                   10. Registrate options ---> ROP  '
      WRITE(*,*)'                   11. Print this page ------> HLP  '
      WRITE(*,*)'                    0. Exit program MODOPT --> EXIT '
      WRITE(*,*)'                   ---------------------------------'
      WRITE(*,*)

      RETURN
      END



      SUBROUTINE DPHLP2

      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)                                                      
      WRITE(*,*)'                      < SUBMENU UTILITIES >       '     
      WRITE(*,*)'            ----------------------------------------- '
      WRITE(*,*)'             1. Info of max imaginary value ---> MAX  '
      WRITE(*,*)'             2. Registrate option on q12 ------> q12  '
      WRITE(*,*)'                   1 - Let MODOPT choose q12.'
      WRITE(*,*)'                   2 - Let  user specify q12.'
      WRITE(*,*)'                   3 - Let MODOPT optimize'
      WRITE(*,*)'                       to find J(q12)=min(Tr(R))'
      WRITE(*,*)'             3. Initiate matrix Q to zero -----> ZERQ '
      WRITE(*,*)'             4. Print part results to DEBUG ---> DEBU '
      WRITE(*,*)'             5. Print this page     -----------> HELP '
      WRITE(*,*)'             0. Return to MAIN MENU -----------> EXIT '
      WRITE(*,*)'            ----------------------------------------- '
      WRITE(*,*)                                                      
      WRITE(*,*)

      RETURN
      END



      SUBROUTINE DMSUM(RES,M1,M2,n,m,nm,mm,flg)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              4.  oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Utforer addisjon eller subtraksjon av to matriser.               c
c                                                                      c
c     INPUT                                                            c
c     M1(n,m),M2(n,m) - endres ikke i rutinen                          c
c     n,m   - aktuelle dimensjoner                                     c
c     nm,mm - max dimensjoner                                          c
c     flg = '+' : RES = M1 + M2                                        c
c         = '-' : RES = M1 - M2                                        c
c                                                                      c
c     OUTPUT                                                           c
c     RES(n,m)                                                         c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         RES(nm,mm), M1(nm,mm), M2(nm,mm)
      CHARACTER*1              flg

      INTEGER                  i,j

c.... flg='+' : adder matrisene .......................................

      IF ( flg .EQ. '+' ) THEN
           DO 20 i = 1,n
              DO 10 j = 1,m
                 RES(i,j) = M1(i,j) + M2(i,j)
10            CONTINUE
20         CONTINUE

c.... flg='-' : subtraher matrisene ...................................

      ELSE IF ( flg .EQ. '-' ) THEN
           DO 40 i = 1,n
              DO 30 j = 1,m
                 RES(i,j) = M1(i,j) - M2(i,j)
30            CONTINUE
40         CONTINUE
      END IF

      RETURN
      END


      SUBROUTINE DTRACE(MAT,n,nm,trace)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              12. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Beregner summen av diagonalelementene i en kvadratisk matrise.   c
c                                                                      c
c     INPUT                                                            c
c     MAT(n,n) - matrise                                               c
c     n        - aktuell dimensjon                                     c
c     nm       - max dimensjon                                         c
c                                                                      c
c     OUTPUT                                                           c
c     trace    - summen av diagonalelementene i MAT(n,n)               c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         MAT(nm,nm), trace

c...  Lokale variable                                      ............
      INTEGER                  i,j

      trace = 0.0

      DO 10 i = 1,n
         DO 20 j = 1,n
            IF ( i .EQ. j ) THEN
               trace = trace + MAT(i,j)
            END IF
20       CONTINUE
10    CONTINUE

      RETURN
      END




      SUBROUTINE DMTSP(RES,MAT,n,m,nn,mm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              4.  oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Transponerer en matrise.                                         c
c                                                                      c
c     INPUT                                                            c
c     MAT(n,m) - endres ikke i rutinen.                                c
c     n,m      - aktuelle dimensjoner                                  c
c     nm,mm    - max dimensjoner                                       c
c                                                                      c
c     OUTPUT                                                           c
c     RES(m,n) - den transponerte av MAT(n,m).                         c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      INTEGER                  n,m,nn,mm
      DOUBLE PRECISION         RES(nn,nm), MAT(nn,nm)

      INTEGER                  i,j

      DO 20 i = 1,n
         DO 10 j = 1,m
            RES(j,i) = MAT(i,j)
10       CONTINUE
20    CONTINUE

      RETURN
      END




      SUBROUTINE DMZLT(T,n,nm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        Teknisk Kybernetikk, NTH                c
c     Versjon 1 :              18. november 1989                       c
c         .                                                            c
c     FORMAL                                                           c
c     Nullstiller alle elementene under 1. subdiagonal                 c
c     under hoveddiagonalen i en matrise. Matrisen blir                c
c     da pa ovre semitriangulaer form.                                 c
c                                                                      c
c     INPUT                                                            c
c     T(n,n) - kvadratisk matrise                                      c
c     n      - aktuell dimensjon                                       c
c     nm     - max dimensjon                                           c
c                                                                      c
c     OUTPUT                                                           c
c     T(n,n) - kvadratisk matrise pa ovre semitriangulaer form         c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         T(nm,nm)

      INTEGER                  i,j

      IF ( n .GE. 3 ) THEN

         DO 10 i = 3,n
            DO 20 j = 1,i-2
               T(i,j) = 0.0
20          CONTINUE
10       CONTINUE

      END IF

      RETURN
      END




      SUBROUTINE DMZER(MAT,n,m,nm,mm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              4.  oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Tilordner alle elementene i en matrise til null.                 c
c                                                                      c
c     INPUT                                                            c
c     MAT(n,m) - matrise                                               c
c     n,m      - aktuelle dimensjoner                                  c
c     nm,mm    - max dimensjoner                                       c
c                                                                      c
c     OUTPUT                                                           c
c     MAT(i,j) = 0.0 for i = 0..n                                      c
c                        j = 0..m                                      c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         MAT(nm,mm)

      INTEGER                  i,j

      DO  20  i=1,n
          DO  10  j=1,m
              MAT(i,j) = 0.0d0
10        CONTINUE
20    CONTINUE

      RETURN
      END





      SUBROUTINE DMZSEL(MAT,n,m,nm,mm)

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         MAT(nm,mm)

      INTEGER                  i,j

c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER. 
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps

      DO  20  i=1,n
          DO  10  j=1,m
              IF ( ABS(MAT(i,j)) .LT. SQRT(eps) ) MAT(i,j) = 0.0d0
10        CONTINUE
20    CONTINUE

      RETURN
      END





      SUBROUTINE DTSYM(MAT,n,nm,sym)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Undersoker om en matrise er tilnaermet symmetrisk.               c
c                                                                      c
c     INPUT                                                            c
c     MAT(n,n) - matrise                                               c
c     n        - aktuelle dimensjon                                    c
c     nm       - max dimensjon                                         c
c                                                                      c
c     OUTPUT                                                           c
c     sym        - logisk variabel som settes til .TRUE.               c
c                  hvis en matrise er tilnaermet symmetrisk.           c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         MAT(nm,nm)
      LOGICAL                  sym
                                                       
      INTEGER                  i,j

c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER. 
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps

      sym = .TRUE.

      DO 20 i = 1,n
         DO 10 j = i,n
            IF ( sym ) sym = ( ABS(MAT(i,j)-MAT(j,i)) .LE. eps )
10       CONTINUE
20    CONTINUE

      RETURN
      END



      SUBROUTINE DMSYM(RES,MAT,n,nm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Omformer en kvadratisk matrise til a bli symmetrisk.             c
c     Hvis matrisen er symmetrisk fra for, returneres den uforandret.  c
c     Bruker formelen :                                                c
c                                                                      c
c                  Q = 0.5 * ( Q + TSP(Q) )     TSP -transponert       c
c                                                                      c
c     INPUT                                                            c
c     MAT(n,n) - matrise                                               c
c     n        - aktuelle dimensjon                                    c
c     nm       - max dimensjon                                         c
c                                                                      c
c     OUTPUT                                                           c
c     RES(n,n) - symmetrisk returmatrise                               c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,nm
      DOUBLE PRECISION         RES(nm,nm),MAT(nm,nm)

      INTEGER                  i,j

      DO 20 i = 1,n
         DO 10 j = 1,n
            RES(i,j) = 0.5 * ( MAT(i,j) + MAT(j,i) )
10       CONTINUE
20    CONTINUE

      RETURN
      END




      SUBROUTINE DTZER(MAT,n,m,nm,mm,zer)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     FORFATTER                                                        c
c     Magne M. Bergslid        NTH oktober 1989                        c
c     Versjon 1 :              12. oktober 1989                        c
c         .                                                            c
c     FORMAL                                                           c
c     Undersoker om alle elementene i en matrise                       c
c     er tilnaermet lik null.                                          c
c                                                                      c
c     INPUT                                                            c
c     MAT(n,m) - matrise                                               c
c     n,m      - aktuelle dimensjoner                                  c
c     nm,mm    - max  dimensjoner                                      c
c                                                                      c
c     OUTPUT                                                           c
c     zer      - logisk variabel som settes til .TRUE.                 c
c                hvis alle elementene i en matrise er tilnaermet       c
c                lik null.                                             c
c                                                                      c
c     RUTINER SOM KALLES : (ingen)                                     c
c                                                                      c
c     NOYAKTIGHET : DOUBLE PRECISION                                   c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      INTEGER                  n,m,nm,mm
      DOUBLE PRECISION         MAT(nm,mm)
      LOGICAL                  zer
                                                       
      INTEGER                  i,j

c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER. 
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps

      zer = .TRUE.

      DO 20 i = 1,n
         DO 10 j = 1,m
            IF ( zer ) zer = ( ABS(MAT(i,j)) .LE. eps )
10       CONTINUE
20    CONTINUE

      RETURN
      END




      SUBROUTINE DCEPS                   

c.... Finner et konvergenskriterium eps, som blir det       ............
c.... minste tallet som denne datamaskinen klarer a         ............
c.... skille fra 0.                                         ............

c.... Benyttes av DEPS*,DCSCHF,DCBRIC,DCEXCH,DCMIVA,DCALQS,DTSYM,DTZER..
      COMMON                   / KCRIT / eps
      DOUBLE PRECISION         eps

      eps = 1.0d0
 100  eps = eps/2.0d0
      if (eps+1.0d0 .gt. 1.0d0) go to 100
      eps = 2.0d0*eps

      RETURN
      END









