function [a,b,d,x0,o,c] = aokit(n,h,ny,nu);
%ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
%C  [a,b,d,x0,o,c] = aokit(n,h,ny,nu)                                  c
%C  Purpose:                                                           c
%C  Estimate system matrices from the block Hankel matrix of impulse   c
%C  responses.                                                         c
%C              x_k+1 = A x_k + B u_k                                  c
%C              y_k   = D x_k                                          c
%C                                                                     c
%C  The impulse responses h_i = d a^(i-1) b are determined             c
%C  from input and output time series of the system.                   c
%C  The time series are determined from simulation experiments         c
%C  from model number im in mod.m. See the routine respi.m for         c
%C  computation of the impulse responses.                              c
%C                                                                     c
%C  On input:                                                          c
%C  n     INTEGER. number of Hankel matrix blocks.                     c
%C  im    INTEGER. Refer to model number im in mod.m.                  c
%C  On output:                                                         c
%C  a     REAL.    State transition matrix.                            c
%C  b     REAL.    Control input matrix.                               c
%C  d     REAL.    State output (measurement) matrix.                  c
%C                                                                     c
%C  Routines called: hanki.m                                           c
%C                   rank, size, svd, sqrt                             c
%C  "Library" routines used: hanki.m, respi.m, loggi.m, mod.m          c
%Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

tol = 1e-6;
%
%****  Set up Hankel matrix with extra row for model # im ****
% [h,ny,nu,hz] = hanki(n,im);
%
%****  Pick up the usfull matrices from the extended Hankel matrix ****
hn = h(1:ny*n,1:nu*n);
[nh,mh] = size(hn);
na = rank(hn);
%
ha = h(ny+1:ny*n+ny,1:nu*n);
hb = hn(1:ny*n,1:nu);
hd = hn(1:ny,1:nu*n);
%
%****  Perform SVD decomposition of hn  *****
[u,s,v] = svd(hn);
qs = sqrt(s);
%
%****  Chose model order as the rank of hn, or user specified  ****
svd_values = diag(s(1:na,1:na))
na
na = input('Type # of states (na =) ')
%
%****  Compute balanced model a,b,d  ****
for i=1:na
   if qs(i,i) >= tol,
    qs(i,i) = 1/qs(i,i);
   end
end
si1 = qs(1:ny*n,1:na);
si2 = qs(1:na,1:mh);
a = si1'*u'*ha*v*si2';
b = si1'*u'*hb;
d = hd*v*si2';
%
%****  Estimate possible initial values x0 etc.  ****
o  = u*sqrt(s);
c  = sqrt(s)*v';
o  = o(1:ny*n,1:na);
c  = c(1:na,1:nu*n);
oa = o*a;
% x0 = inv(oa'*oa)*oa'*hz(1:ny*n,1);
%
%**** B and D are easily given by
% o(1:ny,1:na)
% c(1:na,1:nu)
